<?php 
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * William Adam <wadam@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
// The life saver function to workaround NGINX not logging PHP Fatal errors :)
function shutdown() {
    $error = error_get_last();
    if ($error['type'] === E_ERROR) {
        // fatal error has occured
        error_log('FATAL Error=' . print_r($error, true));
    }
}
// Register shutdown handler
register_shutdown_function('shutdown');

/*
 * NSG REST api server
 */
include 'sng_config_manager_functions.php';
include 'database_classes.php';
include 'wanrouter_controls.php';
include 'classes.php';
include 'mg/mg_config_generate.php';
include 'smg_ctrl/smg_ctrl.php';
include 'freetdm_config/freetdm.php';
include 'installation/transcoding_class.php';
include 'display_class.php';
include "production-test/prod-test-class.php";
include 'generic_functions.php';
$header = "404 Not Found";

$data = array();
// Is this a POST ?
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    // Get body data if any
    $_raw = file_get_contents('php://input', true);
    $data = json_decode($_raw, true);
}else{
    // GET request
    $data = $_REQUEST;
}
// Prepare the data for processing
$json_wrapper = array(
    'status'=>false,
    'module'=>$data['module'],
    'method'=>$data['method'],
    'obj_type'=>$data['obj_type'],
);

/*
 * Dispatch
 *
 */
// Build function name (ie. cat all submitted params relevant to build function name)
$function_name = 'rest';
// !!! Keep fields in order
$func_name_params = array('method', 'module', 'obj_type', 'obj_name');
foreach($func_name_params as $_p){
    if(isset($data[$_p])){
        $function_name .= "_".$data[$_p];
        $json_wrapper[$_p] = $data[$_p];
    }else{
        // this one is not in submitted parameters, move on
        break;
    }
}
//error_log($function_name);

// ready to proceed, check action exists and invoke
if(function_exists($function_name))
{   
    $output = array();
    $response = $function_name($data, $output);
    // Succeed ?
    if($response){
        $header = "200 OK";	
        $json_wrapper['status'] = true;
        $json_wrapper['data'] = $output;
    }else{
        $header = "400 Bad Request";	
        $json_wrapper['status'] = false;
        $json_wrapper['error'] = $output;
    }
}else{
    $json_wrapper['error'] = "Not Found";
}
/*
 * Reply
 */
header('Content-Type: application/json');
header("HTTP/1.1 $header");
echo json_encode($json_wrapper);
exit;
/*-------------------------------------------------------------------------------
 * No code to handle REST below this line, only handler functions
 *-------------------------------------------------------------------------------*/
/*
 * Dialpan module
 */
function rest_retrieve_tdmgw_cfg_dialplan($data, &$output)
{
    $data = file_get_contents("/usr/local/nsg/conf/dialplan/default.xml"); 
    $output = array('content' => $data);
    return true;
}

function rest_update_tdmgw_cfg_dialplan($data, &$output)
{
    // TODO: perform extra validation on xml specific to NSG
    exec('cp /usr/local/nsg/conf/dialplan/default.xml /usr/local/nsg/conf/dialplan/default.xml.ori');
    file_put_contents("/usr/local/nsg/conf/dialplan/default.xml", $data['content']);
    return true;
}
/*
 * SIP configuration
 */
function rest_retrieve_tdmgw_cfg_sip($data, &$output)
{
    $sipdata = file_get_contents("/usr/local/nsg/conf/sip_profiles/internal.xml"); 
    $output = array('content'=>$sipdata);
    return true;
}
function rest_update_tdmgw_cfg_sip($data, &$output)
{
    exec('cp /usr/local/nsg/conf/sip_profiles/internal.xml /usr/local/nsg/conf/sip_profiles/internal.xml.ori');	
    file_put_contents("/usr/local/nsg/conf/sip_profiles/internal.xml", $data['content']);
    $output = array('content' => "SIP cfg restored successfully");	
    return true;
}
/*
 * Span configuration
 */
function rest_retrieve_tdmgw_cfg_span($data, &$output)
{
    /* Due to some nsg sql statement recreating same records, the dump result is
     * not exactly in the same order depending on performed actions (even if
     * content is strickly identical). This is mostly the case for 2 tables
     * being regenerated over again when accessing to TDM cfg page:
     * finished_ports and generated_ports.
     * Thus to workaround this issue we will create the dump manually
     * ensuring table are always in the same order.
     * This is a combination of 2 sql execution:
     * 1/ get the list of tables in db (ordered)
     *    SELECT '.dump ' || name FROM sqlite_master WHERE type='table' order  by name;
     * 2/ the above result is then store to a file we can pipe to sqlite to
     * full dump (we need a temp file as .dump from cli for a specific table
     * doesn't work).
     */
    $dump_list = shell_exec(
        "sqlite3 sqlite/cardinfo.db \"SELECT '.dump ' || name FROM sqlite_master WHERE type='table' order by name;\"");
    file_put_contents("/tmp/sql-dump-list.sql",
        $dump_list);

    $dump = shell_exec("sqlite3 sqlite/cardinfo.db < /tmp/sql-dump-list.sql");

    //$dump = shell_exec('sqlite3 sqlite/cardinfo.db .dump');
    if(NULL != $dump){
        $output = array('content' => $dump);
        return true;
    }
    // TODO: get the error
    return false;
}
function rest_update_tdmgw_cfg_span($data, &$output)
{
    exec('sqlite3 sqlite/cardinfonew.db < '.$_POST['data']);
    exec('cp -f cardinfo.db cardinfo.db.webgui.ori');
    exec('cp -f cardinfonew.db cardinfo.db');
   return true;
}
/*
 * Service related functions
 */
function _service_ctl($ctl, $data, &$output){
    exec("service nsg ".$ctl, $_status, $_output);
    $output = array('status'=>$_status,'status_text'=>get_status_text($_status)); 
    return true;
}
function rest_status_tdmgw_service($data, &$output)
{ 
    return _service_ctl('status', $data, $output);
}
function rest_start_tdmgw_service($data, &$output)
{ 
    return _service_ctl('start', $data, $output);
}
function rest_stop_tdmgw_service($data, &$output)
{ 
    return _service_ctl('stop', $data, $output);
}
/*
 * Global configuration functions
 */
// TODO: probably remove following function
function _rest_tdmgw_configuration_status($data, &$output)
{
    $db_location = "sqlite:sqlite/cardinfo.db";
    $tdm_config = new wanrouter_hwprobe_controls;
    $data = new database_class;
    $tokenization = new wanrouter_hwprobe;
    $display_common = new Display_common;
    $relaytable = "sqlite:sqlite/relay_info.db";
    exec("service nsg status", $output, $response);
    $messages['data']['can_apply'] = ($response == 0 ? false : true);
    $messages['data']['can_reload'] = false;
    $messages['data']['status_text'] = "ok";
    $messages['data']['status'] = true;
    $relay_result = $data->select_statement('sqlite:sqlite/relay_info.db','ss7_relay','machine_type, relay_type','id=1');

    if ($reset)
        $messages['error']['Configuration'][] = 'All gateway configuration has been reset';
    elseif ($restore)
        $messages['error']['Configuration'][]= 'All gateway configuration files have been restored to last generated values';

    if (!check_tdm_configured($data, $db_location)) {
        $messages['error']['Configuration'][]= 'In order to generate configuration, at least 1 TDM Span must be configured.';
    }	
    $out_of_sync = false;
    //Checks for completed ports on a standalone ss7 or multifunction gateway application
    if (strtolower($relay_result['relay_type']) == "slave")
    {

        $messages['error'][] = 'Relay, System configured for SLAVE Relay. Please see Relay Master to generate TDM configuration';

    }
    $check = $data->check_for_table($db_location,'finished_ports');
    if ($check > 1)
    {
        $result = $data->select_statement($db_location, "finished_ports", "id", "");	

        if (count($result) > 1)
        {
            //Check wanrouter status
            exec ("/usr/sbin/wanrouter status | grep -c wanpipe",$output);
            if ($output[0] == "DISABLED" || $output[0] == "INVALID KEY" || $output[0] == 0) {
                if ( !isTdmConfigInSync($db_location) )
                    $out_of_sync = true;
            }
            elseif ($output[0] > 0)
            {
                //Check if the gateway is started on each machine then check if each relay database is in sync
                $gateway_started = "yes";
                if ( !isTdmConfigInSync($db_location) )
                    $out_of_sync = true;
            }
        }
    }

    // Check if TDM configuration is in sync
    if ($relay_result['machine_type'] == "standalone" || ($relay_result['machine_type'] == "relay" && $relay_result['relay_type'] == "MASTER"))	{
        if ( $out_of_sync)
        {
            if ( $slave_error > 0 )
            {
                $messages['data']['modified'] = true;
                $messages['data']['description']= 'Configuration out of sync.';
                $messages['error']['TDM'][] = 'Cannot generate configuration until all Relay Nodes active';
            }
            elseif ($slave_error == 0 || $slave_error == "")
            {
                $messages['data']['modified'] = true;	
                $messages['data']['description']= 'Configuration out of sync.';
                $messages['error']['TDM'][] = 'Cannot generate configuration until all Relay Nodes active';
                if ( $gateway_started == "yes") {
                    //check if there has been a span removed and gateway is started.
                    $finished_boards = $data->array_select_statement($db_location,'finished_ports','card, type','');
                    $generated_boards = $data->array_select_statement($db_location,'generated_ports','card, type','');
                    foreach ($generated_boards as $gen_board) {
                        $found = false;
                        foreach ($finished_boards as $fin_board) {
                            if ($fin_board['card'] == $gen_board['card']) {
                                $found = true;
                                break;
                            }
                        }

                        if (!$found && $_SESSION['shortname'] == "NSG" && ($gen_board['type'] == "M2UA Signalling Gateway" || $gen_board['type'] == "Media Gateway")) {
                            $messages['error']['TDM'][] = $gen_board['card'] . " has been removed. Please <a href='/admin/ss7_profile_panel.php' target='_parent'>Synchronize</a> configuration.";
                            $skip_out_of_sync = true;
                        }
                        elseif (!$found && $_SESSION['shortname'] == "NSG") {
                            $messages['error']['TDM'][] = $gen_board['card'] . " has been removed. Please <a href='/admin/ss7_controls.php' target='_parent'>Stop the Gateway</a> in order to re-configure the span.";
                            $skip_out_of_sync = true;
                        }
                    }

                    if (!$skip_out_of_sync) {
                        $found = false;
                        foreach ($finished_boards as $fin_board) {
                            if ($fin_board['type'] != "Media Gateway" && $fin_board['type'] != "M2UA Signalling Gateway") {
                                $found = true;
                                break;
                            }
                        }

                        if (!$found) {
                            if(isMGConfigInSync($db_location)) {
                                $messages['error']['TDM'][] = "<a href='/admin/ss7_profile_panel.php' target='_parent'>Synchronize</a> configuration.";
                            }
                        }else{
                            $messages['error']['TDM'][] = "<a href='/admin/ss7_controls.php' target='_parent'>Stop</a> the gateway";
                        }
                    }
                }

                $messages['error']['TDM'][] = "Generate TDM configuration.";


            }
        }
        elseif ($gateway_started != "yes" && $slave_error == 0 && count($result) > 0)
        {
            $messages['error']['TDM'][] = "Start the <a href='/admin/ss7_controls.php' target='_parent'>gateway</a>";
        }
        elseif ($gateway_started != "yes" && $slave_error != 0)
            $messages['error'][] = "TDM, Cannot start the gateway until all Relay Nodes active";
    }
    //Run Transcoding configuration check
    if (transcode_check($notif,$data, $db_location))
        $out_of_sync = true;

    //Run Megaco configuration check
    if ($relay_result['machine_type'] == "standalone" || ($relay_result['machine_type'] == "relay" && $relay_result['relay_type'] == "MASTER"))	{
        if (mg_check($notif,$data, $db_location))
            $out_of_sync = true;
    }
    if (!$out_of_sync) {
        $messages['data']['modified'] = false;
        $messages['data']['description']= 'Configuration up to date';
        if ($gateway_started == "yes") {
            $href = "<a href='/admin/ss7_controls.php' target='_parent'>Control Panel</a>";
            $messages['error']['Configuration'][] = 'If you need to regenerate your configuration, please go to the ' . $href . ' to stop the gateway.';
        }
        else {
            $messages['error']['Configuration'][] = 'If you need to regenerate your configuration, please click on <b>Generate Config</b>';	
        }
    }
    $sctp_status = false;
    $sig_ip_status = false;
    $rtp_ip_status = false;
    check_ip_exist($data,$db_location,$sctp_status, $sig_ip_status, $rtp_ip_status);
    if (!$sctp_status)
        $messages['error']['SCTP'][] = 'One of your SCTP Profiles contains an invalid IP Address. Please adjust your SCTP Profiles';
    if (!$sig_ip_status)
        $messages['error']['Media Gateway Profile'][] = 'One of your Media Gateway profiles contains an invalid Signalling IP Address. Please adjust your Media Gateway Profiles';
    if (!$rtp_ip_status)
        $messages['error']['Media Gateway Profile'][] = 'One of your Media Gateway profiles contains an invalid RTP IP Address. Please adjust your Media Gateway Profiles';

    $mediaObj = new prod_test();
    $display_common->get_license_status($data,$mediaObj,$license_err, $channel_err, $trial_license_error, $db_location);
    $mediaObj = NULL;	
    if ( $license_err!= "" )
        $messages['error']['License'][] = $license_err;
    if ( $channel_err!= "" )
        $messages['error']['Channels'][] = $channel_err;
    if (isset($trial_license_error))
        $messages['error']['Timed Trial'][] = $trial_license_error;	
    if(count(($messages['error'])>0)){
        $messages['data']['status_text'] = "error";
        $messages['data']['status'] = false;
        $i = 0;
        foreach($messages['error'] as $objecttype =>$objects)
        {	
            foreach($objects as $description)
            {
                $items[$i]['module']= 'tdm';
                $items[$i]['obj_type'] = strtolower($objecttype);
                $items[$i]['description'] = $description;
                $i++;
            }
        }
        unset($messages['error']);
    }
    $messages['apply']['items'] = $items;
    $output = $messages;
    return (isset($messages['error']));
}
/**
 * @brief Reload configuration. 
 * If FS is running we'll issue a fs_cli -x "reload_xml"
 * If not then we can either fail silently or raise an error, I do prefere 
 * former for now.
 * This only applies to sip and dialplan of course.
 * @return 
 */
function rest_reload_tdmgw_configuration($data, &$output)
{
    shell_exec ('/usr/sbin/nsg_cli -x "reloadxml"');
}

function rest_status_tdmgw_cfg_span($data, &$output)
{
    $db_location = "sqlite:sqlite/cardinfo.db";
    $tdm_config = new wanrouter_hwprobe_controls;
    $data = new database_class;
    $tokenization = new wanrouter_hwprobe;
    $display_common = new Display_common;
    if (isTdmConfigInSync($db_location) ){
        return true;
    }
    $output = array('description'=>'Apply configuration failed.');
    return false;
}

function rest_apply_tdmgw_configuration($data, &$output)
{
    $_mode = "";
    if (isset($data['data']) && isset($data['data']['mode'])){
        $_mode = $data['data']['mode'];
    }
    // Some include that will be required later in processing
    require_once 'sqlite/database_creation.php';
    require_once 'wanpipe_config/wanpipe.php';

    $db_location = "sqlite:sqlite/cardinfo.db";
    $tdm_config = new wanrouter_hwprobe_controls;
    $data_class = new database_class;
    $tokenization = new wanrouter_hwprobe;
    $display_common = new Display_common;
    $relaytable = "sqlite:sqlite/relay_info.db";	
    $data_class->drop_all_tables($db_location);
    $_SESSION['embedded'] = isset($_REQUEST['embedded'])?$_REQUEST['embedded']:$_SESSION['embedded'];	
    $display_common->session_vars("sqlite:sqlite/cardinfo.db");
    // Generate is controlled by wanrouter_control.php and the giant clumsy 
    // redirect() function, we have to let it believe this is a REQUEST from 
    // browser (emulate a $_REQUEST content)
    $_request = array(
        'generate' => "Generate Config"
    );
    cfg_display_files_rest($_request, $tdm_config,$tokenization,$data_class,$_mode,$relaytable);
    if (isTdmConfigInSync($db_location) ){
        return true;
    }
    $output = array('description'=>'Apply configuration failed.');
    return false;
}
function get_status_text($status)
{
    switch($status)
    {
    case -1: return 'ERROR';
    case 1: return 'UNRECOGNIZED';
    case 0: return 'RUNNING';
    case 3: return 'STOPPED';
    case 2:	return 'DEAD';
    default: return $status;
    }
}

