#!/usr/bin/env python2.7
# vim:ts=4:sw=4:softtabstop=4:smarttab:expandtab
"""
* Copyright (C) 2013  Sangoma Technologies Corp.
* All Rights Reserved.
*
* Author(s)
* Tyler Goodlet <tgoodlet@sangoma.com>
* William Adam <wadam@sangoma.com>
*
* This code is Sangoma Technologies Confidential Property.
* Use of and access to this code is covered by a previously executed
* non-disclosure agreement between Sangoma Technologies and the Recipient.
* This code is being supplied for evaluation purposes only and is not to be
* used for any other purpose.

Push rest configurations via safepy + json

"""
import sys
import os
import json
from optparse import OptionParser

# Create logger
import logging
import logging.handlers
logger = logging.getLogger(os.path.basename(os.path.abspath(sys.argv[0])))
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
logger.addHandler(handler)

# Create OUR libs path
base = os.path.dirname(os.path.realpath(__file__))
lib_path = os.path.normpath(base + '/../libs')
# Append OUR lib to path
sys.path.append(lib_path)


def main(argv):

    parser = OptionParser()
    parser.add_option("-f", "--config-file", dest="conf_path",
                      help="config file (JSON)")
    parser.add_option("--package", dest="safe_package",
                      help="path to safepy python package")

    (options, args) = parser.parse_args()
    path = options.conf_path
    if path is None:
        raise ValueError("a config file path must be provided!")

    # import the requested safepy module
    package_name = options.safe_package

    # default to product_release
    if package_name is None:
        package_name = 'product_release'

    # import the requested safepy module
    module = __import__(package_name)
    # create product
    product = module.Product(logger)

    rc = -1

    # read configuration file
    with open(options.conf_path) as f:
        cfg_data = f.read()
        # update system configuration
        response = product.system.configuration.update(
                {
                    "mode": "update",
                    "data": cfg_data
                }
                )

        if not response:
            # something went wrong, get last error and log
            logger.error("Update failed with " + json.dumps(product.last_error))
            logger.error("Reponse: " + product.last_response)
        else:
            logger.error("Update succed with " + json.dumps(response))
            rc = 0

    return rc

if __name__ == '__main__':
    try:
        sys.exit(main(sys.argv))
    except SystemExit:
        raise
    except Exception, e:
        sys.stderr.write("Exception caught: %s\n" % (e))
        raise
