#!/usr/bin/env python2.7
# vim: tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab
"""
* Copyright (C) 2015   Sangoma Technologies Corp.
* All Rights Reserved.
*
* Author(s)
* Leonardo Lang <lang@sangoma.com>
"""

import os
import sys
import logging
import logging.handlers

from ConfigParser import SafeConfigParser

import xml.dom.minidom as xml

from optparse import OptionParser

# Sample output:
#
# <program>
#     <name>Cluster Monitor</name>
#     <message>Output message</message>
# </program>
#
# rc =  0 => success
# rc = -1 => failure

MONITOR_CONFIG = '/usr/local/sng/conf/clustermon.conf'
PENDING_CONFIG = '/var/sng/cluster/pending-synchronization.json'

def get_sync_status(logger):
    return os.path.exists(PENDING_CONFIG) \
        if os.path.exists(MONITOR_CONFIG) else None

def do_check(logger, options):
    rc, status = 0, get_sync_status(logger)

    if status is None:
        message = 'Not configured'
    elif status:
        message = 'OK'
    else:
        message = 'Cluster configuration not synchronized between nodes.'
        rc = -1

    # Create output
    doc = xml.Document()

    root = doc.createElement('program');
    name = doc.createElement('name')
    text = doc.createTextNode("Cluster Monitor")
    name.appendChild(text)
    root.appendChild(name)

    mesg = doc.createElement('message')
    text = doc.createTextNode(message)
    mesg.appendChild(text)
    root.appendChild(mesg)

    # output result
    return rc, root


def main():
    parser = OptionParser()
    parser.add_option('-p', '--pretty', action='store_true', default=False, help="pretty print XML output")
    (options, args) = parser.parse_args()

    # Create logger
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)
    logger.addHandler(logging.StreamHandler())

    # Execute Check
    rc, xmldata = do_check(logger, options)
    sys.stdout.write(xmldata.toprettyxml() if options.pretty else xmldata.toxml())
    sys.stdout.flush()
    return rc

if __name__ == '__main__':
    sys.exit(main())
