#!/bin/bash
exec 0>&- # close stdin
exec 1>&- # close stdout
exec 2>&- # close stderr

PROG=${0##*/}

on_exit() {
  true
}

restart()
{
  logger -t $PROG Restart Requested
  webpid=`cat /var/run/webconfig.pid`

  # check if running
  if [ "w${webpid}" = "w" ]
  then
    logger -t $PROG Graceful Restart Skipped
    exit 0
  fi

  # check if webconfig was updated
  webexe="$(readlink /proc/${webpid}/exe)"
  if [ -e "${webexe}" ]
  then
    logger -t $PROG Graceful Restart Running
    /etc/init.d/webconfig graceful-restart > /dev/null 2>&1
    logger -t $PROG Graceful Restart Complete = $?
  else
    logger -t $PROG Full Restart Waiting
    sleep 5
    logger -t $PROG Full Restart Running
    /etc/init.d/webconfig graceful-stop > /dev/null 2>&1 && \
    /etc/init.d/webconfig start > /dev/null 2>&1
    logger -t $PROG Full Restart Complete = $?
  fi
}

fork() { 
  logger -t $PROG Forking \"$@\"
  ("$@" &)
  logger -t $PROG Fork Complete = $?
}

trap 'on_exit' SIGHUP SIGTERM

fork fork restart
