#!/usr/bin/env python2.7
# vim: tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab
"""
* Copyright (C) 2016   Sangoma Technologies Corp.
* All Rights Reserved.
*
* Author(s)
* Leonardo Lang <lang@sangoma.com>
"""

import os
import sys
import re

import subprocess
import netifaces
import jinja2

from argparse import ArgumentParser

PROD_DEF_TEMPLATE = '/usr/local/sng/conf/prod-def-template.xml'

NET_DEVICES_REGEX = re.compile('^eth[0-9]+$')

def get_cpu_count():
    proc = subprocess.Popen(['/usr/bin/getconf', '_NPROCESSORS_ONLN'], stdout=subprocess.PIPE)
    data = proc.communicate()[0].strip()

    try:
        data = int(data)
    except Exception as e:
        print >> sys.stderr, 'could not convert "{0!s}" to int, using cpu_count = 1'.format(data)
        data = 1

    if proc.wait() != 0:
        print >> sys.stderr, 'getconf returned {0!s}'.format(proc.returncode)

    return data

def get_network_devs():
    return sorted(name for name in netifaces.interfaces() if NET_DEVICES_REGEX.match(name))

def generate_file(filename, **kwargs):
    env = jinja2.Environment(loader=jinja2.FileSystemLoader(os.path.dirname(PROD_DEF_TEMPLATE)), trim_blocks=True)
    cfg = env.get_template(os.path.basename(PROD_DEF_TEMPLATE))

    kwargs.update(network_devs=get_network_devs(), cpu_count=get_cpu_count())

    print >> sys.stderr, 'generating product definition at "{filename}" with: {kwargs!s}'.format(**locals())

    with open(filename, "wb") as fdes:
        fdes.write(cfg.render(**kwargs))

def main():
    parser = ArgumentParser()

    parser.add_argument('--attr', nargs='*', metavar='KEY=VAL', default=list(), help='set template attribute KEY to VAL')
    parser.add_argument('--output', help='output XML filename')

    options = parser.parse_args()

    if options.output is None:
        print >> sys.stderr, "no output file given, aborting"
        return 123

    attrs = dict(map(lambda (key, val): ('attr_{key}'.format(**locals()), val), [ tuple(e.split('=', 1)) for e in options.attr ]))
    return generate_file(options.output, **attrs)

if __name__ == '__main__':
    sys.exit(main())
