#!/bin/sh
HW_IFACES=($(ls -1 /etc/sysconfig/network-scripts/ifcfg-* | grep -v lo | grep -v bond))
NUM_HW_IFACES=$((${#HW_IFACES[@]}))
mac_changed=0
SAFE_DATABASE_FILE=/var/webconfig/SAFe/application/db/safe.db
for (( i=0; i<$NUM_HW_IFACES; i++ ))
do
    #echo "${HW_IFACES[i]}"
    ETH_PORT=$(echo "${HW_IFACES[i]}" | cut -c 38-)
    IF_MAC=$(cat ${HW_IFACES[i]} | grep "HWADDR=" | cut -c 8-)
    NETCLASS_MAC=($(cat "/sys/class/net/$ETH_PORT/address" ))

    if [[ -z "${NETCLASS_MAC// }" ]]; 
    then
       # This means that we have a port in the network 
       # configuration that is not present on the new system
       # let's remove that configuration and also force and reset.
       echo "Interface $ETH_PORT not found in /sys/class/net"
       rm -rf ${HW_IFACES[i]}
       mac_changed=$((mac_changed+1))
    else
       match_mac=$(grep $NETCLASS_MAC ${HW_IFACES[i]})
       echo "Comparing ifcfg-$ETH_PORT hw address $IF_MAC to sys/class/net/ mac $NETCLASS_MAC"
       if [ -z "$match_mac" ];
       then
          echo "Different hardware address for $ETH_PORT"
          # replace the HWADDR=IF_MAC to HWADDR=NETCLASS_MAC in the ifcfg file
          sed -i "s/HWADDR=$IF_MAC/HWADDR=$NETCLASS_MAC/" ${HW_IFACES[i]}
          mac_changed=$((mac_changed+1))
       else
           if [[ $ETH_PORT = *"sngdsp"* ]]; then
               ONBOOT=$(cat ${HW_IFACES[i]} | grep "ONBOOT=" | cut -c 8-)
               if [ "$ONBOOT" = "no" ] || [ "$ONBOOT" = "\"no\"" ] ; then
                   echo "Detected that $ETH_PORT is not configured to startup on boot, forcing network restore hook"
                   mac_changed=$((mac_changed+1))
               fi
           fi
       fi
    fi
done

if [ "$mac_changed" -gt 0 ] && [ -f $SAFE_DATABASE_FILE ]; then
    echo "Resetting network config"
    service network restart
    /usr/local/nsc/hooks/network.restore.post --mode=restore
    /usr/local/nsc/hooks/network.restore.post --mode=update
fi
