<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Reza Samimi <rsamimi@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
 */
/*
 * FS ACL module class
 */

if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_service_class');
safe_module_require_class('fs', 'application');
safe_module_require_class('fs', 'acl_rule');
safe_module_require_class('fs', 'acl_user_acl');
safe_module_require_class('fs', 'acl_user_acl_zone');

class Fs_acl_module_class extends Fs_module_class {
    private $_db = null;
	protected $_obj_base_path = '';
    public function is_enabled($mod_name = "")
    {
        $user_objs = $this->get_aggregate_objects('user_acl');
        if ($user_objs) return true;
        return false;
    }
    public function is_critical($mod_name = "")
    {
        return true;
    }
    /**
     * @brief Return FS module name
     *         
     * @return string
     */
    public function get_fs_module_name()
    {
        return Fs_acl_user_acl_class::get_module_name() . '.so';
    }
    /**
     * @brief
     *
     * @param[in out] $fs_app
     *
     * @return
     */
    public function __construct($fs_app)
    {
        parent::__construct($fs_app, "acl");
        $this->_obj_base_path = $this->path().'/acl';
    }
    /**
     * @brief
     *
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("Access Control List");
        $this->register_aggregate_object('network_list',
            array(
                'name' => 'ACL',
                'dynamic' => true,
                'base_path' => $this->_obj_base_path.'/acl',
                'has_child' => true,
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a new ACL',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve an ACL item',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update an ACL item',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete an ACL item',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('user_acl',
            array(
                'name' => 'User ACL File',
                'dynamic' => true,
                /* Add some extra infos for config_manager create object */
                'controller' => array(
                        'config_manager' => array(
                                'summary_buttons' => 'Delete:confirm',
                                'allow_create_button' => 'upload[Upload]'
                        ),
                ),
                'autoname' => true,
                'popup_add_form' => true,
                'file_def' => array(
                        'filesystem' => true,
                        'file_type' => 'csv'
                ),
                'methods' => array(
                        'create' => array(
                                'icon' => 'create',
                                'name' => 'Create',
                                'description' => 'Create a user ACL',
                                'request' => 'POST',
                        ),
                        'upload' => array(
                                'icon' => 'upload',
                                'name' => 'Upload',
                                'description' => 'Upload a user ACL file',
                                'request' => 'POST',
                        ),
                        'delete' => array(
                                'name' => 'Delete',
                                'description' => 'Delete a user ACL file',
                                'request' => 'POST',
                        ),
                ),
            )
        );
        $this->register_aggregate_object('user_acl_zone',
            array(
                'name' => 'ACL Zone File',
                'dynamic' => true,
                /* Add some extra infos for config_manager create object */
                'controller' => array(
                        'config_manager' => array(
                                'summary_buttons' => 'Delete:confirm',
                                'allow_create_button' => 'upload[Upload]'
                        ),
                ),
                'autoname' => true,
                'popup_add_form' => true,
                'file_def' => array(
                        'filesystem' => true,
                        'file_type' => 'csv'
                ),
                'methods' => array(
                        'create' => array(
                                'icon' => 'create',
                                'name' => 'Create',
                                'description' => 'Create a ACL zone',
                                'request' => 'POST',
                        ),
                        'upload' => array(
                                'icon' => 'upload',
                                'name' => 'Upload',
                                'description' => 'Upload a ACL zone file',
                                'request' => 'POST',
                        ),
                        'delete' => array(
                                'name' => 'Delete',
                                'description' => 'Delete a ACL zone file',
                                'request' => 'POST',
                        ),
                ),
            )
        );
        $this->db();
        return parent::configure();
    }
    /**
     * @brief Create config jobs for ACL module
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        if(!$this->_generate_config($config_manager)) return false;
        return parent::reload_generate_config($config_manager, $obj_type);
    }

    /**
     * @brief
     *
     * @param[in out] $obj_type
     *
     * @return
     */
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }

    /**
     * @brief NSC db
     *
     * @return
     */
    function db()
    {
        if(!$this->_db) {
            $cfg_db = array (
                            'hostname' => 'localhost',
                            'username' => 'root',
                            'password' => '',
                            'port' => '3306',
                            'database' => 'nsc',
                            'dbdriver' => 'mysqli',
                            'dbprefix' => '',
                            'pconnect' => 'false',
                            'db_debug' => 'true',
                            'cache_on' => 'false',
                            'cachedir' => '',
                            'char_set' => 'utf8',
                            'dbcollat' => 'utf8_general_ci',
                            'active-record' => 'true',
                            );
            $this->_db = Safe_database_class::factory($cfg_db);
        }
        return $this->_db;
    }

    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    public function generate_config(&$config_manager)
    {
        if(!$this->_generate_config($config_manager)) return false;
        return parent::generate_config($config_manager);
    }
    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_acl_config(&$config_manager)
    {
        // Cleanup
        $directory = "autoload_configs/acl.conf.xml";
        $config_manager->add_config(
            new Safe_configuration_class(
                $directory, '',
                Safe_configuration_class::CFG_DELETE,
                Safe_configuration_class::CFG_DIR
            )
        );

        // generate the xml configuration
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startDocument('1.0', 'UTF8');
        $xml_writer->startElement("configuration");
        $xml_writer->writeAttribute('name', "acl.conf");
        $xml_writer->writeAttribute('description', 'Network List');

        $xml_writer->startElement("network-lists");

        // Write ACL rules and nodes
        foreach($this->api_retrieve_network_list() as $acl) {
            // Write the rule with respective node elements
            $acl_network_list_cfg = $acl->get_data_values(false);
            $acl_name = $acl->name();

            $xml_writer->startElement("list");
            $xml_writer->writeAttribute('name', $acl_name);
            // Set ACL default policy
            if ($acl_network_list_cfg['default-policy'] == 'true'){
                $xml_writer->writeAttribute('default', 'allow');
            } elseif ($acl_network_list_cfg['default-policy'] == 'false'){
                $xml_writer->writeAttribute('default', 'deny');
            }

            // iterate through the list of nodes for each ACL
            $conditions_def = $acl->get_aggregate_object_definition();
            foreach ($conditions_def as $condition_type => $condition_def) {
                $conditions = $acl->get_aggregate_objects($condition_type);

                foreach ($conditions as $condition) {
                    $cond = $condition->get_data_values(false);
                    // TODO: commented domain_acl out for future use
                    // if ($cond['type'] == 'domain_acl') {
                    //     $xml_writer->startElement("node");
                    //     $xml_writer->writeAttribute('type', $cond['policy']);
                    //     $xml_writer->writeAttribute('domain', 'domain-name');
                    //     $xml_writer->endElement();
                    // }
                    // else if ($cond['type'] == 'cidr_acl') {
                        $xml_writer->startElement("node");
                        $xml_writer->writeAttribute('type', $cond['policy']);
                        $def_prefix = filter_var($cond['ip-address'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) ? '32' : '128';
                        $pfx = $cond['prefix'] ? $cond['prefix'] : $def_prefix;
                        $xml_writer->writeAttribute('cidr', $cond['ip-address'] . "/" . $pfx);
                        $xml_writer->endElement();
                    // }
                    // else return false;
                }
            }

            $xml_writer->fullEndElement(); // end element for list
        }
        $xml_writer->fullEndElement(); // end element for network-lists
        $xml_writer->fullEndElement(); // end element for configuration

        $xml_writer->endDocument();
        // Create directory
        $directory = "autoload_configs/";
        $config_manager->add_config(new Safe_configuration_class($directory, '', Safe_configuration_class::CFG_CREATE, Safe_configuration_class::CFG_DIR));
        $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
        $file = "autoload_configs/acl.conf.xml";
        $config_manager->add_config(new Safe_configuration_class($file, $xmlstr));
        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_user_acl_module_config(&$config_manager)
    {
        // Cleanup
        $directory = "autoload_configs/user_acl.conf.xml";
        $config_manager->add_config(
            new Safe_configuration_class(
                $directory, '',
                Safe_configuration_class::CFG_DELETE,
                Safe_configuration_class::CFG_DIR
            )
        );

        // generate the xml configuration
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startDocument('1.0', 'UTF8');
        $xml_writer->startElement("configuration");
        $xml_writer->writeAttribute('name', "user_acl.conf");
        $xml_writer->writeAttribute('description', 'User ACL');
        $xml_writer->startElement("settings");
        $xml_writer->startElement("param");
        $xml_writer->writeAttribute('name', "odbc-dsn");
        $xml_writer->writeAttribute('value', "nsc:root:");
        $xml_writer->fullEndElement(); // end element for param 
        $xml_writer->fullEndElement(); // end element for settings
        $xml_writer->fullEndElement(); // end element for configuration
        $xml_writer->endDocument();
        // Create directory
        $directory = "autoload_configs/";
        $config_manager->add_config(new Safe_configuration_class($directory, '', Safe_configuration_class::CFG_CREATE, Safe_configuration_class::CFG_DIR));
        $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
        $file = "autoload_configs/user_acl.conf.xml";
        $config_manager->add_config(new Safe_configuration_class($file, $xmlstr));
        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_user_acl_db_config(&$config_manager)
    {
        // Cleanup
        $db = $this->_db;
        $db->connect();
        $db->query('DROP TABLE IF EXISTS user_acl;');
        $db->query('CREATE TABLE IF NOT EXISTS `user_acl` (`user_id`
                    varchar(255) default NULL,`user_domain` varchar(255) default
                    NULL, `zone_name` varchar(255) default NULL,PRIMARY KEY
                    (`user_id`, `user_domain`),INDEX (zone_name)) ENGINE=InnoDB;');
        $db->disconnect();
        // Insert
        $users = $this->get_aggregate_objects('user_acl');
        unset($data['data']);
        $data['data'] = array();
        $data['db_class'] = $db;
        foreach($users as $user){
            $user->create_load_infile($data['data'], $keys);
            $config_manager->add_config(new Safe_configuration_class('user_acl',
                        $data, Safe_configuration_class::CFG_LOAD, Safe_configuration_class::CFG_SQL));
        }

        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_user_acl_zone_db_config(&$config_manager)
    {
        // Cleanup
        $db = $this->_db;
        $db->connect();
        $db->query('DROP TABLE IF EXISTS user_acl_zone;');
        $db->query('CREATE TABLE IF NOT EXISTS `user_acl_zone` (`zone_name`
                    varchar(255) default NULL,`ip_list` varchar(4096) default
                    NULL,PRIMARY KEY  (`zone_name`)) ENGINE=InnoDB;');
        $db->disconnect();
        // Insert
        $zones = $this->get_aggregate_objects('user_acl_zone');
        unset($data['data']);
        $data['data'] = array();
        $data['db_class'] = $db;
        foreach($zones as $zone){
            $zone->create_load_infile($data['data'], $keys);
            $config_manager->add_config(new Safe_configuration_class('user_acl_zone',
                        $data, Safe_configuration_class::CFG_LOAD, Safe_configuration_class::CFG_SQL));
        }

        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_config(&$config_manager)
    {
        if(!$this->_generate_acl_config($config_manager)) return false;
        if(!$this->_generate_user_acl_db_config($config_manager)) return false;
        if(!$this->_generate_user_acl_zone_db_config($config_manager)) return false;
        if(!$this->_generate_user_acl_module_config($config_manager)) return false;
        return true;
    }
    /**
     * @brief Invoked after a successfull write_config
     *
     * @return
     */
    public function post_write_config()
    {
        // Check if FS is running
        if (Safe_service_class::STATUS_RUNNING == $this->fs_app()->status()) {
            $output = array();
            $rc = $this->eslapi_execute("reloadxml", '', $output);
            if ($rc) {
                $rc = $this->eslapi_execute("reloadacl", '', $output);
            }
            return $rc;
        }
        return parent::post_write_config();
    }
     /**
     * @brief create a new ACL item
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_create_network_list($name, $data=null, &$output = null) {
    	$network_list = new Fs_acl_rule_class($this->node(), $this->_obj_base_path .'/acl', $name);
        $network_list->configure();
        if($data) {
            $network_list->set_data_values($data);
        }
        return $network_list;
    }

     /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_retrieve_network_list($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('network_list');
        if ($name) return $objs[$name];
        else return $objs;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_update_network_list($name, $data=null, &$output = null) {
        $network_list = $this->api_retrieve_network_list($name);
        if($network_list) {
            if ($network_list->validate($data,$output)) {
                if (true == $network_list->save()) {
                    return true;
                }
            }
        }
        return false;
    }
     /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_delete_network_list($name, $data=null, &$output = null) {
        $network_list = $this->api_retrieve_network_list($name);
        if ($network_list) {
            if($network_list->can_dispose($output)) {
                $conditions = $network_list->conditions('node');
                if (isset($conditions)) {
                    foreach ($conditions as $condition) $condition->dispose();
                }
                return $network_list->dispose();
            }
        }
        return false;
    }

    public function user_acl(){
        return $this->get_aggregate_objects('user_acl');
    }

    public function api_upload_user_acl($name=null, $data = null, &$output = null){
        $type = 'user_acl';
        $data_cfg = array(
            'description' => 'User ACL File',
            'filesystem' => true,
            'pattern' => '/tmp/upload-files/*.csv',
            'file_ext' => 'csv',
            'singleton' => false
        );

        $file_output = array();
        if(upload_file($type, $data_cfg, $file_output, $from = 'POST')){
            //read content, verified data
            $tmp_file_path =  $file_output['full_path'];
            $tmp_file_name =  $file_output['file_name'];
            if($tmp_file_path && $tmp_file_name){
                $user_data = file_get_contents($tmp_file_path);
                if($user_data){
                    $data['base64_encode_contents'] = base64_encode($user_data);
                    $data['file_name'] = $tmp_file_name;
                    if($name){
                        if(!$this->can_create_aggregate_object($type, $name, $reason)){
                            // Cannont create object, throw an error
                            $output['message'] = 'Create ' .$this->aggregate_object_name($type)." <b>" . $name . "</b> failed.";
                            $output['reason'] = $reason;
                            return false;
                        }else{
                            $tmp_obj_name = $name;
                        }
                    }else{
                        $tmp_obj_name = build_unique_name($type);
                    }
                    $output_msg = array();
                    $new_obj = $this->create_aggregate_object($type, $tmp_obj_name, $output_msg);
                    if($new_obj){
                        if($new_obj->validate($data, $output_msg)){
                            $new_obj->save();
                            return $new_obj;
                        } else {
                            if($output_msg){
                                $output['message'] = "Fail to create user ACL file.";
                                foreach($output_msg as $topic => $err){
                                    $err_string = "";
                                    if(is_array($err)){
                                        foreach($err as $t2 => $r2){
                                            if(is_array($r2)){
                                                $err_string .= $t2 . ':' . implode(' ', array_values($r2)) . '<br/>';
                                            }else{
                                                $err_string .= $t2 . ':' . $r2 . '<br/>';
                                            }
                                        }
                                    }else{
                                        $err_string =  $topic .'=>'.$err;
                                    }
                                    $output['reason'][] = $err_string;
                                }
                                return false; 
                            }
                        }
                    }
                }
            }
        }
        $output['message'] = "Fail to create user ACL file.";
        return false;
    }

     /**
     * @brief create a User ACL item
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_create_user_acl($name=null, $data=null, &$output=null) {
    	$user_acl = new Fs_acl_user_acl_class($this->node(), $this->_obj_base_path .'/user_acl', $name);
        $user_acl->configure();
        if($data) {
            $user_acl->set_data_values($data);
        }
        return $user_acl;
    }

    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
    public function api_download_user_acl($name, $data=null) {
        $objs = $this->api_retrieve_user_acl($name, $data);
        return $objs->download();
    }
     */

     /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_retrieve_user_acl($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('user_acl');
        if ($name) return $objs[$name];
        else return $objs;
    }
     /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_delete_user_acl($name, $data=null, &$output = null) {
        $user_acl = $this->api_retrieve_user_acl($name);
        if ($user_acl) {
            if($user_acl->can_dispose($output)) {
                return $user_acl->dispose();
            }
        }
        return false;
    }

    public function user_acl_zone(){
        return $this->get_aggregate_objects('user_acl_zone');
    }

    public function api_upload_user_acl_zone($name=null, $data = null, &$output = null){
        $type = 'user_acl_zone';
        $data_cfg = array(
            'description' => 'Zone ACL File',
            'filesystem' => true,
            'pattern' => '/tmp/upload-files/*.csv',
            'file_ext' => 'csv',
            'singleton' => false
        );
        $file_output = array();
        if(upload_file($type, $data_cfg, $file_output, $from = 'POST')){
            //read content, verified data
            $tmp_file_path =  $file_output['full_path'];
            $tmp_file_name =  $file_output['file_name'];
            if($tmp_file_path && $tmp_file_name){
                $zone_data = file_get_contents($tmp_file_path);
                if($zone_data){
                    $data['base64_encode_contents'] = base64_encode($zone_data);
                    $data['file_name'] = $tmp_file_name;
                    if($name){
                        if(!$this->can_create_aggregate_object($type, $name, $reason)){
                            // Cannont create object, throw an error
                            $output['message'] = 'Create ' .$this->aggregate_object_name($type)." <b>" . $name . "</b> failed.";
                            $output['reason'] = $reason;
                            return false;
                        }else{
                            $tmp_obj_name = $name;
                        }
                    }else{
                        $tmp_obj_name = build_unique_name($type);
                    }
                    $output_msg = array();
                    $new_obj = $this->create_aggregate_object($type, $tmp_obj_name, $output_msg);
                    if($new_obj){
                        if($new_obj->validate($data, $output_msg)){
                            $new_obj->save();
                            return $new_obj;
                        } else {
                            if($output_msg){
                                $output['message'] = "Fail to create user ACL zone file.";
                                foreach($output_msg as $topic => $err){
                                    $err_string = "";
                                    if(is_array($err)){
                                        foreach($err as $t2 => $r2){
                                            if(is_array($r2)){
                                                $err_string .= $t2 . ':' . implode(' ', array_values($r2)) . '<br/>';
                                            }else{
                                                $err_string .= $t2 . ':' . $r2 . '<br/>';
                                            }
                                        }
                                    }else{
                                        $err_string =  $topic .'=>'.$err;
                                    }
                                    $output['reason'][] = $err_string;
                                }
                                return false; 
                            }
                        }
                    }
                }
            }
        }
        $output['message'] = "Fail to create user ACL zone file.";
        return false;
    }

     /**
     * @brief create a User ACL Zone item
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_create_user_acl_zone($name=null, $data=null, &$output=null) {
    	$user_acl_zone = new Fs_acl_user_acl_zone_class($this->node(), $this->_obj_base_path .'/user_acl_zone', $name);
        $user_acl_zone->configure();
        if($data) {
            $user_acl_zone->set_data_values($data);
        }
        return $user_acl_zone;
    }

     /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_retrieve_user_acl_zone($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('user_acl_zone');
        if ($name) return $objs[$name];
        else return $objs;
    }
     /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_delete_user_acl_zone($name, $data=null, &$output = null) {
        $user_acl_zone = $this->api_retrieve_user_acl_zone($name);
        if ($user_acl_zone) {
            if($user_acl_zone->can_dispose($output)) {
                return $user_acl_zone->dispose();
            }
        }
        return false;
    }
    public function new_fields(&$config_manager, $type)
    {   
        if ($type == 'user_acl') {
            $config_manager->add_upload_field('user_acl_file','User ACL File','upload','',30,true);
            $config_manager->set_field_rules('user_acl_file','optional');
            $config_manager->add_field('save', 'Save', 'hidden', 'save', 10);
        } else if ($type == 'user_acl_zone') {
            $config_manager->add_upload_field('user_acl_zone_file','User ACL Zone File','upload','',30,true);
            $config_manager->set_field_rules('user_acl_zone_file','optional');
            $config_manager->add_field('save', 'Save', 'hidden', 'save', 10);
        } else {
           return parent::new_fields($config_manager, $type); 
        }
    }   
}
?>
