<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Reza Samimi <rsamimi@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * FS ACL Node parameters
**/
require_once ('application/helpers/safe_helper.php');
safe_module_require_class('fs', 'acl_network_list');
class Fs_acl_node_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        // domain_acl is commented out for future use
        // $this->add_enum_field('type', 'Type', 'dropdown', 'domain_acl', 
        //     array('domain_acl' => 'Domain', 'cidr_acl' => 'CIDR')
        // );
        $this->add_enum_field('policy', 'Policy', 'dropdown', 'allow',
            array('deny' => 'Deny', 'allow' => 'Allow')
        );
        $this->set_field_help('policy',"To accept or to reject the access for the specified CIDR");
        // domain_name is commented out for future use
        // $this->add_field('domain-name', 'Domain Name', 'string', '', 50);
        // $this->set_field_rules('domain-name', 'required[type,in,domain_acl]|valid_domain');
        // $this->set_field_help('domain-name', "Which domain's acl to be used");

        $this->add_field('ip-address', 'IP Address', 'text', '',40);
        $this->set_field_help('ip-address', 'Network address or the IP of a host');
        $this->set_field_rules('ip-address', 'required[type,in,cidr_acl]|valid_ip');

        $this->add_field('prefix', 'Prefix', 'text', '',5);
        $this->set_field_help('prefix', 
            array(
                'The network prefix for the target IP address.',
                '',
                'Optional: If not specified, a host rule is created.',
            )
        );
        $this->set_field_rules('prefix', 'valid_ip_prefix[ip-address]');

        $this->composite_layout('ip-address', array('prefix'));

        //////// Domain type commented out for future usage
        //if the type == 'domain' then disable the cidr fields and if 'CIDR' then disable domain field
        // $_data['domain_acl'][] = array('field_name' => 'domain-name', 'status' => 'enable');
        // $condition_data['domain_acl'][] = array('field_name' => 'ip-address', 'status' => 'disable');
        // $condition_data['domain_acl'][] = array('field_name' => 'prefix', 'status' => 'disable');
       
        // $condition_data['cidr_acl'][] = array('field_name' => 'domain-name', 'status' => 'disable');
        // $condition_data['cidr_acl'][] = array('field_name' => 'ip-address', 'status' => 'enable');
        // $condition_data['cidr_acl'][] = array('field_name' => 'prefix', 'status' => 'enable');
        
        // $this->create_group('acl-type',
        //     array(
        //         'domain-name',
        //         'ip-address',
        //         'prefix'
        //     ),
        //     array('default' => array( 'value' => 'true', 'status' => 'enable'),
        //         'condition' => $condition_data
        //     )
        // );
        // $this->conditional_control('type', 'acl-type');
                
        return parent::configure();
    }
    public function dispose()
    {
        if(parent::dispose()){
            $path_info = $this->path_info();
            $receivers = $this->node()->software()->application()->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name']);
            $sip_module = $this->node()->software()->application()->find_module_by_name('sip');
            $profiles = $sip_module->api_retrieve_profile();
            foreach($profiles as $profile){
                if ($profile->acl_check_inbound(basename(dirname($this->path()))))
                {
                    $profile->mark_modified();
                }
            }
            return true;
        }
        return false;
    }
    public function save()
    {
        $path_info = $this->path_info();
        $receivers = $this->node()->software()->application()->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name']);
        $result = parent::save();
        if($this->is_modified()){
            $sip_module = $this->node()->software()->application()->find_module_by_name('sip');
            $profiles = $sip_module->api_retrieve_profile();
            foreach($profiles as $profile){
                if ($profile->acl_check_inbound(basename(dirname($this->path()))))
                {
                    $profile->mark_modified();
                }
            }
        }
        return $result;
    }

    //////// Domain type commented out for future usage
    // public function summary($type = 'horizontal' , $long = false){
    //   $desc_custom = array('type', 'policy', 'domain-name','ip-address', 'prefix');
    //   $table_line = array("Name" => $this->name());
    //   foreach ($desc_custom as $desc_key){
    //       $table_line[$this->get_data_label($desc_key)] = $this->get_data_value($desc_key);
    //   }
    //   return array('data' => $table_line);
    // }

    // hide name from table view
    public function summary($type = 'horizontal' , $long = false){
        $table_line = parent::summary($type , true);
        array_shift($table_line['data']);
        return $table_line;
    }
}
?>
