<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Reza Samimi <rsamimi@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
 */
/**
 * FS Acess Control List Base Class
*/

require_once ('application/helpers/safe_helper.php');
safe_module_require_class('nsc', 'application');
safe_module_require_class('fs', 'acl_node');

class Fs_acl_rule_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    /**
     * @brief 
     *
     * @return 
     */
    public function configure()
    {
        $this->add_enum_field('default-policy', 'Default Policy', 'dropdown', 'false',
            array('false' => 'Deny', 'true' => 'Allow')
        );
        $this->set_field_help('default-policy',"When peer's IP address does NOT fall into any ACL node of the applied ACL, what operation shall be taken");

        // Register objects
        $this->register_aggregate_object('node', 
            array(
                'name' => 'ACL Nodes',
                'dynamic' => true,
                'base_path' => $this->object_name() . '/node',
                'autoname' => true,
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create an ACL node',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve an ACL node',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update an ACL node',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete an ACL node',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        return parent::configure();
    }

    /**
     * Pulls ACL conditions based on rule name and rule type
     *
     * @return array $_conditions
     * Array of conditions based on rule type and rule name
     */
    public function conditions($obj_type=null)
    {
        $conditions = array();
        if($obj_type) {
            $conditions = $this->get_aggregate_objects($obj_type);
        } else {
            $objs_def = $this->get_aggregate_object_definition();

            foreach($objs_def as $obj_type => $obj_def) {
                $tmp = $this->get_aggregate_objects($obj_type);
                if($tmp) {
                    $conditions = array_merge($conditions, $tmp);
                }
            }
        }

        return $conditions;
    }
    /**
     * Returns the ACL nodes in an array
    */
    public function nodes() {
        $this->_nodes = $this->get_aggregate_objects('node');
        return $this->_nodes;
    }
    /**
     * @brief: create a new ACL Node
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_create_node($name, $data=null, &$output = null) {
        $node = new Fs_acl_node_class($this->node(), $this->object_name() . '/node', $name);

        $node->configure();
        if($data) {
            $node->set_data_values($data);
        }
        return $node;
    }
    /**
     * @brief 
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_retrieve_node($name, $data=null, &$output = null) {
        $node = $this->nodes();
        return $node[$name];
    }
    /**
     * @brief 
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_update_node($name, $data=null, &$output = null) {
        $node = $this->api_retrieve_node($name);
        if($node) {
            if ($node->validate($data,$output)) {
                if (true == $node->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief 
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_delete_node($name, $data=null, &$output = null) {
        $acl_node = $this->api_retrieve_node($name);
        if($acl_node) {
            if($acl_node->can_dispose($output)) {
                return $acl_node->dispose();
            }
        }
        return false;
    }
    /**
     * @brief 
     *
     * @return 
     */
    public function can_dispose(&$reason)
    {
        // Check if ACL is being used 
        $usages = $this->node()->software()->application()->check_object_usage('acl', 'network_list', $this->name());
        if($usages){
            $reason['msg'] = "  {$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return parent::can_dispose($reason);
    }
}
?>