<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2017  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * FS ACL Zone Class
 * @author Fenton Har
 */
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_require_class('hardware');
safe_module_require_class('fs', 'application');
class Fs_acl_user_acl_zone_record_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    
    public function configure()
    {
        $this->add_field('zone_name', 'Zone Name', 'string', '', 255);
        $this->set_field_rules('zone_name', 'required|min_length[1]|max_length[255]');
        $this->set_field_help('zone_name', 'A unique name of a ACL zone.');
        $this->add_field('ip_list', 'IP List', 'string', '', 4096);
        $this->set_field_rules('ip_list', 'required|min_length[1]|max_length[4096]');
        $this->set_field_help('ip_list', 'List of IPs separated by the | char.');
        return parent::configure();
    }
    public function description()
    {
        $cfg = $this->get_data_values(false);
        return 'zone_name=' . $cfg['zone_name'] . ' ip_list==' .  $cfg['ip_list'];
    }
    public function summary($type = 'horizontal' , $long = false){
        $table_line = parent::summary($type , $long);
        array_shift($table_line['data']);
        return $table_line;
    }
    
}
class Fs_acl_user_acl_zone_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }

    public function configure()
    {
        $this->add_enum_field('enabled', 'Enabled', 'dropdown', 'true', $this->enable_disable_enum());
        $this->add_field('description', 'Description', 'string', '', 100);
        $this->add_field('file_name', 'File Name', 'string', '', 50);
        $this->set_field_readonly('file_name');
        $this->add_field('count', 'User Count', 'integer', '', 50);
        $this->set_field_readonly('count');
        return parent::configure();
    }
    public function is_enabled(){
        $enabled = $this->get_data_value('enabled', false);
        return ('true' == $enabled);
    }
    public function summary($type = 'horizontal' , $long = false){
        $table_line = parent::summary($type , $long, 5);
        array_shift($table_line['data']);
        return $table_line;
    }
    private function _validate_csv($file, &$line, &$error_messages = null){
        $error_messages = array();
        $rc = true;
        ini_set('auto_detect_line_endings',TRUE);
        $handle = fopen($file,'r');
        if ($handle) {
            $csv_data = array();
            $line = 0;
            $tmp_new_obj = new Fs_acl_user_acl_zone_record_class($this->node(), $this->path(), 'tmp');
            $tmp_new_obj->configure();
            $keys = $tmp_new_obj->get_data_keys();
            $header_checked = false;
            //if user upload a big file it may take mush longer than default 300s
            set_time_limit(0);
            while ( ($csv_data= fgetcsv($handle) ) !== FALSE ) {
                // check header
                if(!$header_checked){
                    // keys must be indentical
                    if($keys != $csv_data){
                        $error_messages[]= array(
                                'message' => "line [" . $line ."] ",
                                'reason' =>  'Invalid CSV header, must be: '.implode(',', $keys)
                        );
                        $rc = false;
                        break;
                    }else{
                        $header_checked = true;
                        continue;
                    }
                }
                // user_acl record, validate
                $name = 'acl_zone_'.$line;
                $obj_data = array_combine($keys,$csv_data);
                $tmp_new_obj->set_data_values($obj_data);
                $ouput_msg = array();
                if(!$tmp_new_obj->validate($obj_data, $ouput_msg)){
                    $error_messages[]= array(
                            'message' => "line [" . $line ."] ",
                            'reason' =>  $ouput_msg
                    );
                    $rc = false;
                }
                $line++;
            }
            fclose($handle);
            // check if we have header checked set and some lines
            if(!$header_checked){
                $error_messages[]= array(
                        'message' =>  'Invalid CSV header, must be: '.implode(',', $keys)
                );
                $rc = false;
            }
            if(!$line){
                $error_messages[]= array(
                        'message' =>  'Empty CSV file.',
                );
                $rc = false;
            }

        }else{
            $error_messages[]= array(
                    'message' => "Fail to open ".$file,
            );
            $rc = false;
        }
        ini_set('auto_detect_line_endings',FALSE);
        return $rc;
    }
    public function validate($data, &$out_message = null){
        // Check if file content is provided
        if(isset($data['base64_encode_contents'])){
            //save in tmp 
            $tmp_path = '/tmp/upload-files';
            $tmp_file_name = md5($this->object_name());
            // Store on disk
            $_zones = base64_decode($data['base64_encode_contents']);
            unset($data['base64_encode_contents']);
            file_put_contents($tmp_path.'/'.$tmp_file_name, $_zones);
            $count = 0;
            if(!$this->_validate_csv($tmp_path.'/'.$tmp_file_name, $count, $out_message)){
                @unlink($tmp_path.'/'.$tmp_file_name);
                return false;
            }
            $data['count'] = $count;
        }
        $this->set_data_values($data);
        return parent::validate($data, $out_message);
    }
    public function save(){
        $rs = parent::save();
        if($rs){
            $save_path = $this->node()->configuration_manager()->directory('configuration');
            $tmp_path = '/tmp/upload-files';
            $obj_path = $save_path . '/ui-store' . $this->path();
            $this->node()->mkdir($obj_path);
            $tmp_file_name = md5($this->object_name());
            //copy temp file to save file
            @copy($tmp_path . '/' . $tmp_file_name , $obj_path . '/' . $this->name());
            @unlink($tmp_path . '/' . $tmp_file_name);
        }
        return $rs;
    }
    public function can_dispose($obj)
    {
        return true;
    }
    public function dispose(){
        $conf = $this->node()->configuration_manager()->directory('configuration');
        $conf = $conf . '/ui-store'.$this->path();
        @unlink($conf.'/'.$this->name());
        return parent::dispose();
    }
    public function create_load_infile(&$data, &$keys){
        $conf = $this->node()->configuration_manager()->directory('configuration');
        $conf = $conf . '/ui-store'.$this->path();
        $file = $conf . '/' . $this->name();
        // Create a temp file
        $tmpfname = tempnam("/tmp", $this->name());
        $data['file_name'] = $tmpfname;
        chmod($tmpfname, 0644);

        ini_set('auto_detect_line_endings',TRUE);
        $handle = fopen($file,'r');
        if ($handle) {
            $csv_data = array();
            $user_acl_zone = new Fs_acl_user_acl_zone_record_class($this->node(), $this->path(), 'tmp');
            $user_acl_zone->configure();
            $keys = $user_acl_zone->get_data_keys();
            $header_skipped = false;
            $keys_in_data = false;
            
            while ( ($csv_data= fgetcsv($handle) ) !== FALSE ) {
                // check header
                if(!$header_skipped){
                    $header_skipped = true;
                    continue;
                }
                // user_acl zone record, validate
                $obj_data = array_combine($keys,$csv_data);
                $user_acl_zone->set_data_values($obj_data);
                $rec = array();
                foreach ($user_acl_zone->get_data_values(false) as $k => $v) {
                    $rec[$k] = $v;
                }
                if(!$keys_in_data){
                    $data['keys'] = array_keys($rec);
                    $keys_in_data = true;
                }

                // Create a csv line
                $line = implode(",", array_values($rec));
                if(strlen(trim($line))) {
                    file_put_contents($tmpfname, $line. PHP_EOL, FILE_APPEND);
                }
            }
            fclose($handle);
        }
        ini_set('auto_detect_line_endings',FALSE);
    }
}
?>
