<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * FS Advanced Dialplan Class
 * @author: Shaunt Libarian
 */
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_require_class('hardware');
safe_module_require_class('fs', 'application');
class Fs_advanced_dialplan_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->add_field('display-name', 'Display Name', 'string', '', 50);
        $this->set_field_rules('display-name','alpha_dash');
        $this->add_field('data', 'XML Dialplan', 'text', '');
        parent::configure();
    }
    /**
     * Returns dialplan type
     * @return string
     */
    public function type()
    {
        return 'advanced';
    }
    public function description()
    {
        $desc = $this->get_data_value('description', false);
        if($desc){
            return $desc;
        }
        return parent::description();
    }
    function display_xml_error($xmlstr)
    {
        $xml = explode("\n", $xmlstr);
        $errors = libxml_get_errors();
        $return = array();
        foreach ($errors as $error) {
            /*
            switch ($error->level) {
            case LIBXML_ERR_WARNING:
            $return .= "Warning $error->code: ";
            break;
            case LIBXML_ERR_ERROR:
            $return .= "Error $error->code: ";
            break;
            case LIBXML_ERR_FATAL:
            $return .= "Fatal Error $error->code: ";
            break;
            }
            */
            $return[] = "Line: $error->line " . trim($error->message);

        }
        libxml_clear_errors();
        return $return;
    }
    public function validate($data, &$out_message=null)
    {
        // Steps to validate:
        //  1- Valid xml
        //  2- doesn't contain context element
        $xml_obj = null;
        // Disable error output
        libxml_use_internal_errors(true);
        $xml_validation = "<validation_____>" . $data['data'] . "\n</validation_____>";
        // Try to create a xml object
        $xml_obj = simplexml_load_string($xml_validation);
        if (!$xml_obj) {
            $out_message[] = "Invalid XML data.";
            $out_message = array_merge($out_message, $this->display_xml_error($xml_validation));
            return false;
        }
        // Check for context element
        if (false != $xml_obj->xpath("//context")) {
            $out_message[] = "Context element not allowed.";
            return false;
        }
        // Check if root has a value (i.e. text element)
        $root_value = trim((string)$xml_obj[0]);
        if (strlen($root_value)) {
            $out_message[] = "Text value not allowed outside XML elements." ;
            $out_message[] = $root_value;
            return false;
        }
        $app = $this->node()->software()->application();
        $res = parent::validate($data, $out_message);

        $routing_mod = $app->find_module_by_name('routing');
        if ($routing_mod->check_display_name_conflict('routing', 'dialplans', $data['display-name'], $this->name())) {
            $out_message[] = 'Display name: <b>' . $data['display-name'] .'</b> already exists.';
            return false;
        }

        return $res;
    }
    public function summary($type = 'horizontal' , $long = false){
        $table_line = parent::summary($type , $long , 1);
        $table_line['data']['Display Name'] = $this->display_name();
        unset($table_line['data']['Name']);
        return $table_line;
    }
    public function can_dispose(&$reason = null)
    {
        $usages = $this->node()->software()->application()->check_object_usage('routing','advanced', $this->name());
        //check_object_usage does not inclue slef module
        $path = $this->object_name();
        $module = $this->node()->software()->application()->find_module_by_path($path);
        $usages = array_merge($usages , $module->check_object_usage('routing', 'advanced', $this->name()));
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return parent::can_dispose($reason);
    }
    public function get_default(){
        $default_str = "<extension name=\"main\" continue=\"true\">\n";
        $default_str .= "\t<condition field=\"destination_number\" expression=\"(.*)\">\n";
        $default_str .= "\t\t<action application=\"\" data=\"\" />\n";
        $default_str .= "\t</condition>\n";
        $default_str .= "</extension>";
        return $default_str;
    }
}
class Fs_advanced_dialplan_hmr_class extends Fs_advanced_dialplan_class
{
    public function save($data, &$error)
    {
        $result = parent::save($data, $error);
        if($result){
            //sent out modification event
            $receivers = $this->node()->software()->application()->check_object_modified($this, 'hmr','basic', $this->name());
        }
        return $result;
    }
    public function can_dispose(&$reason = null)
    {
        $usages = $this->node()->software()->application()->check_object_usage('hmr','advanced', $this->name());
        //check_object_usage does not inclue slef module
        $path = $this->object_name();
        $module = $this->node()->software()->application()->find_module_by_path($path);
        $usages = array_merge($usages , $module->check_object_usage('hmr', 'advanced', $this->name()));
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return true;
    }
}
class Fs_advanced_dialplan_smr_class extends Fs_advanced_dialplan_class
{
    public function save($data="", &$error="")
    {
        $result = parent::save($data, $error);
        if($result){
            //sent out modification event
            $receivers = $this->node()->software()->application()->check_object_modified($this, 'smr','basic', $this->name());
        }
        return $result;
    }
    public function can_dispose(&$reason = null)
    {
        $usages = $this->node()->software()->application()->check_object_usage('smr','advanced', $this->name());
        //check_object_usage does not inclue slef module
        $path = $this->object_name();
        $module = $this->node()->software()->application()->find_module_by_path($path);
        $usages = array_merge($usages , $module->check_object_usage('smr', 'advanced', $this->name()));
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return true;
    }
    public function get_default_header(){
        $default_str  = "<extension name=\"matchAnything\">\n";
        return $default_str;
    }
    public function get_default_footer(){
        $default_str = "\t<condition field=\"destination_number\" expression=\"(.*)\">\n";
        $default_str .= "\t\t<action application=\"respond\" data=\"200 OK\"/>\n";
        $default_str .= "\t</condition>\n";
        $default_str .= "</extension>\n";
        return $default_str;
    }
    public function get_default_condition($type){
        if($type){
            $default_str = "\t<condition field=\"\${sip_unsolicited_message}\" expression=\"{$type}\" break=\"on-true\">\n";
            $default_str .= "\t\t<action application=\"sip_relay\" data=\"sofia/\${sofia_profile_name}/\${sip_req_uri}\"/>\n";
            $default_str .= "\t</condition>\n";
        }else{
            $default_str = '';
        }
        return $default_str;
    }
    public function get_default($data = array('NOTIFY', 'MESSAGE')){
        $default_str  = $this->get_default_header();
        if($data){
            foreach($data as $item){
                $default_str .= $this->get_default_condition($item);
            }
        }
        $default_str .= $this->get_default_footer();
        return $default_str;
    }
}
?>
