<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * FS CA Certificate Class
 * @author Shaunt Libarian
 */
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_module_require_class('fs', 'application');
class Fs_certificate_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->add_field('name', 'Name', 'text', '', 60);
        $this->add_field('issuer', 'Issuer', 'text', '', 60);
        $this->add_field('subject', 'Subject', 'text', '', 60);
        $this->add_field('expires', 'Expires', 'text', '', 60);
        //$this->add_upload_field('certificate', 'CA Certificate', 'upload', '', 30);
        //$this->set_field_help('certificate', 'CA Certificate that will be used. Must be a valid PEM file.');
        $this->add_field('certificate-file-contents', 'file-contents', 'base64', '');
        $this->set_field_help('certificate-file-contents', 'Base64 encoded PEM-formatted string.');
        $this->add_field('certificate-file-name', 'file', 'text', '');
        return parent::configure();
    }
    public function ca_convert($data = '', $type = ''){
        $rs = false;
        if($data['base64_encode_contents'] && $type){
            $tmp_file_name = '/tmp/' . time();
            $content = base64_decode($data['base64_encode_contents']);
            if(file_put_contents("{$tmp_file_name}.{$type}" , $content) !== false){
                if($type == 'p7b'){
                    safe_sudo_exec ( 'openssl pkcs7 -in ' . "{$tmp_file_name}.{$type}" . " -inform der -print_certs -outform PEM -out {$tmp_file_name}.pem",'', '', $retCode);
                }elseif ($type == 'pkcs12'){
                    $ca_password = '';
                    if(isset($data['ca_password'])){
                        $ca_password = $data['ca_password'];
                    }
                    safe_sudo_exec ( 'openssl pkcs12  -in ' . "{$tmp_file_name}.{$type}" . " -out {$tmp_file_name}.pem -nodes -passin pass:{$ca_password}",'', '', $retCode);
                }
                @unlink("{$tmp_file_name}.{$type}");
                $data['base64_encode_contents'] = base64_encode(file_get_contents("{$tmp_file_name}.pem"));
                $rs = $data;
                safe_sudo_exec("rm {$tmp_file_name}.pem");
            }
        }
        return $rs;
    }
    public function validate($data, &$output)
    {
        if(isset($data['base64_encode_contents'])){
            if(isset($data['file_name'])){
                $path_parts = pathinfo($data['file_name']);
                if(isset($path_parts['extension'])){
                    $ca_file_extension = $path_parts['extension'];
                }
            }else{
                $ca_file_extension = 'pem';
            }
            $ca_type = 'pem';
            $ca_ext = array(
                        'pem' => array('pem', 'key' ,'cer', 'cert'),
                        'pkcs12' => array('pkcs12', 'pfx' ,'p12'),
                        'p7b' => array('p7b','pkcs7'),
                    );
            foreach($ca_ext as $tmp_ca_type => $ca_ext_list){
                if (in_array($ca_file_extension, $ca_ext_list)) {
                    $ca_type = $tmp_ca_type;
                    break;
                }
            }
            $tmp_data = $data;
            if(isset($_POST['ca_password']) && $_POST['ca_password'] != ''){
                $tmp_data['ca_password'] = $_POST['ca_password'];
            }
            //convert to pem 
            switch ($ca_type){
                case 'pkcs12':
                case 'p7b':
                    $data = $this->ca_convert($tmp_data, $ca_type);
                    break;
            }
            $data['certificate-file-contents'] = $data['base64_encode_contents'];
            unset($data['base64_encode_contents']);
            if(isset($data['ca_password'])){
                unset($data['ca_password']);
            }
        }
        $ssl = openssl_x509_parse(base64_decode($data['certificate-file-contents']));
        if($ssl){
            if(isset($ssl['issuer']))
                if(isset($ssl['issuer']['CN']))
                $data['issuer'] = $ssl['issuer']['CN'];
            if(isset($ssl['subject']))
                if(isset($ssl['subject']['CN']))
                $data['subject'] = $ssl['subject']['CN'];
            if(isset($ssl['validTo_time_t']))
                $data['expires'] = date("Y-m-d H:i:s", $ssl['validTo_time_t']);
            
            $data['name'] = $this->name();
            
            $data['certificate-file-name'] = $data['file_name'];
            $ext = pathinfo($data['certificate-file-name'], PATHINFO_EXTENSION);
            if('pem' != $ext){
                $data['certificate-file-name'] = $this->name().'.pem';
            }
            $this->set_data_values($data);
            //return parent::validate($data, $output);
            return true;
        }else{
            $this->set_error('certificate-file-contents', 'Certificate format error.' , $output);
            return false;            
        }
    }
    public function summary($type = 'horizontal' , $long = false){
        $table_line = parent::summary($type , $long);
        $table_line['data'][$this->get_data_label('expires')] = $this->get_data_value('expires', true);
        return $table_line;
    }
}
class Fs_ca_certificate_config_class extends Fs_certificate_config_class
{
    public function can_dispose(&$reason)
    {
        $usages = $this->node()->software()->application()->check_object_usage('certificate','ca', $this->name());
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return parent::can_dispose($reason);
    }
}
class Fs_server_certificate_config_class extends Fs_certificate_config_class
{
    public function validate($data, &$output)
    {
        $rs = parent::validate($data, $output);
        if($rs){
            //Validate a private key is included in TLS server certificates
            $file_content = base64_decode($this->get_data_value('certificate-file-contents'));
            $pkey = openssl_pkey_get_private($file_content);
            if(!$pkey){
                $rs = false;
                $this->set_error('certificate-file-contents', 'This Certificate does not contain a private key.', $output);
            }
        }
        return $rs;
    }
    public function can_dispose(&$reason)
    {
        $usages = $this->node()->software()->application()->check_object_usage('certificate','server', $this->name());
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return parent::can_dispose($reason);
    }
}

class Fs_webssl_certificate_config_class extends Fs_certificate_config_class
{
    public function validate($data, &$output)
    {
        $rs = parent::validate($data, $output);
        if($rs){
            $file_content = base64_decode($this->get_data_value('certificate-file-contents'));
            $pkey = openssl_pkey_get_private($file_content);
            if(!$pkey){
                $rs = false;
                $this->set_error('certificate-file-contents', 'This Certificate
                        does not contain a private key. <b>Append Private key
                        content to the uploaded file</b> ', $output);
            }
        }

        return $rs;
    }
    public function can_dispose(&$reason)
    {
        return parent::can_dispose($reason);
    }
}
?>
