<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * FS CORE module class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_service_class');
safe_module_require_class('fs', 'application');
safe_module_require_class('sng', 'transcoding_config');
safe_require_class('safe_license_class');
class Fs_core_config_class extends Safe_configurable_object_class
{
    /**
     * @brief Constructor
     *           
     * @param[in out] $parent_name
     * @param[in out] $name
     *           
     * @return
     */
    public function __construct($node, $parent_name)
    {
        parent::__construct($parent_name, "configuration", $node);
    }
    /**
     * @brief Configure the config class object
     *         
     * @return
     */
    public function configure()
    {
        /*
         * Max Sessions
        */
        $this->add_field('max-sessions', 'Max SIP Sessions', 'int', 2*Safe_license::DEFAULT_MAX_SESSIONS);
        $this->set_field_rules('max-sessions', 'required|greater_than[-1]');
        $this->set_field_help('max-sessions', 'Global maximum number of simultaneous SIP sessions.');
        /*
         * Sessions Per Second
        */
        $this->add_field('sessions-per-second', 'SIP Sessions Per Second', 'int', 150);
        $this->set_field_rules('sessions-per-second', 'required|greater_than[-1]');
        $this->set_field_help('sessions-per-second', 'Global maximum number of SIP sessions per second.');
        /*
         * Log Level
        */
        $log_level = array(
            'debug' => 'Debug',
            'info' => 'Info',
            'notice' => 'Notice',
            'warning' => 'Warning',
            'err' => 'Error',
            'crit' => 'Critical',
            'alert' => 'Alert'
        );
        $this->add_enum_field('loglevel', 'Log Level', 'dropdown', 'info', $log_level);
        $this->set_field_help('loglevel', 'Global logging level for the call control engine. Listed in order of severity.');

        $this->add_enum_field('sofia/tracelevel', 'SIP Trace Log Level', 'dropdown', 'debug', $log_level);
        $this->set_field_help('sofia/tracelevel', 
            array(
                'The log level to use when logging SIP traces (SIP messages logged) for SIP profiles with tracing enabled.',
                'If a log level is used that is below the core log level, the SIP messages will not be logged even when the SIP profiles have tracing enabled.')
            );

        $this->add_enum_field('enable-session-log-dumping', 'Enable Session Log Dumping', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('enable-session-log-dumping', array('When maximum log level is lower or equal to WARNING, ',
                                                                  'this option allows to enable ability to dump all previous ',
																  'debug traces when a WARNING, ERROR, CRITICAL or ALERT event occurs.',
                                                                  '<span class="field_error">Enable this option will increase memory usage and reduce performance.</span>'));
        $this->create_group('session-log-dumping-options',array('session-log-dump-queue-size'));
        $this->conditional_control('enable-session-log-dumping', 'session-log-dumping-options');
        $this->add_field('session-log-dump-queue-size', 'Session Log Dump Queue Size','int', 100);
        $this->set_field_help('session-log-dump-queue-size', array('Maximum number of logging entries ',
                                                                   'to keep in memory when session log',
                                                                   ' dumping is enabled.'));

        $this->add_enum_field('sofia/debug-suppress-ptime-warning', 'Supress ptime warning', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('sofia/debug-suppress-ptime-warning', array('Supress debug warning of ptime change. Restart NSC service to apply change' ));

        $this->add_enum_field('enable-sip-capture', 'Enable SIP Capture', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('enable-sip-capture', array('Enable Sending sip',
                                                          'traces to a
                                                          configured ',
                                                          'HEP capture server'));
        $this->add_field('sofia/capture-server', 'HEP Capture Server', 'string', '', 100);
        $this->set_field_help('sofia/capture-server', array('HEP Capture server to send the sip traces to.',
                                                            'format should be udp:capture_server_ip:capture_server_port'));
        $this->create_group('sip-capture-options',array('sofia/capture-server'));
        $this->conditional_control('enable-sip-capture', 'sip-capture-options');

        $this->add_enum_field('enable-use-system-time', 'Enable Using System Time', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('enable-use-system-time', array('NSC service will use the system time as time reference.',
                                                            '',
                                                            'It is not recommended to use this setting in modern machines, ',
                                                            'as it is likely to break timing and other things like CDR',
                                                            '',
                                                                  '<span class="field_error">Enable this option if only required.</span>'));

        // Tracker to notify when HW mode is forcibly switched to SW mode when the system detects hardware transcoding issues
        $this->add_field('force-software', 'force-software', 'hidden', '0', 1);

        return parent::configure();
    }
    private function get_license_max_session(){
        //check license file
        $license_obj = $this->node()->software()->license();
        return $license_obj->get_max_session();
    }
    public function validate($data, &$output = null){
        $rs = parent::validate($data, $output);
        if($rs){
            $max_sessions = $data['max-sessions'];
            $license_max_session = $this->get_license_max_session();
            if($license_max_session && $license_max_session < $max_sessions){
                $this->set_error('max-sessions', "Max SIP sessions exceeds the license limit({$license_max_session}).", $output);
                return false;
            }

            $sip_capture = $data['enable-sip-capture'];
            if ($sip_capture == 'true' && (!isset($data['sofia/capture-server']) || empty($data['sofia/capture-server']))){
                $this->set_error('sofia/capture-server', "Must spoecify a valid capture server", $output);
                return false;
            }
            if ($sip_capture == 'true' && substr($data['sofia/capture-server'], 0, 4) != 'udp:') {
                $this->set_error('sofia/capture-server', "Capture server address
                        format should be udp:server_ip:server_port", $output);
                return false;
            }
        }
        return $rs;
    }
    public function check_session_limit(){
        $license_max_session = $this->get_license_max_session();
        $max_sessions = $this->get_data_value('max-sessions', false);
        if($license_max_session && $license_max_session < $max_sessions){
            return false;
        }
        return true;
    }
    public function adjust_max_session() {
        $license_max_session = $this->get_license_max_session();
        $data = $this->get_data_values();
        $data['max-sessions'] = $license_max_session;
        $this->set_data_values($data);
        return $this->save();
    }

    public function is_sip_capture_allowed() {
        $sip_capture_enabled = $this->get_data_value('enable-sip-capture', false);

        return $sip_capture_enabled == 'true' ? true : false;
    }

    /**
     * @brief Returns a config description string
     *         
     * @return string
     */
    public function description()
    {
        return parent::description();
    }
}
class Fs_core_module_class extends Fs_module_class
{
    /**
     * @brief
     *           
     * @param[in out] $fs_app
     *           
     * @return
     */
    public function __construct($fs_app)
    {
        parent::__construct($fs_app, "core");
    }
    /**
     * @brief
     *         
     * @return
     */
    public function is_sip_capture_allowed() {
        return $this->config()->is_sip_capture_allowed();
    }
    public function configure()
    {
        // Set the module description
        $this->set_description("Core");
        // Create the config object
        //$cfg = new Fs_core_config_class($this->node() , $this->fs_app()->object_name() , $this->name());
        $cfg = new Fs_core_config_class($this->node() , $this->path().'/core' , $this->name());
        // Synch with DB
        $cfg->configure();
        $cfg->synch();
        // Attach config to module
        $this->set_config($cfg);
        return parent::configure();
    }
    /**
     * @brief
     *           
     * @param[in out] $config_manager
     *           
     * @return
     */
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        return $this->generate_config($config_manager);
    }
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }
    public function generate_config(&$config_manager)
    {
        $this->config()->clear_global_messages_cache($msg_list, 'core', 'license_max_session_adjusted');
        $this->config()->clear_global_messages_cache($msg_list, 'core', 'license_max_session');
        if(!$this->config()->check_session_limit()){
            $this->config()->adjust_max_session();
            $this->node()->handle_global_messages($msg_list, true);
        } else {
            $this->config()->flush_global_messages_cache();
        }
        $skip_capture_server = false;
        // Get our configuration data (unresolved)
        $cfg = $this->config()->get_data_values_exploded(false);
        // Create xml document
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startDocument('1.0', 'UTF-8');
        // Document container
        $xml_writer->startElement('include');
        foreach ($cfg['__all__'] as $k => $v) {
            if ($k == 'enable-sip-capture') {
                if ($v == 'false') {
                    $skip_capture_server = true;
                }
            } else {
                $xml_writer->startElement('param');
                $xml_writer->writeAttribute('name', $k);
                $xml_writer->writeAttribute('value', $v);
                $xml_writer->EndElement();
            }
        }
        // Check if we're in software transcoding mode
        $sngms_module = $this->node()->software()->application()->find_module_by_name('sngms');
        // Grab RTP ports for software transcoding
        // in hw mode find port numbers outside DSPs port 
        $udp_port = $sngms_module->udp_ports();
        if($udp_port){
            $start = $udp_port['start'];
            $stop = $udp_port['stop'];
            $xml_writer->startElement('param');
            $xml_writer->writeAttribute('name', 'rtp-start-port');
            $xml_writer->writeAttribute('value', $start);
            $xml_writer->EndElement();
            $xml_writer->startElement('param');
            $xml_writer->writeAttribute('name', 'rtp-end-port');
            $xml_writer->writeAttribute('value', $stop);
            $xml_writer->EndElement();
        }else{
            $config_manager->add_error(array(
                    'module' => 'sngms',
                    'obj_type' => 'module',
                    'description' => 'Not enough ports for rtp',
                    'url' => '/SAFe/fs_sngms_config'
            ));
        }
        
        // Close settings
        $xml_writer->EndElement();
        $xml_writer->endDocument();
        $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
        $file = 'autoload_configs/switch_additional.xml';
        $config_manager->add_config(new Safe_configuration_class($file, $xmlstr));

        // No take care of other modules if set
        unset($cfg['__all__']);
        foreach($cfg as $mod=>$mod_cfg) {
            // Create xml document
            $xml_writer = new XMLWriter();
            $xml_writer->openMemory();
            $xml_writer->startDocument('1.0', 'UTF-8');
            // Document container
            $xml_writer->startElement('include');
            foreach ($mod_cfg as $k => $v) {
                if ($k == 'capture-server' && $skip_capture_server) {
                    continue;
                }
                $xml_writer->startElement('param');
                $xml_writer->writeAttribute('name', $k);
                $xml_writer->writeAttribute('value', $v);
                $xml_writer->EndElement();
            }

            $xml_writer->EndElement();
            $xml_writer->endDocument();
            $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
            $file = 'autoload_configs/'.$mod.'_additional.xml';
            $config_manager->add_config(new Safe_configuration_class($file, $xmlstr));
        }
        return parent::generate_config($config_manager);
    }
    /**
     * @brief Overload Safe_module::support_reload
     *           
     * @param[in out] $cfg_modified
     *           
     * @return
     */
    public function support_reload(&$cfg_modified = null)
    {
        parent::support_reload($cfg_modified);
        return true;
    }
    /**
     * @brief Invoked after a successfull write_config
     *         
     * @return
     */
    public function post_write_config()
    {
        // Check if FS is running
        if (Safe_service_class::STATUS_RUNNING == $this->fs_app()->status()) {
            $cfg = $this->config()->get_data_values_exploded(false);
            // Prepare switch reload parameters (requires name remapping)
            $data = $cfg['__all__'];
            $reload_param = array(
                'max_sessions' => $data['max-sessions'],
                'sps' => $data['sessions-per-second'],
                'loglevel' => $data['loglevel'],
                'enable_session_log_dumping' => $data['enable-session-log-dumping'],
                'session_log_dump_queue_size' => $data['session-log-dump-queue-size'],
                'reload_license' => ''
            );
            // Loop to apply new parameters value
            foreach ($reload_param as $k => $v) {
                $output = array();
                $rc = $this->eslapi_execute("fsctl " . $k . " " . $v, '', $output);
                foreach($output as $outputdata){
                    if(preg_match("/-ERR /", $outputdata)){
                        safe_log(SAFE_LOG_ERROR, "Failed reload param '". $k ."' result: " . $outputdata );
                        $rc = false;
                    }
                }
                if (!$rc) return $rc;
            }
            // Same but for other modules parameters (assuming no conversion 
            // required)
            unset($cfg['__all__']);
            foreach($cfg as $mod=>$data) {
                // Loop to apply new parameters value
                foreach ($data as $k => $v) {
                    $output = array();
                    $rc = $this->eslapi_execute($mod ." " . $k . " " . $v, '', $output);
                    if (!$rc) return $rc;
                }
            }
        }
        return parent::post_write_config();
    }
    public function check_object_modified($obj, $src_module_name, $obj_type, $obj_name, $sub_type = null, $sub_name = null)
    {
        $modules = array('application');
        if($obj_type == 'license'){
            //if the license was updated
            //mark the core config as modified
            $this->config()->mark_modified();
            $this->config()->clear_list_cache();
        }
    }
    public function handle_global_messages(&$msg_list, $direct_call = false) {
        $btn[] = array(
            "label" => "Core Configuration",
            "url" => '/SAFe/fs_core_config'
        );
        if ($direct_call) {
            if (is_array($this->config()->get_global_messages_cache('core', 'license_max_session_adjusted'))) {
                return false;
            }
            $msg_list[] = array(
                'module' => 'core',
                'cache_tag' => 'license_max_session_adjusted',
                'global_warnings' => array(
                    "message" => "Max SIP sessions exceeds the license limit and was automatically adjusted",
                    "buttons" => $btn
                )
            );
        }
        else {
            if ($this->config()->check_session_limit()) {
                $this->config()->clear_global_messages_cache($msg_list, 'core', 'license_max_session');
                return false;
            }
            if (is_array($this->config()->get_global_messages_cache('core', 'license_max_session'))) {
                return false;
            }
            $msg_list[] = array(
                'module' => 'core',
                'cache_tag' => 'license_max_session',
                'global_errors' => array(
                    "message" => 'Max SIP sessions exceeds the license limit',
                    "buttons" => $btn
                )
            );
        }
    }
    public function checklist(&$list)
    {
        $core_cfg = $this->config();
        $data = $core_cfg->get_data_values(false);
        
        //apply max-session automatically when the max-sessions form field is empty
        if(empty($data['max-sessions'])){
            $data['max-sessions'] = 99999;
            $core_cfg->set_data_values($data);
            $core_cfg->save();
        }
    }
    public function api_update_configuration($obj_name = null, $params = array(), &$output = null, $validation_check = true)
    {
        $core_cfg = $this->config();
        $core_data = $core_cfg->get_data_values(false);

        if(isset($params['force-software'])){
            $core_data['force-software'] = intval($params['force-software']);
            $core_cfg->set_data_values($core_data);
            $core_cfg->save();
        }
        return parent::api_update_configuration($obj_name, $params, $output, $validation_check);
    }
}
/* End of file fs_core_module_class.php */
