<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2017  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Fenton Har <fhar@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
 */
/*
 * DID List module class
 */

if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_service_class');
safe_module_require_class('fs', 'application');
safe_module_require_class('fs', 'did_list');

class Fs_did_list_module_class extends Fs_module_class {
    private $_db = null;
	protected $_obj_base_path = '';
    public function is_enabled($mod_name = "")
    {
        $list_objs = $this->get_aggregate_objects('did_list');
        if ($list_objs) return true;
        return false;
    }
    public function is_critical($mod_name = "")
    {
        return true;
    }
    /**
     * @brief Return FS module name
     *         
     * @return string
     */
    public function get_fs_module_name()
    {
        return Fs_did_list_class::get_module_name() . '.so';
    }
    /**
     * @brief
     *
     * @param[in out] $fs_app
     *
     * @return
     */
    public function __construct($fs_app)
    {
        parent::__construct($fs_app, "did_list");
        $this->_obj_base_path = $this->path().'/did_list';
    }
    /**
     * @brief
     *
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("DID List");
        $this->register_aggregate_object('did_list',
            array(
                'name' => 'DID list File',
                'dynamic' => false,
                /* Add some extra infos for config_manager create object */
                'controller' => array(
                        'config_manager' => array(
                                'summary_buttons' => 'Delete:confirm',
                                'allow_create_button' => 'upload[Upload]'
                        ),
                ),
                'autoname' => true,
                'popup_add_form' => true,
                'file_def' => array(
                        'filesystem' => true,
                        'file_type' => 'csv'
                ),
                'methods' => array(
                        'create' => array(
                                'icon' => 'create',
                                'name' => 'Create',
                                'description' => 'Create DID list file',
                                'request' => 'POST',
                        ),
                        'upload' => array(
                                'icon' => 'upload',
                                'name' => 'Upload',
                                'description' => 'Upload a DID list file',
                                'request' => 'POST',
                        ),
                        'delete' => array(
                                'name' => 'Delete',
                                'description' => 'Delete a DID list file',
                                'request' => 'POST',
                        ),
                ),
            )
        );
        $this->db();
        return parent::configure();
    }
    /**
     * @brief Create config jobs for DID list module
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        if(!$this->_generate_config($config_manager)) return false;
        return parent::reload_generate_config($config_manager, $obj_type);
    }

    /**
     * @brief
     *
     * @param[in out] $obj_type
     *
     * @return
     */
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }

    /**
     * @brief NSC db
     *
     * @return
     */
    function db()
    {
        if(!$this->_db) {
            $cfg_db = array (
                            'hostname' => 'localhost',
                            'username' => 'root',
                            'password' => '',
                            'port' => '3306',
                            'database' => 'nsc',
                            'dbdriver' => 'mysqli',
                            'dbprefix' => '',
                            'pconnect' => 'false',
                            'db_debug' => 'true',
                            'cache_on' => 'false',
                            'cachedir' => '',
                            'char_set' => 'utf8',
                            'dbcollat' => 'utf8_general_ci',
                            'active-record' => 'true',
                            );
            $this->_db = Safe_database_class::factory($cfg_db);
        }
        return $this->_db;
    }

    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    public function generate_config(&$config_manager)
    {
        if(!$this->_generate_config($config_manager)) return false;
        return parent::generate_config($config_manager);
    }
    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_did_list_module_config(&$config_manager)
    {
        // Cleanup
        $directory = "autoload_configs/did_list.conf.xml";
        $config_manager->add_config(
            new Safe_configuration_class(
                $directory, '',
                Safe_configuration_class::CFG_DELETE,
                Safe_configuration_class::CFG_DIR
            )
        );

        // generate the xml configuration
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startDocument('1.0', 'UTF8');
        $xml_writer->startElement("configuration");
        $xml_writer->writeAttribute('name', "did_list.conf");
        $xml_writer->writeAttribute('description', 'DID list');
        $xml_writer->startElement("settings");
        $xml_writer->startElement("param");
        $xml_writer->writeAttribute('name', "odbc-dsn");
        $xml_writer->writeAttribute('value', "nsc:root:");
        $xml_writer->fullEndElement(); // end element for param 
        $xml_writer->fullEndElement(); // end element for settings
        $xml_writer->fullEndElement(); // end element for configuration
        $xml_writer->endDocument();
        // Create directory
        $directory = "autoload_configs/";
        $config_manager->add_config(new Safe_configuration_class($directory, '', Safe_configuration_class::CFG_CREATE, Safe_configuration_class::CFG_DIR));
        $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
        $file = "autoload_configs/did_list.conf.xml";
        $config_manager->add_config(new Safe_configuration_class($file, $xmlstr));
        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_did_list_db_config(&$config_manager)
    {
        // Cleanup
        $db = $this->_db;
        $db->connect();
        $db->query('DROP TABLE IF EXISTS did_list;');
        $db->query('CREATE TABLE IF NOT EXISTS `did_list` (
                    `name` varchar(255) default NULL,
                    `did` varchar(255) default NULL,
                    PRIMARY KEY  (`name`, `did`)
                    ) ENGINE=InnoDB;');
        $db->disconnect();
        // Insert
        $dids = $this->get_aggregate_objects('did_list');
        unset($data['data']);
        $data['data'] = array();
        $data['db_class'] = $db;
        foreach($dids as $did){
            $did->create_load_infile($data['data'], $keys);
            $config_manager->add_config(new Safe_configuration_class('did_list',
                        $data, Safe_configuration_class::CFG_LOAD, Safe_configuration_class::CFG_SQL));
        }

        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_config(&$config_manager)
    {
        if(!$this->_generate_did_list_db_config($config_manager)) return false;
        if(!$this->_generate_did_list_module_config($config_manager)) return false;
        return true;
    }
    /**
     * @brief Invoked after a successfull write_config
     *
     * @return
     */
    public function post_write_config()
    {
        // Check if FS is running
        if (Safe_service_class::STATUS_RUNNING == $this->fs_app()->status()) {
            $output = array();
            $rc = $this->eslapi_execute("reloadxml", '', $output);
            return $rc;
        }
        return parent::post_write_config();
    }

    public function did_list(){
        return $this->get_aggregate_objects('did_list');
    }

    public function api_upload_did_list($name=null, $data = null, &$output = null){
        $type = 'did_list';
        $data_cfg = array(
            'description' => 'DID list File',
            'filesystem' => true,
            'pattern' => '/tmp/upload-files/*.csv',
            'file_ext' => 'csv',
            'singleton' => false
        );

        $file_output = array();
        if(upload_file($type, $data_cfg, $file_output, $from = 'POST')){
            //read content, verified data
            $tmp_file_path =  $file_output['full_path'];
            $tmp_file_name =  $file_output['file_name'];
            if($tmp_file_path && $tmp_file_name){
                $user_data = file_get_contents($tmp_file_path);
                if($user_data){
                    $data['base64_encode_contents'] = base64_encode($user_data);
                    $data['file_name'] = $tmp_file_name;
                    if($name){
                        if(!$this->can_create_aggregate_object($type, $name, $reason)){
                            // Cannont create object, throw an error
                            $output['message'] = 'Create ' .$this->aggregate_object_name($type)." <b>" . $name . "</b> failed.";
                            $output['reason'] = $reason;
                            return false;
                        }else{
                            $tmp_obj_name = $name;
                        }
                    }else{
                        $tmp_obj_name = build_unique_name($type);
                    }
                    $output_msg = array();
                    $new_obj = $this->create_aggregate_object($type, $tmp_obj_name, $output_msg);
                    if($new_obj){
                        if($new_obj->validate($data, $output_msg)){
                            $new_obj->save();
                            return $new_obj;
                        } else {
                            if($output_msg){
                                $output['message'] = "Fail to create DID list file.";
                                foreach($output_msg as $topic => $err){
                                    $err_string = "";
                                    if(is_array($err)){
                                        foreach($err as $t2 => $r2){
                                            if(is_array($r2)){
                                                $err_string .= $t2 . ':' . implode(' ', array_values($r2)) . '<br/>';
                                            }else{
                                                $err_string .= $t2 . ':' . $r2 . '<br/>';
                                            }
                                        }
                                    }else{
                                        $err_string =  $topic .'=>'.$err;
                                    }
                                    $output['reason'][] = $err_string;
                                }
                                return false; 
                            }
                        }
                    }
                }
            }
        }
        $output['message'] = "Fail to create DID list file.";
        return false;
    }

     /**
     * @brief create a DID list item
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_create_did_list($name=null, $data=null, &$output=null) {
    	$did_list = new Fs_did_list_class($this->node(), $this->_obj_base_path .'/did_list', $name);
        $did_list->configure();
        if($data) {
            $did_list->set_data_values($data);
        }
        return $did_list;
    }

     /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_retrieve_did_list($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('did_list');
        if ($name) return $objs[$name];
        else return $objs;
    }
     /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_delete_did_list($name, $data=null, &$output = null) {
        $did_list = $this->api_retrieve_did_list($name);
        if ($did_list) {
            if($did_list->can_dispose($output)) {
                return $did_list->dispose();
            }
        }
        return false;
    }

    public function new_fields(&$config_manager, $type)
    {   
        if ($type == 'did_list') {
            $config_manager->add_upload_field('did_list_file','DID list File','upload','',30,true);
            $config_manager->set_field_rules('did_list_file','optional');
            $config_manager->add_field('save', 'Save', 'hidden', 'save', 10);
        } else {
           return parent::new_fields($config_manager, $type); 
        }
    }   
}
?>
