<?php
/**
 * FS RADIUS Module Class
 * @author: Shaunt Libarian
 *
 */

require_once('application/helpers/safe_helper.php');
safe_require_class('safe_service_class');
safe_module_require_class('fs','application');
safe_module_require_class('fs','radius_config');
safe_module_require_class('fs','radius_vsas_config');

class Fs_radius_module_class extends Fs_module_class  {

	public function __construct($fs_app) {
		parent::__construct($fs_app, "radius");
		$this->_obj_base_path = $this->path() . '/radius';
	}
	
	public function configure(){
		// Set the module description
		$this->set_description("RADIUS");
		// Create the config object
		$cfg = new Fs_radius_config_class( $this->node(), $this->object_name(),$this->name());
		// Synch with DB
		$cfg->configure();
		$cfg->synch();
	
		// Attach config to module
		$this->set_config($cfg);

        // Register aggregated objects
        $this->register_aggregate_object('vsas',
                array(
                        'name' => 'VSAs',
                        'base_path' => $this->_obj_base_path . '/vsas',
                        'dynamic' => true,
                        'methods' => array(
                                'create' => array(
                                        'name' => 'Create',
                                        'description' => 'Create a RADIUS VSAs',
                                        'request' => 'POST',
                                ),
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a RADIUS VSAs',
                                        'request' => 'GET',
                                ),
                                'update' => array(
                                        'name' => 'Update',
                                        'description' => 'Update a RADIUS VSAs',
                                        'request' => 'POST',
                                ),
                                'delete' => array(
                                        'name' => 'Delete',
                                        'description' => 'Delete a RADIUS VSAs',
                                        'request' => 'POST',
                                )
                         )
                )
        );
		return parent::configure();
	}
	
	/**
	 * Returns the radius module names
	 * @see Fs_module_class::get_fs_module_name()
	 */
	public function get_fs_module_name() {
		//two module for radius
		$mod['mod_rad_auth.so'] = 'mod_rad_auth.so';
		$mod['mod_radius_cdr.so'] = 'mod_radius_cdr.so'; 
		
		return $mod;
	}
	
	/**
	 * Whether radius module is enabled or disabled
	 * @see Fs_module_class::is_enabled()
	 */
	public function is_enabled($mod_name="") {
		//mod_rad_auth.so
	    //mod_radius_cdr.so -->disabled if accounting turned off
	    
	    $cfg = $this->config()->get_data_values(false);
	
	    if ($mod_name == "mod_rad_auth.so") {
			if("" != $cfg['authserver'] && "" != $cfg['bindaddr'])
				return true;
			else
				return false;
	    }
	   elseif ($mod_name == "mod_radius_cdr.so") {
	      //Check if radius accounting enabled
	      if ($cfg['enable-accounting'] == "true")
	      	return true;
	      else
	      	return false;
	    }
	}
	
	/**
	 * Returns the name of the aggregate config object
	 * @return string
	 */
	public function aggregate_config_name(&$short_name=NULL) {
    if(isset($short_name))
      $short_name = 'vsas';
		return 'VSAs';
	}
    /**
     * @brief Create config jobs for dialplan module
     *           
     * @param[in out] $config_manager
     *           
     * @return
     */
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        if(!$this->_generate_config($config_manager, $obj_type)) return false;
        return parent::reload_generate_config($config_manager, $obj_type);
    }
	
	/**
	 * @brief
	 *
	 * @param[in out] $obj_type
	 *
	 * @return
	 */
	public function reload_clear_modified($obj_type=null)
	{
	    return $this->clear_configuration_modified();
	}
    /**
     * @brief 
     *
     * @param[in out] $config_manager
     *
     * @return 
     */
    public function generate_config(&$config_manager)
    {
        if(!$this->_generate_config($config_manager)) return false;
        return parent::generate_config($config_manager);
    }

	/**
	 * Serializes the RADIUS configuration for RADIUS and VSAS
	 * @see Safe_module_class::generate_config()
	 */
	private function _generate_config(&$config_manager){
		$cfg = $this->config();
		$file_rad_auth = 'autoload_configs/rad_auth.conf.xml';
		$file_radius_cdr = 'autoload_configs/mod_radius_cdr.conf.xml';
		//delete the file if it exists
		$directory = $config_manager->directory('configuration');
		if ($this->node()->os()->check_file_exists($directory .'/'. $file_rad_auth) ) {
		    $config_manager->add_config(new Safe_configuration_class($file_rad_auth,
		            '',
		            Safe_configuration_class::CFG_DELETE,
		            Safe_configuration_class::CFG_FILE));
		}
		if ($this->node()->os()->check_file_exists($directory .'/'. $file_radius_cdr) ) {
		    $config_manager->add_config(new Safe_configuration_class($file_radius_cdr,
		            '',
		            Safe_configuration_class::CFG_DELETE,
		            Safe_configuration_class::CFG_FILE));
		}
		if (!empty ($cfg) && $cfg->is_enabled()) {
			// Get our configuration data (unresolved)
			$radius_cfg = $cfg->get_data_values(false);
			$vsas = $this->vsas();

			//Create rad_auth.conf.xml
			$xml_writer = new XMLWriter();
			$xml_writer->openMemory();
			$xml_writer->startDocument('1.0','UTF-8');
		
			$xml_writer->startElement('configuration');
			$xml_writer->writeAttribute('name','rad_auth.conf');
			$xml_writer->writeAttribute('description','RADIUS Authentication Configuration');
			
			//RADIUS Client XML
			$xml_writer->startElement('client');
	
			$xml_writer->startElement('param');
			$xml_writer->writeAttribute('name','authserver');
			$xml_writer->writeAttribute('value',$radius_cfg['authserver'].":".$radius_cfg['auth_port'] . ":" . $radius_cfg['shared-secret']);
			$xml_writer->endElement();
            $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
			foreach($radius_cfg as $k=>$v) {
			    if ($k != 'enable' && $k != 'enable-accounting' && $k != 'acct-port' && $k != 'authserver' && $k != "auth_port" && $k != "shared-secret") {
					//Check interface name as IP not required for radius
					if ($k == "bindaddr") {
					    $tmp_ip = '';
					    $network_module = $this->node()->software()->application()->find_module_by_name('network');
                        if($cluster_module->get_cluster_mode()=='active-passive'){
                            $signaling_interfaces_ip = $network_module->signaling_interfaces_ip(false);
                            $tmp_interface = isset($signaling_interfaces_ip[$v])? $signaling_interfaces_ip[$v] : '';
                            $v = preg_replace('#floating/#','',$v);
                            $tmp_ips = $cluster_module->get_floating_ips($tmp_interface);
                            foreach($tmp_ips as $ip_tmp){
                                if($ip_tmp->name()==$v){
                                    $tmp_ip = $ip_tmp->address();
                                }
                            }
                        }else if(($tmp_ip_obj = $network_module->get_ip_by_name($v))){
					        $tmp_ip = $tmp_ip_obj->address();
					    }
					    if($tmp_ip == ''){
					        $config_manager->add_error(array(
					                'module' => 'radius',
					                'obj_type' => 'profile',
					                'description' => 'RADIUS bind to interface ' . $v . ' without IP address.',
					                'url' => '/SAFe/nsc_radius'
					        ));
					    }else{
					        $v = $tmp_ip;
					    }
					}
					
					$xml_writer->startElement('param');
					$xml_writer->writeAttribute('name',$k);
					$xml_writer->writeAttribute('value',$v);
					$xml_writer->endElement();
				}
			}
			
			//Constant Elements
			$xml_writer->startElement('param');
			$xml_writer->writeAttribute('name','dictionary');
			$xml_writer->writeAttribute('value','/usr/local/nsc/etc/radiusclient/dictionary');
			$xml_writer->endElement();
			$xml_writer->startElement('param');
			$xml_writer->writeAttribute('name','seqfile');
			$xml_writer->writeAttribute('value','/usr/local/nsc/run/radius.seq');
			$xml_writer->endElement();
			$xml_writer->startElement('param');
			$xml_writer->writeAttribute('name','radius_deadtime');
			$xml_writer->writeAttribute('value','0');
			$xml_writer->endElement();
			
			$xml_writer->fullEndElement();
			
			//RADIUS VSAS configuration
			$xml_writer->startElement('vsas');
			if (!empty ($vsas)) {
				foreach ($vsas as $object) {
					if (isset ($object)) {
						$prof_data = $object->get_data_values(false);
						$xml_writer->startElement('param');
						$xml_writer->writeAttribute('name',$object->name());
						$xml_writer->writeAttribute('id',$prof_data['vsa-id']);
						$xml_writer->writeAttribute('value',$prof_data['vsa-value']);
						$xml_writer->writeAttribute('pec',$prof_data['vendor-id']);
                        $xml_writer->writeAttribute('expr',$prof_data['vsa-value-type']);
                        $xml_writer->writeAttribute('direction',$prof_data['radius-message']);
                        $xml_writer->endElement();
					}
				}
			}
			$xml_writer->fullEndElement();
			$xml_writer->fullEndElement();
			
			$xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
			$config_manager->add_config(new Safe_configuration_class($file_rad_auth, $xmlstr));
			
			//Create mod_radius_cdr.conf.xml if RADIUS Accounting is turned on
			if ($radius_cfg['enable-accounting'] == "true") {
				$xml_writer = new XMLWriter();
				$xml_writer->openMemory();
				$xml_writer->startDocument('1.0','UTF-8');
				
				$xml_writer->startElement('configuration');
				$xml_writer->writeAttribute('name','mod_radius_cdr.conf');
				$xml_writer->writeAttribute('description','RADIUS CDR Configuration');
				
				$xml_writer->startElement('settings');
				
				$xml_writer->startElement('param');
				$xml_writer->writeAttribute('name','acctserver');
				$xml_writer->writeAttribute('value',$radius_cfg['authserver'].':'.$radius_cfg['acct-port'].':'.$radius_cfg['shared-secret']);
				$xml_writer->endElement();

				$xml_writer->startElement('param');
				$xml_writer->writeAttribute('name','radius_timeout');
				$xml_writer->writeAttribute('value',$radius_cfg['radius_timeout']);
				$xml_writer->endElement();

				$xml_writer->startElement('param');
				$xml_writer->writeAttribute('name','radius_retries');
				$xml_writer->writeAttribute('value',$radius_cfg['radius_retries']);
				$xml_writer->endElement();

				//Constant Elements
				$xml_writer->startElement('param');
				$xml_writer->writeAttribute('name','dictionary');
				$xml_writer->writeAttribute('value','/usr/local/nsc/etc/radiusclient/dictionary.cdr');
				$xml_writer->endElement();
				$xml_writer->startElement('param');
				$xml_writer->writeAttribute('name','seqfile');
				$xml_writer->writeAttribute('value','/usr/local/nsc/run/radiuscdr.seq');
				$xml_writer->endElement();
				$xml_writer->startElement('param');
				$xml_writer->writeAttribute('name','radius_deadtime');
				$xml_writer->writeAttribute('value','0');
				$xml_writer->endElement();
				
				$xml_writer->fullEndElement();
				$xml_writer->fullEndElement();
				
				$xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
				$config_manager->add_config(new Safe_configuration_class($file_radius_cdr, $xmlstr));
			}
		}
		return parent::generate_config($config_manager);
	}
	public function check_object_usage($src_module_name, $obj_type, $obj_name, $sub_type = null, $sub_name = null)
	{
	    $cfg = $this->config();
	    // check usages 
	    $usages = array();
	    if(!$cfg->is_enabled() || !in_array($src_module_name, array('network'))){
	        return $usages;
	    }
	    $data = $cfg->get_data_values(false);
	    if($obj_name == $data['bindaddr']){
	        $tmp = array (
	                'name' => $this->name(),
	                'obj_type' => 'configuration',
	                'obj_name' => ''
	        );
	        if(!in_array($tmp, $usages)){
	            $usages[] = $tmp;
	        }
	    }
	    return $usages;
	}
	public function vsas()
	{
	    return $this->get_aggregate_objects('vsas');
	}
	/**
	 * @brief
	 *
	 * @param[in out] $name
	 * @param[in out] $data
	 * @param[in out] $output
	 *
	 * @return
	 */
	public function api_create_vsas($name, $data=null, &$output = null) {
	    $vsas = new Fs_radius_vsas_config_class($this->node(), $this->_obj_base_path . '/vsas', $name);
	    $vsas->configure();
	    if($data) {
	        $vsas->set_data_values($data);
	    }
	    return $vsas;
	}
	/**
	 * @brief
	 *
	 * @param[in out] $name
	 * @param[in out] $data
	 * @param[in out] $output
	 *
	 * @return
	 */
	public function api_retrieve_vsas($name, $data=null, &$output = null) {
	    $vsas = $this->vsas();
	    return $vsas[$name];
	}
	/**
	 * @brief
	 *
	 * @param[in out] $name
	 * @param[in out] $data
	 * @param[in out] $output
	 *
	 * @return
	 */
	public function api_update_vsas($name, $data=null, &$output = null) {
	    $vsas = $this->api_retrieve_vsas($name);
	    if($vsas) {
	        if ($vsas->validate($data,$output)) {
	            if (true == $vsas->save()) {
	                return true;
	            }
	        }
	    }
	    return false;
	}
	/**
	 * @brief
	 *
	 * @param[in out] $name
	 * @param[in out] $data
	 * @param[in out] $output
	 *
	 * @return
	 */
	public function api_delete_vsas($name, $data=null, &$output = null) {
	    $vsas = $this->api_retrieve_vsas($name);
	    if($vsas) {
	        if($vsas->can_dispose($output)) {
	            return $vsas->dispose();
	        }
	    }
	    return false;
	}
}
?>
