<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * FS siprec module class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');

class Fs_siprec_profile_config_class extends Safe_configurable_object_class
{
    /**
     * @brief Constructor
     *           
     * @param[in out] $parent_name
     * @param[in out] $name
     *           
     * @return
     */
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    /**
     * @brief Configure the config class object
     *         
     * @return
     */
    public function configure()
    {
        $type_enum =  array("failover"=>"Failover", "high-availability" => "High Availability");
        $this->add_enum_field('type', 'Type', 'dropdown', 'failover', $type_enum);
        
        $originator_direction_enum =  array("RX" => "RX", "TX" => "TX", "RXTX_SEPARATE" => "Separate", "RXTX_MIXED" => "Mixed");
        $this->add_enum_field('originator_direction', 'Originator Recording Direction', 'dropdown', 'RX',$originator_direction_enum);
        $this->set_field_help('originator_direction','This parameter controls which stream direction should be recorded on the session on which recording have been initiated.');
        
        $originator_direction_enum =  array("NONE" => "None", "RX" => "RX", "TX" => "TX", "RXTX_SEPARATE" => "Separate", "RXTX_MIXED" => "Mixed");
        $this->add_enum_field('peer_direction', 'Peer Recording Direction', 'dropdown', 'NONE', $originator_direction_enum);
        $this->set_field_help('peer_direction','This parameter controls which stream direction should be recorded on the peer of session on which recording have been initiated. When set to NONE peer session recording is disabled.');
        
        $this->add_enum_field('record_on_hold', 'Record Music On Hold', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('record_on_hold','When set to Enabled, recording will not be paused when the recorded session goes on-hold.');
        
        $this->add_field('srs_user', 'Recording Server User Part', 'string', 'srs', 50);
        $this->set_field_rules('srs_user', 'required|valid_sip_uri_user_part');
        $this->set_field_help('srs_user','String representing the sip contact user part to use for the IINVITE messages sent to the srs.');
        
        $module = $this->node()->software()->application()->find_module_by_name('sip');
        $field_def = $module->object_enum('trunk', ADD_NONE_NEVER);
        $this->add_multiple_field('gateways', 'Gateway List', 'listbox', array(), $field_def, 10);
        //$this->set_field_rules('gateways', 'required');
        return parent::configure();
    }
    public function can_dispose(&$reason = null)
    {
        $usages = $this->node()->software()->application()->check_object_usage('siprec','profile', $this->name());
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return parent::can_dispose($reason);
    }
    
}

class Fs_siprec_module_class extends Fs_module_class
{
    protected $_obj_base_path = '';
    /**
     * @brief
     *           
     * @param[in out] $fs_app
     *           
     * @return
     */
    public function __construct($fs_app)
    {
        parent::__construct($fs_app, "siprec");
        $this->_obj_base_path = $this->path().'/siprec';
    }
    /**
     * @brief
     *         
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("SIPREC");
        // Register objects
        $this->register_aggregate_object('profile', 
            array(
                'name' => 'SIPREC profile',
                'base_path' => $this->_obj_base_path . '/profile',
                'dynamic' => true,
                'has_child' => false,
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a SIPREC profile',
                        'request' => 'POST',
                        ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a SIPREC profile',
                        'request' => 'GET',
                        ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a SIPREC profile',
                        'request' => 'POST',
                        ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a SIPREC profile',
                        'request' => 'POST',
                        ),
                    ),
            )
        );
        return parent::configure();
    }
    public function profiles()
    {
        return $this->get_aggregate_objects('profile');
    }
    /**
     * @brief Create a specific aggregate object
     *
     * @param[in out] $name - Name of object
     *
     * @return
     */
    public function create_aggregate_object($type,$name){
        $obj = new Fs_siprec_profile_config_class($this->node(), $this->_obj_base_path.'/profile', $name);
        $obj->configure();
        return $obj;
    }

    /**
     * @brief 
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_create_profile($name, $data=null) {
        $profile = new Fs_siprec_profile_config_class($this->node(), $this->_obj_base_path . '/profile', $name);
        $profile->configure();
        if($data) {
            $profile->set_data_values($data);
        }
        return $profile;
    }
    /**
     * @brief 
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_retrieve_profile($name, $data=null) {
        $profiles = $this->get_aggregate_objects('profile');
        return $profiles[$name];
    }
    /**
     * @brief 
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_update_profile($name, $data=null, &$output = null) {
        $profile = $this->api_retrieve_profile($name);
        if($profile) {
            if ($profile->validate($data,$output)) {
                if (true == $profile->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief 
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_delete_profile($name, $data=null, &$output = null) {
        $profile = $this->api_retrieve_profile($name);
        if($profile) {
            if($profile->can_dispose($output)) {
                return $profile->dispose();
            }
        }
        return false;
    }
    
    
    /**
     * @brief Create config jobs for dialplan module
     *           
     * @param[in out] $config_manager
     *           
     * @return
     */
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        if(!$this->_generate_config($config_manager)) return false;
        return parent::reload_generate_config($config_manager, $obj_type);
    }

    /**
     * @brief 
     *
     * @param[in out] $obj_type
     *
     * @return 
     */
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }

    /**
     * @brief 
     *
     * @param[in out] $config_manager
     *
     * @return 
     */
    public function generate_config(&$config_manager)
    {
        if(!$this->_generate_config($config_manager)) return false;
        return parent::generate_config($config_manager);
    }
    /**
     * @brief 
     *
     * @param[in out] $config_manager
     *
     * @return 
     */
    private function _generate_config(&$config_manager)
    {
        // Delete all user dialplans
        $directory = "siprec_profiles";
        $conf_dir = $this->node()->configuration_manager()->directory('configuration');
        $file_list = $this->node()->os()->search_files($conf_dir.'/'.$directory.'/'.$this->name().'*.xml');
        foreach($file_list as $file){
            $config_manager->add_config(new Safe_configuration_class($file, '', Safe_configuration_class::CFG_DELETE, Safe_configuration_class::CFG_FILE));
        }
        // Create user dialplans
        foreach ($this->profiles() as $profile) {
            if (!$this->_write_profile($config_manager, $profile)) return false;
        }
        return true;
    }
    private function _write_profile(&$config_manager, $profile)
    {
        $file = "siprec_profiles/" . $profile->name() . ".xml";
        $config_manager->add_config(
                        new Safe_configuration_class(
                                $file, '',
                                Safe_configuration_class::CFG_DELETE,
                                Safe_configuration_class::CFG_FILE));
        
        $data = $profile->get_data_values(false);
        // Create xml document
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startDocument('1.0', 'UTF-8');
        $xml_writer->startElement('profile');
            $xml_writer->writeAttribute('name', $profile->name());
            foreach($data as $key => $value){
                if($key == 'gateways'){
                    $data['gateways'] = $profile->get_data_value('gateways', true);
                    $xml_writer->startElement('gateways');
                    foreach($data['gateways'] as $gateway){
                        $xml_writer->startElement('gateway');
                        $xml_writer->writeAttribute('name', 'sofia/gateway/'.$gateway);
                        $xml_writer->endElement();//gateway
                    }
                    $xml_writer->endElement();//gateways
                }else{
                    $xml_writer->startElement('param');
                    $xml_writer->writeAttribute('name',$key);
                    $xml_writer->writeAttribute('value', $value);
                    $xml_writer->endElement();//param
                }
            }
            
        $xml_writer->endElement();//profile
        $xml_writer->fullEndElement(); // Close Configuration
        // Content
        $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
        $config_manager->add_config(new Safe_configuration_class($file, $xmlstr));
        return true;
    }
    /**
     * @brief Invoked after a successfull write_config
     *         
     * @return
     */
    public function post_write_config()
    {
        // Reload configuration
        if(!$this->fs_app()->reloadxml()) return false;
        return parent::post_write_config();
    }
    public function check_object_usage($src_module_name, $obj_type, $obj_name, $sub_type = null, $sub_name = null) {
        // check usages ?
        $usages = array();
        $profiles = $this->profiles();
        foreach ( $profiles as $profile ) {
            $data = $profile->get_data_value('gateways', false);
            if ($data == null || $data == '') {
                continue;
            } else {
                foreach($data as $item){
                    if ($item == $obj_name && $src_module_name == 'sip' && $obj_type == 'trunk') {
                        $usages[] = array (
                                'name' => $this->name(),
                                'obj_type' => 'profile',
                                'obj_name' => $profile->name(),
                        );
                    }
                }
            }
        }
        return $usages;
    }
    public function can_create_aggregate_object($obj_type=null, $name, &$output = null){
        $module = $this->node()->software()->application()->find_module_by_name('sip');
        if(count($module->get_aggregate_objects('trunk')) <= 0){
            $err_msg[] = 'No SIP trunk defined.';
        }
        if($err_msg){
            $output =  $err_msg;
            return false;
        }
        return parent::can_create_aggregate_object($obj_type, $name, $output);
    }
}

