<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
 */
/*
 * FS Sangoma Media Server module class
 */
require_once('application/helpers/safe_helper.php');
safe_require_class('safe_service_class');
safe_module_require_class('fs','application');
safe_module_require_class('sng','transcoding_config');
safe_module_require_class('sng','transcoding_helper');
safe_module_require_class('fs','transcoding_module');

class Fs_transcoding_helper_class extends  Sng_transcoding_helper_class {

    private $_sngms_module;
    public function __construct($sngms_module){
        $this->_sngms_module = $sngms_module;
        parent::__construct("/", "Fs_transcoding_helper_class");
    }
    protected function create_module($mac){
        $module = new Fs_transcoding_module_class(
            $this->_sngms_module->node(), 
            $this->_sngms_module->object_name()."/module",
            $mac);
        return $module;
    }
}
class Fs_transcoding_device_class extends Safe_configurable_object_class
{
    public function __construct($node, $path,$name)
    {
        parent::__construct($path, $name, $node);
    }
    public function configure($version = null, $capacity = null, $port = null, $network_cfg = null)
    {
        $this->add_enum_field('split', 'Split Modules', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('split', 'When set to true macvlan devices will be created on demand per each vocallo module on this sngdsp interafce.');
        return parent::configure();
    }
    public function mac()
        {
            $network_module = $this->node()->find_module_by_name('network');
            $interfaces = $network_module->api_retrieve_interface();
            foreach ($interfaces as $interface) {
                if ($interface->get_data_value('vlan_type', false) == 'macvlan') {
                    if ($interface->get_data_value('macvlan_device') == $this->name()) {
                        return $interface->get_data_value('macvlan_mac');
                    }
                }
                else {
                    if ($interface->name() == $this->name()) {
                        return $interface->mac();
                    }
                }
            }
        return;
    }
    public function summary()
    {
        $result['data']['Name'] = array(
                'data' => $this->name(),
        );
        $result['data']['MAC'] = array(
                        'data' => $this->mac(),
                );
        $result['data'][$this->get_data_label('split')] = array(
                'data' => $this->get_data_value('split'),
        );
        return $result;
    }
}
class Fs_sngms_module_class extends Fs_module_class  {
    protected $_max_module_per_iface = 5;
    protected $_media_interface_ipv4 = array('sngdsp0' =>'192.0.2.1', 'sngdsp1' => '198.51.100.1');
    protected $_media_interface_ipv6 = array('sngdsp0' =>'2001:db8::1:1', 'sngdsp1' => '2001:db8::2:1');
    protected $_media_interface_ipv4_prefix = array('sngdsp0' => 24, 'sngdsp1' => 24);
    protected $_media_interface_ipv6_prefix = array('sngdsp0' => 120, 'sngdsp1' => 120);

    const UPDATE='update';
    const DETECT='detect';
    const NO_TRANSCODING='no transcoding';
    
    protected $_module_version = null;
    /**
     * @brief
     *
     * @param[in out] $fs_app
     *
     * @return
     */
    public function __construct($fs_app) {
        parent::__construct($fs_app, "sngms");
        $this->_obj_base_path = $this->path().'/sngms';

    }
    /**
     * @brief
     *
     * @return
     */
    public function configure(){
        // Set the module description
        $this->set_description("Media Server");
        $cfg = new Sng_transcoding_config_class($this->node(), $this->path() . '/' . $this->name());
        $cfg->configure();
        $cfg->synch();
        // Attach config to module
        $this->set_config($cfg, '/SAFe/fs_sngms_config');
        // Expose aggregate obj
        //sngms module not support reload
        $this->register_aggregate_object('module', 
            array(
                'name' => 'Module',
                'dynamic' => false,
                'base_path' => $this->_obj_base_path . '/module',
                'controller_url' => '/SAFe/fs_sngms_config',
                /* Add some extra infos for config_manager create object */
                'controller' => array(
                        'config_manager' => array(
                                'allow_create_button' => false,
                                'summary_buttons' => 'edit[Edit]'
                        ),  
                ),  
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a module',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a module',
                        'request' => 'POST',
                    ),
                    'status' => array(
                        'name' => 'Retrieve Status',
                        'description' => 'Retrieve module Status',
                        'request' => 'GET',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('device',
                array(
                        'name' => 'Device',
                        'dynamic' => false,
                        'base_path' => $this->_obj_base_path . '/device',
                        'controller_url' => '/SAFe/fs_sngms_config',
                        /* Add some extra infos for config_manager create object */
                        'controller' => array(
                                'config_manager' => array(
                                        'allow_create_button' => false,
                                        'summary_buttons' => 'edit[Edit]'
                                ),
                        ),
                        'methods' => array(
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a device',
                                        'request' => 'GET',
                                ),
                                'update' => array(
                                        'name' => 'Update',
                                        'description' => 'Update a device',
                                        'request' => 'POST',
                                )
                        ),
                )
        );

        return parent::configure();
    }
    public function udp_ports(){
        $start = 10000;
        $stop = 65000;
        $port_range = 4000;
        $port_start = $start;
        $port_stop = $start;
        if( !$this->config()->is_enabled()) {
            // Grab RTP ports for software transcod
            $this->config()->udp_ports($port_start, $port_stop);
            return array('start' => $port_start, 'stop' => $port_stop);
        }else{
            $objs = $this->api_retrieve_module();
            if($objs){
                foreach($objs as $obj){
                    $udp_begin = (int)$obj->udp_begin();
                    $udp_end = (int)$obj->udp_end();
                    if(($udp_begin < $port_start) || ($port_start == $start)){
                        $port_start = $udp_begin;
                    }
                    if(($udp_end > $port_stop) || ($port_stop == $stop)){
                        $port_stop = $udp_end;
                    }
                }
            }
        }
        if(($stop - $port_stop) > $port_range){
            return  array('start' => $port_stop + 1, 'stop' => $port_stop + $port_range );
        }elseif(($port_start - $start) > $port_range){
            return  array('start' => $port_start - $port_range, 'stop' => $port_start -1);
        }else{
            return false;
        }
    }
    public function api_create_module($obj_name=null)
    {//for doc
        $module = new Fs_transcoding_module_class($this->node(), $this->object_name()."/module", $obj_name);
        $module->configure();
        return $module;
    }
    public function api_retrieve_module($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('module');
        if($name){
            return $objs[$name];
        }else{
            ksort($objs);
            return $objs;
        }
    }
    public function api_update_module($name, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('module');
        if($objs[$name]) {
            if ($objs[$name]->validate($data,$output)) {
                if (true == $objs[$name]->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    public function api_create_device($obj_name=null)
    {//for doc
        $device = new Fs_transcoding_device_class($this->node(), $this->object_name()."/device", $obj_name);
        $device->configure();
        return $device;
    }
    public function api_retrieve_device($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('device');
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    public function api_update_device($name, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('device');
        if($objs[$name]) {
            if ($objs[$name]->validate($data,$output)) {
                if (true == $objs[$name]->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    public function get_action_settings($action=null)
    {
        // Populate action table
        $action_def = array();
        $action_def['detect'] = array(
                'name' => 'Detect Modules',
                'icon' => 'rocket',
                'description' => 'Detect Transcoding Modules',
                'request' => 'POST',
                'reload' => true,
                'redirect_url' => '/SAFe/fs_sngms_config'
        );
        $network = $this->node()->software()->application()->find_module_by_name('network');
        $transcoding_interfaces = $network->hardware_interface(Sng_network_interface_class::TRANSCODING);
        //no internal transcoding interfaces, show the hardware interface for user
        if(!$transcoding_interfaces){
            $action_def['detect']['display'] = false;
        }else{
            //show a confirm dialog
            $action_def['detect']['confirm'] = true;
        }

        // Check version of module if some
        $outdated_cnt = 0;
        $modules = $this->api_retrieve_module();
        if(count($modules)){
            $sng_transcoding_helper = new Fs_transcoding_helper_class($this);
            $modules_status = array();
            $sng_transcoding_helper->check_modules_version($this->node(), $output, $modules_status);
            foreach($modules_status as $module => $status){
                if(!$status){
                    $outdated_cnt++;
                }
            }
        }
        $update_label = 'Firmware Update';
        $button_class = null;
        if($outdated_cnt){
            $update_label .= ' (' . $outdated_cnt . ')';
            $button_class = 'danger';
        }
        $action_def['fwupdate'] = array(
            'name' => $update_label,
            'icon' => 'fire',
            'description' => 'Update Firmware',
            'request' => 'POST',
            'class' => $params,
            'reload' => true,
            'timeout' => '300',
            'button_class' => $button_class,
            'redirect_url' => '/SAFe/fs_sngms_config',
            'waitbox' => array( 'ui_type' => 'processing_log',
                                'log_url' => '/SAFe/fs_sngms_config/get_update_log',
                                'reloadtime' => '5',
                                'log_timeout' => '600',
                                'action_url' => '' //update successful action url
                    )
        );
        if(!$transcoding_interfaces) {
            $action_def['fwupdate']['display'] = false;
        }else{
            $action_def['fwupdate']['confirm'] = true;
        }

        if($action){
            return $action_def[$action];
        }else{
            return $action_def;
        }
    }

  /**
    * @brief Is module enabled
    *
    * @param[in out] $mod_name
    *
    * @return 
   */
  public function is_enabled($mod_name = "") {
    // Check if transcoding enabled ?
    $cfg = $this->config();
    if($cfg->is_enabled()){
      // force load module sngms
      // force noload for TDM
      if ($mod_name=="mod_sangoma_media_server.so"){
        return 'load';
      }
      if ($mod_name=="CORE_PCM_MODULE"){
        return 'noload';
      }

      return 'skip';
    }else{
        if ($mod_name=="CORE_PCM_MODULE"){
            return 'skip';
        }
    }
    return ($mod_name=="mod_sangoma_media_server.so")?'skip':'load';
  }
  /**
   * Returns whether the module is a critical module
   * @return boolean
   */
  public function is_critical($mod_name="") {
  	return ($mod_name=="mod_sangoma_media_server.so")?TRUE:FALSE;
  }
  /**
   * @brief Return FS module name
    *
    * @return string
   */
  public function get_fs_module_name() {
		//two module for Media server
      $transcoding_cfg = $this->config();
      $use_g729_trans = false;
      if(!$transcoding_cfg->is_enabled() && $transcoding_cfg->get_data_value('g729_transcoding', false) == 'true') {
          $use_g729_trans = true;
      }
    $mod = array(
      'mod_sangoma_media_server.so',
      'CORE_PCM_MODULE', 
      'mod_spandsp.so',
      'mod_ilbc.so',
      'mod_opus.so',
      'mod_g723_1.so',
      'mod_siren.so',
    );

    if ($use_g729_trans) {
        $mod [] = 'mod_bcg729.so';
    } else {
        $mod [] = 'mod_g729.so';
    }
    
    return $mod;
  }

    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    public function generate_config(&$config)
    {
        $network= $this->node()->software()->application()->find_module_by_name('network');
        $interfaces = $network->api_retrieve_interface();

        if ($this->config()->is_enabled()) {
            
            $hw_config = array();
            // Add global config
            $hw_config['config'] = array(
                'watchdog_timeout_ms' => 1000,
                'reactor_timeout_ms' => 100,
                'cpu_usage_low_threshold' => 85,
                'cpu_usage_high_threshold' => 95,
            );
            
            $module_objs = $this->api_retrieve_module();
            
            $devices = array();
            $transcoding_mode =  $this->config()->ip_mode();
            $skip_add_error = false;
            $infs = array();
            foreach($interfaces as $key => $interface) {
                if ($interface->interface_type() == Sng_network_interface_class::VIRTUAL && $interface->get_data_value('vlan_type', false) == 'macvlan') {
                    $infs[$interface->get_data_value('macvlan_device')] = $interface;
                }
                else {
                   $infs[$interface->interface_name()] = $interface;
                }
            }
            foreach($module_objs as $module_name => $module){
                if(!$module->enable()){
                    continue;
                }
                // Get the module interface
                $if_name = $module->ifname();
                // Entry created for device ?
                if(!isset($devices[$if_name])){
                    $if = $infs[$if_name];
                    if ($if == "") {
                        if (!$skip_add_error) {
                            $config->add_error(array(
                                'module' => 'sngms',
                                'obj_type' => 'module',
                                'description' => 'Transcoding interface(s) exist(s) on the system but not on configuration. A system restart is required to recreate the interface configuration.',
                                'url' => '/SAFe/sng_network_config'
                            ));
                            $skip_add_error = true;
                        }
                        safe_log(SAFE_LOG_INFO, "WARNING - Transcoding interface for: " . $if_name . " module: " . $module->mac() . " is missing in the configuration.");
                        continue;
                    }
                    if (preg_match("'^[Ss]ng[Dd]sp'", $if->interface_name())){
                        $model = 'D500';
                    }else{
                        $model = 'D150';
                    }
                    $devices[$if_name]= array(
                        'ifname' => $if->interface_name(),
                        'hwaddr' => $if->mac() ,
                        'model' => $model
                    );
                    $tmp_dsp_device = $this->api_retrieve_device($devices[$if_name]['ifname']);
                    if($tmp_dsp_device){
                        $devices[$if_name]['split_modules'] =  $tmp_dsp_device->get_data_value('split', false);
                    }
                    $devices[$if_name]['module'] = array();
                }
                $module_ip = $module->ip_address();
                $host_ip_map_entry = array();
                $ipv4 = array();
                $ipv6 = array();
                $module_ip_map = array();
                $module_ipv4_list = array();
                $module_ipv6_list = array();
                if($transcoding_mode == Sng_transcoding_config_class::HIDDEN_MODE){
                    //hidden mode
                    if($model == 'D150'){
                        $config->add_error(array(
                                'module' => 'sngms',
                                'obj_type' => 'module',
                                'description' => 'Only internal media interfaces can be used in hidden mode.',
                                'url' => '/SAFe/fs_sngms_config'
                        ));
                    }
                    if(isset($this->_media_interface_ipv4[$if_name]) || isset($this->_media_interface_ipv6[$if_name])){
                        $tmp_ip = $module->ip_address(false,Safe_ip_address_class::IP_V4);
                        $ipv4['ip'][] = array('@value' => $tmp_ip . '/' . $this->_media_interface_ipv4_prefix[$if_name]);
                        $module_ipv4_list[] = $tmp_ip;
                    
                        $ipv4['gw'][] = array(
                                              '@attributes' => array('hwaddr' => $devices[$if_name]['hwaddr']),
                                              '@value' => $this->_media_interface_ipv4[$if_name] . '/' .$this->_media_interface_ipv4_prefix[$if_name]
                                              );
                        // ipv6
                        $tmp_ip = $module->ip_address(false,Safe_ip_address_class::IP_V6);
                        $ipv6['ip'][] = array('@value' => $tmp_ip . '/' . $this->_media_interface_ipv6_prefix[$if_name]);
                        $module_ipv6_list[] = $tmp_ip;
                        $ipv6['gw'][] = array(
                                              '@attributes' => array('hwaddr' => $devices[$if_name]['hwaddr']),
                                              '@value' => $this->_media_interface_ipv6[$if_name],
                                              );
                        $ipv6['prefix'] = safe_cidr2network($this->_media_interface_ipv6[$if_name], $this->_media_interface_ipv6_prefix[$if_name]) . '/' . $this->_media_interface_ipv6_prefix[$if_name];
                    }
                }else{
                    $module_ipv4 = $module->ip_address(false, Safe_ip_address_class::IP_V4);
                    if($module_ipv4){
                        $ipv4['ip'][] = array(
                                              '@value' => $module_ipv4 . '/' . safe_netmask2cidr($module->mask()),
                                              );                
                        if (strlen($module->gateway(Safe_ip_address_class::IP_V4))){
                            if(strlen($module->mask())){
                                $ipv4['gw'] = $module->gateway(Safe_ip_address_class::IP_V4) . '/' .safe_netmask2cidr($module->mask());
                            }else{
                                $ipv4['gw'] = $module->gateway(Safe_ip_address_class::IP_V4);
                            }
                        }
                        $module_ipv4_list = array($module_ipv4);
                    }
                    //ipv6
                    $module_ipv6 = $module->ip_address(false, Safe_ip_address_class::IP_V6);
                    if($module_ipv6){
                        $ipv6['ip'][] = array(
                                              '@value' => $module_ipv6 . '/' . $module->prefix(),
                                              );
                        $tmp_ipv6_gateway = $module->gateway(Safe_ip_address_class::IP_V6);
                        if (strlen($tmp_ipv6_gateway)){
                            $ipv6['gw'] = $tmp_ipv6_gateway;
                            $tmp_ipv6_prefix = $module->prefix();
                            if (strlen($tmp_ipv6_prefix)){
                                $ipv6['prefix'] = safe_cidr2network($tmp_ipv6_gateway, $tmp_ipv6_prefix) . '/' . $module->prefix();
                            }
                        }
                        $module_ipv6_list = array($module_ipv6);
                    }
                }
                $module_data['ip_address'] = $module_ipv4_list;
                $module_data['ipv6_address'] = $module_ipv6_list;
                $module_data['ip_map'] = $module_ip_map;
                //set ip map to module obj
                $module->set_data_values($module_data);
                $module->save();
                // Create module configuration
                $module_cfg = array(
                    'hwaddr' => $module->mac() 
                );
                if($ipv4){
                    $module_cfg['ipv4'] = $ipv4;
                }
                if($ipv6){
                    $module_cfg['ipv6'] = $ipv6;
                }
                $module_cfg['eth_port'] = $module->port();
                $module_cfg['udp'] = array(
                        'begin' => $module->udp_begin() ,
                        'end' => $module->udp_end() ,
                    ) ;
                $module_cfg['max_sessions'] = $module->max_sessions();
                
                if (strlen($module->ext_ip_address())){
                    $module_cfg['external_ip'] = $module->ext_ip_address();
                }
                
                $vqe = $module->vqe();
                if($vqe){
                    $module_cfg = array_merge($module_cfg, $vqe);
                }
                // Store module info
                $devices[$if_name]['module'][] = $module_cfg;

                $media_server_config = require('fs_sngms_config_template.php');                
                $jitter = $module->jitter();
                if(!empty($jitter['mode'])){
                    $media_server_config['media']['jitter_mode'] = $jitter['mode'];
                }
                if(!empty($jitter['max_pdv_ms'])){
                    $media_server_config['media']['jitter_max_pdv_ms'] = $jitter['max_pdv_ms'];
                }
                if(!empty($jitter['min_delay_ms'])){
                    $media_server_config['media']['jitter_min_delay_ms'] = $jitter['min_delay_ms'];
                }
                if(!empty($jitter['initial_delay_ms'])){
                    $media_server_config['media']['jitter_initial_delay_ms'] = $jitter['initial_delay_ms'];
                }

                $tmp_xml = Array2XML::createXML('jsr309', $media_server_config);
                $config->add_config(new Safe_configuration_class(
                    'mod_sangoma_media_server/config.xml',
                    Safe_format_xml($tmp_xml->saveXML())));
            }
            // Add devices to hwconfig
            foreach($devices as $device){
                $hw_config['device'][] = $device;
            }
            if (isset($hw_config['device']) && !empty($hw_config['device'])) {
                
                $tmp_xml = Array2XML::createXML('vocallo', $hw_config);
                $config->add_config(
                    new Safe_configuration_class(
                        'mod_sangoma_media_server/vocallo.xml', 
                        Safe_format_xml($tmp_xml->saveXML())));
            } else {
                $config->add_error(array(
                    'module' => 'hardware',
                    'obj_type' => 'adapter',
                    'description' => 'No Media Server Interfaces configured',
                    'url' => '/SAFe/fs_sngms_config'
                )
            );
            }

        } else {
            //Remove Vocallo.xml
            $config->add_config(
                new Safe_configuration_class(
                    'mod_sangoma_media_server/vocallo.xml', 
                    '', 
                    Safe_configuration_class::CFG_DELETE, Safe_configuration_class::CFG_FILE));
        }
        return parent::generate_config($config);
    }
    
    /*
     * Return usage statistics
     */
    public function term_sessions(&$result)
    {
        $output = array();
        if ($this->eslapi_execute('hupall','',$output))
        {
            return true;
        }
        return false;
    }

    //Drops all active sessions
    public function service_request(&$result, $mac, $service_type)
    {
        $cmd = 'sngms_control service ' . $service_type . ' mac ' . $mac;
        if ($this->eslapi_execute($cmd,'',$result))
        {
            return true;
        }
        return false;
    } 

    //Returns usage statistics
    public function stats_usage(&$stats, $summary=false){
        $output = array();
        $cmd = 'sngms_stats_xml usage';
        if (!$summary)
            $cmd = $cmd.' all';

        if($this->eslapi_execute($cmd, "", $output) ) 
        {
            $stats = Safe_xml_to_array($output);
            if (!$summary) {
                // Check if module array is created correctly
                // When only 1 module is returned, conversion from XML to array
                // can't figure out this is and arry. Ending up in:
                // [modules]
                //   [module]
                //     [stats]
                //
                // Instead of
                // [modules]
                //   [module]
                //     [0] <-- Missing info
                //       [stats]
                //
                if (isset($stats['usage'])
                    && isset($stats['usage']['modules'])
                    && isset($stats['usage']['modules']['module'])
                    && !is_indexed_array($stats['usage']['modules']['module']))
                {
                    $tmp = $stats['usage']['modules']['module'];
                    unset($stats['usage']['modules']['module']);
                    $stats['usage']['modules']['module'] = array(
                        '0' => $tmp);
                }
            }
            return true;
        }
        return false;
    }

    //Returns duration statistics
    public function stats_duration(&$stats,$summary=false){
        $output = array();
        $cmd = 'sngms_stats_xml duration';
        if (!$summary)
            $cmd = $cmd.' all';


        if($this->eslapi_execute($cmd,"",$output) ) 
        {
            $stats = Safe_xml_to_array($output);
            if (!$summary) {
                // Check if module array is created correctly
                // When only 1 module is returned, conversion from XML to array
                // can't figure out this is and arry. Ending up in:
                // [modules]
                //   [module]
                //     [stats]
                //
                // Instead of
                // [modules]
                //   [module]
                //     [0] <-- Missing info
                //       [stats]
                //
                if (isset($stats['duration'])
                    && isset($stats['duration']['modules'])
                    && isset($stats['duration']['modules']['module'])
                    && !is_indexed_array($stats['duration']['modules']['module']))
                {
                    $tmp = $stats['duration']['modules']['module'];
                    unset($stats['duration']['modules']['module']);
                    $stats['duration']['modules']['module'] = array(
                        '0' => $tmp);
                }
            }
            return true;
        }
        return false;
    }
    private function log_format($str){
        return '<pre>' .$str . '</pre>';
    }
    public function get_fwupdate_log($name=null, $data=null, &$output=null)
    {
        $log_file = '/tmp/module_update.log';
        if(file_exists($log_file)){
            $tmp_content = file_get_contents($log_file);
            //replace ^M
            $tmp_content = str_replace("\r", PHP_EOL, $tmp_content);
            $tmp_content = str_replace(array('-','/',"\\",'|'), '', $tmp_content);
            $tmp_content = str_replace('Please confirm firmware update (yn)', '', $tmp_content);
            $output = explode(PHP_EOL, $tmp_content);
        }else{
            $tmp_content = $log_file . ' not found!';
        }
        if(is_array($output)){
            $output = array_map(array($this, 'log_format'), $output);
        }else{
            $output[] = '';
        }
        return $rs;
    }
    public function api_fwupdate_action($data=null, &$output=null)
    {
        if (Safe_service_class::STATUS_RUNNING == $this->node()->software()->application()->status()){
            $output['message'] = 'Cannot update module firmware.';
            $output['reason'] = $this->node()->software()->application()->description() . ' is running.';
            return false;
        }
        $log_file = '/tmp/module_update.log';
        if(file_exists($log_file)){
            //clear the old logs
            $this->node()->os()->delete_file($log_file);
        }
        $action_def = $this->get_action_settings('fwupdate');
        if(isset($action_def['class'])){
            $param_class = $action_def['class'];
            if(!isset($data['interfaces'])){
                $data['interfaces'] = '';
            }
            if(!$param_class->validate($data, $output)){
                return false;
            }else{
                $param_class->save();
            }
            $if_names = $param_class->get_data_value('interfaces', false);
        }else{
            $network_module = $this->node()->software()->application()->find_module_by_name('network');
            $interfaces_transcoding = $network_module->hardware_interface(Sng_network_interface_class::TRANSCODING);
            $if_names = array();
            foreach($interfaces_transcoding as $interface_obj_name => $interface_obj){
                $if_names[] = $interface_obj->interface_name();
            }
        }
        $tmp_output = array();
        $rc = true;
        // Get the transcoding helper
        $sng_transcoding_helper = new Fs_transcoding_helper_class($this);
        // Run update on all interfaces
        foreach($if_names as $if_name){
            $_output = array();
            $_rc = $sng_transcoding_helper->upgrade_modules($this->node(), $if_name, $_output);
            if(!$_rc){
                $tmp_output = array_merge($tmp_output, $_output);
                $rc = false;
            }else{
                //request reboot
                $this->node()->software()->application()->add_reboot_request($this->name(), 'Module firmware successfully updated.');
            }
        }
        if(true != $rc){
            $output['message'] = 'Module firmware update failed.';
            $output['reason'] = $tmp_output;
        }else{
            // Verify version
            $_output = array();
            if(!($rc = $sng_transcoding_helper->check_modules_version($this->node(), $_output))){
                $output['message'] = 'Module firmware update failed.';
                $output['reason'] = 'The Media Interfaces may need to reset by restart the system and redetect the modules.';
            }else{
                $output['message'] = 'Module firmware successfully updated to version: '.
                    $sng_transcoding_helper->version($this->node());
                
                //after upgrade fw, need to redetect modules
                foreach($this->api_retrieve_module() as $module){
                    $module->dispose();
                }
                foreach($this->api_retrieve_device() as $tmp_device){
                    $tmp_device->dispose();
                }
                $this->clear_cache('module');
                $network_module = $this->node()->software()->application()->find_module_by_name('network');
                $transcoding_interfaces = $network_module->hardware_interface(Sng_network_interface_class::TRANSCODING);
                //if have internal transcoding interfaces, auto re detect
                if($transcoding_interfaces){
                    if(!$this->api_detect_action(null, $tmp_output)){
                        $output['message']  = $output['message'] . ' <div class="safe_error">But Media Interface detection failed.</div> ';
                    }else{
                        //ask user reboot
                        $output['message']  = $output['message'] . ' <div class="safe_error">Please restart your system.</div> ';
                    }
                }else{
                    $output['message']  = $output['message'] . ' <div class="safe_error">Please redetect modules.</div> ';
                }
            }
            //clear cache
            $this->clear_list_cache();
        }
        return $rc;
    }

    public function api_detect_action($data=null, &$output=null)
    {
        if (Safe_service_class::STATUS_RUNNING == $this->node()->software()->application()->status()){
            $output['message'] = $output['message'] . 'Cannot detect media interface.';
            $output['reason'] = $this->node()->software()->application()->description() . ' is running.';
            return false;
        }

        $params = $this->config()->get_data_values(false);

        if (!$this->config()->is_enabled()) {
            // TODO: return proper error msg
            return false;
        }else{
            $network_module = $this->node()->software()->application()->find_module_by_name('network');
            $interfaces = $network_module->api_retrieve_interface();
            $interfaces_transcoding = $network_module->hardware_interface(Sng_network_interface_class::TRANSCODING);
            //clear all cached files before detect
            $script_path = $this->node()->configuration_manager()->directory('sng-base') . '/bin';
            $rs = $this->node()->execute("sudo nohup {$script_path}/sng-update-transcoding-modules forget ", $output);
            
            if(!empty($interfaces)){
                // Remove all modules
                foreach($this->api_retrieve_module() as $module){
                    $module->dispose();
                }
                foreach($this->api_retrieve_device() as $tmp_device){
                    $tmp_device->dispose();
                }
                $this->clear_cache('module');
                // Keep a copy of parameters
                $params_copy = $params;
                // Check if external ip defined
                if (isset($params_copy['ext_ip_address']) && !strlen($params_copy['ext_ip_address'])){
                    unset($params_copy['ext_ip_address']);
                }
                if (isset($params_copy['ext_ipv6_address']) && !strlen($params_copy['ext_ipv6_address'])){
                    unset($params_copy['ext_ipv6_address']);
                }
                // Detect interfaces
                $detected_modules = 0;
                //Create an array of ip addresses to validate
                $ip_list = array();
                foreach($interfaces as $interface){
                    $ips = $network_module->get_ips($interface->name());
                    foreach($ips as $ip){
                        $ip_list[] = $ip->address();
                    }
                }
                // Build up list of interfaces to run detection on
                $if_list = array();
                if (!empty($data['interfaces'])) {
                    foreach ($data['interfaces'] as $interface) {
                        if(isset($interfaces[$interface])){
                            $if_list[$interface] = $interfaces[$interface];
                        }
                    }
                }else{
                    // internal adapters
                    $if_list = $interfaces_transcoding;
                }
                $sng_transcoding_helper = new Fs_transcoding_helper_class($this);
                // Run detection
                foreach($if_list as $if_name => $if){
                    if ($this->config()->ip_mode() == Sng_transcoding_config_class::HIDDEN_MODE) {
                        //hidden mode
                        $interface_name = strtolower($if->interface_name());
                        // Copy back original parameter for aggregate mode
                        $upd_begin = $params['udp_begin'];
                        $params = $params_copy;
                        // Update IP and gw address
                        $ips = $network_module->get_ips($if->name());
                        $ipv4_obj = false;
                        $ipv6_obj = false;
                        foreach($ips as $ip_item){
                            if($ip_item->ip_type() == Safe_ip_address_class::IP_V4){
                                if(isset($this->_media_interface_ipv4[$interface_name])){
                                    //check if the interface is sngsdp
                                    if($ip_item->address() == $this->_media_interface_ipv4[$interface_name]){
                                        //find the right internal ip
                                        $ipv4_obj = $ip_item;
                                    }
                                }else{
                                    if(!$ipv4_obj){
                                        $ipv4_obj = $ip_item;
                                    }
                                }
                            }
                            if($ip_item->ip_type() == Safe_ip_address_class::IP_V6){
                                if(isset($this->_media_interface_ipv6[$interface_name])){
                                    if($ip_item->address() == $this->_media_interface_ipv6[$interface_name]){
                                        //find the right internal ip
                                        $ipv6_obj = $ip_item;
                                    }
                                }else{
                                    if(!$ipv6_obj){
                                        $ipv6_obj = $ip_item;
                                    }
                                }
                            }
                        }
                        //activate the interface (the sngdspX may down)
                        $set_interface_obj = new Sng_network_link_up_config($interface_name, array('ifname' => $interface_name));
                        $set_interface_obj->run($this->node());
                        if(isset($this->_media_interface_ipv4[$interface_name]) && !$ipv4_obj){
                            //sngdsp no internal ipv4
                            $add_ipv4_result = false;
                            if($if->hardware_type() == Sng_network_interface_class::TRANSCODING || $if->get_data_value('vlan_type', false) == 'macvlan'){
                                //assgin ipv4 to internal transcoding interface
                                $ipv4_obj_name  = $network_module->build_unique_name('ip');
                                $ipv4_obj = $network_module->api_create_ip($ipv4_obj_name);
                                if($ipv4_obj){
                                    $ipv4_data = array(
                                            'interface' => $if->name(),
                                            'proto' => Safe_ip_address_class::PROTO_STATIC4,
                                            'address' => $this->_media_interface_ipv4[$interface_name],
                                            'prefix' => $this->_media_interface_ipv4_prefix[$interface_name],
                                    );
                                    $ipv4_obj->set_data_values($ipv4_data);
                                    $add_ipv4_result = $ipv4_obj->save();
                                    if($add_ipv4_result){
                                        //activate the ip
                                        $set_ip_obj = new Sng_network_ip_add_config($ipv4_obj_name, $ipv4_data);
                                        $set_ip_obj->run($this->node());
                                        $ip_list[] = $ipv4_obj->address();
                                    }
                                }
                            }
                        }
                        if(!$ipv4_obj){
                            $output['message'] = array(
                                    "Media Interface detection failed.",
                                    $interface_name ." interface has no IP address."
                            );
                            return false;
                        }
                        if(isset($this->_media_interface_ipv6[$interface_name]) && !$ipv6_obj){
                            //sngdsp no internal ipv6
                            if($if->hardware_type() == Sng_network_interface_class::TRANSCODING  || $if->get_data_value('vlan_type', false) == 'macvlan'){
                                //assgin ipv6 to internal transcoding interface
                                $ipv6_obj_name  = $network_module->build_unique_name('ip');
                                $ipv6_obj = $network_module->api_create_ip($ipv6_obj_name);
                                if($ipv6_obj){
                                    $ipv6_data = array(
                                            'interface' => $if->name(),
                                            'proto' => Safe_ip_address_class::PROTO_STATIC6,
                                            'address' => $this->_media_interface_ipv6[$interface_name],
                                            'prefix' => $this->_media_interface_ipv6_prefix[$interface_name],
                                    );
                                    $ipv6_obj->set_data_values($ipv6_data);
                                    $add_ipv6_result = $ipv6_obj->save();
                                    if($add_ipv6_result){
                                        //activate the ip
                                        $set_ip_obj = new Sng_network_ip_add_config($ipv6_obj_name, $ipv6_data);
                                        $set_ip_obj->run($this->node());
                                        $ip_list[] = $ipv6_obj->address();
                                    }
                                }
                            }
                        }
                        
                        $ipv4_addr = $ipv4_obj->address();
                        $params['gateway'] = $ipv4_addr;
                        increment_ip($ipv4_addr);
                        $params['ip_address'] = $ipv4_addr;
                        $params['mask'] = $ip->mask();
                        $params['udp_begin'] = $upd_begin; 
                        $params['udp_end'] = $params['udp_begin'] + 3999;
                        
                        //ipv6 can be empty
                        if($ipv6_obj){
                            $ipv6_addr = $ipv6_obj->address();
                            $params['ipv6_gateway'] = $ipv6_addr;
                            increment_ip($ipv6_addr);
                            $params['ipv6_address'] = $ipv6_addr;
                            $params['ipv6_prefix'] = $ipv6_obj->prefix();
                        }else{
                            $params['ipv6_gateway'] = '';
                            $params['ipv6_address'] = '';
                            $params['ipv6_prefix'] = '';
                        }
                    }
                    $detected_modules+= $sng_transcoding_helper->detect_modules(
                        $this->node(), $if->interface_name(), $params, $ip_list);
                }

                if (!$detected_modules) {
                    $output['message'] = array(
                        "Media Interface detection failed.",
                        "No modules detected."
                    );
                    return false;
                }else{
                    $output['message'] = array(
                        "Media interface detection succeeded.",
                        $detected_modules . " modules detected."
                    );
                    //create device by detected module ifname
                    $dsp_devices = array();
                    $tmp_modules = $this->api_retrieve_module();
                    foreach($tmp_modules as $tmp_module){
                        $ifname = $tmp_module->ifname();
                        if(!isset($dsp_devices[$ifname])){
                            $device_obj = $this->api_create_device($ifname);
                            $device_obj->save();
                        }
                    }
                    return true;
                }
            }
        }
        return false;
    }
    public function checklist(&$list)
    {
        $nsc_session = safe_get_session('stopping_nsc');
        if($nsc_session){
            safe_unset_session('stopping_nsc');
            return true;
        }

        // HW transcoding enabled ?
        $cfg = $this->config();
        if($cfg->is_enabled()){
            $modules = $this->api_retrieve_module();
            if(!count($modules)){
                $list[] = array(
                    'module' => $this->name(),
                    'obj_type' => 'configuration',
                    'description' => 'No Transcoding Modules',
                    'claim_to'=>self::NO_TRANSCODING
                );
            }else{
                // Check version is up to date
                $sng_transcoding_helper = new Fs_transcoding_helper_class($this);
                $modules_status = array();
                $sng_transcoding_helper->check_modules_version($this->node(), $output, $modules_status);
                //check if have ipv6 address
                $network_module = $this->node()->software()->application()->find_module_by_name('network');
                $have_ipv6 = $network_module !== NULL && $network_module->have_ip_type(Safe_ip_address_class::IP_V6);
                foreach($modules as $module){
                    if($module->enable() && !$modules_status[$module->name()]){
                        $list[] = array(
                            'module' => $this->name(),
                            'obj_type' => 'module',
                            'obj_name' => $module->name(),
                            'description' => 'Firmware is outdated. Please update firmware or disable module.',
                            'claim_to'=>self::UPDATE
                        );
                    }
                    if($have_ipv6 && $module->enable() && empty($module->ip_address(false, Safe_ip_address_class::IP_V6))){
                        $list[] = array(
                                        'module' => $this->name(),
                                        'obj_type' => 'module',
                                        'obj_name' => $module->name(),
                                        'description' => 'Detect Modules on Media Interface is requered',
                                        'claim_to'=>self::DETECT
                                        );
                    }
                }
            }
        }
    }
    public function check_object_modified($obj, $src_module_name, $obj_type, $obj_name, $sub_type = null, $sub_name = null)
    {
        if($src_module_name == 'transcoding' && 'configuration' == $obj_type){
            // Remove all modules if sngms configuration modfied
            foreach($this->api_retrieve_module() as $module){
                $module->dispose();
            }
            foreach($this->api_retrieve_device() as $tmp_device){
                $tmp_device->dispose();
            }
            $this->clear_cache('module');
        }
        if($src_module_name == 'network' && 'ip' == $obj_type){
            // mark modules as modified when any ip changes
            foreach($this->api_retrieve_module() as $module){
                $module->mark_modified();
            }
        }
        return parent::check_object_modified($obj, $src_module_name, $obj_type, $obj_name, $sub_type, $sub_name);
    }
    public function get_module_version()
    {
        //cache the version info
        if(!$this->_module_version){
            $sng_transcoding_helper = new Fs_transcoding_helper_class($this);
            $modules_status = array();
            $sng_transcoding_helper->check_modules_version($this->node(), $output, $modules_status);
            $this->_module_version = $modules_status;
        }
        return $this->_module_version;
    }
    //get the vocallos modules fw is uptodate
    public function api_status_module($name=null, $data=null, &$output = null) {
        $objs = $this->get_module_version();
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    //sngms module and associated configuration should not support reload
    public function support_reload(&$need_reload = NULL)
    {
        if (isset($need_reload)) {
            $need_reload = $this->is_configuration_modified(true);
        }
        return false;
    }
    
    public function post_restore($info, &$reason)
    {
        $rs = parent::post_restore($info, $reason);
        //trigger rescan vocallos when restore
        //(HW mode with internal adapters) 
        safe_log(SAFE_LOG_INFO, ' Trigger media interfaces auto detect.');
        if($this->config()){
            $this->config()->mark_modified();
            $this->config()->save();
        }
        return $rs;
    }
    public function array_to_eventxml($event_data){
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startDocument('1.0','UTF8');
        $xml_writer->startElement('monit');
        $xml_writer->startElement('event');
            foreach($event_data as $key => $value){
                $xml_writer->startElement($key);
                if($key === 'message'){
                    $xml_writer->writeCdata($value);
                } else {
                    $xml_writer->text($value);
                }
                $xml_writer->endElement();
            }
        $xml_writer->endElement();
        $xml_writer->fullEndElement();
        $xml_writer->endDocument();
        $event_xmlstr = $xml_writer->outputMemory(true);
        return $event_xmlstr;
    }
    public function handle_global_messages(&$msg_list)
    {
        $this->clear_global_messages_cache($msg_list, 'core', 'force_software');
        $core_module = $this->node()->software()->application()->find_module_by_name('core');
        $core_data = $core_module->config()->get_data_values(false);
        $sngms_data = $this->config()->get_data_values(false);

        if(isset($core_data) && isset($sngms_data)){
            if(intval($core_data['force-software']) == 1 && $sngms_data['mode'] == 'software'){
                $msg_list[] = array(
                    'module' => 'sngms',
                    'cache_tag' => 'forced_software',
                    'global_errors' => array(
                        "message" => "The NSC service forced HW mode to switch to SW mode",
                        "buttons" => array(array(
                            'label' => 'Check',
                            'url' => '/SAFe/fs_sngms_config/modify/sngms/force-software',
                        ))
                    )
                );
            }
        }
    }
    public function api_update_configuration($obj_name = null, $params = array(), &$output = null, $validation_check = true)
    {
        $core_module = $this->node()->software()->application()->find_module_by_name('core');
        $core_data = $core_module->config()->get_data_values(false);
        $sngms_data = $this->config()->get_data_values(false);

        if(intval($core_data['force-software']) == 1 && $sngms_data['mode'] == 'hardware-hidden'){
            $monitor = $this->node()->software()->application()->find_module_by_name('monitor');
            $auditor = $this->node()->software()->application()->find_module_by_name('auditor');
            $notifier = $this->node()->software()->application()->find_module_by_name('notifier');

            $event_data = array(
                'collected_sec' => gettimeofday()['sec'],
                'collected_usec' => gettimeofday()['usec'],
                'service' => 'nsc/log/error',
                'type' => 1,
                'id' => 512,
                'state' => 2,
                'action' => 1,
                'message' => 'The system was in HW mode but the system detected a hardware transcoding issues so the system switched automatically to SW mode'
            );
            $event_xml_str = $this->array_to_eventxml($event_data);
            $result = $auditor->collector($event_xml_str, true);
            if($result && isset($result['events'])){
                foreach($result['events'] as $level => $events_cat){
                    $notifier->notify($level, $events_cat);
                    $monitor->add_events($level, $events_cat);
                }
            }
        }
        return parent::api_update_configuration($obj_name, $params, $output, $validation_check);
    }
}
/* End of file fs_sngms_module_class.php */
