<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: * */
/** coding: utf-8: * */
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * William Adam <wadam@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
 */
/**
 * TDM Gateway service wrapper (NSG)
 *
 * @contributors
 *  Abhishek Arora
 * @version
 */
if (!defined('BASEPATH'))
    exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');
safe_require_class('safe_auditor_class');
require_once ('application/helpers/Rest_client.php');

/* *****************************************************************************
 *   TDMGW Module Configuration
 * **************************************************************************** */
class Fs_tdmgw_config_class extends Safe_configurable_object_class
{
    /**
     * @brief Constructor
     *
     * @param[in out] $node
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return 
     */
    public function __construct($node, $parent_name) {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, "configuration", $node);
    }

    /**
     * @brief Configure fields
     *
     * @return 
     */
    public function configure() {

        $this->add_enum_field(
            'enable', 
            'Enable TDM Gateway', 
            'dropdown',
            'false',
            $this->enable_disable_enum()
        );

        return parent::configure();
    }
}

class Fs_tdmgw_generic_config_class extends Safe_configurable_object_class
{
    /**
     * @brief Constructor
     *
     * @param[in out] $node
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return 
     */
    public function __construct($node, $parent_name, $name) {
        parent::__construct($parent_name, $name, $node);
    }

    /**
     * @brief Configure fields
     *
     * @return 
     */
    public function configure() {

        $this->add_field('content', 'Configuration', 'text', '');

        return parent::configure();
    }
}

class Fs_push_nsg_cfg_class extends Safe_configuration_user_class {
    public function __construct($cfg_list)
    {
        parent::__construct('Push TDMGW cfg', Safe_configuration_class::CFG_UPDATE, $cfg_list);
    }

    public function run($node) {
        $tdmgw= $node->find_module_by_name('tdmgw');
        // Push all cfgs to NSG
        foreach($this->data() as $cfg_item){
            // TODO: Account for errors
            $tdmgw->push_cfg($cfg_item);
        }
        return true;
    }
}
class Fs_pull_nsg_cfg_class extends Safe_configuration_user_class {
    public function __construct($cfg_list)
    {
        parent::__construct('Pull TDMGW cfg', Safe_configuration_class::CFG_UPDATE, $cfg_list);
    }

    public function run($node) {
        $tdmgw= $node->find_module_by_name('tdmgw');
        // Push all cfgs to NSG
        foreach($this->data() as $cfg_item){
            // TODO: Account for errors
            $tdmgw->pull_cfg($cfg_item);
        }
        return true;
    }
}
class Fs_reload_nsg_cfg_class extends Safe_configuration_user_class {
    public function __construct()
    {
        parent::__construct('Reload TDMGW cfg', Safe_configuration_class::CFG_UPDATE);
    }

    public function run($node) {
        $tdmgw= $node->find_module_by_name('tdmgw');
        $tdmgw->_reload_configuration();
        return true;
    }
}
class Fs_apply_nsg_cfg_class extends Safe_configuration_user_class {
    private $_restart;
    public function __construct($restart=false)
    {
        $this->_restart = $restart;

        parent::__construct('Apply TDMGW cfg', Safe_configuration_class::CFG_UPDATE);
    }

    public function run($node) {
        $tdmgw= $node->find_module_by_name('tdmgw');
        $tdmgw->_apply_configuration($this->_restart);
        return true;
    }
}

/* *****************************************************************************
 *   TDMGW Module
 * **************************************************************************** */
class Fs_tdmgw_service_class extends Safe_service_class
{
    private $_rest_client = null;
    // NSG REST api cfg
    private $_rest_config = array('server' => 'http://127.0.0.1:8888/');
    // List of cfg item owned by NSC
    private $_owned_cfg_list = array('sip', 'dialplan');
    // List of cfg item NOT owned by NSC (ie. owned by NSG like span)
    private $_not_owned_cfg_list = array('span');
    // Complete list (merge of the above lists)
    private $_cfg_list = array();
    // List of cfg item that may be reloaded
    private $_reloadable_cfg_list = array('sip', 'dialplan');

    /**
     * @brief Constructor
     *
     * @param[in out] $app
     *
     * @return 
     */
    public function __construct($app) {
        // For now use directly the nsg service instead of turning to REST to 
        // control service
        parent::__construct($app, "tdmgw", "nsg");
        // Create full cfg list
        $this->_cfg_list = array_merge($this->_owned_cfg_list, $this->_not_owned_cfg_list);
    }

    /**
     * @brief Create REST connection
     *
     * @return 
     */
    private function _connect()
    {
        if(null == $this->_rest_client){
            $this->_rest_client = new Rest_client();
            $this->_rest_client->initialize($this->_rest_config);
        }
    }

    /**
     * @brief Confifure service
     *
     * @return 
     */
    public function configure()
    {
        // Set service description
        $this->set_description("TDM Gateway");
        // Create configuration
        /*
        $cfg = new Fs_tdmgw_config_class($this->node(), $this->path() . '/' . $this->name());
        $cfg->configure();
        $cfg->synch();
         */
        // Attach config to module
        //$this->set_config($cfg);

        // Register objects
        // To control NSG we have to take care of following config points:
        // - span (TDM config)
        // - SIP (FS sip configuration)
        // - dialplan (sip to TDM dialplan XML)
        //
        // As NSG handles all above cfg points, we should turns to its REST api 
        // SAP to get cfg, update (from global cfg push) and generate cfg 
        // related functions.
        // Until we control span, sip or dialplan as real objects in FS (NSC) 
        // product we should only store/push info from NSG directly
        // thus the singleton object that will be most likely not exposed in UI
        $this->register_aggregate_object(
            'cfg', 
            array(
                'name' => 'Configuration',
                'dynamic' => true,
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve TDM Gateway configuration',
                        'request' => 'GET'
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update TDM Gateway configuration',
                        'request' => 'POST'
                    ),
                )
            )
        );

        // Check if cfg_list items  exists (and allowed to be created, ie. nsg 
        // + rest + some TDM boards
        // TODO: check allow/disallow tdm service
        if(true){
            // Get all cfg items
            foreach($this->_cfg_list as $cfg_item){
                $this->pull_cfg($cfg_item);
            }
        }

        return parent::configure();
    }
    /*
     * API cfg section
     */
    public function api_create_cfg($name=null, $data=null, &$output=null)
    {
        $obj = new Fs_tdmgw_generic_config_class($this->node(), $this->path() . '/' . $this->name() . '/cfg', $name);
        $obj->configure();
        return $obj;
    }

    public function api_retrieve_cfg($name=null, $data=null, &$output=null)
    {
        if(null == $name){
            return $this->get_aggregate_objects('cfg');
        }else{
            return $this->get_aggregate_object('cfg', $name);
        }
    }
    public function api_update_cfg($name=null, $data=null, &$output=null)
    {
        $obj = $this->api_retrieve_cfg($name);

        if($obj){
            if($obj->validate($data, $output)){
                // TODO: Here we push cfg upon update. This SHOULD NOT be the 
                // case in final version. This is only meant to test the 
                // push_cfg function using REST easily without having to 
                // generate cfg every time
                // the push_cfg should be done on generate cfg as explain in IM
                if($this->push_cfg($name, $data, $output)){
                    return $obj->save();
                }
                return $obj->save();
            }
        }
        return false;
    }

    public function pull_cfg($cfg_item)
    {
        // We own the SIP/dialplan configuration, however at install time this is empty
        // and we need to turn to NSG to get original content (this should 
        // probably be also cleanup on SW update so we get new sip config if 
        // ever modified.
        // 1st check if current sip content is empty and if so turn to NSG rest 
        // api to get it, otherwise just return object content
        //
        // todo: we are applying exact same behaviour as for sip and span
        // but we are not responsible for span cfg edition this is performed by 
        // nsg ui directly. we need to add a method in nsg to actually inform 
        // nsc that span cfg has been modified thus we need to fetch new config 
        // over rest
        $_cfg = $this->api_retrieve_cfg($cfg_item);
        if(!$_cfg){
            $_cfg = $this->api_create_cfg($cfg_item);
        }
        if($_cfg){
            $_content = $_cfg->get_data_value('content', false);
            // empty ? or not owned by nsc - TODO: optimize so we don't fetch 
            // from NSG on every requests
            if(empty($_content) || in_array($cfg_item, $this->_not_owned_cfg_list)){
                // yes, so query content to nsg rest
                $this->_connect();
                $data = (array)$this->_rest_client->get(
                    'rest.php', 
                    array(
                        'method'=>'retrieve',
                        'module'=> $this->name(),
                        'obj_type'=> 'cfg',
                        'obj_name'=> $cfg_item,
                    )
                );
                if($data && $data['status'] == true){
                    $_content = $data['data']->content;
                    $_cfg->set_data_values(array('content' => $_content));
                    $_cfg->save();
                }
            }
        }
        return $_cfg;
    }
    public function push_cfg($cfg_item)
    {
        // Retrieve the cfg item (from db)
        $_cfg = $this->api_retrieve_cfg($cfg_item);
        if($_cfg){
            $data = $_cfg->get_data_values(false);

            // Push to NSG
            $this->_connect();
            $data = (array)$this->_rest_client->post(
                'rest.php', 
                json_encode(
                    array(
                        'method'=>'update',
                        'module'=> $this->name(),
                        'obj_type'=> 'cfg',
                        'obj_name'=> $cfg_item,
                        'content' => $data['content'],
                    )
                ),
                "json"
            );
            if($data && $data['status'] == true){
                return true;
            }
        }
        // TODO - return proper error
        return false;
    }
    public function _reload_configuration()
    {
        // Push to NSG
        $this->_connect();
        $data = (array)$this->_rest_client->post(
            'rest.php', 
            json_encode(
                array(
                    'method'=>'reload',
                    'module'=> $this->name(),
                    'obj_type'=> 'configuration',
                )
            ),
            "json"
        );
        if($data && $data['status'] == true){
            return true;
        }
        // TODO - return proper error
        return false;
    }
    public function _apply_configuration($restart = true)
    {
        // Push to NSG
        $this->_connect();
        $data = (array)$this->_rest_client->post(
            'rest.php', 
            json_encode(
                array(
                    'method'=>'apply',
                    'module'=> $this->name(),
                    'obj_type'=> 'configuration',
                    'data' => array('mode' => $restart ? "restart" : "generate"),
                )
            ),
            "json"
        );
        if($data && $data['status'] == true){
            return true;
        }
        // TODO - return proper error
        return false;
    }

    /**
     * @brief Generate configuration and/or reload
     *
     * Only 'owned' object for NSG should be pushed, then reload can be 
     * invoked (in post_reload)
     *
     * First push all nsc 'owned' cfg item
     * 2nd, instruct nsg to apply cfg
     * 3rd, pull all non nsc 'owned' cfg as they may have changed
     *
     * Then if tdmgw process is started, then:
     * 1/ ask for restart if any 'non owned' cfg are modified
     * 2/ ask for reload if not and any 'owned' cfg are modified
     *
     * @param[in out] $config_manager
     * @param[in out] $obj_type
     *
     * @return 
     */
    private function _generate_config(&$config_manager, $obj_type=null)
    {
        // Check tdmgw is running ?
        $_running = false;
        if(Safe_service_class::STATUS_RUNNING == $this->status()){
            $_running = true;
        }
        $_reload = false;
        // Every cfg owned by NSC must be pushed to NSG
        foreach($this->_owned_cfg_list as $_cfg_item){
            $_cfg = $this->get_aggregate_object('cfg', $_cfg_item);
            if($_cfg && $_cfg->is_modified()){
                $config_manager->add_config(new Fs_push_nsg_cfg_class(array($_cfg_item)));
                $_reload = true;
            }
        }
        // Check if some 'not owned' cfg items are modified
        foreach($this->_not_owned_cfg_list as $_cfg_item){
            $_cfg = $this->get_aggregate_object('cfg', $_cfg_item);
            if($_cfg && $_cfg->is_modified()){
                // Then ask for apply
                $config_manager->add_config(new Fs_apply_nsg_cfg_class($_running));
                // No need to reload as we will restart service anyway
                $_reload = false;
                // Retrieve latest version of non owned cfg
                $config_manager->add_config(new Fs_pull_nsg_cfg_class($this->_not_owned_cfg_list));
            }
        }
        // Check if running and reload needed
        if($_running && $_reload){
            $config_manager->add_config(new Fs_reload_nsg_cfg_class());
        }

        return parent::reload_generate_config($config_manager, $obj_type);
    }

    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        $this->_generate_config($config_manager, $obj_type);
        return parent::reload_generate_config($config_manager, $obj_type);
    }
    public function reload_clear_modified($obj_type=null)
    {
        $this->clear_configuration_modified(true);
    }
    public function generate_config(&$config_manager)
    {
        $this->_generate_config($config_manager, $obj_type);
        return parent::generate_config($config_manager);
    }

    public function get_data_settings()
    {
        $cfg_manager = $this->node()->configuration_manager();
        $data = array(
            'log' => array(
                'description' => 'Log File',
                'filesystem' => TRUE,
                'pattern' => '/var/log/' . $this->name() . '.log*',
                'global_capabilities' => array(
                    'download',
                    //'rotate'
                ) ,
                'capabilities' => array(
                    'download',
                    'delete',
                    'view'
                ),
                'methods' => array(
                        'download' => array(
                                'name' => 'Download',
                                'description' => array( 'Download log files.' ),
                                'request' => 'GET',
                        ),
                        'delete' => array(
                                'name' => 'Delete',
                                'description' => array( 'Delete a  Core Dump files.' ),
                                'request' => 'POST',
                        ),
                ),
            ) ,
            'core' => array(
                'description' => 'Core Dump',
                'filesystem' => TRUE,
                'pattern' => $cfg_manager->directory('core-dump') . '/core.' . $this->name() . '*',
                'global_capabilities' => array(
                    'delete'
                ) ,
                'capabilities' => array(
                    'download',
                    'delete'
                ),
                'methods' => array(
                        'download' => array(
                                'name' => 'Download',
                                'description' => array( 'Download a Core Dump files.' ),
                                'request' => 'GET',
                        ),
                        'delete' => array(
                                'name' => 'Delete',
                                'description' => array( 'Delete a  Core Dump files.' ),
                                'request' => 'POST',
                        ),
                ),
            ) ,
            'corebt' => array(
                'description' => 'Core Dump Back Trace',
                'filesystem' => TRUE,
                'pattern' => $cfg_manager->directory('core-dump') . '/core.' . $this->name() . '*.bt',
                'global_capabilities' => array(
                    'download',
                    'delete'
                ) ,
                'capabilities' => array(
                    'download',
                    'delete',
                    'view'
                ),
                'methods' => array(
                        'download' => array(
                                'name' => 'Download',
                                'description' => array( 'Download a Core Dump files.' ),
                                'request' => 'GET',
                        ),
                        'delete' => array(
                                'name' => 'Delete',
                                'description' => array( 'Delete a  Core Dump files.' ),
                                'request' => 'POST',
                        ),
                ),
            ) ,
        );
        return $data;
    }
}
