<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
* All Rights Reserved.
*
* Author(s)
* your name <your_name@sangoma.com>
*
* This code is Sangoma Technologies Confidential Property.
* Use of and access to this code is covered by a previously executed
* non-disclosure agreement between Sangoma Technologies and the Recipient.
* This code is being supplied for evaluation purposes only and is not to be
* used for any other purpose.
*/
/*
 * Node class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_os_class');
safe_require_class('safe_software_class');
safe_require_class('safe_object_class');
safe_module_require_class('sng','transcoding_module');

class Fs_transcoding_module_class extends Sng_transcoding_module_class
{
    public function configure($version = null, $capacity = null, $port = null, $network_cfg = null)
    {
        parent::configure($version, $capacity, $port, $network_cfg);
        //data come form detect 
        if (isset($network_cfg['ip_address'])) {
            $this->add_field('ip_address', 'IPv4 address', 'text', array($network_cfg['ip_address']));
        }
        if (isset($network_cfg['ipv6_address'])) {
            $this->add_field('ipv6_address', 'IPv6 address', 'text', array($network_cfg['ipv6_address']));
        }
        //defines mapping between external host IP to vocallo IP
        $this->add_field('ip_map', 'IP Map', 'hidden', array(), 50);
    }
    public function set_data_values($data){
        if(isset($data['ip_address'])){
            if(!is_array($data['ip_address'])){
                $data['ip_address'] =  to_index_array($data['ip_address']);
            }
        }
        if(isset($data['ipv6_address'])){
            if(!is_array($data['ipv6_address'])){
                $data['ipv6_address'] =  to_index_array($data['ipv6_address']);
            }
        }
        return parent::set_data_values($data);
    }
    public function ip_address($all = false, $type = Safe_ip_address_class::IP_V4)
    {
        if($type == Safe_ip_address_class::IP_V6){
            $ip_list = $this->get_data_value('ipv6_address');
        }else{
            $ip_list = $this->get_data_value('ip_address');
        }
        if($all){
            return $ip_list;
        }else{
            $ip = '';
            if($ip_list){
                $ip = $ip_list[0];
            }
            return $ip;
        }
    }
    public function get_ip_map(){
        return $this->get_data_value('ip_map', false);
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        if($this->node()->get_config_status('update') == 'rest'){
            //if update from rest, remove some validation rules
            $this->set_field_readonly('ifname', false);
            $this->set_field_readonly('version', false);
            
            $this->unset_field('capacity');
            $this->unset_field('port');
            $this->add_field('capacity', 'Capacity', 'string', '');
            $this->add_field('port', 'Port', 'string', '');
            $this->set_field_disabled('jitter/mode', false);
            $this->set_field_disabled('jitter/initial_delay_ms', false);
            $this->set_field_disabled('jitter/max_pdv_ms', false);
            $this->set_field_disabled('jitter/min_delay_ms', false);
        }else{
            $sngms_module = $this->node()->software()->application()->find_module_by_name('sngms');
            $transcoding_mode = $sngms_module->config();
            $mode = $transcoding_mode->ip_mode();
            if($mode == Sng_transcoding_config_class::HIDDEN_MODE){
                $ip_list = $this->get_data_value('ip_address');
                if(is_array($ip_list)){
                    $ip_list = implode(PHP_EOL,$ip_list);
                }
                $this->unset_field('ip_address');
                $this->add_field('ip_address', 'IPv4 address', 'textarea', $ip_list, array('30', '5'));
                $this->set_field_rules('ip_address', '');
                $this->set_field_readonly('ip_address');
            
                $ipv6_list = $this->get_data_value('ipv6_address');
                if(is_array($ipv6_list)){
                    $ipv6_list = implode(PHP_EOL,$ipv6_list);
                }
                $this->unset_field('ipv6_address');
                $this->add_field('ipv6_address', 'IPv6 address', 'textarea', $ipv6_list, array('30', '5'));
                $this->set_field_rules('ipv6_address', '');
                $this->set_field_readonly('ipv6_address');
            }else{
                $ip = $this->ip_address(false, Safe_ip_address_class::IP_V4);
                $this->unset_field('ip_address');
                $this->add_field('ip_address', 'IPv4 address', 'text', $ip);
                $ipv6 = $this->ip_address(false, Safe_ip_address_class::IP_V6);
                $this->unset_field('ipv6_address');
                $this->add_field('ipv6_address', 'IPv6 address', 'text', $ipv6);
            }    
        }
    }
    public function validate($data, &$output){
        if($this->node()->get_config_status('update') != 'rest'){
            $sngms_module = $this->node()->software()->application()->find_module_by_name('sngms');
            $transcoding_mode = $sngms_module->config();
            $mode = $transcoding_mode->ip_mode();
            if($mode == Sng_transcoding_config_class::HIDDEN_MODE){
                unset($data['ip_address']);
                unset($data['ipv6_address']);
            }
            $this->unserialize();
        }
        return parent::validate($data, $output);
    }
    public function summary($type = 'horizontal' , $long = false){
        $version = $this->version();
        $sngms_module = $this->node()->software()->application()->find_module_by_name('sngms');
        $vesion_list = $sngms_module->get_module_version();
        if(isset($vesion_list[$this->mac()]) && !$vesion_list[$this->mac()]){
            $version =  ' <div class="safe_error">'. $this->version() . safe_icon('exclamation-circle','error','lg') .'</div>';
        }
        if($this->enable()){
            $icon = safe_icon('check','info', 'lg');
        }else{
            $icon = safe_icon('times', 'error', 'lg');
        }
        $ipv4_list = $this->get_data_value('ip_address',false);
        if(is_array($ipv4_list)){
            $ip[Safe_ip_address_class::IP_V4] = array_shift($ipv4_list);
            if(count($ipv4_list)>0){
                $ip[Safe_ip_address_class::IP_V4] = $ip[Safe_ip_address_class::IP_V4] . ' - '. array_pop($ipv4_list);
            }
        }else{
            $ip[Safe_ip_address_class::IP_V4] = $ipv4_list;
        }
        $ipv6_list = $this->get_data_value('ipv6_address',false);
        if(is_array($ipv6_list)){
            $ip[Safe_ip_address_class::IP_V6] = array_shift($ipv6_list);
            if(count($ipv6_list)>0){
                $ip[Safe_ip_address_class::IP_V6] = $ip[Safe_ip_address_class::IP_V6] . ' - '. array_pop($ipv6_list);
            }
        }else{
            $ip[Safe_ip_address_class::IP_V6] = $ipv6_list;
        }
        $table_line = array(
                "" => array('data' => $icon, 'formatter' => 'none'),
                "MAC" => $this->mac(),
                "Version" => array('data' => $version, 'formatter' => 'none'),
                "IP address" => $ip,
                "UDP Ports" => $this->udp_begin() . ' - ' . $this->udp_end()
        );
        return array('data' => $table_line);
    }
}
?>
