<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Network Config wrapper page
 * @author William Adam <william.adam@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_controller('sng','data_manager');
safe_require_class('safe_file_data_source_class');
safe_require_class('safe_data_table_class');
class Fs_cdr_data extends Sng_data_manager
{
    public function __construct()
    {
        parent::__construct();
    }
    public function index($module_name = 'cdr', $data_name = 'cdr', $offset = NULL)
    {
        return $this->manage($module_name, $data_name, $offset);
    }

    /**
     * @brief This method overwrites the one from sng_data_manager. It creates
     * an empty table that will get the data making ajax request back to the
     * server (ajax method from this controller)
     */
    public function manage($module_name,$data_name,$offset=""){
        
        $module = $this->_get_module($module_name);
        $data_cfg = $module->get_data_settings();
        
        if (isset($data_cfg[$data_name])) $data_cfg = $data_cfg[$data_name];
    
        $data_cnt = $module->get_data_count($data_name);
        $glob_caps = array();
        if ($data_cfg['global_capabilities_no_data']) $glob_caps = array_merge($glob_caps, $data_cfg['global_capabilities_no_data']);
        if ($data_cnt && $data_cfg['global_capabilities']) $glob_caps = array_merge($glob_caps, $data_cfg['global_capabilities']);
        $action_def = array();

        foreach ($glob_caps as $glob_cap) {
            $action_def[$glob_cap] = $data_cfg['methods'][$glob_cap];
            //fix the backup url error
            unset($action_def[$glob_cap]['class']);
        }

        // Get the module actions
        $module_actions = array(
            'action_settings' => $action_def,
            'base_url' => '/SAFe/' . $this->router->fetch_class() . '/',
            'url_suffix' => '/'.$module_name . '/' . $data_name,
            'action_param_suffix' => '_form',
        );
        // Crud table auxiliary to create empty table
        $table = new Safe_crud_table_class();
        // Data table manager, useful for dataTables.js compatibility and to
        // perform pagination and filtering in a optimal way for memory use
        $data_table = new Safe_data_table_class(array('data_source'=>new Safe_file_data_source($data_cfg['file_dir'],$data_cfg['recursive']),
                                                      'settings'=>array('serverSide'=>true,
                                                                        'processing'=>true,
                                                                        'ajax'=>array('url'=>$module_actions['base_url'].'/ajax')                                                                      
                                                                        ),
                                                      'rows_per_page'=>10
                                                      ));
        
        $data=array();

        $data['page_title'] = 'Application Data Manager';
        $data['page_summary'] = 'This page allows managing '.$module->description().' '.$data_cfg['description'].'.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-backup-settings.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-backup-settings.png';
        $data['table_size']= '100%';
        $data['table_id'] = 'CDR_report-table';
        //        $data['table_container_class'] = 'safe_datatable';
        $data['table_title'] = $table->set_title($module->description().' - '.$data_cfg['description']);
        $data['table_settings'] = json_encode($data_table->settings());
        $table->set_headings(array('Name',''));
        $table->set_footer('&nbsp;');
        $data['table'] = $table->generate_table();
        $this->load->view('safe_header',$data);
        $this->load->view("safe_module_actions", $module_actions);
        $this->load->view('safe_table',$data);
        $this->load->view('safe_footer',$data);
    }

    /**
     * @brief This method handles all request comming from CDR report dataTable
     * view. It uses the new Safe_data_table class to perform filtering,
     * quick data access and pagination.
     */
    public function ajax($module_name='cdr',$data_name='cdr',$offset=null){
        $module = $this->_get_module($module_name);
        $data_cfg = $module->get_data_settings();
        
        if (isset($data_cfg[$data_name])) $data_cfg = $data_cfg[$data_name];
        
        $data_table = new Safe_data_table_class(array('data_source'=>new Safe_file_data_source($data_cfg['file_dir'],$data_cfg['recursive']),
                                                      'rows_per_page'=>$_GET['length']));
        $total = $data_table->total();
        $query = array();
        // If search is on request and it is not null, we filtering results
        if(isset($_GET['search']) && $_GET['search']['value']){
            $query = array('file'=>$_GET['search']['value']);
            $data_table->find($query);
        }
        $rows = array();
        // Crud table auxiliary to buttons the same way we done before.
        $table = new Safe_crud_table_class();
        // Arguments are going to be passed as parametter for each row callback.
        $args=(object) array('rows'=>&$rows,'safe_table'=>$table,'data_cfg'=>$data_cfg,'self'=>$this,'module_name'=>$module_name,'data_name'=>$data_name);
        // Page we want
        $page = intval($_GET['start']/$_GET['length']);
        // Performing data transformation as dataTable.js is expecting
        $data_table->each_row($page,function($i,$row,&$args){
                // removing extension
                $ext_regexp = isset($args->data_cfg['file_ext'])? '\.'.$args->data_cfg['file_ext'] : '\.[a-z]{1,3}';
                $name = preg_replace('#'.$ext_regexp.'#','',basename($row['path']));
                // buttons creation
                $line_button = $args->self->create_line_button($args->data_cfg,array('Name'=>$name));
                $buttons = safe_format_button($line_button);
                $div = '<div id="buttons'.$i.'" style="float:right;">';
                foreach($buttons as $key => $button){
                    $button['name'] = ucfirst($key);
                    $url_infix = $key;
                    if(preg_match('#'.$key.':([^|]+)#',$line_button,$match))
                        $url_infix.="_".$match[1];
                    $div.= $args->safe_table->button($button,'SAFe/' . $args->self->router->fetch_class() . '/'.$url_infix.'/'.$args->module_name.'/'.$args->data_name.'/'.$name,
                                                     isset($button['extra_arguments'])? $button['extra_arguments'] : '',
                                                     isset($button['extra_class'])? $button['extra_class'] : '');
                }
                $div.= '</div>';
                // adding row
                $args->rows[] = array($name,$div);
            },$args);
        $filtered = $data_table->total($query);
        $output = array('draw'=>$_GET['draw'],
                        'recordsTotal'=>$total,
                        'recordsFiltered'=>$filtered,
                        'data'=>$rows);
        print json_encode($output);
        die();
    }
}
