<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
 */
/**
 * FS SIP TDMGW  Config wrapper page
 * @author Abhishek Arora <aarora@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_controller('sng','config_manager');
class Fs_tdmgw_config_edit extends Sng_config_manager
{
    protected $_module = null;
    protected $_module_name = null;
    protected $_obj_name = null;
    private $_preferences;
    public function __construct($module_name = 'tdmgw')
    {
        parent::__construct();
        $this->_module_name = $module_name;
        $this->_module = $this->_the_app->find_module_by_name($module_name);
        $this->_preferences = $this->_the_app->api_retrieve_preferences();
    }
    public function index()
    {
        // This is the SIP configuration edit page for TDMGW module, only allow 
        // edition of this specific obj: mod=tdmgw, type=cfg, name=sip
        // All other config_manager function should not be accessible to user
        return $this->edit($this->_module_name, 'cfg', $this->_obj_name);
    }

    public function edit($module_name, $obj_type = null, $obj_name = null, $sub_type = null, $sub_name = null)
    {
        // Prepare view data
        $data = array(); 
        //Prepare table structure
        $data['table_size'] = '100%';
        $data['text_mode'] = "xml";
        //$data['keyboard_mode'] = "emacs";
        //$data['keyboard_mode'] = "vim";
        $data['keyboard_mode'] = "standard";
        // Get editor mode from preferences
        if(isset($this->_preferences)){
            $data['keyboard_mode'] = $this->_preferences->get_data_value('keyboard_mode',false);
            //$data['auto_hide'] = 'notif_warn';
            $data['notif_warn'][] = 'Editor mode is '.$this->_preferences->get_data_value('keyboard_mode',true)
                .' click '.safe_hyperlink('/SAFe/sng_settings_config','here').' to change.';
        }
        // Get the object
        $cfg_item = $this->_module->get_aggregate_object($obj_type, $obj_name);
        // If obj found then get data from the object
        if($cfg_item){
            // TODO: Find a name for this
            $data['table_title'] = 'Configuration Object-Name: '.$obj_name;
            // Editor view accepts only only buffer (not array), so 
            // implode to create buffer.
            $data['editor_data'] = $cfg_item->get_data_value('content', false);
            // Get the form data
            $config_form = $cfg_item->get_data();
            unset($config_form['data']);
        }
        // Set view url for Save only
        $data['url_save'] = base_url($this->router->fetch_class()).'/save/'.$module_name.'/'.$obj_type.'/'.$obj_name;
        //$data['url_cancel'] = base_url($this->router->fetch_class());

        $this->load->view("safe_header", $data );
        $this->load->view("safe_confirm_dialog", $data);//load confirm dialog js and css
        $this->load->view("safe_hide_menu", $data );

        $this->load->view('safe_notifications',$data);
        if($data['keyboard_mode'] == "standard")
            $this->load->view("safe_texteditor",$data);
        else
            $this->load->view("safe_texteditor_ace",$data);
        $this->load->view("safe_footer", $data );
    }
    public function save($module_name, $obj_type, $obj_name){
        // verify if logged
        //safe_authenticate();
        // Check post request
        $post = $this->input->post();

        // Default is error
        $response = array(
            'result' => 'ERROR' );

        if (isset($post['data'])) {
            // Retrieve object (in our case a cfg type with name=sip)
            $cfg_item = $this->_module->get_aggregate_object($obj_type, $obj_name);
            // If obj found then validate if submitted data is correct
            if($cfg_item){
                $_data = array(
                    'content' => base64_decode($post['data']));
                // validate
                $error = "";
                if($this->_module->api_update_cfg($obj_name, $_data, $error)){
                    $response['result'] = "OK";
                    $response['redirect'] = base_url($this->router->fetch_class());
                }else{
                    $error = "Fail to save cfg item";
                    $response['message'] = $error;
                }
            }else{
                $error = "Invalid configuration item";
                $response['message'] = $error;
            }
        }
        echo json_encode($response);
    }
}
