<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * NSC Domain Profile Class
 * @author: Shaunt Libarian
 *
 */
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_require_class('hardware');
safe_module_require_class('nsc', 'application');
safe_module_require_class('nsc', 'domain_users');
safe_module_require_class('fs', 'domain');
class Nsc_domain_class extends Fs_domain_class
{
    /**
     * @brief
     *
     * @param[in out] $node
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return
     */
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($node, $parent_name, $name);
    }
    public function create_user($user_name)
    {
        return new Nsc_domain_users_class($this->_node, $this->object_name() . '/user', $user_name);
    }
    private function _sip_profiles()
    {
        $_the_app = $this->_node->software()->application();
        return $_the_app->sip_profiles();
    }
    /**
     * @brief
     *
     * @return
     */
    public function configure()
    {
        $this->add_field('display-name', 'Display Name', 'string', '', 50);
        $this->set_field_rules('display-name','alpha_dash_star');
        
        $this->add_enum_field('multiple-registrations', 'Multiple registrations', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('multiple-registrations','Enable or Disable ability to register several phones using the same extension.');
        
        $this->add_enum_field('forward-registration', 'Forward Registration / Authentication', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('forward-registration', 'Enable/Disable registration and authentication requests to a registrar server.');
        // SIP profile enum list
        $sip_module = $this->_node->software()->application()->find_module_by_name('sip');
        $sip_profile_enum = $sip_module->object_enum('profile', ADD_NONE_ALWAYS, false, '--- Select Profile ---');

        $this->add_enum_field('registrar-profile', 'Forward SIP profile', 'dropdown', '', $sip_profile_enum);
        $this->set_field_rules('registrar-profile', 'required[forward-registration]|required[,in,none]');
        $this->set_field_help('registrar-profile', 'This SIP profile will be used to contact the registrar server and forward the REGISTER message');

        $sip_module = $this->node()->software()->application()->find_module_by_name('sip');
        $field_def = $sip_module->object_enum('trunk', ADD_NONE_NEVER);
        $this->add_multiple_field('registrar-gateways', 'Trunk List', 'listbox', array(), $field_def, 10);
        $this->set_field_help('registrar-gateways', 'List of sip trunk to be used for forwarding registrations on this domain. Round robbin load balancing will be used to forward the REGISTRATIONS to the configured list of gateways.');

        $this->add_field('sip-force-expires', 'Force Expires', 'string', '', 10);
        $this->set_field_rules('sip-force-expires', 'integer|greater_or_equal[5]');
        $this->set_field_help('sip-force-expires', 'If specified, it is the number of seconds to force registration expiration to. Minimum value is 5.');
        
        $this->add_enum_field('block-ip-on-unresponsive-challenge', 'Block IP On Unresponsive Challenged Registration', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('block-ip-on-unresponsive-challenge', 'Enable/Disable IP Blocking on unresponsive challenged registration.');

        $this->add_field('block-ip-duration', 'IP Blocking duration', 'string', '60', 10);
        $this->set_field_rules('block-ip-duration', 'integer|greater_or_equal[0]');
        $this->set_field_help('block-ip-duration', 'If specified, it is the number of seconds to block an IP address. 0 = forever.');

        $this->add_enum_field('sip-forward-refer-headers', 'Forward Refer Headers', 'dropdown', 'true', $this->enable_disable_enum());
        $this->set_field_help('sip-forward-refer-headers', 'Enable/Disable forward refer headers.');

        $this->add_enum_field('forward-registration-remote-network-info', 'Forward Registration Origination Network Info', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('forward-registration-remote-network-info','If Enabled SBC will forward the network information from the regsitration origination endpoint into an X header in the forwarded REGISTER message.');


        $this->add_field('reg-server-error-map', 'Server Error Codes', 'string', '408', '100%');
        $this->set_field_help('reg-server-error-map', 'Comma delimited error codes that are considered as server errors. Firewall will not block registration errors matching any item in the list.');

        // Create conditional group
        $this->create_group('register', array(
            'registrar-profile',
            'registrar-gateways',
            'sip-force-expires',
            'block-ip-on-unresponsive-challenge',
            'block-ip-duration',
            'sip-forward-refer-headers',
            'reg-server-error-map'
        ));

        $this->create_group('blockip',array('block-ip-duration'));
        // 2. tell the field 'register' which existing group it can control (register)
        $this->conditional_control('forward-registration', 'register');
        $this->conditional_control('block-ip-on-unresponsive-challenge', 'blockip');
        return parent::configure();
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        // Check if system is running and object as been serialized
        $the_app = $this->_node->software()->application();
        if (Safe_service_class::STATUS_RUNNING == $the_app->status()
                && Safe_object_serializer_class::OBJ_STATUS_NEW != $this->status()) {
            
            $usages = $this->node()->software()->application()->check_object_usage('directory','domain', $this->name());
            if($usages){
                $sip_module = $the_app->find_module_by_name('sip');
                foreach($usages as $tmp_item){
                    if($tmp_item['name'] == 'sip' && $tmp_item['obj_type'] == 'profile'){
                        // Check binded sip profile status
                        $profile_status = $sip_module->api_status_profile($tmp_item['obj_name']);
                        if($profile_status){
                            $sip_profile_obj = $sip_module->api_retrieve_profile($tmp_item['obj_name']);
                            $sip_profile_display_name = $sip_profile_obj->display_name();
                            $ro_fields = array(
                                    'display-name'
                            );
                            // Prevent edit
                            foreach($ro_fields as $f){
                                $this->set_field_disabled($f, true);
                                // Adjust help so user knows why :)
                                $help = $this->get_field_help($f);
                                if(!is_array($help)){
                                    $help = array($help);
                                }
                                $help = array_merge(
                                        array(
                                                "<font color='red'>This parameter cannot be changed because SIP Profile '{$sip_profile_display_name}' is running.</font>",
                                                '',
                                        ), $help);
                                $this->set_field_help($f, $help);
                            
                            }
                            break;
                        }
                    }
                }
            }
        }
    }
    public function validate($data, &$output){
        $rs =  parent::validate($data, $output);
        // validate that all configured gateways are using the registrar profileas sip profile
        if($this->is_forward_enabled()) {
            $gateway_count = 0;
            $registrar_profile = $this->get_data_value('registrar-profile',false);
            $module = $this->_node->software()->application()->find_module_by_name('sip');
            $gateways = $this->get_data_value('registrar-gateways',false);
            if($gateways){
                foreach($gateways as $gateway){
                    if ($gateway != '') {
                        $gateway_count++;
                    }
                    $trunk = $module->api_retrieve_trunk($gateway);
                    if ( !$trunk || $trunk->sip_profile_check($registrar_profile) == false) {
                        if ($trunk) {
                            $this->set_error('registrar-gateways', "Trunk ".$trunk->name()." must use sip profile ".$registrar_profile.
                                    " to match domain forward profile configuration", $output);
                        } else {
                            $this->set_error('registrar-gateways', "Invalid registrar PBX ".$gateway, $output);
                        }
                        return false;
                    }
                }
            }
            if ($gateway_count <= 0) {
                $this->set_error('registrar-gateways', 'You have to assign at least one registrar gateway when forward registration is enabled', $output);
                return false;
            }
        }
        if ($this->node()->software()->application()->check_display_name_conflict('directory', 'domain', $this->display_name(), $this->name())) {
            $this->set_error('display-name', $this->display_name() .' already exists.', $output);
            return false;
        }
        return $rs;
    }
    /**
     * Checks to see if object can be deleted
     * @see Safe_serializable_object_class::can_dispose()
     */
    public function can_dispose(&$reason)
    {
        //Check if media profile used
        $usages = $this->node()->software()->application()->check_object_usage('directory','domain', $this->name());
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return parent::can_dispose($reason);
    }
    public function is_sip_profile_used($sip_profile)
    {
        $cfg = $this->get_data_values(false);
        if (isset($cfg['registrar-profile']) && $sip_profile == $cfg['registrar-profile']) return true;
        return false;
    }
    public function is_forward_enabled()
    {
        $forward = $this->get_data_value('forward-registration', false);
        return ('true' == $forward);
    }

    public function api_create_user($name, $data=null) {
        $user = new Nsc_domain_users_class($this->_node, $this->object_name() . '/user', $name);
        $user->configure();
        if($data) {
            $user->set_data_values($data);
        }
        return $user;
    }

    public function sip_trunk_check($sip_trunk) {

        if (!$this->is_forward_enabled()) {
            return false;
        }
        $gateways = $this->get_data_value('registrar-gateways',false);
        foreach($gateways as $gateway) {
            if ($gateway == $sip_trunk) {
                return true;
            }
        }
        return false;
    }

    public function summary($type = 'horizontal' , $long = false){
        $gateways = array();
        foreach($this->get_data_value('registrar-gateways', false) as $gateway) {
            $_obj = $this->node()->software()->application()->find_obj_by_name('sip', 'trunk', $gateway);
            if($_obj)
                $gateways[] = $_obj->display_name();
        }
        $table_line = array("Domain Name" => $this->display_name(),
                "Forward" => $this->get_data_value('forward-registration'),
                "Registrar Gateways" => ($this->is_forward_enabled()) ? $gateways : 'N/A');
        return array('data' => $table_line);
    }
    public function new_fields(&$config_manager, $type)
    {
        parent::new_fields($config_manager, $type);
        $condition_number = count($this->get_aggregate_objects($type))+1;
        $new_name = ucfirst($type) . $condition_number;
        $config_manager->add_field('profile-name', 'Domain User ID', 'string', $new_name, 50);
    }
    public function sip_profile_check($sip_profile){
        $prof_data = $this->get_data_values(false);
        if($prof_data['forward-registration'] == 'true'){
            return ($prof_data['registrar-profile'] == $sip_profile) ? true : false;
        }else{
            return false;
        }
    }
    /**
     * @brief Check if ACL is used by any domain user
     *
     * @param[in out] $acl_profile
     *
     * @return true if used
     */
    public function acl_check($acl_user) {
        $user_data = $this->users();
        $users = array();
        foreach ($user_data as $user) {
            $cfg = $user->get_data_values(false);
            if ($cfg['cidr'] == $acl_user)
                $users[] = $user->name();
        }
        return $users;
    }
}
?>
