<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * NSC Domain Users Class
 * @author: Shaunt Libarian
 */
require_once ('application/helpers/safe_helper.php');
safe_module_require_class('nsc', 'application');
safe_module_require_class('fs', 'domain_user');
class Nsc_domain_users_class extends Fs_domain_users_class
{
    private $_the_app = "";
    /**
     * @brief
     *           
     * @param[in out] $node
     * @param[in out] $parent_name
     * @param[in out] $name
     *           
     * @return
     */
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($node, $parent_name, $name);
        $this->_the_app = $node->software()->application();
    }
    /**
     * @brief
     *         
     * @return
     */
    public function configure()
    {
        $this->add_field('password', 'Password', 'password', '', 50);
        $this->set_field_rules('password', "required|valid_sip_password");
        $this->set_field_help('password', array('SIP Password for this domain user',
                'Valid password characters: A-Z / a-z / 0-9 / "-" / "_" / "." / "!" / "~" / "*" / "\'" / "(" / ")" / "&" / "=" / "+" / "$" / "," '));
        $this->add_field('effective_caller_id_name', 'Caller ID Name', 'string', '', 50);
        $this->set_field_rules('effective_caller_id_name', '');
        $this->set_field_help('effective_caller_id_name', 'Caller ID Name of this Domain User');
        $this->add_field('effective_caller_id_number', 'Caller ID Number', 'string', '', 50);
        $this->set_field_rules('effective_caller_id_number', '');
        $this->set_field_help('effective_caller_id_number', 'Caller ID Number of this Domain User.');
        $this->add_field('sip_force_expires', 'Force Expires', 'string', '', 10);
        $this->set_field_rules('sip_force_expires', 'integer|greater_or_equal[5]');
        $this->set_field_help('sip_force_expires', 'If specified, it is the number of seconds to force registration expiration to. Minimum value is 5.');
        //Add dialplans drop downs
        $routing_module = $this->_the_app->find_module_by_name('routing');
        $dialplans = $routing_module->object_enum('dialplans', ADD_NONE_ALWAYS, true, '( SIP Profile Default )');
        $this->add_enum_field('call-routing', 'Routing Plan', 'dropdown', '__none__', $dialplans);
        $this->set_field_help('call-routing', 'Use a specific routing plan for this domain user.');

        $acl_list = $this->_the_app->find_module_by_name('acl');
        $acls = array();
        if ($acl_list) {
            $acls = $acl_list->object_enum('network_list', ADD_NONE_ALWAYS, true, '( none )');
        }
        $this->add_enum_field('cidr', 'ACL', 'dropdown', '__none__', $acls);
        $this->set_field_help('cidr', 'List of CIDR defined in ACL module from which this user can access NSC. ');
        return parent::configure();
    }
    public function validate($data, &$output = null)
    {
        $need_encrypt = true;
        if(isset($data['password']) && $data['password'] != ''){
            //check if password is encrypt
            if(safe_decrypt_password($data['password']) !== false){
                $this->set_field_rules('password', "");
                $need_encrypt = false;
            }
        }else{
            //no password or password is empty
            $need_encrypt = false;
        }
        $rs = parent::validate($data, $output);
        if($rs){
            if($need_encrypt){
                $new_data['password'] = safe_encrypt_password($data['password']);
                $this->set_data_values($new_data);
            } 
        }
        return $rs;
    }
}
?>
