<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * NSC Media Monitor Service Class
 * @author William Adam
 */
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_service_class');
safe_require_class('safe_db_class');
safe_module_require_class('nsc', 'product');
safe_module_require_class('nsc', 'application');
safe_module_require_class('fs', 'module');
safe_module_require_class('nsc', 'mediamon_config');
safe_module_require_class('nsc', 'sip_profile');

class Nsc_mediamon_service_class extends Safe_service_class
{
    private $_rules;
    function __construct($node, $software)
    {
        parent::__construct($software, 'mediamon');
        $this->_node = $node;
        // Override run_dir for ipd file
        $this->_run_dir = '/usr/local/nsc/run';
    }
    //Information pulled for the object
    public function configure()
    {
        // Create the object
        $cfg = new Nsc_mediamon_config_class($this->_node, $this->object_name(), $this->name());
        // Create the fields
        $cfg->configure();
        // Synchronize with persistent DB (safe_object table)
        $cfg->synch();
        // Register config to service
        $this->set_config($cfg);
        // Set service description
        $this->set_description('Media Firewall');
        return parent::configure();
    }
    /**
     * @brief indicate if user can control service start/stop...
     *         Overloaded from Safe_service_class
     *         
     * @return false
     */
    public function allow_user_ctl()
    {
        // User not allowed to control service
        return false;
    }
    public function can_restore($info, &$reason)
    {
        return true;
    }
    function db()
    {
        $tmp_db_cfg = $this->config()->get_data_values_exploded();
        // Filter out db-cfg
        $cfg_db = $tmp_db_cfg['db-cfg'];
        //will return the array needed to create the db connection (key=>value)
        $db = Safe_database_class::factory($cfg_db);
        return $db;
    }

    private function _write_ttl_hop_conf(&$config_manager,$ip_ttl_hop_conf=array(),$sofia_ttl_file_path="autoload_configs/sofia_ip_ttl_hop.xml"){
        $config_manager->add_config(
                new Safe_configuration_class(
                        $sofia_ttl_file_path, '',
                        Safe_configuration_class::CFG_DELETE,
                        Safe_configuration_class::CFG_FILE));
        
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startElement('include');
        foreach($ip_ttl_hop_conf as $conf){
            $xml_writer->startElement('param');
            $xml_writer->writeAttribute('ip',$conf['ip']);
            $xml_writer->writeAttribute('ttl_hop',$conf['ttl_hop']);
            $xml_writer->endElement();//param
        }
        $xml_writer->endElement();//include
        
        $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
        $config_manager->add_config(new Safe_configuration_class($sofia_ttl_file_path, $xmlstr));
    }

    private function _write_source_routing_ttl(&$config_manager, $ip_ttl_hop_conf=array(), $source_routing_ttl_file_path='/etc/sysconfig/network-scripts/source-routing-ttl.cfg'){
        if($this->node()->os()->check_file_exists($source_routing_ttl_file_path)){
            $config_manager->add_config(new Safe_configuration_class($source_routing_ttl_file_path,'',Safe_configuration_class::CFG_DELETE,Safe_configuration_class::CFG_FILE));
        }
        $dsp_out = array();
        $ttl_out = array();
        foreach($ip_ttl_hop_conf as $conf){
            if(($ip_address = Sng_ip_address_class::model($this->_node)->findOne(array('address'=>$conf['ip']))) &&
               ($sip_profiles = Nsc_sip_profile_class::model($this->_node)->findOne(array('sip-ip'=>$ip_address->name())))){
                $ttl_out[] = $ip_address->get_interface_name()."\t".$conf['ip']."\t0x".$conf['ttl_hop'];
            }
        }
        if(!empty($ip_ttl_hop_conf)){
            $modules = Fs_transcoding_module_class::model($this->_node)->find();
            foreach($modules as $module){
                $module_ipv4 = $module->ip_address(false, Safe_ip_address_class::IP_V4);
                if($module_ipv4){
                    $dsp_out[] = $module->ifname()."\t".$module_ipv4;
                }
            }
        }
        $dsp_out_str = sprintf("DSP_IFACE=(\n".implode("\n",$dsp_out)."\n)\n\n");
        $ttl_out_str = sprintf("SIP_PROFILE=(\n".implode("\n",$ttl_out)."\n)\n");
        
        $config_manager->add_config(new Safe_configuration_class($source_routing_ttl_file_path,$dsp_out_str . $ttl_out_str,Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_FILE));
        $config_manager->add_config(new Safe_configuration_class($source_routing_ttl_file_path,'root',Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
    }
    /**
     * @brief Generate MEDIAMON configuration
     *           
     * @param[in out] $config_manager
     *           
     * @return true on success
     */
    public function generate_config(&$config_manager)
    {
        $sofia_ttl_file_path = 'autoload_configs/sofia_ip_ttl_hop.xml';
        $source_routing_ttl_file_path = '/etc/sysconfig/network-scripts/source-routing-ttl.cfg';
        //Generates service configuration files
        if (!$this->_generate_service_config($config_manager)) return false;
        //Generates database configuration
        // If we are in software mode the ttl_hop array is empty and if so, we
        // remove sofia_ip_ttl_hop.xml if file exists. If we work in hardware
        // mode sofial_ip_ttl_hop.xml should be filled  with ip_ttl_hop_conf var content.
        if(!($ip_ttl_hop_conf = $this->_generate_db_config($config_manager)) || empty($ip_ttl_hop_conf)){
            $config_manager->add_config(
                                        new Safe_configuration_class(
                                                                     $sofia_ttl_file_path, '',
                                                                     Safe_configuration_class::CFG_DELETE,
                                                                     Safe_configuration_class::CFG_FILE));
        }else{
            $this->_write_ttl_hop_conf($config_manager,$ip_ttl_hop_conf,$sofia_ttl_file_path);
        }
        $this->_write_source_routing_ttl($config_manager,$ip_ttl_hop_conf,$source_routing_ttl_file_path);
        // Invoke parent generate
        return parent::generate_config($config_manager);
    }

    public function _generate_service_config(&$config_manager)
    {
        // Get our configuration data (unresolved)
        $cfg = $this->config()->get_data_values_exploded(false);
        // Remove global config - Used by service control script (application)
        unset($cfg['global']);
        if (isset($cfg['__all__'])) unset($cfg['__all__']);
        // Create xml document
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startDocument('1.0', 'UTF-8');
        // Document container
        $xml_writer->startElement('document');
        $xml_writer->writeAttribute('type', 'mediamon/xml');
        // Process database configuration
        $xml_writer->startElement('database');
        // Create connection-sting
        // <param name="connection-string"
        // value="DRIVER={MySQL};DATABASE=nsc;server=localhost;user=root;pwd=sangoma;port=3306"/>
        $connection_string = 'DRIVER={' . $cfg['db-cfg']['dbdriver'] . '};' . 'DATABASE=' . $cfg['db-cfg']['database'] . ';' . 'server=' . $cfg['db-cfg']['hostname'] . ';' . 'user=' . $cfg['db-cfg']['username'] . ';' . 'pwd=' . $cfg['db-cfg']['password'] . ';' . 'port=' . $cfg['db-cfg']['port'] . ';';
        $xml_writer->startElement('param');
        $xml_writer->writeAttribute('name', 'connection-string');
        $xml_writer->writeAttribute('value', $connection_string);
        $xml_writer->fullEndElement();
        foreach ($cfg['database'] as $k => $v) {
            $xml_writer->startElement('param');
            $xml_writer->writeAttribute('name', $k);
            $xml_writer->writeAttribute('value', $v);
            $xml_writer->fullEndElement();
        }
        // Remove database config entries
        $xml_writer->fullEndElement();
        unset($cfg['database']);
        unset($cfg['db-cfg']);
        // Add all parameters
        foreach ($cfg as $category => $cat_cfg) {
            $xml_writer->startElement($category);
            foreach ($cat_cfg as $k => $v) {
                $xml_writer->startElement('param');
                $xml_writer->writeAttribute('name', $k);
                $xml_writer->writeAttribute('value', $v);
                $xml_writer->fullEndElement();
            }
            $xml_writer->fullEndElement();
        }
        // Close settings
        $xml_writer->fullEndElement();
        $xml_writer->endDocument();
        $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
        $file = $config_manager->directory('base') . '/etc/' . $this->name() . '.xml';
        $config_manager->add_config(new Safe_configuration_class($file, $xmlstr));
        return true;
    }

    public function _generate_db_config(&$config_manager)
    {
        // First clear all records from db
        $db = $this->db();
        $data['db_class'] = $this->db();
        //clear mediamon_media_map before clear media_modules
        $config_manager->add_config(new Safe_configuration_class('media_host_ipv4',$data,Safe_configuration_class::CFG_EMPTY,Safe_configuration_class::CFG_SQL));
        $config_manager->add_config(new Safe_configuration_class('media_host_ipv6',$data,Safe_configuration_class::CFG_EMPTY,Safe_configuration_class::CFG_SQL));
        $config_manager->add_config(new Safe_configuration_class('media_modules_ipv4',$data,Safe_configuration_class::CFG_EMPTY,Safe_configuration_class::CFG_SQL));
        $config_manager->add_config(new Safe_configuration_class('media_modules_ipv6',$data,Safe_configuration_class::CFG_EMPTY,Safe_configuration_class::CFG_SQL));
        $config_manager->add_config(new Safe_configuration_class('media_modules', $data, Safe_configuration_class::CFG_EMPTY, Safe_configuration_class::CFG_SQL));

        $sngms_module = $this->node()->software()->application()->find_module_by_name('sngms');
        $media_modules = $sngms_module->api_retrieve_module();
        // Get mediamon state
        $cfg = $this->config()->get_data_values_exploded(false);
        // Do we need to generate a new modules tables ?
        $ipv4_ttl_hop = 2;
        $ipv6_ttl_hop = 2;
        $ip_ttl_hop_conf = array();
        if ('true' == $cfg['global']['NSC_MEDIAMON_ENABLE']) {
            $rtp_ip_list = $this->get_network_ip_list();
            foreach($rtp_ip_list as $ip_type => $ips){
                foreach($ips as $ip){
                    $data['data'] = array('TTL_HOP'=>($ip_type==4? ++$ipv4_ttl_hop : ++$ipv6_ttl_hop),'ip'=>$ip);
                    $ip_ttl_hop_conf[] = array('ttl_hop'=>$data['data']['TTL_HOP'],'ip'=>$ip);
                    $config_manager->add_config(new Safe_configuration_class('media_host_ipv'.$ip_type,$data,Safe_configuration_class::CFG_CREATE,Safe_configuration_class::CFG_SQL));
                }
            }
        }
        foreach($media_modules as $module){
            $data['data'] = array('id'=>$module->id(),'access_interface'=>$module->ifname(),'port_range_start'=>$module->udp_begin(),'port_range_stop'=>$module->udp_end());
            $config_manager->add_config(new Safe_configuration_class('media_modules', $data,Safe_configuration_class::CFG_CREATE,Safe_configuration_class::CFG_SQL));
            $data['data'] = array('media_module_id'=>$module->id(),'ip'=>$module->ip_address(false));
            $config_manager->add_config(new Safe_configuration_class('media_modules_ipv4',$data,Safe_configuration_class::CFG_CREATE,Safe_configuration_class::CFG_SQL));
            $data['data']['ip'] = $module->ip_address(false,Safe_ip_address_class::IP_V6);
            $config_manager->add_config(new Safe_configuration_class('media_modules_ipv6',$data,Safe_configuration_class::CFG_CREATE,Safe_configuration_class::CFG_SQL));
        }
        return $ip_ttl_hop_conf;
    }
    
    private function get_network_ip_list(){
        $ip_addresses = array();
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $interface_objs = $network_module->api_retrieve_interface();
        foreach($interface_objs as $interface_name  => $interface_obj){
            //pass by lo and sngsdp
            $real_interface_name = $interface_obj->interface_name();
            if($interface_obj->interface_type() == Sng_network_interface_class::LOOPBACK || $interface_obj->hardware_type() == Sng_network_interface_class::TRANSCODING){
                continue;
            }
            $tmp_ip_obj_list = $network_module->get_signaling_ips($interface_name);
            if($tmp_ip_obj_list){
                foreach($tmp_ip_obj_list as $ip_name => $ip_obj){
                    $ip_address = $ip_obj->address();
                    if(!empty($ip_address)){
                        if($ip_obj->ip_type() == Safe_ip_address_class::IP_V6){
                            $ip_addresses[Safe_ip_address_class::IP_V6][] = $ip_obj->address();
                        }else{
                            $ip_addresses[Safe_ip_address_class::IP_V4][] = $ip_obj->address();
                        }
                    }
                }
            }
        }
        return $ip_addresses;
    }


    public function get_global_variables()
    {
        $vars = parent::get_global_variables();
        $cfg = $this->config()->get_data_values_exploded(false);
        // Get mediamon state
        $vars['single_ip_media'] = $cfg['global']['NSC_MEDIAMON_ENABLE'];
        return $vars;
    }
    public function category()
    {
        return 'Security';
    }
    /**
     * @brief
     *         
     * @return
     */
    public function get_data_settings()
    {
        $cfg_manager = $this->node()->configuration_manager();
        $data = array(
            'log' => array(
                'description' => 'Log file',
                'filesystem' => true,
                'pattern' => '/var/log/' . $this->name() . '.log*',
                'global_capabilities' => array(
                    'download'
                ) ,
                'capabilities' => array(
                    'download',
                    'view'
                ) ,
                'methods' => array(
                        'download' => array(
                                'name' => 'Download',
                                'description' => array( 'Download log files.' ),
                                'request' => 'GET',
                                'icon' => 'download',
                                'button' => true,
                                'action_url' => '/SAFe/sng_data_manager/download/mediamon/log'
                        )
                )
            )
        );
        return $data;
    }
    
    public function checklist(&$list){
        parent::checklist($list);
    }

    public function api_download_log($name=null, $data = null, &$output = null){
        $this->download_data('log', $name);
        exit();
    }
}
