<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * NSC SIP module
 * @author: William Adam
 *          
 */
require_once ('application/helpers/safe_helper.php');
safe_module_require_class('fs', 'sip_module');
safe_module_require_class('nsc', 'sip_profile');
safe_module_require_class('nsc', 'sip_trunk');

class Nsc_sip_module_class extends Fs_sip_module_class
{
    /**
     * @brief Create a SIP profile (overload from base class)
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_create_profile($name, $data=null) {
        $profile = new Nsc_sip_profile_class($this->node() , $this->_obj_base_path . '/profile', $name);
        $profile->configure();
        if($data) {
            $profile->set_data_values($data);
        }
        return $profile;
    }
    /**
     * @brief Create a SIP trunk (overload from base class)
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return 
     */
    public function api_create_trunk($name, $data=null, &$output = null) {
        if($this->can_create_aggregate_object('trunk', $name , $output)){
            $trunk = new Nsc_sip_trunk_class($this->node() , $this->_obj_base_path . '/trunk', $name);
            $trunk->configure();
            if($data) {
                $trunk->set_data_values($data);
            }
            return $trunk;
        }else{
            return false;
        }
        
    }
    public function check_object_usage($src_module_name, $obj_type, $obj_name, $sub_type = null, $sub_name = null)
    {
        $usages = array();
        $profiles = $this->profiles();
        foreach($profiles as $profile){
            if ($src_module_name == 'directory' && $obj_type == 'domain' ) {
                $domain_check = $profile->domain_check($obj_name);
                if ($domain_check) {
                    $usages[] = array (
                        'name' => $this->name(),
                        'obj_type' => 'profile',
                        'obj_name' => $profile->name()
                    );
                }
            }
            // checks to see if the SIP profile is using the ACL for either apply-inbound-acl or apply-register-acl
            if ($src_module_name == 'acl' && $obj_type == 'network_list' ) {
                $sip_profile = $profile->acl_check($obj_name);
                if ($sip_profile) {
                    $usages[] = array (
                        'name' => $this->name(),
                        'obj_type' => 'profile',
                        'obj_name' => $profile->name()
                    );
                }
            }
            if ($src_module_name == 'media' && $obj_type == 'profile' ) {
                $media_check = $profile->media_check($obj_name);
                if ($media_check) {
                    $usages[] = array (
                        'name' => $this->name(),
                        'obj_type' => 'profile',
                        'obj_name' => $profile->name()
                    );
                }
            }
        }
        $trunks = $this->trunks();
        foreach($trunks as $trunk){
            if ($src_module_name == 'media' && $obj_type == 'profile' ) {
                $media_check_trunk = $trunk->media_check($obj_name);
                if ($media_check_trunk) {
                    $usages[] = array (
                        'name' => $this->name(),
                        'obj_type' => 'trunk',
                        'obj_name' => $trunk->name()
                    );
                }
            }
            if ($src_module_name == 'sip' && $obj_type == 'profile' ) {
                $sip_profile_check = $trunk->sip_profile_check($obj_name);
                if ($sip_profile_check) {
                    $usages[] = array (
                        'name' => $this->name(),
                        'obj_type' => 'trunk',
                        'obj_name' => $trunk->name()
                    );
                }
            }
        }
        return array_merge($usages , parent::check_object_usage($src_module_name, $obj_type, $obj_name, $sub_type, $sub_name));
    }
}
?>
