<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * NSC SIP Security Monitor Service Class
 * @author Shaunt Libarian
 */
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_service_class');
safe_require_class('safe_db_class');
safe_module_require_class('nsc', 'product');
safe_module_require_class('nsc', 'application');
safe_module_require_class('nsc', 'sipsecmon_config');
safe_module_require_class('nsc', 'sipsecmon_rules_config');

class Nsc_sipsecmon_db_config_class extends Safe_configuration_user_class
{
    private $_args;
    public function __construct($service_name, $process_name, $args)
    {
        $this->_service_name = $service_name;
        $this->_process_name = $process_name;
        $this->_args = $args;
        parent::__construct('Sipsecmon Database Config');
    }
    public function run($node)
    {
        $result = null;
        $node->service_ctl($this->_service_name, $this->_args, $result, $this->_process_name);
        return true;
    }
}

class Nsc_sipsecmon_service_class extends Safe_service_class
{
    private $_rules;
    function __construct($node, $software)
    {
        parent::__construct($software, 'sipsecmon');
        $this->_node = $node;
        // Override run_dir for ipd file
        $this->_run_dir = '/usr/local/nsc/run';
    }
    //Information pulled for the object
    public function configure()
    {
        // Create the object
        $cfg = new Nsc_sipsecmon_config_class($this->_node, $this->object_name(), $this->name());
        // Create the fields
        $cfg->configure();
        // Synchronize with persistent DB (safe_object table)
        $cfg->synch();
        // Register config to service
        $this->set_config($cfg);
        // Set service description
        $this->set_description('SIP Security Monitor');
        // Register rule objects
        $this->register_aggregate_object('rules',
           array(
                'name' => 'Rules',
                'dynamic' => true,
                'base_path' => parent::object_name().'/rules',
                'methods' => array(
                           'create' => array(
                                   'name' => 'Create',
                                   'description' => 'Create a rule',
                                   'request' => 'POST',
                           ),
                           'retrieve' => array(
                                   'name' => 'Retrieve',
                                   'description' => 'Retrieve a rule',
                                   'request' => 'GET',
                           ),
                           'update' => array(
                                   'name' => 'Update',
                                   'description' => 'Update a rule',
                                   'request' => 'POST',
                           ),
                           'delete' => array(
                                   'name' => 'Delete',
                                   'description' => 'Delete a rule',
                                   'request' => 'POST',
                           ),
                   ),
            )   
        );  
        return parent::configure();
    }
    function db_configuration($tmp_cfg)
    {
        $cfg_db = array();
        $cluster_module = $this->_node->software()->application()->find_module_by_name('cluster');

        if ($cluster_module != NULL && $cluster_module->get_cluster_mode() !== 'disabled') {
            safe_log(SAFE_LOG_INFO, 'selected cluster database for sipsecmon');
            $cfg_db = $tmp_cfg['cluster-db-cfg'];
        } else {
            safe_log(SAFE_LOG_INFO, 'selected default database for sipsecmon');
            $cfg_db = $tmp_cfg['db-cfg'];
        }

        return $cfg_db;
    }

    function db()
    {
        $tmp_db_cfg = $this->config()->get_data_values_exploded();
        $cfg_db = $this->db_configuration($tmp_db_cfg);

        //will return the array needed to create the db connection (key=>value)
        $db = Safe_database_class::factory($cfg_db);
        return $db;
    }
    /**
     * SIPSecmon unblock_ip Command
     */
    function unblock_ip($argument, &$output = null)
    {
        $cmd = "unblock " . $argument;
        // Invoke service execute
        $rc = $this->execute($cmd, $output);
        return ($rc == 0 ? TRUE : FALSE);
    }
    /**
     * SIPSecmon reload Command
     */
    function reload_setup(&$config_manager)
    {
        //$cmd = "reload";
        // Invoke service execute
        ////$rc = $this->execute($cmd, $output);
        ////return ($rc == 0 ? TRUE : FALSE);
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $mariadbgalera = $this->node()->software()->application()->find_module_by_name('mariadbgalera');
        if ($cluster_module != NULL && $cluster_module->get_cluster_mode() !== 'disabled' && $mariadbgalera->status()!==Safe_service_class::STATUS_RUNNING){ 
            $clustermon_service = $this->node()->software()->application()->find_module_by_name('clustermon');
            $clustermon_service->need_force_start(true);
        }
        $config_manager->add_config(new Nsc_sipsecmon_db_config_class($this->name(), $this->_process_name, "reload"));
        return;
    }
    /**
     * Returns sip secmon rules in an array
     */
    public function rules()
    {
        $rules = Safe_object_serializer_class::get_serializer()->find_objects(parent::object_name() . '/rules');
        foreach ($rules as $k => $v) {
            $v = new Nsc_sipsecmon_rules_config_class($this->node() , parent::object_name() . '/rules', $k);
            $v->configure();
            $v->synch();
            $this->_rules[$k] = $v;
        }
        return $this->_rules;
    }
    /**
     * Function which returns whether the function is enabled or not
     * @return boolean
     */
    public function is_enabled($mod_name = "")
    {
        return $this->config()->enabled();
    }
    /**
     * @brief Generate SIPSECMON configuration
     *           
     * @param[in out] $config_manager
     *           
     * @return true on success
     */
    public function generate_config(&$config_manager)
    {
        //Generates service configuration files
        if (!$this->_generate_service_config($config_manager)) return false;
        // make sure database tables exists
        $config_manager->add_config(new Nsc_sipsecmon_db_config_class($this->name(), $this->_process_name, "initdb"));
        //Generates database configuration
        if (!$this->_generate_db_config($config_manager)) return false;
        // Remove deleted objects
        if (!$this->_handle_deleted_objects($config_manager)) return false;
        return parent::generate_config($config_manager);
    }
    public function _generate_service_config(&$config_manager)
    {
        // Get our configuration data (unresolved)
        $cfg = $this->config()->get_data_values_exploded(false);
        // Remove global config - Used by service control script (application)
        unset($cfg['global']);
        // Create xml document
        $xml_writer = new XMLWriter();
        $xml_writer->openMemory();
        $xml_writer->startDocument('1.0', 'UTF-8');
        // Document container
        $xml_writer->startElement('document');
        $xml_writer->writeAttribute('type', 'sipsecmon/xml');
        // Process database configuration
        $xml_writer->startElement('database');
        // Create connection-sting
        // <param name="connection-string"
        // value="DRIVER={MySQL};DATABASE=nsc;server=localhost;user=root;pwd=sangoma;port=3306"/>
        $cfg_db = $this->db_configuration($cfg);
        $connection_string = 'DRIVER={' . $cfg_db['dbdriver'] . '};DATABASE=' . $cfg_db['database'] . ';server=' . $cfg_db['hostname'] .
            ';socket=' . $cfg_db['socket'] . ';user=' . $cfg_db['username'] . ';pwd=' . $cfg_db['password'] . ';port=' . $cfg_db['port'] . ';';
        $xml_writer->startElement('param');
        $xml_writer->writeAttribute('name', 'connection-string');
        $xml_writer->writeAttribute('value', $connection_string);
        $xml_writer->endElement();
        foreach ($cfg['database'] as $k => $v) {
            $xml_writer->startElement('param');
            $xml_writer->writeAttribute('name', $k);
            $xml_writer->writeAttribute('value', $v);
            $xml_writer->endElement();
        }
        // Remove database config entries
        $xml_writer->endElement();
        unset($cfg['database']);
        unset($cfg['db-cfg']);
        unset($cfg['cluster-db-cfg']);
        // Add all parameters
        foreach ($cfg as $category => $cat_cfg) {
            $xml_writer->startElement($category);
            foreach ($cat_cfg as $k => $v) {
                $xml_writer->startElement('param');
                $xml_writer->writeAttribute('name', $k);
                $xml_writer->writeAttribute('value', $v);
                $xml_writer->endElement();
            }
            $xml_writer->endElement();
        }
        // Close settings
        $xml_writer->endElement();
        $xml_writer->endDocument();
        $xmlstr = Safe_format_xml($xml_writer->outputMemory(true));
        $file = $config_manager->directory('base') . '/etc/' . $this->name() . '.xml';
        $config_manager->add_config(new Safe_configuration_class($file, $xmlstr));
        return true;
    }


    public function initialize_db_tables(&$config_manager) {
        //Generates service configuration files
        if(!$this->_generate_service_config($config_manager)) return false;
        // make sure database tables exists
        $config_manager->add_config(new Nsc_sipsecmon_db_config_class($this->name(), $this->_process_name, "initdb"));
        return true;
    }

    public function _generate_profile_security_db_config($profile, &$config_manager)
    {

        if (!$this->is_enabled()) {
            return;
        }

        $sip_profile_cfg = $profile->get_data_values_exploded(false);
        $ip_if = $sip_profile_cfg['__all__']['sip-ip'];
        $ip = $profile->get_ip_address($ip_if);
        $profile->get_sip_transport_info($trans, $port);

        // first make sure this profile is listed within the list of profiles
        //  that require strick security
        $data = array();
        $db = $this->db();
        $data['db_class'] = $db;
        $data['data'] = array();
        $data['data']['name'] = $profile->display_name();
        $data['data']['strict_security_enabled'] = ($profile->is_strict_security_enabled() && $profile->status() != Safe_object_serializer_class::OBJ_STATUS_DELETED) ? true : false;
        $data['data']['ip'] = $ip;
        $data['data']['port'] = $port;
        $profile_acls = array();

        if ( $data['data']['strict_security_enabled'] == true) {
            $acl_module = $this->node()->software()->application()->find_module_by_name('acl');
            $acls = $acl_module->api_retrieve_network_list();

            foreach($acls as $acl) {

                if (!$profile->acl_check_inbound($acl->name())) {
                    continue;
                }

                $conditions_def = $acl->get_aggregate_object_definition();
                foreach ($conditions_def as $condition_type => $condition_def) {
                    $conditions = $acl->get_aggregate_objects($condition_type);

                    foreach ($conditions as $condition) {
                        $cond = $condition->get_data_values(false);
                        if (!$cond['policy'] || $cond['policy'] != 'allow') {
                            continue;
                        }
                        $def_prefix = filter_var($cond['ip-address'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) ? '32' : '128';
                        $pfx = $cond['prefix'] ? $cond['prefix'] : $def_prefix;
                        //$profile_acls .= "," . $cond['ip-address'] .  "/" . $pfx;
                        $profile_acls [] = $cond['ip-address'] .  "/" . $pfx;
                    }
                }
            }
        }
        $data['data']['acl'] = implode(",", $profile_acls);
        $config_manager->add_config(new Safe_configuration_class('sip_profile_new_state', $data, Safe_configuration_class::CFG_CREATE,Safe_configuration_class::CFG_SQL));
    }

    public function _generate_db_config(&$config_manager)
    {
        $sip_secmon_rules = $this->rules();
        if (isset($sip_secmon_rules)) {
            foreach ($sip_secmon_rules as $rules) {
                //build data array
                unset($data);
                $db = $this->db();
                $data['db_class'] = $db;
                $data['data'] = $rules->get_data_values(false);
                $data['data']['name'] = $rules->name();
                // process action_expr and action param
                if ('false' == $data['data']['action_expr']) $data['data']['action_expr'] = "";
                else {
                    $data['data']['action_expr'].= "=" . $data['data']['action_param'];
                }
                unset($data['data']['action_param']);
                if ($data['data']['profile_filter'] == '__none__') {
                    $data['data']['profile_filter'] = "";
                } else {
                    $_obj = $this->node()->software()->application()->find_obj_by_name('sip', 'profile', $data['data']['profile_filter']);
                    if($_obj)
                        $data['data']['profile_filter'] = $_obj->display_name();
                }
                //Determine whether to update or insert a rule
                $status = $rules->status();
                if ($status == Safe_object_serializer_class::OBJ_STATUS_MODIFIED) {
                    $data['primary_key'] = array(
                        'name'
                    );
                    $config_manager->add_config(new Safe_configuration_class('rules', $data, Safe_configuration_class::CFG_UPDATE, Safe_configuration_class::CFG_SQL));
                } elseif ($status == Safe_object_serializer_class::OBJ_STATUS_NEW) {
                    $config_manager->add_config(new Safe_configuration_class('rules', $data, Safe_configuration_class::CFG_CREATE, Safe_configuration_class::CFG_SQL));
                }
            }
        }
        return true;
    }
    private function _handle_deleted_objects(&$config_manager)
    {
        //Handle deleted SIPsecmon rules
        $deleted_sip_secmon_rules = Safe_object_serializer_class::get_serializer()->find_deleted_objects(parent::object_name() . '/rules');
        foreach ($deleted_sip_secmon_rules as $rules => $value) {
            //build data array
            unset($data);
            $data['db_class'] = $this->db();
            $value['data']['name'] = $rules;
            $data['data'] = $value['data'];
            $data['primary_key'] = array(
                'name'
            );
            $config_manager->add_config(new Safe_configuration_class('rules', $data, Safe_configuration_class::CFG_DELETE, Safe_configuration_class::CFG_SQL));
        }
        return true;
    }
    public function category()
    {
        return 'Security';
    }
    /**
     * @brief
     *         
     * @return
     */
    public function get_data_settings()
    {
        $cfg_manager = $this->node()->configuration_manager();
        $data = array(
            'log' => array(
                'description' => 'Log file',
                'filesystem' => TRUE,
                'pattern' => '/var/log/' . $this->name() . '.log*',
                'global_capabilities' => array(
                    'download'
                ) ,
                'capabilities' => array(
                    'download',
                    'view'
                ) ,
                'methods' => array(
                        'download' => array(
                                'name' => 'Download',
                                'description' => array( 'Download log files.' ),
                                'request' => 'GET',
                                'icon' => 'download',
                                'button' => true,
                                'action_url' => '/SAFe/sng_data_manager/download/sipsecmon/log'
                        )
                )
            )
        );
        return $data;
    }
    public function support_reload(&$need_reload = null) {
        parent::support_reload($need_reload);
        return true;
    }   
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {   
        if(!$this->generate_config($config_manager)) return false;
        return parent::reload_generate_config($config_manager, $obj_type);
    }   
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }
    public function post_write_config()
    {
        // Is service enabled ?
        $enabled = $this->is_enabled();
        // Check current status
        switch($this->status()){
        case Safe_service_class::STATUS_RUNNING:
            if($enabled){
                $this->restart();
            }else{
                $this->stop();
            }
            break;
        case Safe_service_class::STATUS_STOPPED:
            if($enabled){
                // This service depends on application being started, so check 
                // if application is running
                if($this->node()->software()->application()->status() == STATUS_STOPPED){
                    $this->start();
                }
            }
            break;
        }
        return parent::post_write_config();
    }
    
    /*
    public function rules()
    {
        return $this->get_aggregate_objects('rule');
    }*/
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_create_rules($name, $data=null) {
        $obj = new Nsc_sipsecmon_rules_config_class($this->_node, parent::object_name() . '/rules', $name);
        $obj->configure();
        if($data) {
            $obj->set_data_values($data);
        }
        return $obj;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_retrieve_rules($name, $data=null) {
        $rules = $this->rules();
        return $rules[$name];
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_update_rules($name, $data=null, &$output = null) {
        $rule = $this->api_retrieve_rules($name);
        if($rule) {
            if ($rule->validate($data,$output)) {
                if (true == $rule->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_delete_rules($name, $data=null, &$output = null) {
        $rules = $this->rules();
        if($rules[$name]) {
            if($rules[$name]->can_dispose($output)) {
                return $rules[$name]->dispose();
            }
        }
        return false;
    }
    public function api_download_log($name=null, $data = null, &$output = null){
        $this->download_data('log', $name);
        exit();
    } 
    public function api_reload_rules_action($name=null,&$output=null){
        $db = $this->db();
        if(!$db->connect()){
            $output['message'] = array("Sorry. Rules reload failed!");
            $output['reason'] = "Unable to connect database";
            return false;
        }
        $db->query('TRUNCATE TABLE sip_secmon_rules;');
        $db->disconnect();
        $config_manager = $this->node()->configuration_manager();
        $sip_secmon_rules = $this->rules();
        if (isset($config_manager) && (isset($sip_secmon_rules))) {
             foreach ($sip_secmon_rules as $rules) {
                 unset($data);
                 $data['db_class'] = $db;
                 $data['data'] = $rules->get_data_values(false);
                 $data['data']['name'] = $rules->name();
                  if ('false' == $data['data']['action_expr'])
                      $data['data']['action_expr'] = "";
                  else {
                        $data['data']['action_expr'].= "=" . $data['data']['action_param'];
                  }
                  unset($data['data']['action_param']);
                  if ($data['data']['profile_filter'] == '__none__') {
                      $data['data']['profile_filter'] = "";
                  } else {
                      $_obj =
                          $this->node()->software()->application()->find_obj_by_name('sip',
                                  'profile', $data['data']['profile_filter']);
                      if($_obj){
                           $data['data']['profile_filter'] =
                               $_obj->display_name();
                      }
                  }
                  $config_manager->add_config(new
                          Safe_configuration_class('rules', $data,
                              Safe_configuration_class::CFG_CREATE,Safe_configuration_class::CFG_SQL));
             }
            if($config_manager->write_config($this->node())) {
                // clear modified flags
                $this->clear_configuration_modified(true);
            }
            return true;
        }else{
            $output['message'] = array("Sorry. Rules reload failed!");
            return false;
        }
    }
    public function can_restore($info, &$reason)
    {
        return true;
    }

    public function depends_on(){
        $cluster_module = $this->_node->software()->application()->find_module_by_name('cluster');
        $mariadbgalera = $this->_node->software()->application()->find_module_by_name('mariadbgalera');
        $cluster_service = $this->_node->software()->application()->find_module_by_name('clustermon');
        if ($cluster_module != NULL && $cluster_module->get_cluster_mode() !== 'disabled' && $mariadbgalera->status()!==Safe_service_class::STATUS_RUNNING && $cluster_service->need_force_start()) {
            return array(array('mariadbgalera',true),array('nsc',true));
        }else{
            return array();
        }
    }

    public function get_action_settings($action=null){
        $action_def = array();
        $action_def['reload_rules'] = array(
                    'name' => 'Reload Rules',
                    'icon' => 'repeat',
                    'description' => 'Reload Rules',
                    'request' => 'POST',
                    'confirm' => true,
                    'reload' => true
                );
        return $action_def;
     }
}
