<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * Safe global application settings class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');
class Safe_application_settings_class extends Safe_configurable_object_class
{
    /**
     * @brief Constructor
     *           
     * @param[in out] $parent_name
     * @param[in out] $name
     *           
     * @return
     */
    public function __construct($parent_name,$node=null)
    {
        parent::__construct($parent_name , "configuration",$node);
    }
    /**
     * @brief Configure the config class object
     *         
     * @return
     */
    public function configure($node=null)
    {
        /*
         * Editor key binding mode
        */
        $editor_mode = array(
            'standard' => 'Standard',
            'vim' => 'Vi',
            'emacs' => 'Emacs'
        );
        $this->add_enum_field('keyboard_mode', 'Editor Key Bindings Mode', 'dropdown', 'standard', $editor_mode);
        $this->set_field_help('keyboard_mode', array(
            'Keyboard layout for text editor component.',
            'Select Standard if unsure.'
        ));

        $this->add_field('web_session_timeout', 'Web Session Timeout', 'text', '180', 8);
        //session timeout must less than 35792, otherwise in javacript, it will reach the 32 int number limit
        $this->set_field_rules('web_session_timeout', 'required|is_natural|greater_than[0]|less_than[35792]');
        $this->set_field_help('web_session_timeout','Time in minutes for web session to expire.');
        // Ftp configuration fields for update checker
                
        $conf_manager = $this->node()->configuration_manager();
        $up_package_file = $conf_manager->get_update_config('package');
        // If update package configuration is available show ftp config form
        if($up_package_file && ($ftp_config = parse_ini_file($up_package_file))){            

            $this->set_field_category('keyboard_mode','General');
            $this->set_field_category('web_session_timeot','General');
            
            // To ensure fields are setted
            $ftp_config = array_merge(array('ftp_server'=>null,'ftp_base_dir'=>null,'ftp_username'=>null,'ftp_password'=>null,'ftp_port'=>null,'connection_timeout'=>null),$ftp_config);
            $this->add_field('ftp_server','Server','text',$ftp_config['ftp_server'],32);
            $this->set_field_help('ftp_server','Server and base directory to check for new updates.');
            $this->set_field_rules('ftp_server','required');
            $this->set_field_category('ftp_server','Ftp Configuration');
            
            $this->add_field('ftp_base_dir','/','text',$ftp_config['ftp_base_dir'],16);
            $this->set_field_rules('ftp_base_dir','required');
            $this->composite_layout('ftp_server',array('ftp_base_dir'));

            $this->add_field('ftp_port','Port','number',$ftp_config['ftp_port'],4);
            $this->set_field_rules('ftp_port','required');
            $this->set_field_help('ftp_port','A valid ftp port connection on server.');
            $this->set_field_category('ftp_port','Ftp Configuration');            
            
            $this->add_field('ftp_username','Username','text',$ftp_config['ftp_username'],16);
            $this->set_field_help('ftp_username','A valid ftp account username.');
            $this->set_field_rules('ftp_username','required');
            $this->set_field_category('ftp_username','Ftp Configuration');
            
            $this->add_field('ftp_password','Password','password',$ftp_config['ftp_password'],16);
            $this->set_field_help('ftp_password','A valid ftp account password.');
            $this->set_field_category('ftp_password','Ftp Configuration');

            $this->add_field('connection_timeout','Timeout','number',$ftp_config['connection_timeout'],2);
            $this->set_field_rules('connection_timeout','required');
            $this->set_field_help('connection_timeout','Ftp connection timeout. Too high value may user interface');
            $this->set_field_category('connection_timeout','Ftp Configuration');

            }
        return parent::configure();
    }
    /**
     * @brief Returns a config description string
     *         
     * @return string
     */
    public function description()
    {
        return parent::description();
    }
    public function serialize()
    {
        $res = parent::serialize();
        $this->clear_modified();
        return $res;
    }
}
/* End of file safe_application_settings_class.php */
