<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * Configuration class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_object_class');
safe_require_class('safe_db_class');
class Safe_configuration_class
{
    private $_file_name;
    private $_data;
    // Action enumeration
    const CFG_CREATE = 'create';
    const CFG_UPDATE = 'update';
    const CFG_DELETE = 'delete';
    const CFG_EMPTY = 'empty';
    const CFG_LINK = 'link';
    const CFG_UNLINK = 'unlink';
    const CFG_NONE = 'none';
    const CFG_LOAD = 'load';
    private $_action;
    // Configuration type enumeration
    const CFG_FILE = 'file';
    const CFG_DIR = 'dir';
    const CFG_SQL = 'sql';
    const CFG_USER = 'user';
    const CFG_SED = 'sed';
    const CFG_CHOWN = 'chown';
    const CFG_CHMOD = 'chmod';
    
    private $_type;
    public function __construct($file_name, $data, $action = Safe_configuration_class::CFG_UPDATE, $type = Safe_configuration_class::CFG_FILE)
    {
        $this->_file_name = $file_name;
        $this->_data = $data;
        $this->_action = $action;
        $this->_type = $type;
    }
    public function file_name()
    {
        return $this->_file_name;
    }
    public function data()
    {
        return $this->_data;
    }
    public function action()
    {
        return $this->_action;
    }
    public function type()
    {
        return $this->_type;
    }
}
class Safe_configuration_user_class extends Safe_configuration_class
{
    public function __construct($file_name = '', $action_name = Safe_configuration_class::CFG_NONE, $data = array())
    {
        parent::__construct($file_name, $data, $action_name, Safe_configuration_class::CFG_USER);
    }
    public function run($node)
    {
        return false;
    }
}
class Safe_configuration_manager_class extends Safe_object_class
{
    private $_node = null;
    private $_directories;
    private $_configs = array();
    private $_services = array();
    private $_audits = array();
    private $_license = array();
    private $_about = array();
    private $_update = array();
    private $_url = array();
    private $_errors = array();
    private $_progress = array();
    private $_archive_manager = null;
    private $_forget_error = false;
    public function __construct($parent, $prod_def)
    {
        parent::__construct($parent->object_name() , "configuration_manager");
        if (!is_subclass_of($parent, 'Safe_node')) throw new Exception('Parent object MUST be a node objet.');
        $this->_node = $parent;
        // Latch known existing stuffs
        if (isset($prod_def['directories'])) $this->_directories = $prod_def['directories'];
        // Check services
        if (isset($prod_def['services'])) {
            $services = $prod_def['services'];
            
            // Section exists check if some services defined
            if (isset($services['service'])) {
                $services['service'] = to_index_array($services['service']);
            }
            $this->_services = $services['service'];
        }
        // Process audit configuration if any
        if (isset($prod_def['audit'])) {
            $_audits = array();
            $audits = $prod_def['audit']['modules'];
            // Section exists check if some modules defined
            if (isset($audits['module'])) {
                $audits['module'] = to_index_array($audits['module']);
                // Arrange to array
                foreach($audits['module'] as $mod){
                    $general = $mod['@attributes'];
                    $name = $general['name'];
                    unset($general['name']);
                    if($name){
                        $points = to_index_array($mod['point']);
                        foreach($points as $point){
                            $point_def = array();
                            $point_name = $point['@attributes']['name'];
                            if(!$point_name){
                                $point_name = $point['@attributes']['type'];
                            }
                            //unset($point['@attributes']['name']);
                            foreach($point['@attributes'] as $k => $v){
                                $point_def[$k] = $v;
                            }
                            // loop on param
                            foreach(array('arg', 'param') as $t){
                                if($point[$t]){
                                    $param = to_index_array($point[$t]);
                                    foreach($param as $p){
                                        $pname = $p['@attributes']['name'];
                                        unset($p['@attributes']['name']);
                                        if($pname){
                                            foreach($p['@attributes'] as $k => $v){
                                                $point_def[$t][$pname][$k] = $v;
                                            }
                                        }
                                    }
                                }
                            }
                            if($point['help']){
                                $point_def['help'] = $point['help'];
                            }
                            if($point_def){
                                $_audits[$name][$point_name] = $point_def;
                            }
                        }
                    }
                }
                $this->_audits = $_audits;
            }
        }
        // check audit parameters
        // Make sure configuration,log directories exist and webconfig
        // can play with it
        if (isset($this->_directories['configuration'])) $this->_node->mkdir($this->_directories['configuration']);
        if (isset($this->_directories['log']))
            $this->_node->mkdir($this->_directories['log'] , false);
        if (isset($this->_directories['core-dump']))
            $this->_node->mkdir($this->_directories['core-dump'] , false);
        if (isset($this->_directories['data']))
            $this->_node->mkdir($this->_directories['data'] , false);
        // License
        $this->_license['file'] = $prod_def['license'];
        $this->_license['pubkey'] = $prod_def['pubkey'];
        // About
        $this->_about['version'] = $prod_def['about'];
        // Update
        if (isset($prod_def['update'])) $this->_update = $prod_def['update'];
        // URL
        if (isset($prod_def['url'])) $this->_url = $prod_def['url'];
    }
    public function forget_error($forget)
    {
        $this->_forget_error = $forget;
    }
    public function add_config($config)
    {
        $this->_configs[] = $config;
    }
    public function add_error($error = array())
    {
        if (!$this->_forget_error) {
            $this->_errors[] = $error;
            $error['type'] = 'ERROR';
            $this->_progress[] = $error;
        }
    }
    public function reset_errors()
    {
        $this->_errors = array();
    }
    public function errors()
    {
        if (isset($this->_errors)) return $this->_errors;
        else return array();
    }
    public function reset_progress()
    {
        $this->_progress = array();
    }
    public function progress()
    {
        if (isset($this->_progress)) return $this->_progress;
        else return array();
    }
    public function configs()
    {
        return $this->_configs;
    }
    public function directories()
    {
        return $this->_directories;
    }
    public function directory($type, $default=null, $fatal=true)
    {
        if (isset($this->_directories[$type])){
            return $this->_directories[$type];
        }elseif(null != $default){
            return $default;
        }else{
            if($type == 'sng-base'){
                return dirname($this->_directories['sng-bin']);
            }
        }
        if($fatal){
            throw new Exception('Undefined directory (' . $type . ')');
        }
        return false;
    }
    public function about()
    {
        return $this->_about;
    }
    public function services()
    {
        return $this->_services;
    }
    public function audits()
    {
        return $this->_audits;
    }
    public function license()
    {
        return $this->_license;
    }
    public function url()
    {
        return $this->_url;
    }

    public function get_update_config($key){
        return !isset($this->_update[$key])? null : $this->_update[$key];
    }

    public function write_config($node)
    {
        // Write all configuration files
        foreach ($this->_configs as $config) {
            // Assume action will fail
            $success = false;
            $cfg_dir = $this->directory('configuration');
            if (is_array($config->file_name())) {
                $file_name = $config->file_name();
                $display_action = $file_name['target'];
                $full_path = $file_name['target'];
            } else {
                $fname = $config->file_name();
                if ($fname[0] == '/') {
                    $cfg_dir = dirname($fname);
                    $fname = basename($fname);
                }
                $display_action = $config->file_name();
                $full_path = $cfg_dir . '/' . $fname;
            }
            // What do we need to do ?
            switch ($config->type()) {
                // USER action
                
            case Safe_configuration_class::CFG_USER:
                try {
                    $success = $config->run($node);
                }
                catch(Exception $e) {
                }
                break;
                // FILE
                
            case Safe_configuration_class::CFG_FILE:
                switch ($config->action()) {
                case Safe_configuration_class::CFG_UPDATE:
                    $success = $node->write_file($full_path, $config->data());
                    break;

                case Safe_configuration_class::CFG_DELETE:
                    $success = $node->delete_file($full_path);
                    break;

                case Safe_configuration_class::CFG_LINK:
                    $success = $node->symlink_file($file_name['original'], $file_name['target']);
                    break;

                case Safe_configuration_class::CFG_UNLINK:
                    $success = $node->unlink_file($file_name['target']);
                    break;
                }
                break;
                // DIRECTORY
                
            case Safe_configuration_class::CFG_DIR:
                switch ($config->action()) {
                case Safe_configuration_class::CFG_DELETE:
                    $success = $node->rmdir($full_path);
                    break;

                case Safe_configuration_class::CFG_CREATE:
                    $success = $node->mkdir($full_path);
                    break;

                case Safe_configuration_class::CFG_LINK:
                    $success = $node->symlink_file($file_name['original'], $file_name['target']);
                    break;

                case Safe_configuration_class::CFG_UNLINK:
                    $success = $node->unlink_file($file_name['target']);
                    break;
                }
                break;
                // SED
                
            case Safe_configuration_class::CFG_SED:
                switch ($config->action()) {
                    case Safe_configuration_class::CFG_UPDATE:
                    $success = $node->sed_file($full_path, $config->data());
                    break;
                }
                break;
                // chmod
                
            case Safe_configuration_class::CFG_CHMOD:
                    switch ($config->action()) {
                        case Safe_configuration_class::CFG_UPDATE:
                            $success = $node->chmod_file($full_path, $config->data());
                            break;
                    }
                    break;
                    // chown
                    
            case Safe_configuration_class::CFG_CHOWN:
                        switch ($config->action()) {
                            case Safe_configuration_class::CFG_UPDATE:
                                $success = $node->chown_file($full_path, $config->data());
                                break;
                        }
                        break;
                    
                // SQL db
                
            case Safe_configuration_class::CFG_SQL:
                        // Check that arguments are valid
                // $config->file_name() MUST contain table name
                // $config->data() MUST be an array containing the following entries
                //   ['db_class'] = The Safe_database_class object to access SQL DB
                //   Depending on action:
                //     For CREATE
                //       ['data'] = an array containing data to insert in a key/value pair format
                //     For UPDATE and DELETE
                //       ['primary_key'] = an array containing primary keys field names  for where clause
                //                         !! Only field names are required, field value
                //                         is extracted from ['data'] array.
                //       ['data'] = an array containing data to update in a key/value pair format
                $data = $config->data();
                if (!isset($data['db_class']) || !is_subclass_of($data['db_class'], 'Safe_database_class')) {
                    $display_action = 'db_class is not a Safe_database_object.';
                } elseif ((!isset($data['data']) || !is_array($data['data'])) && $config->action() != Safe_configuration_class::CFG_EMPTY) {
                    $display_action = 'No valid data supplied.';
                } elseif (!$data['db_class']->connect()) {
                    $display_action = 'Fail to connect to data base';
                } else {
                    $display_action = $fname;
                    // All good :)
                    switch ($config->action()) {
                    case Safe_configuration_class::CFG_CREATE:
                        // Insert a record
                        $success = $data['db_class']->insert($fname, $data['data']);
                        break;

                    case Safe_configuration_class::CFG_UPDATE:
                        if (!isset($data['primary_key']) || !is_array($data['primary_key'])) {
                            $display_action = 'No primary keys supplied.';
                        } else {
                            // Update a record
                            $success = $data['db_class']->update($fname, $data['primary_key'], $data['data']);
                        }
                        break;

                    case Safe_configuration_class::CFG_DELETE:
                        if (!isset($data['primary_key']) || !is_array($data['primary_key'])) {
                            $display_action = 'No primary keys supplied.';
                        } else {
                            // Delete a record
                            $success = $data['db_class']->delete($fname, $data['primary_key'], $data['data']);
                        }
                        break;

                    case Safe_configuration_class::CFG_EMPTY:
                        // Delete all record
                        $success = $data['db_class']->empty_table($fname);
                        break;

                    case Safe_configuration_class::CFG_LOAD:
                        // Load data to DB
                        $success = $data['db_class']->load_data($fname, $data['data']);
                        break;
                    }
                    if (true != $success) $display_action = $fname . " - " . $data['db_class']->_error_message();
                    // Disconnect from database
                    $data['db_class']->disconnect();
                }
                break;
            }
            // Report error
            $this->_progress[] = array(
                    'module' => 'Configuration Manager',
                    'obj_type' => $config->type() . ' ' . $config->action(),
                    'description' => $display_action,
                    'type' => (true == $success) ? 'OK' : 'ERROR'
            );
            if(true != $sucess){
                safe_log(SAFE_LOG_INFO, 'write_config - ' .
                    $config->type() . ' ' . $config->action() . ' - ' .
                    $display_action . ' - ' . ((true == $success) ? 'OK' : 'ERROR'));
            }
            if (true != $success){
                return false;
            }
        }
        // Update configuration status
        $this->_update_status();
        return true;
    }
    private function _get_status()
    {
        $ser = safe_object_serializer_class::get_serializer();
        $ser->retrieve_object('/tmp', 'configuration_status', $data);
        return $data;
    }
    private function _update_status()
    {
        // Update configuration status
        $ser = safe_object_serializer_class::get_serializer();
        $ser->retrieve_object('/tmp', 'configuration_status', $data);
        $data['generate']['count'] += 1;
        $data['generate']['date'] = date('Y-m-d-H-i-s');
        $ser->store_object('/tmp', 'configuration_status', 'none', $data, 'tmp');
    }

    public function generate_count()
    {
        $data = $this->_get_status();
        return $data['generate']['count'];
    }
    public function generate_status()
    {
        $data = $this->_get_status();
        if($data){
            return $data['generate'];
        }else{
            return array('count' => 0);
        }
    }

    public function node()
    {
        return $this->_node;
    }
    public function &archive_manager()
    {
        // Create on demand
        if (!isset($this->_archive_manager)) {
            $this->_archive_manager = new Safe_archive_manager_class($this);
            $this->_archive_manager->configure();
        }
        return $this->_archive_manager;
    }
    public function checklist(&$list, $check_generate=true)
    {
        if($check_generate){
            $status = $this->_get_status();
            if(!$status['generate']['count']){
                $list[] = array(
                    'module' =>'application',
                    'obj_type' =>'configuration',
                    'description' => 'Not generated',
                );

            }
        }
    }
}
class Safe_archive_manager_class extends Safe_object_class
{
    private $_config_manager = null;
    private $_node = null;
    private $_arch_dir = null;
    public function __construct($parent)
    {
        parent::__construct($parent->object_name() , "archive_manager");
        $this->_config_manager = $parent;
        $this->_node = $parent->node();
        // Ensure archives directories are created
        try {
            $this->_arch_dir = $this->_config_manager->directory('archive');
        }
        catch(Exception $e) {
            // use base dir and cat archive
            $this->_arch_dir = $this->_config_manager->directory('base');
            $this->_arch_dir.= '/archive';
        }
        // Create directories
        $this->_node->mkdir($this->_arch_dir);
        $this->_node->mkdir($this->_arch_dir . '/tmp');
    }
    private $_script_path = '';
    private function get_script_path()
    {
        if($this->_script_path == ''){
            $this->_script_path = $this->_node->configuration_manager()->directory('sng-base') . '/bin';
        }
        return $this->_script_path;
    }
    public function restore_archive($archive_name, $skip_license = false, $skip_tdm = false)
    {
        safe_log(SAFE_LOG_INFO, 'restore_archive - '.$archive_name);
        $os = $this->_node->os();
        $skip = $skip_license ? ' --skip-license' : '';
        $skip .= $skip_tdm ? ' --skip-tdm' : '';
        $cmd = 'nohup ' . $this->get_script_path() . '/sng-archive-restore '.$this->_arch_dir.'/'.$archive_name . $skip;
        $output = array();
        $rc = $os->execute($cmd, $output, true, true);
        return $rc;
    }
    public function create_archive($archive_type = 'system', $name=null)
    {
        if(empty($archive_type)){
            $archive_type = 'system';
        }
        $os = $this->_node->os();
        // Ready to ZIIIIP....
        if(!empty($name)) {
            // Make sure we don't ends up with 2 .tgz
            $name = str_replace(".tgz", "", $name);
            $name .= '.tgz';
        }else{
            $name = php_uname('n') . '-'.$archive_type.'-backup-' . date('Y-m-d-H-i-s') . '.tgz';
        }
        safe_log(SAFE_LOG_INFO, 'create_archive - '.$name);
        //Generate a mode file
        $sngms_module = $this->_node->software()->application()->find_module_by_name('sngms');
        $transcoding_mode = $sngms_module->config();
        $mode = $transcoding_mode->ip_mode();
        file_put_contents('/tmp/nsc_mode.txt', $mode);
        // Create tar command line
        $cmd = 'nohup ' . $this->get_script_path() . '/sng-archive-backup '.$archive_type.' '.$this->_arch_dir.'/tmp/'.$name;
        $output = array();
        $rc = $os->execute($cmd, $output, true, true);
        if ($rc) {

            if ($archive_type == 'template') {
                // extra steps for template type backup
                //1. untar the archive
                $cmd = 'nohup tar -xzvf ' .$this->_arch_dir.'/tmp/'.$name .' -C /provisioning/template/';
                $rc = $os->execute($cmd, $output, true, true);
                if ($rc){
                    //2. prune the template 
                    $cmd = 'nohup /provisioning/cleanup.sh';
                    $rc = $os->execute($cmd, $output, true, true);
                    //3. Create a new template
                    $cmd = 'nohup /provisioning/archive.sh '. $name;
                    $rc = $os->execute($cmd, $output, true, true);
                    if (!$rc || ! $os->move_file('/provisioning/' . $name, $this->_arch_dir . '/tmp/' . $name)){
                        return false;
                    }
                } else {
                    return false;
                }
            }
            // Succeed move archive to vault
            if ($os->move_file($this->_arch_dir . '/tmp/' . $name, $this->_arch_dir . '/' . $name)){
                $file = $os->find_file($this->_arch_dir, $name.'.info');
                if($file){
                    $os->delete_file($file);
                }
                return $name;
            }
        }
        return false;
    }
    public function directory($pattern = false)
    {
        if ($pattern) return $this->_arch_dir . '/*.tgz';
        else return $this->_arch_dir;
    }
    public function info($archive_name,$more_info=array(),$force=false)
    {
        $ar_more_info = array();
        foreach($more_info as $file => $attributes){
            foreach($attributes as $name)
                $ar_more_info[] = $file.':'.$name;
        }
        $info = array();
        // Check if specified archive can be restored
        //create a tmp info file to cache the backup file info
        $info_file = $this->_arch_dir. '/' . $archive_name . '.info';
        if(file_exists($info_file) && !$force){
            $tmp_data = file_get_contents($info_file);
            if($tmp_data){
                $info = json_decode($tmp_data, true);
            }
        }else{
            $os = $this->_node->os();
            $cmd = 'nohup ' . $this->get_script_path() . '/sng-archive-info '.$this->_arch_dir.'/'.$archive_name;
            if($ar_more_info!=array()){
                $cmd.=' --info "'.implode(',',$ar_more_info).'"';
            }
            $output = array();
            $rc = $os->execute($cmd, $output, true, true);
            $ini_string = implode("\n",$output);
            $info = parse_ini_string($ini_string);
            //write to json file
            if($info){
                $tmp_data = json_encode($info);
                file_put_contents($info_file, $tmp_data);
            }    
        }
        return $info;
    }
    public function can_restore($archive_name, &$reasons)
    {
        // Check if specified archive can be restored
        $os = $this->_node->os();
        $cmd = 'nohup ' . $this->get_script_path() . '/sng-archive-test '.$this->_arch_dir.'/'.$archive_name;
        $rc = $os->execute($cmd, $reasons, true, true);
        return $rc;
    }
    public function get_archive_configurable_object()
    {
        $os = $this->_node->os();
        $conf_file_list =  $os->search_files('/etc/*-backup.conf');
        $type = array();
        foreach($conf_file_list as $file_name){
             $tmp_type= basename($file_name, '-backup.conf');
             $type[$tmp_type] = ucfirst($tmp_type).' '.'Backup';
        }
        $param =  new Safe_configurable_object_class('', 'archive');
        $param->add_field('name', 'Backup Name', 'text', '', 30);
        $param->set_field_rules('name', 'alpha_dash');
        $param->add_enum_field('backup_type', 'Backup Type', 'dropdown', '', $type);
        return $param;
    }
    public function get_archive_restore_configurable_object()
    {
        $param =  new Safe_configurable_object_class('', 'archive');
        $methods = array(
            'Network' => 'Skip Network Configuration',
            'License' => 'Skip License Configuration',
        );
        $tdmgw = $this->_node->software()->application()->find_module_by_name('tdmgw');
        if(isset($tdmgw)){
            $methods['TDM'] = 'Skip TDM Configuration';
        }
        $param->add_multiple_field('backup_exclude_opts', 'Restore Options', 'inline_checkbox', array(), $methods);
        return $param;
    }

}
/* End of file safe_config_manager_class.php */
