<?php
  // This is an alternative to CodeIgniter ftp class. This is because
  // CodeIgniter ftp implementation is a poor one. It has no file size
  // implementation, needed for downloading progress.
class Safe_ftp{
    private $protocol;
    private $connectID;
    private $server;
    private $logged;
    private $curDir;

    public function __construct(){
        $this->protocol = null;
        $this->connectID = null;
        $this->server = null;
        $this->logged = null;
        $this->curDir = '/';
    }

    public function __destruct(){
        $this->close();
    }

    public function isSecure(){
        return $this->protocol == 'ftps';
    }

    public function connect($config){
        $this->protocol = isset($config['protocol'])? $config['protocol'] : 'ftp';
        $this->server = isset($config['hostname'])? $config['hostname'] : null;
        $username = isset($config['username'])? $config['username'] : 'anonymous';
        $password = isset($config['password'])? $config['password'] : '';
        $port = isset($config['port'])? $config['port'] : 21;
        $timeout = isset($config['timeout'])? $config['timeout'] : 10;

        $this->connectID = $this->isSecure()? ftp_ssl_connect($this->server,$port,$timeout) : ftp_connect($this->server,$port,$timeout);    
        return $this->login($username,$password);
    }

    public function close(){
        if($this->connectID){
            @ftp_close($this->connectID);
            $this->server=null;
            $this->protocol = null;
            $this->connectID=null;
        }
        return $this;
    }

    public function list_files($path){
        return $this->ls($path);
    }

    public function ls($path='.'){
        return @ftp_nlist($this->connectID,$path);
    }

    public function cd($path='.'){
        try{
            @ftp_chdir($this->connectID,$path);
            return true;
        }catch(Exception $e){
            return false;
        }
    }

    public function up(){
        $this->cd('../');
        return $this;
    }

    public function pwd(){
        return @ftp_pwd($this->connectID);
    }

    public function login($user,$passw){
        $this->logged = @ftp_login($this->connectID,$user,$passw);
        $this->cd($this->curDir);
        return $this;
    }

    public function isLoggedIn(){
        return $this->logged;
    }
    
    public function size($file){
        return (int) @ftp_size($this->connectID,$file);
    }
  }


?>