<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * Hardware class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_node_class');
safe_require_class('safe_configuration_manager_class');
safe_require_class('safe_object_class');
class Safe_hardware_adapter_class extends Safe_serializable_object_class
{
    /*
     * Adapter type
    */
    const ETHERNET_ADAPTER = 'ETHERNET';
    const TRANSCODING_ADAPTER = 'TRANSCODING';
    private $_ifname;
    private $_type;
    private $_node;
    public function __construct($node, $path, $name)
    {
        $this->_node = $node;
        parent::__construct($path, $name);
    }
    /*
    public function __construct($parent, $ifname, $type, $sync_type = Safe_serializable_object_class::SYNC_MEM)
    {
        $this->_ifname = $ifname;
        $this->_type = $type;
        if(get_class($parent) == 'Safe_local_node'){
            $parent = $parent->hardware();
        }
        $this->_node = $parent->node();
        parent::__construct($parent->object_name() . "/network/" . $type, $ifname, $sync_type);
    }*/
    public function is_configured()
    {
        true;
    }
    public function configure($node=null)
    {
        return TRUE;
    }
    public function type()
    {
        return $this->_type;
    }
    public function node()
    {
        return $this->_node;
    }
    // Serializable interface implementation
    protected function _get_serialized_data()
    {
        $data = array();
        $data['type'] = $this->_type;
        $data['ifname'] = $this->_ifname;
        return $data;
    }
    // Set the php array of retrieved data
    protected function _set_unserialized_data($data)
    {
        $this->_type = $data['type'];
        $this->_ifname = $data['ifname'];
    }
    public function generate_config(&$config)
    {
        // Default is no config
        return TRUE;
    }
}
class Safe_ethernet_adapter_class extends Safe_hardware_adapter_class
{
    private $_mac;
    private $_ip_address;
    private $_mask;
    private $_gateways = array();
    
    private $_mode;
    private $_speed;
    private $_configuration;
    
    public function __construct($node, $path, $name)
    {
        parent::__construct($node, $path, $name);
    }
    /*
    public function __construct($hardware, $ifname, $mac, $ip, $mask, $type = null, $sync_type = Safe_serializable_object_class::SYNC_MEM)
    {
        if (!isset($type)) $type = Safe_hardware_adapter_class::ETHERNET_ADAPTER;
        $this->_mac = $mac;
        $this->_ip_address = $ip;
        $this->_mask = $mask;
        parent::__construct($hardware, $ifname, $type, $sync_type);
    }*/
    public function mac()
    {
        return $this->_mac;
    }
    public function ip_address()
    {
        return $this->_ip_address;
    }
    public function mask()
    {
        return $this->_mask;
    }
    public function gateways()
    {
        return $this->_gateways();
    }
    public function gateway()
    {
        if (isset($this->_gateways[0])) return $this->_gateways[0];
        else return '';
    }
    // Serializable interface implementation
    // Cumulative array with parent
    protected function _get_serialized_data()
    {
        $data = parent::_get_serialized_data();
        $data['mac'] = $this->_mac;
        $data['ip_address'] = $this->_ip_address;
        $data['mask'] = $this->_mask;
        
        $data['mode'] = $this->mode;
        $data['speed'] = $this->speed;
        $data['configuration'] = $this->configuration;
        
        return $data;
    }
    public function get_data_values(){
        return $this->_get_serialized_data();
    }
    public function set_data_values($data){
        $obj_data = $this->_get_serialized_data();
        foreach($data as $key => $value){
            $obj_data[$key] = $value;
        }
        return $this->_set_unserialized_data($obj_data);
    }
    // Set the php array of retrieved data
    protected function _set_unserialized_data($data)
    {
        $this->_mac = $data['mac'];
        $this->_ip_address = $data['ip_address'];
        $this->_mask = $data['mask'];
        
        $this->mode = $data['mode'];
        $this->speed =  $data['speed'];
        $this->configuration = $data['configuration'] ;
        
        parent::_set_unserialized_data($data);
    }
    public function summary($type = 'horizontal' , $long = false, $min=3)
    {
        $result = array(
                    'Name' => $this->name(),
                    'Mac Address' => $this->_mac
                );
        return array( 'data' => $result);
    }
}
class Safe_hardware_adapter_detector_class
{
    private $_detector_type;
    public function __construct($type)
    {
        $this->_type = $type;
    }
    public function type()
    {
        return $this->_type;
    }
    public function detect($hardware_classe)
    {
        return array();
    }
}
class Safe_hardware_class extends Safe_module_class
{
    private $_node = null;
    protected $_adapters = array();
    private $_detectors = array();
    public function __construct($node)
    {
        parent::__construct($node, 'hardware');
        $this->_node = $node;
    }
    public function configure($node=null)
    {
        // Set the module description
        $this->set_description("Hardware");
        // Register objects
        $this->register_aggregate_object('adapter',
            array(
                'name' => 'Adapter',
                'dynamic' => false,
                'has_child' => true,
                'controller_url' => '/SAFe/sng_network_config/modify/network',
                'base_path' => false, // Prevent using base_path, force usage of api_retrieve_adapter()
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve Adapter',
                        'request' => 'GET',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('ETHERNET',
                array(
                        'name' => 'ETHERNET Adapter',
                        'dynamic' => true,
                        'has_child' => false,
                        'base_path' => parent::object_name(). '/network' . '/ETHERNET',
                )
        );
        $this->register_aggregate_object('TRANSCODING',
                array(
                        'name' => 'TRANSCODING Adapter',
                        'dynamic' => true,
                        'has_child' => false,
                        'base_path' => parent::object_name(). '/network' . '/TRANSCODING',
                )
        );

        return parent::configure();
    }
    public function api_retrieve_ethernet($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('ETHERNET');
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    public function api_retrieve_transcoding($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('TRANSCODING');
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    public function api_retrieve_adapter($obj_name=null)
    {
        $adapters = $this->adapters();
        if($obj_name){
            return $adapters[$obj_name];
        } else {
            return $adapters;
        }
    }
    //Type can be null for all or ethernet for management interfaces
    public function adapters($type = null)
    {
        $adapters = array();
        if (isset($type)) {
            return  $this->get_aggregate_objects($type);
        } else {
            $adapters = $this->get_aggregate_objects(Safe_hardware_adapter_class::ETHERNET_ADAPTER);
            $adapters = array_merge($adapters, $this->get_aggregate_objects(Safe_hardware_adapter_class::TRANSCODING_ADAPTER));
        }
        return $adapters;
    }
    public function adapters_ip($type = Safe_hardware_adapter_class::ETHERNET_ADAPTER)
    {
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        return $network_module->interfaces_ip($type);
    }
    public function register_detector($detector)
    {
        // For now assume only one detector per type
        $this->_detectors[$detector->type()] = $detector;
    }
    public function detect_adapters($type = null, $user_arguments = null)
    {
        if (!isset($type)) {
            // Invoke all detectors
            $this->detect_adapters(Safe_hardware_adapter_class::ETHERNET_ADAPTER, $user_arguments);
            $this->detect_adapters(Safe_hardware_adapter_class::TRANSCODING_ADAPTER, $user_arguments);
            return TRUE;
        } else {
            // Drop only specified adapters type
            if (isset($this->_adapters[$type])) {
                foreach ($this->_adapters[$type] as $adapter) {
                    $adapter->dispose();
                }
                // Drop adapters array
                unset($this->_adapters[$type]);
            }
            // Create type container
            $this->_adapters[$type] = array();
            if (isset($this->_detectors[$type])) {
                // push detected adapters
                $detected_adapters = $this->_detectors[$type]->detect($this, $user_arguments);
                foreach ($detected_adapters as $key => $adapter) $this->_adapters[$type][$key] = $adapter;
                // TODO(wadam) - Delete disposed objects
                return TRUE;
            } else return FALSE;
        }
    }
    public function node()
    {
        return $this->_node;
    }
    public function generate_config(&$configs)
    {
        // Iterate on adapters
        foreach ($this->_adapters as $adapter_type) {
            foreach ($adapter_type as $adapter) {
                if (!$adapter->generate_config($config)) return FALSE;
            }
        }
        return TRUE;
    }
    
    /**
     * @brief Gets HDD information. The value returned is as follows: 
     * array('sda'=>array(key=>value,...))
     *
     * @return array 
     */
    public function hdd_info(){
        $output = array();
        $this->node()->execute('for DISK in `eval echo /dev/[h,s]d?`; do sudo /sbin/hdparm -i $DISK; done',$output,$escape=false);        
        $hdd = array();
        $device = null;
        foreach($output as $row){
            $row = trim($row);
            if(preg_match('#^/dev/(.+)#',$row,$match)){
                $device= preg_replace('#:$#','',$match[1]);
                $hdd[$device] =array();
            }else if(preg_match_all('#([^=]+)=((\{\s*[^\}]+\s*\})|([^\s]+))#',$row,$match)){
                foreach($match[1] as $pos => $m)
                    $hdd[$device][strtolower(trim(preg_replace('#\W#','',$match[1][$pos])))] = preg_replace('#,$#','',trim($match[2][$pos]));
            }
        }
        return $hdd;
    }

    /**
     * @brief Checks if hardware matches
     * @param string network|hdd
     * @param array  Array with hardware information. If type is network this
     * array should be as follows: 
     *    arr_hw[interface_name] = mac
     * If type is hdd it should be
     *    arr_hw[whatever] = hdd-serial
     * @return boolean
     */
    public function match($type='network',$arr_hw=array()){
        $match = true;
        switch(strtolower($type)){
        case 'network':
            $network = $this->node()->software()->application()->find_module_by_name('network');
            $interfaces = $network->api_retrieve_interface();
            foreach($interfaces as $key => $obj){
                if($obj->interface_type()==Sng_network_interface_class::PHYSICAL){
                    $match = $match && (isset($arr_hw[$key]) && strtolower($arr_hw[$key])==strtolower($obj->mac()));
                }
            }
            break;
        case 'hdd':
            $arr_aux = array();
            foreach($arr_hw as $key => $serial){
                $arr_aux[strtolower($serial)] = true;
            }
            $hdds = $this->hdd_info();
            foreach($hdds as $dev => $hd_info){
                $match = $match && isset($arr_aux[$hd_info['serialno']]);
            }
            break;
        default:
            return false;
        }
        return $match;
    }
    
    /**
     * @brief check if on virtual machine
     *
     * @return TRUE/FALSE
     */
    public function is_virtual(){
        $hdd = $this->hdd_info();
        $hdd = array_pop($hdd);
        // We should figure out how other virtualization systems set hdd labels
        return in_array($hdd['model'],array('VBOX'));
    }

    /**
     * @brief Pre backup hook
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function pre_backup($type, &$reason)
    {
        return TRUE;
    }
    /**
     * @brief Post backup hook
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function post_backup($type, &$reason)
    {
        return TRUE;
    }
    /**
     * @brief Check if archive restore can be done
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function can_restore($info, &$reason)
    {
        return TRUE;
    }
    /**
     * @brief Pre restore hook
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function pre_restore($info, &$reason)
    {
        return TRUE;
    }
    /**
     * @brief Post restore hook
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function post_restore($info, &$reason)
    {
        // Restart network
        $this->node()->os()->service_ctl('network', 'restart');
        // Reconfigure
        $this->configure();
        return TRUE;
    }
}
/* End of file safe_hardware_class.php */
