<?php
/*
 * License class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
class Safe_license extends Safe_configurable_object_class
{
    // Default is no license
    private $_status = false;
    /*
     * License type
    */
    const NOT_INSTALLED = 'Not Installed';
    const INVALID = 'Invalid';
    const FULL = 'Installed';
    const DEMO = 'Demo';
    const TIME_BOMB = 'timebomb';
    const DEFAULT_MAX_SESSIONS=2000;
    
    private $_type = Safe_license::INVALID;
    protected $_info;
    public function __construct($node)
    {
      parent::__construct($node->object_name(), 'license', Safe_serializable_object_class::SYNC_MEM);
    }
    public function configure($node = null)
    {
        // Get about file
        $config = $node->configuration_manager()->license();
        $this->_node = $node;
        $file = $config['file'];
        // TODO(wadam) - need to validate license against key
        // Read license file (as array)
        if ($node->read_file($file, $content, 'r', true)) {
            // convert to proper array
            foreach ($content as $line) {
                $line = trim($line);
                if (strlen($line)) {
                    // License format is
                    // KEY=VALUE
                    $kv = explode("=", $line);
                    // Check we have enough info
                    if (isset($kv[0]) && isset($kv[1])) {
                        // Add as key/value in array                        
                        $this->_info[$kv[0]] = $kv[1];                     
                    }
                    unset($kv);
                }
            }
            $this->_status = true;
            $this->_type = Safe_license::FULL;
        } else {
            $this->_status = false;
            $this->_type = Safe_license::NOT_INSTALLED;
        }
        // Build fields list
        //$this->add_field('status', 'Status', 'text', $this->_type);
        //$this->set_field_disabled('status');
        if(!empty($this->_info)) {
          foreach($this->_info as $k => $v){
            $this->add_field(strtolower($k), $k, 'text', $v);
            $this->set_field_disabled(strtolower($k));
          }
          $tmp_data = $this->get_data_values();
        }
        return parent::configure();
    }
    public function info($key = null)
    {
        if (isset($key)) return (isset($this->_info[$key])) ? $this->_info[$key] : "";
        return $this->_info;
    }
    public function status(&$type = null)
    {
        if (isset($type)) $type = $this->_type;
        return $this->_status;
    }
    public function get_max_session()
    {
        if($this->status()){
            $license_max_session = $this->info('Max-Sessions');
            if($license_max_session){
                return $license_max_session;
            }else if($this->info('Name')=='Sangoma Development'){
                return self::DEFAULT_MAX_SESSIONS;
            }
        }
        return false;
    }
    public function describe()
    {
        $describe['Status'] = $this->_type;
        $describe = array_merge($describe, $this->get_data_values_with_field_desc());
        return $describe;
    }
    // TODO(wadam) - copy the get_hook_script and pre/do/post_reset function
    // need to convert license to a module
    protected function get_hook_script($step , $type)
    {
        $base = $this->_node->configuration_manager()->directory('base');
        $script = $base.'/hooks/'.$this->name().'.' . $type .'.'.$step;
        if(file_exists($script)) {
            return $script;
        }else{
            return '';
        }
    }
    public function pre_reset($info, &$reason)
    {
        safe_log(SAFE_LOG_INFO, 'pre_reset - '.$this->name().' - '.$info['type']);
        // Check if product script exists
        $script = $this->get_hook_script('pre', 'reset.'.$info['type']);
        if($script) {
            safe_log(SAFE_LOG_INFO, 'pre_reset - Invoking ' . $script);
            $this->node()->execute('nohup '.$script. ' ' . $info['type'], $retCode, true, true, $rc);
        }
        return true;
    }
    public function do_reset($info, &$reason)
    {
        safe_log(SAFE_LOG_INFO, 'do_reset - '.$this->name().' - '.$info['type']);
        if($info['type'] == 'config' || $info['type'] == 'factory' ){
            // All modules have performed their cleanup, now remove all objects:
            // 1- Remove all (N) record from object table
            // 2- Change all records to (D)
            // 3- Disable error checking in configuration manager
            // 4- Generate config (this will proceed to all delete)
            $ser = Safe_object_serializer_class::get_serializer(TRUE);
            $ser->delete_object_with_status($this->object_name() , TRUE, Safe_object_serializer_class::OBJ_STATUS_NEW);
            $ser->update_object_with_status($this->object_name() , TRUE, NULL, Safe_object_serializer_class::OBJ_STATUS_DELETED);
            //clear the checklist and modify list cache
            $this->clear_list_cache();
        }
        return true;
    }
    public function post_reset($info, &$reason)
    {
        safe_log(SAFE_LOG_INFO, 'post_reset - '.$this->name().' - '.$info['type']);
        // Check if product script exists
        $script = $this->get_hook_script('post', 'reset.'.$info['type']);
        if($script) {
            safe_log(SAFE_LOG_INFO, 'post_reset - Invoking ' . $script);
            $this->node()->execute('nohup '.$script. ' ' . $info['type'], $retCode, true, true, $rc);
        }
        return true;
    }

    public function allow_service(&$service){
        $unallowed_services = array('NetBorder SS7 Gateway'=>array('Security'));
        $ret = true;
        $product = $this->info('Product');
        if(isset($unallowed_services[$product]) && in_array($service->category(),$unallowed_services[$product]))
            $ret = false;
        return $ret;
    }

    public function equal($license_arr){
        foreach($license_arr as $key => $value)
            $license_arr[strtolower($key)] = $value;
        foreach($this->_info as $key => $value){
            $key = strtolower($key);
            if(!isset($license_arr[$key]) || $license_arr[$key] != $value)
                return false;
        }
        return empty($license_arr) || empty($this->_info)? false : true;
    }
}
/* End of file safe_license_class.php */
