<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * SAFe object classes
*/
require_once ('application/helpers/safe_helper.php');
require_once ('system/libraries/Form_validation.php');
require_once ('application/libraries/safe_db_data_source_class.php');
require_once ('safe_form_class.php');
class Safe_object_serializer_class
{
    const OBJ_STATUS_NEW = 'N';
    const OBJ_STATUS_MODIFIED = 'M';
    const OBJ_STATUS_DELETED = 'D';
    const OBJ_STATUS_UP_TO_DATE = 'U';
    const OBJ_TYPE_CONFIGURABLE = 'configurable';
    const OBJ_TYPE_AUTOFILL = 'autofill';
    private static $_serializer = null;
    public static function get_serializer($force = FALSE)
    {
        if ($force) Safe_object_serializer_class::$_serializer = null;
        if (null == Safe_object_serializer_class::$_serializer) {
            Safe_object_serializer_class::$_serializer = new Safe_object_serializer_class();
        }
        return Safe_object_serializer_class::$_serializer;
    }
    /**
     * @brief Return next unique id
     *
     * @return unique id (must be used before new record added)
     */
    public static function get_next_unique_id()
    {
        $ser = Safe_object_serializer_class::get_serializer();
        return $ser->next_insert_id();
    }
    public static function build_unique_name($prefix)
    {
        $prefix =  preg_replace('/[^0-9a-zA-Z\-_]/', '', $prefix);
        $id = Safe_object_serializer_class::get_next_unique_id();
        return $prefix.'_'.$id;
    }
    /**
     * @var CI_DB_pdo_driver
     */
    private $db = null; // the codeigniter db object
    private $db_groupname = 'safe'; // the CodeIgniter groupname set in
    // application/config/database.php
    private $db_table = 'safe_object';
    // TODO: handling CodeIgniter database errors
    // using this $this->db->_error_message(); will return empty if no error.
    // and in ./config/database.php: $db['default']['db_debug'] = TRUE; Set it
    // to false.
    // or force it by $this->db->db_debug= flase;
    // TODO: put some method private and proteceted.
    public function __construct()
    {
        safe_enter_critical_section();
        try {
            $CI = & get_instance();
            // if the files for the db doesn't it will be created.
            $this->db = $CI->load->database($this->db_groupname, TRUE);
            if (!$this->db->table_exists($this->db_table)) if (!$this->create_db()) return false;
        }
        catch(Exception $e) {
            // TODO: catch CodeIgniter DB_Driver exception.
            // we will never go here since CodeIgniter DB_Driver will fail
            // with a PHP error on the user browser
            safe_log(SAFE_LOG_ERROR, "ERROR - Cannot open database ".$e->getMessage());
            show_error(__FUNCTION__ . ": can't open databases");
        }
        safe_leave_critical_section();
    }
    public function __destruct()
    {
        try {
            if ($this->db) $this->db->close();
        }
        catch(Exception $e) {
        }
    }
    /**
     *
     * @return boolean
     */
    private function create_db()
    {
        try {
            safe_log(SAFE_LOG_INFO, "Safe_object_serializer: Trying to create safe_object DB");
            $sqlq = "
        CREATE TABLE '" . $this->db_table . "' (
          'id' INTEGER PRIMARY KEY AUTOINCREMENT,
          'path' TINYTEXT NOT NULL,
          'name' VARCHAR(128) NOT NULL,
          'type' VARCHAR(32) NOT NULL DEFAULT '" . Safe_object_serializer_class::OBJ_TYPE_CONFIGURABLE . "',
          'class' VARCHAR(32) NOT NULL,
          'modified' TINYINT NOT NULL DEFAULT 1,
          'status' CHAR(1) NOT NULL DEFAULT '" . Safe_object_serializer_class::OBJ_STATUS_NEW . "',
          'data' TEXT,
          'modified_date' DATE DEFAULT (datetime('now','localtime')),
          'modified_by'   VARCHAR(128),
          'data_ref' TEXT,
          CONSTRAINT unique_object UNIQUE ('type', 'path', 'name') ) ;
      ";
            if (!$this->db->query($sqlq)) throw new Exception("Cannot create DB!");
        }
        catch(Exception $e) {
            //safe_add_error(__FUNCTION__ . ":{$this->filename}: " . $e->getMessage() , true);
            safe_log(SAFE_LOG_ERROR, "ERROR - Cannot create_db() ".$e->getMessage());
            return false;
        }
        return true;
    }

    /**
     * @brief Returns the database object
     * @returns Object The database object.
     */ 
    public function get_db(){
        return $this->db;
    }
    
    public function store_object($path, $name, $class, $object_data, $object_type = NULL, &$object_status = NULL, &$object_id = NULL, $reuse=true)
    {
        if(empty($name)){
            error_log('ERROR - Store object Error : object name is empty.');
            return false;
        }
        $insert['path'] = $path;
        $insert['name'] = $name;
        $insert['class'] = $class;
        $insert['data'] = json_encode($object_data);
        date_default_timezone_set('UTC');
        $insert['modified_date'] = date('Y-m-d H:i:s');
        if(isset($_SESSION['user_login'])){
            $insert['modified_by'] = $_SESSION['user_login'];
        }
        if (isset($object_type)) $insert['type'] = $object_type;
        // Object exists ?
        $old_data = array();
        $obj_status = '';
        $existing_obj = $this->retrieve_object($path, $name, $old_data, $obj_status, $object_id);
        // Object exists and allowed to be re-used ?
        // OR
        // Object exists and NOT allowed to be re-used and not yet serialized (ie. NEW) ?
        if (
            (TRUE == $reuse && TRUE == $existing_obj)
            ||
            (TRUE != $reuse && TRUE == $existing_obj && $obj_status == Safe_object_serializer_class::OBJ_STATUS_NEW)
           )
        {
            // Yes, need to update
            if (json_encode($old_data) != $insert['data']) {
                // Data modified, update data and status to modified
                $insert['modified'] = 1;
                // Keep the NEW status
                if ($obj_status != Safe_object_serializer_class::OBJ_STATUS_NEW) {
                    $obj_status = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
                }
                //safe_log(SAFE_LOG_INFO, 'store_object - ' . $obj_status . ' - ' . $path . '/' . $name);
            } else {
                // data not modified, check if need to revert Deleted
                if ($obj_status != Safe_object_serializer_class::OBJ_STATUS_DELETED) {
                    // Not a deleted object, skip update
                    //safe_log(SAFE_LOG_INFO, 'Skipping store_object with same data for ' . $path . '/' . $name);
                    return TRUE;
                }
                // Deleted object, revert
                //safe_log(SAFE_LOG_INFO, 'store_object with same data (reverting DELETED) for ' . $path . '/' . $name);
                $obj_status = Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE;
                $insert['modified'] = 0;
            }
            $insert['status'] = $obj_status;
            $where = array(
                'path' => $path,
                'name' => $name,
            );
            $this->db->where($where);
            if (!$this->db->update($this->db_table, $insert)) {
                //safe_log(SAFE_LOG_ERROR, "Fail to UPDATE serialize object " . $name);
                return FALSE;
            }
            if (isset($object_status)) $object_status = $obj_status;
        } else {
            // Existing and not allowed to be re-used
            // yes, rename old record
            if (true != $reuse && true == $existing_obj){
                $update = $insert;
                $update['name'] = $this->build_unique_name('__deleted__'.$name);
                $update['modified'] = 1;
                $update['status'] = Safe_object_serializer_class::OBJ_STATUS_DELETED;
                unset($update['data']);
                $this->db->where('id', $object_id);
                if (!$this->db->update($this->db_table, $update)) {
                    safe_log(SAFE_LOG_ERROR, "Fail to UPDATE to DEL serialize object " . $name);
                    return FALSE;
                }
            }
            // No, insert it
            if (!$this->db->insert($this->db_table, $insert)) {
                safe_log(SAFE_LOG_ERROR, "Fail to INSERT serialize object " . $name);
                return FALSE;
            }
            if (isset($object_id)) $object_id = $this->db->insert_id();
            if (isset($object_status)) $object_status = Safe_object_serializer_class::OBJ_STATUS_NEW;
        }
        return TRUE;
    }
    /**
     * @brief Mark an object as deleted
     *
     * @param[in out] $path
     * @param[in out] $name
     *
     * @return
     */
    public function dispose_object($path, $name, $reuse=true)
    {
        // If NEW object then delete it
        // else mark as deleted (and modified)
        $dummy = array();
        $status = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
        $id = 0;
        if(isset($_SESSION['user_login'])){
            $insert['modified_by'] = $_SESSION['user_login'];
        }
        // Retrieve object status and id
        if ($this->retrieve_object($path, $name, $dummy, $status, $id)) {
            if ($status == Safe_object_serializer_class::OBJ_STATUS_NEW) {
                // Not yet fetched, delete
                $this->db->where('id', $id);
                $this->db->delete($this->db_table);
            } else {
                // Existing and fetched object, mark as deleted
                $insert['modified'] = 1;
                $insert['status'] = Safe_object_serializer_class::OBJ_STATUS_DELETED;
                if(true != $reuse){
                    // We'd like this object to be gone for good so cannot be
                    // updated after delete
                    $insert['name'] = Safe_object_serializer_class::build_unique_name('__deleted__'.$name);
                }
                $this->db->where('id', $id);
                if (!$this->db->update($this->db_table, $insert)) {
                    safe_log(SAFE_LOG_ERROR, "Fail to UPDATE serialize object " . $name);
                    return FALSE;
                }
            }
            return TRUE;
        }
        return FALSE;
    }
    /**
     * @brief Mark an object as up to date
     *
     * @param[in out] $path
     * @param[in out] $name
     *
     * @return
     */
    public function mark_object_uptodate($path, $name)
    {
        $dummy = array();
        $status = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
        $id = 0;
        // Retrieve object status and id
        if ($this->retrieve_object($path, $name, $dummy, $status, $id)) {
            // Existing and fetched object, mark as deleted
            $insert['modified'] = 0;
            $insert['status'] = Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE;
            $this->db->where('id', $id);
            if (!$this->db->update($this->db_table, $insert)) {
                safe_log(SAFE_LOG_ERROR, "Fail to UPDATE serialize object " . $name);
                return FALSE;
            }
            return TRUE;
        }
        return FALSE;
    }
    /**
     * @brief Mark an object as modified
     *
     * @param[in out] $path
     * @param[in out] $name
     *
     * @return
     */
    public function mark_object_modified($path, $name)
    {
        $dummy = array();
        $status = Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE;
        $id = 0;
        // Retrieve object status and id
        if ($this->retrieve_object($path, $name, $dummy, $status, $id)) {
            // Existing and fetched object, mark as deleted
            $insert['modified'] = 1;
            $insert['status'] = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
            $this->db->where('id', $id);
            if (!$this->db->update($this->db_table, $insert)) {
                safe_log(SAFE_LOG_ERROR, "Fail to UPDATE serialize object " . $name);
                return FALSE;
            }
            return TRUE;
        }
        return FALSE;
    }
    /**
     * @brief
     *
     * @param[in out] $path
     * @param[in out] $name
     * @param[in out] $object_data
     *
     * @return
     */
    public function retrieve_object($path, $name, &$object_data = NULL, &$object_status = NULL, &$object_id = NULL, &$class = NULL, &$type = NULL)
    {
        $object_data = array();
        $q = $this->db->get_where($this->db_table, array(
            'path' => $path,
            'name' => $name
        ));
        if ($q->num_rows()) {
            $r = $q->row();
            $object_data = json_decode($r->data, true);
            if (isset($class)) $class = $r->class;
            if (isset($object_status)) $object_status = $r->status;
            if (isset($object_id)) $object_id = $r->id;
            if (isset($type)) $type = $r->type;
            return TRUE;
        }
        return FALSE;
    }
    /**
     * @brief
     *
     * @param[in out] $path
     * @param[in out] $like
     *
     * @return
     */
    public function find_objects($path, $like = null)
    {
        $obj_list = array();
        $this->db->where('path', $path);
        $this->db->where('status !=', Safe_object_serializer_class::OBJ_STATUS_DELETED);
        if (isset($like)) $this->db->like('name', $like, 'after');
        $query = $this->db->get($this->db_table);
        foreach ($query->result() as $row) {
            $obj_list[$row->name] = array(
                'class' => $row->class,
                'data' => json_decode($row->data, true)
            );
        }
        //echo $this->db->last_query() . "<br />";
        return $obj_list;
    }
    /**
     * @brief
     *
     * @param[in out] $path
     * @param[in out] $like
     *
     * @return
     */
    public function find_deleted_objects($path, $like = null)
    {
        $obj_list = array();
        $this->db->where('path', $path);
        $this->db->where('status', Safe_object_serializer_class::OBJ_STATUS_DELETED);
        if (isset($like)) $this->db->like('name', $like, 'after');
        $query = $this->db->get($this->db_table);
        foreach ($query->result() as $row) {
            $obj_list[$row->name] = array(
                'class' => $row->class,
                'data' => json_decode($row->data, true)
            );
        }
        return $obj_list;
    }
    /**
     * @brief
     *
     * @param[in out] $path
     * @param[in out] $name
     *
     * @return
     */
    public function delete_object($path, $name = null)
    {
        $this->db->where('path', $path);
        if (isset($name)) $this->db->where('name', $name);
        $this->db->delete($this->db_table);
    }
    /**
     * @brief
     *
     * @param[in out] $path
     * @param[in out] $like_path
     * @param[in out] $type
     * @param[in out] $object_list
     *
     * @return
     */
    public function find_modified_objects($path = null, $like_path = true, $type = Safe_object_serializer_class::OBJ_TYPE_CONFIGURABLE, &$object_list = null)
    {
        $this->db->where('modified !=', 0);
        if (isset($path)) {
            if (true == $like_path) $this->db->like('path', $path, 'after');
            else $this->db->where(array(
                'path' => $path
            ));
        }
        if (isset($type)) {
            $this->db->where(array(
                'type' => $type
            ));
        }
        $q = $this->db->get($this->db_table);
        if ($q->num_rows()) {
            if (isset($object_list)) {
                $object_list = array();
                foreach ($q->rows() as $row) $object_list[] = $row->path . $row->name;
            }
        }
        return $q->num_rows();
    }
    /**
     * @brief Clear all modified objects
     *
     * @param[in out] $path
     * @param[in out] $recurse
     *
     * @return
     */
    public function clear_modified($path, $recurse, $type = Safe_object_serializer_class::OBJ_TYPE_CONFIGURABLE)
    {
        // Delete disposed objects
        $this->delete_object_with_status($path, $recurse, Safe_object_serializer_class::OBJ_STATUS_DELETED, $type);
        // update status and modified flags for other modified objects
        $this->update_object_with_status($path, $recurse, NULL, Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE, $type);
    }
    public function update_object_with_status($path, $recurse, $old_status, $new_status, $type = Safe_object_serializer_class::OBJ_TYPE_CONFIGURABLE)
    {
        // update status and modified flags for other modified objects
        //$this->db->where (array('modified !=' => 0));
        // If old_status not set, assume all
        if (isset($old_status)) $this->db->where('status', $old_status);
        if (isset($type)) $this->db->where('type', $type);
        if ($recurse) $this->db->like('path', $this->db->escape_like_str($path) , 'after');
        else $this->db->where('path', $path);
        if (Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE == $new_status) $update['modified'] = 0;
        else $update['modified'] = 1;
        $update['status'] = $new_status;
        $query = $this->db->update($this->db_table, $update);
    }
    /**
     * @brief Clear all object in path with specific status
     *
     * @param[in out] $path
     * @param[in out] $recurse
     * @param[in out] $status
     *
     * @return
     */
    public function delete_object_with_status($path, $recurse, $status, $type = Safe_object_serializer_class::OBJ_TYPE_CONFIGURABLE)
    {
        // Delete disposed objects
        $this->db->where('status', $status);
        if (isset($type)) $this->db->where('type', $type);
        if ($recurse) $this->db->like('path', $this->db->escape_like_str($path) , 'after');
        else $this->db->where('path', $path);
        $this->db->delete($this->db_table);
    }
    public function get_objects()
    {
        $objects = array();
        $q = $this->db->get($this->db_table);
        $objects = $q->result_array();
        return $objects;
    }
    public function get_object($id)
    {
        $object = array();
        $this->db->where('id', $id);
        $q = $this->db->get($this->db_table);
        if ($q->num_rows()) {
            $object = $q->row_array();
        }
        return $object;
    }
    public function remove_object($id)
    {
        $this->db->where('id', $id);
        $this->db->delete($this->db_table);
    }
    /**
     * @brief
     *
     * @return
     */
    public function next_insert_id()
    {
        $last_row = array();
        // Performance fix: with $this->db->get('safe_object') the whole table
        // safe_object is loaded in memory. If there is 1000 objects into the
        // table, they are all loaded in memory, just to get the next insert
        // id. With the following line we just load the last row, no matter how
        // many objects there are in the table
        $query = $this->db->order_by('id','desc')->get('safe_object',1);
        $last = $query->last_row('array');
        $id = $last['id'];
        return $id + 1;
    }
}
class Safe_object_class
{
    private $_path;
    private $_name;
    private $_description = '';
    private $_documentation = '';
    private $_class = "";
    private $_ha_active = "";
    private $_aggr_obj_type = array();
    protected $_properties = array();

    private static $_memory_cache = array();

    public function __construct($path, $name)
    {
        $this->_path = $path;
        $this->_name = $name;
        $this->_description = $name;
        $this->_class = get_class($this);
    }
    public function _get_aggregate_object_definition($type = null)
    {
        if($type !== null) {
            return $this->_aggr_obj_type[$type];
        } else {
            return $this->_aggr_obj_type;
        }
    }
    public function object_name()
    {
        return $this->_path . "/" . $this->_name;
    }
    public function class_name()
    {
        return $this->_class;
    }
    public function path()
    {
        return $this->_path;
    }
    public function set_path($path)
    {
        $this->_path = $path;
        return true;
    }
    /**
     * @brief Allow to set the name to the current object 
     * @param string The new name of the object
     */
    public  function set_name($name){
        $this->_name = $name;
    }
    public function name()
    {
        return $this->_name;
    }
    public function display_name()
    {
      return $this->name();
    }
    public function path_info() {
        $path =  substr($this->object_name(), 1);
        $info = array(
                'product' => null,
                'node' => null,
                'app_type' => null,
                'module' => null,
                'type' => null,
                'name' => null,
                'sub_type' => null,
                'sub_name' => null
                );

        list ($info['product'], $info['node'], $info['app_type'],
              $info['module'], $info['type'], $info['name'],
              $info['sub_type'], $info['sub_name'] ) = explode ( '/', $path );
        return $info;
    }
    public function is_configuration_modified($include_childs = true)
    {
        $ser = Safe_object_serializer_class::get_serializer();
        $modified_cnt = $ser->find_modified_objects($this->object_name() , $include_childs);
        return (0 != $modified_cnt);
    }
    public function configure($node=null)
    {
        return TRUE;
    }

    /**
     * @brief Return object description
     *
     * @return string - object description
     */
    public function description()
    {
        return $this->_description;
    }
    /**
     * @brief Set object decsription
     *
     * @param[in] $description - string
     *
     * @return
     */
    public function set_description($description)
    {
        $this->_description = $description;
    }
    public function set_properties($properties)
    {
        $this->_properties = $properties;
    }
    public function property($name, $default)
    {
        if($this->_properties[$name]){
            return $this->_properties[$name];
        }else{
            return $default;
        }
    }
    /**
     * @brief Return object Documentation
     *
     * Some documentation about object
     *
     * @return string - object summary
     */
    public function documentation()
    {
        return $this->_documentation;
    }
    /**
     * @brief Set object documentation
     *
     * @param[in] $documentation - string
     *
     * @return
     */
    public function set_documentation($documentation)
    {
        $this->_documentation = $documentation;
    }
    /**
     * Used to tell application configuration manager that this is an aggregated objects
     * @return array ( 'type' => object definition )
     */
    public function has_aggregate_objects($include_all=false, $internal=false)
    {
        $types = $this->_get_aggregate_object_definition();
        if(!$include_all) {
            // Remove configuration
            unset($types['configuration']);
            // Filter out service
            unset($types['service']);
        }
        if(!$internal){
            // Filter out controller specific parameters
            unset($types['controller']);
            // as well as for objects
            foreach($types as &$type) {
                unset($type['controller']);
            }
        }
        return $types;
    }
    public function get_aggregate_object_definition($type=null)
    {
        if($type) {
            return $this->_get_aggregate_object_definition($type);
        } else {
            return $this->has_aggregate_objects();
        }
    }
    /**
     * @brief Register aggregate object definition
     *
     * @param[in out] $type (string) object type (singular)
     * @param[in out] $definition (array)
     *
     * ['name']           => (string) Long name for display
     * ['description']    => (string) Object description
     * ['dynamic']        => (boolean) object can/can't be reloaded dynamically
     * ['base_path']      => (string) Base object path in database
     * ['class']          => (string) Object class
     * ['has_child']      => (boolean) Object can have children objects
     * ['methods']        => (array) Exposed object methods
     *                          4 fixed entries for CRUD ('create', 'retrieve',
     *                          'update' and 'delete') + control methods
     *                          entry (ie. anything not a CRUD entry is control).
     *                          Method name is resolved in the object
     *                          (reflection) based on following rules:
     *                            For CRUD - <method>_<object type>
     *                               function prototype:
     *                                  create - <obj/array error>  = create_<object type>($name = null, $data  = null);
     *                                  retrieve - <obj/array> = retrieve_<object type>($filter_name = null);
     *                                  update - <obj/array error> = update_<object type>($name, $data);
     *                                  delete - <true/array error> = delete_<object type>($name);
     *                            For control - ctl_<object type>_<method>
     *                               function prototype:
     *                                  <true/array error> = ctl_<object type>_<method>($name, $params = null);
     *                          Method entry format is:
     *                          [method] => array(
     *                             ['name'] => (string) Long name for display
     *                             ['description'] => (string) Method description
     *                             ['request'] => (string) REST request method allowed (GET, POST, PUT or DELETE)
     *                             (optional) ['param class'] => (string) name of the configurable object containing
     *                                                       parameter descrition (mostly for control methods)
     *
     * @return
     */
    public function register_aggregate_object($type, $definition = null)
    {
        if(isset($definition)) {
            if(!is_array($definition)) {
                // Old style name
                $definition = array(
                    'name' => $definition);
            }
        }
        // Validate some mandatory entries
        if(!$definition['name']){
            $definition['name'] = $type;
        }
        if(!$definition['description']){
            $definition['description'] = $definition['name'];
        }
        if(!$definition['dynamic']){
            $definition['dynamic'] = false;
        }
        if(!$definition['base_path'] && false !== $definition['base_path']){
            $definition['base_path'] = $this->object_name().'/'.$type;
        }
        if(!isset($definition['configurable'])){
            $definition['configurable'] = true;
        }
        if(!isset($definition['pagination'])){
            $definition['pagination'] = false;
        }
        if(!isset($definition['global_methods'])){
            $definition['global_methods'] = false;
        }
        if(isset($definition['methods']) && is_array($definition['methods'])){
            foreach($definition['methods'] as $method){
                if(isset($method['scope']) && ($method['scope'] == 'global' || $method['scope'] == 'both')){
                    $definition['global_methods'] = true;
                }
            }
        }

        // ...
        /*
        echo '<pre>';
        echo 'Obj='.$type;
        print_r($definition);
        echo '</pre>';
        */

        $this->_aggr_obj_type[$type] = $definition;
    }
    public function update_aggregate_object($type, $name, $obj)
    {
        if($this->_agg_objects_cache[$type]){
            $this->_agg_objects_cache[$type]['objects'][$name] = $obj;
        }
    }
    /**
     * Returns an array of all aggregated objects for a specific module
     * Optionally return all objects regardless of their status
     * @return $array
     *                			Array of aggregated configs
     */
    private $_agg_objects_cache = array();
    public function get_aggregate_objects($type, $all = false, $params = array())
    {
        $definition = $this->_get_aggregate_object_definition($type);
        // Do we have base path ?
        if($definition['base_path']) {
            // Check objects cache (or requesting all)
            if(!$this->_agg_objects_cache[$type] || $this->_agg_objects_cache[$type]['all'] != $all){
                $objects = array();
                // Re-create object list
                $_objects = Safe_object_serializer_class::get_serializer()->find_objects($definition['base_path']);
                foreach ($_objects as $k => $v) {
                    $obj = new $v['class']($this->node(), $definition['base_path'], $k);
                    if(!$obj->is_configured()){
                        $obj->configure();
                    }
                    $obj->synch();
                    $objects[$k] = $obj;
                }
                // Check if we want the deleted objects as well
                if($all){
                    // Re-create object list
                    $_deleted = Safe_object_serializer_class::get_serializer()->find_deleted_objects($definition['base_path']);
                    foreach ($_deleted as $k => $v) {
                        $obj = new $v['class']($this->node(), $definition['base_path'], $k);
                        if(!$obj->is_configured()){
                            $obj->configure();
                        }
                        $obj->unserialize();
                        $objects[$k] = $obj;
                    }
                }
                // Store in cache
                $this->_agg_objects_cache[$type]['objects'] = $objects;
                $this->_agg_objects_cache[$type]['all'] = $all;
            }
            // Use cache
            return $this->_agg_objects_cache[$type]['objects'];
        } else if (method_exists($this, 'api_retrieve_'.$type)) {
            return call_user_func(array(
                $this,
                'api_retrieve_'.$type
            ), null, $params);
        } else if (method_exists($this, $type)) {
            return call_user_func(array(
                $this,
                $type
            ));
        }
        //safe_log(SAFE_LOG_ERROR, "WARNING - Cannot retrieve aggregate_object ${type} for module ".$this->name());
        return array();
    }
    /**
     * Returns an array of all aggregated objects name list for a specific module
     * Optionally return all objects regardless of their status
     * @return $array
     *                			Array of aggregated obj names
     */
    private $_agg_object_name_list_cache = array();
    public function get_aggregate_object_name_list($type, $all = false, $params = array())
    {

        $definition = $this->_get_aggregate_object_definition($type);
        // Do we have base path ?
        if($definition['base_path']) {
            // Check objects cache (or requesting all)
            if(!$this->_agg_object_name_list_cache[$type] || $this->_agg_object_name_list_cache[$type]['all'] != $all){
                $object_names = array();
                // Re-create object list
                $_objects = Safe_object_serializer_class::get_serializer()->find_objects($definition['base_path']);
                foreach ($_objects as $k => $v) {
                    if (isset($v['data']['display-name']) && $v['data']['display-name'] != '') {
                        $object_names[$k] = $v['data']['display-name'];
                    }
                    else {
                      $object_names[$k] = $k;
                    }
                }
                // Check if we want the deleted objects as well
                if($all){
                    // Re-create object list
                    $_deleted = Safe_object_serializer_class::get_serializer()->find_deleted_objects($definition['base_path']);
                    foreach ($_deleted as $k => $v) {
                        $object_names[$k] = $k;
                    }
                }
                // Store in cache
                $this->_agg_object_name_list_cache[$type]['names'] = $object_names;
                $this->_agg_object_name_list_cache[$type]['all'] = $all;
            }
            // Use cache
            return $this->_agg_object_name_list_cache[$type]['names'];
        }
        //safe_log(SAFE_LOG_ERROR, "WARNING - Cannot retrieve aggregate_object ${type} names for module ".$this->name());
        return array();
    }
    public function get_aggregate_object($type, $name, $all=false)
    {
        // Request a specific object by name (and type)
        // Lazy implementation ask for all objs and return by key
        $objs = $this->get_aggregate_objects($type, $all);
        return $objs[$name];
    }
    public function get_aggregate_objects_keys($type, $all = false)
    {
        // Default lazy implementation, ask for objects than extract keys
        $objs = $this->get_aggregate_objects($type, $all);
        if(is_array($objs)){
            return array_keys($objs);
        }else{
            if(is_object($objs)){
                $tmp_data[] = $objs->name();
                return $tmp_data;
            }else{
                return false;
            }
        }
    }

    /**
     * @brief clear objs in cache (so it can be regenerate from db )
     *
     * @param[in out] $type
     *
     * @return
     */
    protected function clear_cache($type = null)
    {
        //clear objs and objs name list in cache
        if ($type){
            if($this->_agg_objects_cache[$type]){
                unset($this->_agg_objects_cache[$type]);
            }
            if($this->_agg_object_name_list_cache[$type]){
                unset($this->_agg_object_name_list_cache[$type]);
            }
        }else{
            $this->_agg_objects_cache = array();
            $this->_agg_object_name_list_cache = array();
        }
    }
    /**
     * @brief Check if specified aggregate object can be created
     *
     * @param[in out] $_type
     * @param[in out] $name
     * @param[in out] $reason)
     *
     * @return
     */
    public function can_create_aggregate_object($type, $name, &$reason="")
    {
        // Default implementation - Check object name exists
        $obj_keys = $this->get_aggregate_objects_keys($type);
        //Add case-insensitive object name validation
        if(safe_in_arrayi($name, $obj_keys)){
            $reason = "Already exists";
            return false;
        }else{
            // user can not create obj name with '__' or start with 'auto_'
            if (preg_match('/__/i', $name) ||
                preg_match('/^auto_/i', $name)
                ) {
                $reason = " '{$name}' is not a valid name.";
                return false;
            }
        }
        return true;
    }
    public function create_aggregate_object($type, $name='default', &$output=null, $data = null)
    {
        if('configuration'==$type && $this->config()) {
            return $this->config();
        }
        $obj = null;
        $definition = $this->_get_aggregate_object_definition($type);

        // Create allowed ?
       // if($definition['methods']['create'])
        {
            $create_name = 'api_create_'.$type;
            if(method_exists($this, $create_name)) {
                $obj =  call_user_func_array(
                    array(
                    $this,
                    $create_name),
                    array($name , $data , &$output)
                );
            }
        }
        return $obj;
    }
    public function delete_aggregate_object($type, $name='default')
    {
        $definition = $this->_get_aggregate_object_definition($type);
        // delete allowed ?
        if($definition['methods']['delete']){
            $delete_name = 'api_delete_'.$type;
            if(method_exists($this, $delete_name)) {
                return  call_user_func_array(
                        array(
                                $this,
                                $delete_name),
                        array($name)
                );
            }
        }
        return false;
    }
    public function get_aggregate_object_method_params($type, $method)
    {
        $definition = $this->_get_aggregate_object_definition($type);
        if($definition['methods'][$method]){
            $method_name = 'api_'.$method.'_'.$type.'_parameters';
            if(method_exists($this, $method_name)) {
                $param =  call_user_func_array( array( $this, $method_name), array() );
                return $param;
            }
        }
        return false;
    }


    /**
     * @brief
     *
     * @param[in out] $type
     *
     * @return
     */
    public function aggregate_object_name($type)
    {
        $definition = $this->_get_aggregate_object_definition($type);
        if($definition)
            return $definition['name'];
        else
            return $type;
    }
    /**
     * @brief Return object type base path
     *
     * @param[in out] $type
     *
     * @return
     */
    public function aggregate_object_base_path($type)
    {
        $definition = $this->_get_aggregate_object_definition($type);
        if($definition)
            return $definition['base_path'];
        else
            return null;
    }
    /**
     * @brief new_fields
     *
     * @param[in out] $config_manager
     * @param[in out] $type
     *
     * Return
     *
     * @return
     */
    public function new_fields(&$config_manager, $type)
    {
        $obj_def = $this->get_aggregate_object_definition($type);
        $label = 'New ' . $obj_def['name'];
        $keys = $this->get_aggregate_objects_keys($type);
        $number = count($keys)+1;
        $new_name = ucfirst($type) . $number;
        while(in_array($new_name, $keys)){
            $number++;
            $new_name = ucfirst($type) . $number;
        };
        $config_manager->add_field('profile-name', $label, 'string', $new_name, 50);
        $config_manager->set_field_rules('profile-name', 'required|alpha_dash');
    }
    public function create_obj_name($data, $type)
    {
        //using the data to create  obj name
        if(isset($data['profile-name'])){
            return $data['profile-name'];
        }else{
            return false;
        }
    }
    public function build_unique_name($type)
    {
        return Safe_object_serializer_class::build_unique_name($type);
    }

    /**
     * @brief Populate auditor with audit point for object
     *
     * @param[in out] $auditor
     *
     * @return
     */
    public function get_audit_points(&$audit_points)
    {
        return true;
    }
    public function download(){
        header('HTTP/1.0 501 Not Implemented');
        exit();
    }
    /**
     * @brief set_cache save data to tmp db record
     *
     * @return
     */
    protected function set_data_cache($name, $data)
    {
        $cache_data = array();
        if($name){
            $ser = Safe_object_serializer_class::get_serializer();
            $cache_data['data'] = $data;
            $ser->store_object('/tmp', $name, 'none', $cache_data, 'tmp');
        }
    }
    /**
     * @brief get_cache get data from tmp db record
     *
     * @return
     */
    protected function get_data_cache($name, &$data)
    {
        if($name){
            $cache_data = array();
            $ser = Safe_object_serializer_class::get_serializer();
            $ser->retrieve_object('/tmp', $name, $cache_data);
            if($cache_data){
                //have cache
                $data = $cache_data['data'];
                return true;
            }else{
                $data = array();
                return false;
            }
        }
    }

    protected function get_memory_data_cache($name, &$data, $populate = true)
    {
        if($name) {
            if (isset(Safe_object_class::$_memory_cache[$name])) {
                $data = Safe_object_class::$_memory_cache[$name];
                return true;
            } else {
                if (!$populate) {
                    return false;
                }
                if ($this->get_data_cache($name, $data)) {
                    Safe_object_class::$_memory_cache[$name] = $data;
                    return true;
                } else {
                    return false;
                }
            }
        }
    }

    protected function set_memory_data_cache($name, $data)
    {
        if($name) {
            Safe_object_class::$_memory_cache[$name] = $data;
        }
    }

    protected function clear_global_messages_cache(&$msg_list, $module, $cache_tag)
    {
        $update_cache = false;
        $cache = array();

        if (!$this->get_memory_data_cache('new_global_messages', $cache, false)) {
            $this->get_memory_data_cache('global_messages', $cache);
        }

        // clear cache
        foreach ($cache as $key => $value) {
            if ($value['module'] == $module && $value['cache_tag'] == $cache_tag) {
                unset($cache[$key]);
                $update_cache = true;
            }
        }
        // clear broadcast's msg_list
        if($msg_list){
            foreach ($msg_list as $key => $value) {
                if ($value['module'] == $module && $value['cache_tag'] == $cache_tag) {
                    unset($msg_list[$key]);
                }
            }
        }
        if ($update_cache) {
            $this->set_memory_data_cache('new_global_messages', $cache);
        }
    }
    protected function flush_global_messages_cache()
    {
        $data = array();
        if (!$this->get_memory_data_cache('new_global_messages', $data, false)) {
            if (!$this->get_memory_data_cache('global_messages', $data, false)) {
                return; // nothing to flush, just return
            }
        }
        $this->set_data_cache('global_messages', $data);
        $this->set_memory_data_cache('global_messages', $data);
        $this->set_memory_data_cache('new_global_messages', $data);
    }

    protected function get_global_messages_cache($module, $cache_tag)
    {
        $cache = array();
        if (!$this->get_memory_data_cache('new_global_messages', $cache, false)) {
            $this->get_memory_data_cache('global_messages', $cache);
        }
        foreach ($cache as $key => $value) {
            if ($value['module'] == $module && $value['cache_tag'] == $cache_tag) {
                return $value;
            }
        }
    }
    /**
     * @brief remove_cache remove cache from tmp db record
     *
     * @return
     */
    protected function remove_data_cache($name, $delete = false)
    {
        if($name){
            $ser = Safe_object_serializer_class::get_serializer();
            if($delete){
                $ser->delete_object('/tmp', $name);
            }else{
                $ser->store_object('/tmp', $name, 'none', array(), 'tmp');
            }
        }
    }
    protected function clear_list_cache()
    {
        //clear check_list,modified_object_list cache in tmp db obj
        //when update or delete objs from web ui or rest api
        $this->remove_data_cache('check_list');
        $this->remove_data_cache('modified_object_list');
    }
    /**
     * @brief validate field value using form validation rules
     *
     * @return boolean
     */
    public function validate_value($field_data, $rules, &$output=null)
    {
        $validation_obj = new CI_Form_validation();
        $result = $validation_obj->run_value_validation($field_data, $rules);
        if($result === true){
            return true;
        }else{
            if($result === false){
                return false;
            }
            $output = $result;
            return false;
        }
    }
    /**
     * @brief return HA mode activity
     *
     * @return boolean
     */
    public function ha_mode_active(){
        $_objects = Safe_object_serializer_class::get_serializer()->find_objects('/NSC/local/software/cluster');
        foreach($_objects as $k => $v){
            if($v['data']['mode'] == 'active-passive'){
                return true;
            }
        }
        return false;
    }
}
abstract class Safe_serializable_object_class extends Safe_object_class
{
    const SYNC_DB = 'db';
    const SYNC_MEM = 'mem';
    private $_sync_strategy = Safe_serializable_object_class::SYNC_DB;
    private $_status = Safe_object_serializer_class::OBJ_STATUS_NEW;
    private $_id = 0;
    private $_type = Safe_object_serializer_class::OBJ_TYPE_CONFIGURABLE;
    private $_reuse = true;
    static private $_serializer = null;
    public function __construct($path, $name, $sync_strategy = Safe_serializable_object_class::SYNC_DB)
    {
        parent::__construct($path, $name);
        $this->_sync_strategy = $sync_strategy;
    }
    protected function _set_sync_strategy($sync_strategy)
    {
        $this->_sync_strategy = $sync_strategy;
    }
    /* Set re-use object strategy:
     * true = keep same record on delete/update just change status
     * false = change old obj name and set to deleted, then create new one
     */
    protected function _set_reuse($reuse=true)
    {
        $this->_reuse = $reuse;
    }
    /**
     * @brief Synchronize serializable object according to startegy
     *
     * @return
     */
    public function synch()
    {
        // Get current data to check if changed
        $data = $this->_get_serialized_data();
        // Try to retrieve object
        if (TRUE != $this->unserialize() || Safe_object_serializer_class::OBJ_STATUS_DELETED == $this->_status) {
            // Not yet stored, store it
            $this->_set_unserialized_data($data);
            $this->serialize();
        } else {
            // Check if retrieved data is same as the one supplied at constructor
            // and sync strategy set to memory then update db with new content
            $db_data = array();
            $this->_serializer()->retrieve_object($this->path() , $this->name() , $db_data);
            if ($this->_sync_strategy == Safe_serializable_object_class::SYNC_MEM && $data != $db_data) {
                //safe_log(SAFE_LOG_INFO, 'synch - flush - ' . $this->object_name() . ' Status = ' . $this->_status);
                $this->_status = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
                // Need to update cache
                $this->_set_unserialized_data($data);
                $this->_serializer()->store_object($this->path() , $this->name() , $this->class_name() , $data, $this->_type, $this->_status, $this->_id);
                //$this->serialize();
            }
        }
    }
    private function _serializer()
    {
        Safe_serializable_object_class::$_serializer = Safe_object_serializer_class::get_serializer();
        return Safe_serializable_object_class::$_serializer;
    }
    public function serialize()
    {
        //safe_log(SAFE_LOG_INFO, 'serialize - flush - ' . $this->object_name() . ' - Status = ' . $this->_status);
        // Get the data to serialize
        $data = $this->_get_serialized_data();
        if (!empty($data)) {
            // Invoke serializer
            return $this->_serializer()->store_object($this->path() , $this->name() , $this->class_name() , $data, $this->_type, $this->_status, $this->_id, $this->_reuse);
        }
        // No data to serialize
        return FALSE;
    }
    public function unserialize()
    {
        // Invoke serializer
        $data = array();
        $obj_class = '';
        if ($this->_serializer()->retrieve_object($this->path() , $this->name() , $data, $this->_status, $this->_id, $obj_class, $this->_type)) {
            $this->_set_unserialized_data($data);
            return TRUE;
        }
        return FALSE;
    }
    /**
     * @brief Return object status (from DB)
     *         See Safe_object_serializer_class::OBJ_STATUS_*
     *
     * @return object status
     */
    public function status()
    {
        return $this->_status;
    }
    /**
     * @brief Returns object id
     *
     * @return Object id (auto inc id from sqlite)
     */
    public function id()
    {
        return $this->_id;
    }
    public function type()
    {
        return $this->_type;
    }
    public function set_type($type)
    {
        $this->_type = $type;
    }
    /**
     * @brief Indicates if cofigurable object is modified
     *
     * @return true if modified
     */
    public function is_modified()
    {
        return ($this->status() != Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE);
    }
    /**
     * @brief Clear modified object flag and status
     *
     * @return
     */
    public function clear_modified()
    {
        $this->clear_list_cache();
        return $this->_serializer()->mark_object_uptodate($this->path() , $this->name());
    }
    public function clear_deleted()
    {
        $this->clear_list_cache();
        return $this->_serializer()->delete_object($this->path() , $this->name());
    }
    public function mark_modified()
    {
        $this->clear_list_cache();
        $this->_status = Safe_object_serializer_class::OBJ_STATUS_MODIFIED;
        return $this->_serializer()->mark_object_modified($this->path() , $this->name());
    }
    /**
     * @brief Check if the object can be deleted
     *         This function should be implemented in extended objects
     *         to validate any dependencies before deleting an object
     *
     * @return
     *         true - Object can be deleted
     *         false - Object has dependencies
     *
     */
    public function can_dispose(&$reason = NULL)
    {
        return TRUE;
    }
    /**
     * @brief Check if the object can be modified
     *         This function should be implemented in extended objects
     *         to validate any dependencies before modify an object
     *
     * @return
     *         true - Object can be modified
     *         false - Object can not modified
     *
     */
    public function can_modify(&$reason = null)
    {
        return true;
    }
    /**
     * @brief Check if the object is readonly
     *         This function should be implemented in extended objects
     *         to validate it can be edit or deleted
     *
     * @return
     *         true - Object can not be edit or deleted
     *         false - default
     */
    public function is_readonly(&$reason = NULL)
    {
        return false;
    }
    public function can_edit(&$reason = NULL)
    {
        if($this->is_readonly($reason)){
            return false;
        }
        return true;
    }
    /**
     * @brief Delete an object
     *         Mark the object as deleted
     *
     * @return
     *         true on success
     */
    public function dispose()
    {
        //clear check_list,modified_object_list cache
        $this->clear_list_cache();
        $rc = $this->_serializer()->dispose_object($this->path() , $this->name(), $this->_reuse);
        if($rc){
            $this->_status = Safe_object_serializer_class::OBJ_STATUS_DELETED;
        }
        return $rc;
    }
    public function display_name()
    {
      if ($this->get_data_value('display-name'))
        return $this->get_data_value('display-name');
      return $this->name();
    }
    //
    // Return a php array of the data that need to be serialized
    abstract protected function _get_serialized_data();
    // Set the php array of retrieved data
    abstract protected function _set_unserialized_data($data);
    /**
     * @brief get_data_values
     *
     * for compatibility on objects not configurable
     *
     * @return
     */
    public function get_data_values()
    {
        return $this->_get_serialized_data();
    }
}
class Safe_configurable_object_class extends Safe_serializable_object_class
{
    private static $_field_data = array();
    private static $_field_status = array();
    /**
     * @var _group array
     */
    private static $_group = array();
    private static $_aggr_obj_type_data = array();

    private $_data = array();
    private $_field_data_key = null;
    private $_removed_field = array();
    private $_user_rule_msgs = array();
    protected $_node = null;
    private $_form = null;
    private $_group_condition = array();
    private $_default_category = null;
    private $_default_category_setting = 'expanded';
    private $_obj_message = array();
    /**
     * @brief
     *
     * @param[in out] $path
     * @param[in out] $name
     *
     * @return
     */
    public function __construct($path, $name, $node=null)
    {
        $this->_node = $node;
        parent::__construct($path, $name);

    }
    public function __destruct()
    {//remove the tmp obj field and group data from field_data
        $key = $this->_get_field_key();
        $path = $this->path();
        if(substr($key, 0, 9) == '/__tmp__/' || $path == ''){
            unset(self::$_field_data[$key]);
            unset(self::$_field_status[$key]);
            unset(self::$_group[$key]);
            unset(self::$_aggr_obj_type_data[$key]);
        }
    }
    public function _get_aggregate_object_definition($type = null)
    {
        $definition = parent::_get_aggregate_object_definition();
        if(empty($definition)){
            if($this->_configure_locked()){
                $key = $this->_get_field_key();
                if(isset(self::$_aggr_obj_type_data[$key])){
                    foreach(self::$_aggr_obj_type_data[$key] as $tmp_type => $data){
                        //generate 'base_path'
                        if(isset($data['base_path']) && $data['base_path']!==false) {
                            $data['base_path'] = $this->object_name() . "/" .$tmp_type;
                        }
                        parent::register_aggregate_object($tmp_type, $data);
                    }
                    return parent::_get_aggregate_object_definition($type);
                }
            }
            return array();
        }else{
            if($type !== null) {
                return $definition[$type];
            } else {
                return $definition;
            }
        }
    }
    /**
     * @brief Configure the object by adding fields
     *
     * @return
     */
    public function configure($node=null)
    {
        return TRUE;
    }
    public function prepare_edit()
    {
        //discard the lock, allow the obj to modify the config
        $this->discard_lock();
    }
    public function set_obj_message($key = '', $message = ''){
        if(empty($key)){
            return false;
        }else{
            $this->_obj_message[$key] = $message;
        }
        return true;
    }
    public function get_obj_message($key = ''){
        if(!empty($key) && isset($this->_obj_message[$key])){
                return $this->_obj_message[$key];
        }
        return '';
    }
    /**
     * @brief Return node object
     *
     * @return
     */
    public function node()
    {
        return $this->_node;
    }
    /**
     * @brief
     *
     * @return
     */
    public function enable_disable_enum($string_value=true)
    {
        if($string_value){
            return array(
                'true' => 'Enable',
                'false' => 'Disable'
            );
        }else{
            return array(
                '1' => 'Enable',
                '0' => 'Disable'
            );
        }
    }
    /**
     * @brief return the key of $_field_data, by default using the obj path() as key
     *
     *
     * @return
     */
    private function _get_field_key()
    {
        if($this->_field_key === null){
            $key = get_class($this);
            if($key == 'Safe_configurable_object_class'){
                //generate a uniq key for Safe_configurable_object_class obj
                $key = '/__tmp__/'.md5(time() . rand(1,2000));
            }
            $this->_field_key = $key;
        }
        return $this->_field_key;
    }
    /**
     * @brief return all this obj configured field data in $_field_data
     *
     *
     * @return
     */
    private function _get_fields()
    {
        $tmp_field_data = array();
        $key = $this->_get_field_key();
        if(isset(self::$_field_data[$key])){
            $tmp_field_data = self::$_field_data[$key];
        }
        if($this->_removed_field){
            foreach($this->_removed_field as $key => $value){
                //filter out marked as removed fields
                unset($tmp_field_data[$key]);
            }
        }
        return $tmp_field_data;
    }
    /**
     * @brief return obj configured field data by field name
     *
     * @param[in out] $name
     *
     * @return
     */
    private function _get_field_by_name($name)
    {
        $tmp_field_data = $this->_get_fields();
        return $tmp_field_data[$name];
    }
    /**
     * @brief check  obj configure status, locked or not(only the first obj can configure it).
     *
     *
     * @return
     */
    private $_lock_flag = null;
    protected function discard_lock(){
        $this->_lock_flag = false;
    }
    private function _configure_locked(){
        if($this->_lock_flag === null){
            $field_key = $this->_get_field_key();
            if(isset(self::$_field_status[$field_key])){
                //use the object_name as lock, same class may have same obj name
                if(self::$_field_status[$field_key]['configure_obj_name'] == $this->object_name()){
                    $this->_lock_flag = false;
                }else{
                    $this->_lock_flag = true;
                }
            }else{
                $this->_lock_flag = false;
                //initialize the field_status with current obj name
                self::$_field_status[$field_key]['configure_obj_name'] = $this->object_name();
            }
        }
        return $this->_lock_flag;
    }
    public function is_configured(){
        return $this->_configure_locked();
    }
    /**
     * @brief set field configure data
     *
     * @param[in out] $name
     * @param[in out] $param_name
     * @param[in out] $param_value
     *
     * @return
     */
    private function _set_field_param($name, $param_name, $param_value)
    {
        $field_key = $this->_get_field_key();
        if($this->_configure_locked()){
            //check if the data different with the init data
            //for now just log
            /*
            $class_name = get_class($this);
            if(isset(self::$_field_data[$field_key][$name])){
                $tmp_field_old_data = self::$_field_data[$field_key][$name];
                if(isset($tmp_field_old_data[$param_name])){

                    if($tmp_field_old_data[$param_name] !== $param_value){
                        if($tmp_field_old_data['type'] == 'dropdown'){
                            //dropdown will set the rule twice
                        }else{
                            safe_log(SAFE_LOG_DEBUG, "{$class_name}::[{$name}]::[{$param_name}]::try to change value [{$tmp_field_old_data[$param_name]}] => [{$param_value}]");
                        }
                    }
                }else{
                    safe_log(SAFE_LOG_DEBUG, "{$class_name}::[{$name}]::try to add param: [{$param_name}] => {$param_value}");
                }
            }else{
                safe_log(SAFE_LOG_DEBUG, "{$class_name}:: try to add field:[{$field_key}]:: [{$name}]:: [{$param_name}] => {$param_value}");
            }*/
        }else{
            if(isset(self::$_field_data[$field_key][$name]) || $param_name == 'field'){
                self::$_field_data[$field_key][$name][$param_name] = $param_value;
                //check if marked as removed before
                if(isset($this->_removed_field[$name])){
                    unset($this->_removed_field[$name]);
                }
                return true;
            }
        }
        return false;
    }
    /**
     * @brief unset field configure data
     *
     * @param[in out] $name
     * @param[in out] $param_name
     *
     * @return
     */
    private function _unset_field_param($name, $param_name)
    {//for composite_layout to merge helps
        if($this->_configure_locked()){

        }else{
            unset(self::$_field_data[$this->_get_field_key()][$name][$param_name]);
        }
        return true;
    }
    /**
     * @brief add obj data
     *
     * @param[in out] $name
     * @param[in out] $param_name
     *
     * @return
     */
    private function _add_obj_data($name, $default)
    {
        $this->_data[$name] = $default;
        return true;
    }
    /**
     * @brief get obj data by field name
     *
     * @param[in out] $name
     * @param[in out] $param_name
     *
     * @return
     */
    private function _get_obj_data($name)
    {
        if(isset($this->_data[$name])){
            return $this->_data[$name];
        }else{
            $tmp_data = $this->_get_field_by_name($name);
            return $tmp_data['default'];
        }
    }
    /**
     * @brief merge obj data with the field configure info
     *
     *
     * @return
     */
    private function _merge_field_data()
    {
        $temp_data = $this->_get_fields();
        foreach($this->_data as $k => $v){
            $temp_data[$k]['default'] = $v;
        }
        return $temp_data;
    }
    /**
     * @brief add field info and default obj data
     *
     * @param[in out] $name
     * @param[in out] $label
     * @param[in out] $type
     * @param[in out] $default
     * @param[in out] $size
     *
     * @return
     */
    public function add_field($name, $label, $type, $default, $size = null)
    {
        $this->_set_field_param($name, 'field' , $name);
        $this->_set_field_param($name, 'label' , $label);
        $this->_set_field_param($name, 'type' , $type);
        $this->_set_field_param($name, 'default' , $default);

        if (isset($size)) {
            $this->_set_field_param($name, 'size' , $size);
        }
        //set defualt category
        if($this->_default_category != null){
            $this->set_field_category($name , $this->_default_category, $this->_default_category_setting, false);
        }
        return true;
    }
    /**
     * @brief set field info
     *
     * @param[in out] $new_field
     *
     * @return
     */
    public function set_field($new_field)
    {//for now only used in dialplan module
        if(isset($new_field['field']) && isset($new_field['default'])){
            $name = $new_field['field'];
            $this->_add_obj_data($name, $new_field['default']);
            if($this->_set_field_param($name, 'field' , $name)){
                //copy field settings to this obj
                foreach($new_field as $key => $value){
                    $this->_set_field_param($name, $key , $value);
                }
                return true;
            }
        }
        return false;
    }
    /**
     * @brief unset field info
     *
     * @param[in out] $new_field
     *
     * @return
     */
    public function unset_field($field_name)
    {//this now only used in dialplan module
        //add flag , mark this field was removed in this obj
        $this->_removed_field[$field_name] = true;
        unset($this->_data[$field_name]);
        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $label
     * @param[in out] $type
     * @param[in out] $default
     * @param[in out] $enum_array
     *
     * @return
     */
    public function add_enum_field($name, $label, $type, $default, $enum_array)
    {
        if(isset($enum_array[''])) {
        /*
            throw new Exception("Field ".$name." enumeration contains empty key <".implode(',',array_keys($enum_array).">"));
            safe_log(SAFE_LOG_DEBUG, "Field ".$name." enumeration contains empty key <".implode(',',array_keys($enum_array).">"));
         */
            // Fixing enum
            $fixed = array();
            foreach($enum_array as $k => $v) {
                if('' == $k){
                    $k = '__empty__';
                }
                $fixed[$k] = $v;
            }
            $enum_array = $fixed;
        }
        if('' == $default){
            $default = '__empty__';
            if(!$enum_array[$default]){
                $keys = array_keys($enum_array);
                $default = $keys[0];
            }
        }
        if (strtolower($type) == "dropdown" || strtolower($type) == "radio") {
            if (true == $this->add_field($name, $label, $type, $default)) {
                $this->_set_field_param($name, 'value' , $enum_array);
                // Add field rule for enumeration
                $this->set_field_rules($name, '');
                $this->_set_field_param($name, 'rules' , 'required|in_list[' . implode(',', array_keys($enum_array)) . ']');
                // Add ha field rule for enumeration
                $this->set_ha_field_rules($name, '');
                $this->_set_field_param($name, 'ha_rules' , 'required|in_list[' . implode(',', array_keys($enum_array)) . ']');
                return true;
            }
        }
    }
    public function add_textarea_field($name, $label, $type, $default, $rows, $cols)
    {
      if (strtolower($type) == "textarea") {
        if ($this->add_field($name, $label, $type, $default)) {
          $this->_set_field_param($name, 'rows', $rows);
          $this->_set_field_param($name, 'cols', $cols);
          $this->_set_field_param($name, 'placeholder', "paste the access key here");
          return true;
        }
      }
    }
    public function add_upload_field($name, $label, $type, $default, $size = null, $no_textbox = false)
    {
        if (strtolower($type) == "upload") {
            $this->add_field($name, $label, $type, $default, $size);
            if ($no_textbox) $this->_set_field_param($name, 'no_box' , $no_textbox);
            return true;
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $label
     * @param[in out] $type
     * @param[in out] $default
     * @param[in out] $enum_array
     *
     * @return
     */
    public function add_multiple_field($name, $label, $type, $default = array() , $enum_array, $size = null)
    {
        if (strtolower($type) == "inline_checkbox"){
            $type = "checkbox";
            $inline = true;
        }else{
            $inline = false;
        }
        if (strtolower($type) == "sortable_listbox"){
            $type = "listbox";
            $sortable = true;
        }else{
            $sortable = false;
        }
        if (strtolower($type) == "checkbox" || strtolower($type) == "listbox") {
            $tmp_rs = $this->add_field($name, $label, $type, $default, $size);
            $this->_set_field_param($name, 'value', $enum_array);
            if($inline) {
                $this->_set_field_param($name, 'inline', 'true');
            }
            if($sortable) {
                $this->_set_field_param($name, 'sortable', 'true');
            }
            $this->set_field_rules($name, '');
            $this->set_ha_field_rules($name, '');
            if(is_array($enum_array)) {
                $this->_set_field_param($name, 'rules', 'is_array|in_list[' . implode(',', array_keys($enum_array)).']');
                $this->_set_field_param($name, 'ha_rules', 'is_array|in_list[' . implode(',', array_keys($enum_array)).']');
            }
        }
        return true;
    }

    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $rules
     *
     * @return
     */
    public function set_field_rules($name, $rules)
    {
        $field_data = $this->_get_field_by_name($name);
        if ($rules != 'disabled' && in_array($field_data['type'],array("dropdown","radio","checkbox","listbox"))) {
            // Ensure enum check is added to rule
            if(in_array($field_data['type'],array("dropdown","radio"))){
                $rules.= '|required';
            }
            if(in_array($field_data['type'],array("checkbox","listbox"))){
                $rules.= '|is_array';
            }
            $rules.= '|in_list[' . implode(',', array_keys($field_data['value'])) . ']';
        }
        return $this->_set_field_param($name, 'rules', $rules);
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $rules
     *
     * @return
     */
    public function set_ha_field_rules($name, $rules)
    {
        $field_data = $this->_get_field_by_name($name);
        if ($rules != 'disabled' && in_array($field_data['type'],array("dropdown","radio","checkbox","listbox"))) {
            // Ensure enum check is added to rule
            if(in_array($field_data['type'],array("dropdown","radio"))){
                $rules.= '|required';
            }
            if(in_array($field_data['type'],array("checkbox","listbox"))){
                $rules.= '|is_array';
            }
            $rules.= '|in_list[' . implode(',', array_keys($field_data['value'])) . ']';
        }
        return $this->_set_field_param($name, 'ha_rules', $rules);
    }
    public function set_field_disabled($name, $is_disabled = TRUE)
    {
        return $this->_set_field_param($name, 'disabled', (TRUE == $is_disabled) ? "disabled" : "enabled");
    }
    public function set_field_readonly($name, $readonly = 'readonly')
    {
        return $this->_set_field_param($name, 'readonly', $readonly);
    }
    public function set_field_style($name,$style='')
    {
        return $this->_set_field_param($name, 'style', $style);
    }
    public function set_field_category($name, $category, $setting = 'expanded', $set_as_default = false)
    {
        $this->_set_field_param($name, 'category_setting', $setting);
        if($set_as_default == true){
            $this->set_default_category($category, $setting);
        }
        return  $this->_set_field_param($name, 'category', $category);
    }
    public function set_default_category($category, $setting = 'expanded')
    {
        $this->_default_category = $category;
        $this->_default_category_setting = $setting;
    }
    public function get_default_category()
    {
        return $this->_default_category;
    }
    public function get_default_category_setting()
    {
        return $this->_default_category_setting;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $value
     *
     * @return
     */
    public function set_field_value($name, $value)
    {
        return $this->_set_field_param($name, 'value', $value);
    }
    public function set_field_onchange($name, $value)
    {
        return $this->_set_field_param($name, 'onchange', $value);
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $default
     *
     * @return
     */
    public function set_field_default($name, $default)
    {
        return $this->_add_obj_data($name, $default);
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $help
     *
     * @return
     */
    public function set_field_help($name, $help)
    {
        return $this->_set_field_param($name, 'help', $help);
    }
    public function get_field_help($name)
    {
        $help = '';
        $field_data = $this->_get_field_by_name($name);
        if (isset($field_data['help'])) {
            $help = $field_data['help'];
        }
        return $help;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $events
     *
     * @return
     */
    public function set_field_events($name, $events)
    {
        return $this->_set_field_param($name, 'events', $events);
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $attributes array()
     *
     * @return
     */
    public function set_field_attributes($name, $attributes = array())
    {
        if(is_array($attributes)){
            foreach($attributes as $key => $value){
                $this->_set_field_param($name, $key, $value);
            }
        }
        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $flag
     *
     * @return
     */
    // TODO: remove after the tests
    public function enable_conditional_control($name, $flag)
    {
        return $this->_set_field_param($name, 'enable_cc', $flag);
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $group
     *
     * @return
     */
    public function conditional_control($name, $group)
    {
        return $this->_set_field_param($name, 'group', $group);
    }
    /**
     * @brief
     *
     * @param[in out] $groupname
     * @param[in out] $fields (one dimentional array)
     *
     * @return
     */
    public function create_group($groupname, $fields , $condition = array('default' => array( 'value' => 'false', 'status' => 'disable')))
    {
        $field_key = $this->_get_field_key();
        if($this->_configure_locked()){
            //check if the data different with the init data
            //for now just log
            /*
            $class_name = get_class($this);
            if(isset(self::$_group[$field_key][$groupname])){
                $tmp_group_old_data = self::$_group[$field_key][$groupname];
                //check if group data same
                if(self::$_group[$field_key][$groupname]['fields'] != $fields){
                    safe_log(SAFE_LOG_DEBUG, "{$class_name}::[{$name}]::[{$param_name}]::try to change group [fields] => " .var_export($fields,true));
                }
                if(isset($condition['default']) && $tmp_group_old_data['default'] != $condition['default']){
                    safe_log(SAFE_LOG_DEBUG, "{$class_name}::[{$name}]::[{$param_name}]::try to change group [default] => " .var_export($condition['default'],true));
                }
                 if(isset($condition['condition']) && $tmp_group_old_data['condition'] != $condition['condition']){
                    safe_log(SAFE_LOG_DEBUG, "{$class_name}::[{$name}]::[{$param_name}]::try to change group [condition] => ");
                }
            }else{
                safe_log(SAFE_LOG_DEBUG, "{$class_name}:: try to add new group:[{$field_key}]:: [{$groupname}]");
            }*/
        }else{
            if ($groupname != '' && is_array($fields)) {
                self::$_group[$field_key][$groupname] = array();
                self::$_group[$field_key][$groupname]['fields'] = $fields;
                if(isset($condition['default'])){
                    self::$_group[$field_key][$groupname]['default'] = $condition['default'];
                }
                if(isset($condition['condition'])){
                    self::$_group[$field_key][$groupname]['condition'] = $condition['condition'];
                }
            }
            return true;
        }
        return false;
    }
    public function composite_layout($name, $fields)
    {
        if($this->_set_field_param($name, 'composite', $fields)){
            $_help_msg = array();
            foreach($fields as $field){
                $this->_set_field_param($field, 'composite_parent', $name);
                $field_data = $this->_get_field_by_name($field);
                if (isset($field_data['help'])) {
                    if(is_array($field_data['help'])){
                        $field_help =  $field_data['help'];
                    }else{
                        $field_help =  array($field_data['help']);
                    }
                    $_help_msg = array_merge($_help_msg , array($this->get_data_label($field)), $field_help, array(' '));
                    $this->_unset_field_param($field, 'help');
                }
            }
            if($_help_msg){
                $field_data = $this->_get_field_by_name($name);
                if(isset($field_data['help'])){
                    if(is_array($field_data['help'])){
                       $parent_help =  $field_data['help'];
                    }else{
                       $parent_help =  array($field_data['help']);
                    }
                    $this->_set_field_param($name, 'help', array_merge(array($this->get_data_label($name)), $parent_help, array(' '), $_help_msg));
                }else{
                    $this->_set_field_param($name, 'help',  $_help_msg);
                }
            }
        }
    }

    public function create_group_condition($field_name, $condition)
    {
        if ($field_name != '' && is_array($condition)) {
            $this->_group_condition[$field_name] = $condition;
            return $this->_set_field_param($field_name, 'group_condition', 'yes');
        }
    }
    /**
     * @brief
     *
     * @return
     */
    public function get_group()
    {
        $field_key = $this->_get_field_key();
        return self::$_group[$field_key];
    }
    public function get_group_condition()
    {
        return $this->_group_condition;
    }
    /**
     * @brief Return object data
     *
     * @return
     */
    public function get_data($include_hidden = true)
    {
        $data = $this->_merge_field_data();
        if(!$include_hidden){
            // filter out hidden fields
            foreach($data as $k => $v) {
                if($v['type'] == 'hidden') unset($data[$k]);
            }
        }
        return $data;
    }
    /**
     * @brief
     *
     * @return
     */
    public function get_data_keys()
    {
        $keys = array();
        // walk data and strip values, just keep keys
        foreach ($this->_get_fields() as $v) $keys[] = $v['field'];
        return $keys;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $resolve_enum
     *
     * @return
     */
    public function get_data_value($name, $resolve_enum = true)
    {
        $field_data = $this->_get_field_by_name($name);
        if (!isset($field_data['value']) || false == $resolve_enum){
            if('__empty__' == $this->_get_obj_data($name)){
                return '';
            }else{
                return $this->_get_obj_data($name);
            }
        }else{
            $tmp_data = $this->_get_obj_data($name);
            if(is_array($tmp_data)){
                $tmp_data_value = array();
                foreach ($tmp_data as $tmp_item){
                    if(isset($field_data['value'][$tmp_item])){
                        $tmp_data_value[] = $field_data['value'][$tmp_item];
                    }else{
                        $tmp_data_value[] = 'INVALID';
                        error_log("WARNNING - Get data value error : {$tmp_item} is an invalid key.");
                    }
                }
                return $tmp_data_value;
            }else{
                return $field_data['value'][$tmp_data];
            }
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $resolve_enum
     *
     * @return
     */
    public function get_data_label($name)
    {
        $field_data = $this->_get_field_by_name($name);
        if (!isset($field_data['label']) ){
            return '';
        }else{
            return $field_data['label'];
        }
    }
    /**
     * @brief Explode data values array into array of data vlues
     *           based on following parameter name pattern
     *           xx/yy where xx = configuration category and yy = parameter name
     *           Returned array is as follow:
     *           $values[xx][yy] = value
     *
     * @param[in out] $resolve_enum
     *
     * @return
     */
    public function get_data_values_exploded($resolve_enum = true, $include_hidden = true)
    {
        $values = $this->get_data_values($resolve_enum, $include_hidden);
        $exploded_values = array();
        foreach ($values as $k => $v) {
            $exploded_key = explode('/', $k);
            if(sizeof($exploded_key) > 1){
                // loop around exploded key(s)
                $entry = &$exploded_values;
                foreach($exploded_key as $key){
                    $entry = &$entry[$key];
                }
                $entry = $v;
            }else{
                $exploded_values['__all__'][$k] = $v;
            }
        }
        return $exploded_values;
    }
    /**
     * @brief
     *           If resolve_enum is set to true, when the field value is an enum, the
     *           returned value will be the label of the enum. To return the actual value,
     *           set to false.
     *
     * @param[in out] $resolve_enum
     *
     * @return
     */
    public function get_data_values($resolve_enum = true, $include_hidden = true)
    {
        $values = array();
        // walk data and strip keys, just keep values
        foreach ($this->_get_fields() as $k => $v) {

            $field_default = $this->_get_obj_data($k);

            if (!$include_hidden && $v['type'] == 'hidden'){
                continue;
            }
            if (!isset($v['value']) || is_array($field_default) || false == $resolve_enum){
                $values[$k] = $field_default;
            }else{
                //enum list select one
                if (isset($v['value'][$field_default])){
                    $values[$k] = $v['value'][$field_default];
                } else {
                    if ($resolve_enum){
                        $values[$k] = 'INVALID';
                    }else{
                        $values[$k] = $field_default;
                    }
                }
            }
            // Adjust known special values
            if(!$resolve_enum && '__empty__' == $values[$k]){
                $values[$k] = '';
            }
        }
        return $values;
    }
    public function get_data_values_with_field_desc($resolve_enum = true, $include_hidden = true, $include_password = true, $include_base64=true)
    {
        $values = array();
        // walk data and strip keys, just keep values
        foreach ($this->_get_fields() as $k => $v) {
            $field_default = $this->_get_obj_data($k);
            if (!$include_hidden && $v['type'] == 'hidden'){
                continue;
            }
            if (!$include_password && $v['type'] == 'password'){
                continue;
            }
            if (!$include_base64 && $v['type'] == 'base64'){
                continue;
            }
            if (!isset($v['value']) || false == $resolve_enum){
                $values[$v['label']] = $field_default;
            }else{
                if(is_array($field_default)){
                    foreach($field_default as $v1){
                        if (isset($v['value'][$v1])) {
                            if($values[$v['label']]){
                                $values[$v['label']] .= ', ';
                            }
                            $values[$v['label']] .= $v['value'][$v1];
                        }else{
                            $values[$v['label']] .= 'INVALID - ' . $v1;
                        }
                    }
                }else{
                    if (isset($v['value'][$field_default])) {
                        $values[$v['label']] = $v['value'][$field_default];
                    }else {
                        $values[$v['label']] = 'INVALID - ' . $field_default;
                    }
                }
                // Adjust known special values
                if(!$resolve_enum && '__empty__' == $values[$k]){
                    $values[$k] = '';
                }
            }
        }
        return $values;
    }
    public function summary($type = 'horizontal' , $long = false, $min=3)
    {
        $list =  $this->get_data_values_with_field_desc(true, false, false, false);
        if($long){
            array_unshift($list,array('Name' => $this->name()));
            return array( 'data' => $list);
        }else{
            $result =  array('Name' => $this->name());
            foreach($list as $key => $item){
                $min--;
                $result[$key] = $item;
                if($min == 0) break;
            }
            return array( 'data' => $result);
        }
    }
    /**
     * @brief
     *
     * @param[in out] $data
     *
     * @return
     */
    public function set_data_values($data)
    {
        // Walk data and adjust _fields
        $field_datas = $this->_get_fields();
        if(is_array($data)){
            foreach ($data as $k => $v) {
                if (!isset($field_datas[$k])) {
                    // TODO(wadam) - Does not exist in the current object
                    // Need to create field

                } else {
                    //remove the white space
                    if(is_array($v)){
                        $v = array_map('safe_trim',$v);
                    }else{
                        $v = safe_trim($v);
                    }
                    // Update the field value or default
                    if (isset($field_datas[$k]['value']) && !is_array($v)) {
                        if('' == $v){
                            $v = '__empty__';
                            if($field_datas[$k]['type'] == 'dropdown'){
                                //if the valaue is empty string (""), map to option value 
                                $empty_keys = array('__none__','__selectone__');
                                foreach($empty_keys as $tmp_item){
                                    if(isset($field_datas[$k]['value'][$tmp_item])){
                                        $v = $tmp_item;
                                        break;
                                    }
                                }
                            }
                        }
                        // Enum value, figure out if we have label or real value
                        if (isset($field_datas[$k]['value'][$v])){
                            $this->_add_obj_data($k, $v);
                        }else {
                            // Default is to use supplied value
                            $this->_add_obj_data($k, $v);
                            // Label, walk enum to get the key
                            foreach ($field_datas[$k]['value'] as $enum_key => $enum_label) {
                                if ($enum_label == $v) {
                                    $this->_add_obj_data($k, $enum_key);
                                    break;
                                }
                            }
                        }
                    }else{
                        $this->_add_obj_data($k, $v);
                    }
                }
            }
        }else{
            //safe_log(SAFE_LOG_INFO, "WARNING - data empty or format error " . var_export($_type,true));
        }
        return true;
    }
    /**
     * @brief
     *
     * @return
     */
    public function get_data_labels()
    {
        $values = array();
        // walk data and strip keys, just keep values
        foreach ($this->_get_fields() as $v) $values[$v['field']] = $v['label'];
        return $values;
    }
    /**
     * @brief Serializable object interface implementation
     *
     * @return
     */
    protected function _get_serialized_data()
    {
        return $this->get_data_values(false);
    }
    /**
     * @brief Set the php array of retrieved data
     *
     * @param[in out] $data
     *
     * @return
     */
    protected function _set_unserialized_data($data)
    {
        $this->set_data_values($data);
    }
    /**
     * @brief
     *
     * @return
     */
    public function save(&$output = null)
    {
        //clear check_list,modified_object_list cache
        $this->clear_list_cache();
        return $this->serialize();
    }
    /**
     * @brief Set user rule messages for error
     *
     * @param[in out] $user_rule_msgs
     *           format=  array('rule'=>'message',...);
     *
     * @return
     */
    public function set_user_rule_msgs($user_rule_msgs)
    {
        $this->_user_rule_msgs = $user_rule_msgs;
    }
    /**
     * @brief Return user rule messages
     *
     * @return array('rule'=>'message',...);
     */
    public function user_rule_msgs()
    {
        return $this->_user_rule_msgs;
    }
    /**
     * @brief get safe from obj
     *
     * @return
     */
    public function get_form(){
        if($this->_form != null){
            $this->_form->set_config($this->get_data());
        }else{
            $this->_form = new Safe_form_class($this->get_data());
        }
        return $this->_form;
    }
    public function data_filter($data){
        // filter out disabled data fields
        if(is_array($data)){
            $fields = $this->get_data(true);
            foreach($data as $key => $value) {
                if($fields[$key]) {
                    if($fields[$key]['disabled'] == 'disabled') {
                        unset($data[$key]);// ignore disabled field
                    }
                }
            }
        }
        return $data;
    }
    /**
     * @brief validation post data
     *
     * @param obj $obj (obj to get config)
     * @param array $post (post data to be validate)
     * @param array &$validation_errors (Array output msg)
     *
     * @return
     */
    public function validate($data, &$out_message = null){
        if(!$this->can_modify($out_message)){
            return false;
        }
        if(!isset($out_message)) {
            $out_message = array();
        }
        // Validate field exists and allowed to be modified
        $fields = $this->get_data(true);
        if(is_array($data)){
            foreach($data as $key => $value) {
                // Exists
                if($fields[$key]) {
                    if($fields[$key]['type'] == 'hidden' || $fields[$key]['readonly'] == 'readonly') {
                        unset($data[$key]);//ignore readonly field
                    }
                    if($fields[$key]['disabled'] == 'disabled') {
                        $this->set_field_rules($key, 'disabled');
                    }
                    if($fields[$key]['type'] == 'listbox' || $fields[$key]['type'] == 'checkbox') {
                        if(!is_array($value)){
                            if($value){
                                $data[$key] = array($value);
                            }else{
                                $data[$key] = array();
                            }
                        }
                    }
                } else {
                    $out_message[$key] = 'Unknown Parameter.';
                    safe_log(SAFE_LOG_ERROR, "WARNING - ". $this->object_name() . " {$key} : {$out_message[$key]}");
                }
            }
        }else{
            //safe_log(SAFE_LOG_INFO, "WARNING - data empty or format error " . var_export($_type,true));
        }

        // Any error ?
        if($out_message)
            return false;
        // Update data in object
        $this->set_data_values($data);
        $_POST = $this->get_data_values(false);
        $this->get_form();
        //set validation obj (for call back check rule)
        $this->_form->set_validation_obj($this);
        // apply user_rules_msgs array (for custom rules message error)
        $rule_messages = $this->user_rule_msgs();
        if($rule_messages){
            $this->_form->set_message($rule_messages);
        }
        $ha_active = $this->ha_mode_active();
        return $this->_form->run_validation($out_message, '', $ha_active);
    }
    protected function set_form_error($field = '', $message = ''){
        $this->_form->set_error($field , $message);
    }
    protected function set_error($field = '', $message = '', &$output = null){
        $output[$field] = $message;
        if(!$this->_form){
            $this->get_form();
        }
        $this->_form->set_error($field , $message);

    }

    public function register_aggregate_object($type, $definition = null)
    {
        $key = $this->_get_field_key();
        if(!$this->_configure_locked()){
            parent::register_aggregate_object($type, $definition);
            self::$_aggr_obj_type_data[$key][$type] = $definition;
        }else{
            //log diff register_aggregate_object
            $class = get_class($this);
            $name = $this->name();
            if(isset(self::$_aggr_obj_type_data[$key][$type]) ){
                if(self::$_aggr_obj_type_data[$key][$type] != $definition){
                    foreach(self::$_aggr_obj_type_data[$key][$type] as $key => $value){
                        if($value != $definition[$key]){
                            safe_log(SAFE_LOG_DEBUG, "change definition =>{$class}=>{$name}=>{$type}=>{$key}=>old" . var_export($value,true));
                            safe_log(SAFE_LOG_DEBUG, "change definition =>{$class}=>{$name}=>{$type}=>{$key}=>new" . var_export($definition[$key],true));
                        }
                    }
                }
            }else{
                safe_log(SAFE_LOG_DEBUG, " add new definition => {$class}=>{$name} => {$type}=>" . var_export($definition,true));
            }
        }
    }
    public function compare($obj)
    {
        $result = 0;
        if(is_object($obj) && get_class($obj) == get_class($this)){
            $obj_data = $obj->get_data_values(false);
            $this_data = $this->get_data_values(false);
            $diff = array();
            foreach($this_data as $key => $value){
                if(isset($obj_data[$key]) && $obj_data[$key] === $value){
                    continue;
                }else{
                    $diff[$key] = $value;
                }
            }
            if(count($diff) > 0){
                $result = $diff;
            }
        }else{
            $result = 1;
        }
        return $result;
    }
    public function get_port()
    {
        return array();
    }

    /**
     * @brief This method just returns one record from those matching the
     * query. This lets to create clean code when trying to get entries from
     * database. For example, to get the interface object address: 192.168.1.128
     * Sng_network_interface_class::model($this->_node)->findOne(array('address'=>'192.168.1.128'));
     * @param Array dictionary based query to search the database.
     * @param integer Offset to start getting results.
     * @return Object If the query succeed it return the object defined by the
     * calling class, otherwise null;
     */
    public function findOne($query=array(),$offset=0){
        $result = Safe_db_data_source_class::model($this->node(),get_class($this))->limit(1)->find($query);
        return $result->count()? $result->current() : null;
    }    

    /**
     * @brief This method returns a list of limit records matching the query
     * from a given offset. For example, to get all ip addresses for eth1
     * interface:
     * Sng_ip_address_class::model($this->_node)->find(array('interface'=>'eth1'))
     * @param integet set the max number of objects returned as a result.
     * @param integer set the offset from where to start getting results.
     * @return Object Safe_db_data_source_class object with the matching results.
     */    
    public function find($query=array(),$limit=100,$offset=0){
       return Safe_db_data_source_class::model($this->node(),get_class($this))->limit($limit)->offset($offset)->find($query);
    }
}
/* End of file safe_object_class.php */
