<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * Product class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_node_class');
safe_require_class('safe_application_class');
define('SNG_PRODUCT_DEF', '/usr/local/sng/conf/prod-def.xml');
$g_safe_product = null;
function &Safe_get_product()
{
    global $g_safe_product;
    if(null == $g_safe_product) {
        /*
        * Retrieve product from prod-def.xml
        */
        //after upgrade, need to reload the prod_def
        $reboot_data = safe_need_reboot();
        if($reboot_data && $reboot_data['reboot_request']){
            unset($_SESSION['prod_def']);
        }
        if(!isset($_SESSION['prod_def'])){
            $dom = new DOMDocument();
            $dom->load(SNG_PRODUCT_DEF);
            $dom->xinclude();

            $xml = simplexml_import_dom($dom);
            // Get only the product
            $xml = $xml->xpath('//product');
            $tmp_def = json_decode(json_encode($xml) , true);
            session_start();
            $_SESSION['prod_def'] = array('product' => $tmp_def[0]);
            session_write_close();

            unset($xml);
            unset($tmp_def);
        }
        $prod_def = $_SESSION['prod_def'];

        // Try to locate the product specific class
        $product_name = strtolower($prod_def['product']['@attributes']['name']);
        $safe_product = safe_new_module_class($product_name, 'product');
        if (null == $safe_product){
            safe_log(SAFE_LOG_ERROR, 'ERROR - Cannot find product class for <'.$product_name.'>');
            $safe_product = new Safe_product_class();
        }
        // Configure product
        $safe_product->configure($prod_def);
        $safe_product->post_configure();
        unset($prod_def);
        // Latch to global
        $g_safe_product = $safe_product;
        
    }
    return $g_safe_product;
}
class Safe_product_class extends Safe_object_class
{
    private $_name;
    private $_nodes = array();
    public $_application = NULL;
    public function __construct($name = "Product name")
    {
        parent::__construct('', $name);
    }
    public function configure($product_def = array())
    {
        parent::configure();
        // Create local node
        $this->_nodes['local'] = new safe_local_node($this);
        $this->_nodes['local']->configure($product_def);
        // Create empty remote nodes
        $this->_nodes['remote'] = array();
        // Create the application
        $this->_application = $this->create_application();
        // Configure and register application 
        // in some module configure() need to call $this->_node->software()->application()->find_module_by_name(),
        // so  the _application must set before configure
        $this->local_node()->software()->set_application($this->_application);
        $this->_application->configure();
        // Create additional services (main step)
        $this->_create_services('main');
        return TRUE;
    }
    protected function _create_services($step='main'){
        // Add services as defined in configuration
        $services = $this->local_node()->configuration_manager()->services();
        foreach ($services as $service) {
            // Check if this service should be created at this step
            $service_step = 'main';
            if($service['@attributes']['create']){
                $service_step = $service['@attributes']['create'];
            }
            if($service_step != $step){
                continue;
            }
            // Check if generic service class is implemented by application
            $prefix_list = array(
                strtolower($this->name()),
                'fs',
                'sng',
            );
            $service_name = str_replace('-', '_', $service['name']) . '_service';
            $service_object = null;

            foreach($prefix_list as $prefix){
                if(safe_module_require_class($prefix, $service_name)) {
                    $service_class = Safe_module_class_name($prefix, $service_name);
                    $service_object = new $service_class($this->local_node()->software() , $service['name']);
                    break;
                }
            }
            // Generic service
            if(null === $service_object){
                $service_object = new Safe_service_class($this->local_node()->software() , $service['name']);
            }
            // Register
            $this->local_node()->software()->add_service($service_object);
            $service_object = $this->local_node()->software()->service($service_object->name());
            // Configure service
            $service_object->configure();
            if (isset($service['description'])) $service_object->set_description($service['description']);
            if (isset($service['category'])) $service_object->set_category($service['category']);
            // Process properties
            if(isset($service['properties'])){
                $service['properties']['property'] = to_index_array($service['properties']['property']);
                $_properties = array();
                foreach($service['properties']['property'] as $property){
                    if($property['@attributes']['name'] && isset($property['@attributes']['value'])){
                        $_properties[$property['@attributes']['name']] = $property['@attributes']['value'];
                    }
                }
                $service_object->set_properties($_properties);
            }
        }

        // Check for audit parameters
        $audits = $this->local_node()->configuration_manager()->audits();
        foreach($audits as $audit_name => $audit){
            $module = $this->local_node()->find_module_by_name($audit_name);
            if ($module){
                $module->set_audit_params($audit);
            }
        }
    }

    public function post_configure()
    {
        $this->_create_services('post');
        // Add services as defined in configuration
        $services = $this->local_node()->software()->services();
        foreach ($services as $service) {
            $service->post_configure();
        }
    }
    public function &local_node()
    {
        return $this->_nodes['local'];
    }
    public function remote_nodes()
    {
        return $this->_nodes['remote'];
    }
    // Can be overloaded by real product class
    public function &create_application()
    {
        safe_log(SAFE_LOG_INFO, 'WARNING - Create default Safe_application_class');
        return new Safe_application_class();
    }
    public function create_software_class($node)
    {
        return new Safe_software_class($node);
    }
    public function create_hardware_class($node)
    {
        return new Safe_hardware_class($node);
    }
    /*
     * Backup/restore hooks
     * To loop around all services
    */
    public function can_backup($type, &$reason)
    {
        foreach ($this->local_node()->software()->services() as $service) {
            if (TRUE != $service->can_backup($type, $reason)) return FALSE;
        }
        return TRUE;
    }
    /**
     * @brief Pre backup hook
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function pre_backup($type, &$reason)
    {
        $service_status = array();
        foreach ($this->local_node()->software()->services() as $service) {
            if (TRUE != $service->pre_backup($type, $reason)) return FALSE;
            $service_status[$service->name() ]['status'] = $service->status();
            $service_status[$service->name() ]['startup'] = $service->startup();
        }
        // Store service status in db
        $ser = safe_object_serializer_class::get_serializer();
        $ser->store_object('/tmp', 'service_status', 'none', $service_status, 'tmp');
        // close and reopen db so cache is flushed to disk
        $ser = Safe_object_serializer_class::get_serializer(TRUE);
        return TRUE;
    }
    /**
     * @brief Post backup hook
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function post_backup($type, &$reason)
    {
        foreach ($this->local_node()->software()->services() as $service) {
            if (TRUE != $service->post_backup($type, $reason)) return FALSE;
        }
        return TRUE;
    }
    /**
     * @brief Check if archive restore can be done
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function can_restore($archive_name, &$reason)
    {
        $cfg_mgr = $this->local_node()->configuration_manager();
        $archive_mgr = $cfg_mgr->archive_manager();
        // Ask archive manager if this archive can be restored
        if(!$archive_mgr->can_restore($archive_name, $reason)){
            return false;
        }
        // Get archive info from archive manager
        $info = $archive_mgr->info($archive_name);

        foreach ($this->local_node()->software()->services() as $service) {
            if (TRUE != $service->can_restore($info, $reason)) return FALSE;
        }
        return TRUE;
    }
    /**
     * @brief Pre restore hook
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function pre_restore($info, &$reason)
    {
        foreach ($this->local_node()->software()->services() as $service) {
            if (TRUE != $service->pre_restore($info, $reason)) return FALSE;
        }
        if($info['db'] === 'true'){
            // All good, now here are pre restore steps:
            // 1- Remove all (N) record from object table
            // 2- Change all records to (D)
            // 3- Disable error checking in configuration manager
            // 4- Generate config (this will proceed to all delete)
            $ser = Safe_object_serializer_class::get_serializer(TRUE);
            $ser->delete_object_with_status($this->local_node()->object_name() , TRUE, Safe_object_serializer_class::OBJ_STATUS_NEW);
            $ser->update_object_with_status($this->local_node()->object_name() , TRUE, NULL, Safe_object_serializer_class::OBJ_STATUS_DELETED);
            //clear the checklist and modify list cache
            $this->clear_list_cache();
            $this->local_node()->configuration_manager()->forget_error(TRUE);
            $this->local_node()->generate_config();
        }
        return TRUE;
    }
    /**
     * @brief Post restore hook
     *           
     * @param[in out] $reason
     *           
     * @return TRUE/FALSE
     */
    public function post_restore($info, &$reason)
    {
        // Ensure DB is accessible as well as conf directory
        $this->local_node()->mkdir('/var/webconfig/SAFe/application/db');

        try {
            $conf = $this->local_node()->configuration_manager()->directory('configuration');
            $this->local_node()->mkdir($conf);
        } catch( Exception $e){
        }
        if($info['db'] === 'true'){
            // All good, now here are post restore steps
            // 1- Remove all (D) records, should not happen but just in case
            // 2- Update all records to (N)
            // 3- Generate config
            $ser = Safe_object_serializer_class::get_serializer(TRUE);
            $ser->delete_object_with_status($this->local_node()->object_name() , TRUE, Safe_object_serializer_class::OBJ_STATUS_DELETED);
            $ser->update_object_with_status($this->local_node()->object_name() , TRUE, NULL, Safe_object_serializer_class::OBJ_STATUS_NEW);
            //clear the checklist and modify list cache
            $this->clear_list_cache();
            // Re configure hardware
            $this->local_node()->hardware()->post_restore($info, $reason);
            // Re configure software as record content may have changed
            $this->local_node()->software()->configure();

            // Retrieve service status in db
            $service_status = array();
            $ser = safe_object_serializer_class::get_serializer();
            $ser->retrieve_object('/tmp', 'service_status', $service_status);
        }

        // Now inform all service restore is done
        foreach ($this->local_node()->software()->services() as $service) {
            // Should not stop on error :(
            $service->post_restore($info, $reason);
            // Check if we can restore startup ?
            if (isset($service_status[$service->name() ]) && isset($service_status[$service->name() ]['startup'])) {
                $service->set_startup($service_status[$service->name() ]['startup']);
            }
        }
        if($info['db'] === 'true'){
            // Re generate configuration
            if($info['type'] == 'system') {
                // For system archive restore we might face some issue but we 
                // need at least network to be restored, so forget error. 
                // They'll be reported either by checklist or on apply
                $this->local_node()->configuration_manager()->forget_error(TRUE);
            }else{
                $this->local_node()->configuration_manager()->forget_error(FALSE);
            }
            if (TRUE != $this->local_node()->generate_config()){
                $reason = $this->local_node()->configuration_manager()->progress();
                return FALSE;
            }
        }

        // let's make sure we apply back network config on startup if network
        // settings were overriden by the restore
        $result = array();
        $bootstrap_path = $this->local_node()->configuration_manager()->directory('sng-base') . '/var/bootstrap';
        if ($info['skip_network'] === false || $info['skip_network'] === 'false') {
            $this->local_node()->os()->execute('nohup rm -f ' . $bootstrap_path . '/000-000-apply-network.bootstrap.done', $result, true,true);
        }
        // Ensure product is up to date
        $result = array();
        $script_path = $this->local_node()->configuration_manager()->directory('sng-base') . '/scripts';
        $this->local_node()->os()->execute('nohup ' . $script_path . '/update-product.sh', $result, true, true);
        //check if firewall is running 
        $firewall_module = $this->local_node()->software()->application()->find_module_by_name('firewall');
        if($firewall_module){
            if($firewall_module->status() == Safe_service_class::STATUS_RUNNING){
                $firewall_module->restart();
            }
        }
        return TRUE;
    }
    public function restore_progress()
    {
        return $this->local_node()->configuration_manager()->progress();
    }
    /**
     * @brief Check if update package can be install
     *
     * @param[in out] $reason
     *
     * @return TRUE/FALSE
     */
    public function can_install_data($info, &$reason)
    {
        return $this->local_node()->can_install_data($info, $reason);
    }
    public function do_reset($info, &$reason)
    {
        $this->local_node()->set_config_status('reset', 'pre');
        if(TRUE != $this->local_node()->pre_reset($info, $reason)) {
            $this->local_node()->clear_config_status('reset');
            return FALSE;
        }
        $this->local_node()->set_config_status('reset', 'doing');
        if(TRUE != $this->local_node()->do_reset($info, $reason)) {
            $this->local_node()->clear_config_status('reset');
            return FALSE;
        }
        if($info['type'] == 'config' || $info['type'] == 'factory' ){
            // All modules have performed their cleanup, now remove all objects:
            // 1- Remove all (N) record from object table
            // 2- Change all records to (D)
            // 3- Disable error checking in configuration manager
            // 4- Generate config (this will proceed to all delete)
            $this->local_node()->configuration_manager()->forget_error(TRUE);
            $this->local_node()->generate_config();
            // Ensure product is up to date
            $result = array();
            $script_path = $this->local_node()->configuration_manager()->directory('sng-base') . '/scripts';
            $this->local_node()->os()->execute('nohup ' . $script_path . '/update-product.sh', $result, true, true);
        }
        $this->local_node()->set_config_status('reset', 'post');
        $tmp_post_reset_rs = $this->local_node()->post_reset($info, $reason);
        $this->local_node()->clear_config_status('reset');
        if(true != $tmp_post_reset_rs) {
            return FALSE;
        }
        return TRUE;
    }
}
/* End of file safe_product_class.php */
