<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * Software class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_node_class');
safe_require_class('safe_version_class');
safe_require_class('safe_license_class');
safe_require_class('safe_service_class');
class Safe_software_class extends Safe_object_class
{
    private $_application = null;
    private $_node = null;
    private $_services = array();
    protected $_version = null;
    protected $_license = null;
    public function __construct($node)
    {
        $this->_node = $node;
        parent::__construct($node->object_name() , 'software');
    }
    public function node()
    {
        return $this->_node;
    }
    protected function create_version()
    {
        // Process known stuff
        $this->_version = new Safe_version($this);
        $this->_version->configure($this->_node);
    }
    protected function create_license()
    {
        // Process known stuff
        $this->_license = new Safe_license($this);
        $this->_license->configure($this->_node);
    }

    public function &version()
    {
        if(!$this->_version){
            $this->create_version();
        }
        return $this->_version;
    }
    public function &license($reload = false)
    {
        if(!$this->_license || $reload){
            $this->create_license();
        }
        return $this->_license;
    }
    public function &service($name)
    {
        if (isset($this->_services[$name])) return $this->_services[$name];
        else return null;
    }
    public function services()
    {
        return $this->_services;
    }
    public function add_service($service)
    {
        if (!isset($this->_services[$service->name() ])) {
            $this->_services[$service->name() ] = $service;
            return true;
        }
        safe_add_error("Service " . $service->name() . " already registered.");
        return false;
    }
    public function application()
    {
        return $this->_application;
    }
    public function set_application($application)
    {
        $this->_application = $application;
    }

    public function resolve_dependencies(&$service,&$config_mgr,&$configured,&$started){
        $dependencies = $service->depends_on();
        safe_log(SAFE_LOG_INFO,__METHOD__.'::> Checking out dependencies for '.$service->name());
        foreach($dependencies as $dependency){
            $service_name = $dependency[0];
            $start = $dependency[1];
            if(isset($this->_services[$service_name])){
                safe_log(SAFE_LOG_INFO,__METHOD__.'::> Configuring dependency: '.$service_name);
                $this->resolve_dependencies($this->_services[$service_name],$config_mgr,$configured,$started);
                if(!isset($configured[$service_name]) &&
                    $this->_services[$service_name]->generate_config($config_mgr)){
                    safe_log(SAFE_LOG_INFO,__METHOD__.'::> Configured successfully');
                    if($start && $this->_services[$service_name]->api_start_service()){
                        safe_log(SAFE_LOG_INFO,__METHOD__.'::> Started successfully');
                        $started[$service_name] = true;
                    }
                    $configured[$service_name] = true;
                }else if(isset($configured[$service_name]) && $start &&
                        $this->_services[$service_name]->api_start_service()){
                        safe_log(SAFE_LOG_INFO,__METHOD__.'::> Started successfully');
                    $started[$service_name] = true;
                }else if(isset($configured[$service_name])){
                    safe_log(SAFE_LOG_INFO,__METHOD__.'::> Error starting service '.$service_name);
                }else
                    safe_log(SAFE_LOG_INFO,__METHOD__.'::> Service '.$service_name.' not configured properly');
            }
        }
    }

    public function generate_config(&$config_mgr)
    {
        $configured = array();
        $started = array();
        // Invoke generate on application (an application is a service ;)
        foreach ($this->_services as $service) {
            $this->resolve_dependencies($service,$config_mgr,$configured,$started);
            if (!$service->generate_config($config_mgr)) {
                return false;
            }else{
                $configured[$service->name()] = true;
            }
        }
        return true;
    }
    public function handle_global_messages(&$msg_list, $direct_call = false)
    {
        // Loop around services
        foreach ($this->_services as $service) {
            $service->handle_global_messages($msg_list, $direct_call);
        }
    }
    public function checklist(&$list)
    {
        // Check license
        $type='';
        if(!$this->_license->status($type)){
            $list[] = array(
                'module' =>'application',
                'obj_type' =>'license',
                'description' => $type,
            );
        }
        // Loop around services
        foreach ($this->_services as $service) {
            $service->checklist($list);
        }
    }
    public function ha_checklist(&$list)
    {
        // Loop around services
        foreach ($this->_services as $service) {
            $service->ha_checklist($list);
        }
    }
    public function get_audit_points(&$audit_points)
    {
        // Loop around services
        foreach ($this->_services as $service) {
            $service->get_audit_points($audit_points);
        }
    }
    public function can_install_data($info, &$reason)
    {
        foreach ($this->services() as $service) {
            if (true != $service->can_install_data($info, $reason)) return false;
        }
        return true;
    }
    public function pre_reset($info, &$reason)
    {
        foreach ($this->services() as $service) {
            if (true != $service->pre_reset($info, $reason)) return false;
        }
        $this->license()->pre_reset($info, $reason);
        return true;
    }
    public function do_reset($info, &$reason)
    {
        foreach ($this->services() as $service) {
            if (true != $service->do_reset($info, $reason)) return false;
        }
        $this->license()->do_reset($info, $reason);
        return true;
    }
    public function post_reset($info, &$reason)
    {
        foreach ($this->services() as $service) {
            if (true != $service->post_reset($info, $reason)) return false;
        }
        $this->license()->post_reset($info, $reason);
        return true;
    }
    
}
/* End of file safe_software_class.php */
