<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Safe Timeout Command class
 * @author Santiago Tramoyeres Cuesta <scuesta@sangoma.com>
 *         
 */

/**
 * Following calls let command execution with timeout. If command reach the
 * timeout, the process will be terminated and a given callback could be called.
 *
 * Use example:
 * 
 *   For test this example we should comment escapeshellcmd method.
 *
 *   print date("[Y-m-d H:i:s]")." Stating execution\n";
 *    $tcommand = new Safe_timeout_command_class("sleep 5 && echo \"HELLO WORLD\",3,function(){
 *       print date("[Y-m-d H:i:s]")." Execution ends by timeout\n";
 *   });
 *
 *  if(($return = $tcommand->exec()))
 *   print date("[Y-m-d H:i:s]")." Execution ends normaly\nAnd return: $return";
 */
class Safe_timeout_command_class{

    private $cmd;
    private $callback;
    private $args;
    private $timeout;

    public function __construct($cmd,$timeout,$callback=null,&$args=array()){
        
        $this->cmd = escapeshellcmd($cmd);
        $this->timeout = $timeout;
        $this->callback = $callback;
        $this->args = $args;
    }

    private function exec_callback(){
        $callback = $this->callback;
        if($callback)
            $callback($this->args);
    }

    public function exec(){
        $descriptorspec = array(0=>array("pipe","r"),
                                1=>array("pipe","w"),
                                2=>array("pipe","w"));
        
        $pipes = array();
        $return = "";
        $write = NULL;
        $exceptions = NULL;
        $process = proc_open($this->cmd,$descriptorspec,$pipes);
        if(is_resource($process)){
            stream_set_blocking($pipes[1],false);
            $timeleft = $this->timeout;
            do{
                $start = time();
                $read = array($pipes[1]);
                stream_select($read,$write,$exceptions,$timeleft);

                if($read!=array())
                    $return.=fread($pipes[1],1024);
                $timeleft = $timeleft-(time()-$start);
            }while(!feof($pipes[1]) && $timeleft>0);

            for($i=0;$i<count($pipes);$i++)
                fclose($pipes[$i]);

            if($timeleft<=0){
                $this->exec_callback();
                // 9 => SIGKILL
                proc_terminate($process,9);
                return false;
            }
            else
                proc_close($process);
        }        
        return $return;
    }

  }