<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Network Config wrapper page
 * @author Santiago Tramoyeres Cuesta <scuesta@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
safe_require_class('safe_ftp_class');

abstract class Safe_update_checker
{
    protected $config;
    
    /*public function __construct(&$configuration=array()){
      $this->config = self::exted(array(
      'hostname'=>'ftp.sangoma.com',
      'username'=>'anonymous',
      'password'=>'',
      'download_id_file'=>'SAFEUpdateChecker.did',
      'threshold_time'=>1200
      ),
      $configuration);
      }*/
    
    
    /**
   * @brief Returns link from version
   * @param Safe_vinfo
   * return string
   */
    abstract public function get_link(&$vinfo);
    
    /**
     * @brief Gets next version from ftp for a given Safe_vinfo
     * @param ftp Resource
     * @param Safe_vinfo
     * return Safe_vinfo
     */
    abstract public function get_next_update_version(&$ftp,&$vInfo);
    
    public function __get($key){
        return isset($this->config[$key])? $this->config[$key] : null;
    }
    
    public function __set($key,$value){
        if(isset($this->config[$key]))
            $this->config[$key] = $value;
    }
    
    /**
     * @brief Merges several arrays
     * @param array,array,array
     * return array
     */
    public static function extend(){
        $numArgs = func_num_args();
        $dest = $numArgs>0? func_get_arg(0) : array();
        if($numArgs>1)
            for($i=1;$i<$numArgs;$i++){
                $aux = func_get_arg($i);
                $dest = array_merge($dest,$aux);
            }
        return $dest;
    }
    
    /**
     * @brief Get array connection credentials
     * return array(hostname,port,username,password,timeout)
     */
    abstract public function get_connection_credentials();
    
    /**
     * @brief Check if a given version exists on server
     * @param Safe_vinfo
     * return boolean
     */
    abstract public function exists(&$version);

    /**
     * @brief Get version file size
     * @param ftp resource
     * @param Safe_vinfo
     * return integer
     */
    abstract public function get_version_size(&$ftp,&$version);

    /**
     * @brief Get next version info for a given Safe_vinfo
     * @param Safe_vinfo
     * return array
     */
    public function get_next_version(&$vInfo){
        $arEq=true;
        $isGreat = false;
        $link = null;
        $next = "";
        $size = 0;
        $status = null;
        $rate = 0;
        if(!$vInfo->is_dev()){
            list($hostname,$port,$username,$password,$timeout) = $this->get_connection_credentials();
            $ftp = & self::get_ftp_connection($hostname,$username,$password,$port,$timeout);
            $next = $this->get_next_update_version($ftp,$vInfo);
            $next = $next!=null? $next : $vInfo;
            $size = $this->get_version_size($ftp,$next);
            $ftp->close();
            if(!$next->is_beta()){
                $arEq = $next->eq($vInfo);
                $isGreat = $next->gth($vInfo);
                $link = $this->get_link($next);
                $downloading = $this->is_downloading();
                $status = $downloading? 'downloading' : '';
                if($downloading)
                    $rate = min(100,round(($this->progress()/$size)*100,2));  
            }
        }
        return array('current'=>''.$vInfo,'new_version'=>($arEq? null : "".$next),'is_greater'=>$isGreat,'link'=>$link,'size'=>$size,'status'=>$status,'rate'=>$rate);
    }
    
    /**
     * @brief Gets ftp connection Resource for a given hostname
     * @param string $hostname
     * @parma string $username
     * @param string $password
     */
    public static function &get_ftp_connection($hostname,$username,$password,$port=21,$timeout=10){
        /*     $CI = & get_instance();
        $CI->load->library('ftp');
        $config = array('hostname'=>$hostname,'username'=>$username,'password'=>$password);
        $CI->ftp->connect($config);
        return $CI->ftp;*/
        $config = array('hostname'=>$hostname,'port'=>$port,'timeout'=>$timeout,'username'=>$username,'password'=>$password);
        $ftp = new Safe_ftp;
        $ftp->connect($config);
        return $ftp;
    }
    
    /**
     * @brief Downloads the file from url and save into to_file path
     * @param os wrapper
     * @param Safe_vinfo
     * @param string to_file
     * return boolean 
     */
    public function download(&$os_wrapper,&$vinfo,$to_file){
        $url = $this->get_link($vinfo);
        if($url && !$this->is_downloading()){
            file_put_contents($this->download_id_file,json_encode(array('version'=>"".$vinfo,'time'=>strtotime('now'),'file'=>$to_file)));
            // Trying to use ftp to download files from ftp, but for a big files,
            // (tested with nsc-2.2.7-47-GA.update.tgz, about 354MB) before download is
            // finished, ftp library gets back the control of execution to the main
            // thread and still downloading the file.
            
            $rc = $os_wrapper->execute("wget -c ".$url." --user=".$this->ftp_username." --password='".$this->ftp_password."' -O ".$to_file) && $os_wrapper->execute("wget -c ".$url.".md5 --user=".$this->ftp_username." --password='".$this->ftp_password."' -O ".$to_file.".md5");
            if($rc)
                $rc = md5_file($to_file)==trim(file_get_contents($to_file.".md5")); // removing \n from file content
            if(!$rc)
                @unlink($to_file);
            @unlink($to_file.".md5");
            @unlink($this->download_id_file);
            return $rc;
        }
        return false;
    }
    
    /**
     * @brief get downloading progress in bytes
     * return integer
     */
    public function progress(){
        $stat = null;
        if(file_exists($this->download_id_file)){
            $json = (array)json_decode(file_get_contents($this->download_id_file));
            if(isset($json['file']))
                $stat = @stat($json['file']);
        }
        return $stat && isset($stat['size'])? $stat['size'] : 0;
    }

    /**
     * @brief Checks if there is a downloading process already running
     * return boolean
     */
    public function is_downloading(){
        $downloading = true;
        if(file_exists($this->download_id_file)){
            $json= (array)json_decode(file_get_contents($this->download_id_file));
            if(isset($json['time']) && (strtotime('now')-((int)$json['time']))>$this->threshold_time){
                @unlink($this->download_id_file);
                $downloading = false;
            }else if(!isset($json['time']))
                $downloading = false;
        }else
            $downloading = false;
        return $downloading;
    }
    
    /**
     * @brief Cancel downloading job
     * return boolean
     */
    public function cancel(&$os_wrapper){
        if(file_exists($this->download_id_file)){
            $json = (array)json_decode(file_get_contents($this->download_id_file));
            if(isset($json['file'])){                
                $rc = $os_wrapper->execute('ps ax | grep "'.$json['file'].'" | grep "wget \\-c" | grep -v "sh \\-c wget" | awk \'{print $1}\'',$pid,false);
                if($rc && is_array($pid) && $pid!=array()){
                    $pid = $pid[0];
                    $os_wrapper->execute("kill ".$pid);
                }
            }
        }
        return true;
    }

    /**
     * @brief Get downloading version
     * return string
     */ 
    public function downloading_version(){
        return file_exists($this->download_id_file) && ($json = (array)json_decode(file_get_contents($this->download_id_file))) && isset($json['version'])? $json['version'] : null;
    }
}

?>
