/*
  Menu handlers
*/
function onMenuClick(event, entry){
  //console.log(entry);
  if($(entry).hasClass('safe_menu_section')){
    var menuEntry = $('.safe_menu', $(entry));

    // Collapse all
    $('ul.safe_menu').each(function(){
        $(this).hide('blind');
        });
    $('.safe_menu_section_label').removeClass('safe_menu_selected');
    $('.safe_menu_section_label i').removeClass('fa-chevron-right fa-chevron-down');
    $('.safe_menu_section_label i').addClass('fa-chevron-right');

    // Expand selected
    menuEntry.show('blind');
    $('.safe_menu_section_label',$(entry)).addClass('safe_menu_selected');
    $('.safe_menu_section_label i',$(entry)).removeClass('fa-chevron-right fa-chevron-down');
    $('.safe_menu_section_label i',$(entry)).addClass('fa-chevron-down');
  }
  event.stopPropagation();
  resizeMenu();
}

function resizeMenu(){
    var menuContainer = $('#safe_left_menu');
    var header = $('#safe_header');
    var footer = $('#safe_footer');
    menuContainer.css({height: footer.position().top - header.height()});
}
function resizeWrapper(){
    var contentWrapper = $('#safe_content_wrapper');
    /* Only resize if one safe_content_wrapper */
    if(contentWrapper.length === 1){
      var footer = $('#safe_footer');
      var height = footer.position().top - contentWrapper.offset().top - 5;
      // TODO: Fix using debounce or throttle
      if(height > 50){
        contentWrapper.css({height: height});
      }
    }
}

/*
  Plot label formatters
*/
function plotPieLabelFormatterSmall(label, series) {
    return "<div style='font-size:7pt; text-align:center; padding:2px; color:white;'>" + Math.round(series.percent) + "%</div>";
}
function plotPieLabelFormatter(label, series) {
    return "<div style='font-size:8pt; text-align:center; padding:2px; color:white;'>" + label + "<br></br>" + Math.round(series.percent) + "%</div>";
}
/* Misc helper functions */
function ucFirst(string)
{
      return string.charAt(0).toUpperCase() + string.slice(1);
}

function bytesToSize(bytes) {
    var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
    if (bytes == 0) return 'N/A';
    var i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
    if (i == 0) return bytes + ' ' + sizes[i];
    return (bytes / Math.pow(1024, i)).toFixed(1) + ' ' + sizes[i];
};


var timers = new Array();               //all ajax request timers
function startAjaxTimer(timeoutCallback, timeoutValue) {
    timers[timeoutCallback.name] = setTimeout(timeoutCallback, timeoutValue);
    return timers[timeoutCallback.name];
}

var logoutUrl = '?reserved_logout=y';
var networkDialog = null;               //ajax request error dialog
var debugMode = false;

$.xhrPool = [];                         //ajax request pool
//stop all ajax
$.xhrPool.abortAll = function() {
    $.each(function(idx, jqXHR) {
        jqXHR.abort();
    });
    $.each(function(idx, jqXHR) {
        var index = $.inArray(jqXHR, $.xhrPool);
        if (index > -1) {
            $.xhrPool.splice(index, 1);
        }
    });
};
//save all ajax call into xhrPool
$.ajaxSetup({
    beforeSend: function(jqXHR, settings) {
        jqXHR.url = settings.url;
        $.xhrPool.push(jqXHR);
        if(debugMode){
            if(!$("#ajax_active").length){
                $(".safe_debug").append('<li><span class="safe_debug_label">Active request:</span><span id="ajax_active" class="safe_debug_value"></span></li>');
            }
            $("#ajax_active").html($.active);
        }
    },
    complete: function(jqXHR) {
        if(debugMode){
            if($("#ajax_active").length){
                $("#ajax_active").html($.active-1);
            }
        }
        var index = $.inArray(jqXHR, $.xhrPool);
        if (index > -1) {
            $.xhrPool.splice(index, 1);
        }
    }
});

// Error handler
function onDataError(xhr, textStatus){
    console.log('RX Error:' +  xhr.status + ' - ' + textStatus);
    //clear ajax request timers
    for (var x in timers) {
        clearTimeout(timers[x]);
    }
    //stop all active ajax request
    $.xhrPool.abortAll();
    var msg = '';
    if(xhr.status === 0) {
        msg = 'Network connection error.';
    }else if(textStatus === 'parsererror') {
        msg = 'Return data parser error.';
    }else if(textStatus === 'timeout') {
        msg = 'Request timeout.';
    }else if(textStatus === 'abort') {
        msg = '';
    }else{
        msg = xhr.statusText;
    }
    if(xhr.status == 403){
        window.location = logoutUrl;
    }else{
        //popup a dialog
        var tmp_refresh = function(){location.reload();};
        var close_method = {
                "dialogbeforeclose": tmp_refresh
            };
        if (networkDialog == null && msg != '') {
            networkDialog = safe_message_box('Get data error!', safe_message_style(xhr.status + " - " + msg, 'alert'), close_method, 'Refresh', tmp_refresh);
        }
    }
}

/*
  Configuration status
*/
function updateConfigurationStatus(data, params){
  params = params || {inital:false};
  // Get container
  var cfgDialog = $('#config_status_dialog');
  var cfgContainer = $('#config_status');
  cfgDialog.empty();
  cfgContainer.find( "i" ).removeClass('fa-spin');
  //Reboot request ?
  if(data.hasOwnProperty('reboot')){
    // Update Reboot top container class so it blinks and appear in warning
    // color
    var rebootContainer = $('#reboot_status');
    rebootContainer.addClass('safe_blink safe_warning');
    // Add tooltip so user knows why
    // TODO: this is a poor tooltip implementation and it deserves a proper
    // tooltip handling and maybe modifying reboot container option so it
    // presents a nice table with reboot request reason and the reboot button
    // only.
    var rebootReason = "";
    $.each(data.reboot, function(i, val){
        console.log(val);
        $.each(val, function(module, reason){
          rebootReason = rebootReason + "\n" + reason;
          });
    });
    rebootContainer.attr('title',
        "System Reboot Required:" +
        rebootReason
        );
    return;
  }else{
      var rebootContainer = $('#reboot_status');
      rebootContainer.removeClass('safe_blink safe_warning');
      rebootContainer.attr('title','');
  }
  
  if(data.status == true){
    cfgContainer.removeClass('safe_error safe_warning');
    // Add title
    cfgDialog.append(
      "<p class='safe_stats_table_title'>"+data.description+"</p");
    if(!data.hasOwnProperty('reboot')){
        return;
    }
  }else{
    cfgContainer.addClass('safe_' + data.status_text);
  }
  // Checklist ?
  if(data.hasOwnProperty('checklist')){
    // Add title
    cfgDialog.append(
      "<p class='safe_stats_table_title'>"+data.checklist.description+"</p");
    var cfgChecklistTable;
    cfgChecklistTable = $(
      "<table class='safe_status_table'></table>");
    cfgDialog.append(cfgChecklistTable);

    // Append checklist rows
    $.each(data.checklist.items, function(key, value){
      var action = 'configure';
      if(value.hasOwnProperty('action')){
        action = value.action;
      }
      cfgChecklistTable.append(
        "<tr>" +
        "<td class='safe_"+data.status_text+"'>" + "<span><i class='fa fa-exclamation-triangle'></i></span>" + "</td>" +
        "<td class='safe_stats_table_label'>" +  value.description + "</td>"+
        "<td>" +
        "<form action='"+value.url+"'>" +
        "<input value='"+ucFirst(action)+"' type='submit' class='ui-state-default ui-corner-all'>"+
        "</form>"+
        "</tr>");

      });
  }

  // Apply, Reload or Restart ?
  var cfgTable = null;
  var cfgControl = null;
  if(data.hasOwnProperty('reload') || data.hasOwnProperty('restart') || data.hasOwnProperty('apply')){
    // Add title
    cfgDialog.append(
      "<p class='safe_stats_table_title'>"+data.description+"</p>");
    // Add table
    cfgTable = $(
      "<div class='modified_configuration'><table class='safe_status_table'><tr id='tfoot'><td></td><td></td><td></td></tr></table></div>");
    cfgDialog.append(cfgTable);
    // Add Config Control for button
    cfgControl = $(
      "<div class='safe_status_table_footer'></div>");
    cfgDialog.append(cfgControl);
  }
  var footer = $(".safe_status_table #tfoot");
  // Reload ?
  if(data.hasOwnProperty('reload')){
    // Append checklist rows

    $.each(data.reload.items, function(key, value){
      footer.before(
        "<tr>" +
        "<td class='safe_info'>" + "<span><i class='fa fa-refresh'></i></span>" + "</td>" +
        "<td class='safe_stats_table_label' style='max-width:90%'>" +  value.description + "</td>"+
        "<td class='safe_stats_table_label'>" +  value.status + "</td>"+
        "</form>"+
        "</tr>");

      });
    // Add button control
    if(!data.hasOwnProperty('display') || data.display !== false){
        cfgControl.append(
                "<div class='safe_status_table_footer_item'>" +
                "<form action='"+data.reload.url +"/"+data.reload.action+"'>" +
                "<input value='"+ucFirst(data.reload.action)+"' type='submit' class='ui-state-default ui-corner-all'>"+
                "</form>"+
                "</div>");
    }
    
  }
  // Restart ?
  if(data.hasOwnProperty('restart')){
    // Append checklist rows
    $.each(data.restart.items, function(key, value){
        footer.before(
        "<tr>" +
        "<td class='safe_"+data.status_text+"'>" + "<span><i class='fa fa-exclamation-triangle'></i></span>" + "</td>" +
        "<td class='safe_stats_table_label' style='max-width:90%'>" +  value.description + "</td>"+
        "<td class='safe_stats_table_label'>" +  value.status + "</td>"+
        "</form>"+
        "</tr>");
      });
    // Add button control
    if(!data.hasOwnProperty('display') || data.display !== false){
        cfgControl.append(
            "<div class='safe_status_table_footer_item'>" +
            "<form action='"+data.restart.url +"/"+data.restart.action+"'>" +
            "<input value='"+ucFirst(data.restart.action)+"' type='submit' class='ui-state-default ui-corner-all'>"+
            "</form>" +
            "</div>");
    }
  }
  // Apply ?
  if(data.hasOwnProperty('apply')){
    // Append checklist rows
    $.each(data.apply.items, function(key, value){
        footer.before(
        "<tr>" +
        "<td class='safe_"+data.status_text+"'>" + "<span><i class='fa fa-exclamation-triangle'></i></span>" + "</td>" +
        "<td class='safe_stats_table_label' style='max-width:90%' >" +  value.description + "</td>"+
        "<td class='safe_stats_table_label'>" +  value.status + "</td>"+
        "</form>"+
        "</tr>");
      });
    // Add button control
    if(!data.hasOwnProperty('display') || data.display !== false){
        cfgControl.append(
            "<div class='safe_status_table_footer_item'>" +
            "<form action='"+data.apply.url +"/"+data.apply.action+"'>" +
            "<input value='"+ucFirst(data.apply.action)+"' type='submit' class='ui-state-default ui-corner-all'>"+
            "</form>" +
            "</div>");
    }
  }
    var adjustMenu = function(){
        // Adjust rigth postion (offset to ensure all div is visible)
        // Get top menu parent (label) width and right offset
        parentWidth = $('#config_status').outerWidth();
        parentPosition = $('#config_status').position();
        parentOffset = $('#config_status').offset();
        windowWidth = $(window).width();
        viewSizeRight = windowWidth - (parentOffset.left)? parentOffset.left : 0;
        
        // container should be ideally aligned on the left side of parent
        dialogWidth = $('#config_status_dialog').outerWidth();
        var menuWidth = $('#safe_left_menu').outerWidth();
        
        if(parentOffset.left+dialogWidth<windowWidth){
            cfgDialog.css('left',parentOffset.left-menuWidth);
        }else if(parentOffset.left+(dialogWidth*.5)<windowWidth){
            cfgDialog.css('left',((parentOffset.left-menuWidth*.5)-(dialogWidth*.5)));
        }else{
            cfgDialog.css('left',(parentOffset.left-dialogWidth));
        }    
    }
    adjustMenu();
    // This is a bit hack, but we should chage the way javascript works here. As
    // we execute this function each time we request the server for configuration
    // data, and it takes a few minutes between request, in that time user can
    // resize the window and we can't wait for next configuration request to
    // adjust the menu. So in order to don't have a lot of resize handlers, we
    // first make sure adjustMenu function is not binded to window resize event,
    // and then we bind it back to make sure there is just one adjustMenu handler
    $(window).off('resize',adjustMenu);
    $(window).on('resize',adjustMenu);    
}
var gFirstConfigutationStatus = true;
function fetchConfigurationStatus(params){
  if (typeof gConfigurationStatusTimeout === 'undefined') {
    timeoutSec = 25;
  }else{
    timeoutSec = gConfigurationStatusTimeout;
  }
  params = params || {async:true};
  function onDataReceived(response){
    //console.log('RX' + JSON.stringify(data));
    updateConfigurationStatus(response.data, params);
    startAjaxTimer(fetchConfigurationStatus, timeoutSec * 1000);
    $($('div.modified_configuration')[0]).css('max-height', $(window).height() - 200 + 'px');
  }
  var cfgContainer = $('#config_status');
  if(gFirstConfigutationStatus){
    cfgContainer.find( "i" ).addClass('fa-spin');
    gFirstConfigutationStatus = false;
  }
  $.ajax({
    url:'/SAFe/rest/api/status/application/configuration',
    type: 'GET',
    dataType:'json',
    async:params.async,
    success:onDataReceived,
    error:onDataError
    });
}

function displayServicesStatus(services){
    var tableBody = $('#sng_service_stats_table');
    var i = 0;
    tableBody.empty();
    $.each(services, function(index, entry){
        if(entry.hasOwnProperty('cpu')){
            var value = {
                percent: entry.cpu.percent,
                color: '#6699FF'
                };
            var cpuCell =
                "<td><div class='safe_stats_table_value safe_stats_table_value_progress' " +
                "style='background:-moz-linear-gradient(left center, "+
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%, " +
                "#FFFFFF "+parseFloat(value.percent).toFixed(0) + "%); "+
                "background:-o-linear-gradient(left center, "+
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%, " +
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%); "+
                "background-image:-ms-linear-gradient(left, "+
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%, " +
                "#FFFFFF "+parseFloat(value.percent).toFixed(0) + "%, " +
                "#FFFFFF 100%); "+
                "background:-webkit-gradient(linear, left top, right top, "+
                "color-stop("+parseFloat(value.percent).toFixed(0) + "%, " +
                value.color+"), "+
                "color-stop("+parseFloat(value.percent).toFixed(0) + "%, " +
                "#FFFFFF)); "+
                "background:linear-gradient(left center, "+
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%, " +
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%); "+
                "'>" +
                parseFloat(value.percent).toFixed(1) + "%" +
                "</div>" +
                "</td>";
        }else{
            var cpuCell =
                "<td> <div class='safe_stats_table_value safe_stats_table_value_progress disabled'>" +
                "N/A" +
                "</div></td>";
        }
        if(entry.hasOwnProperty('mem')){
            var value = {
                percent: entry.mem.percent,
                kb: parseFloat(entry.mem.kb) * 1024,
                color: '#6699FF'
                };
            var memCell =
                "<td> <div class='safe_stats_table_value safe_stats_table_value_progress'" +
                "style='background:-moz-linear-gradient(left center, "+
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%, " +
                "#FFFFFF "+parseFloat(value.percent).toFixed(0) + "%); "+
                "background:-o-linear-gradient(left center, "+
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%, " +
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%); "+
                "background-image:-ms-linear-gradient(left, "+
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%, " +
                "#FFFFFF "+parseFloat(value.percent).toFixed(0) + "%, " +
                "#FFFFFF 100%); "+
                "background:-webkit-gradient(linear, left top, right top, "+
                "color-stop("+parseFloat(value.percent).toFixed(0) + "%, " +
                value.color+"), "+
                "color-stop("+parseFloat(value.percent).toFixed(0) + "%, " +
                "#FFFFFF)); "+
                "background:linear-gradient(left center, "+
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%, " +
                value.color+" "+parseFloat(value.percent).toFixed(0) + "%); "+
                "' title='" + parseFloat(value.percent).toFixed(2)+ " %'>" +
                bytesToSize(value.kb) +
                "</div>" +
                "</td>";
        }else{
            var memCell =
                "<td><div class='safe_stats_table_value safe_stats_table_value_progress disabled'>" +
                "N/A" +
                "</div>" +
                "</td>";
        }
        tableBody.append(
            "<tr id='service_" + index + "_row'>" +
                "<td id='service_" + index + "_name'" + "class='" + (0==((i++)%2)?"even":"odd") + "'>" +entry.description + "</td>" +
                "<td id='service_" + index + "_status' class='safe_status_table_value safe_status_"+entry.status_text.replace(" ", "_").toLowerCase()+"'>" + entry.status_text + "</td>" +
                cpuCell +
                memCell +

            "</tr>");
    });
}
/*
  Application status
*/
function updateApplicationStatus(status, params){
  params = params || {inital:false};
  var systemContainer = $('#application_status');
  /*
  if(status.system.status == 'ok'){
    systemContainer.removeClass('safe_error safe_warning');
  }else{
    systemContainer.addClass('safe_' + status.system.status);
  }
  */

  var serviceDialog = $('#application_status_dialog');
  serviceDialog.empty();

  $.each(status, function(key, value){
    var categoryTable = $('#service_category_'+ value.category.replace(/ /g,'') +' .safe_status_table', serviceDialog);

    if( 0 == categoryTable.length){
      // Create div
      var categoryDiv = $(
        "<div id='service_category_" +  value.category.replace(/ /g,'')  + "'>" +
        "<p class='safe_stats_table_title'>"+value.category+"</p>"+
        "</div>"
        );
      categoryTable = $(
        "<table class='safe_status_table'/>");
      categoryDiv.append(categoryTable);
      serviceDialog.append(categoryDiv);
    }

    categoryTable.append(
        "<tr>" +
        "<td class='safe_status_table_label'>" + value.description + "</td>" +
        "<td class='safe_status_table_value safe_status_"+value.status_text.replace(" ", "_").toLowerCase() +"'" +
        ">" +
        value.status_text +
        "</td>" +
        "</tr>"
      );
    });
}
function fetchApplicationStatus(params){
  params = params || {async:true};
  function onDataReceived(response){
    //console.log('RX' + JSON.stringify(data));
    displayServicesStatus(response.data);
    updateApplicationStatus(response.data);
    startAjaxTimer(fetchApplicationStatus, 25 * 1000);
  }
  $.ajax({
    url:'/SAFe/rest/api/status/system/service',
    type: 'GET',
    dataType:'json',
    async:params.async,
    success:onDataReceived,
    error:onDataError
    });
}

/*
  System status
*/
function updateSystemStatus(status){
    try{
      // Top bar update
    var placeholder = $("#top_plot_pie_system_cpu");
    $.plot(placeholder, status.series.cpu, {
      series: {
        pie: {
          show: true,
            radius: 1,
            label: {
              show: true,
              radius: 2/3,
              formatter: plotPieLabelFormatterSmall,
              threshold: 0.1
            },
          stroke: {
            width: 0
          }
        }
      },
      legend: {
        show: false,
      }
    });
    var placeholder = $("#top_plot_pie_system_memory");
    $.plot(placeholder, status.series.memory, {
      series: {
        pie: {
          show: true,
            radius: 1,
            label: {
              show: true,
              radius: 2/3,
              formatter: plotPieLabelFormatterSmall,
              threshold: 0.1
            },
          stroke: {
            width: 0
          }
        }
      },
      legend: {
        show: false,
      }
    });
    // Update top dialog
    var topDialogTable = $('#system_cpu_details .safe_stats_table');
    topDialogTable.empty();
    $.each(status.series.cpu, function(key, value){
      topDialogTable.append(
        "<tr>" +
        "<td class='safe_stats_table_label'>" + value.label + "</td>" +
        "<td class='safe_stats_table_value safe_stats_table_value_progress' align='center' " +
        "style='background:-moz-linear-gradient(left center, "+
        value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
        "#FFFFFF "+parseFloat(value.data).toFixed(0) + "%); "+
        "background:-o-linear-gradient(left center, "+
        value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
        value.color+" "+parseFloat(value.data).toFixed(0) + "%); "+
        "background:-webkit-gradient(linear, left top, right top, "+
        "color-stop("+parseFloat(value.data).toFixed(0) + "%, " +
        value.color+"), "+
        "color-stop("+parseFloat(value.data).toFixed(0) + "%, " +
        "#FFFFFF)); "+
        "background-image:-ms-linear-gradient(left, "+
        value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
        "#FFFFFF "+parseFloat(value.data).toFixed(0) + "%, " +
        "#FFFFFF 100%); "+
        "background:linear-gradient(left center, "+
        value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
        value.color+" "+parseFloat(value.data).toFixed(0) + "%); "+
        "'>" +
        parseFloat(value.data).toFixed(0) + "%" +
        "</td>" +
        "</tr>");

    });
    var topDialogTable = $('#system_memory_details .safe_stats_table');
    topDialogTable.empty();
    $.each(status.series.memory, function(key, value){
      topDialogTable.append(
        "<tr>" +
        "<td class='safe_stats_table_label'>" + value.label + "</td>" +
        "<td class='safe_stats_table_value safe_stats_table_value_progress' align='center' " +
        "style='background:-moz-linear-gradient(left center, "+
        value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
        "#FFFFFF "+parseFloat(value.data).toFixed(0) + "%); "+
        "background:-o-linear-gradient(left center, "+
        value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
        value.color+" "+parseFloat(value.data).toFixed(0) + "%); "+
        "background-image:-ms-linear-gradient(left, "+
        value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
        "#FFFFFF "+parseFloat(value.data).toFixed(0) + "%, " +
        "#FFFFFF 100%); "+
        "background:-webkit-gradient(linear, left top, right top, "+
        "color-stop("+parseFloat(value.data).toFixed(0) + "%, " +
        value.color+"), "+
        "color-stop("+parseFloat(value.data).toFixed(0) + "%, " +
        "#FFFFFF)); "+
        "background:linear-gradient(left center, "+
        value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
        value.color+" "+parseFloat(value.data).toFixed(0) + "%); "+
        "'>" +
        parseFloat(value.data).toFixed(0) + "%" +
        "</td>" +
        "</tr>");

    });

    var topDialogTable = $('#network_status_details .safe_stats_table');
    networkStatus = true;
    topDialogTable.empty();
    topDialogTable.append(
        "<tr>" +
        "<th class='safe_stats_table_title '></th>" +
        "<th class='safe_stats_table_title '>Link</th>" +
        "<th class='safe_stats_table_title '>Status</th>" +
        "<th class='safe_stats_table_title '>TX</th>" +
        "<th class='safe_stats_table_title '>RX</th>" +
        "</tr>"
        );
    $.each(status.stats.network.interface, function(if_name, if_stats){
      stats = if_stats.stats;
      if_status = if_stats.operstate.toLowerCase();
      if_status_class = "safe_info";
      if(if_status == "unknown"){
        // loopback interface reports unknown
        if_status = 'up';
      }
      if(if_status == "down" || if_status == 'lowerlayerdown'){
        if_status_class = "safe_error";
        networkStatus = false;
      }
      if_status = if_status.toUpperCase();
      // Check carrier
      if_carrier_class = "safe_info";
      if(1 != parseInt(if_stats.carrier)){
        if_carrier_class = "safe_error";
        networkStatus = false;
      }
      topDialogTable.append(
        "<tr>" +
        "<td class='safe_stats_table_label'>" + if_name + "</td>" +
        "<td class='safe_stats_table_value " + if_carrier_class + "'><span><i class='fa fa-circle'></i></span></td>" +
        "<td class='safe_stats_table_value " + if_status_class + "'>" + if_status + "</td>" +
        "<td class='safe_stats_table_value '>" +
        bytesToSize(stats.tx_bytes) +
        "</td>" +
        "<td class='safe_stats_table_value '>" +
        bytesToSize(stats.rx_bytes) +
        "</td>" +
        "</tr>");

      // Update network summary if needed
         //if_name have '.' in it, jquery need to fix the selector
        networkSummaryCell = $('#network_interface_'+if_name.replace('.','\\.')+'_status');
        if(networkSummaryCell){
          if(if_status == "UP"){
              //lo no duplex
            tmp_status = "<span><i class='fa fa-circle " + if_status_class +"'></i></span> " + if_status;
            if(typeof if_stats.duplex !== 'undefined'  && null !== if_stats.duplex){
                tmp_status =  tmp_status + " " + if_stats.speed + " MB/s " + if_stats.duplex.charAt(0).toUpperCase() + if_stats.duplex.slice(1) + "-Duplex";
            }
            networkSummaryCell.html(tmp_status);
          }else{
            networkSummaryCell.html(
                "<span><i class='fa fa-circle " + if_status_class +"'></i></span> " + if_status
                );
          }
        }
    });
    $.each(status.stats.network.ip, function(ip_name, ip_address){
        // Update ip summary if needed
          ipSummaryCell = $('#network_ip_'+ip_name);
          if(ipSummaryCell){
              if(ip_address != ''){
                ipSummaryCell.html(ip_address);
              }
          }
      });

    // Is network OK ?
    if(!networkStatus){
      $('#network_status').addClass('safe_error');
    }else{
      $('#network_status').removeClass('safe_error');
    }


    // dashboard update if present
    var placeholder = $("#plot_pie_system_cpu");
    if(placeholder.length){
      $.plot(placeholder, status.series.cpu, {
        series: {
          pie: {
            show: true,
              radius: 1,
              label: {
                show: true,
                radius: 2/3,
                formatter: plotPieLabelFormatter,
                threshold: 0.1
              }
          }
        },
        legend: {
          show: true,
          container: $("#system_cpu_legend"),
          noColumns:1
        }
      });
    }
    var placeholder = $("#plot_pie_system_memory");
    if(placeholder.length){
      $.plot(placeholder, status.series.memory, {
        series: {
          pie: {
            show: true,
              radius: 1,
              label: {
                show: true,
                radius: 2/3,
                formatter: plotPieLabelFormatter,
                threshold: 0.1
              }
          }
        },
        legend: {
          show: true,
          container: $("#system_memory_legend"),
          noColumns:1
        }
      });
    }
    }catch(err){
      console.log(err.message);
    }
}
/*
Storage status
*/
var recovery = false;
var reload_page = false;
function updateStorageStatus(status){
    try{
        var topDialogTable = $('#storage_fs_details .safe_stats_table');
        topDialogTable.empty();
        $.each(status.series.fs, function(fs_name, fs_stats){

          $.each(fs_stats, function(key, value){
            if('Used' == value.label){
              // set color to blue
              value.color = '#6699FF';
              topDialogTable.append(
                "<tr>" +
                "<td class='safe_stats_table_label'>" + fs_name + "</td>" +
                "<td class='safe_stats_table_value safe_stats_table_value_progress' align='center' " +
                "style='background:-moz-linear-gradient(left center, "+
                value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
                "#FFFFFF "+parseFloat(value.data).toFixed(0) + "%); "+
                "background:-o-linear-gradient(left center, "+
                value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
                value.color+" "+parseFloat(value.data).toFixed(0) + "%); "+
                "background-image:-ms-linear-gradient(left, "+
                value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
                "#FFFFFF "+parseFloat(value.data).toFixed(0) + "%, " +
                "#FFFFFF 100%); "+
                "background:-webkit-gradient(linear, left top, right top, "+
                "color-stop("+parseFloat(value.data).toFixed(0) + "%, " +
                value.color+"), "+
                "color-stop("+parseFloat(value.data).toFixed(0) + "%, " +
                "#FFFFFF)); "+
                "background:linear-gradient(left center, "+
                value.color+" "+parseFloat(value.data).toFixed(0) + "%, " +
                value.color+" "+parseFloat(value.data).toFixed(0) + "%); "+
                "'>" +
                parseFloat(value.data).toFixed(0) + "%" +
                "</td>" +
                "</tr>");

             // Update fs summary if needed
             fsSummaryCell = $('#fs_'+fs_name.replace(/\//g,'_'));
             if(fsSummaryCell){
                 fs_used = parseFloat(value.data).toFixed(0);
                 tmp_status = "<div class='safe_stats_table_value safe_stats_table_value_progress' " +
                               "style='margin-top: 5px;margin-bottom: 5px;background:-moz-linear-gradient(left center, "+ value.color + " "+ fs_used + "%, #FFFFFF "+ fs_used + "%); " +
                               "background:-o-linear-gradient(left center, "+ value.color + " "+ fs_used + "%, "+ value.color + " "+ fs_used + "%); " +
                               "background-image:-ms-linear-gradient(left, "+ value.color + " "+ fs_used + "%, #FFFFFF "+ fs_used + "%, #FFFFFF 100%); " +
                               "background:-webkit-gradient(linear, left top, right top, color-stop("+ fs_used + "%, "+ value.color + "), " +
                               "color-stop("+ fs_used + "%, #FFFFFF)); background:linear-gradient(left center, "+ value.color + " "+ fs_used + "%, "+ value.color + " "+ fs_used + "%);' " +
                               ">"+ fs_used + "%</div>";
                 fsSummaryCell.html(tmp_status);
             }
            }
          });
        });

        storage_status = true;
        recovery = false;
        var topDialogTable = $('#storage_raid_details .safe_stats_table');
        topDialogTable.empty();
        if(typeof status.stats.raid !== 'undefined'){
            $.each(status.stats.raid, function(raid_name, raid_stats){
                raid_status = raid_stats.state.toLowerCase();
                raid_status_class = "safe_info";
                if(raid_status != "normal"){
                  raid_status_class = "safe_error";
                  storage_status = false;
                }
                raid_status = raid_status.toUpperCase();
                topDialogTable.append(
                  "<tr>" +
                  "<td width='10%'>&nbsp;</td>" +
                  "<td class='safe_stats_table_label'>" + raid_name + "</td>" +
                  "<td class='safe_stats_table_value " + raid_status_class + "'><span><i class='fa fa-circle'></i></span></td>" +
                  "<td class='safe_stats_table_value " + raid_status_class + "'>" + raid_status + "</td>" +
                  "</tr>");

                // Update raid summary if needed
                raidSummaryCell_status = $('#raid_' + raid_name + '_status');
                raidSummaryCell_percent = $('#raid_' + raid_name + '_percent');
                if(raidSummaryCell_status){
                    if(raid_status == 'RECOVERY'){
                        recovery = true;
                        reload_page = true;
                        raid_stats.progress = raid_stats.progress.replace('%','');
                        percent = parseFloat(raid_stats.progress).toFixed(0);
                        color = '#6699FF';
                        tmp_percent = "<div class='safe_stats_table_value safe_stats_table_value_progress' " +
                        "style='margin-top: 5px;margin-bottom: 5px;background:-moz-linear-gradient(left center, "+ color + " "+ percent + "%, #FFFFFF "+ percent + "%); " +
                        "background:-o-linear-gradient(left center, "+ color + " "+ percent + "%, "+ color + " "+ percent + "%); " +
                        "background-image:-ms-linear-gradient(left, "+ color + " "+ percent + "%, #FFFFFF "+ percent + "%, #FFFFFF 100%); " +
                        "background:-webkit-gradient(linear, left top, right top, color-stop("+ percent + "%, "+ color + "), " +
                        "color-stop("+ percent + "%, #FFFFFF)); background:linear-gradient(left center, "+ color + " "+ percent + "%, "+ color + " "+ percent + "%);' " +
                        ">"+ percent + "%</div>";
                        raidSummaryCell_percent.html(tmp_percent);
                    }else{
                        raidSummaryCell_percent.html('');
                    }
                    tmp_arry_status = "<i class='fa fa-circle fa-fw  " + raid_status_class + "'></i>" + raid_status;
                    raidSummaryCell_status.html(tmp_arry_status);
                }
              });
            //reload page after RECOVERY
            if(!recovery && reload_page ){
                location.reload();
            }
            if(!storage_status){
                $('#storage_status').addClass('safe_error');
            }else{
                $('#storage_status').removeClass('safe_error');
            }
        }else{
            //update title
            var raidDialogTitle = $('#storage_raid_details .safe_stats_table_title');
            raidDialogTitle.html('RAID is not available');
        }
    }catch(err){
      console.log(err.message);
    }
}

/*
  Update Checker status
*/ 

function updateCheckerStatus(response){
    var updateContainer = $('#update_status_dialog'),
	updateItemMenu = $('#update_checker_status'),
	refresh = updateItemMenu.find('.fa-refresh'),
	table = updateContainer.find('table'),
	tableInner = '<tr>';
    tableInner+='<td width="10%">&nbsp;</td>'+
        '<td class="safe_stats_table_label">Current Version:</td>'+
        '<td class="safe_stats_table_value" style="text-align:right"><span>'+response.current+'</span></td>'+
        '</tr>';
    if(response.new_version){
        tableInner+='<tr>'+
            '<td width="10%">&nbsp;</td>'+
            '<td class="safe_stats_table_label">New Version:</td>'+
            '<td class="safe_stats_table_value" style="text-align:right"><span>'+response.new_version+'</span></td>'+
            '</tr>';
    }
    if(response.is_greater){
        refresh.removeClass('fa-spin').addClass('fa-spin');
        updateItemMenu.parent().addClass('update-available');//css('color','#FFB56C');        
        setTimeout(function(){
                refresh.removeClass('fa-spin');
            },2000); // a half minute
        if(window.location.href.match(/sng_update_manager_alias/)){
            if(response.status=='downloading')
                $('td.mytablesubheader').each(function(){
                        if($(this).html()=='Status'){
                            var capital = response.status[0].toUpperCase()+response.status.split('').splice(1).join('');                        
                            $(this).next().html(capital+' '+response.rate+'%');                        
                            if(response.rate>=100)
                                setTimeout(function(){window.location.reload();},5000);
                            else
                                setTimeout(fetchUpdateCheckerStatus,10000);
                        }
                    });
            else if($('form input[value="Cancel"]').length)
                window.location.reload();
        }
    }
    table.html(tableInner);
}

function fetchUpdateCheckerStatus(params){
    params = params || {async:true};
    
    function onDataReceived(response){
        if(response.status)
            updateCheckerStatus(response.data);
    }

    function checkAvailablePackage(){
        $.ajax({
                url:'/SAFe/sng_rest/api/check/update/package',
                    type:'GET',
                    dataType:'json',
                    async:params.async,
                    success:onDataReceived,
                    error:onDataError
                    });
    }

    checkAvailablePackage();

    if(window.location.href.match(/sng_update_manager_alias/)){
        var location = window.location,
            callbacks = {},
            replaces = {'download':'cancel','cancel':'download'};
            // Downloading asynchronous emulation with timeout and refreshing
        $(['download','cancel']).each(function(i,key){
                callbacks[key] = function(){                    
                    var form = $(this),
                    rep = replaces[key],
                    action = form.attr('action'),
                    capital = rep[0].toUpperCase()+rep.split('').splice(1).join(''),
                    reg = new RegExp(rep);
                    $('input',form).val(capital);
                    console.log(action);
                    form.attr('action',action.replace(key,rep));
                }
            });
        $('form input[value="Download"]').each(function(){
                var form = $(this).parent();                
                form.on('submit',function(){
                        var form = $(this),
                            action = $('input',this).val().toLowerCase(),
                            rep = replaces[action];
                        $.ajax({
                                url:$(this).attr('action'),
                                    type:'GET',
                                    async:true,
                                    timeout:1000,
                                    // timeot launch error callback 
                                    error:function(){
                                    callbacks[action].call(form);
                                    window.location = location;
                                }
                            });
                        return false;
                    });
            });
        $('form input[value="View"]').each(function(){
                var form = $(this).parent();
                form.on('submit',function(){
                        var wtarget = $('#safe_content');
                        wtarget.addClass('progress');
                        $.ajax({
                                url:$(this).attr('action')+'/ajax',
                                    type:'GET',
                                    async:true,
                                    dataType:'json',
                                    success:function(result){                                    
                                    var width = $(window).width(),
                                        height = $(window).height(),
                                        dialog = safe_message_box(result.title,result.content.replace(/\n/g,'<br/>')),
                                        top = (height*0.5)-(dialog.height()*0.5),
                                        left = (width*0.5)-(dialog.width()*0.5);
                                    dialog.parent().css({top:top,left:left});
                                    wtarget.removeClass('progress');
                                }
                            });
                        return false;
                    });
                
            });
    }
    
}

function fetchSystemStatus(params){
  params = params || {async:true};

  function onDataReceived(response){
    //console.log('RX' + JSON.stringify(data));
    updateSystemStatus(response.data);
    startAjaxTimer(fetchSystemStatus, 45 * 1000);
  }
  $.ajax({
    url:'/SAFe/rest/api/status/system/os',
    type: 'GET',
    dataType:'json',
    async:params.async,
    success:onDataReceived,
    error:onDataError
    });
}
function fetchStorageStatus(params){
    params = params || {async:true};

    function onDataReceived(response){
      updateStorageStatus(response.data);
      //when recovery is true, update the page ever 5 second
      if(recovery == true){
          delay_time = 5;
      }else{
          delay_time = 60;
      }
      startAjaxTimer(fetchStorageStatus, delay_time * 1000);
    }
    $.ajax({
      url:'/SAFe/rest/api/status/system/storage',
      type: 'GET',
      dataType:'json',
      async:params.async,
      success:onDataReceived,
      error:onDataError
      });
  }
/*
  Events History
*/
function updateEventsHistory(data_events){
    try{
    var container = $("#plot_bar_system_events");
    if(container.length){
      $.plot(container, data_events.events, {
          xaxis: {
              mode: "time",
              timezone: "browser",
              minTickSize: [1, "hour"],
              twelveHourClock: true,
              min: dateMin.getTime(),
              max: dateMax.getTime()
          },
        series: {
          stack: true,
          bars: {
              show: true,
              align: "right",
              barWidth: 60*60*1000
          },
          lines: {
              show: false
          }

        },
        legend: {
          show: true,
          container: $("#plot_bar_system_events_legend"),
          noColumns:3
        }
      });
    }
    }catch(err){
      console.log('Events update exception' + err.message);
    }
}
// A custom label formatter used by several of the plots
function plotPieLabelFormatter(label, series) {
    return "<div style='font-size:8pt; text-align:center; padding:2px; color:white;'>" + label + "<br></br>" + Math.round(series.percent) + "%</div>";
}

function renderEvents(data_events){
    try{
    var placeholder = $("#plot_bar_system_events");
    var dateMax = new Date(data_events.current_date);
    var dateMin = new Date(dateMax - (data_events.hour_range*60*60*1000));
    $.plot(placeholder, data_events.events, {
        xaxis: {
            mode: "time",
            timezone: "browser",
            minTickSize: [1, "hour"],
            twelveHourClock: true,
            min: dateMin.getTime(),
            max: dateMax.getTime()
        },
      series: {
        stack: true,
        bars: {
            show: true,
            align: "right",
            barWidth: 60*60*1000
        },
        lines: {
            show: false
        }

      },
      legend: {
        show: true,
        container: $("#system_events_legend"),
        noColumns:1
      }
    });
    }catch(exp){
        console.log(exp);
    }
}
function fetchEventsHistory(params){
  params = params || {async:true};
    //  Check if needed
    var container = $("#plot_bar_system_events");
    if(container.length){
      function onDataReceived(data){
          renderEvents(data);
          updateEventsHistory(data.summary);
          startAjaxTimer(fetchEventsHistory, 60 * 1000);
      }
      $.ajax({
          url:"/SAFe/rest/api/history/monitor/event",
              type: "GET",
              dataType:"json",
              async: params.async,
              success:onDataReceived,
              error:onDataError
      });
    }
}
/*
  Events Status
*/
function updateEventsStatus(data){
    // Update top dialog
    var topDialogTable = $('#notification_status_details .safe_stats_table');
    topDialogTable.empty();
    $.each(data, function(key, value){
      topDialogTable.append(
        "<tr>" +
        "<td class='safe_stats_table_label'>" + key + "</td>" +
        "<td class='safe_stats_table_value' style='width:20%;'>" +
        value.info +
        "</td>" +
        "<td class='safe_stats_table_value' style='width:20%;'>" +
        value.error +
        "</td>" +
        "<td class='safe_stats_table_value' style='width:20%;'>" +
        value.critical +
        "</td>" +
        "</tr>");

    });
}
function fetchEventsStatus(params){
  params = params || {async:true};
    function onDataReceived(response){
        updateEventsStatus(response.data);
        startAjaxTimer(fetchEventsStatus, 65 * 1000);
    }
    $.ajax({
        url:"/SAFe/rest/api/status/monitor/event",
            type: "GET",
            dataType:"json",
            async: params.async,
            success:onDataReceived,
            error:onDataError
    });
}
/*
  Breadcrumb
*/
function updateBreadCrumb(){
    // Check if breadcumb need to be adjusted
    try{
    if(typeof safe_breadcrumb_data !== 'undefined' && null !== safe_breadcrumb_data){
      var _breadcrumb = $('#safe_breadcrumb .safe_breadcrumb');
      $.each(JSON.parse(safe_breadcrumb_data), function(key, value){
        if(null !== value){
          console.log(value.label);
          console.log(value.url);
          // Append to breadcrumb
          if(undefined !== value.url){
            _breadcrumb.append(
              "<li>" +
              "<span class='divider'>/</span>" +
              "<a href='" + value.url + "'>" + value.label + "</a>" +
              "</li>");
          }else{
            _breadcrumb.append(
              "<li>" +
              "<span class='divider'>/</span>" +
              "<span>" + value.label + "</span>" +
              "</li>");
          }
        }
      });
    }
    } catch(err){
      console.log('breadcrump update exception' + err.message);
    }
}
/**
 * @brief
 *
 * @return
 */
function fetchDebugInfo(){

  if (performance.timing.loadEventEnd){
    var networkTiming, serverTiming, browserTiming;

    networkTiming = performance.timing.requestStart - performance.timing.navigationStart;
    serverTiming = performance.timing.responseEnd - performance.timing.requestStart;
    browserTiming = performance.timing.loadEventEnd - performance.timing.responseEnd;

    $(".safe_debug").append('<li><span class="safe_debug_label">Network:</span><span class="safe_debug_value">' + networkTiming + ' ms</span></li>');
    $(".safe_debug").append('<li><span class="safe_debug_label">Server:</span><span class="safe_debug_value">' + serverTiming + ' ms</span></li>');
    $(".safe_debug").append('<li><span class="safe_debug_label">Browser:</span><span class="safe_debug_value">' + browserTiming + ' ms</span></li>');
  }else{
    startAjaxTimer(fetchDebugInfo, 100);
  }

}
/**
 * @brief Warning message to display beside an input field
 *
 * @return DOM element
 */
function warnOnInput(event, msg){
   $warningMsg = $("<p>" + msg + "</p>").css("display", "inline");
   if($(event.target).parent().has("p").length == 0){
	$(event.target).parent().append($warningMsg);
   }
}
/*
 On Ready ;)
*/
$(function() {
    //check if on login page
    if(!($(".signin-box").length)){
        //debug mode
        if($(".safe_debug").length){
          debugMode = true;
          // Debug info update
          fetchDebugInfo();
        }
        // Update breadcrumnb
        updateBreadCrumb();
        // Resize on first display
        resizeMenu();
        resizeWrapper();

        // Start dynamic refresh requests
        //fetchApplicationStatus({async:false});
        //fetchConfigurationStatus({initial:true, async:false});
        //fetchSystemStatus({async:true});
        //fetchEventsStatus({async:true});
        //
        startAjaxTimer(fetchApplicationStatus, 2000);//service status
        startAjaxTimer(fetchConfigurationStatus, 100);//config changed
        //wait 3s for dhcp return
        startAjaxTimer(fetchSystemStatus, 3000);//network status
        startAjaxTimer(fetchStorageStatus, 500);//storage status
        startAjaxTimer(fetchEventsStatus, 4000);//event status
        startAjaxTimer(fetchUpdateCheckerStatus, 500);//update checker status

        // Session timeout handler
        safe_create_session_timeout();
        // Resize handlers (run on every window resize)
        $(window).on("resize",resizeMenu);
        $(window).on("resize",resizeWrapper);
    }
    $(window).on('beforeunload', function () {
        networkDialog = true; //pass by network error dialog when unload page
    });
});

