<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * William Adam <wadam@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * SNG Application class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_application_class');
safe_module_require_class('sng', 'system_module');
safe_module_require_class('sng', 'storage_module');
safe_module_require_class('sng', 'rest_module');
safe_module_require_class('sng', 'update_module');
safe_module_require_class('sng', 'network_service');
safe_module_require_class('sng', 'troubleshooter_service');
safe_module_require_class('sng', 'cluster_module');
safe_module_require_class('sng', 'provisioning_module');

class Sng_application_class extends Safe_application_class
{
    public function register_modules() 
    {
        // register base modules
        parent::register_modules();

        // SNG modules
        $this->register_module(new Sng_system_module_class($this));
        $this->register_module(new Sng_network_service_class($this));
        $this->register_module(new Sng_storage_module_class($this));
        $this->register_module(new Sng_rest_module_class($this));
        $this->register_module(new Sng_update_module_class($this));
        $this->register_module(new Sng_cluster_module_class($this));
        $this->register_module(new Sng_provisioning_module_class($this));

        //troubleshooter service is not ready yet
        //$this->register_module(new Sng_troubleshooter_service_class ($this));
    }
    /**
     * @brief Return System information used to generate license
     *
     * @return array of key value pairs
     */
    public function system_info()
    {
        $info = parent::system_info();

        $network = $this->find_module_by_name('network');
        if($network){
            $adapters = $network->api_retrieve_interface();
            foreach($adapters as $adapter){
                if($adapter->interface_type() == Sng_network_interface_class::PHYSICAL){
                    $info[$adapter->name().' MAC'] = $adapter->mac();
                }
            }
        }
        return $info;
    }
    /**
     * @brief Return the result of creating an archive
     *
     * @return boolean of the result
    */
    public function create_autoname_backup($type, $name, &$output=null)
    {
        $timestamp = date('Y-m-d_H-i-s', time());
        $backup_name = $name . '_' . $timestamp;
        $data = array(
            'backup_type' => $type,
            'name' => $backup_name
        );
        $application_module = $this->node()->software()->application()->find_module_by_name('application');
        $archive = $application_module->api_backup_archive($data['name'], $data, $output);
        if($archive){
            return true;
        }else{
            return false;
        }
    }
}
/* End of file sng_application_class.php */

