<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015 Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Cluster Peer Access Key Class
 * @author Reza Samimi
 */
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_module_require_class('sng', 'application');
safe_module_require_class('sng', 'cluster_network_config');
safe_module_require_class('sng', 'cluster_interface_config');
safe_module_require_class('sng', 'cluster_ip_config');
safe_module_require_class('sng', 'cluster_route_config');
safe_module_require_class('sng', 'cluster_media_interfaces_config');

class Sng_cluster_access_key_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }

    public function configure()
    {
        //discard the lock, allow the obj to modify the config
        $this->discard_lock();
        
    	// $this->add_field('node', 'Node', 'text', '', 15);
    	$this->add_field('uuid', 'UUID', 'text', '', 60);
    	$this->set_field_rules('uuid', 'required|valid_uuid');
    	$this->add_field('fingerprint', 'Fingerprint', 'text', '', 50);
    	$this->set_field_rules('fingerprint', 'required');
    	// $this->add_field('keyname', 'Key Name', 'text', '', 50);
    	//$this->add_field('general_settings', 'General Network', 'hidden', '', 500);

        $this->add_field('node_registered', 'flag', 'hidden', 'false', 10);
    	$this->add_field('network', 'Network', 'hidden', '', 500);
    	$this->add_field('sngms', 'Media Interfaces', 'hidden', '', 500);

    	$this->register_aggregate_object('network_configuration',
            array(
                'name' => 'General Network',
                'base_path' => $this->object_name() . '/network_configuration',
                'autoname' => true,
                'singleton' => true,
                'controller' => array(
                    'config_manager' => array(
                        'allow_create_button' => false,
                        'summary_buttons' => 'edit[Edit]'
                    ),
                ),
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create Network Configuration',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve Network Configuration',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update Network Configuration',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete Network Configuration',
                        'request' => 'POST',
                    ),
                    'list' => array(
                    'name' => 'List',
                    'description' => 'List Network Configuration',
                    'request' => 'GET',
                    )
                ),
            )
        );
        $this->register_aggregate_object('network_interface',
            array(
                'name' => 'Network Interfaces',
                'base_path' => $this->object_name() . '/network_interface',
                'autoname' => 'create_obj_name',
                'controller' => array(
                    'config_manager' => array(
                        'allow_create_button' => false,
                        'summary_buttons' => 'Edit|Delete:confirm'
                    ),
                ),
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a Network Interface',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a Network Interface',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a Network Interface',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a Network Interface',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('network_ip',
            array(
                'name' => 'IP Addresses',
                'base_path' => $this->object_name() . '/network_ip',
                'autoname' => true,
                'controller_url' => '/SAFe/sng_cluster_config',
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a Network IP',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a Network IP',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a Network IP',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a Network IP',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('network_route',
            array(
                'name' => 'Static Routes',
                'base_path' => $this->object_name() . '/network_route',
                'autoname' => true,
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a Network Route',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a Network Route',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a Network Route',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a Network Route',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('sngms_configuration',
            array(
                'name' => 'Media Interface',
                'base_path' => $this->object_name() . '/sngms_configuration',
                'autoname' => true,
                'singleton' => true,
                'controller' => array(
                    'config_manager' => array(
                        'allow_create_button' => false,
                        'summary_buttons' => 'edit[Edit]'
                    ),
                ),
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a SNGMS Configuration',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a SNGMS Configuration',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a SNGMS Configuration',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a SNGMS Configuration',
                        'request' => 'POST',
                    ),
                    'list' => array(
                        'name' => 'List',
                        'description' => 'List a SNGMS Configuration',
                        'request' => 'GET',
                    )
                ),
            )
        );
        $this->register_aggregate_object('sngms_module',
            array(
                'name' => 'Media Interface Modules',
                'base_path' => $this->object_name() . '/sngms_module',
                'autoname' => true,
                'controller' => array(
                    'config_manager' => array(
                        'allow_create_button' => false,
                        'summary_buttons' => 'edit[Edit]'
                    ),
                ),
                'methods' => array(
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a SNGMS Module',
                        'request' => 'POST',
                    ),
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a SNGMS Module',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a SNGMS Module',
                        'request' => 'GET',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a SNGMS Module',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('sngms_device',
            array(
                'name' => 'Media Interface Device',
                'base_path' => $this->object_name() . '/sngms_device',
                'autoname' => true,
                'controller' => array(
                    'config_manager' => array(
                        'allow_create_button' => false,
                        'summary_buttons' => 'edit[Edit]'
                    ),
                ),
                'methods' => array(
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a SNGMS Device',
                        'request' => 'POST',
                    ),
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a SNGMS Device',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a SNGMS Device',
                        'request' => 'GET',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a SNGMS Device',
                        'request' => 'POST',
                    ),
                ),
            )
        );
    	return parent::configure();
    }
    // public function prepare_edit() {
    //     $this->set_field_disabled('uuid', true);
    //     $this->set_field_disabled('fingerprint', true);
    // }
    /**
     * Returns peer node's media interfaces
     */

    public function validate($data, &$output)
    {
        $parent_result = parent::validate($data, $output);
        $conflict_list = array();

        if (!array_key_exists('access-key', $data) && array_key_exists('uuid', $data) && array_key_exists('fingerprint', $data)) {
            // This is a REST API call with full node configuration, let it go through.
            return $parent_result;
        }
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $result = $cluster_module->register_ckey($data['access-key']);

        if ($result) {
            if(isset($result['failover']['nodes'])){
                $result = reset($tmp_result['failover']['nodes']);
            }
            $node = array();
            if (!isset($result['uuid'])) {
                $this->set_error('uuid', 'UUID not present on access key.' , $output);
                return false;
            } else if (!isset($result['fingerprint'])) {
                $this->set_error('fingerprint', 'Fingerprint not present on access key.' , $output);
                return false;
            } else {
                $node['uuid'] = $result['uuid'];
                $node['fingerprint'] = $result['fingerprint'];
                //$node['general_settings'] = $result['general_settings'];
                $node['network'] = $result['network'];
                $node['sngms'] = $result['sngms'];
                $this->set_data_values($node);

                if (!isset($result['network']['configuration'])) {
                    $this->set_error('uuid', 'Network configuration not present on access key.' , $output);
                    return false;
                } else {
                    $parent_result = true;
                }
            }

            $parent_result = true;
        }else{
            $this->set_error('uuid', 'Data format error.' , $output);
            return false;
        }
        return $parent_result;
    }
    public function save() {
    	$result = parent::save();
    	$sub_values = $this->get_data_values();
        if ($sub_values['node_registered'] != 'true') {
            $sub_values['node_registered'] = 'true';
            $this->set_data_values($sub_values);
            $this->save();


            // create network_configuration object ///////////////
            $sub_obj = $this->get_aggregate_objects('network_configuration');
            $name = Safe_object_serializer_class::build_unique_name('network_configuration');
            unset($sub_values['network']['configuration']['autodns']);//remove the auto dns
            $network_configuration = $this->api_create_network_configuration($name, $sub_values['network']['configuration']);
            $network_configuration->save();

        	// create network_interface object ///////////////
        	$sub_obj = $this->get_aggregate_objects('network_interface');
        	if($sub_values['network']['interface']){
        	    foreach(array_reverse($sub_values['network']['interface']) as $if_name => $if_data){
                    //$name = 'network_interface' . Safe_object_serializer_class::build_unique_name('network_interface');
                    if ($if_data['type'] == Sng_network_interface_class::VIRTUAL || $if_data['type'] == Sng_network_interface_class::BONDING) {
                        $network_interface = $this->api_create_network_interface($if_name, $if_data);
                    }else {
                        $network_interface = $this->api_create_network_physical_interface($if_name, $if_data);
                    }
                    $network_interface->set_data_values($if_data);
                    $network_interface->save();
                }
            }

            // create network_ip object ///////////////
            $sub_obj = $this->get_aggregate_objects('network_ip');
            if($sub_values['network']['ip']){
                foreach($sub_values['network']['ip'] as $ip_name => $ip_data){
        	        //$name = 'network_ip' . Safe_object_serializer_class::build_unique_name('network_ip');
                    if ($ip_data['interface'] == 'lo') {
                        $network_ip = $this->api_create_network_loopback_ip($ip_name, $ip_data);
                    }
                    else {
                        $network_ip = $this->api_create_network_ip($ip_name, $ip_data);
                    }
        	        $network_ip->save();
        	    }
        	}

        	// create network_route object ///////////////
        	$sub_obj = $this->get_aggregate_objects('network_route');
        	if($sub_values['network']['route']){
        	    foreach($sub_values['network']['route'] as $route_name => $route_data){
        	        //$name = 'Network_Route' . Safe_object_serializer_class::build_unique_name('network_route');
        	        $network_route = $this->api_create_network_route($route_name, $route_data);
        	        $network_route->save();
        	    }
        	}

            // create sngms_configuration object ////////////////
            $sub_obj = $this->get_aggregate_objects('sngms_configuration');
            $name = Safe_object_serializer_class::build_unique_name('sngms_configuration');
            $sngms_configuration = $this->api_create_sngms_configuration($name, $sub_values['sngms']['configuration']);
            $sngms_configuration->save();

        	// create sngms_module object ////////////////
            $sub_obj = $this->get_aggregate_objects('sngms_module');
            if ($sub_values['sngms']['module']) {
                foreach ($sub_values['sngms']['module'] as $module_name => $module) {
                    $sngms_configuration = $this->api_create_sngms_module($module_name, $module);
                    $sngms_configuration->save();
                }
            }
            // create sngms_device object ////////////////
            $sub_obj = $this->get_aggregate_objects('sngms_device');
            if ($sub_values['sngms']['device']) {
                foreach ($sub_values['sngms']['device'] as $device_name=>$device) {
                    $sngms_device = $this->api_create_sngms_device($device_name, $device);
                    $sngms_device->save();
                }
            }
        }
    	return $result;
    }
    public function dispose()
    {
        foreach($this->network_configurations() as $obj){
            $obj->dispose();
        }
        foreach($this->network_interfaces() as $obj){
            $obj->dispose();
        }
        foreach($this->network_ips() as $obj){
            $obj->dispose();
        }
        foreach($this->network_routes() as $obj){
            $obj->dispose();
        }
        foreach($this->sngms_configurations() as $obj){
            $obj->dispose();
        }
        foreach($this->sngms_modules() as $obj){
            $obj->dispose();
        }
        foreach($this->sngms_device() as $obj){
            $obj->dispose();
        }
        return parent::dispose();
    }
    /**
     * get ip obj list by hardward interface name
     * @return ip obj list
     */
    public function get_network_ips($interface = null)
    {
        //get this HW interface related ip objs
        $ips = array();
        $objs = $this->network_ips();
        //an obj with a empty name '' will cause error
        if($interface){
            $interface_obj = $this->api_retrieve_network_interface($interface);
            if($interface_obj){
                $interface_obj_name = $interface_obj->name();
                foreach($objs as $obj_name => $obj){
                    $if_name = $obj->get_interface();//interface obj name
                    if($if_name == $interface_obj_name){
                        $ips[] = $obj;
                    }
                }
            }
        }
        return $ips;
    }
    public function network_configurations()
    {
    	$nodes = $this->get_aggregate_objects('network_configuration');
        return $nodes;
    }
    public function network_interfaces()
    {
        $nodes = $this->get_aggregate_objects('network_interface');
        return $nodes;
    }
    public function network_ips()
    {
        $nodes = $this->get_aggregate_objects('network_ip');
        return $nodes;
    }
    public function network_routes()
    {
        $nodes = $this->get_aggregate_objects('network_route');
        return $nodes;
    }
    public function sngms_configurations()
    {
        $nodes = $this->get_aggregate_objects('sngms_configuration');
        return $nodes;
    }
    public function sngms_modules()
    {
        $nodes = $this->get_aggregate_objects('sngms_module');
        return $nodes;
    }
    public function sngms_device()
    {
        $nodes = $this->get_aggregate_objects('sngms_device');
        return $nodes;
    }
    public function hardware_interface($if_type = 'all')
    {
        $allowed_infs = array(
            Sng_network_interface_class::PHYSICAL,
            Sng_network_interface_class::LOOPBACK,
            Sng_network_interface_class::VIRTUAL,
            Sng_network_interface_class::BONDING
        );
        //$if_type management or transcoding
        $interfaces = $this->api_retrieve_network_interface();
        $hardware_interfaces = array();
        foreach($interfaces as $key => $obj){
            $vlan_type = $obj->get_data_value('vlan_type', false);
            $is_macvlan = (isset($vlan_type) && ($vlan_type == Sng_network_interface_class::MACVLAN)) ? true : false;
            if( in_array($obj->interface_type(), $allowed_infs) || $is_macvlan){
                if($if_type == 'all' || $obj->hardware_type() == $if_type || $is_macvlan) {
                    $hardware_interfaces[$key] = $obj;
                }
            }
        }
        return $hardware_interfaces;
    }
    public function get_ips($interface)
    {
        //get this HW interface related ip objs
        $ips = array();
        $objs = $this->api_retrieve_network_ip();
        //an obj with a empty name '' will cause error
        if($interface){
            $interface_obj = $this->api_retrieve_network_interface($interface);
            if($interface_obj){
                $interface_obj_name = $interface_obj->name();
                foreach($objs as $obj_name => $obj){
                    $if_name = $obj->get_interface();//interface obj name
                    if($if_name == $interface_obj_name){
                        $ips[] = $obj;
                    }
                }
            }
        }
        return $ips;
    }
    // Peer General Network Interface
    //
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_list_network_configuration() 
    {
        return $this->api_retrieve_network_configuration();
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_network_configuration($name=null, $data=null, &$output = null)
    {
        $network_configuration = $this->network_configurations();
        if ($name) {
            return $network_configuration[$name];
        }
        else {
            if ($network_configuration) {
                return array_shift($network_configuration);
            }
            else return array();
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_network_configuration($name, $data=null, &$output = null)
    {
        $network_configurations = $this->api_retrieve_network_configuration();
        if (!$network_configurations) {
            $peer = new Sng_cluster_network_config_class($this->node(), $this->object_name() . '/network_configuration', $name);
            $peer->configure();
            if ($data) {
                $peer->set_data_values($data);
            }
            return $peer;
        }
        return false;
    }
    //
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_network_configuration($name, $data=null, &$output = null) {
        $network_configuration = $this->api_retrieve_network_configuration($name);
        if($network_configuration) {
            if ($network_configuration->validate($data,$output)) {
                if (true == $network_configuration->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    //
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_network_interface($name=null, $data=null, &$output = null)
    {
        $peers = $this->network_interfaces();
        if ($name)
            return $peers[$name];
        else
            return $peers;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_network_interface($name, $data=null, &$output = null)
    {
        if(isset($data['type'])){
            $class = '';
            //for configuration update, to create physical interface
            if($data['type'] == Sng_network_interface_class::PHYSICAL || $data['type'] ==Sng_network_interface_class::LOOPBACK){
                $class = "Sng_cluster_interface_config_class";
            }elseif($data['type'] == Sng_network_interface_class::VIRTUAL){
                $class = "Sng_cluster_network_link_class";
            }elseif($data['type'] == Sng_network_interface_class::BONDING){
                $class = "Sng_cluster_network_bonding_class";
            }else{
                $output['type'] = 'Interface type error.';
                return false;
            }
        }else{
            //no data
            $class = "Sng_cluster_network_link_class";
        }
        if($class){
            $obj = new $class($this->node(), $this->object_name().'/network_interface', $name);
            $obj->configure();
            return $obj;
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_network_interface($name, $data=null, &$output = null) {
        $network_interface = $this->api_retrieve_network_interface($name);
        if($network_interface) {
            if ($network_interface->validate($data,$output)) {
                if (true == $network_interface->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_delete_network_interface($name, $data=null, &$output = null) {
        $inf = $this->api_retrieve_network_interface($name);
        if($inf) {
            if($inf->can_dispose($output)) {
                return $inf->dispose();
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_network_physical_interface($name, $data=null, &$output = null)
    {
        $network_inf = new Sng_cluster_interface_config_class($this->node(), $this->object_name() . '/network_interface', $name);
        $network_inf->configure();
        return $network_inf;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_network_loopback_ip($name, $data=null, &$output = null)
    {
        $peer = new Sng_cluster_loopback_ip_config_class($this->node(), $this->object_name() . '/network_ip', $name);
        $peer->configure();
        if ($data) {
            $peer->set_data_values($data);
        }
        return $peer;
    }
    // Peer Network IP
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_network_ip($name, $data=null, &$output = null)
    {
        $peer = new Sng_cluster_ip_config_class($this->node(), $this->object_name() . '/network_ip', $name);
        $peer->configure();
        if ($data) {
            $peer->set_data_values($data);
        }
        return $peer;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_network_ip($name=null, $data=null, &$output = null)
    {
        $peers = $this->network_ips();
        if ($name) {
            return $peers[$name];
        }
        else {
            return $peers;
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_network_ip($name, $data=null, &$output = null) {
        $network_ip = $this->api_retrieve_network_ip($name);
        if($network_ip) {
            if ($network_ip->validate($data,$output)) {
                if (true == $network_ip->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_delete_network_ip($name, $data=null, &$output = null) {
        $ip = $this->api_retrieve_network_ip($name);
        if($ip) {
            if($ip->can_dispose($output)) {
                return $ip->dispose();
            }
        }
        return false;
    }
    // Peer Network Static Route
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_network_route($name, $data=null, &$output = null)
    {
        $peer = new Sng_cluster_route_config_class($this->node(), $this->object_name() . '/network_route', $name);
        $peer->configure();
        if ($data) {
            $peer->set_data_values($data);
        }
        return $peer;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_network_route($name=null, $data=null, &$output = null)
    {
        $peers = $this->network_routes();
        if ($name) {
            return $peers[$name];
        }
        else {
            return $peers;
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_network_route($name, $data=null, &$output = null) {
        $network_route = $this->api_retrieve_network_route($name);
        if($network_route) {
            if ($network_route->validate($data,$output)) {
                if (true == $network_route->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_delete_network_route($name, $data=null, &$output = null) {
        $route = $this->api_retrieve_network_route($name);
        if($route) {
            if($route->can_dispose($output)) {
                return $route->dispose();
            }
        }
        return false;
    }
    // Peer Media Interface
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_sngms_configuration($name, $data=null, &$output = null)
    {
        $medias = $this->api_retrieve_sngms_configuration();
        if (!$medias) {
            $sngms_configuration = new Sng_cluster_media_interfaces_config_class($this->node(), $this->object_name() . '/sngms_configuration', $name);
            $sngms_configuration->configure();
            if ($data) {
                $sngms_configuration->set_data_values($data);
            }
            return $sngms_configuration;
        }
        else
            return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_sngms_configuration($name=null, $data=null, &$output = null)
    {
        $sngms_configurations = $this->sngms_configurations();
        if ($name) {
            return $sngms_configurations[$name];
        }
        if ($sngms_configurations) {
            return array_shift($sngms_configurations);
        }
        else {
            return array();
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_list_sngms_configuration() 
    {
        return $this->api_retrieve_sngms_configuration();
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_sngms_configuration($name, $data=null, &$output = null) {
        $sngms_configuration = $this->api_retrieve_sngms_configuration($name);
        if($sngms_configuration) {
            if ($sngms_configuration->validate($data,$output)) {
                if (true == $sngms_configuration->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    // Peer Media Interface > Modules
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_sngms_module($name, $data=null, &$output = null)
    {
        $medias = $this->api_retrieve_sngms_module();
        if (!$medias) {
            $sngms_module = new Sng_cluster_media_modules_config_class($this->node(), $this->object_name() . '/sngms_module', $name);
            $sngms_module->configure();
            if ($data) {
                $sngms_module->set_data_values($data);
            }
            return $sngms_module;
        }
        else
            return false;
    }

    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_sngms_module($name=null, $data=null, &$output = null)
    {
        $sngms_modules = $this->sngms_modules();
        if ($name) {
            return $sngms_modules[$name];
        }
        return $sngms_modules;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_sngms_module($name, $data=null, &$output = null) {
        $sngms_module = $this->api_retrieve_sngms_module($name);
        if($sngms_module) {
            if ($sngms_module->validate($data,$output)) {
                if (true == $sngms_module->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    // Peer Media Interface > Device
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_sngms_device($name, $data=null, &$output = null)
    {
        $medias = $this->api_retrieve_sngms_device();
        if (!$medias) {
            $sngms_device = new Sng_cluster_media_device_config_class($this->node(), $this->object_name() . '/sngms_device', $name);
            $sngms_device->configure();
            if ($data) {
                $sngms_device->set_data_values($data);
            }
            return $sngms_device;
        }
        else
            return false;
    }

    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_sngms_device($name=null, $data=null, &$output = null)
    {
        $sngms_devices = $this->sngms_device();
        if ($name) {
            return $sngms_devices[$name];
        }
        return $sngms_devices;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_sngms_device($name, $data=null, &$output = null) {
        $sngms_device = $this->api_retrieve_sngms_device($name);
        if($sngms_device) {
            if ($sngms_device->validate($data,$output)) {
                if (true == $sngms_device->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    
    public function new_fields(&$config_manager, $type)
    {
        if($type == 'network_interface'){
            $config_manager->add_enum_field('type', 'Interface Type', 'dropdown', '', array('virtual'=>'VLAN','bonding'=>'Bonding'));
        }else{
            parent::new_fields($config_manager, $type);
        }
        return;
    }
    public function create_obj_name($data, $type)
    {
        //using the data to create  obj name
        if(isset($data['profile-name'])){
            return $data['profile-name'];
        }else{
            if($type == 'network_interface'){
                if(isset($data['type'])){
                    $keys = $this->get_aggregate_objects_keys($type);
                    $hardware_interfaces = $this->hardware_interface();
                    $tmp_num_bonding = 0;
                    $tmp_num_interface = 0;
                    foreach($hardware_interfaces as $hardware_interface_name => $obj) {
                        if($obj->interface_type() == Sng_network_interface_class::BONDING){
                            $tmp_num_bonding++;
                        }else{
                            $tmp_num_interface++;
                        }
                    }
                    if($data['type'] == 'bonding'){
                        while(in_array($name, $keys)){
                            $tmp_num_bonding++;
                        };
                        $name = 'bond' . $tmp_num_bonding;
                        return $name;
                    }elseif($data['type'] == 'virtual'){
                        while(in_array($name, $keys)){
                            $tmp_num_interface++;
                        };
                        $name = 'interface' . $tmp_num_interface;
                        return $name;
                    }    
                }
            }
        }
        return false;
    }
    
    public function create_aggregate_object($type, $name='default', &$output=null, $data = null)
    {
        if($type == 'network_interface' && isset($data['type'])){
            $new_data['type'] = $data['type'];//filter out other no use data
        }else{
            $new_data = $data;
        }
        return parent::create_aggregate_object($type, $name, $output, $new_data);
    }
}

