<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Cluster Peer Node > Network Interfaces
 * @author Reza Samimi
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
class Sng_cluster_interface_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        //used in ip command to up or down interface
        $this->add_field('ifname', 'Interface', 'text', '',20);
        $this->set_field_readonly('ifname');
        $this->add_field('read_only', 'Read Only', 'hidden', 'true');
        $this->add_field('mac', 'MAC', 'text', '', 40);
        $this->set_field_readonly('mac');

        //add Auto Negotiation settings for interface
        $this->add_enum_field('autoneg', 'Auto Negotiation', 'dropdown', 'true', $this->enable_disable_enum());
        $this->add_enum_field('speed', 'Speed', 'dropdown', '1000', array('10'=>'10 MB/s', '100'=>'100 MB/s','1000'=>'1000 MB/s'));
        $this->add_enum_field('duplex', 'Duplex', 'dropdown', 'full', array('half'=>'Half', 'full'=>'Full'));

        $neg_group = array('speed', 'duplex');
        $neg_condition = array();
        foreach($neg_group as $item){
            $neg_condition['true'][] = array('field_name' => $item, 'status' => 'disable');
            $neg_condition['false'][] = array('field_name' => $item, 'status' => 'enable');
        }
        $this->create_group('negotiation', $neg_group,
                array('default' => array( 'value' => 'true', 'status' => 'disable'),
                        'condition' =>$neg_condition
                )
        );
        $this->conditional_control('autoneg', 'negotiation');

        $this->add_field('type', 'Type', 'hidden', '', 20);
        //"management" or "transcoding"
        $this->add_field('if_mode', 'Interface Mode', 'hidden', '', 20);
        $this->add_field('configuration', 'Configuration', 'text', '', 40);
        $this->add_field('bonding_interface', 'Bonding Interface', 'hidden', '', 40);
    }
    public function bonding_interface()
    {// return the bonding interface name
        return $this->get_data_value('bonding_interface', false);
    }
    public function is_bonded()
    {// if not empty , then this interface was bonded
        return !empty($this->bonding_interface());
    }
    public function is_readonly()
    {
        if($this->hardware_type() == Sng_network_interface_class::TRANSCODING || $this->interface_type() == Sng_network_interface_class::LOOPBACK ){
            if($this->node()->get_config_status('update') == 'rest'){
                return false;
            }else{
                return true;
            }
        }
        elseif($this->get_data_value('vlan_type', false) == 'macvlan' && $this->get_data_value('read_only') == 'true') {
            return true;
        }elseif($this->is_bonded()){
            return true;
        }else{
            return false;
        }
    }
    public function can_dispose(&$reason = null){
        $type = $this->get_data_value('type');
        if($type == Sng_network_interface_class::PHYSICAL){
            $reason = 'Cannot delete ' . $type . ' interface.';
            return false;
        }elseif($this->is_readonly()){
            $reason = 'Cannot delete system interface.';
            return false;
        }else{
            return parent::can_dispose($reason);
        }
    }
    public function can_modify(&$output = null)
    {
        if($this->is_readonly()){
            $this->set_error('ifname', "Cannot modify system interface.", $output);
            return false;
        }
        return parent::can_modify($output);
    }
    public function summary()
    {
        $row_class = '';
        if ($this->get_data_value('vlan_type') && $this->get_data_value('vlan_type', false) == 'macvlan') {
            $name = $this->get_data_value('macvlan_device');
            $mac = $this->get_data_value('macvlan_mac');
        } else {
            $name = $this->interface_name();
            $mac = $this->get_data_value('mac');
        }
        $result =  array(
            'Name' => array(
                'data' => $name,
                'class' => $row_class,
            ),

            'MAC'  => array(
                'data' => $mac,
                'class' => $row_class,
            ),
            'Speed'  => array(
                'data' => $this->get_data_value('speed'),
                'class' => $row_class,
            ),

        );
        return array( 'data' => $result);
    }
    public function save() {
        $result = parent::save();
        $path_info = $this->path_info();

        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);
        if (!$peer_obj) return $result;

        if ($peer_obj->get_data_value('node_registered') == 'true') {
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name'], 'save');
        }
        return $result;
    }
    public function dispose() {
        if (parent::dispose()) {
            $path_info = $this->path_info();

            $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name']);

            return true;
        }
        return false;
    }
    public function interface_name()
    {
        $interface_name = $this->get_data_value('ifname');;
        return $interface_name;
    }
    public function interface_type()
    {// physical virtual loopback
        return $this->get_data_value('type', false);
    }
    public function hardware_type()
    {// "management" or "transcoding"
        return $this->get_data_value('if_mode');
    }
    public function mac()
    {
        return $this->get_data_value('mac');
    }
}
class Sng_cluster_network_bonding_class extends Sng_cluster_interface_config_class
{
    public function configure()
    {
        $this->add_field('display_name', 'Bonding Interface Name', 'text', '', 20);
        $this->set_field_rules('bonding_interface_name', 'required|alpha_numeric');
        $this->add_multiple_field('bonded_interface', 'Physical Interfaces', 'checkbox', array(), array());
        $this->set_field_rules('bonded_interface', 'callback_check_interface');
        $this->add_enum_field('type', 'Interface Type', 'dropdown', Sng_network_interface_class::BONDING, array(Sng_network_interface_class::BONDING => 'Bonding'));
        $this->set_obj_message('delete_confirm_body', "All IPs on this bonding interface will be moved to the first bonded physical interface.");
        $this->set_obj_message('add_form_title', "All IPs on the bonded physical interfaces will be move to the new bonding interface.");
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $this->path_info()['name']);
        $hardware_interfaces = $peer_obj->hardware_interface();
        $interfaces_enum = array();
        $tmp_num = 0;
        foreach($hardware_interfaces as $hardware_interface_name => $obj) {
            if($obj->hardware_type() == Sng_network_interface_class::TRANSCODING
                    || $obj->interface_type() == Sng_network_interface_class::LOOPBACK
                    || $obj->is_bonded() ){
                continue;
            }else{
                if($obj->interface_type() == Sng_network_interface_class::BONDING){
                    $tmp_num++;
                }else{
                    $interfaces_enum[$hardware_interface_name] = $obj->interface_name();
                }
            }
        }
        $this->add_field('display_name', 'Bonding Interface Name', 'text', $this->name(), 20);
        $this->set_field_readonly('display_name');
        $this->add_multiple_field('bonded_interface', 'Physical Interfaces', 'checkbox', array(), $interfaces_enum);
    }
    public function check_interface($field_name)
    {
        $interface_list = parent::get_data_value('bonded_interface',false);
        if($interface_list && count($interface_list) == 2){
            return true;
        }else{
            return array('check_interface' => 'The Interface field must contain 2 physical interfaces.');
        }
    }
    public function get_bonded_interface()
    {
        $interface_list = $this->get_data_value('bonded_interface',false);
        return $interface_list;
    }
    public function mac()
    {
        return '01:00:00:00:00:00';
    }
    public function hardware_type()
    {
        return Sng_network_interface_class::MANAGEMENT;
    }
    public function interface_name()
    {
        //need to review , now just return obj name
        return $this->name();
    }
    public function summary()
    {
        $result = parent::summary();
        //no mac in bonding interface
        $bonded_interface = $this->get_data_value('bonded_interface', false);
        if($bonded_interface){
            $result['data']['MAC']['data'] = implode(',' , $bonded_interface);
        }
        return $result;
    }
    public function save(&$output = null)
    {
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $this->path_info()['name']);
        $bonded_interfaces = $this->get_bonded_interface();
        if($bonded_interfaces){
            foreach($bonded_interfaces as $tmp_interface_name){
                $tmp_interface_obj = $peer_obj->api_retrieve_network_interface($tmp_interface_name);
                if($tmp_interface_obj){
                    //mark this interface is bonded
                    $tmp_interface_obj->set_data_values(array('bonding_interface' => $this->name()));
                    $tmp_interface_obj->save();
                    $tmp_ip_list = $peer_obj->get_ips($tmp_interface_name);
                    if($tmp_ip_list){
                        foreach($tmp_ip_list as $ip => $tmp_ip_obj){
                            //move the ip to this bonding interface
                            $tmp_ip_obj->set_data_values(array('interface' => $this->name()));
                            $tmp_ip_obj->save();
                        }
                    }
                }
            }
        }
        return parent::save($output);
    }
    public function dispose(&$output = null)
    {
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $this->path_info()['name']);
        $tmp_ip_list = $peer_obj->get_ips($this->name());
        $bonded_interfaces = $this->get_bonded_interface();
        if($bonded_interfaces){
            //move the bonding interface ips to first interface
            foreach($bonded_interfaces as $tmp_interface_name){
                if($tmp_ip_list){
                    foreach($tmp_ip_list as $ip => $tmp_ip_obj){
                        //move the ip to this bonding interface
                        $tmp_ip_obj->set_data_values(array('interface' => $tmp_interface_name));
                        $tmp_ip_obj->save();
                    }
                }
                break;
            }
            foreach($bonded_interfaces as $tmp_interface_name){
                $tmp_interface_obj = $peer_obj->api_retrieve_network_interface($tmp_interface_name);
                if($tmp_interface_obj){
                    //remove bonding interface name from the phsycal interface
                    $tmp_interface_obj->set_data_values(array('bonding_interface' => ''));
                    $tmp_interface_obj->save();
                }
            }
        }
        return parent::dispose();
    }
    public function can_edit(&$reason = NULL)
    {
        return false;
    }
}
class Sng_cluster_network_link_class  extends Sng_cluster_interface_config_class
{
    /**
     * Constructor for network interface config class
     * @param $node
     * @param $parent_name
     * @param $name
     */
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($node, $parent_name, $name);
        $this->_set_reuse(false);
    }
    public function configure()
    {
        $vlan_type = array('vlan' => 'VLAN', 'macvlan' => 'MAC VLAN');
        $this->add_enum_field('vlan_type', 'VLAN Type', 'dropdown', 'vlan', $vlan_type);
        $this->add_field('read_only', 'Read Only', 'hidden', 'false');
        $this->add_enum_field('ifname', 'Interface', 'dropdown', '', array());
        $this->add_field('id', 'Virtual interface ID', 'text', '', 20);
        // MAC VLAN
        $this->add_field('macvlan_device', 'MAC VLAN Interface', 'text', '', 20);
        $this->set_field_readonly('macvlan_device');
        $this->add_field('macvlan_mac', 'MAC Address', 'text', '', 17);
        $this->set_field_readonly('macvlan_mac');
        $this->add_field('type', 'Type', 'hidden', Sng_network_interface_class::VIRTUAL, 20);

        $fields = array('id', 'macvlan_parent', 'macvlan_mode', 'macvlan_device', 'macvlan_mac');
        $condition = array();
        foreach ($fields as $field) {
            if ($field == 'id') {
                $condition['vlan'][] = array('field_name' => $field, 'status' => 'enable');
                $condition['macvlan'][] = array('field_name' => $field, 'status' => 'disable');
            }
            else {
                $condition['vlan'][] = array('field_name' => $field, 'status' => 'disable');
                $condition['macvlan'][] = array('field_name' => $field, 'status' => 'enable');
            }
        }
        $this->create_group('vlan_type', $fields, array(
                'default' => array( 'value' => 'macvlan', 'status' => 'disable'),
                'condition' => $condition
            )
        );
        $this->conditional_control('vlan_type', 'vlan_type');
    }
    public function hardware_type()
    { // management or transcoding
        if ($this->get_data_value('vlan_type', false) == Sng_network_interface_class::MACVLAN) {
            //mac vlan only for TRANSCODING d150
            return Sng_network_interface_class::TRANSCODING;
        }else{
            //vlan only for MANAGEMENT
            return Sng_network_interface_class::MANAGEMENT;
        }
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $this->path_info()['name']);
        $hardware_interfaces = $peer_obj->hardware_interface();
        $hardware_interface_enum = array();
        foreach($hardware_interfaces as $hardware_interface_name => $obj) {
            if($obj->hardware_type() == Sng_network_interface_class::TRANSCODING 
                    || $obj->interface_type() == Sng_network_interface_class::LOOPBACK
                    || $obj->is_bonded()){
                continue;
            }else{
                $hardware_interface_enum[$hardware_interface_name] = $obj->interface_name();
            }
        }
        $this->add_enum_field('ifname', 'Interface', 'dropdown', '', $hardware_interface_enum);
        if ($this->get_data_value('vlan_type') == 'VLAN') {
            $data = $this->get_data_values();
            if (isset($data['macvlan_device']) && $data['macvlan_device'] == '') {
                $data['macvlan_device'] = Sng_network_link_class::generate_macvlan('device');
            }
            if (isset($data['macvlan_mac']) && $data['macvlan_mac'] == '') {
                $data['macvlan_mac'] = Sng_network_link_class::generate_macvlan('mac');
            }
            $this->set_data_values($data);
        }
    }
    public function interface_name()
    {
        if ($this->get_data_value('vlan_type', false) == Sng_network_interface_class::MACVLAN) {
            $interface_name = $this->get_data_value('macvlan_device');
        }
        else {
            $interface_name = $this->get_data_value('ifname', false) . '.' . $this->get_data_value('id');
        }
        return $interface_name;
    }
    public function save() {
        $result = parent::save();
        $data = $this->get_data_values();
        if ($this->get_data_value('vlan_type') == 'VLAN' && $data['macvlan_mac'] != '' && $data['macvlan_device'] != '') {
            $data['macvlan_mac'] = '';
            $data['macvlan_device'] = '';
            $this->set_data_values($data);
            $this->save();
        }
        return $result;
    }
    public function validate($data, &$output)
    {
        $mvlan_count = 0;
        if ($this->get_data_value('vlan_type') == 'VLAN') {
            $this->set_field_rules('id', 'required|integer|greater_than[0]|less_than[4095]');
        }
        $parent_result = parent::validate($data, $output);
        if ($parent_result && $this->interface_type() == Sng_network_interface_class::VIRTUAL) {

            $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
            $peer_obj = $cluster_module->get_aggregate_object('peer', $this->path_info()['name']);
            $interface_objs = $peer_obj->api_retrieve_network_interface();
            $id = $this->get_data_value('id');
            // block if there is a physical DSP card on the peer node
            if ($this->get_data_value('vlan_type', false) == 'macvlan') {
                $transcoding_interfaces = $peer_obj->hardware_interface(Sng_network_interface_class::TRANSCODING);
                foreach ($transcoding_interfaces as $k=>$v) {
                    if ($v->hardware_type() == 'transcoding') {
                        $internal_dsp = true;
                        break;
                    }
                }
                if($internal_dsp){
                    $this->set_error('vlan_type', 'We cannot create DSP MACVLANs on a system with existing physical DSP card.', $output);
                    return false;
                }
            }
            foreach($interface_objs as $interface_obj_name => $interface_obj){
                if ($interface_obj->get_data_value('vlan_type', false) == 'macvlan' && $this->name() !== $interface_obj->name()) {
                    $mvlan_count++;
                }
                else if ($interface_obj->get_data_value('vlan_type', false) == 'vlan') {
                    if($interface_obj_name != $this->name() && $interface_obj->interface_type() == Sng_network_interface_class::VIRTUAL){
                        if($interface_obj->interface_name() == $this->interface_name()){
                            $rule_msgs['id'] = "This VLAN id:{$id} is already used.";
                            $this->set_error('id', $rule_msgs['id'], $output);
                            return false;
                        }
                    }
                }
            }
            if ($interface_obj->get_data_value('vlan_type', false) == 'macvlan' && $mvlan_count >= 2) {
                $msg = "Already $mvlan_count interfaces exist in the system. Only a maximum of 2 media interfaces are supported.";
                $this->set_error('vlan_type', $msg, $output);
                $parent_result = false;
            }
        }
        return $parent_result;
    }
    public function summary()
    {
        $result = parent::summary();
        //no mac in vlan interface
        if($result && isset($result['data']['MAC']) && substr($result['data']['Name']['data'], 0, 6) !== 'sngdsp'){
            $result['data']['MAC']['data'] ='';
        }
        return $result;
    }
    public function can_dispose(&$reason = null){
        //check if this interface used by ip
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $this->path_info()['name']);
        $network_ips = $peer_obj->api_retrieve_network_ip();
        $my_name = $this->name();
        foreach($network_ips as $ip_obj_name => $ip_obj){
            if($ip_obj->get_interface() == $my_name){
                $reason = 'You must remove all the IPs on this interface before you can delete it.';
                return false;
            }
        }
        return parent::can_dispose($reason);
    }
}
