<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Reza Samimi <rsamimi@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Cluster (floating) IP 
**/
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_module_require_class('sng', 'application');
class Sng_cluster_ip_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $interfaces = $network_module->api_retrieve_interface();
        $interfaces_enum = array();
        foreach($interfaces as $key => $obj){
            if($obj->hardware_type() == Sng_network_interface_class::TRANSCODING && !safe_development_mode()){
                //only in development_mode allow user to add/remove/change the ip on sngdspX interfaces
                continue;
            } elseif ($obj->get_data_value('type') == 'loopback' || $obj->is_bonded()) {
                continue;
            } else {
                $interfaces_enum[$obj->name()] = $obj->interface_name();
            }
        }

        $proto_enum = array(
            Safe_ip_address_class::PROTO_STATIC4 => 'IPv4 - Static',
            Safe_ip_address_class::PROTO_STATIC6 => 'IPv6 - Static'
        );

        $this->add_enum_field('proto', 'Configuration', 'dropdown', Safe_ip_address_class::PROTO_STATIC4, $proto_enum);
        $this->set_field_help('proto','The protocol family type should be used.');

        $this->add_field('address', 'IP Address', 'text', '',40);
        $this->set_field_rules('address', 'required|valid_ip');
        $this->add_field('prefix', 'Prefix', 'text', '', 5);
        $this->set_field_rules('prefix', 'valid_ip_prefix[address]|required');
        $this->composite_layout('address', array('prefix'));

        $this->add_enum_field('interface', 'Interface', 'dropdown', '', $interfaces_enum);

        $this->add_field('interval', 'Monitor Interval', 'text', '3', 40);
        $this->set_field_rules('interval', 'greater_than[0]|less_than[90]|required');

        return parent::configure();
    }
    public function ip_type()
    {
        $proto = $this->get_data_value('proto', false);
        $type_data = explode('-', $proto);
        $tmp_type = $type_data[1];
        return $tmp_type;
    }
    public function address()
    {
        return $this->get_data_value('address');
    }
    public function prefix()
    {
        return $this->get_data_value('prefix');
    }
    public function mask()
    {
        return safe_cidr2netmask($this->get_data_value('prefix'));
    }
    public function get_interface_name()
    {
        return $this->get_data_value('interface', false);
    }
    public function validate($data, &$output)
    {
        $parent_result = parent::validate($data, $output);
        if($parent_result){
            //check ip
            $ip_type = $this->ip_type();
            $ip = $this->address();
            //save the compress ip into db
            if($ip_type == Safe_ip_address_class::IP_V6){
                $this->set_data_values(array('address' => safe_ipv6_compress($ip)));
            }
            //check IP type
            $ip_type_error_msg = "The Address field must contain a valid {$ip_type} IP.";
            if($ip_type == Safe_ip_address_class::IP_V6){
                if(!filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)) {
                    $this->set_error('address', $ip_type_error_msg, $output);
                    $parent_result =  false;
                }
            }else{
                if(!filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)) {
                    $this->set_error('address', $ip_type_error_msg, $output);
                    $parent_result =  false;
                }
            }
            //check conflict with other IP
            if($parent_result){
                $network_module = $this->node()->software()->application()->find_module_by_name('network');
                $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
                //check if ip conflict
                if($ip){
                    //check if conflict with other floating ip
                    $ip_conflict_list = $cluster_module->check_ip_conflict($this);
                    //check if conflict with network ip
                    $network_ip_conflict_list = $network_module->check_ip_conflict($this);
                    if($network_ip_conflict_list){
                        $ip_conflict_list = array_merge($ip_conflict_list, $network_ip_conflict_list);
                    }
                    if($ip_conflict_list){
                        $error_msg = 'This ip is conflict with : ';
                        $list = array();
                        foreach($ip_conflict_list as $obj){
                            $list[] = $obj->name() . "[{$obj->address()}]";
                        }
                        $error_msg .= join(",", $list);
                        $this->set_error('address', $error_msg, $output);
                        $parent_result = false;
                    }
                }
            }
        }
        return $parent_result;
    }
    public function can_dispose(&$reason = null)
    {
        $usages = $this->node()->software()->application()->check_object_usage('cluster','floating_ip', $this->name());
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return parent::can_dispose($reason);
    }
}
?>
