<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Cluster Peer Node > IP
 * @author Reza Samimi
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
class Sng_cluster_ip_config_class extends Safe_ip_address_class
{
	private $_readonly = false;
    public function configure()
    {
        $cluster = $this->node()->software()->application()->find_module_by_name('cluster');
        $path_info = $this->path_info();
        $interfaces_enum = $cluster->get_network_interfaces($path_info['name']);
        $this->add_enum_field('interface', 'Interface', 'dropdown', '', $interfaces_enum);

        $proto_enum = array(Safe_ip_address_class::PROTO_DHCP4 =>'IPv4 - Dynamic',
                Safe_ip_address_class::PROTO_STATIC4=>'IPv4 - Static',
                Safe_ip_address_class::PROTO_DHCP6 =>'IPv6 - Dynamic',
                Safe_ip_address_class::PROTO_SLAAC6 =>'IPv6 - Automatic',
                Safe_ip_address_class::PROTO_STATIC6 =>'IPv6 - Static'
        );
        $this->add_field('read_only', 'Read Only', 'hidden', 'false');
        $this->add_enum_field('proto', 'Configuration', 'dropdown', Safe_ip_address_class::PROTO_STATIC4, $proto_enum);

        $this->add_field('hostname', 'Hostname', 'text', '', 40);
        $proto_in_value = Safe_ip_address_class::PROTO_DHCP4 . ',' . Safe_ip_address_class::PROTO_DHCP6 . ',' . Safe_ip_address_class::PROTO_SLAAC6;
        $this->set_field_rules('hostname', 'required[proto,in,'. $proto_in_value .']|valid_domain');

        $this->add_enum_field('peerdns', 'Use automatic DNS servers', 'dropdown', 'true', $this->enable_disable_enum());

        $this->add_enum_field('persistent', 'Persistent DHCP client', 'dropdown', 'true', $this->enable_disable_enum());
        $this->set_field_help('persistent', 'Keep retrying to get a DHCP lease indefinitely');

        $group = array('hostname', 'peerdns', 'persistent', 'address', 'prefix');
        $condition = array();
        foreach($group as $item){
            if(in_array($item, array('hostname', 'peerdns', 'persistent'))){
                $condition[Safe_ip_address_class::PROTO_DHCP4][] = array('field_name' => $item, 'status' => 'enable');
                $condition[Safe_ip_address_class::PROTO_DHCP6][] = array('field_name' => $item, 'status' => 'enable');

                if ($item == 'hostname') {
                    $condition[Safe_ip_address_class::PROTO_SLAAC6][] = array('field_name' => $item, 'status' => 'enable');
                } else {
                    $condition[Safe_ip_address_class::PROTO_SLAAC6][] = array('field_name' => $item, 'status' => 'disable');
                }

                $condition[Safe_ip_address_class::PROTO_STATIC4][] = array('field_name' => $item, 'status' => 'disable');
                $condition[Safe_ip_address_class::PROTO_STATIC6][] = array('field_name' => $item, 'status' => 'disable');
            }else{
                $condition[Safe_ip_address_class::PROTO_DHCP4][] = array('field_name' => $item, 'status' => 'disable');
                $condition[Safe_ip_address_class::PROTO_DHCP6][] = array('field_name' => $item, 'status' => 'disable');
                $condition[Safe_ip_address_class::PROTO_SLAAC6][] = array('field_name' => $item, 'status' => 'disable');
                $condition[Safe_ip_address_class::PROTO_STATIC4][] = array('field_name' => $item, 'status' => 'enable');
                $condition[Safe_ip_address_class::PROTO_STATIC6][] = array('field_name' => $item, 'status' => 'enable');
            }
        }
        $this->create_group('proto', $group,
                array('default' => array( 'value' => Safe_ip_address_class::PROTO_STATIC4, 'status' => 'disable'),
                        'condition' =>$condition
                )
        );
        $this->conditional_control('proto', 'proto');

        $this->add_field('address', 'Address', 'text', '', 40);
        $this->set_field_rules('address', 'required[proto,in,'. Safe_ip_address_class::PROTO_STATIC4 .','. Safe_ip_address_class::PROTO_STATIC6 .']|valid_ip');
        $this->add_field('prefix', ' / ', 'string', '', 10);
        $this->set_field_rules('prefix','required[proto,in,'. Safe_ip_address_class::PROTO_STATIC4 . ','. Safe_ip_address_class::PROTO_STATIC6 .']|integer|greater_than[0]|less_or_equal[128]');
        $this->composite_layout('address', array('prefix'));
    }
    public function address(){
        return $this->get_data_value('address');
    }
    public function ip_type()
    {

        $proto = $this->get_data_value('proto', false);
        $type_data = explode('-', $proto);
        $tmp_type = $type_data[1];
        return $tmp_type;
    }
    public function is_dhcp()
    {
        $proto = $this->get_data_value('proto', false);
        return in_array($proto, $this->_dhcp);
    }
    public function get_interface(){
        //get interface obj name
        $interface_obj_name  = $this->get_data_value('interface', false);
        return $interface_obj_name;
    }
    public function get_interface_obj()
    {
        $path_info = $this->path_info();
        $interface_obj_name = $this->get_interface();
        $network_module = $this->node()->software()->application()->find_module_by_name('cluster');

        $peer_obj = $network_module->get_aggregate_object('peer', $path_info['name']);
        $interface_obj = $peer_obj->get_aggregate_object('network_interface', $interface_obj_name);

        if($interface_obj){
            return $interface_obj;
        }else{
            return false;
        }
    }
    public function check_ha_ip_conflict() {
        $path_info = $this->path_info();
        $network_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $network_module->get_aggregate_object('peer', $path_info['name']);
        $conflict_list = array();
        $objs = $peer_obj->get_aggregate_objects('network_ip');

        if($objs){
            foreach($objs as $name => $obj){
                $to_obj_address = $obj->address();
                if($name != $this->name() && $obj->get_data_value('address')){
                    if($obj->get_data_value('address') == $this->get_data_value('address')){
                        $conflict_list[] = $obj;
                    }
                }
            }
        }
        return $conflict_list;
    }
    public function get_interface_type()
    {
        $interface_type = false;
        $interface_obj = $this->get_interface_obj();
        if($interface_obj){
            $interface_type = $interface_obj->get_data_value('type');
        }

        return $interface_type;
    }
    public function validate($data, &$output)
    {
        $parent_result = parent::validate($data, $output);
        $path_info = $this->path_info();

        //check ip
        $ip_type = $this->ip_type();
        $ip = $this->get_data_value('address');
        $prefix = $this->get_data_value('prefix');

        $ip_error_msg = 'The Address field must contain a valid IP.';
        $prefix_error_msg = 'The prefix must contain a number less than 32';

        if($ip_type == Safe_ip_address_class::IP_V4){
            if(!filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)){
                $this->set_error('address', $ip_error_msg, $output);
                $parent_result =  false;
            }
            if($prefix > 32){
                $this->set_error('prefix', $prefix_error_msg, $output);
                $parent_result =  false;
            }
        }
        if($ip_type == Safe_ip_address_class::IP_V6){
            if(!filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)){
                $this->set_error('address', $ip_error_msg, $output);
                $parent_result =  false;
            }
            // prevent from link local address fe80
            else {
                if(strncasecmp($ip, 'fe80', 4) == 0) {
                    $this->set_error('address',$ip_error_msg, $output);
                    $parent_result =  false;
                }
            }
        }

        $network_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $network_module->get_aggregate_object('peer', $path_info['name']);
        // //check if ip conflict
        $ip = $this->get_data_value('address');
        if($ip){
            $ip_conflict_list = $this->check_ha_ip_conflict();
            if($ip_conflict_list){
                $error_msg = 'This ip is in conflict with : ';
                $list = array();
                foreach($ip_conflict_list as $obj){
                    $list[] = $obj->name();
                }
                $error_msg .= join(",", $list);
                $this->set_error('address', $error_msg, $output);
                $parent_result = false;
            }
        }
        //check if already have dhcp ip obj
        if($this->is_dhcp()){
            $ip_objs = $peer_obj->get_aggregate_objects('network_ip');
            $interface = $this->get_interface();
            $my_name = $this->name();
            $my_ip_type = $this->ip_type();
            foreach($ip_objs as $ip_obj_name => $ip_obj){
                if($ip_obj_name != $my_name && $ip_obj->ip_type() == $my_ip_type && $ip_obj->get_interface() == $interface && $ip_obj->is_dhcp()){
                    $rule_msgs['proto'] = 'Already have a DHCP IP attached to this Interface.';
                    $this->set_error('proto', $rule_msgs['proto'], $output);
                    return false;
                }
            }
        }
        //lo interface
        if($this->get_interface_type() == Sng_network_interface_class::LOOPBACK){
            if($this->is_dhcp()){
                $rule_msgs['proto'] = 'Loopback interface does not support DHCP.';
                $this->set_error('proto', $rule_msgs['proto'], $output);
                $parent_result = false;
            }else{
            	//enforce address being on 127 network
                if($this->ip_type() == Safe_ip_address_class::IP_V4){
                    if(!safe_cidr_match_str($this->get_data_value('address'),'127.0.0.0/8') || $this->get_data_value('prefix') < 8){
                        $rule_msgs['address'] = 'Loopback interface supports IPv4 addresses only in the 127.0.0.0/8 range.';
                        $this->set_error('address', $rule_msgs['address'], $output);
                        $parent_result = false;
                    }
                }else{
                    $rule_msgs['address'] = 'Loopback interface supports only one IPv6 address.';
                    $this->set_error('address', $rule_msgs['address'], $output);
                    $parent_result = false;
                }
            }
        }
        // TODO: This should be temporary until a proper solution is in place to account for 'short' (composite) and 'long' (error msg for example) labels
        if ($output['prefix']) {
		    $this->set_error('prefix',str_replace('/', 'prefix', $output['prefix']) , $output);
		}
        return $parent_result;
    }
    public function can_modify(&$output = null)
    {
        if ($this->is_readonly()) {
            $this->set_error('interface', "Cannot modify system IP address.", $output);
            return false;
        }
        return parent::can_modify($output);
    }

    public function save() {
        $result = parent::save();
        $path_info = $this->path_info();

        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);

        if (!$peer_obj) return $result;

        if ($peer_obj->get_data_value('node_registered') == 'true') {
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name'], 'save');
        }

        return $result;
    }
    public function dispose() {
        if (parent::dispose()) {
            $path_info = $this->path_info();

            $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name']);

            return true;
        }
        return false;
    }
    public function is_readonly(&$output = null)
    {
        $interface_obj = $this->get_interface_obj();
        if($interface_obj && $interface_obj->hardware_type() == Sng_network_interface_class::TRANSCODING && !safe_development_mode()){
            //only in development_mode allow user to add/remove/change the ip on sngdspX interfaces
            return true;
        }elseif($this->ip_type() == Safe_ip_address_class::IP_V6 &&
                strncasecmp($this->address(), 'fe80', 4) == 0) {
            return true;
        } elseif ($this->get_data_value('read_only') == 'true') {
            return true;
        }else{
            return parent::is_readonly($output) ;
        }
    }
    public function summary()
    {
        $ip = '';
        $address = $this->get_data_value('address') ;
        $prefix = $this->get_data_value('prefix');
        if($address){
            $ip = $address;
            if($prefix){
                $ip = $ip . '/' . $prefix;
            }
        }else{
            $status_data = Safe_icon('spinner', 'info', array('spin'));
            $ip = array(
                'id' => 'network_ip_' . $this->name(),
                'data' => $status_data,
                'formatter' => 'none',
            );
        }
        $hostname = $this->get_data_value('hostname');
        $result =  array(
                'Interface' => $this->get_data_value('interface', false),
                'Type' => $this->get_data_value('proto'),
                'IP Address' => $ip,
                'Hostname' => $hostname
        );
        return array( 'data' => $result);
    }
}

class Sng_cluster_loopback_ip_config_class  extends Sng_cluster_ip_config_class
{
    public function configure()
    {
        $interfaces_enum = array('lo' => 'lo');
        $this->add_field('read_only', 'Read Only', 'hidden', 'true');
        $this->add_enum_field('interface', 'Interface', 'dropdown', '', $interfaces_enum);
        $proto_enum = array(
                Safe_ip_address_class::PROTO_STATIC4=>'IPv4 - Static',
                Safe_ip_address_class::PROTO_STATIC6 =>'IPv6 - Static'
        );
        $this->set_field_disabled('interface', true);
        $this->add_enum_field('proto', 'Configuration', 'dropdown', Safe_ip_address_class::PROTO_STATIC4, $proto_enum);
        $this->set_field_disabled('proto', true);
        $this->add_field('hostname', 'Hostname', 'text', '', 40);
        $this->set_field_disabled('hostname', true);
        $this->add_field('address', 'Address', 'text', '', 40);
        $this->set_field_disabled('address', true);
        $this->add_field('prefix', ' / ', 'string', '', 10);
        $this->set_field_disabled('prefix', true);
        $this->composite_layout('address', array('prefix'));
    }
    public function is_readonly()
    {
        if($this->node()->get_config_status('update') == 'rest'){
            return false;
        }else{
            return true;
        }
    }
    public function prepare_edit()
    {
        if($this->node()->get_config_status('update') == 'rest'){
            $this->set_field_disabled('proto', false);
            $this->set_field_disabled('hostname', false);
            $this->set_field_disabled('address', false);
            $this->set_field_disabled('prefix', false);
        }
    }
    public function can_dispose(&$reason = null)
    {
        if($this->node()->get_config_status('update') == 'rest'){
            return parent::can_dispose();
        }else{
            $reason = " Loopback IP cannot be deleted. ";
            return false;
        }
    }
    public function can_modify(&$output = null)
    {
        if($this->node()->get_config_status('update') == 'rest'){
            return parent::can_modify();
        }else{
            $this->set_error('interface', "Loopback IP cannot be modified.", $output);
            return false;
        }
    }
    public function validate($data, $output)
    {
        if(isset($data['peerdns'])){
            unset($data['peerdns']);
        }
        if(isset($data['persistent'])){
            unset($data['persistent']);
        }
        return parent::validate($data, $output);
    }
}
