<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Cluster Peer Node > Media Interfaces
 * @author Reza Samimi
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
class Sng_cluster_media_interfaces_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {

        $network= $this->node()->software()->application()->find_module_by_name('network');
        $transcoding_interfaces = $network->hardware_interface(Sng_network_interface_class::TRANSCODING);

        $internal_adapter = false;
        if($transcoding_interfaces){
            $internal_adapter = true;
        }
        $mode = array(
            Sng_transcoding_config_class::SOFTWARE_MODE => 'Software',
            Sng_transcoding_config_class::HIDDEN_MODE  => 'Hardware Hidden',
        );

        $media_int_group_field = array();
        $global_group_field = array();
        if ($internal_adapter) {
            $default_udp_begin = '14000';
            $default_udp_end = '17999';
        }else{
            $default_udp_begin = '10000';
            $default_udp_end = '20000';
        }

        $this->add_enum_field('mode', 'Transcoding Mode', 'dropdown', Sng_transcoding_config_class::HIDDEN_MODE, $mode);
        $this->set_field_help('mode',
                array(
                        'Sets the Transcoding Mode',
                        array(
                                'Software' => array(
                                        'Disable media interfaces.',
                                        ),
                              ),
                        array(
                                'Hidden' => array(
                                        'All media interfaces will be hidden and protected from the external network. Signaling and media will flow together thru the primary network interfaces (ie, eth0 and eth1).',
                                        'Do NOT plug an ethernet cable to the media ethernet port (D500/D100).'
                                        )
                             )
                    )
            );

        $this->add_field('udp_begin', 'First UDP port', 'text', $default_udp_begin);
        $this->set_field_rules('udp_begin', 'required|valid_port_number|greater_or_equal[5004]');
        $this->set_field_help('udp_begin', 'Starting UDP port to be assigned to the media modules.');
        $this->add_field('udp_end', 'Last UDP port', 'text', $default_udp_end);
        $this->set_field_rules('udp_end', 'valid_port_number');
        $this->set_field_help('udp_end', 'Ending UDP port to be assigned to the media modules.');
        $this->add_field('vlan_id', 'VLAN Identifier', 'string', '', 10);
        $this->set_field_rules('vlan_id', 'greater_than[0]|less_than[4097]');
        $this->set_field_help('vlan_id', 'Specify the VLAN identifier that will be used on all media modules discovered.');
        $this->set_field_category('mode', 'General');

        $this->add_field('ip_address', 'Base IPv4 address', 'text', '10.10.0.1');

        $this->set_field_rules('ip_address', 'required|valid_ip[ipv4]');

        $this->set_field_help('ip_address', array(
            'Starting IP Address to be assigned to the media modules.',
            'For every module discovered, the IP address will be incremented by 1.'
        ));
        $this->add_field('ext_ip_address', 'Base External IPv4 address', 'text', '');
        $this->set_field_rules('ext_ip_address', 'valid_ip[ipv4]');
        $this->set_field_help('ext_ip_address', array(
                'This is the starting IP address that system will use in the SDP offer or answer. For every module discovered, the IP address will be incremented by 1.',
                'If not specified, system will use the assigned IP address to the media module(default behavior).',
                '',
                'When system is behind a firewall (ie: DMZ or port-forwarding) with RTP ports being forwarded to the media module,
    		it may be desirable to specify this parameter to an IP address that will be mapped by the firewall port-forwarding capabilities to the IP address of the media module.
    		If not set when working under a DMZ or port-forwarding scenario, calls may end up not having audio.
    		'
        ));
        $this->add_field('mask', 'IPv4 Network Mask', 'text', '255.255.255.0');
        $this->set_field_rules('mask', 'required|valid_ip[ipv4]');
        $this->set_field_help('mask', 'Subnet mask to be assigned to the media modules.');
        $this->add_field('gateway', 'Default IPv4 Gateway', 'text', '10.10.0.100');
        $this->set_field_rules('gateway', 'valid_ip[ipv4]');
        $this->set_field_help('gateway', 'Default Gateway to be assigned to the media modules.');

        //set a default ip for Base IPv6 address
        $this->add_field('ipv6_address', 'Base IPv6 address', 'text', '2001:db8::99:2', 40);
        $this->set_field_rules('ipv6_address', 'valid_ip[ipv6]');

        $this->add_field('ext_ipv6_address', 'Base External IPv6 address', 'text', '', 40);
        $this->set_field_rules('ext_ipv6_address', 'valid_ip[ipv6]');

        $this->add_field('ipv6_prefix', 'IPv6 prefix', 'text', '120');
        $this->set_field_rules('ipv6_prefix', 'integer|greater_than[0]|less_or_equal[128]');

        $this->add_field('ipv6_gateway', 'Default IPv6 Gateway', 'text', '2001:db8::99:1', 40);
        $this->set_field_rules('ipv6_gateway', 'valid_ip[ipv6]');


        // Create group for mode control
        $media_int_group_field = array_merge($media_int_group_field, array(
                    'ip_address',
                    'ipv6_address',
                    'vlan_id',
                    'ext_ip_address',
                    'ext_ipv6_address',
                    'mask',
                    'ipv6_prefix',
                    'gateway',
                    'ipv6_gateway',
                    'udp_end',
                    'vqe/acoustic_echo',
                    'vqe/rx_adaptive_noise_reduction',
                    'vqe/rx_automatic_level_control',
                    'vqe/rx_automatic_level_control_target_dbm0',
                    'vqe/tx_automatic_level_control',
                    'vqe/tx_automatic_level_control_target_dbm0',
                    'jitter/mode',
                    'jitter/max_pdv_ms',
                    'jitter/min_delay_ms',
                    'jitter/initial_delay_ms'
                ));
        $hardware_enable_file = array(
                    'ip_address',
                    'ipv6_address',
                    'vlan_id',
                    'ext_ip_address',
                    'ext_ipv6_address',
                    'mask',
                    'ipv6_prefix',
                    'gateway',
                    'ipv6_gateway',
                    'udp_end'
                    );
        foreach($media_int_group_field as $group_key){
            if($group_key == 'udp_end'){
                $condition_data[Sng_transcoding_config_class::HIDDEN_MODE][] = array('field_name' =>$group_key, 'status' => 'disable');
                $condition_data[Sng_transcoding_config_class::SOFTWARE_MODE][] = array('field_name' =>$group_key, 'status' => 'enable');
            }elseif(in_array($group_key, $hardware_enable_file)){
                $condition_data[Sng_transcoding_config_class::HIDDEN_MODE][] = array('field_name' =>$group_key, 'status' => 'disable');
                $condition_data[Sng_transcoding_config_class::SOFTWARE_MODE][] = array('field_name' =>$group_key, 'status' => 'disable');
            }else{
                $condition_data[Sng_transcoding_config_class::HIDDEN_MODE][] = array('field_name' =>$group_key, 'status' => 'enable');
                $condition_data[Sng_transcoding_config_class::SOFTWARE_MODE][] = array('field_name' =>$group_key, 'status' => 'disable');
            }
        }
        $this->create_group('media_int',
                $media_int_group_field,
                array('default' => array( 'value' => 'true', 'status' => 'enable'),
                        'condition' => $condition_data
                )
        );
        $this->conditional_control('mode', 'media_int');

        // VQE
        $this->add_enum_field('vqe/acoustic_echo', 'Acoustic Echo Cancellation', 'dropdown', '0', $this->enable_disable_enum(false));
        $this->set_field_help('vqe/acoustic_echo',
            array(
                'Enable acoustic echo cancellation for all media sessions.',
                'This reduces echo on calls but increases the possibility of double-talk clipping.',
                'Use this value for situations where the echo path is highly non-linear.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 35%.</font>')
            );

        $this->add_enum_field('vqe/rx_adaptive_noise_reduction', 'Adaptive Noise Reduction', 'dropdown', '0', $this->enable_disable_enum(false));
        $this->set_field_help('vqe/rx_adaptive_noise_reduction',
            array(
                'Enable Adaptive Noise Reduction for all media sessions.',
                'This removes the noise present while leaving voice intact.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 35%.</font>')
            );

        $this->add_enum_field('vqe/rx_automatic_level_control', 'Automatic Level Control (RX)', 'dropdown', '0', $this->enable_disable_enum(false));
        $this->set_field_help('vqe/rx_automatic_level_control',
            array(
                'Enable RX Automatic Level Control for all media sessions.',
                'Using the level threshold limit specified in Automatic Level Control Target, dBm0 level is being filtered.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 15%.</font>')
            );
        $this->add_field('vqe/rx_automatic_level_control_target_dbm0', 'Target', 'integer', -21, 5);
        $this->set_field_rules('vqe/rx_automatic_level_control_target_dbm0', 'required[vqe/rx_automatic_level_control]|integer|greater_or_equal[-42]|less_or_equal[-6]');
        $this->composite_layout('vqe/rx_automatic_level_control', array('vqe/rx_automatic_level_control_target_dbm0'));

        $this->add_enum_field('vqe/tx_automatic_level_control', 'Automatic Level Control (TX)', 'dropdown', '0', $this->enable_disable_enum(false));
        $this->set_field_help('vqe/tx_automatic_level_control',
            array(
                'Enable TX Automatic Level Control for all media sessions.',
                'Using the level threshold limit specified in Automatic Level Control Target, dBm0 level is being filtered.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 15%.</font>')
            );
        $this->add_field('vqe/tx_automatic_level_control_target_dbm0', 'Target', 'integer', -21, 5);
        $this->set_field_rules('vqe/tx_automatic_level_control_target_dbm0', 'required[vqe/tx_automatic_level_control]|integer|greater_or_equal[-42]|less_or_equal[-6]');
        $this->composite_layout('vqe/tx_automatic_level_control', array('vqe/tx_automatic_level_control_target_dbm0'));

        // Create conditional groups
        $this->create_group('rx_alc', array('vqe/rx_automatic_level_control_target_dbm0'),
                array(  'default' => array( 'value' => '0', 'status' => 'disable'),
                        'condition' => array('1' => array('field_name' =>'vqe/rx_automatic_level_control_target_dbm0','status' => 'disable'))
                        )
                );
        $this->conditional_control('vqe/rx_automatic_level_control', 'rx_alc');

        $this->create_group('tx_alc', array('vqe/tx_automatic_level_control_target_dbm0'),
                array(  'default' => array( 'value' => '0', 'status' => 'disable'),
                        'condition' => array('1' => array('field_name' =>'vqe/tx_automatic_level_control_target_dbm0','status' => 'disable'))
                )
                );
        $this->conditional_control('vqe/tx_automatic_level_control', 'tx_alc');

        // jitter buffer configuration
        $this->add_enum_field('jitter/mode', 'Jitter Mode', 'dropdown',
                'adaptive', array (
                    'static' => 'Static',
                    'adaptive' => 'Adaptive'
               )
            );
        $this->set_field_help('jitter/mode', array (
                    'Jitter buffer mode to operate on.',
                    'Static allocates a fixed buffer.',
                    'Adaptive changes the buffer on runtime depending on the network'
                    )
                );

        $this->add_field('jitter/initial_delay_ms', 'Jitter Initial
                Delay', 'integer', 40, 5);
        $this->set_field_rules('jitter/initial_delay_ms',
                'required[jitter/mode]|integer|greater_or_equal[20]|less_or_equal[jitter/max_pdv_ms]');
        $this->set_field_help('jitter/initial_delay_ms','Initial delay (buffer size) in milliseconds.');

        $this->add_field('jitter/max_pdv_ms', 'Jitter Max PDV',
                'integer', 100, 5);
        $this->set_field_rules('jitter/max_pdv_ms',
                'required[jitter/mode]|integer|greater_or_equal[20]|less_or_equal[200]');
        $this->set_field_help('jitter/max_pdv_ms','Maximum tolerated
                packet delay variation in milliseconds.');

        $this->add_field('jitter/min_delay_ms', 'Jitter Minimal Delay',
                'integer', 20, 5);
        $this->set_field_rules('jitter/min_delay_ms',
                'required[jitter/mode]|integer|greater_or_equal[20]|less_or_equal[jitter/initial_delay_ms]');
        $this->set_field_help('jitter/min_delay_ms','Minimal delay tolerated in milliseconds.');

        $conditions['adaptive'][] = array ( 'field_name' =>
                'jitter/min_delay_ms', 'status' => 'enable');

        $conditions['adaptive'][] = array ( 'field_name' =>
                'jitter/max_pdv_ms', 'status' => 'enable');

        $conditions['static'][] = array ( 'field_name' =>
                'jitter/min_delay_ms', 'status' => 'disable');

        $conditions['static'][] = array ( 'field_name' =>
                'jitter/max_pdv_ms', 'status' => 'disable');

        $this->create_group('jitter_group', array('jitter/max_pdv_ms',
                    'jitter/min_delay_ms'),
                array(
                    'default' => array( 'value' => 'adaptive', 'status' => 'enable'),
                     'condition' => $conditions
                     )
                );
        $this->conditional_control('jitter/mode', 'jitter_group');

        // Set categories

        $this->set_field_category('ip_address', ' Network IPv4');
        $this->set_field_category('ipv6_address', 'Network IPv6');
        $this->set_field_category('vqe/acoustic_echo', 'VQE');

        $this->set_field_category('jitter/mode', 'Jitter');

        return parent::configure();
    }
    /**
     * Returns the IP Mode of the media interfaces (Hidden)
     * @return string
     */
    public function ip_mode()
    {
        $transcoding_mode_data = $this->get_data_value('mode', false);
        return $transcoding_mode_data;
    }
    public function save() {
        $result = parent::save();
        $path_info = $this->path_info();

        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);
        if (!$peer_obj) return $result;

        if ($peer_obj->get_data_value('node_registered') == 'true') {
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name'], 'save');
        }
        return $result;
    }
    public function summary() {
        $result = parent::summary();
        // replace name with 'Network Configuration'
        if($result && isset($result['data']['Name'])){
            $result['data']['Name'] ='Media Server Configuration';
        }
        return $result;
    }
}
class Sng_cluster_media_modules_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        // Ensure the configurable objec is not locked (cause we're using the
        // name() to get MAC address
        $this->discard_lock();
        if (!isset($network_cfg)) {
            $network_cfg = array(
                'ifname' => '',
                'ip_address' => '',
                'ipv6_address' => '',
                'mask' => '',
                'ipv6_prefix' => '',
                'gateway' => '',
                'ipv6_gateway' => '',
                'udp_begin' => '14000',
                'udp_end' => '16000',
                'ext_ip_address' => '',
                'ext_ipv6_address' => '',
                'vlan_id' => '',
                'vqe/acoustic_echo' => '0',
                'vqe/rx_adaptive_noise_reduction' => '0',
                'vqe/rx_automatic_level_control' => '0',
                'vqe/rx_automatic_level_control_target_dbm0' => -21,
                'vqe/tx_automatic_level_control' => '0',
                'vqe/tx_automatic_level_control_target_dbm0' => -21
            );
        }
        if (empty($version)) $capacity = '';
        if (empty($capacity)) $capacity = '480';
        if (empty($port)) $port = '1';

        // Create configuratble fields
        $this->add_field('ip_map', 'IP Map', 'hidden', array(), 50);
        $this->add_enum_field('enable', 'Enable', 'dropdown', 'true', $this->enable_disable_enum());
        $this->set_field_help('enable', 'Enable/Disable this particular module.');
        $this->add_field('ifname', 'Interface', 'string', $network_cfg['ifname']);
        $this->set_field_readonly('ifname');
        $this->add_field('mac', 'MAC Address', 'string', $this->name());
        $this->set_field_readonly('mac');
        $this->add_field('version', 'Version', 'string', $version);
        $this->set_field_readonly('version');
        $this->add_field('capacity', 'Capacity', 'hidden', $capacity);
        $this->add_field('port', 'Port', 'hidden', $port);

        $this->add_field('vlan_id', 'VLAN Identifier', 'string', $network_cfg['vlan_id'], 10);
        $this->set_field_rules('vlan_id', 'greater_than[0]|less_than[4097]');
        $this->set_field_help('vlan_id', array(
                '(Optional) Identifies which VLAN this module is part of.',
                'If the VLAN ID was entered in the Media Interface, it does not need to be entered here unless the VLAN ID is different.',
                'A VLAN ID is not required, and will be ignored, when the Media Interfaces are set to Hidden.'
        ));
        $this->add_field('udp_begin', 'First UDP port', 'text', $network_cfg['udp_begin']);
        $this->set_field_rules('udp_begin', 'required|valid_port_number|greater_or_equal[5004]');
        $this->set_field_help('udp_begin', 'Starting UDP port being used by this module');
        $this->add_field('udp_end', 'Last UDP port', 'text', $network_cfg['udp_end']);
        $this->set_field_rules('udp_end', 'required|valid_port_number');
        $this->set_field_help('udp_end', 'Ending UDP port being used by this module.');


        //IPv4 config
        $this->add_field('ip_address', 'IPv4 address', 'text', $network_cfg['ip_address']);
        $this->set_field_rules('ip_address', 'required[enable]|valid_ip_array[ipv4]');
        $this->set_field_help('ip_address', 'IPv4 address of this particular module.');

        $this->add_field('ext_ip_address', 'External IPv4 address', 'text', $network_cfg['ext_ip_address']);
        $this->set_field_rules('ext_ip_address', 'valid_ip[ipv4]');
        $this->set_field_help('ext_ip_address', array(
                'Optional parameter.',
                'IPv4 address used by gateway in the SDP offer or answer. ',
                'If not specified, gateway uses the assigned IP address to the media interface(default behavior).',
                'When gateway is behind a firewall (ie: DMZ or port-forwarding) with RTP ports being forwarded to the media interface, it may be desirable to specify this parameter to an IP address that will be mapped by the firewall port-forwarding capabilities to the internal IP address of the media interface, if not set when working under a DMZ or port-forwarding scenario, calls may end up not having audio.'
        ));

        $this->add_field('mask', 'IPv4 Network Mask', 'text', $network_cfg['mask']);
        $this->set_field_rules('mask', 'required[enable]|valid_ip[ipv4]');
        $this->set_field_help('mask', 'Subnet Mask of this particular module');

        $this->add_field('gateway', 'IPv4 Gateway', 'text', $network_cfg['gateway']);
        $this->set_field_rules('gateway', 'valid_ip[ipv4]');
        $this->set_field_help('gateway', 'Default IPv4 gateway of this particular module.');

        //IPv6 config
        $this->add_field('ipv6_address', 'IPv6 address', 'text', $network_cfg['ipv6_address'], 40);
        $this->set_field_rules('ipv6_address', 'valid_ip[ipv6]');

        $this->add_field('ext_ipv6_address', 'External IPv6 address', 'text', $network_cfg['ext_ipv6_address'], 40);
        $this->set_field_rules('ext_ipv6_address', 'valid_ip[ipv6]');

        $this->add_field('ipv6_prefix', 'IPv6 prefix', 'text', $network_cfg['ipv6_prefix']);
        $this->set_field_rules('ipv6_prefix', 'integer|greater_than[0]|less_or_equal[128]');

        $this->add_field('ipv6_gateway', 'IPv6 Gateway', 'text', $network_cfg['ipv6_gateway'], 40);
        $this->set_field_rules('ipv6_gateway', 'valid_ip[ipv6]');

        // Hidden stuff for module config
        $this->add_field('max_sessions', 'Maximum number of sessions', 'hidden', '960');
        $this->add_field('session_high_watermark', 'Session High Watermark', 'hidden', '500');
        $this->add_field('session_low_watermark', 'Session Low Watermark', 'hidden', '450');
        // VQE
        $this->add_enum_field('vqe/acoustic_echo', 'Acoustic Echo Cancellation', 'dropdown',  $network_cfg['vqe/acoustic_echo'], $this->enable_disable_enum(false));
        $this->set_field_disabled('vqe/acoustic_echo', 'disabled');
        $this->set_field_help('vqe/acoustic_echo',
            array(
                'Enable acoustic echo cancellation for all media sessions.',
                'This reduces echo on calls but increases the possibility of double-talk clipping.',
                'Use this value for situations where the echo path is highly non-linear.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 35%.</font>')
            );

        $this->add_enum_field('vqe/rx_adaptive_noise_reduction', 'Adaptive Noise Reduction', 'dropdown', $network_cfg['vqe/rx_adaptive_noise_reduction'], $this->enable_disable_enum(false));
        $this->set_field_disabled('vqe/rx_adaptive_noise_reduction', 'disabled');
        $this->set_field_help('vqe/rx_adaptive_noise_reduction',
            array(
                'Enable Adaptive Noise Reduction for all media sessions.',
                'This removes the noise present while leaving voice intact.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 35%.</font>')
            );

        $this->add_enum_field('vqe/rx_automatic_level_control', 'Automatic Level Control (RX)', 'dropdown', $network_cfg['vqe/rx_automatic_level_control'], $this->enable_disable_enum(false));
        $this->set_field_disabled('vqe/rx_automatic_level_control', 'disabled');
        $this->set_field_help('vqe/rx_automatic_level_control',
            array(
                'Enable RX Automatic Level Control for all media sessions.',
                'Using the level threshold limit specified in Automatic Level Control Target, dBm0 level is being filtered.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 15%.</font>')
            );
        $this->add_field('vqe/rx_automatic_level_control_target_dbm0', 'Target', 'integer', $network_cfg['vqe/rx_automatic_level_control_target_dbm0'], 5);
        $this->set_field_rules('vqe/rx_automatic_level_control_target_dbm0', 'required[vqe/rx_automatic_level_control]|integer|greater_or_equal[-42]|less_or_equal[-6]');
        $this->set_field_disabled('vqe/rx_automatic_level_control_target_dbm0', 'disabled');
        $this->composite_layout('vqe/rx_automatic_level_control', array('vqe/rx_automatic_level_control_target_dbm0'));

        $this->add_enum_field('vqe/tx_automatic_level_control', 'Automatic Level Control (TX)', 'dropdown', $network_cfg['vqe/tx_automatic_level_control'], $this->enable_disable_enum(false));
        $this->set_field_help('vqe/tx_automatic_level_control',
            array(
                'Enable TX Automatic Level Control for all media sessions.',
                'Using the level threshold limit specified in Automatic Level Control Target, dBm0 level is being filtered.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 15%.</font>')
            );
        $this->set_field_disabled('vqe/tx_automatic_level_control', 'disabled');
        $this->add_field('vqe/tx_automatic_level_control_target_dbm0', 'Target', 'integer',  $network_cfg['vqe/tx_automatic_level_control_target_dbm0'], 5);
        $this->set_field_rules('vqe/tx_automatic_level_control_target_dbm0', 'required[vqe/tx_automatic_level_control]|integer|greater_or_equal[-42]|less_or_equal[-6]');
        $this->set_field_disabled('vqe/tx_automatic_level_control_target_dbm0', 'disabled');
        $this->composite_layout('vqe/tx_automatic_level_control', array('vqe/tx_automatic_level_control_target_dbm0'));

        // jitter buffer configuration
        $this->add_enum_field('jitter/mode', 'Jitter Mode', 'dropdown',
                $network_cfg['jitter/mode'], array (
                    'static' => 'Static',
                    'adaptive' => 'Adaptive'
               )
            );
        $this->set_field_disabled('jitter/mode', 'disabled');
        $this->set_field_help('jitter/mode', array (
                    'Jitter buffer mode to operate on.',
                    'Static allocates a fixed buffer.',
                    'Adaptive changes the buffer on runtime depending on the network.'
                    )
                );

        $this->add_field('jitter/initial_delay_ms', 'Jitter Initial
                Delay', 'integer',
                $network_cfg['jitter/initial_delay_ms'], 5);
        $this->set_field_disabled('jitter/initial_delay_ms', 'disabled');
        $this->set_field_help('jitter/initial_delay_ms','Initial delay (buffer size) in milliseconds.');


        $this->add_field('jitter/max_pdv_ms', 'Jitter Max PDV',
                'integer', $network_cfg['jitter/max_pdv_ms'], 5);
        $this->set_field_disabled('jitter/max_pdv_ms', 'disabled');
        $this->set_field_help('jitter/max_pdv_ms','Maximum tolerated
                packet delay variation in milliseconds.');

        $this->add_field('jitter/min_delay_ms', 'Jitter Minimal Delay',
                'integer', $network_cfg['jitter/min_delay_ms'], 5);
        $this->set_field_disabled('jitter/min_delay_ms', 'disabled');
        $this->set_field_help('jitter/min_delay_ms','Minimal delay tolerated in milliseconds.');

        // Set categories
        $this->set_field_category('enable', 'General');
        $this->set_field_category('ip_address', 'Network IPv4');
        $this->set_field_category('ipv6_address', 'Network IPv6');
        $this->set_field_category('vqe/acoustic_echo', 'VQE');

        $this->set_field_category('jitter/mode', 'Jitter');
        $this->set_field_category('jitter/max_pdv_ms', 'Jitter');
        $this->set_field_category('jitter/initial_delay_ms', 'Jitter');
        $this->set_field_category('jitter/min_delay_ms', 'Jitter');

        return parent::configure();
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        $path_info = $this->path_info();
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        if ($cluster_module != NULL) {
            $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);
            if ($peer_obj) {
                $sngms_configuration = $peer_obj->get_aggregate_objects('sngms_configuration');
                foreach ($sngms_configuration as $key=>$module) {
                   $mode = $module->ip_mode();
                }
                $ip_list = $this->get_data_value('ip_address');
                if (is_array($ip_list)) {
                    $ip_list = implode(PHP_EOL,$ip_list);
                }
                $this->unset_field('ip_address');
                $this->add_field('ip_address', 'IPv4 address', 'textarea', $ip_list, array('30', '5'));
                $this->set_field_rules('ip_address', '');
                $this->set_field_readonly('ip_address');

                $ipv6_list = $this->get_data_value('ipv6_address');
                if (is_array($ipv6_list)) {
                    $ipv6_list = implode(PHP_EOL,$ipv6_list);
                }
                $this->unset_field('ipv6_address');
                $this->add_field('ipv6_address', 'IPv6 address', 'textarea', $ipv6_list, array('30', '5'));
                $this->set_field_rules('ipv6_address', '');
                $this->set_field_readonly('ipv6_address');

                if($mode == Sng_transcoding_config_class::HIDDEN_MODE){
                    $this->set_field_disabled('ext_ip_address', true);
                    $this->set_field_disabled('ext_ipv6_address', true);
                    $this->create_group('module', array(
                        'ip_address',
                        'ipv6_address',
                        'mask',
                        'ipv6_prefix',
                        'gateway',
                        'ipv6_gateway',
                        'udp_begin',
                        'udp_end',
                        'vlan_id',
                    ));
                }else {
                    $this->create_group('module', array(
                        'ip_address',
                        'ipv6_address',
                        'ext_ip_address',
                        'ext_ipv6_address',
                        'mask',
                        'ipv6_prefix',
                        'ipv6_gateway',
                        'gateway',
                        'udp_begin',
                        'udp_end',
                        'vlan_id',
                    ));
                }
                $this->conditional_control('enable', 'module');
            }
        }
    }
    public function mac()
    {
        return $this->get_data_value('mac');
    }
    public function ifname()
    {
        return $this->get_data_value('ifname', false);
    }
    public function ip_address($type = Safe_ip_address_class::IP_V4)
    {
        if($type == Safe_ip_address_class::IP_V6){
            return $this->get_data_value('ipv6_address');
        }else{
            return $this->get_data_value('ip_address');
        }
    }
    public function vlan_id()
    {
        return $this->get_data_value('vlan_id');
    }
    public function ext_ip_address()
    {
        return $this->get_data_value('ext_ip_address');
    }
    public function mask()
    {
        return $this->get_data_value('mask');
    }
    public function prefix()
    {
        return $this->get_data_value('ipv6_prefix');
    }
    public function gateway($type = Safe_ip_address_class::IP_V4)
    {
        if($type == Safe_ip_address_class::IP_V6){
            return $this->get_data_value('ipv6_gateway');
        }else{
            return $this->get_data_value('gateway');
        }
    }
    public function version()
    {
        return $this->get_data_value('version');
    }
    public function capacity()
    {
        return $this->get_data_value('capacity');
    }
    public function port()
    {
        return $this->get_data_value('port');
    }
    public function udp_begin()
    {
        return $this->get_data_value('udp_begin');
    }
    public function udp_end()
    {
        return $this->get_data_value('udp_end');
    }
    public function enable()
    {
        return ('true' == $this->get_data_value('enable', false));
    }
    public function max_sessions()
    {
        return $this->get_data_value('max_sessions');
    }
    public function vqe()
    {
        $data = $this->get_data_values_exploded(false);

        return $data['vqe'];
    }
    public function jitter()
    {
        $data = $this->get_data_values_exploded(false);

        return $data['jitter'];
    }
    public function validate($data, &$output){
        if (isset($data['ip_address']) && !is_array($data['ip_address'])) {
            $data['ip_address'] = explode(PHP_EOL, $data['ip_address']);
            $this->set_data_values($data);
        }
        if (isset($data['ipv6_address']) && !is_array($data['ipv6_address'])) {
            $data['ipv6_address'] = explode(PHP_EOL, $data['ipv6_address']);
            $this->set_data_values($data);
        }
        $validate_result = parent::validate($data, $output);
        $new_data = $this->get_data_values(false);
        if($new_data['enable'] == 'true'){
            $port_begin = intval($new_data['udp_begin']);
            $port_end = intval($new_data['udp_end']);
            $rule_msgs = array('udp_end' => 'Last UDP port must be greater than first UDP port');
            if($port_begin >= $port_end ){
                $validate_result = false;
                $this->set_error('udp_end', $rule_msgs['udp_end'], $output);
            }
        }
        return $validate_result;
    }
    public function summary($type = 'horizontal' , $long = false){
        $version = $this->version();

        $ipv4_list = $this->get_data_value('ip_address',false);
        if(is_array($ipv4_list)){
            $ip[Safe_ip_address_class::IP_V4] = array_shift($ipv4_list);
            if(count($ipv4_list)>0){
                $ip[Safe_ip_address_class::IP_V4] = $ip[Safe_ip_address_class::IP_V4] . ' - '. array_pop($ipv4_list);
            }
        }else{
            $ip[Safe_ip_address_class::IP_V4] = $ipv4_list;
        }
        $ipv6_list = $this->get_data_value('ipv6_address',false);
        if(is_array($ipv6_list)){
            $ip[Safe_ip_address_class::IP_V6] = array_shift($ipv6_list);
            if(count($ipv6_list)>0){
                $ip[Safe_ip_address_class::IP_V6] = $ip[Safe_ip_address_class::IP_V6] . ' - '. array_pop($ipv6_list);
            }
        }else{
            $ip[Safe_ip_address_class::IP_V6] = $ipv6_list;
        }
        $table_line = array(
                "MAC" => $this->mac(),
                "Version" => array('data' => $version, 'formatter' => 'none'),
                "IP address" => $ip,
                "UDP Ports" => $this->udp_begin() . ' - ' . $this->udp_end()
        );
        return array('data' => $table_line);
    }
    public function save() {
        $result = parent::save();
        $path_info = $this->path_info();

        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);
        if (!$peer_obj) return $result;

        if ($peer_obj->get_data_value('node_registered') == 'true') {
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name'], 'save');
        }
        return $result;
    }
}
class Sng_cluster_media_device_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $path,$name)
    {
        parent::__construct($path, $name, $node);
    }
    public function configure($version = null, $capacity = null, $port = null, $network_cfg = null)
    {
        $this->add_enum_field('split', 'Split Modules', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('split', 'When set to true macvlan devices will be created on demand per each vocallo module on this sngdsp interafce.');
        return parent::configure();
    }
    public function mac()
    {
        $path_info = $this->path_info();
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);
        $interfaces = $peer_obj->api_retrieve_network_interface();
        foreach ($interfaces as $interface) {
            if ($interface->get_data_value('vlan_type', false) == 'macvlan') {
                if ($interface->get_data_value('macvlan_device') == $this->name()) {
                    return $interface->get_data_value('macvlan_mac');
                }
            }
            else {
                if ($interface->name() == $this->name()) {
                    return $interface->mac();
                }
            }
        }
        return;
    }
    public function summary()
    {
        $result['data']['Name'] = array(
                'data' => $this->name(),
        );
        $result['data']['MAC'] = array(
                        'data' => $this->mac(),
                );
        $result['data'][$this->get_data_label('split')] = array(
                'data' => $this->get_data_value('split'),
        );
        return $result;
    }
        public function save() {
        $result = parent::save();
        $path_info = $this->path_info();

        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);
        if (!$peer_obj) return $result;

        if ($peer_obj->get_data_value('node_registered') == 'true') {
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name'], 'save');
        }
        return $result;
    }
}

