<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * SNG HA Cluster Module
 * @author Reza Samimi
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
// safe_require_class('safe_module_class');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');
safe_module_require_class('sng', 'application');
safe_module_require_class('sng', 'cluster_access_key_config');
safe_module_require_class('sng', 'cluster_ip');
safe_module_require_class('sng', 'cluster_trigger');

class Sng_cluster_peer_config_class extends Safe_configurable_object_class
{
    /**
     * @brief Constructor
     *
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return
     */
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    /**
     * @brief Configure the config class object
     *
     * @return
     */
    public function configure()
    {
    }
}
class Sng_cluster_config_class extends Safe_configurable_object_class
{
    private $_the_node = NULL;
    const HA_MODE = 'ha';
    const STANDALONE_MODE = 'standalone';
    const DEDICATED_INTERFACE_HA_VALIDATION = 'required|not_matches[backup_interface]|different_than[__selectone__]';

	/**
     * @brief Constructor
     *
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return
     */
    public function __construct($node, $parent_name)
    {
        $this->_the_node = $node;
        parent::__construct($parent_name, "configuration");
    }

    /**
     * @brief Validate
     *
     * @return
     */
    public function validate($data, &$output)
    {
        $mode = $this->get_data_value('mode', false);

        $cluster_module = $this->_the_node->find_module_by_name('cluster');
        $cluster_module->remap_ip_objects();

        $result = parent::validate($data, $output);

        if ($mode == 'disabled')
            return $result;
        else{
            $network_module = $this->_the_node->find_module_by_name('network');
            if(!$network_module->check_hostname($out)){
                $this->set_error('mode',$out['message'].". Change it <a href=\"/SAFe/sng_network_config/edit/network/\">here</a>",$output);
                return false;
            }

            $field_data = array(
                'dedicated_interface' => $this->get_data_value('dedicated_interface', false),
                'backup_interface' => $this->get_data_value('backup_interface', false)
            );
            $validate_result = $this->validate_value($field_data, Sng_cluster_config_class::DEDICATED_INTERFACE_HA_VALIDATION, $output);
            if(!$validate_result){
                $this->set_error('dedicated_interface', 'Please select valid IP address - no transcoding interfaces allowed.', $output);
                $result = false;
            }
        }

        $dedicated_devnet = $this->get_data_value('dedicated_interface', false);
        $backup_devnet = $this->get_data_value('backup_interface', false);

        $dedicated_iface = explode("/", $dedicated_devnet, 2)[0];
        $backup_iface = explode("/", $backup_devnet, 2)[0];

        if ($result && $dedicated_iface === $backup_iface) {
            $this->set_error('backup_interface', 'Please select a different interface from the dedicated interface.', $output);
            $result = false;
        }

        $current_name = $this->get_data_value('current_node_name', false);

        $peers = $this->_the_node->find_module_by_name('cluster')->peers();

        if (in_array($current_name, array_keys($peers))) {
            $this->set_error('current_node_name', 'Current node name conflicts with another peer node name.', $output);
            $result = false;
        }

        return $result;
    }

    /**
     * @brief Configure the config class object
     *
     * @return
     */
    public function configure()
    {
        $backup_interfaces = array();
        $dedicated_interfaces = array();

        $network_module = $this->_the_node->find_module_by_name('network');

        if ($network_module != NULL) {
// TODO: revisit after BETA
//            $dedicated_blacklist = array('eth0', 'eth1', 'lo');
            $dedicated_blacklist = array('lo');
            $backup_blacklist = array('lo');

            $ips = $network_module->interfaces_ip(Sng_network_interface_class::MANAGEMENT);
            $ip_objs = $network_module->get_aggregate_objects('ip');

            foreach ($ips as $ip_name => $ip_data) {
                $obj_data = $ip_objs[$ip_name]->get_data_values(false);

                if (substr($obj_data['proto'], -1) !== '4') {
                    continue;
                }

                if (!in_array($obj_data['interface'], $dedicated_blacklist)) {
                    safe_inner_array_push($dedicated_interfaces, $obj_data['interface'],
                        array($obj_data['address'], $obj_data['prefix']));
                }

                if (!in_array($obj_data['interface'], $backup_blacklist)) {
                    safe_inner_array_push($backup_interfaces, $obj_data['interface'],
                        array($obj_data['address'], $obj_data['prefix']));
                }
            }
        }

        $dedicated_interface_enum = array();
        $backup_interface_enum = array();

        foreach ($dedicated_interfaces as $if_name => $ip_data_list) {
            foreach ($ip_data_list as $ip_data) {
                $net_address = safe_cidr2network($ip_data[0], $ip_data[1]);
                $dedicated_interface_enum[$if_name . '/' . $net_address] = $if_name . ' (' . $net_address . '/' . $ip_data[1] . ')';
            }
        }

        foreach ($backup_interfaces as $if_name => $ip_data_list) {
            foreach ($ip_data_list as $ip_data) {
                $net_address = safe_cidr2network($ip_data[0], $ip_data[1]);
                $backup_interface_enum[$if_name . '/' . $net_address] = $if_name . ' (' . $net_address . '/' . $ip_data[1] . ')';
            }
        }

        $dedicated_interface_enum = safe_create_enum_list($dedicated_interface_enum, ADD_NONE_ALWAYS, false, '(None)');
        $backup_interface_enum = safe_create_enum_list($backup_interface_enum, ADD_NONE_ALWAYS, false, '(None)');

        $this->add_enum_field('dedicated_interface', 'Dedicated Network / Interface', 'dropdown', '__selectone__', $dedicated_interface_enum);
        $this->set_field_category('dedicated_interface', 'Network Interfaces');
// TODO: revisit after BETA
//        $this->set_field_help('dedicated_interface', 'Network interface used on dedicated network link between servers - cannot be an address from signaling interfaces ("eth0" or "eth1").');
        $this->set_field_help('dedicated_interface', array(
               'Network interface used on dedicated network link between servers.',
               '',
               'Use private IP address range for dedicated network in case of direct cable connection',
               '10.0.0.0 to 10.255.255.255',
               '172.16.0.0 to 172.31.255.255',
               '192.168.0.0 to 192.168.255.255',
               ''
               ));
        $this->set_ha_field_rules('dedicated_interface', Sng_cluster_config_class::DEDICATED_INTERFACE_HA_VALIDATION);

        $this->add_enum_field('backup_interface', 'Backup Network / Interface', 'dropdown', '__selectone__', $backup_interface_enum);
        $this->set_field_category('backup_interface', 'Network Interfaces');
        $this->set_field_help('backup_interface', 'Network interface used on backup link, in case the dedicated link is broken.');

        $mode = array(
            'disabled' => 'Disabled',
            'active-passive' => 'Active/passive',
            // 'loadbalance' => 'Load Balancing',
        );
        // General settings
        $this->add_enum_field('mode', 'Operational Mode', 'dropdown', 'disabled', $mode);
        $this->set_field_help('mode',
            array(
                'Configures the mode of operation for high availability',
                array('Disabled' => array('Disable high availability feature.',),),
                array('Active/passive' => array('Enable active-passive mode',),),
//              array('Load Balancing' => array(''))
                )
            );

        $this->set_field_category('mode', 'General settings');

        $log_level_enum = array('debug' => 'Debug', 'info' => 'Info');
        $this->add_enum_field('loglevel', 'Log level', 'dropdown', 'debug', $log_level_enum);
//        $this->set_field_category('loglevel', 'Advanced Cluster Options');
        $this->set_field_help('loglevel', 'Minimum level for log messages.');

        $this->add_field('current_node_name', 'Name for current node on peers', 'string', 'Primary_Node', '15');
        $this->set_field_help('current_node_name', 'Name used to identify this node on peer node configuration.');

        // Advanced cluster options
        $this->add_field('migrate_threshold', 'Number of failures for triggering migration', 'string', '3', 2);
        $this->set_field_category('migrate_threshold', 'Advanced Cluster Options');
        $this->set_field_help('migrate_threshold', 'Number of attempts.');

        $this->add_field('failed_timeout', 'Resource ban timeout after migration due to failures','string', '300', 3);
        $this->set_field_category('failed_timeout', 'Advanced Cluster Options');
        $this->set_field_help('failed_timeout', 'Time in seconds to kee resources away after migration due to failed resource startup.');

        $stickiness_options = array('sticky' => 'Fully sticky', 'score' => 'Stickiness score');
        $this->add_enum_field('resource_stickiness', 'Resources stickiness after migration', 'dropdown', 'score', $stickiness_options);
        $this->set_field_category('resource_stickiness', 'Advanced Cluster Options');
        $this->set_field_help('resource_stickiness',
            '"Stickiness score": migrate resources if constraint score is higher that stickiness, use "0" to migrate resources back after failure recover of original master node.<br>' .
            '"Fully sticky": cluster will not migrate resources unless resource fails or node leaves cluster.');

        $this->add_field('resource_stickiness_score', 'score', 'string', '1', '4');
        $this->composite_layout('resource_stickiness', array('resource_stickiness_score'));

        $control_group_fields = array('resource_stickiness_score');
        $condition_data = array();

        foreach ($control_group_fields as $group_key) {
            $condition_data['sticky'][] = array('field_name' => $group_key, 'status' => 'disable');
            $condition_data['score'][]  = array('field_name' => $group_key, 'status' => 'enable');
        }

        $this->create_group('automatic', $control_group_fields,
            array('default' => array('value' => 'score', 'status' => 'enable'), 'condition' => $condition_data)
        );

        $this->conditional_control('resource_stickiness', 'automatic');

        // Advanced network options
        $octet3 = (time() % 253) + 1;
        $octet4 = rand(1,253);

        $lastoctets = $octet3 . '.' . $octet4;

        $this->add_field('primary_bus_addr', 'Primary Bus', 'string', '226.24.' . $lastoctets, '15');
        $this->add_field('primary_bus_port', 'port', 'string', '5405', '5');
        $this->composite_layout('primary_bus_addr', array('primary_bus_port'));
        $this->set_field_category('primary_bus_addr', 'Advanced Network Options');
        $this->set_field_help('primary_bus_addr', '');

        $this->add_field('secondary_bus_addr', 'Secondary Bus', 'string', '226.26.' . $lastoctets, '15');
        $this->add_field('secondary_bus_port', 'port', 'string', '5405', '5');
        $this->composite_layout('secondary_bus_addr', array('secondary_bus_port'));
        $this->set_field_category('secondary_bus_addr', 'Advanced Network Options');
        $this->set_field_help('secondary_bus_addr', '');

        $this->add_field('db_cluster_port', 'Database Cluster Port', 'string', '3307', '5');
        $this->set_field_category('db_cluster_port', 'Advanced Network Options');
        $this->set_field_help('db_cluster_port', '');


        return parent::configure();
    }

    public function prepare_edit()
    {
        parent::prepare_edit();
        $the_app = $this->_the_node->software()->application();
        if (Safe_service_class::STATUS_RUNNING == $the_app->status() &&
            Safe_object_serializer_class::OBJ_STATUS_NEW != $this->status())
        {
            $ro_fields = array('mode');
            foreach($ro_fields as $field)
            {
                $this->set_field_disabled($field, true);
                $note = array('<font color="red">Parameter cannot be changed - NSC is running</font>', '');
                $help = $this->get_field_help($field);
                $help = array_merge($note, is_array($help) ? $help : array($help));
                $this->set_field_help($field, $help);

            }
        }
    }
}

class Sng_cluster_module_class extends Safe_module_class
{
    const CLUSTER_PENDING_CONFIG = '/var/sng/cluster/pending-synchronization.json';
    const CLUSTERMON_CONFIG_FILE = '/usr/local/sng/conf/clustermon.conf';
    const CLUSTERMON_WARNING_FILE  = '/tmp/clustermon.warning';  

    private $_base_path = '';
    private $_sng_config;
    private $_the_app;
    public function __construct($app)
    {
        parent::__construct($app->node()->software(), "cluster");
        $this->_obj_base_path = $this->path() .'/cluster';
        $this->_base_path = $this->node()->configuration_manager()->directory('sng-base');
        $this->_sng_config = parse_ini_file(__DIR__."/../../application/config/config.ini");
        $this->_the_app = $app;
    }

     public function is_crud_allowed(){
        $clustermon = $this->_the_app->find_module_by_name("clustermon");
        $str_status = $clustermon->node_str_status();
        return !$clustermon || $this->get_cluster_mode() == "disabled" || $str_status[0]!="SLAVE" || $this->_sng_config["edit_on_slave"];
    }

    public function pending_config_filename()
    {
        return Sng_cluster_module_class::CLUSTER_PENDING_CONFIG;
    }

    public function pending_synchronize()
    {
        return (file_exists(Sng_cluster_module_class::CLUSTERMON_CONFIG_FILE) && file_exists(Sng_cluster_module_class::CLUSTER_PENDING_CONFIG));
    }

    public function can_copy_configuration(){
        $clustermon_service = $this->node()->software()->application()->find_module_by_name('clustermon');
        if(!$clustermon_service->is_local_node_online() && $clustermon_service->is_peer_node_online()){
                return false;
        }
        return true;
    }

    /**
     * @brief
     *
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("High Availability");

        $this->register_aggregate_object('peer',
            array(
                'name' => 'Peer nodes',
                'has_child' => true,
                'base_path' => $this->_obj_base_path . '/peer',
                'controller' => array(
                    'config_manager' => array(
                        'summary_buttons' => 'Modify|Delete:confirm',
                        'allow_create_button' => 'backup[Add]'
                    ),
                ),
                'controller_url' => '/SAFe/sng_cluster_config',
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a peer node',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a peer node',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a peer node',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a peer node',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('floating_ip',
            array(
                'name' => 'Floating IPs',
                'base_path' => $this->object_name() . '/floating_ip',
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a floating IP address',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a floating IP address',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a floating IP address',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a floating IP address',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        $this->register_aggregate_object('failover_trigger',
            array(
                'name' => 'Failover triggers',
                'base_path' => $this->object_name() . '/failover_trigger',
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a failover trigger',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a failover trigger',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a failover trigger',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a failover trigger',
                        'request' => 'POST',
                    ),
                ),
            )
        );
        return parent::configure();
    }
    public function post_configure()
    {
        // Create the config object
        $cfg = new Sng_cluster_config_class($this->node(), $this->object_name());
        // Synch with DB
        $cfg->configure();
        $cfg->synch();
        // Attach config to module
        $this->set_config($cfg, '/SAFe/sng_cluster_config');
    }

    public function get_action_settings($action=null)
    {
        // $smtp_test_param =  new Safe_configurable_object_class('autofill', 'smtp_test_param');
        // $smtp_test_param->add_field('email', 'Email Address', 'text', '','50');
        // $smtp_test_param->set_field_rules('email','required|valid_email');
        $action_def = array(
            'reset_uuid' => array(
                'name' => 'Reset UUID and Access Key',
                'icon' => 'key',
                'description' => 'Generate new UUID and access key for local node',
                'request' => 'POST',
                'confirm' => true,
                'reload' => true
            ),
            'local_key' => array(
                'name' => 'Generate Access Key',
                'icon' => 'key',
                'description' => 'Show access key for the local node',
                'request' => 'POST',
                'confirm' => true,
                'reload' => true
            ),
            'synchronize' => array(
                'name' => 'Copy configuration to ' . $this->peer_node_type(),
                'icon' => 'save',
                'description' => 'Copy configuration to ' . $this->peer_node_type(),
                'request' => 'POST',
                'confirm' => true,
                'reload' => true,
                'redirect_url' => ''
            ),
        );

        // TODO: set button as hidden if already synchronized?
        if ($this->pending_synchronize()) {
            $action_def['synchronize']['button_class'] = 'danger';
        }
        // in slave mode, hide "synchronize" button
        $clustermon_service = $this->node()->software()->application()->find_module_by_name('clustermon');
        $res = $clustermon_service->check_master();
        $configuration = $this->node()->software()->application()->api_status_configuration();
        if($res === false || $configuration['modified'] || isset($configuration['checklist']) || !$this->can_copy_configuration()){
            $action_def['synchronize']['display'] = false;
        }

        $peers = $this->peers();
        if ($this->ha_mode_active() && count($peers) != 0) {
            $action_def['reset_uuid']['display'] = false;
            $action_def['local_key']['display'] = false;
        }

        if($action) {
            return $action_def[$action];
        }
        return $action_def;
    }

    public function api_reset_uuid_action($data=null, &$output = null)
    {
        $clustermon_service = $this->node()->software()->application()->find_module_by_name('clustermon');

        if ($clustermon_service !== NULL) {
            if ($clustermon_service->status() == Safe_service_class::STATUS_RUNNING) {
                $output['message'] = 'Cannot reset UUID nor access keys while <strong>Cluster Management</strong> is running!';
                return false;
            }
        }

        list($rc, $outdata, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-uuid-generate --force", '', '', false, 45);

        if ($rc != 0) {
            $errormsgs = '<br><div>' . implode(", ", preg_split("/\r\n|\n|\r/", $stderr)) . '</div>';
            if (strlen($stderr) != 0) {
                $output['reason'] = $errormsgs;
            }
            $output['message'] = 'Failed to generate new UUID!';
            return false;
        }

        $tmpconfigfile = tempnam("/tmp", "cluster-config.");

        list($retcode, $errmessage) = $this->output_configuration_raw($tmpconfigfile);

        if ($retcode != true) {
            $output['message'] = 'Failed to create temporary file "' . $tmpconfigfile . '"!';
            $output['reason'] = '<br>' . $errmessage;
            unlink($tmpconfigfile);
            return false;
        }

        list($rc, $outdata, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-ckey-create --force-new --load '" . $tmpconfigfile . "'", '', '', false, 45);

        if ($rc != 0) {
            $errormsgs = '<br><div>' . implode(", ", preg_split("/\r\n|\n|\r/", $stderr)) . '</div>';
            if (strlen($stderr) != 0) {
                $output['reason'] = $errormsgs;
            }
            $output['message'] = 'Failed to generate access key!';
            unlink($tmpconfigfile);
            return false;
        }

        list($rc, $stdout, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-cluster-configure --force --load '" . $tmpconfigfile . "' --loglevel " . $this->get_loglevel(), '', '', false, 45);

        if ($rc != 0) {
            $errormsgs = '<br><div>' . implode(", ", preg_split("/\r\n|\n|\r/", $stderr)) . '</div>';
            if (strlen($stderr) != 0) {
                $output['reason'] = $errormsgs;
            }
            $output['message'] = 'Failed to re-generate configuration!';
            unlink($tmpconfigfile);
            return false;
        }

        unlink($tmpconfigfile);

        if (safe_array_get($data['raw'], 'false') !== 'true') {
            $splitdata = trim(chunk_split($outdata, 86));
            $rownumber = substr_count($splitdata, "\n") + 1;
            $output['message'] = '<div>Please copy this access key and paste it on the other node, in the "Add peer nodes" screen:</div><br>'.
                '<textarea readonly="1" cols="82" rows="'.$rownumber.'" style="font-family: monospace; font-size: 75%">'.$splitdata.'</textarea>';
        } else {
            $output['message'] = $outdata;
        }

        return true;
    }

    public function api_local_key_action($data=null, &$output = null)
    {
        $tmpconfigfile = tempnam("/tmp", "cluster-config.");

        list($retcode, $errmessage) = $this->output_configuration_raw($tmpconfigfile);

        if ($retcode != true) {
            $output['message'] = 'Failed to create temporary file "' . $tmpconfigfile . '"!';
            $output['reason'] = '<br>' . $errmessage;
            unlink($tmpconfigfile);
            return false;
        }

        list($rc, $outdata, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-ckey-create --load '" . $tmpconfigfile . "'", '', '', false, 45);

        if ($rc != 0) {
            $errormsgs = '<br><div>' . implode(", ", preg_split("/\r\n|\n|\r/", $stderr)) . '</div>';
            if (strlen($stderr) != 0) {
                $output['reason'] = $errormsgs;
            }
            $output['message'] = 'Failed to generate access key!';
            unlink($tmpconfigfile);
            return false;
        }

        if (safe_array_get($data['raw'], 'false') !== 'true') {
            $splitdata = trim(chunk_split($outdata, 86));
            $rownumber = substr_count($splitdata, "\n") + 1;
            $output['message'] = '<div>Please copy this access key and paste it on the other node, in the "Add peer nodes" screen:</div><br>'.
                '<textarea readonly="1" cols="82" rows="'.$rownumber.'" style="font-family: monospace; font-size: 75%">'.$splitdata.'</textarea>';
        } else {
            $output['message'] = $outdata;
        }

        unlink($tmpconfigfile);
        return true;
    }

    public function api_synchronize_action($data=null, &$output = null)
    {
        $filename = $this->pending_config_filename();
        $temporary = false;

        if (!$this->pending_synchronize()) {
            $filename = tempnam('/tmp', 'synchronize.');
            $temporary = true;

            list($ret, $msg) = $this->output_configuration_raw($filename);

            if ($ret === false) {
                $output['message'] = $msg;
                return false;
            }
        }

        list($rc, $stdout, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-cluster-synchronize --load '" . $filename . "'", '', '', false, 90);

        if ($rc != 0) {
            $output['message'] = 'Unable to copy configuration to ' . $this->peer_node_type() . ' (rc=' . $rc . ').';
            $errormsgs = '<br><div>' . implode("<br> ", preg_split("/\r\n|\n|\r/", $stderr)) . '</div>';
            if (strlen($stderr) != 0) {
                $output['message'] .= $errormsgs;
            }
            if ($temporary) {
                unlink($filename);
            }
            return false;
        }

        unlink($filename);

        $output['message'] = 'Copy configuration to ' . $this->peer_node_type() . ' completed.';
        $output['message'] .= "<p style='color: red;'>Warning: Please create a configuration backup.</p>";
        $output['message'] .= "<a href=" . base_url('sng_archive_manager') . ">Click here to be redirected to create a backup</a>";
        return true;
    }

    /**
     * Floating ips
     */
    public function floating_ips($name = null)
    {
        $this->_floating_ips = $this->get_aggregate_objects('floating_ip');
        return $name == null ? $this->_floating_ips : $this->_floating_ips[$name];
    }
    public function api_retrieve_floating_ip($name, $data=null, &$output = null)
    {
        $ips = $this->floating_ips();
        return $ips[$name];
    }
    public function api_create_floating_ip($name, $data=null, &$output = null)
    {
        $peer = new Sng_cluster_ip_class($this->node(), $this->object_name() . '/floating_ip', $name);
        $peer->configure();
        if ($data) {
            $peer->set_data_values($data);
        }
        return $peer;
    }
    public function api_delete_floating_ip($name, $data=null, &$output = null)
    {
        $ip = $this->api_retrieve_floating_ip($name);
        if($ip) {
            if($ip->can_dispose($output)) {
                return $ip->dispose();
            }
        }
        return false;
    }
    public function api_update_floating_ip($name, $data=null, &$output = null) {
        $node = $this->api_retrieve_floating_ip($name);
        if($node) {
            if ($node->validate($data,$output)) {
                if (true == $node->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * Peers
     */
    public function api_retrieve_peer($name, $data=null, &$output = null)
    {
        $peers = $this->peers();
        return $peers[$name];
    }
    public function api_create_peer($name, $data=null, &$output = null)
    {
        $peer = new Sng_cluster_access_key_config_class($this->node(), $this->object_name() . '/peer', $name);
        $peer->configure();
        if ($data) {
            $peer->set_data_values($data);
        }
        return $peer;
    }

    public function api_delete_peer($name, $data=null, &$output = null)
    {
        $key = $this->api_retrieve_peer($name);

        // delete Peer Node's IP addresses from the ssh firewall rule whitelist
        $firewall_module = $this->node()->software()->application()->find_module_by_name('firewall');
        $port_objs = $firewall_module->get_aggregate_objects('port');
        if(isset($port_objs)){
            $sshd_service = $this->node()->software()->application()->find_module_by_name('sshd');
            foreach($port_objs as $port_obj){
                $ssh_conf = $sshd_service->config()->get_data_values();
                if($port_obj->get_data_value('begin') == $ssh_conf['port']){
                    $whitelist_obj = $port_obj;
                }
            }

            if(isset($whitelist_obj)){
                $whitelist = $whitelist_obj->api_retrieve_whitelist();
                if(isset($whitelist) && count($whitelist) > 0){
                    $peer_address_list = array();

                    foreach($key->api_retrieve_network_ip() as $peer_ip){
                        if(preg_match('/^eth/', $peer_ip->get_data_value('interface')) && $peer_ip->ip_type()==Safe_ip_address_class::IP_V4){
                            array_push($peer_address_list, $peer_ip->get_data_value('address') . '/32');
                        }
                    }
                    foreach($whitelist as $whitelist_ip){
                        if(in_array($whitelist_ip->get_address(),$peer_address_list)){
                            $whitelist_obj->api_delete_whitelist($whitelist_ip->name());
                            $whitelist_obj->save();
                        }
                    }
                }
            }
        }

        if($key) {
            if($key->can_dispose($output)) {
                return $key->dispose();
            }
        }
        return false;
    }
    /**
     * Failover triggers
     */
    public function failover_triggers()
    {
        $this->_failover_triggers = $this->get_aggregate_objects('failover_trigger');
        return $this->_failover_triggers;
    }
    public function api_retrieve_failover_trigger($name, $data=null, &$output = null)
    {
        $ips = $this->failover_triggers();
        return $ips[$name];
    }
    public function api_create_failover_trigger($name, $data=null, &$output = null)
    {
        $peer = new Sng_cluster_trigger_class($this->node(), $this->object_name() . '/failover_trigger', $name);
        $peer->configure();
        if ($data) {
            $peer->set_data_values($data);
        }
        return $peer;
    }
    public function api_delete_failover_trigger($name, $data=null, &$output = null)
    {
        $ip = $this->api_retrieve_failover_trigger($name);
        if($ip) {
            if($ip->can_dispose($output)) {
                return $ip->dispose();
            }
        }
        return false;
    }
    public function api_update_failover_trigger($name, $data=null, &$output = null) {
        $node = $this->api_retrieve_failover_trigger($name);
        if($node) {
            if ($node->validate($data,$output)) {
                if (true == $node->save()) {
                    return true;
                }
            }
        }
        return false;
    }

    public function is_configuration_modified($include_childs = true)
    {
        return parent::is_configuration_modified($include_childs);
    }
    public function generate_config(&$config_manager)
    {
        if (!$this->_generate_config($config_manager)) return false;
        return parent::generate_config($config_manager);
    }
    private function output_configuration_raw($filename)
    {
        $handle = fopen($filename, "w");

        if ($handle === FALSE) {
            return array(false, 'Unable to open temporary file for configuration. Please check available disk space.');
        }

        $system_this = $this->node()->software()->application()->find_module_by_name('system');
        $system_data = $system_this->api_retrieve_configuration();

        $jsondata = json_encode($system_data);

        $wrote_len = fwrite($handle, $jsondata);
        fclose($handle);

        if ($wrote_len !== strlen($jsondata)) {
            return array(false, 'Unable to write configuration to temporary file. Please check available disk space.');
        }

        return array(true, NULL);
    }

    private function output_configuration(&$config_manager, $filename)
    {
        list ($ret, $retdata) = $this->output_configuration_raw($filename);

        if ($ret === false) {
            if ($config_manager !== NULL) {
                $error = array(
                    'module' => 'cluster',
                    'obj_type' => 'configuration',
                    'description' => $retdata,
                    'url' => base_url('/sng_cluster_config/'),
                );
                $config_manager->add_error($error);
            }
        }

        return $ret;
    }

    public function store_configuration(&$config_manager)
    {
        $filename = $this->pending_config_filename();
        return array($this->output_configuration($config_manager, $filename), $filename);
    }

    public function clear_configuration(&$config_manager)
    {
        list($rc, $stdout, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-cluster-configure --clear --loglevel " . $this->get_loglevel(), '', '', false, 25);

        if ($rc != 0) {
            $emsgs = implode("<br>", preg_split("/\r\n|\n|\r/", $stderr));
            $error = array(
                'module' => 'cluster', 'obj_type' => 'configuration', 'description' => ('Configuration failed:<br>' . $emsgs), 'url' => base_url('/sng_cluster_config/')
            );
            $config_manager->add_error($error);
        }

        return true;
    }

    private function _generate_config(&$config_manager)
    {
        if ($this->node()->get_config_status('reset') !== false) {
            return $this->clear_configuration($config_manager);
        }

        if ($this->get_cluster_mode() == 'disabled') {
            return $this->clear_configuration($config_manager);
        }

        list($status, $retdata) = $this->store_configuration($config_manager);

        if ($status === false) {
            return true;
        }

        list($rc, $stdout, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-cluster-configure --load '" . $retdata . "' --loglevel " . $this->get_loglevel(), '', '', false, 45);

        if ($rc != 0) {
            $emsgs = implode("<br>", preg_split("/\r\n|\n|\r/", $stderr));
            $error = array(
                'module' => 'cluster', 'obj_type' => 'configuration', 'description' => ('Configuration failed:<br>' . $emsgs), 'url' => base_url('/sng_cluster_config/')
            );
            $config_manager->add_error($error);
        }

        return true;
    }

    /**
     * Handles serialization of access keys
     * @param object  $config_manager
     * @param object  $access_keys
     * @return boolean
     */
    protected function _serialize_key(&$config_manager, $access_keys)
    {
        // Remove directories
        // $config_manager->add_config(new Safe_configuration_class(Fs_certificate_module_class::CA_STORE, '', Safe_configuration_class::CFG_DELETE, Safe_configuration_class::CFG_DIR));
        // $config_manager->add_config(new Safe_configuration_class(Fs_certificate_module_class::CA_STORE, '', Safe_configuration_class::CFG_CREATE, Safe_configuration_class::CFG_DIR));
        // foreach ($ca_certificates as $cert) {
        //     $cert_file = $cert->get_data_value('certificate-file-name', false);
        //     //Place certificate in CA_store
        //     $file = Fs_certificate_module_class::CA_STORE . $cert_file;
        //     $cert_content = $cert->get_data_value('certificate-file-contents', false);
        //     $config_manager->add_config(
        //         new Safe_configuration_class(
        //             $file,
        //             base64_decode($cert_content) ,
        //             Safe_configuration_class::CFG_UPDATE,
        //             Safe_configuration_class::CFG_FILE
        //         )
        //     );
        // }
    }
    /**
     * Returns all the cluster nodes
     */
    public function peers()
    {
        $this->_peer = $this->get_aggregate_objects('peer');
        return $this->_peer;
    }

    public function check_peer_nodes_config(&$data){
        $ret = true;
        if(($peers=$this->peers())){
            foreach($peers as $peer){
                if(($peer_network=$peer->api_retrieve_network_configuration())){
                    if(!preg_match(Sng_network_service_class::DOMAIN_REGEX,$peer_network->get_data_value('global/hostname'))){
                        $data['message'] = 'The peer node <b>'.$peer->name().'</b> has no valid domain definition';
                        $data['redirect_url'] = '/SAFe/sng_cluster_config/edit/cluster/peer/'.$peer->name().'/network_configuration/'.$peer_network->name();
                        $ret = false;
                    }
                }
                else{
                    $data['message'] = 'There is an error in peer nodes configuration';
                    $data['redirect_url'] = '/SAFe/sng_cluster_config/';
                    $ret = false;
                }
            }
        }
        else{
            $data['message'] = 'There is no peer node in the configuration';
            $data['redirect_url'] = '/SAFe/sng_cluster_config/';
            $ret = false;
        }
        return $ret;
    }

    public function check_ha_config(&$data)
    {
        $ret = true;
        $ha_checklist = array();
        $this->node()->ha_checklist($ha_checklist);
        if($ha_checklist){
            foreach($ha_checklist as $item){
                $data['message'] .= ucfirst($item['module']) . ' ' . ucfirst($item['obj_type']) . ' Error<br/>';
                $data['message'] .= $item['description'] . '<a href="' . $item['url'] . '"> ' . $item['label'] . '</a><br/>';
            }
            $data['redirect_url'] = '/SAFe/sng_cluster_config/';
            $ret = false;
        }
        return $ret;
    }

    public function ha_checklist(&$list)
    {
        if(count($this->peers())==0){
            $list[] = array(
                'module' => $this->name(),
                'obj_type' => 'configuration',
                'description' => 'Peer node is missing.'
            );
        }

        $dedicated_interface = $this->config()->get_data_value('dedicated_interface', true);
        if(!isset($dedicated_interface)){
            $list[] = array(
                'module' => $this->name(),
                'obj_type' => 'configuration',
                'description' => 'Dedicated Network/Interface is missing a network interface',
                'url' => '/SAFe/sng_network_config/edit/cluster/',
                'label' => 'Edit'
            );
        }

        $backup_interface = $this->config()->get_data_value('backup_interface', true);
        if(!isset($backup_interface)){
            $list[] = array(
                'module' => $this->name(),
                'obj_type' => 'configuration',
                'description' => 'Backup Network/Interface is missing a network interface',
                'url' => '/SAFe/sng_network_config/edit/cluster/',
                'label' => 'Edit'
            );
        }

        if(($peers=$this->peers())){
            foreach($peers as $peer){
                $ip_objs = $peer->get_aggregate_objects('network_ip');
                foreach($ip_objs as $ip_obj_name => $ip_obj){
                    if($ip_obj->is_dhcp()){
                        $field_data = array(
                                'hostname' => $ip_obj->get_data_value('hostname', false),
                                );
                        $validate_result = $this->validate_value($field_data, "required|valid_domain", $output);
                        if(!$validate_result){
                            $list[] = array(
                                'module' => $this->name(),
                                'obj_type' => 'configuration',
                                'description' => 'The peer node <b>'.$peer->name().'</b> has no valid hostname in dynamic ip config <b>' . $ip_obj_name . '</b>.
                                <br>  Please correct the peer node configuration and re-add.',
                                'url' => '/SAFe/sng_cluster_config/edit/cluster/peer/'.$peer->name().'/network_ip/'.$ip_obj_name,
                                'label' => 'Edit'
                            );
                        }
                    }else{
                        $field_data = array(
                                'address' => $ip_obj->get_data_value('address', false),
                                );
                        $validate_result = $this->validate_value($field_data, "required|valid_ip", $output);
                        if(!$validate_result){
                            $list[] = array(
                                'module' => $this->name(),
                                'obj_type' => 'configuration',
                                'description' => 'The peer node <b>'.$peer->name().'</b> has no valid IP address in static ip config <b>' . $ip_obj_name . '</b>.
                                <br>  Please correct the peer node configuration and re-add.',
                                'url' => '/SAFe/sng_cluster_config/edit/cluster/peer/'.$peer->name().'/network_ip/'.$ip_obj_name,
                                'label' => 'Edit'
                            );
                        }
                    }
                }
            }
        }
    }

    public function register_ckey($keydata) {
        list($rc, $stdout, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-ckey-register --string '" . $keydata . "'", '', '', false);

        if ($rc != 0) {
            $errormsgs = '<div>' . implode(", ", preg_split("/\r\n|\n|\r/", $stderr)) . '</div>';
            if (strlen($stderr) != 0) {
                return array('check_ckey_register' => $errormsgs);
            }
            return array('check_ckey_register' => 'invalid registration key!');
        }
        $result = json_decode($stdout, true);

        // check if UUID is unique
        $peer_nodes = $this->peers();
        foreach ($peer_nodes as $peer) {
            $uuid = $peer->get_data_value('uuid');
            if ($uuid == $result['uuid']) {
                return array('check_ckey_register' => "Duplicated UUID with <b>" . $peer->name() . "</b>");
            }
        }

        // check if peer eth* interfaces match local node
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        if ($network_module != NULL) {
            $ifaces_peer = array();
            $ifaces_mine = array();
            foreach ($result['network']['interface'] as $if_name => $if_data) {
                $interface_name = isset($if_data['vlan_type']) &&
                                $if_data['vlan_type']==Sng_network_interface_class::VLAN ?
                                $if_data['ifname'].'.'.$if_data['id'] : '';
                if (strpos($if_name, 'eth') !== 0 &&
                    strpos($interface_name,'eth')!==0) {
                    continue;
                }else if(strpos($interface_name,'eth')===0)
                    $if_name = $interface_name;
                $ifaces_peer[$if_name] = true;
            }

            foreach ($network_module->get_aggregate_objects('interface') as $if_name => $if_obj) {
                $interface_name = trim($if_obj->interface_name());
                if (strpos($if_name, 'eth') !== 0 &&
                    strpos($interface_name,'eth')!==0) {
                    continue;
                }else if(strpos($interface_name,'eth')===0)
                    $if_name = $interface_name;

                $ifaces_mine[$if_name] = true;
            }

            $missing_peer = array();
            $missing_mine = array();
            foreach ($ifaces_peer as $if_name => $unused) {
                if ($ifaces_mine[$if_name] !== true) {
                    $missing_mine[] = $if_name;
                }
            }
            foreach ($ifaces_mine as $if_name => $unused) {
                if ($ifaces_peer[$if_name] !== true) {
                    $missing_peer[] = $if_name;
                }
            }
            if (count($missing_peer) !== 0 || count($missing_mine) !== 0) {
                $errormsg = array();
                if (count($missing_peer) !== 0) {
                    sort($missing_peer);
                    $errormsg[] = 'peer node missing "' . implode('", "', $missing_peer) . '"';
                }
                if (count($missing_mine) !== 0) {
                    sort($missing_mine);
                    $errormsg[] = 'local node does not have "' . implode('", "', $missing_mine) . '"';
                }
                return array('check_ckey_register' => 'Network devices mismatch: ' . implode(', ', $errormsg));
            }
        }

        //check if local and peer HA nodes versions match
        $application_module = $this->node()->software()->application()->find_module_by_name('application');
        $master_node_data = $application_module->api_retrieve_version()->get_data_values(false);
        $peer_node_data = $result['nsc']['version'];
        $versions = array('major_version', 'minor_version', 'patch_version', 'release_version');
        $master_node_version = array();
        $peer_node_version = array();
        foreach($versions as $version){
            array_push($master_node_version, $master_node_data[$version]);
            array_push($peer_node_version, $peer_node_data[$version]);
        }
        $master_node_version = join('.', $master_node_version);
        $peer_node_version = join('.', $peer_node_version);
        if(!($master_node_version === $peer_node_version)){
            return array('check_ckey_register' => "HA nodes version mismatch: Local version is " . $master_node_version . ", peer version is " . $peer_node_version);
        }

        return $result;
    }
    public function check_ckey_register($field_name, $param, $data) {
        $result = $this->register_ckey($data);
        if (isset($result['check_ckey_register'])) {
            return $result;
        }
        return true;
    }

    public function new_fields(&$config_manager, $type)
    {
        if ($type == 'peer') {
            $peers = count($this->peers()) + 1;
            $config_manager->add_field('profile-name', 'Node Name', 'text', "Peer_Node" . $peers, 50);
            $config_manager->add_textarea_field('access-key', 'Access Key', "textarea", '', 5, 40);
            $config_manager->set_field_rules('access-key', 'required|callback_check_ckey_register');
            $config_manager->add_field('save', 'Save', 'hidden', 'save', 10);
        } else {
            parent::new_fields($config_manager, $type);
        }

    }
    public function get_network_interfaces($peer, $physical_only=false) {
        $peer = $this->node()->software()->application()->find_obj_by_name('cluster', 'peer', $peer);
        if (!$peer) return false;
        $interfaces = $peer->api_retrieve_network_interface();
        if (!$interfaces) return false;
        $interfaces_enum = array();
        foreach($interfaces as $key => $obj){
            if($obj->hardware_type() == Sng_network_interface_class::TRANSCODING && !safe_development_mode()){
                //only in development_mode allow user to add/remove/change the ip on sngdspX interfaces
                continue;
            }elseif($obj->is_bonded()){
                //also pass by bonded interface
                continue;
            }else{
                if ($physical_only && $obj->get_data_value('type') != 'physical' &&
                        $obj->get_data_value('type')!='virtual') continue;
                $interfaces_enum[$obj->name()] = $obj->interface_name();
            }
        }
        return $interfaces_enum;
    }

    public function get_cluster_mode() {
        // This fix the crash when calling
        // network_module->signaling_interface_ip method on
        // Fs_radius_config_class
        $config = $this->config();
        if(!$config){
            $this->post_configure();
            $config = $this->config();
        }
        return $config->get_data_value('mode', false);
    }

    public function get_loglevel() {
        return $this->config()->get_data_value('loglevel', false);
    }

    public function get_floating_interfaces_ip($with_ip = true)
    {
        $ip_objs = $this->floating_ips();
        $if_enum = array();

        foreach($ip_objs as $ip_obj) {
            $if_enum[$ip_obj->name()] = $with_ip ?
                $ip_obj->get_interface_name() . ' - ' . $ip_obj->address() . ' (' . $ip_obj->name() . ')' :
                $ip_obj->get_interface_name();
        }

        return $if_enum;
    }

    public function get_floating_ips($interface)
    {
        $ip_objs = $this->floating_ips();
        $ips = array();

        foreach($ip_objs as $ip_obj) {
            if ($ip_obj->get_interface_name() == $interface) {
                $ips[] = $ip_obj;
            }
        }

        return $ips;
    }
    public function check_ip_conflict($from_obj) {
        $from_obj_name  = $from_obj->name();
        $from_obj_address = $from_obj->address();
        $conflict_list = array();
        $objs = $this->get_aggregate_objects('floating_ip');
        $ip_type = $from_obj->ip_type();
        $ipv6 = false;
        if($ip_type == Safe_ip_address_class::IP_V6){
            $cmp_ip = safe_ipv6_compress($from_obj->address());
            $ipv6 = true;
        }
        if($objs){
            foreach($objs as $name => $obj){
                $to_obj_address = $obj->address();
                //from_obj could be from network ip, the name may same, but the path must different
                if( ($name != $from_obj_name || $from_obj->path() != $obj->path()) && $obj->address()){
                    if($obj->address() == $from_obj_address){
                        $conflict_list[] = $obj;
                    }else{
                        //compare ipv6
                        if($ipv6 && $ip_type == $obj->ip_type()){
                            if( $cmp_ip == safe_ipv6_compress($obj->address()) ){
                                $conflict_list[] = $obj;
                            }
                        }
                    }
                }
            }
        }
        return $conflict_list;
    }

    public function check_object_modified($obj, $src_module_name, $obj_type, $obj_name, $sub_type = null, $sub_name = null, $source_type=null)
    {
        if ($src_module_name == 'cluster') {
            $data = $obj->get_data_values(false);
            $peer_obj = $this->get_aggregate_object('peer', $obj_name);
            $node = array();
            $node = $peer_obj->get_data_values();

            switch ($sub_type) {
                case 'network_configuration':
                    unset($node['network']['configuration']);
                    if ($source_type == 'save') $node['network']['configuration'] = $data;
                break;
                case 'network_interface':
                    unset($node['network']['interface'][$sub_name]);
                    if ($source_type == 'save') $node['network']['interface'][$sub_name] = $data;
                break;
                case 'network_ip':
                    unset($node['network']['ip'][$sub_name]);
                    if ($source_type == 'save') $node['network']['ip'][$sub_name] = $data;
                break;
                case 'network_route':
                    unset($node['network']['route'][$sub_name]);
                    if ($source_type == 'save') $node['network']['route'][$sub_name] = $data;
                break;
                case 'sngms_configuration':
                    unset($node['sngms']['configuration']);
                    if ($source_type == 'save') $node['sngms']['configuration'] = $data;
                break;
                case 'sngms_module':
                    unset($node['sngms']['module'][$sub_name]);
                    if ($source_type == 'save') $node['sngms']['module'][$sub_name] = $data;
                break;
                case 'sngms_device':
                    unset($node['sngms']['device']);
                    if ($source_type == 'save') $node['sngms']['device'] = $data;
                break;
            }
            $peer_obj->set_data_values($node);
            $peer_obj->save();
        }
        return parent::check_object_modified($obj, $src_module_name, $obj_type, $obj_name, $sub_type, $sub_name);
    }

    public function checklist(&$list)
    {
        $mode = $this->get_cluster_mode();
        $peers = $this->peers();

        if ($mode == 'active-passive' && count($peers) != 1) {
           $list[] = array(
                'module' => $this->name(),
                'obj_type' => 'configuration',
                'description' => count($peers) == 0 ? "No peer nodes configured" : "Only one peer allowed in active/passive mode",
           );
        }else if($mode!='disabled'){
            $network_module = $this->node()->software()->application()->find_module_by_name('network');
            $cluster_config = $this->config();
            $dedicated_ip = $cluster_config->get_data_value('dedicated_interface',false);
            $dedicated_ip = ($dedicated_ip = explode('/',$dedicated_ip)) && count($dedicated_ip)>1? $dedicated_ip[1] : null;
            $ips = $network_module->api_retrieve_ip();
            foreach($peers as $peer){
                $same_subnet = false;
                $nslookupissue = false;
                foreach($peer->api_retrieve_network_ip() as $ip){
                    if($ip->ip_type()==4 && safe_cidr_match($ip->address(),$dedicated_ip, $ip->get_data_value('prefix'))){
                        $starttime = time();
                        /* Since ssh access will do reverse dns lookup, this nslookup check on peer node IP should not have delay */
                        list($rc, $stdout, $stderr) = safe_pipe_exec('nslookup ' . $ip->address() , '', '', false, '', 4);
                        $exectime = time() - $starttime ;
                        safe_log(SAFE_LOG_INFO, "nslookup on peer ip " .  $ip->address() . " exec time: " . $exectime);
                        if($exectime > 2){
                            safe_log(SAFE_LOG_ERROR, "ERROR nslookup on peer ip " .  $ip->address() . " take excess time: " . $exectime);
                            $nslookupissue = true;
                        }
                        $same_subnet = true;
                        break;
                    }
                }
            }
            if($nslookupissue == true){
                $list[] = array(
                    'module'=>$this->name(),
                    'obj_type'=>'configuration',
                    'description'=>'Peer node network response time on dedicated network exceeded require limit.<br>
                    Use private IP address range for dedicated network in case of direct cable connection <br>
                    10.0.0.0 to 10.255.255.255 <br> 172.16.0.0 to 172.31.255.255 <br> 192.168.0.0 to 192.168.255.255'
                );
            }

            if(!$same_subnet){
                $list[] = array(
                    'module'=>$this->name(),
                    'obj_type'=>'configuration',
                    'description'=>'Dedicated ip has no ip address in peer node in the same subnet'
                );
            }
        }

        //add Peer Nodes IP addresses to ssh firewall rule whitelist
        if($mode == 'active-passive' && count($peers) != 0){
            $firewall_module = $this->node()->software()->application()->find_module_by_name('firewall');
            $port_objs = $firewall_module->get_aggregate_objects('port');
            if(isset($port_objs)){
                $sshd_service = $this->node()->software()->application()->find_module_by_name('sshd');
                $ssh_conf = $sshd_service->config()->get_data_values();
                foreach($port_objs as $port_obj){
                    if($port_obj->get_data_value('begin') == $ssh_conf['port']){
                        $whitelist_obj = $port_obj;
                    }
                }
                if(isset($whitelist_obj)){
                    $whitelist = $whitelist_obj->api_retrieve_whitelist();
                    if(isset($whitelist) && count($whitelist) > 0){
                        $address_list = array();
                        $peer_address_list = array();

                        foreach($whitelist as $whitelist_ip){
                            $address_list[] = $whitelist_ip->get_address();
                        }
                        foreach($peers as $peer){
                            foreach($peer->api_retrieve_network_ip() as $peer_ip){
                                if(preg_match('/^eth/', $peer_ip->get_data_value('interface')) && $peer_ip->ip_type()==Safe_ip_address_class::IP_V4){
                                    $peer_address_list[] = array(
                                        'address' => $peer_ip->get_data_value('address'),
                                        'prefix' => '32'
                                    );
                                }
                            }
                        }
                        foreach($peer_address_list as $peer_address){
                            $ip_address = $peer_address['address'] . '/' . $peer_address['prefix'];
                            if(!in_array($ip_address, $address_list)){
                                $name = Safe_object_serializer_class::build_unique_name('whitelist');
                                $whitelist_ip = $whitelist_obj->api_create_whitelist($name, $peer_address);
                                $whitelist_ip->save();
                            }
                        }
                    }
                }
            }
        }
    }

    public function handle_global_messages(&$msg_list)
    {
        $this->config()->clear_global_messages_cache($msg_list, 'cluster', 'synchronize');
        $this->config()->clear_global_messages_cache($msg_list, 'cluster', 'configuration');
        $this->config()->clear_global_messages_cache($msg_list, 'cluster', 'validate_version');

        $clustermon_service = $this->node()->software()->application()->find_module_by_name('clustermon');
        $res = $clustermon_service->check_master();
        $cluster_mode = $this->get_cluster_mode();
        $peers = $this->peers();

        $configuration = $this->node()->software()->application()->api_status_configuration();
        if($this->pending_synchronize() && !$configuration['modified'] && !isset($configuration['checklist']) && $cluster_mode=='active-passive' && count($peers)!=0){
            // in slave mode, hide "synchronize" button
            if($res !== false && $this->can_copy_configuration()){
                $msg_list[] = array(
                        'module' => 'cluster',
                        'cache_tag' => 'synchronize',
                        'global_errors' => array(
                                "message" => "Configuration has not been copied to " . $this->peer_node_type(),
                                "buttons" => array(array(
                                        'label' => 'Copy configuration to ' . $this->peer_node_type(),
                                        'onclick' => 'safe_handle_confirm_button(this);',
                                        'url' => '/SAFe/sng_cluster_config/action_confirm/cluster/synchronize'
                                ))
                        )
                );
            }
        }
        if($res===false && $cluster_mode=='active-passive' && count($peers)!=0 && $this->node()->is_configuration_modified()){
            $msg_list[] = array(
                    'module' => 'cluster',
                    'cache_tag' => 'configuration',
                    'global_errors' => array(
                            "message" => "Running in SLAVE mode, cannot apply configuration changes."
                    )
            );
        }

        if($res===true && $cluster_mode=='active-passive' && count($peers)!=0 && $this->node()->is_configuration_modified()){
            foreach($peers as $peer){
                list($rc, $stdout, $stderr) = safe_sudo_pipe_exec("{$this->_base_path}/bin/sng-cluster-tool version --json --peer", '', '', false, 90);

                if($rc==0 && strlen($stdout)){
                    $application_module = $this->node()->software()->application()->find_module_by_name('application');
                    $master_version_data = $application_module->api_retrieve_version()->get_data_values(false);
                    $json_array = json_decode(trim($stdout), true);
                    $peer_version_data = reset($json_array)['data'];

                    if(isset($master_version_data) && isset($peer_version_data)){
                        $master_version = $master_version_data['major_version'] . '.' . $master_version_data['minor_version'] . '.' . $master_version_data['patch_version'] . '.' . $master_version_data['release_version'];
                        $peer_version = $peer_version_data['major_version'] . '.' . $peer_version_data['minor_version'] . '.' . $peer_version_data['patch_version'] . '.' . $peer_version_data['release_version'];
                        if($master_version != $peer_version){
                            $message = 'The configuration changes may cause a problem on ' . $peer->name() . ' due to version mismatch. ' .
                                        $peer->name() . ' should be upgraded or reinstated to ' . $master_version . ' before copying configuration';
                            $msg_list[] = array(
                                'module' => 'cluster',
                                'cache_tag' => 'validate_version',
                                'global_errors' => array('message' => $message)
                            );
                        }
                    }
                }
            }
        }
         if($this->is_configuration_modified()) {
            $dedicated_interface = explode(" ",$this->config()->get_data_value('dedicated_interface',true));
            $module = $this->_the_app->find_module_by_name('cluster');
            $obj_keys = $module->get_aggregate_objects_keys('failover_trigger');
            $show_warnings = false;
            foreach ($obj_keys as $key) {
                $object = $module->get_aggregate_object("failover_trigger", $key);
                if (isset($object)) {
                    $summary_desc = $object->summary('horizontal', false);
                    if(strpos($summary_desc['data']['Audit point'],$dedicated_interface[0])!==false){
                        $show_warnings = true;
                    }
                }
            }

            if($show_warnings){
                $msg_list[] = array(
                        'module' => 'cluster',
                        'cache_tag' => 'dedicated_interface_on_failover_trigger',
                        'global_warnings' => array(
                                "message" => "Dedicated Interface(".$dedicated_interface[0].") found in the
                                failover triggers list. Please use a dedicated interface other than ".$dedicated_interface[0]." if direct cable connection used between two SBCs",
                                "buttons" => array(array( 'label' => 'Check','url' => '/SAFe/sng_cluster_config'))
                         ));
            }
        }
    }
    public function can_install_data($info = null, &$reason = null)
    {
        $cluster_monitor = $this->node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor != NULL) {
            $clustermon_stats = array();
            $clustermon_status = $cluster_monitor->status($clustermon_stats);

            if ($clustermon_status == Safe_service_class::STATUS_RUNNING) {
                $reason = array($cluster_monitor->description() . ' is Running');
                return false;
            }
        }
        return true;
    }

    public function remap_ip_objects()
    {
        $sip_module = $this->node()->software()->application()->find_module_by_name('sip');
        foreach ($sip_module->profiles() as $obj) {
            $obj->remap_ip_objects();
        }
    }

    public function peer_node_type()
    {
        $node_type = 'Peer';
        if($this->ha_mode_active()){
            $clustermon_service = $this->node()->software()->application()->find_module_by_name('clustermon');
            $nodes = $clustermon_service->nodes();
            $nodes_status = $clustermon_service->check_nodes_status();
            $master_node = $clustermon_service->check_master();

            if(count($nodes)!=0 && $nodes_status==true && $master_node===true){
                $node_type = 'Slave';
            }
        }
        return $node_type;
    }

    public function no_peer_on_active_ha()
    {
        $application_module = $this->_the_app->find_module_by_name('application');
        $clustermon_service = $this->_the_app->find_module_by_name('clustermon');

        $configuration = $application_module->node()->software()->application()->api_status_configuration();
        $local_node_status = $clustermon_service->is_local_node_online();
        $cluster_mode = $this->get_cluster_mode();
        $peer_nodes = $this->peers();

        if((isset($configuration['checklist']) && count($peer_nodes)===0) && ($cluster_mode=='active-passive' || $local_node_status==1)){
            return true;
        }
        return false;
    }
}
