<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Cluster Peer Node > Network Configuration
 * @author Reza Samimi
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
class Sng_cluster_network_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $cluster = $this->node()->software()->application()->find_module_by_name('cluster');
        $infs = $cluster->get_network_interfaces($this->path_info()['name']);
        $infs = array_merge(array('' => '(Auto)'), $infs);

        // General settings
        $this->add_field('global/networking', 'networking', 'hidden', 'yes', 20);
        $this->add_field('global/nozeroconf', 'nozeroconf', 'hidden', 'yes', 20);
        $this->add_field('global/hostname', 'Host Name', 'text', '', 40);
        $this->set_field_rules('global/hostname', 'required');
        // Add General category
        $this->set_field_category('global/hostname','General');
        
        $this->add_enum_field('global/gatewaydev', 'Default Gateway Interface', 'dropdown', '', $infs);
        $this->add_field('ipv4/default_gw', 'Default IPV4 Gateway', 'text', '', 40);
        $this->set_field_rules('ipv4/default_gw', 'valid_ip_or_domain[ipv4]');
        
        $this->add_field('ipv6/default_gw', 'Default IPV6 Gateway', 'text', '', 40);
        $this->set_field_rules('ipv6/default_gw', 'valid_ip_or_domain[ipv6]');
        $this->set_field_category('global/gatewaydev','Gateway');

        foreach (array(1,2,3,4) as $num) {
            $this->add_field('dns/' . $num, 'Static DNS #' . $num, 'text', '', 40);
            $this->set_field_rules('dns/' . $num, 'valid_ip_or_domain');
        }
        $this->set_field_category('dns/1','DNS');

        return parent::configure();
    }
    public function summary() {
        $result = parent::summary();
        // replace name with 'Network Configuration'
        if($result && isset($result['data']['Name'])){
            $result['data']['Name'] ='Network Configuration';
        }
        return $result;
    }
    public function save() {
        $result = parent::save();
        $path_info = $this->path_info();

        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);

        if (!$peer_obj) return $result;

        if ($peer_obj->get_data_value('node_registered') == 'true') {
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name'], 'save');
        }

        return $result;
    }
}