<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Cluster Peer Node > Static Routes
 * @author Reza Samimi
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
class Sng_cluster_route_config_class extends Safe_configurable_object_class
{
	public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        // General settings
        $path_info = $this->path_info();
        $cluster = $this->node()->software()->application()->find_module_by_name('cluster');
        $interfaces_enum = $cluster->get_network_interfaces($path_info['name']);

        $this->add_field('address', 'Destination', 'text', '',40);
        $this->set_field_help('address', 'IP address.');
        $this->set_field_rules('address', 'required|valid_ip');

        $this->add_field('prefix', 'Prefix', 'text', '',5);
        $this->set_field_help('prefix', 'The network prefix for the destination IP address.');
        $this->set_field_rules('prefix', 'required|greater_than[1]|less_or_equal[128]');
        $this->composite_layout('address', array('prefix'));

        $this->add_enum_field('interface', 'Network Interface', 'dropdown', '', $interfaces_enum);
        $this->set_field_help('interface', 'The network interface used to reach destination IP address.' );

        $this->add_field('gateway', 'Gateway', 'text', '',40);
        $this->set_field_help('gateway', 'The default gateway, or an IP address that can be used to reach the destination IP address.');
        $this->set_field_rules('gateway', 'valid_ip');

        return parent::configure();
    }
    public function save() {
        $result = parent::save();
        $path_info = $this->path_info();

        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $path_info['name']);
        if (!$peer_obj) return $result;

        if ($peer_obj->get_data_value('node_registered') == 'true') {
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name'], 'save');
        }
        return $result;
    }
    public function dispose() {
        if (parent::dispose()) {
            $path_info = $this->path_info();

            $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
            $cluster_module->check_object_modified($this, $path_info['module'], $path_info['type'], $path_info['name'], $path_info['sub_type'], $path_info['sub_name']);

            return true;
        }
        return false;
    }
    public function validate($data, &$output)
    {
        // Run regular validation first
        $validate = parent::validate($data, $output);
        if (!$validate) {
            return $validate;
        }
        // Now take care of extra rules:
        // 1/ destination must be unique
        $cluster_module = $this->node()->find_module_by_name('cluster');
        $peer_obj = $cluster_module->get_aggregate_object('peer', $this->path_info()['name']);

        if($data['address']){
            $routes = $peer_obj->api_retrieve_network_route();
            if($routes){
                foreach($routes as $route){
                    // Skip ourself
                    if($this->name() == $route->name()){
                        continue;
                    }
                    $route_data = $route->get_data_values(false);
                    // 1/ check unique route
                    if($data['address'] == $route_data['address']){
                        $validate = false;
                        $this->set_error('address', 'Destination already used by route ' . $route->name(), $output);
                        break;
                    }
                }
            }
        }
        // 2/ if a gateway is specified, it must be on same subnet as interface
        if($data['gateway']){
            // Get adapter and associated IP address
            // if update from rest api, may not include 'interface'
            $ips = $peer_obj->get_network_ips($this->get_data_value('interface', false));
            if(filter_var($data['gateway'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)){
                $gateway_ip_type = Safe_ip_address_class::IP_V6;
            }else{
                $gateway_ip_type = Safe_ip_address_class::IP_V4;
            }
            $sub_net = false;
            if($ips){
                foreach($ips as $ip_obj_name => $ip_obj){
                    $ip_address = $ip_obj->address();
                    if($ip_address &&  $gateway_ip_type == $ip_obj->ip_type()){
                        $ip_prefix = $ip_obj->prefix();
                        //ipv4 && ipv6
                        if(safe_cidr_match($data['gateway'], safe_cidr2network($ip_address, $ip_prefix), $ip_prefix)){
                            $sub_net = true;
                            break;
                        }
                    }
                }
            }
            if(!$sub_net){
                $this->set_error('gateway', 'Gateway is not on '.$data['interface'].' subnet.', $output);
                $validate = false;
            }
        }
        return $validate;
    }
    public function description()
    {
        $data = $this->get_data_values();
        return $data['interface'] . ' / ' . $data['prefix'] . ' - ' . $data['gateway'];
    }
    public function summary($type , $long, $min = 4)
    {
        return parent::summary($type, $long, $min);
    }
}
