<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2016  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Leonardo Lang <lang@sangoma.com>
*/
/**
 * Failover triggers for cluster module
**/
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_module_require_class('sng', 'application');
class Sng_cluster_trigger_class extends Safe_configurable_object_class
{
    private $_the_node;
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
        $this->_the_node = $node;
    }
    private function description_sort($a, $b)
    {
        return strcmp($a->description(), $b->description());
    }
    public function configure()
    {
        $audit_points = array();
        $this->_the_node->get_audit_points($audit_points);

        // normalize audit points
        $audit_categories = array();
        foreach ($audit_points as $point_name => $point_obj) {
            $categ_name = $point_obj->category() ? $point_obj->category() : 'NetBorder Session Controller';
            $audit_categories[$categ_name][$point_name] = $point_obj;
        }

        // sort everything
        ksort($audit_categories);
        foreach ($audit_categories as $categ_name => $categ_data) {
            uasort($categ_data, array($this, 'description_sort'));
        }

        // remove HA from trigger list (not applicable)
        unset($audit_categories['High Availability']);

        $persistence_condition = array();

        $audit_points_enum = array('__none__' => '(select one audit point)');
        foreach ($audit_categories as $categ_name => $categ_data) {
            foreach ($categ_data as $point_name => $point_obj) {
                $audit_points_enum[$point_name] = ($point_obj->category() ? $point_obj->category() : 'NetBorder Session Controller')
                    . ' > ' . $point_obj->description();

                $fixed_persistence = ($point_name == 'system/raid' ||
                                      strpos($point_name, 'system/filesystem/') === 0 ||
                                      strpos($point_name, 'network/network/interface') === 0);

                $persistence_condition[$point_name][] = array(
                    'field_name' => 'persistence',
                    'status' => ($fixed_persistence ? 'disable' : 'enable')
                );
            }
        }

        $this->add_enum_field('point-name', 'Audit point', 'dropdown', '__none__', $audit_points_enum);

        $this->add_field('persistence', 'Failure persistence', 'string', '1', '4');
        $this->set_field_rules('persistence', 'greater_than[0]|less_than[99999]|required');

        $this->add_enum_field('action', 'Failover action', 'dropdown', 'migrate-away',
            array('standby' => 'Disable node',
                  'migrate-away' => 'Migrate resources away')
        );

        $this->create_group('persistence', array('persistence'),
                array('default' => array( 'value' => '__none__', 'status' => 'disable'),
                      'condition' => $persistence_condition)
        );

        $this->conditional_control('point-name', 'persistence');

        // TODO: add "migrate-away delay", enabled when migrate-away is selected

        return parent::configure();
    }
}
?>
