<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
if (!defined('BASEPATH')) exit('No direct script access allowed');
// safe_require_class('safe_module_class');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_service_class');

//debug_print_backtrace();
//exit(1);

function safe_array_normalize($values, $defaults, $mapping = array())
{
    foreach ($defaults as $key => $value) {
        if (!isset($values[$key])) {
            $values[$key] = $value;
        }
    }
    foreach ($values as $key => $value) {
        if (!array_key_exists($key, $mapping)) continue;
        $values[$key] = $mapping[$key][$value];
    }

    return $values;
}

class Sng_clustermon_monitor_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->_set_sync_strategy(Safe_serializable_object_class::SYNC_MEM);
        return parent::configure();
    }
}
class Sng_clustermon_services_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->_set_sync_strategy(Safe_serializable_object_class::SYNC_MEM);
        return parent::configure();
    }
}
class Sng_clustermon_database_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->_set_sync_strategy(Safe_serializable_object_class::SYNC_MEM);
        return parent::configure();
    }
}
class Sng_clustermon_resource_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->_set_sync_strategy(Safe_serializable_object_class::SYNC_MEM);
        return parent::configure();
    }
}
class Sng_clustermon_group_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->_set_sync_strategy(Safe_serializable_object_class::SYNC_MEM);
        return parent::configure();
    }
}
class Sng_clustermon_node_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->_set_sync_strategy(Safe_serializable_object_class::SYNC_MEM);
        return parent::configure();
    }
}
class Sng_clustermon_options_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->_set_sync_strategy(Safe_serializable_object_class::SYNC_MEM);
        return parent::configure();
    }
}
class Sng_clustermon_rings_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->_set_sync_strategy(Safe_serializable_object_class::SYNC_MEM);
        return parent::configure();
    }
}

class Sng_clustermon_service_class extends Safe_service_class
{
    const FLOATING_IP_PREFIX = 'FloatingIP_';

    const CLUSTER_SWITCH_NAME = 'Cluster';
    const CLUSTER_GROUP_NAME = 'NSC';

    const CLUSTER_STATUS_TIMEOUT = 15;

    const CLUSTERMON_CONFIG_FILE = '/usr/local/sng/conf/clustermon.conf';

    const CLUSTERMON_FORCE_START_FILE = '/tmp/clustermon.forcestart';

    private $_boolean_to_switch = array('true' => 'on', 'false' => 'off');
    private $_switch_to_boolean = array('on' => 'true', 'off' => 'false');

    private $_script_path = '';

    private $_config_buffer_cache = null;
    private $_status_buffer_cache = null;

    private $_service_status = null;

    private $_resource_cache = null;
    private $_group_cache = null;
    private $_node_cache = null;
    private $_options_cache = null;
    private $_rings_cache = null;
    private $_services_cache = null;

    private $_services_name_cache = null;
    private $_services_status_cache = null;

    private $_the_node = null;

    public function __construct($app)
    {
        $this->_the_node = $app->node();
        parent::__construct($app->node()->software(), "clustermon");
        $this->_script_path = $this->node()->configuration_manager()->directory('sng-base');
    }

    public function configure()
    {
        $this->register_aggregate_object('resource',
            array(
                'name' => 'Resource',
                'dynamic' => true,
                'readonly' => true,
                'configurable' => false,
                'display' => false,
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve resource properties',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update resource properties',
                        'request' => 'POST',
                    ),
                    'cleanup' => array(
                        'name' => 'Cleanup',
                        'description' => 'Cleanup errors from previous operations',
                        'request' => 'POST',
                    ),
                    'status' => array(
                        'name' => 'Status',
                        'description' => 'Retrieve a resource status',
                        'request' => 'POST',
                    ),
                ),
            )
        );

        $this->register_aggregate_object('group',
            array(
                'name' => 'Group',
                'dynamic' => true,
                'readonly' => true,
                'configurable' => false,
                'display' => false,
                'methods' => array(
                    'status' => array(
                        'name' => 'Status',
                        'description' => 'Retrieve a group status',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve group properties',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update group properties',
                        'request' => 'POST',
                    ),
                ),
            )
        );

        $this->register_aggregate_object('node',
            array(
                'name' => 'Node',
                'dynamic' => true,
                'readonly' => true,
                'configurable' => false,
                'display' => false,
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve node properties',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update node properties',
                        'request' => 'POST',
                    ),
                    'status' => array(
                        'name' => 'Status',
                        'description' => 'Retrieve a node status',
                        'request' => 'POST',
                    ),
                    'migrate' => array(
                        'name' => 'Migrate',
                        'description' => 'Migrate resources to this node',
                        'request' => 'POST',
                    ),
                ),
            )
        );

        $this->register_aggregate_object('options',
            array(
                'name' => 'Cluster Option Group',
                'dynamic' => true,
                'readonly' => true,
                'configurable' => false,
                'display' => false,
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a cluster option group',
                        'request' => 'GET',
                    ),
                ),
            )
        );

        $this->register_aggregate_object('rings',
            array(
                'name' => 'Cluster Rings',
                'dynamic' => true,
                'readonly' => true,
                'configurable' => false,
                'display' => false,
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a cluster ring',
                        'request' => 'GET',
                    ),
                ),
            )
        );

        $this->register_aggregate_object('database',
            array(
                'name' => 'Database',
                'dynamic' => true,
                'readonly' => true,
                'configurable' => false,
                'singleton' => true,
                'display' => false,
                'methods' => array(
                    'status' => array(
                        'name' => 'Status',
                        'description' => 'Retrieve the database status',
                        'request' => 'POST',
                    ),
                ),
            )
        );

        $this->register_aggregate_object('monitor',
            array(
                'name' => 'Monitor',
                'dynamic' => true,
                'readonly' => true,
                'configurable' => false,
                'singleton' => true,
                'display' => false,
                'methods' => array(
                    'status' => array(
                        'name' => 'Status',
                        'description' => 'Retrieve the monitor status',
                        'request' => 'POST',
                    ),
                ),
            )
        );

        $this->register_aggregate_object('services',
            array(
                'name' => 'Services',
                'dynamic' => true,
                'readonly' => true,
                'configurable' => false,
                'display' => false,
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a service',
                        'request' => 'GET',
                    ),
                    'start' => array(
                        'name' => 'Start',
                        'description' => 'Start service on nodes',
                        'request' => 'POST',
                    ),
                    'stop' => array(
                        'name' => 'Stop',
                        'description' => 'Stop service on nodes',
                        'request' => 'POST',
                    ),
                    'restart' => array(
                        'name' => 'Restart',
                        'description' => 'Restart service on nodes',
                        'request' => 'POST',
                    ),
                    'status' => array(
                        'name' => 'Status',
                        'description' => 'Retrieve a service status',
                        'request' => 'POST',
                    ),
                ),
            )
        );

        // Set the module description
        $this->set_description("Cluster Management");
        return parent::configure();
    }

    public function get_data_settings()
    {
        $cfg_manager = $this->node()->configuration_manager();
        $data = array(
            'log' => array(
                'description' => 'Log file',
                'filesystem' => true,
                'pattern' => '/var/log/cluster-service.log*',
                'global_capabilities' => array(
                    'download'
                ) ,
                'capabilities' => array(
                    'download',
                    'view'
                ) ,
                'methods' => array(
                        'download' => array(
                                'name' => 'Download',
                                'description' => array( 'Download log files.' ),
                                'request' => 'GET',
                                'icon' => 'download',
                                'button' => true,
                                'action_url' => '/SAFe/sng_data_manager/download/cluster/log'
                        )
                )
            )
        );
        return $data;
    }

    public function get_aggregate_objects($type, $all = false, $params = array())
    {
        if ($type == 'resource') {
            if ($this->_resource_cache === null) {
                $this->_resource_cache = array();
                foreach ($this->list_resources() as $name => $obj) {
                    $this->_resource_cache[$name] = $this->api_create_resource($name, array(), $output);
                }
            }
            return $this->_resource_cache;
        } else if ($type == 'group') {
            if ($this->_group_cache === null) {
                $this->_group_cache = array();
                foreach ($this->list_groups() as $name => $obj) {
                    $this->_group_cache[$name] = $this->api_create_group($name, array(), $output);
                }
            }
            return $this->_group_cache;
        } else if ($type == 'node') {
            if ($this->_node_cache === null) {
                $this->_node_cache = array();
                foreach ($this->list_nodes() as $name => $obj) {
                    $this->_node_cache[$name] = $this->api_create_node($name, array(), $output);
                }
                foreach ($this->list_config_nodes() as $name => $data) {
                    if (array_key_exists($name, $this->_node_cache)) continue;
                    $this->_node_cache[$name] = $this->api_create_node($name, array(), $output);
                }
            }
            return $this->_node_cache;
        } else if ($type == 'options') {
            if ($this->_options_cache === null) {
                $this->_options_cache = array();
                foreach ($this->list_options() as $name => $obj) {
                    $this->_options_cache[$name] = $this->api_create_options($name, array(), $output);
                }
            }
            return $this->_options_cache;
        } else if ($type == 'rings') {
            if ($this->_rings_cache === null) {
                $this->_rings_cache = array();
                foreach ($this->list_rings() as $name => $obj) {
                    $this->_rings_cache[$name] = $this->api_create_rings($name, array(), $output);
                }
            }
            return $this->_rings_cache;
        } else if ($type == 'services') {
            if ($this->_services_cache === null) {
                $this->_services_cache = array();
                $services = $this->node()->software()->services();
                if ($services) {
                    foreach ($services as $service) {
                        $this->_services_cache[$service->process_name()] = $this->api_create_services($service->process_name(), array(), $output);
                    }
                }
            }
            return $this->_services_cache;
        } else {
            return parent::get_aggregate_objects($type, $all, $params);
        }
    }

    /* helper methods for listing objects */
    public function resources()
    {
        return $this->get_aggregate_objects('resource');
    }
    public function groups()
    {
        return $this->get_aggregate_objects('group');
    }
    public function nodes()
    {
        return $this->get_aggregate_objects('node');
    }
    public function options()
    {
        return $this->get_aggregate_objects('options');
    }
    public function rings()
    {
        return $this->get_aggregate_objects('rings');
    }

    /* resource methods */
    public function api_create_resource($name=null, $data=null, &$output = null)
    {
        $obj = new Sng_clustermon_resource_class($this->node(), $this->object_name() . '/resource', $name);
        $obj->configure();
        if ($obj) {
            $obj->set_data_values($data);
        }
        return $obj;
    }
    public function api_cleanup_resource($name=null, $data=null, &$output = null)
    {
        $resource = safe_array_get($this->list_resources()[$name], null);
        if ($resource === null) {
            $output['message'] = 'Unknown resource "'.$name.'"';
            return false;
        }
        list($rc, $ignored, $errmsgs) = $this->cluster_tool('cleanup '.$name);
        if ($rc !== 0) {
            $output['message'] = 'Failed to "cleanup" resource "'.$name.'" (code='.$rc.').'.$errmsgs;
            return false;
        }
        return true;
    }
    public function api_retrieve_resource($name=null, $data=null, &$output = null)
    {
        $ret = array();
        $resource = safe_array_get($this->list_resources()[$name], null);
        if ($resource === null) {
            $output['message'] = 'Unknown resource "'.$name.'"';
            return false;
        }
        $target_role_default = 'Started';
        $maintenance_default = 'off';
        $group_name = safe_array_get($resource['group'], null);
        if ($group_name != null) {
            $group = $this->api_retrieve_group($group_name, $data, $output);
            if ($group == false) {
                $output['message'] = 'Error retrieving group for resource "'.$name.'": ' . $output['message'];
                return false;
            }
            $target_role_default = $group['target-role'];
            $maintenance_default = safe_array_get($this->_switch_to_boolean[$group['maintenance']], 'false');
        }

        return safe_array_normalize(safe_array_get($resource['attributes'], array()),
            array('maintenance' => $maintenance_default, 'target-role' => $target_role_default),
            array('maintenance' => $this->_boolean_to_switch));
    }
    public function api_update_resource($name=null, $data=null, &$output = null)
    {
        $current = $this->api_retrieve_resource($name);
        if ($current === false) {
            $output['message'] = 'Unknown resource "'.$name.'"';
            return false;
        }

        $target_role = safe_array_get($data['target-role'], null);
        if ($target_role !== null) { /* NOTE: rewrite target-role even if the same is already configured */
            if (!$this->check_valid_role($target_role, 'target-role', $output))
                return false;

            list($rc, $ignored, $errmsgs) = $this->cluster_tool(($target_role == 'Started' ? 'start' : 'stop').' '.$name);
            if ($rc !== 0) {
                $output['message'] = 'Unable to set "target-role" to "'.$target_role.'" (code='.$rc.').'.$errmsgs;
                return false;
            }
        }

        $maintenance = safe_array_get($data['maintenance'], null);
        if ($maintenance !== null && $maintenance !== $current['maintenance']) {
            if (!$this->check_valid_switch($maintenance, 'maintenance', $output))
                return false;

            list($rc, $ignored, $errmsgs) = $this->cluster_tool('maintenance ' . ($maintenance == 'off' ? '--off ' : '') . $name);
            if ($rc !== 0) {
                $output['message'] = 'Unable to set "maintenance" to "'.$maintenance.'" (code='.$rc.').'.$errmsgs;
                return false;
            }
        }

        return true;
    }
    public function api_status_resource($name=null, $data=null, &$output = null)
    {
        $resource = safe_array_get($this->list_resources()[$name], null);
        if ($resource === null) {
            $output['message'] = 'Unknown resource "'.$name.'"';
            return false;
        }
        unset($resource['attributes']);
        return $resource;
    }

    /* group methods */
    public function api_create_group($name=null, $data=null, &$output = null)
    {
        $obj = new Sng_clustermon_group_class($this->node(), $this->object_name() . '/group', $name);
        $obj->configure();
        if ($obj) {
            $obj->set_data_values($data);
        }
        return $obj;
    }
    public function api_retrieve_group($name=null, $data=null, &$output = null)
    {
        $group = safe_array_get($this->list_groups()[$name], null);
        if ($group === null) {
            $output['message'] = 'Unknown group "'.$name.'"';
            return false;
        }
        return safe_array_normalize(safe_array_get($group['attributes'], array()),
            array('maintenance' => 'false', 'target-role' => 'Started'),
            array('maintenance' => array('true' => 'on', 'false' => 'off')));
    }
    public function api_update_group($name=null, $data=null, &$output = null)
    {
        $current = $this->api_retrieve_group($name);
        if ($current === false) {
            return false;
        }

        $target_role = safe_array_get($data['target-role'], null);
        if ($target_role !== null) { /* NOTE: rewrite target-role even if the same is already configured */
            if (!$this->check_valid_role($target_role, 'target-role', $output))
                return false;

            list($rc, $ignored, $errmsgs) = $this->cluster_tool(($target_role == 'Started' ? 'start' : 'stop').' '.$name);
            if ($rc !== 0) {
                $output['message'] = 'Unable to set "target-role" to "'.$target_role.'" (code='.$rc.').'.$errmsgs;
                return false;
            }
        }

        $maintenance = safe_array_get($data['maintenance'], null);
        if ($maintenance !== null && $maintenance !== $current['maintenance']) {
            if (!$this->check_valid_switch($maintenance, 'maintenance', $output))
                return false;

            list($rc, $ignored, $errmsgs) = $this->cluster_tool('maintenance ' . ($maintenance == 'off' ? '--off ' : '') . $name);
            if ($rc !== 0) {
                $output['message'] = 'Unable to set "maintenance" to "'.$maintenance.'" (code='.$rc.').'.$errmsgs;
                return false;
            }
        }

        return true;
    }
    public function api_status_group($name=null, $data=null, &$output = null)
    {
        $group = safe_array_get($this->list_groups()[$name], null);
        if ($group === null) {
            $output['message'] = 'Unknown group "'.$name.'"';
            return false;
        }
        unset($group['attributes']);
        return $group;
    }

    /* node methods */
    private function _normalize_location(&$location)
    {
        $result = array();
        foreach (safe_array_get($location, array()) as $loc_res => $loc_data) {
            $result[$loc_res] = $loc_data['score'];
        };
        return $result;
    }

    public function api_create_node($name=null, $data=null, &$output = null)
    {
        $obj = new Sng_clustermon_node_class($this->node(), $this->object_name() . '/node', $name);
        $obj->configure();
        if ($obj) {
            $obj->set_data_values($data);
        }
    }
    public function api_retrieve_node($name=null, $data=null, &$output = null)
    {
        $node_status = safe_array_get($this->list_nodes()[$name], null);
        $node_config = safe_array_get($this->list_config_nodes()[$name], null);
        if ($node_status === null) {
            if ($node_config === null) {
                $output['message'] = 'Unknown node "'.$name.'"';
                return false;
            }
            return array('uuid' => $node_config['uuid']); // no status attributes, node from config
        }
        $result = safe_array_normalize(safe_array_get($node_status['attributes'], array()),
            array('maintenance' => 'false', 'standby' => 'off'),
            array('maintenance' => array('true' => 'on', 'false' => 'off')));

        $result['location'] = $this->_normalize_location($node_status['location']);
        $result['uuid'] = $node_config['uuid'];

        return $result;
    }
    public function api_update_node($name=null, $data=null, &$output = null)
    {
        $current = $this->api_retrieve_node($name);
        if ($current === false) {
            $output['message'] = 'Unknown node "'.$name.'"';
            return false;
        }

        $maintenance = safe_array_get($data['maintenance'], null);
        if ($maintenance !== null && $maintenance !== $current['maintenance']) {
            if (!$this->check_valid_switch($maintenance, 'maintenance', $output))
                return false;

            list($rc, $ignored, $errmsgs) = $this->cluster_tool('maintenance --node ' . ($maintenance == 'off' ? '--off ' : '') . $name);
            if ($rc !== 0) {
                $output['message'] = 'Unable to set "maintenance" to "'.$maintenance.'" (code='.$rc.').'.$errmsgs;
                return false;
            }
        }
        $standby = safe_array_get($data['standby'], null);
        if ($standby !== null && $standby !== $current['standby']) {
            if (!$this->check_valid_switch($standby, 'standby', $output))
                return false;

            list($rc, $ignored, $errmsgs) = $this->cluster_tool('standby ' . ($standby == 'off' ? '--off ' : '') . $name);
            if ($rc !== 0) {
                $output['message'] = 'Unable to set "standby" to "'.$standby.'" (code='.$rc.').'.$errmsgs;
                return false;
            }
        }
        $location = safe_array_get($data['location'], null);
        if ($location !== null) {
            $valid_score = array('-INFINITY', 'INFINITY', 'null'); // TODO: support other scores?
            $valid_res = array(Sng_clustermon_service_class::CLUSTER_GROUP_NAME); // TODO: support other resources?

            $cur_location = safe_array_get($current['location'], array());
            foreach ($location as $loc_res => $loc_score) {
                if (!in_array($loc_res, $valid_res)) {
                    $output['message'] = 'Invalid resource "' . $loc_res . '" (valid: "' . implode('", "', $valid_res) . '").';
                    return false;
                }

                if (!in_array($loc_score, $valid_score)) {
                    $output['message'] = 'Invalid score "' . $loc_score . '" (valid: "' . implode('", "', $valid_score) . '").';
                    return false;
                }

                $run_remove = false;
                $run_create = false;

                $cur_score = safe_array_get($cur_location[$loc_res], 'null');

                if ($loc_score == 'null') {
                    $run_remove = array_key_exists($loc_res, $cur_location);
                } else {
                    if ($loc_score !== $cur_score) {
                        $run_remove = true;
                        $run_create = true;
                    }
                }

                if ($run_remove) {
                    $action = ($cur_score == 'INFINITY' ? 'migrate-to' : 'migrate-away');
                    list($rc, $ignored, $errmsgs) = $this->cluster_tool($action . ' --undo');
                    if ($rc !== 0) {
                        $output['message'] = 'Unable to clear location for "' . $name . '" (code='.$rc.').'.$errmsgs;
                        return false;
                    }
                }

                if ($run_create) {
                    $action = ($loc_score == 'INFINITY' ? 'migrate-to' : 'migrate-away');
                    list($rc, $ignored, $errmsgs) = $this->cluster_tool($action . ' --persist ' . $name);
                    if ($rc !== 0) {
                        $output['message'] = 'Unable to set location for "' . $name . '" (code='.$rc.').'.$errmsgs;
                        return false;
                    }
                }
            }
        }
        return true;
    }
    public function api_migrate_node($name=null, $data=null, &$output = null)
    {
        $node_status = $this->api_status_node($name);
        if ($node_status === false) {
            $output['message'] = 'Unknown node "'.$name.'"';
            return false;
        }

        if ($node_status['master'] == 'true') {
            $output['message'] = 'Node "'.$name.'" is already the master node.';
            return false;
        }

        list($rc, $ignored, $errmsgs) = $this->cluster_tool($action . ' migrate-to ' . $name);
        if ($rc !== 0) {
            $output['message'] = 'Unable to migrate to "' . $name . '" (code='.$rc.').'.$errmsgs;
            return false;
        }

        return true;
    }

    public function api_status_node($name=null, $data=null, &$output = null)
    {
        $node_status = safe_array_get($this->list_nodes()[$name], null);
        $node_config = safe_array_get($this->list_config_nodes()[$name], null);
        if ($node_status === null) {
            if ($node_config === null) {
                $output['message'] = 'Unknown node "'.$name.'"';
                return false;
            }
            return array();
        }
        $resource_status = $this->api_status_resource(Sng_clustermon_service_class::CLUSTER_SWITCH_NAME, null, $output);
        if ($resource_status === false) {
            safe_log(SAFE_LOG_ERROR, 'unable to call status for resource "' . Sng_clustermon_service_class::CLUSTER_SWITCH_NAME .
                '", cannot populate "master" attribute for node "' . $name . '"');
        } else {
            $master_status = 'false';
            if (safe_array_get($resource_status['status']['name'], 'unknown') == 'start') {
                $master_status = in_array($name, safe_array_get($resource_status['status']['nodes'], array())) ? 'true' : 'false';
            }
            $node_status['master'] = $master_status;
        }
        unset($node_status['attributes']);
        return $node_status;
    }

    /* options methods */
    public function api_create_options($name=null, $data=null, &$output = null)
    {
        $obj = new Sng_clustermon_options_class($this->node(), $this->object_name() . '/options', $name);
        $obj->configure();
        if ($obj) {
            $obj->set_data_values($data);
        }
    }
    public function api_retrieve_options($name=null, $data=null, &$output = null)
    {
        $options = safe_array_get($this->list_options()[$name], null);
        if ($options === null) {
            $output['message'] = 'Unknown option group "'.$name.'"';
            return false;
        }
        return $options;
    }

    /* rings methods */
    public function api_create_rings($name=null, $data=null, &$output = null)
    {
        $obj = new Sng_clustermon_rings_class($this->node(), $this->object_name() . '/rings', $name);
        $obj->configure();
        if ($obj) {
            $obj->set_data_values($data);
        }
    }
    public function api_retrieve_rings($name=null, $data=null, &$output = null)
    {
        $rings = safe_array_get($this->list_rings()[$name], null);
        if ($rings === null) {
            $output['message'] = 'Unknown ring "'.$name.'"';
            return false;
        }
        return $rings;
    }

    /* monitor methods */
    public function api_create_monitor($name=null, $data=null, &$output = null)
    {
        $obj = new Sng_clustermon_monitor_class($this->node(), $this->object_name() . '/monitor', $name);
        $obj->configure();
        if ($obj) {
            $obj->set_data_values($data);
        }
    }
    public function api_status_monitor($name=null, $data=null, &$output = null)
    {
        if ($this->_service_status === null) {
            $this->monitor_status();
        }
        return $this->_service_status;
    }

    /* services methods */
    public function api_retrieve_services($name=null, $data=null, &$output = null)
    {
        if ($this->_services_cache === null) {
            $this->_services_cache = array();
            $services = $this->node()->software()->services();
            if ($services) {
                foreach ($services as $service) {
                    if ($name != $service->process_name())
                        continue;

                    return $this->api_create_services($name, array(), $output);
                }

                $output['message'] = 'Unknown service "'.$name.'"';
                return false;
            }
        }

        $obj = safe_array_get($this->_services_cache[$name], null);
        if ($obj === NULL) {
            $output['message'] = 'Unknown service "'.$name.'"';
            return false;
        }
        return $obj;
    }

    public function api_create_services($name=null, $data=null, &$output = null)
    {
        $obj = new Sng_clustermon_services_class($this->node(), $this->object_name() . '/services', $name);
        $obj->configure();
        if ($obj) {
            $obj->set_data_values($data);
        }
        return $obj;
    }

    private function _nodes_argument($nodelist)
    {
        return count($nodelist) != 0 ? '--nodes ' . implode(',', $nodelist) : '--all';
    }

    private function _nodes_parameter($data)
    {
        $nodelist = ($data !== null && isset($data['nodes'])) ? $data['nodes'] : array();
        if (is_array($nodelist)) {
            return array($this->_nodes_argument($nodelist), $nodelist);
        } else {
            return array(false, false);
        }
    }

    private function _services_status($service_name, $nodelist, &$output = null)
    {
        $nodediffs = $nodelist;
        $is_cached = true;

        if ($this->_services_status_cache !== null) {
            $nodediffs = array_diff(array_keys($this->_services_status_cache), $nodelist);

            foreach ($nodediffs as $node) {
                if (isset($this->_services_status_cache[$node][$service_name])) continue;
                $is_cached = false;
            }
        } else {
            $is_cached = false;
        }

        if ($is_cached) {
            return $this->_services_status_cache;
        }

        $nodeparam = $this->_nodes_argument($nodediffs);

        if ($this->_services_names_cache === null) {
            $services = $this->node()->software()->services();
            $this->_services_names_cache = array();
            if ($services) {
                foreach ($services as $service) {
                    $this->_services_names_cache[] = $service->process_name();
                }
            }
        }

        if (!in_array($service_name, $this->_services_names_cache)) {
            $this->_services_names_cache[] = $service_name;
        }

        $serviceparam = implode(' ', $this->_services_names_cache);

        list($rc, $outmsgs, $errmsgs) = $this->cluster_tool('service --json '.$nodeparam.' status '.$serviceparam);

        if ($rc !== 0) {
            $this->_services_status_cache = array();
            $output['message'] = 'Failed to query status for service "'.$service_name.'" (code='.$rc.').'.$errmsgs;
            return null;
        };

        $this->_services_status_cache = json_decode($outmsgs, true);
        return $this->_services_status_cache;
    }
    public function services_status($node_name, $service_name)
    {
        $status_data = $this->_services_status($service_name, array($node_name), $output);
        if ($status_data === null) {
            return array(Safe_service_class::STATUS_ERROR, null);
        }
        $rc = safe_array_get($status_data[$node_name][$service_name]['status']['returncode'], Safe_service_class::STATUS_ERROR);
        $stats = safe_array_get($status_data[$node_name][$service_name]['stats'], array());
        return array($rc, $stats);
    }

    public function api_status_services($name=null, $data=null, &$output = null)
    {
        $status_result = array();

        list($nodeparam, $nodelist) = $this->_nodes_parameter($data);
        if ($nodeparam === false) {
            $output['message'] = 'Incorrect "nodes" parameter (must be a list).';
            return false;
        }
        $status_data = $this->_services_status($name, $nodelist, $output);
        if ($status_data === null) {
            return false;
        }
        foreach ($status_data as $node_name => $node_data) {
            if (count($nodelist) !== 0 && !in_array($node_name, $nodelist)) continue;
            $status = safe_array_get($node_data[$name]['status']['returncode'], Safe_service_class::STATUS_ERROR);
            $status_text = Safe_service_class::get_status_text($status);
            $status_result[$node_name] = array('status' => $status, 'status_text' => $status_text);
        }
        return $status_result;
    }
    public function api_start_services($name=null, $data=null, &$output = null)
    {
        list($nodeparam, $nodelist) = $this->_nodes_parameter($data);
        if ($nodeparam === false) {
            $output['message'] = 'Incorrect "nodes" parameter (must be a list).';
            return false;
        }
        $waitparam = $data['wait-master'] == 'true' ? ' --wait-master' : '';
        list($rc, $outmsgs, $errmsgs) = $this->cluster_tool('service '.$nodeparam.$waitparam.' start '.$name);
        if ($rc !== 0) {
            $output['message'] = 'Failed to start service "'.$name.'" (code='.$rc.').'.$errmsgs;
            return false;
        };
        return true;
    }
    public function api_stop_services($name=null, $data=null, &$output = null)
    {
        list($nodeparam, $nodelist) = $this->_nodes_parameter($data);
        if ($nodeparam === false) {
            $output['message'] = 'Incorrect "nodes" parameter (must be a list).';
            return false;
        }
        list($rc, $outmsgs, $errmsgs) = $this->cluster_tool('service '.$nodeparam.' stop '.$name);
        if ($rc !== 0) {
            $output['message'] = 'Failed to stop service "'.$name.'" (code='.$rc.').'.$errmsgs;
            return false;
        };
        return true;
    }
    public function api_restart_services($name=null, $data=null, &$output = null)
    {
        list($nodeparam, $nodelist) = $this->_nodes_parameter($data);
        if ($nodeparam === false) {
            $output['message'] = 'Incorrect "nodes" parameter (must be a list).';
            return false;
        }
        list($rc, $outmsgs, $errmsgs) = $this->cluster_tool('service '.$params.' restart '.$name);
        if ($rc !== 0) {
            $output['message'] = 'Failed to restart service "'.$name.'" (code='.$rc.').'.$errmsgs;
            return false;
        };
        return true;
    }

    /* database methods */
    public function api_create_database($name=null, $data=null, &$output = null)
    {
        $obj = new Sng_clustermon_database_class($this->node(), $this->object_name() . '/database', $name);
        $obj->configure();
        if ($obj) {
            $obj->set_data_values($data);
        }
    }
    public function api_status_database($name=null, $data=null, &$output = null)
    {
        return $this->get_database_status();
    }

    /* log methods */
    public function api_download_log($name=null, $data = null, &$output = null)
    {
        $this->download_data('log', $name);
        exit();
    }

    /* status queries */
    private function monitor_status()
    {
        if ($this->_status_buffer_cache === null)
        {
            $socket = socket_create(AF_INET, SOCK_STREAM, 0);

            if (!socket_connect($socket, '127.0.0.1', 1010)) {
                $this->_service_status = array('status' => 'not-running');
                return array();
            }

            $rset = array();
            $wset = array($socket);
            $eset = array();

            if (socket_select($rset, $wset, $eset, Sng_clustermon_service_class::CLUSTER_STATUS_TIMEOUT) == 0) {
                $this->_service_status = array('status' => 'error', 'reason' => 'timeout-send');
                $this->_status_buffer_cache = '{}';
                return array();
            }

            $status_text = "STATUS\n";
            socket_send($socket, $status_text, strlen($status_text), 0);

            $buffer = '';
            while (TRUE) {
                $rset = array($socket);
                $wset = array();
                $eset = array();

                if (socket_select($rset, $wset, $eset, Sng_clustermon_service_class::CLUSTER_STATUS_TIMEOUT) == 0) {
                    $this->_service_status = array('status' => 'error', 'reason' => 'timeout-recv');
                    $this->_status_buffer_cache = '{}';
                    return array();
                }

                $rdata = '';
                $bytes = socket_recv($socket, $rdata, 4096, 0);
                if ($bytes === 0) {
                    break;
                }
                $buffer = $buffer . $rdata;
            }

            socket_close($socket);

            $this->_status_buffer_cache = $buffer;
        }

        $ret = json_decode($this->_status_buffer_cache, true);

        if ($ret === NULL) {
            $this->_service_status = array('status' => 'error', 'reason' => 'decode-failure');
            return array();
        }

        $this->_service_status = array('status' => safe_array_list_get($ret, array('cluster', 'service'), 'unknown'));
        return $ret;
    }

    public function drop_caches()
    {
        $this->_service_status = null;
        $this->_status_buffer_cache = null;

        $this->_resource_cache = null;
        $this->_group_cache = null;
        $this->_node_cache = null;
        $this->_options_cache = null;
        $this->_rings_cache = null;

        $this->_services_name_cache = null;
        $this->_services_status_cache = null;
    }

    public function get_local_hostname($use_hostname = false)
    {
        $target_node = $use_hostname ? gethostname() : NULL;

        $nodes_config = $this->list_config_nodes();

        if ($nodes_config === NULL || count($nodes_config) === 0) {
            return NULL;
        }

        $target_uuid = Sng_clustermon_service_class::machine_uuid();

        foreach ($nodes_config as $node_name => $node_data) {
            if ($node_data['uuid'] === $target_uuid) {
                $target_node = $node_name;
                break;
            }
        }

        return $target_node;
    }

    public function need_force_start($update = false)
    {
        if($update == true){
            file_put_contents(Sng_clustermon_service_class::CLUSTERMON_FORCE_START_FILE,'force-start');
            return true;
        } else {
            if(file_exists(Sng_clustermon_service_class::CLUSTERMON_FORCE_START_FILE))
                return true;
        }
        return false;
    }

    public function force_stop()
    {
        $result = $this->stop();
        unlink(Sng_clustermon_service_class::CLUSTERMON_FORCE_START_FILE);

        return (!$result)?true:false;
    }

    public function get_cluster_nodes_id($status = NULL)
    {
        return $this->list_nodes_id($status);
    }

    public function check_master($status = NULL, $target_node = NULL)
    {
        if ($status === NULL) {
            $status = $this->monitor_status();

            if (!array_key_exists('cluster', $status)) {
                return NULL;
            }
        }

        if ($target_node === NULL) {
            $target_node = $this->get_local_hostname();
            if ($target_node === NULL) {
                return NULL;
            }
            foreach ($this->list_nodes() as $name => $obj) {
                if($target_node == $name ){
                    $target_node_id = $obj['id'];
                    break;
                }
            }
        }

        $resource = safe_array_list_get($status, array('cluster', 'resources', 'Cluster'));

        if ($resource == NULL) {
            return NULL;
        }

        if (safe_array_list_get($resource, array('status', 'name'), 'unknown') != 'start') {
            return NULL;
        }

        foreach (safe_array_list_get($resource, array('status', 'nodes'), array()) as $node_name) {
            if ($target_node_id == $node_name) {
                return true;
            }
        }

        return false;
    }

    public function check_errors($status = NULL, $node = NULL, $resource = NULL)
    {
        $operations_errors = array();
        // scan for errors on node operations
        foreach ($this->list_nodes() as $node_name => $node_data) {
            if ($node !== NULL && $node != $node_name) continue;

            foreach (safe_array_get($node_data['resources'], array()) as $res_name => $res_data) {
                if ($resource !== NULL && $resource != $res_name) continue;

                foreach (safe_array_get($res_data['operations'], array()) as $oper_data) {
                    if ($oper_data['name'] != 'start' || $oper_data['rc'][0] == 'success') {
                        continue;
                    }

                    $operations_errors[$node_name][$res_name] = $oper_data['rc'][1];
                }
            }
        }

        return $operations_errors;
    }

    /* helper for configuration apply and restart */
    public function restart_service($service_obj, $force = false, &$result = null)
    {
        $resources_map = array('nsc' => 'Cluster');

        $output = array();
        $need_maintenance = false;

        $res_name = safe_array_get($resources_map[$service_obj->process_name()], null);

        if ($res_name == null) {
            $result = 'Cannot restart "' . $service_obj->name() . '": service not mapped to a cluster resource.';
            return 1;
        }

        /* NOTE: we should be master - or there should be no master - so we can restart services on our node */
        if (safe_array_get($this->api_status_monitor()['status'], '<null>') === 'running') {
            if ($this->check_master() === false) {
                $result = 'Cannot restart "' . $service_obj->name() . '" (resource "'.$res_name.'"): current node is not the master.';
                return 1;
            }
            $need_maintenance = true;
        }

        if ($need_maintenance) {
            if (!$this->api_update_resource(Sng_clustermon_service_class::CLUSTER_SWITCH_NAME, array('maintenance' => 'on'), $output)) {
                $result = $output['message'];
                return 1;
            }
        }

        $rc = $service_obj->restart_service($force, $result);

        $this->drop_caches();

        $is_running = false;

        for ($num = 0; $num < 60; $num++) {
            if ($service_obj->status() == Safe_service_class::STATUS_RUNNING) {
                $is_running = true;
                break;
            }
            time_nanosleep(0, 333000000);
        };

        if (!$is_running) {
            $result = 'Service "' . $service_obj->name() . '" (resource "'.$res_name.'") failed to restart - check logs for more information.';
            return 1;
        }

        if ($need_maintenance) {
            if (!$this->api_update_resource(Sng_clustermon_service_class::CLUSTER_SWITCH_NAME, array('maintenance' => 'off'), $output)) {
                $result = $output['message'];
                return 1;
            }
        }

        return $rc;
    }

    /* helpers for listing objs */
    private function list_config_nodes()
    {
        if ($this->_config_buffer_cache === null) {
            list($rc, $stdout, $stderr) = $this->cluster_tool('nodes', true);

            if ($rc === 0) {
                $this->_config_buffer_cache = $stdout;
            } else {
                safe_log(SAFE_LOG_ERROR, 'unable to load nodes configuration: ' . str_replace(array("\r\n", "\n", "\r"), ', ', trim($stderr)));
            }
        }

        if ($this->_config_buffer_cache !== null) {
            return json_decode($this->_config_buffer_cache, true);
        } else {
            return array();
        }
    }


    private function list_resources($status = NULL)
    {
        if ($status == NULL) {
            $status = $this->monitor_status();
        }

        return safe_array_list_get($status, array('cluster', 'resources'), array());
    }

    private function list_groups($status = NULL)
    {
        if ($status == NULL) {
            $status = $this->monitor_status();
        }

        return safe_array_list_get($status, array('cluster', 'groups'), array());
    }

    private function list_nodes($status = NULL)
    {
        if ($status == NULL) {
            $status = $this->monitor_status();
        }

        $result = array();
        $test = safe_array_list_get($status, array('cluster', 'nodes'), array());
        foreach($test as $key => $val){
            if(is_array($val) && isset($val['name'])){
                $result[$val['name']] = $val;
                $result[$val['name']]['id'] = $key;
            }
        }

        return $result;
    }

    private function list_nodes_id($status = NULL)
    {
        if ($status == NULL) {
            $status = $this->monitor_status();
        }

        return safe_array_list_get($status, array('cluster', 'nodes'), array());
    }

    private function list_options($status = NULL)
    {
        if ($status == NULL) {
            $status = $this->monitor_status();
        }

        return safe_array_list_get($status, array('cluster', 'options'), array());
    }

    private function list_rings($status = NULL)
    {
        if ($status == NULL) {
            $status = $this->monitor_status();
        }

        return safe_array_list_get($status, array('cluster', 'corosync', 'rings'), array());
    }

    private function get_database_status($status = NULL)
    {
        if ($status == NULL) {
            $status = $this->monitor_status();
        }

        return safe_array_get($status['db'], array());
    }


    /* cli tool */
    private function cluster_tool($cmdextra, $raw_stderr = false)
    {
        $cmdline = "{$this->_script_path}/bin/sng-cluster-tool " . $cmdextra;
        list($rc, $stdout, $stderr) = safe_sudo_pipe_exec($cmdline, '', '', false);

        $errmsgs = null;

        if ($raw_error) {
            $errmsgs = $stderr;
        } else {
            if ($rc != 0 && strlen($stderr) != 0) {
                $errmsgs = '<br><br><div>' . implode("<br>", preg_split("/\r\n|\n|\r/", $stderr)) . '</div>';
            }
        }

        return array($rc, $stdout, $errmsgs);
    }

    /* helpers */
    private function check_valid($value, $name, $valid, &$output)
    {
        if (!in_array($value, $valid)) {
            $output['message'] = 'Invalid value "' . $value . '" for "'. $name .'", expecting "' . implode('" or "', $valid) . '".';
            return false;
        }
        return true;
    }
    private function check_valid_switch($value, $name, &$output)
    {
        return $this->check_valid($value, $name, array('on', 'off'), $output);
    }

    private function check_valid_role($value, $name, &$output)
    {
        return $this->check_valid($value, $name, array('Started', 'Stopped'), $output);
    }

    public static function resource_pretty_name($resource_name)
    {
        if ($resource_name === Sng_clustermon_service_class::CLUSTER_SWITCH_NAME)
            return 'NetBorder Session Controller';

        if ($resource_name === Sng_clustermon_service_class::CLUSTER_GROUP_NAME)
            return '(all resources)';

        if (strpos($resource_name, Sng_clustermon_service_class::FLOATING_IP_PREFIX) === 0)
            return ('Floating IP &laquo;' . substr($resource_name, strlen(Sng_clustermon_service_class::FLOATING_IP_PREFIX)) . '&raquo;');

        return ('Resource &laquo;' . $resource_name . '&raquo;');
    }

    public static function machine_uuid()
    {
        list($rc, $stdout, $stderr) = safe_sudo_pipe_exec('/usr/local/sng/bin/sng-uuid-generate --verbose', '', '', false);

        if ($rc === 0) {
            return trim($stdout);
        } else {
            safe_log(SAFE_LOG_ERROR, 'unable to retrieve machine UUID: ' . str_replace(array("\r\n", "\n", "\r"), ', ', trim($stderr)));
            return NULL;
        }
    }

    public function node_str_status(){
        $monitor_status = safe_array_get($this->api_status_monitor()['status'], 'unknown');
        $master_status = $this->check_master();

        list($red, $yellow, $orange, $green, $grey) = array("#FF0000", "#8e8e00", "#E56717", "#008e00", "#8e8e8e");

        $nsc_stats = array();
        $nsc_status = $this->_the_node->software()->application()->status($stats);

        $desc_and_color = ($master_status !== NULL) ?
            ($master_status ? array('MASTER', $green) : array('SLAVE', $yellow)) :
            ($monitor_status == 'running' ? array('NO MASTER', $red) :
             ($nsc_status === 0 ? array('UNMONITORED', $orange) : array('NOT RUNNING', $grey)));
        return $desc_and_color;
    }

    public function handle_global_messages(&$msg_list)
    {
        $this->clear_global_messages_cache($msg_list, 'clustermon', 'node_status');

        // NOTE: we can't load cluster module here, so we need to check if HA is enabled
        // NOTE: in some other way before trying to fetch the monitor service status

        if (file_exists(Sng_clustermon_service_class::CLUSTERMON_CONFIG_FILE))
        {

            list($description, $color)=$this->node_str_status();
            $msg_list[] = array(
                'module' => 'clustermon',
                'cache_tag' => 'node_status',
                'global_message' =>
                    '<div style="display: table; padding-right: 8px; padding-top: 8px;">
                        <span style="display: table-cell; vertical-align: middle; text-align: center; font-weight: bold;">
                            <span>'.$this->get_local_hostname(true).'</span>
                        </span>
                        <span style="display: table-cell; float: left; padding-left: 5px; padding-right: 5px;">
                            <img height="24" width="24" src="/images/icons/32x32/icon-network.png">
                        </span>
                        <span style="display: table-cell; vertical-align: middle; text-align: center; font-weight: bold;">
                            <span style="color:'.$color.'">'.$description.'</span>
                        </span></div>'
            );
        }
    }

    public function check_nodes_status(&$data=""){
        $monitor_nodes = array_keys($this->nodes());
        foreach($monitor_nodes as $node_name){
            $node_data = (object) $this->api_status_node($node_name);
            $node_attr = $this->api_retrieve_node($node_name);

            if($node_data->status!=='online'){
                $node_status = $node_data->status;
                list($rc, $stats) = $this->services_status($node_name, 'clustermon');
                if($nade_status===null){
                    $node_status = ($rc !== Safe_service_class::STATUS_ERROR) ? strtolower(Safe_service_class::get_status_text($rc)) : 'unreachable';
                }
                else if ($node_status !== 'offline') {
                    if ($rc == Safe_service_class::STATUS_ERROR) {
                        // unable to connect, but heartbeat didn't timeout yet
                        $node_status = 'unavailable';
                    } else if ($rc != Safe_service_class::STATUS_RUNNING) {
                        // clustermon stopped but cluster is still running
                        $node_status = 'unmonitored';
                    }
                }
                if ($node_status == 'running') {
                        $node_status = 'not joined';
                }
                $data['message'] = "Cannot copy configuration to Peer because ".$node_name." is ".strtoupper($node_status);
                $data['redirect_url'] = "/SAFe/sng_control_panel";
                return false;
            }
        }
        return true;
    }

    public function is_local_node_online()
    {
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peers = $cluster_module->peers();
        if($this->ha_mode_active() && count($peers)){
            $hostname = $this->get_local_hostname();
            list($rc, $outmsgs, $errmsgs) = $this->cluster_tool('service status clustermon --json --nodes ' . $hostname);
            if($rc==0){
                $json_output_message = reset(json_decode(trim($outmsgs), true));
                $return_code = $json_output_message['clustermon']['status']['returncode'];
                return $return_code==Safe_service_class::STATUS_RUNNING ? true : false;
            }
        }
        return false;
    }

    public function is_peer_node_online()
    {
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $peers = $cluster_module->peers();
        if($this->ha_mode_active() && count($peers)){
            foreach($peers as $peer){
                $peer_network = $peer->api_retrieve_network_configuration();
            }
            $peer_hostname = $peer_network->get_data_value('global/hostname');
            list($rc, $outmsgs, $errmsgs) = $this->cluster_tool('service status clustermon --json --nodes ' . $peer_hostname);
            if($rc==0){
                $json_output_message = reset(json_decode(trim($outmsgs), true));
                $return_code = $json_output_message['clustermon']['status']['returncode'];
                return $return_code==Safe_service_class::STATUS_RUNNING ? true : false;
            }
        }
        return false;
    }
}
