<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * William Adam <wadam@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG CWMP (TR-069) Linux service wrapper
 *
 * @author William Adam
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');
require_once('sng_cwmp_config_class.php');


/******************************************************************************
 *   CWMP Module
 ******************************************************************************/
class Sng_cwmp_service_class extends Safe_service_class
{
    public function __construct($app)
    {
        parent::__construct($app->node()->software(), "cwmp");
    }
    /**
     * @brief
     *         
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("CWMP");
        // Create the config object
        $cfg = new Sng_cwmp_config_class($this->node(), $this->path() . '/' .$this->name());
        $cfg->configure();
        $cfg->synch();
        // Attach config to module
        $this->set_config($cfg);
        return parent::configure();
    }
    /**
     * @brief 
     *
     * @param[in out] $config_manager
     *
     * @return 
     */
    public function generate_config(&$config_manager)
    {
        return parent::generate_config($config_manager);
    }
    public function can_restore($info, &$reason)
    {
        // Prevent calling parent as service status will be checked
        return true;
    }
    /**
     * @brief Create config jobs for dialplan module
     *           
     * @param[in out] $config_manager
     *           
     * @return
     */
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        return $this->generate_config($config_manager);
    }
    /**
     * @brief 
     *
     * @param[in out] $obj_type
     *
     * @return 
     */
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }
    /**
     * @brief 
     *
     * @param[in out] $need_reload
     *
     * @return 
     */
    public function support_reload(&$need_reload = null) 
    {
        parent::support_reload($need_reload);
        return true;
    }

    public function get_data_settings()
    {
        $cfg_manager = $this->node()->configuration_manager();
        $data = array(
            'log' => array(
                'description' => 'Log file',
                'filesystem' => true,
                'pattern' => '/var/log/cwmp-service.log*',
                'global_capabilities' => array(
                    'download'
                ) ,
                'capabilities' => array(
                    'download',
                    'view'
                ) ,
                'methods' => array(
                        'download' => array(
                                'name' => 'Download',
                                'description' => array( 'Download log files.' ),
                                'request' => 'GET',
                                'icon' => 'download',
                                'button' => true,
                                'action_url' => '/SAFe/sng_data_manager/download/cwmp/log'
                        )
                )
            )
        );
        return $data;
    }
    public function api_download_log($name=null, $data = null, &$output = null){
        $this->download_data('log', $name);
        exit();
    }
}
