<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Firewall Linux service wrapper
 *
 * @author William Adam
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('service');
class Sng_firewall_config_class extends Safe_configurable_object_class
{
    /**
     * @brief Constructor
     *           
     * @param[in out] $parent_name
     * @param[in out] $name
     *           
     * @return
     */
    public function __construct($node, $parent_name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, "configuration", $node);
    }
    /**
     * @brief Configure the config class object
     *         
     * @return
     */
    public function configure()
    {
        // Synch with memory not DB
        $this->_set_sync_strategy( Safe_serializable_object_class::SYNC_MEM);
        // Grab adapters
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        //extif need to include hw interface and vlan interface
        $interface_objs = $network_module->api_retrieve_interface();
        $adapter_list = '';
        
        if($interface_objs){
            foreach($interface_objs as $interface_obj){
                //passby loopback and transcoding
                if($interface_obj->interface_type() != Sng_network_interface_class::LOOPBACK
                        && $interface_obj->hardware_type() != Sng_network_interface_class::TRANSCODING){
                    $if_name = $interface_obj->interface_name();
                    $adapter_list .= $if_name . ' ';
                }
            }
        }

        // General settings
        $this->add_field('extif', 'External Interface List', 'text', $adapter_list, 70);

        return parent::configure();
    }
}

class Sng_firewall_rule_class extends Safe_configurable_object_class
{
    // flags for firwall rules
    const INCOMING_ALLOW    = 0x00000001;   // Incoming allow; port/port range
    const INCOMING_BLOCK    = 0x00000002;   // Incoming block; host
    const OUTGOING_BLOCK    = 0x00000004;   // Outgoing block; host, port/range
    const FORWARD           = 0x00000008;   // Forward; port/port range
    const DMZ_PINHOLE       = 0x00000010;   // DMZ pinhole; host/port
    const DMZ_INCOMING      = 0x00000020;   // DMZ incoming; host/port
    const RESERVED_0        = 0x00000040;   // Reserved
    const ONE_TO_ONE        = 0x00000080;   // One-to-one NAT
    const PPTP_FORWARD      = 0x00000100;   // PPTP forward rule
    const MAC_FILTER        = 0x00000200;   // HW/MAC filter rule
    const SBR_PORT          = 0x00000400;   // SBR: by port
    const SBR_HOST          = 0x00000800;   // SBR: by host
    const BANDWIDTH_RATE    = 0x00001000;   // Bandwidth rate rule
    const BANDWIDTH_PRIO    = 0x00002000;   // Bandwidth priority rule
    const BANDWIDTH_BASIC   = 0x00004000;   // "Basic" bandwidth rule
    const RESERVED_1        = 0x00008000;   // Reserved
    const RESERVED_2        = 0x00010000;   // Reserved
    const RESERVED_3        = 0x00020000;   // Reserved
    const RESERVED_4        = 0x00040000;   // Reserved
    const RESERVED_5        = 0x00080000;   // Reserved
    const LOCAL_NETWORK     = 0x00100000;   // Create rule for local networks
    const EXTERNAL_ADDR     = 0x00200000;   // Create rule for external addr
    const PROXY_BYPASS      = 0x00400000;   // Web Proxy Bypass
    const L7FILTER_BYPASS   = 0x00800000;   // Layer7 Filter Bypass
    const MAC_SOURCE        = 0x01000000;   // HW/MAC source address
    const WIFI              = 0x02000000;   // Wireless rule
    const IFADDRESS         = 0x04000000;   // Interface address 'addr' field
    const IFNETWORK         = 0x08000000;   // Interface network 'addr' field
    const ENABLED           = 0x10000000;   // Rule is enabled
    const CUSTOM            = 0x20000000;   // Custom rule
    const RESERVED_6        = 0x40000000;   // Reserved
    const RESERVED_7        = 0x80000000;   // Do not use this bit!
    
    const PROTO_IP = 0;
    const PROTO_TCP = 6;
    const PROTO_UDP = 17;
    const PROTO_GRE = 47;
    const PROTO_ESP = 50;
    const PROTO_AH = 51;
    
    const PROTO_TYPE_TCP = 'tcp';
    const PROTO_TYPE_UDP = 'udp';
    const PROTO_TYPE_ANY = 'any';
    
    protected $group = '';
    protected $flags = 0;
    protected $proto = Sng_firewall_rule_class::PROTO_IP;
    protected $addr = '';
    protected $port = '';
    protected $param = '';
    
    private $rule_type = null;
    /**
     * Constructor for firewall rule config class (base class)
     * @param $node
     * @param $parent_name
     * @param $name
     */
    public function __construct($parent_name, $name, $node)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
        $this->rule_type = basename($parent_name);
    }
    public function configure()
    {
        $this->add_field('description', 'Description', 'string', '', 50);
        $this->add_field('rank', 'Rank', 'hidden', '', 5);
        $this->add_field('read_only', 'Read Only', 'hidden', 'false');
        $this->add_field('reference', 'Reference', 'hidden', array(), 10);
        return parent::configure();
    }
    public function protocol_enum(){
        return array('any'  => 'UDP + TCP', 'udp' => 'UDP', 'tcp' => 'TCP');
    }
    public function can_modify(&$output = null)
    {
        if($this->is_readonly()){
            $this->set_error('description', "This rule is readonly.", $output);
            return false;
        }
        return parent::can_modify($output);
    }
    public function validate($data, &$out_message = null){
        $tmp_rs = parent::validate($data, $out_message);
        if($tmp_rs){
            $path = $this->path();
            $parent_obj = $this->node()->software()->application()->find_module_by_path($path);
            //Each rule MUST be unique in its name space
            if(!$parent_obj->is_unique($this->rule_type, $this, $reason)){
                $rule_msgs = array('description' => $reason );
                $tmp_rs = false;
                $this->set_error('description',$rule_msgs['description'], $out_message);
            }else{
                $conflicts = $parent_obj->check_conflict($this->rule_type, $this);
                if($conflicts){
                    $rule_msgs = array('description' => "This " . $this->name() . " is in conflict with {$conflicts['name']} on {$conflicts['type']} {$conflicts['value']} " );
                    $tmp_rs = false;
                    $this->set_error('description',$rule_msgs['description'], $out_message);
                }
            }
        }
        return $tmp_rs;
    }
    //implement in extended classes
    public function have_conflict($object){
        return array();
    }
    
    /**
     * Return validated rule in packed format.
     *
     * Rule format, 7 fields with a pipe '|' delimiter:
     * name|group|flags|proto|addr|port|param
     *
     * @return string Valid rule in packed format
     */
    public function get_rule_value()
    {
        return array(sprintf("%s|%s|0x%08x|%d|%s|%s|%s",
                $this->name() . '_' . $this->proto, $this->group, $this->flags,
                $this->proto, $this->addr, $this->port, $this->param));
    }
    /**
     * Set rule group name.
     *
     * @param string $val Firewall rule group 
     * @return void
     */
    public function set_group($val = '')
    {
        $this->group = $val;
    }
    /**
     * Set rule flags 
     *
     * @param string $val Firewall rule flags
     * @return void
     */
    public function set_flags($val = '')
    {
        $this->flags = $val;
    }
    /**
     * Set rule Protocol
     *
     * @param string $val Firewall rule Protocol
     * @return void
     */
    public function set_protocol($val = '')
    {
        $this->proto = $val;
    }
    /**
     * Set rule address
     *
     * @param string $val Firewall rule address
     * @return void
     */
    public function set_address($val = '')
    {
         $this->addr = $val;
    }
    /**
     * Set rule port range.
     *
     * @param int $from from port
     * @param int $to to port
     * @return void
     */
    public function set_port_range($from, $to = '', $split_str = ":" )
    {
        $this->port = $this->get_port_str($from, $to , $split_str);
    }
    public function get_port_str($from, $to = '', $split_str = ":" )
    {
        if($to != ''){
            return trim($from) . $split_str . trim($to);
        }else{
            return trim($from);
        }
    }
    /**
     * Set rule Parameter.
     *
     * @param string $val Firewall rule Parameter
     * @return void
     */
    public function set_parameter($val)
    {
        $this->param = $val;
    }
    /**
     * Returns protocol flag for given protocol name.
     *
     * @param string $protocol protocol name
     * @return flag protocol flag
     */
    
    public function get_protocol_id($protocol)
    {
        switch ($protocol) {
            case "TCP":
                $protocolflag = Sng_firewall_rule_class::PROTO_TCP;
                break;
    
            case "UDP":
                $protocolflag = Sng_firewall_rule_class::PROTO_UDP;
                break;
    
            case "GRE":
                $protocolflag = Sng_firewall_rule_class::PROTO_GRE;
                break;
    
            case "ESP":
            case "ipv6-crypt":
                $protocolflag = Sng_firewall_rule_class::PROTO_ESP;
                break;
    
            case "AH":
            case "ipv6-auth":
                $protocolflag = Sng_firewall_rule_class::PROTO_AH;
                break;
        }
        return $protocolflag;
    }
    /**
     * Enable rule.
     *
     */
    public function enable_rule()
    {
        $this->flags |= Sng_firewall_rule_class::ENABLED;
    }
    /**
     * Disable rule.
     *
     */
    public function disable_rule()
    {
        $this->flags &= ~Sng_firewall_rule_class::ENABLED;
    }
    public function is_readonly(&$reason = null)
    {
        $readonly  = $this->get_data_value('read_only', false);
        if($readonly == 'true'){
            return true;
        }else{
            return parent::is_readonly($reason);
        }
    }
    public function can_dispose(&$reason = NULL)
    {
        if($this->is_readonly()){
            $reason = "{$this->display_name()} is readonly.";
            return false;
        }
        return parent::can_dispose($reason);
    }
    public function data_filter($data){
        //allow to update even the read_only is true
        if(isset($data['read_only']) && $data['read_only'] == 'true'){
            $data['read_only'] = 'false';
        }
        return $data;
    }
    
    public function display_name()
    {
        //hide the original obj name, display a readable name
        $tmp_rule_name = parent::display_name();
        $system_prefix = 'auto_';
        if(strpos($tmp_rule_name, $system_prefix) !== false){
            $path = $this->path();
            $parent_obj = $this->node()->software()->application()->find_module_by_path($path);
            $type_name = $parent_obj->aggregate_object_name($this->rule_type);
            $tmp_rule_name = substr($tmp_rule_name, strlen($system_prefix));
            list($module, $type, $obj, $field) = explode("__", $tmp_rule_name);
            if($type == 'configuration'){
                //for module configuration
                $field = $obj;
                $tmp_rule_name = " {$module}/{$type}/{$field}";
            }else{
                $tmp_rule_name = " {$module}/{$type}/{$obj}/{$field}";
            }
        }else{
            $tmp_description = $this->get_data_value('description');
            if($tmp_description){
                return $tmp_description;
            }
        }
        return $tmp_rule_name;
    }
    public function get_reference(){
        return $this->get_data_value('reference', false);
    }
    public function have_reference($reference_name){
        $reference_list =  $this->get_reference();
        if($reference_list && isset($reference_list[$reference_name])){
            return $reference_list[$reference_name];
        }
        return false;
    }
    public function add_reference($reference_name, $reference_data){
        $reference_list =  $this->get_reference();
        $reference_list[$reference_name] = $reference_data;
        $this->set_data_values(array('reference' => $reference_list));
        return true;
    }
    public function delete_reference($reference_name){
        $reference_list =  $this->get_reference();
        if($reference_list && isset($reference_list[$reference_name])){
            unset($reference_list[$reference_name]);
            $this->set_data_values(array('reference' => $reference_list));
        }
        return true;
    }
}
class Sng_firewall_port_rule_class extends Sng_firewall_rule_class
{
    const PORT_RULE_PROFIX = 'auto_';
    /**
     * Constructor for firewall rule config class 
     * @param $node
     * @param $parent_name
     * @param $name
     */
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        parent::configure();
        $this->add_enum_field('protocol', 'Protocol', 'dropdown', '', $this->protocol_enum());
        $this->set_field_help('protocol', 'What protocol to consider in the rule');
        
        $this->add_field('begin', 'Port', 'string', '', 10);
        $this->set_field_rules('begin', 'required|valid_port_number');
        
        $this->add_field('end', ' : ', 'string', '', 10);
        $this->set_field_rules('end','valid_port_number');
        $port_help_msg = array(
                array ('Port Begin' => array ('Starting port for rule.')),
                array ('Port end'   => array ('Last port for rule (optional)'))
        );
        $this->set_field_help('begin', $port_help_msg);
        $this->composite_layout('begin', array('end'));
        $this->register_aggregate_object('whitelist',
                array(
                        'name' => 'White List',
                        'dynamic' => true,
                        'base_path' => $this->object_name().'/whitelist',
                        'autoname' => true,
                        'methods' => array(
                                'create' => array(
                                        'name' => 'Create',
                                        'description' => 'Create a White list IP',
                                        'request' => 'POST',
                                ),
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a White list IP',
                                        'request' => 'GET',
                                ),
                                'update' => array(
                                        'name' => 'Update',
                                        'description' => 'Update a White list IP',
                                        'request' => 'POST',
                                ),
                                'delete' => array(
                                        'name' => 'Delete',
                                        'description' => 'Delete a White list IP',
                                        'request' => 'POST',
                                ),
                        ),
                )
        );
    }
    public function validate($data, &$output){
        $validate_result = true;
        if(isset($data['end']) && $data['end'] != ''){
            $port_begin = intval($data['begin']);
            $port_end = intval($data['end']);
            $rule_msgs = array('end' => 'Last port must greater than first port');
            if($port_begin >= $port_end ){
                $validate_result = false;
                $this->set_error('end',$rule_msgs['end'], $output);
            }
        }
        //check port must before check port conflict
        return $validate_result && parent::validate($data, $output);
    }
    function get_rule_value()
    {
        //port
        $port_begin  = $this->get_data_value('begin', false);
        $port_end  = $this->get_data_value('end', false);
        $this->set_port_range($port_begin, $port_end);
        //flag
        $this->set_flags(Sng_firewall_rule_class::INCOMING_ALLOW | Sng_firewall_rule_class::ENABLED);
        
        $protocol = $this->get_data_value('protocol', false);
        //white list
        $whitelist = $this->api_retrieve_whitelist();
        if($whitelist){
            $address_list = array();
            foreach($whitelist as $whitelist_obj){
                $address_list[] = $whitelist_obj->get_address();
            }
            if($address_list){
                $this->set_address(join(',', $address_list));
            }
        }
        $rules = array();
        if($protocol == 'any'){
            $this->set_protocol($this->get_protocol_id('TCP'));
            $rule_tcp = parent::get_rule_value();
            $this->set_protocol($this->get_protocol_id('UDP'));
            $rule_udp = parent::get_rule_value();
            $rules = array_merge($rule_tcp, $rule_udp);
        }else{
            $this->set_protocol($this->get_protocol_id(strtoupper($protocol)));
            $rules = parent::get_rule_value();
        }
        return $rules;
    }
    public function summary($type = 'horizontal' , $long = false){
        $port_begin  = $this->get_data_value('begin', false);
        $port_end  = $this->get_data_value('end', false);
        $this->set_port_range($port_begin, $port_end);
        if($this->is_readonly()){
            $icon = safe_icon('lock');
        }else{
            $icon = safe_icon('check');
        }
        
        $reference = $this->get_reference();
        if(!$reference){
            $tmp_description = $this->get_data_value('description');
            if($tmp_description == ''){
                $tmp_description = $this->name();
            }
        }else{
            $tmp_description = array();
            foreach($reference as $key => $reference_data){
                $tmp_description[] = $reference_data['desc'];
            }
        }
        $table_line = array(
                "Editable" => array( 'data' => $icon, 'formatter' => 'none'),
                "Description" => $tmp_description,
                "Protocol" => $this->get_data_value('protocol'),
                "Port" => $this->port
                );
        return array('data' => $table_line);
    }
    public function have_conflict($object){
        $conflict = array();
        //When user add a new port rule, it will check conflict with other user rules
        //pass by system obj auto added rules
        if(Sng_firewall_port_rule_class::PORT_RULE_PROFIX ==
                substr($this->name(), 0, strlen(Sng_firewall_port_rule_class::PORT_RULE_PROFIX))){
            return $conflict;
        }
        $this_port_begin    = $this->get_data_value('begin', false);
        $this_port_end      = $this->get_data_value('end', false);
        $this_protocol      = $this->get_data_value('protocol', false);
        $this_port_string   = $this->get_port_str($this_port_begin, $this_port_end);
        if($this_port_end == ''){
            $this_port_end = $this_port_begin;
        }
        $obj_port_begin    = $object->get_data_value('begin', false);
        $obj_port_end      = $object->get_data_value('end', false);
        $obj_protocol      = $object->get_data_value('protocol', false);
    
        if($obj_port_begin == ''){
            return $conflict;
        }
        if($obj_port_end == ''){
            $obj_port_end = $obj_port_begin;
        }
        if($this_protocol == $obj_protocol || $this_protocol == 'any' || $obj_protocol == 'any'){
            if(!($obj_port_begin > $this_port_end || $obj_port_end < $this_port_begin)){
                $conflict = array(
                        'name' => $this->name(),
                        'type' => $this->get_data_value('protocol') . ' port',
                        'value' => $this_port_string
                );
            }
        }
        return $conflict;
    }
    public function get_port_data(){
        return array(
                'begin' => $this->get_data_value('begin', false),
                'end' => $this->get_data_value('end', false),
                'protocol' => $this->get_data_value('protocol', false)
                );
    }
    public function get_reference_protocol(){
        //merge all reference protocol
        $reference_list = $this->get_reference();
        $reference_protocol = false;
        if($reference_list){
            foreach($reference_list as $reference_item){
                if(!$reference_protocol){
                    $reference_protocol = $reference_item['protocol'];
                }else{
                    if($reference_item['protocol'] && $reference_protocol != $reference_item['protocol']){
                        $reference_protocol = Sng_firewall_rule_class::PROTO_TYPE_ANY;
                    }
                }
            }
        }
        return $reference_protocol;
    }
    public function add_reference($reference_name, $reference_data){
        $rs = parent::add_reference($reference_name, $reference_data);
        if($rs){
            //merge all reference protocol(tcp + udp => any)
            $tmp_protocol = $this->get_reference_protocol();
            if($tmp_protocol && $tmp_protocol != $this->get_data_value('protocol', false)){
                $this->set_data_values(array('protocol' => $tmp_protocol));
            }
        }
        return $rs;
    }
    public function delete_reference($reference_name){
        $rs = parent::delete_reference($reference_name);
        if($rs){
            //need to reset the protocol
            $tmp_protocol = $this->get_reference_protocol();
            if($tmp_protocol && $tmp_protocol != $this->get_data_value('protocol', false)){
                $this->set_data_values(array('protocol' => $tmp_protocol));
            }
        }
        return $rs;
    }
    public function api_create_whitelist($name, $data=null) {
        $whitelist = new Sng_whitelist_class($this->node() , $this->object_name() . '/whitelist', $name);
        $whitelist->configure();
        if($data) {
            $whitelist->set_data_values($data);
        }
        return $whitelist;
    }
    public function api_retrieve_whitelist($name=null, $data=null) {
        $objs = $this->get_aggregate_objects('whitelist');
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    public function api_update_whitelist($name, $data=null, &$output = null) {
        $obj = $this->api_retrieve_whitelist($name);
        if($obj) {
            if ($obj->validate($data,$output)) {
                if (true == $obj->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    public function api_delete_whitelist($name, $data=null, &$output = null) {
        $obj = $this->api_retrieve_whitelist($name);
        if($obj) {
            if ($obj->can_dispose($output)) {
                return $obj->dispose();
            }
        }
        return false;
    }
}
class Sng_whitelist_class extends Safe_configurable_object_class
{
    private $_the_app = "";
    /**
     * @brief
     *
     * @param[in out] $node
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return
     */
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
        $this->_the_app = $node->software()->application();
    }
    public function configure()
    {
        $this->add_field('address', 'IP address', 'string', '', 50);
        $this->set_field_rules('address', "required|valid_ip");
        $this->add_field('prefix', '/', 'string', '', 5);
        $this->set_field_rules('prefix','integer|greater_than[0]');
        $this->composite_layout('address', array('prefix'));
        $this->set_field_help('address', ' The IP address/Network allow to access this port.');
        return parent::configure();
    }
    
    public function validate($data, &$output = null)
    {
        $result = parent::validate($data, $output);
        $prefix = $this->get_data_value('prefix');
        if($prefix){
            $prefix_error_msg = 'The prefix must contain a number less than';
            $address = $this->get_data_value('address');
            if(filter_var($address, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)){
                $max_prfix = 32;
            }else{
                $max_prfix = 128;
            }
            if($prefix > $max_prfix){
                $this->set_error('address', $prefix_error_msg . " {$max_prfix}. ", $output);
                $result =  false;
            }
        }
        $path = $this->object_name();
        $parent_obj = $this->node()->software()->application()->find_obj_by_path(dirname($path));
        if($parent_obj){
            $whitelist = $parent_obj->api_retrieve_whitelist();
            foreach($whitelist as $whitelist_obj){
                if($whitelist_obj->name() != $this->name()){
                    if($whitelist_obj->get_address() == $this->get_address()){
                        $this->set_error('address', "The address is conflict with {$whitelist_obj->name()} : {$whitelist_obj->get_address()} ", $output);
                        $result =  false;
                        break;
                    }
                }
            }
        }
        return $result;
    }
    public function get_address()
    {
        $address = $this->get_data_value('address') ;
        $prefix = $this->get_data_value('prefix');
        if($prefix){
            $address = $address . '/' . $prefix;
        }
        return $address;  
    }
    public function summary($type = 'horizontal' , $long = false)
    {
        $result =  array(
                'IP Address' => $this->get_address(),
        );
        return array('data' => $result);
    }

}
class Sng_firewall_ip_rule_class extends Sng_firewall_rule_class
{
    /**
     * Constructor for firewall ip rule config class
     * @param $node
     * @param $parent_name
     * @param $name
     */
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        parent::configure();
        //add address type: host  or network  
        $type_enum = array(
                    'false' => 'Host',
                    'true' => 'Network'
                );
        $this->add_enum_field('type', 'Type', 'dropdown', 'true', $type_enum);
        $this->add_field('address', 'Address', 'string', '', 40);
        $this->set_field_rules('address', 'required|valid_ip');
        $this->set_field_help('address', 'The host IP address or network address to be filtered.');
        $this->add_field('prefix', 'Prefix', 'string', '', 10);
        $this->set_field_rules('prefix','required[type]|integer|greater_than[0]|less_than[128]');
        $this->set_field_help('prefix', 'How many bits are used for the network prefix.');
        
        //if the type == 'false' then disable the prefix field 
        $this->create_group('network_prefix',
                array(
                        'prefix'
                )
        );
        $this->conditional_control('type', 'network_prefix');
    }
    function get_rule_value()
    {
        //flag
        $this->set_flags(Sng_firewall_rule_class::INCOMING_BLOCK | Sng_firewall_rule_class::ENABLED);
        //ip
        $ip = $this->get_data_value('address', false);
        $prefix = $this->get_data_value('prefix', false);
        if($prefix){
            $ip_address = $ip . "/" . $prefix;
        }else{
            $ip_address = $ip;
        }
        $this->set_address($ip_address);
        return  parent::get_rule_value();
    }
    public function summary($type = 'horizontal' , $long = false){
        $ip  = $this->get_data_value('address', false);
        $prefix  = $this->get_data_value('prefix', false);
        if($prefix){
            $ip_address = $ip . "/" . $prefix;
        }else{
            $ip_address = $ip;
        }
        if($this->is_readonly()){
            $icon = safe_icon('lock');
        }else{
            $icon = safe_icon('check');
        }
        $type = $this->get_data_value('type');
        $tmp_description = $this->get_data_value('description');
        if($tmp_description == ''){
            $tmp_description = $this->name();
        }
        $table_line = array(
                "" => array( 'data' => $icon, 'formatter' => 'none'),
                "Description" => $tmp_description,
                "IP address" => $ip_address
        );
        return array('data' => $table_line);
    }
    public function have_conflict($object){
        $conflict = array();
        
        $this_ip = $this->get_data_value('address', false);
        $this_prefix = $this->get_data_value('prefix', false);
        $obj_ip = $object->get_data_value('address', false);
        $obj_prefix = $object->get_data_value('prefix', false);
        
        //same IPv4 or IPv6
        if(safe_get_ip_type($this_ip) != safe_get_ip_type($obj_ip)){
           return $conflict;
        }else{
            //'4' or '6'
            $ip_type = safe_get_ip_type($this_ip);
        }
        
        if($this_prefix){
            $this_ip_address = $this_ip . "/" . $this_prefix;
        }else{
            $this_prefix = safe_get_ip_max_prefix($ip_type);
            $this_ip_address = $this_ip;
        }
        
        if($obj_prefix){
            $obj_ip_address = $obj_ip . "/" . $obj_prefix;
        }else{
            $obj_prefix = safe_get_ip_max_prefix($ip_type);
            $obj_ip_address = $obj_ip;
        }
        if($ip_type == SAFE_IPV4){
            $this_ip_range = safe_get_ip_range($this_ip, safe_cidr2netmask($this_prefix));
            $obj_ip_range = safe_get_ip_range($obj_ip, safe_cidr2netmask($obj_prefix));
        }else{
            $this_ip_range = safe_ipv6_to_range($this_ip, $this_prefix);
            $this_ip_range['network'] =  safe_ip2long6($this_ip_range[0]);
            $this_ip_range['broadcast'] =  safe_ip2long6($this_ip_range[1]);
            
            $obj_ip_range = safe_ipv6_to_range($obj_ip, $obj_prefix);
            $obj_ip_range['network'] =  safe_ip2long6($obj_ip_range[0]);
            $obj_ip_range['broadcast'] =  safe_ip2long6($obj_ip_range[1]);
        }
        if(!($obj_ip_range['network'] > $this_ip_range['broadcast'] || $obj_ip_range['broadcast'] < $this_ip_range['network'])){
            $conflict = array(
                    'name' => $this->name(),
                    'type' => 'IP',
                    'value' => $this_ip_address
            );
        }
        return $conflict;
        
    }
    
    public function validate($data, &$output = null){
        if(!isset($data['type'])){
            $ip_rule_type = $this->get_data_value('type', false);
        }else{
            $ip_rule_type = $data['type'];
        }
        if($ip_rule_type == 'false'){
            //if host IP, clear the prefix (data may submitted from rest api)
            $data['prefix'] = '';
        }else{
            //if ipv4 and network address, check prefix < 32
            if(!isset($data['address'])){
                $ip_rule_address = $this->get_data_value('address', false);
            }else{
                $ip_rule_address = $data['address'];
            }
            if(!isset($data['prefix'])){
                $ip_rule_prefix = $this->get_data_value('prefix', false);
            }else{
                $ip_rule_prefix = $data['prefix'];
            }
            $ip_type = safe_get_ip_type($ip_rule_address);
            if($ip_type == SAFE_IPV4){
                $max_prefix = safe_get_ip_max_prefix($ip_type);
                if($ip_rule_prefix >= $max_prefix){
                    $this->set_error('prefix',"Network prefix must be less than {$max_prefix}.", $output);
                }
            }
        }
        return parent::validate($data, $output);
    }
    
}
class Sng_firewall_forward_rule_class extends Sng_firewall_rule_class
{
    /**
     * Constructor for firewall forward rule config class
     * @param $node
     * @param $parent_name
     * @param $name
     */
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }    
    public function configure()
    {
        parent::configure();
        $this->add_enum_field('protocol', 'Protocol', 'dropdown', '', $this->protocol_enum());
        $this->set_field_help('protocol', 'What protocol to consider in the rule');

        $this->add_field('port/from/begin', 'From Port', 'string', '', 10);
        $this->set_field_rules('port/from/begin', 'required|valid_port_number');

        $this->add_field('port/from/end', ' : ', 'string', '', 10);
        $this->set_field_rules('port/from/end','valid_port_number');
        
        $from_port_help_msg = array(
                array ('From Port Begin' => array ('Forward from starting port for rule')),
                array ('From Port end'   => array ('Forward from last port for rule(optional)'))
        );
        $this->set_field_help('port/from/begin', $from_port_help_msg);
                
        $this->composite_layout('port/from/begin', array('port/from/end'));
        
        $this->add_field('port/to/begin', 'To Port', 'string', '', 10);
        $this->set_field_rules('port/to/begin', 'valid_port_number');
                
        $this->add_field('port/to/end', ' : ', 'string', '', 10);
        $this->set_field_rules('port/to/end','valid_port_number');
        $to_port_help_msg = array(
                array ('To Port Begin' => array ('Forward to starting port for rule(optional), if not set then forward to same port as from port.')),
                array ('To Port end'   => array ('Forward to last port for rule(optional)'))
        );
        $this->set_field_help('port/to/begin', $to_port_help_msg);
        
        $this->composite_layout('port/to/begin', array('port/to/end'));
        
        $this->add_field('to/ip', 'IP', 'string', '', 20);
        $this->set_field_rules('to/ip', 'required|valid_ip');
        $this->set_field_help('to/ip', 'The IP address to be forward');

        $this->add_enum_field('partial', 'Partial port forwarding', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('partial', 'Forward packets without changing the source address.<br><br>' .
            'This option may be useful if the forwarded IP needs to have the exact source address at the IP level. ' .
            'However, proper operation requires the default route of the forwarded IP to be pointing to the SBC. ' .
            '<strong>For normal port forwarding scenarios, this option should be disabled</strong>.');

    }
    public function validate($data, &$output){
        
        $validate_result = true;
        $new_data = $data;
        $rule_msgs = array('port_range_msg' => 'Last port must greater than first port',
                           'port_number_msg' => 'The number in to port range must be same in from port range');
        if($new_data['port/from/end'] != ''){
            $port_begin = intval($new_data['port/from/begin']);
            $port_end = intval($new_data['port/from/end']);
            if($port_begin >= $port_end ){
                $validate_result = false;
                $this->set_error('port/from/end',$rule_msgs['port_range_msg'], $output);
            }else{
                $from_port_number = $port_end - $port_begin;
            }
        }
        
        if($new_data['port/to/end'] != ''){
            $to_port_begin = intval($new_data['port/to/begin']);
            $to_port_end = intval($new_data['port/to/end']);
            if($to_port_begin >= $to_port_end ){
                $validate_result = false;
                $this->set_error('port/to/end',$rule_msgs['port_range_msg'], $output);
            }else{
                $to_port_number = $to_port_end - $to_port_begin;
            }
        }
        if($validate_result && ($new_data['port/from/end'] || $new_data['port/to/end'] != '')){
            if($from_port_number != $to_port_number){
                if($new_data['port/to/begin'] !='' || $new_data['port/to/end'] !=''){
                    $validate_result = false;
                    $this->set_error('port/to/end',$rule_msgs['port_number_msg'], $output);
                    $output['port/to/end'] = $rule_msgs['port_number_msg'];
                }
            }
        }
        //must validate the port rage before check conflict
        $parent_result = parent::validate($data, $output);
        if($validate_result && $parent_result && $data['port/to/begin'] == ''){
            $data['port/to/begin'] = $data['port/from/begin'];
            $data['port/to/end'] = $data['port/from/end'];
            $this->set_data_values($data);
        }
        return $validate_result && $parent_result;
    }
    function get_rule_value()
    {
        //set from port into parameter
        $from_port_begin  = $this->get_data_value('port/from/begin', false);
        $from_port_end  = $this->get_data_value('port/from/end', false);
        $this->set_port_range($from_port_begin, $from_port_end);
        //param (partial + dest range)
        $partial = $this->get_data_value('partial', false);
        $this->set_parameter(($partial == 'true' ? '!' : '') . $this->port);
        //set to port into port
        $to_port_begin  = $this->get_data_value('port/to/begin', false);
        $to_port_end  = $this->get_data_value('port/to/end', false);
        $this->set_port_range($to_port_begin, $to_port_end, '-');
        
        //ip
        $ip = $this->get_data_value('to/ip', false);
        $this->set_address($ip);
        //flag
        $this->set_flags(Sng_firewall_rule_class::FORWARD | Sng_firewall_rule_class::ENABLED);
    
        $protocol = $this->get_data_value('protocol', false);
        $rules = array();
        if($protocol == 'any'){
            $this->set_protocol($this->get_protocol_id('TCP'));
            $rule_tcp = parent::get_rule_value();
            $this->set_protocol($this->get_protocol_id('UDP'));
            $rule_udp = parent::get_rule_value();
            $rules = array_merge($rule_tcp, $rule_udp);
        }else{
            $this->set_protocol($this->get_protocol_id(strtoupper($protocol)));
            $rules = parent::get_rule_value();
        }
        return $rules;
    }
    public function summary($type = 'horizontal' , $long = false){
        $port_begin  = $this->get_data_value('port/from/begin', false);
        $port_end  = $this->get_data_value('port/from/end', false);
        $this->set_port_range($port_begin, $port_end);
        $from_port = $this->port;
        
        $to_port_begin  = $this->get_data_value('port/to/begin', false);
        $to_port_end  = $this->get_data_value('port/to/end', false);
        $this->set_port_range($to_port_begin, $to_port_end);
        $to_port = $this->port;
        
        if($this->is_readonly()){
            $icon = safe_icon('lock');
        }else{
            $icon = safe_icon('check');
        }
        $tmp_description = $this->get_data_value('description');
        if($tmp_description == ''){
            $tmp_description = $this->name();
        }
        $table_line = array(
                "" => array( 'data' => $icon, 'formatter' => 'none'),
                "Description" => $tmp_description,
                "Protocol" => $this->get_data_value('protocol'),
                "From Port" => $from_port,
                "To Port" => $to_port,
                "To IP" => $this->get_data_value('to/ip')
        );
        return array('data' => $table_line);
    }
    public function have_conflict($object){
        $conflict = array();
        $this_from_port_begin    = $this->get_data_value('port/from/begin', false);
        $this_from_port_end      = $this->get_data_value('port/from/end', false);
        $this_to_port_begin    = $this->get_data_value('port/to/begin', false);
        $this_to_port_end      = $this->get_data_value('port/to/end', false);
        $this_protocol      = $this->get_data_value('protocol', false);
        $this_from_port_string   = $this->get_port_str($this_from_port_begin, $this_from_port_end);
        $this_to_port_string   = $this->get_port_str($this_to_port_begin, $this_to_port_end, '-');
        if($this_from_port_end == ''){
            $this_from_port_end = $this_from_port_begin;
        }
        if($this_to_port_end == ''){
            $this_to_port_end = $this_to_port_begin;
        }
        $this_to_ip = $this->get_data_value('to/ip');
        
        $obj_from_port_begin    = $object->get_data_value('port/from/begin', false);
        $obj_from_port_end      = $object->get_data_value('port/from/end', false);
        $obj_to_port_begin    = $object->get_data_value('port/to/begin', false);
        $obj_to_port_end      = $object->get_data_value('port/to/end', false);
        $obj_protocol      = $object->get_data_value('protocol', false);
        $obj_from_port_string   = $object->get_port_str($obj_from_port_begin, $obj_from_port_end);
        $obj_to_port_string   = $object->get_port_str($obj_to_port_begin, $obj_to_port_end);
        if($obj_from_port_end == ''){
            $obj_from_port_end = $obj_from_port_begin;
        }
        if($obj_to_port_end == ''){
            $obj_to_port_end = $obj_to_port_begin;
        }
        $obj_to_ip = $object->get_data_value('to/ip');
    
        if($this_protocol == $obj_protocol || $this_protocol == 'any' || $obj_protocol == 'any'){
            if(!($obj_from_port_begin > $this_from_port_end || $obj_from_port_end < $this_from_port_begin)){
                $conflict = array(
                        'name' => $this->name(),
                        'type' => $this->get_data_value('protocol') . ' forward from port',
                        'value' => $this_from_port_string
                );
            }
        }
        return $conflict;
    }
}
class Sng_firewall_service_class extends Safe_service_class
{
    private $firewall_rules_file = '/etc/firewall.d/firewall.rules';
    public function __construct($software, $name, $process_name = null)
    {
        parent::__construct($software, $name, $process_name);
    }
    public function configure()
    {
        // Create the config object
        $cfg = new Sng_firewall_config_class($this->node(), $this->path() . '/' . $this->name());
        // Synch with DB
        $cfg->configure();
        $cfg->synch();
        // Attach config to module
        $this->set_config($cfg);
        // Register the configuration as being pseudo so not exposed to rest
        $this->register_aggregate_object('configuration', 
            array(
                'pseudo' => true,
                'configurable' => false
            )
        );
        $this->register_aggregate_object('port',
                array(
                        'name' => 'Open Port',
                        'dynamic' => true,
                        'base_path' => parent::object_name().'/port',
                        'has_child' => true,
                        'controller_url' => '/SAFe/sng_firewall_config',
                        'methods' => array(
                                'create' => array(
                                        'name' => 'Create',
                                        'description' => 'Create a port rule',
                                        'request' => 'POST',
                                ),
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a port rule',
                                        'request' => 'GET',
                                ),
                                'update' => array(
                                        'name' => 'Update',
                                        'description' => 'Update a port rule',
                                        'request' => 'POST',
                                ),
                                'delete' => array(
                                        'name' => 'Delete',
                                        'description' => 'Delete a port rule',
                                        'request' => 'POST',
                                ),
                        ),
                )
        );
        $this->register_aggregate_object('ip',
                array(
                        'name' => 'Blocked IP',
                        'dynamic' => true,
                        'base_path' => parent::object_name().'/ip',
                        'methods' => array(
                                'create' => array(
                                        'name' => 'Create',
                                        'description' => 'Create a IP rule',
                                        'request' => 'POST',
                                ),
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a IP rule',
                                        'request' => 'GET',
                                ),
                                'update' => array(
                                        'name' => 'Update',
                                        'description' => 'Update a IP rule',
                                        'request' => 'POST',
                                ),
                                'delete' => array(
                                        'name' => 'Delete',
                                        'description' => 'Delete a IP rule',
                                        'request' => 'POST',
                                ),
                        ),
                )
        );
        $this->register_aggregate_object('forward',
                array(
                        'name' => 'Port Forward',
                        'dynamic' => true,
                        'base_path' => parent::object_name().'/forward',
                        'methods' => array(
                                'create' => array(
                                        'name' => 'Create',
                                        'description' => 'Create a forward rule',
                                        'request' => 'POST',
                                ),
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a forward rule',
                                        'request' => 'GET',
                                ),
                                'update' => array(
                                        'name' => 'Update',
                                        'description' => 'Update a forward rule',
                                        'request' => 'POST',
                                ),
                                'delete' => array(
                                        'name' => 'Delete',
                                        'description' => 'Delete a forward rule',
                                        'request' => 'POST',
                                ),
                        ),
                )
        );

        return parent::configure();
    }
    public function can_restore($info, &$reason)
    {
        // Prevent calling parent as service status will be checked
        return true;
    }
    public function handle_global_messages(&$msg_list){
      	if($this->status()!=Safe_service_class::STATUS_RUNNING){
	    $msg_list[] = array(
 			'module'=>'firewall',
			'cache_tag'=>'firewall_stopped',
			'global_errors'=>array(
				'message'=>'The IP Firewall service is stopped.',
				'buttons'=>array(array(
					'label'=>'Start',
					'url'=>'/SAFe/sng_control_panel/start_confirm/firewall',
					'onclick'=>'safe_handle_confirm_button(this);'
				))
			)
	    );
	}
    }
    // Overloaded service status to check for running file created by firewall
    public function status(&$stats = null)
    {
        if (!isset($stats)) $stats = array();
        $extra_firewall_state = '/var/run/firewall.state';
        if($this->node()->os()->check_file_exists($extra_firewall_state)){
            // state file with additional information for extra states
            $statusdata = trim(file_get_contents($extra_firewall_state, false, NULL, 0, 128));
            if ($statusdata == "started") {
               $status = Safe_service_class::STATUS_RUNNING;
            } else if ($statusdata == "stopped") {
               $status = Safe_service_class::STATUS_STOPPED;
            } else if ($statusdata == "panic") {
               $status = Safe_service_class::STATUS_ERROR;
            } else {
               $status = Safe_service_class::STATUS_DEAD;
            }
        } else {
            $name = strtolower($this->node()->software()->application()->name());
            $stat_file = $this->node()->os()->stat_file('/var/run/' . $name . '.firewall.running', $stats);
            // general running file (started/stopped states)
            if ($stat_file){
                $status = Safe_service_class::STATUS_RUNNING;
            } else {
                $status = Safe_service_class::STATUS_STOPPED;
            }
        }
        if (isset($stats)) {
            $stats['process'] = 'N/A';
            if (isset($stat_file[9])) {
                $uptime = time() - $stat_file[9];
                $days = '';
                if ($uptime > 24 * 60 * 60) {
                    $days = (int)($uptime / (24 * 60 * 60));
                    $days = ((string)$days) . '-';
                    $uptime = $uptime - ($days * (24 * 60 * 60));
                }
                $uptime = date('H:i:s', $uptime);
                $stats['cpu']['time'] = $days . $uptime;
            }
        }
        return $status;
    }
    /**
     * @brief Generate sshd configuration
     *
     * @param[in out] $config_manager
     *
     * @return true on success
     */
    public function generate_config(&$config_manager)
    {
        //Generates service configuration files
        if (!$this->_generate_service_config($config_manager)) return false;
        // Invoke parent generate
        return parent::generate_config($config_manager);
    }
    public function _generate_service_config(&$config_manager)
    {
        $data= $this->config()->get_data_values(false);
        // Get our configuration data (unresolved)
        //$firewall_conf_file  = '/etc/firewall';
        $firewall_extif_file  = '/etc/firewall.d/firewall.extif';
        $firewall_extif = 'EXTIF="'.$data['extif'].'"';
        if($this->node()->os()->check_file_exists($firewall_extif_file)){
            $config_manager->add_config(new Safe_configuration_class($firewall_extif_file, '', Safe_configuration_class::CFG_DELETE,Safe_configuration_class::CFG_FILE));
        }
        $config_manager->add_config(new Safe_configuration_class($firewall_extif_file, $firewall_extif, Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_FILE));
        $config_manager->add_config(new Safe_configuration_class($firewall_extif_file, 'root', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
        
        $overwrite = true;
        $firewall_rules_file  = $this->firewall_rules_file;
        //get all rules
        $buffer = "RULES=\"\\\n";
        foreach ($this->has_aggregate_objects(false, true) as $_type => $_definition) {
            $rule_objs = $this->get_aggregate_objects($_type);
            if($rule_objs){
                foreach($rule_objs as $rule_obj){
                    if($_type == 'port'){
                        $tmp_begin_port = $rule_obj->get_data_value('begin', false);
                        if(empty($tmp_begin_port)){
                            $config_manager->add_error(array(
                                    'module' => 'firewall',
                                    'obj_type' => 'port',
                                    'obj_name' => $rule_obj->name(),
                                    'description' => 'The port field is required.'
                            ));
                        }
                    }
                    $rules = $rule_obj->get_rule_value();
                    if($rules){
                        foreach($rules as $rule_string){
                            $buffer .= sprintf("\t%s \\\n", $rule_string);
                        }
                    }
                }
            }else{
                //if no port rules, do not overwrite the old firewall.rules file
                if($_type == 'port'){
                    $overwrite = false;
                }
            }
        }
        $buffer .= '"';
        if($overwrite){
            if($this->node()->os()->check_file_exists($firewall_rules_file)){
                $config_manager->add_config(new Safe_configuration_class($firewall_rules_file, '', Safe_configuration_class::CFG_DELETE,Safe_configuration_class::CFG_FILE));
            }
            $config_manager->add_config(new Safe_configuration_class($firewall_rules_file, $buffer, Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_FILE));
            $config_manager->add_config(new Safe_configuration_class($firewall_rules_file, 'root', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
        }
        return true;
    }
    
    //check if firewall rule file is empty
    public function is_rule_file_empty()
    {
        $rule_data = '';
        if($this->node()->os()->read_file($this->firewall_rules_file, $rule_data)){
            if($rule_data){
                $rs = str_replace(array("RULES=\"\\\n",'"',"\n") , array('','',''), $rule_data);
                if($rs){
                    return false;
                }
            }
        }
        return true;
    }
    public function start(&$result = null)
    {
        if($this->is_rule_file_empty()){
            $result[] = 'Firewall rule not found.';
            return -1;
        }else{
            return parent::start($result);
        }
    }
    public function restart(&$result = null)
    {
        if($this->is_rule_file_empty()){
            $result[] = 'Firewall rule not found.';
            return -1;
        }else{
            return parent::restart($result);
        }
    }
    /**
     * @brief Invoked after a successfull write_config
     *
     * @return
     */
    public function post_write_config($obj_type=null)
    {
        // restart service
        if($this->status() == Safe_service_class::STATUS_RUNNING){
            $this->restart();
        }
        return parent::post_write_config($obj_type);
    }
    public function support_reload(&$need_reload = null) {
        parent::support_reload($need_reload);
        return true;
    }
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        return $this->generate_config($config_manager);
    }
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_create_port($name, $data=null) {
        $rule = new Sng_firewall_port_rule_class($this->node(), parent::object_name() . '/port', $name);
        $rule->configure();
        if($data) {
            $rule->set_data_values($data);
        }
        return $rule;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_retrieve_port($name, $data=null, &$output = null) {
        $rules = $this->get_aggregate_objects('port');
        return $rules[$name];
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_update_port($name, $data=null, &$output = null) {
        $rule = $this->api_retrieve_port($name);
        if($rule) {
            if ($rule->validate($data,$output)) {
                if (true == $rule->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_delete_port($name, $data=null, &$output = null) {
        $rule = $this->api_retrieve_port($name);
        if($rule) {
            if($rule->can_dispose($output)) {
                return $rule->dispose();
            }
        }
        return false;
    }
    
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_create_ip($name, $data=null) {
        $rule = new Sng_firewall_ip_rule_class($this->node(), parent::object_name() . '/ip', $name);
        $rule->configure();
        if($data) {
            $rule->set_data_values($data);
        }
        return $rule;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_retrieve_ip($name, $data=null, &$output = null) {
        $rules = $this->get_aggregate_objects('ip');
        return $rules[$name];
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_update_ip($name, $data=null, &$output = null) {
        $rule = $this->api_retrieve_ip($name);
        if($rule) {
            if ($rule->validate($data,$output)) {
                if (true == $rule->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_delete_ip($name, $data=null, &$output = null) {
        $rule = $this->api_retrieve_ip($name);
        if($rule) {
            if($rule->can_dispose($output)) {
                return $rule->dispose();
            }
        }
        return false;
    }
    
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_create_forward($name, $data=null) {
        $rule = new Sng_firewall_forward_rule_class($this->node(), parent::object_name() . '/forward', $name);
        $rule->configure();
        if($data) {
            $rule->set_data_values($data);
        }
        return $rule;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_retrieve_forward($name, $data=null, &$output = null) {
        $rules = $this->get_aggregate_objects('forward');
        return $rules[$name];
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_update_forward($name, $data=null, &$output = null) {
        $rule = $this->api_retrieve_forward($name);
        if($rule) {
            if ($rule->validate($data,$output)) {
                if (true == $rule->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     *
     * @return
     */
    public function api_delete_forward($name, $data=null, &$output = null) {
        $rule = $this->api_retrieve_forward($name);
        if($rule) {
            if($rule->can_dispose($output)) {
                return $rule->dispose();
            }
        }
        return false;
    }
    public function is_unique($obj_type, $object, &$reason) {
        $rule_objs = $this->get_aggregate_objects($obj_type);
        if($rule_objs){
            foreach($rule_objs as $rule_name => $rule){
                if($rule_name != $object->name()){
                    if($rule->compare($object) === 0){
                        $reason = 'This rule is duplicated with ' . $rule_name ;
                        return false;
                    }
                }
            }
        }
        return true;
    }
    public function check_conflict($obj_type, $object) {
        $conflict = array();
        $rule_objs = $this->get_aggregate_objects($obj_type);
        if($rule_objs){
            foreach($rule_objs as $rule_name => $rule){
                if($rule_name != $object->name()){
                    $conflict = $rule->have_conflict($object);
                    if($conflict){
                        return $conflict;
                    }
                }
            }
        }
        return $conflict;
    }
    public function check_protocol($protocol){
        $protocol_array= array( 'udp' => Sng_firewall_rule_class::PROTO_TYPE_UDP,
                                'tcp' => Sng_firewall_rule_class::PROTO_TYPE_TCP,
                                'udptcp' => Sng_firewall_rule_class::PROTO_TYPE_ANY,
                                'udptcptls' => Sng_firewall_rule_class::PROTO_TYPE_ANY,
                                'tls'=> Sng_firewall_rule_class::PROTO_TYPE_TCP 
                               );
        $protocol = strtolower($protocol);
        if(isset($protocol_array[$protocol])){
            return $protocol_array[$protocol];
        }else{
            safe_log(SAFE_LOG_ERROR,  "Protocol not found : {$protocol} ");
            return '';
        }
    }
    /**
     * If an obj is modified, and the obj have ports to listen
     * on, it will tigger the firewall update the port rules.
     * If the port match one of the existing system rules(not include the user rules), 
     * then add a reference on the rule, else create a new system rule and add the reference
     * 
     *  [reference] => (
     *    [sip/profile/NSC_SIP_Profile1] => stdClass Object
     *        (
     *         [desc] => SIP Profile NSC_SIP_Profile1 Port
     *		   [field] => sip-port
     *		   [port] => 5061
     *		   [protocol] => tcp
     *        )
     *   )
     *  
     *  if the obj was deleted or the port was changed, remove the reference from this system rule
     *  if all references on this system rule was removed then delete this rule
     */
    public function check_object_modified($obj, $src_module_name, $obj_type, $obj_name, $sub_type = null, $sub_name = null)
    {
        $modules = array('sip', 'webconfig', 'sshd', 'snmpd','sngms');
        if(in_array($src_module_name, $modules)){
            $rules = $this->get_aggregate_objects('port');
            
            $modified_obj_ports = array();
            $obj_status = $obj->status();
            $modified_obj_ports = $obj->get_port();
            
            $reference_name = $src_module_name . '/' . $obj_type . '/' . $obj_name;
            //find out which rule have this reference
            $rules_have_reference = array();
            $rules_have_port = array();
            if($rules){
                foreach($rules as $rule_obj){
                    //only check system obj added rules
                    if(Sng_firewall_port_rule_class::PORT_RULE_PROFIX != 
                            substr($rule_obj->name(), 0, strlen(Sng_firewall_port_rule_class::PORT_RULE_PROFIX))){
                        continue;
                    }
                    //get all rules have this obj reference
                    if($rule_obj->have_reference($reference_name)){
                        $rules_have_reference[$rule_obj->name()] = $rule_obj;
                    }
                    
                    //get all rules have port matched with this obj ports
                    //if obj was DELETED no need to check port(just delete the reference)
                    if($obj_status != Safe_object_serializer_class::OBJ_STATUS_DELETED){
                        foreach($modified_obj_ports as $key => $tmp_port_data){
                            $rule_obj_port= $rule_obj->get_port_data();
                            //check if it is a port range
                            if($tmp_port_data['port_end'] == $tmp_port_data['port_begin']){
                                $tmp_port_data['port_end'] = '';
                            }
                            if($rule_obj_port['begin'] == $tmp_port_data['port_begin'] && $rule_obj_port['end'] == $tmp_port_data['port_end']){
                                $rules_have_port[$key] = $rule_obj;
                            }
                        }    
                    }
                }
                //if obj is not DELETED and have port matched then add or update the reference
                if($obj_status != Safe_object_serializer_class::OBJ_STATUS_DELETED  && $rules_have_port){
                    foreach($rules_have_port as $key => $port_rule){
                        $reference_data = $modified_obj_ports[$key];
                        $reference_data['protocol'] = $this->check_protocol($modified_obj_ports[$key]['protocol']);
                        $port_rule->add_reference($reference_name, $reference_data);
                        $port_rule->save();
                        if(isset($rules_have_reference[$port_rule->name()])){
                            //no need to delete the reference from this rule 
                            unset($rules_have_reference[$port_rule->name()]);
                        }
                        if(isset($modified_obj_ports[$key])){
                            //no need to add new rule for this port
                             unset($modified_obj_ports[$key]);   
                        }

                        $tmp_rule_name = Sng_firewall_port_rule_class::PORT_RULE_PROFIX .  'firewall_rule_' .  $reference_data['protocol'] .  '_' . $reference_data['port_begin'] ;
                        /* If the rule name is different re-create the rule with new name need for HA sync */
                        if($tmp_rule_name != $port_rule->name()){
                            $port_rule_new = $this->api_create_port($tmp_rule_name, $port_rule->get_data_values());
                            $port_rule_new->save();
                            $whitelist = $port_rule->api_retrieve_whitelist();
                            foreach($whitelist as $whitelist_obj){
                                $tmp_whitelist_name = $port_rule_new->build_unique_name('whitelist');
                                $new_whitelist_obj = $port_rule_new->api_create_whitelist($tmp_whitelist_name, $whitelist_obj->get_data_values());
                                $new_whitelist_obj->save();
                            }
                            /* Now delete the rule with old name */
                            $whitelist_del = $port_rule->api_retrieve_whitelist();
                            if($whitelist_del){
                                foreach($whitelist_del as $whitelist_obj){
                                    $port_rule->api_delete_whitelist($whitelist_obj->name());
                                }
                            }
                            $port_rule->dispose();
                        }
                    }    
                }
                //if have reference but no port match then delete the reference
                if($rules_have_reference){
                    foreach($rules_have_reference as $rule_obj){
                        $rule_obj->delete_reference($reference_name);
                        //if reference is empty , delete the rule
                        if(!$rule_obj->get_reference()){
                            $whitelist_del = $rule_obj->api_retrieve_whitelist();
                            if($whitelist_del){
                                foreach($whitelist_del as $whitelist_obj){
                                    $rule_obj->api_delete_whitelist($whitelist_obj->name());
                                }
                            }
                            $rule_obj->dispose();
                        }else{
                            $rule_obj->save();
                        }
                    }
                }
            }
            
            //if port did not match any rule yet, create a new rule
            if($obj_status != Safe_object_serializer_class::OBJ_STATUS_DELETED){
                foreach($modified_obj_ports as $key => $tmp_port_data){
                    //add new rule
                    $tmp_dst_port_data = array(
                            'begin' => $tmp_port_data['port_begin'],
                            'end' => $tmp_port_data['port_end'],
                            'protocol' => $this->check_protocol($tmp_port_data['protocol'])
                    );
                    if($tmp_port_data['port_end'] == $tmp_port_data['port_begin']){
                        $tmp_dst_port_data['end'] = '';
                    }
                    if(empty($tmp_dst_port_data['begin'])){
                        //if port begin is empty, pass by this rule
                        continue;
                    }
                    //$tmp_rule_name = Sng_firewall_port_rule_class::PORT_RULE_PROFIX . $this->build_unique_name('firewall_rule');
                    $tmp_rule_name =
                        Sng_firewall_port_rule_class::PORT_RULE_PROFIX .  'firewall_rule_' . $tmp_dst_port_data['protocol'] . '_' . $tmp_dst_port_data['begin'] ;
                    $rule_obj = $this->api_create_port($tmp_rule_name, $tmp_dst_port_data);
                    //add reference to rule
                    $reference_data = $tmp_port_data;
                    $reference_data['protocol'] = $this->check_protocol($tmp_port_data['protocol']);
                    $rule_obj->add_reference($reference_name, $reference_data);
                    $rule_obj->set_data_values(array('read_only' => 'true'));
                    $rule_obj->save();
                }
            }
        }else{
            // IP changes need to restart firewall
            if($src_module_name == 'network' && 'ip' == $obj_type){
                $this->config()->mark_modified();
            }
        }
        return parent::check_object_modified($obj, $src_module_name, $obj_type, $obj_name, $sub_type, $sub_name);
    }
    /*
     * overwrite the base service start function for firewall, check if the script return error
     */
    public function api_start_service($obj_name=null, $params=null, &$output=null)
    {
        $rs = parent::api_start_service($obj_name, $params, $output);
        if($rs){
            //wait for firewall start
            sleep(3);
            //check status
            $stats = $this->api_status_service($obj_name);
            if($stats){
                if( $stats['status'] == Safe_service_class::STATUS_RUNNING){
                    return true;
                }else{
                    $output = array($stats['status_text']);
                }
            }
        }
        return false;
    }
}
/* End of file sng_firewall_service_class.php */
