<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG httpd(webconfig) service wrapper
 *
 * @author William Adam
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('service');

class Sng_httpd_config_class extends Safe_configurable_object_class
{
    const HTTP = 'http';
    const HTTPS = 'https';
    const HTTP_HTTPS = 'http_https';
    
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, 'configuration', $node);
    }
    public function configure()
    {
        // Default interface name
	    $def_if = 'all';
	    $if_enum = array();
	    $if_enum['all'] = 'All interfaces';
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $if_enum = array_merge($if_enum, $network_module->interfaces_ip(Sng_network_interface_class::MANAGEMENT));
        
            
        $this->add_enum_field('interface', 'Network Interface', 'dropdown', '', $if_enum);
        $this->set_field_help('interface', 'Select interface that web server listens on.');

        $this->add_field('http_port', 'HTTP port', 'text', '80',10);
        $this->set_field_rules('http_port', 'required|valid_port_number|less_than[61000]|not_matches[https_port]|different_than[81]|callback_port_check[http_port]');
        $this->set_field_help('http_port', 'Port number for HTTP protocol.');
        
        $this->add_field('https_port', 'HTTPS port', 'text', '443',10);
        $this->set_field_rules('https_port', 'required|valid_port_number|less_than[61000]|not_matches[http_port]|different_than[81]|callback_port_check[https_port]');
        $this->set_field_help('https_port', 'Port number for HTTPS protocol.');
        $port_type_enum = array(Sng_httpd_config_class::HTTP_HTTPS => 'HTTP + HTTPS',
                                Sng_httpd_config_class::HTTP =>'HTTP',
                                Sng_httpd_config_class::HTTPS => 'HTTPS');
        $this->add_enum_field('protocol', 'Protocol', 'dropdown', Sng_httpd_config_class::HTTP_HTTPS, $port_type_enum);
        $this->set_field_help('protocol', array(
                                                 array('HTTP + HTTPS' => 'Enable both HTTP protocol and HTTPS protocol.'),
                                                 array('HTTP' => 'Only enable HTTP protocol.'),
                                                 array('HTTPS' => 'Only enable HTTPS protocol.')
                                            )
                                );

        return parent::configure();
    }
    public function description()
    {
        return parent::description();
    }
    public function port_check($field_name = null){
        if($field_name){
            $port_name = $this->get_data_label($field_name);
            $port_value = $this->get_data_value($field_name);
            $port_type = 'TCP';
        }else{
            return true;
        }
        //adapters interfaces name to ip address
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $if_name = $this->get_data_value('interface',false);
        $ip_obj = $network_module->get_ip_by_name($if_name);
        if($ip_obj){
            $ip =  $ip_obj->address();
        }else{
            $ip = 'all';
        }
        //ip port
        $ports[] = array('name' =>$port_name,'port'=>$port_value, 'ip'=>$ip, 'type' => $port_type);
    
        //check $conflicts
        $conflicts = $this->node()->software()->application()->check_resource_conflict('webconfig','IP_PORT', $ports);
    
        foreach($conflicts as $conflict){
            $mod = $this->node()->software()->application()->find_module_by_name($conflict['name']);
            $name = $mod->description();
            $error_msg = $error_msg . " with {$name} at {$conflict['desc']}, " ;
        }
    
        $error_msg_2 ='';
        //check system port
        foreach($ports as $port_item){
            if(!$this->node()->os()->valid_ip_port($port_item['ip'], $port_item['port'], $port_item['type'], 'webconfig', $prog_name)){
                $error_msg_2 = " Port used by ".$prog_name;
            }
        }
        if($error_msg == '' && $error_msg_2 == ''){
            return true;
        }else{
            if ($error_msg!=''){
                $error['port_check'] = 'Port conflict'.substr($error_msg, 0, -2).'. ';
            }
            if ($error_msg_2!=''){
                $error['port_check'] = $error['port_check'] . $error_msg_2 .' ';
            }
            return $error;
        }
    }
    /**
     * @brief Save method - Override to send obj modified event
     *
     * @return
     */
    public function save()
    {
        $result = parent::save();
        if($result){
            //send modification event out on save
            //when update from 2.1 to 2.2 even  is_modified() == false, but still need to 
            //tigger the firewall module to create port rule 
            $path_info = $this->path_info();
            $receivers = $this->node()->software()->application()->check_object_modified($this, $path_info['module'], $path_info['type'], $this->name());
        }
        return $result;
    }
    public function is_enabled_protocol($type = Sng_httpd_config_class::HTTP)
    {
        $protocol = $this->get_data_value('protocol', false);
        if($protocol == Sng_httpd_config_class::HTTP_HTTPS || $protocol == $type){
            return true;
        }
        return false;
    }
    /**
     * @brief get_port method - return the port used in this config
     *
     * @return
     */
    public function get_port()
    {
        $port = array();
        $module = $this->node()->software()->application()->find_module_by_path($this->object_name());
        $desc = '';
        if($module){
            $path_info = $this->path_info();
            $desc = $module->description() . ' ' . $path_info['type'];
        }
        $protocol = $this->get_data_value('protocol', false);
        //to open the port on firewall
        if($this->is_enabled_protocol(Sng_httpd_config_class::HTTP)){
            $port[] = array(
                        'protocol' => 'tcp', 
                        'field'=> 'http_port', 
                        'port_begin' => $this->get_data_value('http_port', false),
                        'port_end' => $this->get_data_value('http_port', false),
                        'desc' => $desc . ' ' . $this->get_data_label('http_port'));
        }
        if($this->is_enabled_protocol(Sng_httpd_config_class::HTTPS)){
            $port[] = array(
                        'protocol' => 'tcp', 
                        'field'=> 'https_port', 
                        'port_begin' => $this->get_data_value('https_port', false),
                        'port_end' => $this->get_data_value('https_port', false),
                        'desc' => $desc . ' ' .$this->get_data_label('https_port'));
        }
        return $port;
    }
}
class Sng_httpd_service_class extends Safe_service_class
{
    public function __construct($software)
    {
        parent::__construct($software, 'webconfig');
    }
    /**
     * @brief
     *
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("Web Server");
        // Create the config object
        //pass the module obj to config obj, when loading modules, the find_module_by_name() does not work
        $cfg = new Sng_httpd_config_class($this->node(), $this->object_name(),$this->name());
        // Synch with DB
        $cfg->configure();
        $cfg->synch();
        // Attach config to module
        $this->set_config($cfg);
        //$cfg->set_parent($this);
        if($cfg->is_modified()){
            $path_info = $cfg->path_info();
            $receivers = $this->node()->software()->application()->check_object_modified($cfg, $this->name(), $path_info['type'], $cfg->name());
        }
        
        
        return parent::configure();
    }
    public function can_restore($info, &$reason)
    {
        // Prevent calling parent as service status will be checked
        return true;
    }
    public function allow_user_ctl()
    {
        // User not allowed to control service
        return 'hide';
    }
    /**
     * @brief Generate httpd configuration
     *
     * @param[in out] $config_manager
     *
     * @return true on success
     */
    public function generate_config(&$config_manager)
    {
        //Generates service configuration files
        if (!$this->_generate_service_config($config_manager)) return false;
        // Invoke parent generate
        return parent::generate_config($config_manager);
    }

    private function interface_to_ip($interface)
    {
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $ip_obj = $network_module->get_ip_by_name($interface);
        if($ip_obj){
            $ip =  $ip_obj->address();
        }else{
            $ip = 'all';
        }
        return $ip;
    }
    public function check_resource_conflict($src_module_name, $resource_type, $resource_data)
    {
        // IP_PORT conflicts ?
        $conflicts = array();
        if('IP_PORT' == $resource_type){
            // get our configuration
            $data = $this->config()->get_data_values(false);
            $ip = $this->interface_to_ip($data['interface']);
            $ports[] = array('name' =>$this->name(),'port'=>$data['http_port'], 'ip'=>$ip, 'type' => 'TCP');
            $ports[] = array('name' =>$this->name(),'port'=>$data['https_port'], 'ip'=>$ip, 'type' => 'TCP');
            // Loop around resource data
            foreach($resource_data as $resource){
                foreach($ports as $port_item){
                    if(safe_check_port_conflict($port_item, $resource)){
                        $conflicts[] = array('name' => $this->name(), 'obj_type' =>'Configuration', 'desc' =>'port '.$port_item['port']);
                    }
                }
            }
        }
        return $conflicts;
    }
    public function check_object_usage($src_module_name, $obj_type, $obj_name, $sub_type = null, $sub_name = null)
    {
        // check usages ?
        $usages = array();
        if(!in_array($src_module_name, array('network'))){
            return $usages;
        }
    
        $data = $this->config()->get_data_values(false);
        $_ip = array();
        $interface = $data['interface'];
        if($interface != 'all'){
            $_ip[] = $interface;
        }
        if('network' == $src_module_name && in_array($obj_name, $_ip)){
            $tmp = array (
                    'name' => $this->name(),
                    'obj_type' => 'configuration',
                    'obj_name' => ''
            );
            if(!in_array($tmp, $usages)){
                $usages[] = $tmp;
            }
        }
    
        return $usages;
    }

    public function _generate_service_config(&$config_manager)
    {
        $data = $this->config()->get_data_values(false);
        $ip = $this->interface_to_ip($data['interface']); 
        
        $ports[] = array('name' =>$this->name(),'port'=>$data['http_port'], 'ip'=>$ip, 'type' => 'TCP');
        $ports[] = array('name' =>$this->name(),'port'=>$data['https_port'], 'ip'=>$ip, 'type' => 'TCP');
        
        //check system port
        foreach($ports as $port_item){
            if(!$this->node()->os()->valid_ip_port($port_item['ip'], $port_item['port'], $port_item['type'], 'webconfig')){
                $config_manager->add_error(array(
                        'module' => 'webconfig',
                        'obj_type' => '',
                        'description' => "{$this->description()} port {$port_item['port']} is used by other program.",
                        'url' => '/SAFe/sng_httpd_config'
                    )
                );
            }
        }
        //check $conflicts
        $conflicts = $this->node()->software()->application()->check_resource_conflict($this->name(), 'IP_PORT', $ports);
        foreach($conflicts as $conflict){
            $mod = $this->node()->software()->application()->find_module_by_name($conflict['name']);
            $name = $mod->description();
            $config_manager->add_error(array(
                    'module' => 'webconfig',
                    'obj_type' => '',
                    'description' => "{$this->description()} has IP_PORT conflict with {$name} at {$conflict['desc']}.",
                    'url' => '/SAFe/sng_httpd_config'
                )
            );
            
        }
        
        // Get our configuration data (unresolved)
        $http_conf_file  = '/usr/webconfig/conf/httpd.d/server_http.conf';
        $https_conf_file = '/usr/webconfig/conf/httpd.d/server_https.conf';
        if($ip == 'all'){
            $http_conf_string = 'Listen '.$data['http_port'];
        }else{
            $http_conf_string = 'Listen '.$ip.':'.$data['http_port'];
        }
        //if http protocol not enabled, comment out the #Listen http port, webconfig will not listen on http port
        if(!$this->config()->is_enabled_protocol(Sng_httpd_config_class::HTTP)){
            $http_conf_string = '#'. $http_conf_string;
        }
        $config_manager->add_config(new Safe_configuration_class($http_conf_file, 'webconfig', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
        $sed_data[] = "/^[\s|#]*listen.*/Id";
        $sed_data[] = "1i\\$http_conf_string\\";
        $sed_data[] = "s/^\s*<VirtualHost.*/<VirtualHost \_default_:".$data['http_port'].">/";
        
        $config_manager->add_config(new Safe_configuration_class($http_conf_file, $sed_data, Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_SED));
        $config_manager->add_config(new Safe_configuration_class($http_conf_file, 'root', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
        unset($sed_data);
        
        if($ip == 'all'){
            $https_conf_string = 'Listen '.$data['https_port'];
        }else{
            $https_conf_string = 'Listen '.$ip.':'.$data['https_port'];
        }
        //if https protocol not enabled, comment out the #Listen https port, webconfig will not listen on https port
        if(!$this->config()->is_enabled_protocol(Sng_httpd_config_class::HTTPS)){
            $https_conf_string = '#'. $https_conf_string;
        }
        $config_manager->add_config(new Safe_configuration_class($https_conf_file, 'webconfig', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
        $sed_data[] = "/^[\s|#]*listen.*/Id";
        $sed_data[] = "1i\\$https_conf_string\\";
        $sed_data[] = "s/^\s*<VirtualHost.*/<VirtualHost \_default_:".$data['https_port'].">/";
        $config_manager->add_config(new Safe_configuration_class($https_conf_file, $sed_data, Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_SED));
        $config_manager->add_config(new Safe_configuration_class($https_conf_file, 'root', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
        
        return true;
    }
    /**
     * @brief Override httpd restart to use reload
     *
     * @param[in out] $force
     * @param[in out] $result
     *
     * @return 
     */
    public function reload($force = false, &$result = null)
    {
        $script_path = $this->node()->configuration_manager()->directory('sng-base') . '/scripts';
        return $this->node()->os()->execute("nohup {$script_path}/restart-webconfig.sh", $result, true, true);
    }

    public function restart($force = false, &$result = null)
    {
       return $this->reload($force, $result);
    }

    /**
     * @brief Invoked after a successfull write_config
     *
     * @return
     */
    public function post_write_config($obj_type=null)
    {
        //clear before httpd restart else it will not clear
        if($this->is_configuration_modified()){
            $this->clear_configuration_modified();
            // restart service (in fact reload)
            $this->reload();
        }
        return parent::post_write_config($obj_type);
    }
    public function support_reload(&$need_reload = null) {
        parent::support_reload($need_reload);
        return true;
    }
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        return $this->generate_config($config_manager);
    }
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }
    /**
     * @brief Create httpd server URI
     *
     * @param[in out] $hostname
     *
     * @return 
     */
    public function server_uri($hostname, $protocol='http')
    {
        $uri = 'http://'.$hostname.':'.$this->config()->get_data_value($protocol.'_port',false);
        return $uri;
    }
    public function post_restore($info, &$reason)
    {
        $rs = parent::post_restore($info, $reason);
        //trigger to generate firewall rules when restore
        $this->config()->save();
        
        return $rs;
    }
}
/* End of file sng_httpd_service_class.php */
