<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2014  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Network interface class
 *
 * @author Kent
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_require_class('hardware');

class Sng_network_interface_class  extends Safe_configurable_object_class
{
    const PHYSICAL = 'physical';
    const VIRTUAL  = 'virtual';
    const BONDING  = 'bonding';
    const LOOPBACK = 'loopback';
    const MACVLAN = 'macvlan';
    const VLAN = 'vlan';

    const MANAGEMENT  = 'management';
    const TRANSCODING = 'transcoding';

    /**
     * Constructor for network interface config class
     * @param $node
     * @param $parent_name
     * @param $name
     */
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        //used in ip command to up or down interface
        $this->add_field('ifname', 'Interface', 'text', '',20);
        $this->add_field('read_only', 'Read Only', 'hidden', 'false');
        $this->set_field_readonly('ifname');
        $this->add_field('mac', 'MAC', 'text', '', 40);
        $this->set_field_readonly('mac');
        $this->add_enum_field('onboot', 'Onboot', 'dropdown', 'auto', array('auto' => 'Auto','yes' => 'Yes','no' => 'No'));
        $this->set_field_help('onboot', array(
                'yes' => 'Yes : This device should be activated at boot-time or network service restart.',
                'no' => 'No : This device should not be activated at boot-time or network service restart.',
                'auto' => 'Auto - If there are IP(s) address assgined to this device, this device will be activated at boot-time or network service restart.'
                ));
        //add Auto Negotiation settings for interface
        $this->add_enum_field('autoneg', 'Auto Negotiation', 'dropdown', 'true', $this->enable_disable_enum());
        $this->add_enum_field('speed', 'Speed', 'dropdown', '1000', array('10'=>'10 MB/s', '100'=>'100 MB/s','1000'=>'1000 MB/s'));
        $this->add_enum_field('duplex', 'Duplex', 'dropdown', 'full', array('half'=>'Half', 'full'=>'Full'));

        $neg_group = array('speed', 'duplex');
        $neg_condition = array();
        foreach($neg_group as $item){
            $neg_condition['true'][] = array('field_name' => $item, 'status' => 'disable');
            $neg_condition['false'][] = array('field_name' => $item, 'status' => 'enable');
        }
        $this->create_group('negotiation', $neg_group,
                array('default' => array( 'value' => 'true', 'status' => 'disable'),
                        'condition' =>$neg_condition
                )
        );
        $this->conditional_control('autoneg', 'negotiation');

        $this->add_field('type', 'Type', 'hidden', '', 20);
        //"management" or "transcoding"
        $this->add_field('if_mode', 'Interface Mode', 'hidden', '', 20);
        $this->add_field('configuration', 'Configuration', 'text', '', 40);
        $this->add_field('bonding_interface', 'Bonding Interface', 'hidden', '', 40);
    }
    public function validate($data, &$output)
    {
        $rs = parent::validate($data, $output);
        if($rs){
            if($this->speed() == '1000' && $this->autoneg() == 'false'){
                $this->set_error('autoneg', 'Autonegotiation is a requirement for using 1000BASE-T according to Section 28D.5 Extensions required for Clause40 (1000BASE-T).');
                $rs = false;
            }
        }
        return $rs;
    }
    public function bonding_interface()
    {// return the bonding interface name
        return $this->get_data_value('bonding_interface', false);
    }
    public function is_bonded()
    {// if not empty , then this interface was bonded
        return !empty($this->bonding_interface());
    }
    public function is_macvlan()
    {
        $vlan_type = $this->get_data_value('vlan_type', false);
        return $vlan_type == Sng_network_interface_class::MACVLAN;
    }
    public function is_vlan()
    {
        $vlan_type = $this->get_data_value('vlan_type', false);
        return $vlan_type == Sng_network_interface_class::VLAN;
    }
    public function interface_type()
    {// physical virtual loopback
        return $this->get_data_value('type', false);
    }
    public function hardware_type()
    {// "management" or "transcoding"
        return $this->get_data_value('if_mode');
    }
    public function mac()
    {
        return $this->get_data_value('mac');
    }
    public function autoneg()
    {
        return $this->get_data_value('autoneg', false);
    }
    public function speed()
    {
        return $this->get_data_value('speed', false);
    }
    public function duplex()
    {
        return $this->get_data_value('duplex', false);
    }
    public function onboot()
    {
        return $this->get_data_value('onboot', false);
    }
    public function interface_name()
    {
        $interface_name = $this->get_data_value('ifname');;
        return $interface_name;
    }
    public function link_status()
    {
        $network_module = $this->node()->find_module_by_name('network');
        $stats = $network_module->api_status_interface($this->interface_name());
        if($stats && $stats["operstate"] == 'up'){
            return true;
        }
        return false;
    }
    public function can_dispose(&$reason = null){
        $type = $this->interface_type();
        if($type == Sng_network_interface_class::PHYSICAL){
            $reason = 'Cannot delete ' . $type . ' interface.';
            return false;
        }elseif($this->is_readonly()){
            $reason = 'Cannot delete system interface.';
            return false;
        }else{
            return parent::can_dispose($reason);
        }
    }
    public function can_modify(&$output = null)
    {
        if($this->is_readonly()){
            $this->set_error('ifname', "Cannot modify system interface.", $output);
            return false;
        }
        return parent::can_modify($output);
    }
    public function summary()
    {
        $row_class = '';
        $status_data = Safe_icon('spinner', 'info', array('spin'));
        if ($this->is_macvlan()) {
            $name = $this->get_data_value('macvlan_device');
            $mac = $this->get_data_value('macvlan_mac');
        } else {
            $name = $this->interface_name();
            $mac = $this->get_data_value('mac');
        }
        $result =  array(
            'Name' => array(
                'data' => $name,
                'class' => $row_class,
            ),

            'MAC'  => array(
                'data' => $mac,
                'class' => $row_class,
            ),
            'Status' => array(
                'id' => 'network_interface_'.$this->interface_name().'_status',
                'data' => $status_data,
                'formatter' => 'none',
            )
        );
        return array( 'data' => $result);
    }
    public function save(&$output = null)
    {
        $result = parent::save($output);
        if($this->is_readonly() && $this->get_data_value('read_only') == 'false'){
            $this->set_data_values(array('read_only' => 'true'));
            $this->save();
        }
        return $result;
    }
    public function is_readonly()
    {
        if($this->hardware_type() == Sng_network_interface_class::TRANSCODING || $this->interface_type() == Sng_network_interface_class::LOOPBACK ){
            if($this->node()->get_config_status('update') == 'rest'){
                return false;
            }else{
                return true;
            }
        }
        elseif($this->is_macvlan() && $this->get_data_value('read_only') == 'true') {
            return true;
        }elseif($this->is_bonded()){
            return true;
        }else{
            return false;
        }
    }
    public function data_filter($data){
        //allow to update the ifname and mac for system configuration update
        $this->discard_lock();
        $this->set_field_readonly('ifname', 'false');
        $this->set_field_readonly('mac', 'false');
        $this->add_field('type', 'Type', 'text', '', 20);
        $this->add_field('if_mode', 'Interface Mode', 'text', '', 20);
        //for old version with auto speed value
        if($data['speed'] == 'auto'){
            $data['speed'] = '1000';
        }
        return $data;
    }

    public static function model(&$node){
        $module = $node->software()->application()->find_module_by_name('network');
        $class = get_called_class();
        return new $class($module->node(),$module->object_name().'/interface',$module->build_unique_name('interface'));
    }
}
class Sng_network_bonding_class extends Sng_network_interface_class
{
    public function configure()
    {
        $this->add_field('display_name', 'Bonding Interface Name', 'text', '', 20);
        $this->set_field_rules('bonding_interface_name', 'required|alpha_numeric');
        $this->add_enum_field('bonding_type','Bonding Type','dropdown','6',array('6'=>'balance-alb','4'=>'802.3ad'));
        $this->add_multiple_field('bonded_interface', 'Physical Interfaces', 'checkbox', array(), array());
        $this->set_field_rules('bonded_interface', 'callback_check_interface');
        $this->add_field('type', 'Type', 'hidden', Sng_network_interface_class::BONDING, 20);
        $this->set_obj_message('delete_confirm_body', "All IPs on this bonding interface will be moved to the first bonded physical interface.");
        $this->set_obj_message('add_form_title', "All IPs on the bonded physical interfaces will be move to the new bonding interface.");
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $bonding_interface_obj = $network_module->get_interface_list('bonding');
        $tmp_num = 0;
        if($bonding_interface_obj){
            $tmp_num = count($bonding_interface_obj);
        }
        $this->add_field('display_name', 'Bonding Interface Name', 'text', 'bond' . $tmp_num, 20);

        $interfaces_enum = array();
        $interface_obj = $network_module->get_interface_list('physical_management');
        if($interface_obj){
            foreach($interface_obj as $key => $obj){
                if($obj->is_bonded()){
                    continue;
                }else{
                    $interfaces_enum[$obj->name()] = $obj->interface_name();
                }
            }
        }
        $this->add_multiple_field('bonded_interface', 'Physical Interfaces', 'checkbox', array(), $interfaces_enum);
    }
    public function check_interface($field_name)
    {
        $interface_list = parent::get_data_value('bonded_interface',false);
        if($interface_list && count($interface_list) == 2){
            return true;
        }else{
            return array('check_interface' => 'The Interface field must contain 2 physical interfaces.');
        }
    }
    public function get_bonded_interface()
    {
        $interface_list = $this->get_data_value('bonded_interface',false);
        return $interface_list;
    }
    public function mac()
    {
        return '01:00:00:00:00:00';
    }
    public function hardware_type()
    {
        return Sng_network_interface_class::MANAGEMENT;
    }
    public function interface_name()
    {
        //need to review , now just return obj name
        return $this->name();
    }
    public function summary()
    {
        $result = parent::summary();
        //no mac in bonding interface
        $bonded_interface = $this->get_data_value('bonded_interface', false);
        if($bonded_interface){
            $result['data']['MAC']['data'] = implode(',' , $bonded_interface);
        }
        return $result;
    }
    public function save(&$output = null)
    {
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $bonded_interfaces = $this->get_bonded_interface();
        if($bonded_interfaces){
            foreach($bonded_interfaces as $tmp_interface_name){
                $tmp_interface_obj = $network_module->api_retrieve_interface($tmp_interface_name);
                if($tmp_interface_obj){
                    //mark this interface is bonded
                    $tmp_interface_obj->set_data_values(array('bonding_interface' => $this->name()));
                    $tmp_interface_obj->save();
                    $tmp_ip_list = $network_module->get_ips($tmp_interface_name);
                    if($tmp_ip_list){
                        foreach($tmp_ip_list as $ip => $tmp_ip_obj){
                            //move the ip to this bonding interface
                            $tmp_ip_obj->set_data_values(array('interface' => $this->name()));
                            $tmp_ip_obj->save();
                        }
                    }
                }
            }
        }
        return parent::save($output);
    }
    public function dispose(&$output = null)
    {
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $tmp_ip_list = $network_module->get_ips($this->name());
        $bonded_interfaces = $this->get_bonded_interface();
        if($bonded_interfaces){
            //move the bonding interface ips to first interface
            foreach($bonded_interfaces as $tmp_interface_name){
                if($tmp_ip_list){
                    foreach($tmp_ip_list as $ip => $tmp_ip_obj){
                        //move the ip to this bonding interface
                        $tmp_ip_obj->set_data_values(array('interface' => $tmp_interface_name));
                        $tmp_ip_obj->save();
                    }
                }
                break;
            }
            foreach($bonded_interfaces as $tmp_interface_name){
                $tmp_interface_obj = $network_module->api_retrieve_interface($tmp_interface_name);
                if($tmp_interface_obj){
                    //remove bonding interface name from the phsycal interface
                    $tmp_interface_obj->set_data_values(array('bonding_interface' => ''));
                    $tmp_interface_obj->save();
                }
            }
        }
        return parent::dispose();
    }
    public function can_edit(&$reason = NULL)
    {
        return false;
    }
}
class Sng_network_link_class  extends Sng_network_interface_class
{
    /**
     * Constructor for network interface config class
     * @param $node
     * @param $parent_name
     * @param $name
     */
    public function __construct($node, $parent_name, $name)
    {
        parent::__construct($node, $parent_name, $name);
        $this->_set_reuse(false);
    }
    public function configure()
    {
        $vlan_type = array('vlan' => 'VLAN', 'macvlan' => 'MAC VLAN');
        $this->add_enum_field('vlan_type', 'VLAN Type', 'dropdown', 'vlan', $vlan_type);
        $this->add_enum_field('ifname', 'Interface', 'dropdown', '', array());
        $this->add_field('read_only', 'Read Only', 'hidden', 'false');
        $this->add_field('id', 'Virtual interface ID', 'text', '', 20);
        // MAC VLAN
        $this->add_field('macvlan_device', 'MAC VLAN Interface', 'text', '', 20);
        $this->set_field_readonly('macvlan_device');
        $this->add_field('macvlan_mac', 'MAC Address', 'text', '', 17);
        $this->set_field_readonly('macvlan_mac');
        $this->add_field('type', 'Type', 'hidden', Sng_network_interface_class::VIRTUAL, 20);

        $fields = array('id', 'macvlan_parent', 'macvlan_mode', 'macvlan_device', 'macvlan_mac');
        $condition = array();
        foreach ($fields as $field) {
            if ($field == 'id') {
                $condition['vlan'][] = array('field_name' => $field, 'status' => 'enable');
                $condition['macvlan'][] = array('field_name' => $field, 'status' => 'disable');
            }
            else {
                $condition['vlan'][] = array('field_name' => $field, 'status' => 'disable');
                $condition['macvlan'][] = array('field_name' => $field, 'status' => 'enable');
            }
        }
        $this->create_group('vlan_type', $fields, array(
                'default' => array( 'value' => 'macvlan', 'status' => 'disable'),
                'condition' => $condition
            )
        );
        $this->conditional_control('vlan_type', 'vlan_type');
    }
    public function hardware_type()
    {// management or transcoding
        if ($this->get_data_value('vlan_type', false) == Sng_network_interface_class::MACVLAN) {
            //mac vlan only for TRANSCODING d150
            return Sng_network_interface_class::TRANSCODING;
        }else{
            //vlan only for MANAGEMENT
            return Sng_network_interface_class::MANAGEMENT;
        }
    }
    public function mac()
    {// mac from hardware interface
        if ($this->is_macvlan()) {
            return $this->get_data_value('macvlan_mac');
        }
        $interface = $this->get_data_value('ifname', false);
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $interface_obj = $network_module->get_aggregate_object('interface', $interface);
        if($interface_obj){
            return $interface_obj->mac();
        }else{
            return false;
        }
    }
    public function is_vlan()
    {
        $vlan_type = $this->get_data_value('vlan_type', false);
        return $vlan_type == Sng_network_interface_class::VLAN;
    }
    public function is_macvlan()
    {
        $vlan_type = $this->get_data_value('vlan_type', false);
        return $vlan_type == Sng_network_interface_class::MACVLAN;
    }
    public function interface_name()
    {
        if ($this->is_macvlan()) {
            $interface_name = $this->get_data_value('macvlan_device');
        }
        else {
            $interface_name = $this->get_data_value('ifname', false) . '.' . $this->get_data_value('id');
        }
        return $interface_name;
    }
    public function onboot()
    {
        return 'yes';
    }
    public function save() {
        $result = parent::save();
        $data = $this->get_data_values();
        if ($this->is_vlan() && $data['macvlan_mac'] != '' && $data['macvlan_device'] != '') {
            $data['macvlan_mac'] = '';
            $data['macvlan_device'] = '';
            $this->set_data_values($data);
            $this->save();
        }
        return $result;
    }
    public function summary()
    {
        $result = parent::summary();
        //no mac in vlan interface
        if($result && isset($result['data']['MAC']) && substr($result['data']['Name']['data'], 0, 6) !== 'sngdsp'){
            $result['data']['MAC']['data'] ='';
        }
        return $result;
    }
    public function can_dispose(&$reason = null){
        //check if this interface used by ip
        $macvlan_device = $this->get_data_value('macvlan_device', false);
        $verify_ip_objs = (strncmp($macvlan_device, 'sngdsp', 6) !== 0);
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $ip_objs = $network_module->get_aggregate_objects('ip');
        $my_name = $this->name();
        foreach($ip_objs as $ip_obj_name => $ip_obj){
            if($ip_obj->get_interface() == $my_name){
                if ($verify_ip_objs) {
                    $reason = 'You must remove all the IPs on this interface before you can delete it.';
                    return false;
                }
            }
        }
        return parent::can_dispose($reason);
    }
    public function validate($data, &$output)
    {
        $mvlan_count = 0;
        if ($this->is_vlan()) {
            $this->set_field_rules('id', 'required|integer|greater_than[0]|less_than[4095]');
        }
        $parent_result = parent::validate($data, $output);
        if ($parent_result && $this->interface_type() == Sng_network_interface_class::VIRTUAL) {
            $network_module = $this->node()->software()->application()->find_module_by_name('network');
            $interface_objs = $network_module->get_aggregate_objects('interface');
            $id = $this->get_data_value('id');
            // block if there is a physical DSP card on the system
            if ($this->is_macvlan()) {
                $transcoding_interfaces = $network_module->hardware_interface(Sng_network_interface_class::TRANSCODING);
                    foreach ($transcoding_interfaces as $k=>$v) {
            if ($v->hardware_type() == 'transcoding') {
                $internal_dsp = true;
                break;
            }
                    }
                    if($internal_dsp){
                        $this->set_error('vlan_type', 'We cannot create DSP MACVLANs on a system with existing physical DSP card.', $output);
                        return false;
                    }
            }
            foreach($interface_objs as $interface_obj_name => $interface_obj){
                if ($interface_obj->is_macvlan() && $this->name() !== $interface_obj->name()) {
                    $mvlan_count++;
                }
                else if ($interface_obj->is_vlan()) {
                    if($interface_obj_name != $this->name() && $interface_obj->interface_type() == Sng_network_interface_class::VIRTUAL){
                        if($interface_obj->interface_name() == $this->interface_name()){
                            $rule_msgs['id'] = "This VLAN id:{$id} is already used.";
                            $this->set_error('id', $rule_msgs['id'], $output);
                            return false;
                        }
                    }
                }
            }
            if ($interface_obj->is_macvlan() && $mvlan_count >= 2) {
                $msg = "Already $mvlan_count interfaces exist in the system. Only a maximum of 2 media interfaces are supported.";
                $this->set_error('vlan_type', $msg, $output);
                $parent_result = false;
            }
        }
        return $parent_result;
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $hardware_interface_enum = $network_module->get_interface_list('physical_management', false);
        $this->add_enum_field('ifname', 'Interface', 'dropdown', '', $hardware_interface_enum);
        if ($this->is_vlan()) {
            $data = $this->get_data_values();
            if (isset($data['macvlan_device']) && $data['macvlan_device'] == '') {
                $data['macvlan_device'] = $this->generate_macvlan('device');
            }
            if (isset($data['macvlan_mac']) && $data['macvlan_mac'] == '') {
                $data['macvlan_mac'] = $this->generate_macvlan('mac');
            }
            $this->set_data_values($data);
        }
    }
    public function generate_macvlan($type='mac')
    {
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $interfaces = $network_module->api_retrieve_interface();
        $dsp_inf = array();
        $macs = array();
        foreach($interfaces as $interface_name => $obj) {
            if (strpos($obj->interface_name(), 'sngdsp') > -1) {
                $dsp_inf[] = $obj->interface_name();
            }
            if (substr($obj->get_data_value('mac'), 0, 8) == '02:19:23') {
                $macs[] = $obj->get_data_value('mac');
            }
        }

        switch ($type) {
            case 'mac':
                $valid_mac = false;
                while (!$valid_mac) {
                    $mac = '02:19:23:' . implode(':', str_split(substr(md5(mt_rand()), 0, 6),2));
                    if (!in_array($mac, $macs)) {
                        $valid_mac = true;
                    }
                }
                return $mac;
                break;
            case 'device':
                if (count($dsp_inf) == 0) return 'sngdsp0';
                else {
                    $lastnum = (preg_match_all('/\d+/', max($dsp_inf), $numbers)) ? end($numbers[0]) + 1 : 0;
                    return 'sngdsp' . $lastnum;
                }
                break;
        }
        return FALSE;
    }
}
class Sng_network_link_add_config extends Safe_configuration_user_class
{
    protected $_operator = 'add';

    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_CREATE)
    {
        parent::__construct($route_name, $action, $data);
    }
    public function run($node, &$output_msg = null)
    {
        $ret = true;
        $data = $this->data();
        $interface = $data['ifname'];
        $interface_name = $interface;
        $vlan_id = '';
        $mac = '';
        if (isset($data['vlan_type'])) {
            if ($data['vlan_type'] == Sng_network_interface_class::VLAN) {
                $interface_name = $interface . '.' . $data['id'];
                $vlan_id = $data['id'];
            }else {
                $interface_name = $data['macvlan_device'];
                $mac = isset($data['macvlan_mac']) ? $data['macvlan_mac'] : '';
            }
        }
        // Prepare command:
        if($this->_operator == 'add'){
            if($data['type'] == Sng_network_interface_class::VIRTUAL){
                $cmd_add_vlan = ($data['vlan_type'] == Sng_network_interface_class::MACVLAN)
                    ? '/sbin/ip link add link ' . $interface . ' name ' . $interface_name . ' address ' . $mac . ' type macvlan mode bridge'
                    : '/sbin/ip link add link ' . $interface . ' name ' . $interface_name . ' type vlan id ' . $vlan_id;
                $ret = $node->execute('sudo nohup ' . $cmd_add_vlan, $output_msg);
            }
            if($data['type'] == Sng_network_interface_class::BONDING){
                //load the bonding modul
                $cmd_bonding = '/sbin/modprobe bonding ';
                $ret = $node->execute('sudo nohup ' . $cmd_bonding, $output_msg);
                sleep(1);
            }
            $ret = true;
        }elseif($this->_operator == 'del'){
            $if_type = '';
            if($data['type'] == Sng_network_interface_class::VIRTUAL){
                $if_type = ($data['vlan_type'] == Sng_network_interface_class::MACVLAN) ? 'macvlan' : 'vlan';
                $if_type = ' type ' . $if_type;
            }
            $cmd_remove_vlan = '/sbin/ip link delete  ' . $interface_name . $if_type ;
            $ret = $node->execute('sudo nohup ' . $cmd_remove_vlan, $output_msg);
            $ret = true;
        }elseif($this->_operator == 'up'){
            if($data['type'] != Sng_network_interface_class::LOOPBACK){
                //check interface is up '/sys/class/net/eth2/operstate'
                $cmd_check_link_up = "/bin/cat  /sys/class/net/{$interface_name}/operstate" ;
                $ret = $node->execute('sudo nohup ' . $cmd_check_link_up, $output_msg);
                if($ret && isset($output_msg[0]) && trim($output_msg[0]) == 'up'){
                    error_log( "Interface {$interface_name} already up, skipping.");
                }else{
                    $cmd_link_up = '/sbin/ifup ' . $interface_name;
                    $ret = $node->execute('sudo nohup ' . $cmd_link_up, $output_msg);
                    sleep(1);
                }
            }
            $ret = true;
        }elseif($this->_operator == 'down'){
            if($data['type'] != Sng_network_interface_class::LOOPBACK){
                $cmd_link_down = '/sbin/ifdown ' . $interface_name;
                $ret = $node->execute('sudo nohup ' . $cmd_link_down, $output_msg);
                sleep(1);
            }
            $ret = true;
        }elseif($this->_operator == 'restart'){
            if($data['type'] != Sng_network_interface_class::LOOPBACK){
                $cmd_link_restart1 = '/sbin/ifdown ' . $interface_name;
                $cmd_link_restart2 = '/sbin/ifup ' . $interface_name;
                $ret1 = $node->execute('sudo nohup ' . $cmd_link_restart1);
                sleep(1);
                $ret2 = $node->execute('sudo nohup ' . $cmd_link_restart2);
                sleep(1);
                $ret = $ret1 && $ret2;
            }
        }
        return $ret;
    }
}
class Sng_network_link_delete_config extends Sng_network_link_add_config
{
    protected $_operator = 'del';

    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_DELETE)
    {
        parent::__construct($route_name, $data, $action);
    }
}
class Sng_network_link_up_config extends Sng_network_link_add_config
{
    protected $_operator = 'up';

    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_DELETE)
    {
        parent::__construct($route_name, $data, $action);
    }
}
class Sng_network_link_down_config extends Sng_network_link_add_config
{
    protected $_operator = 'down';

    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_DELETE)
    {
        parent::__construct($route_name, $data, $action);
    }
}
class Sng_network_link_restart_config extends Sng_network_link_add_config
{
    protected $_operator = 'restart';

    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_DELETE)
    {
        parent::__construct($route_name, $data, $action);
    }
}
?>
