<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2014  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Network interface class
 *
 * @author Kent
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('object');
safe_module_require_class('sng', 'network_interface');

class Sng_ip_address_class  extends Safe_ip_address_class
{
    private $_readonly = false;
    public function configure()
    {
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $interfaces = $network_module->api_retrieve_interface();
        $interfaces_enum = array();
        foreach($interfaces as $key => $obj){
            if($obj->hardware_type() == Sng_network_interface_class::TRANSCODING){
                //only in development_mode, or rest replace mode allow user to add/remove/change the ip on sngdspX interfaces
                if(!safe_development_mode() && $this->node()->get_config_status('update') != 'rest'){
                    continue;
                }
            }
            if($obj->is_bonded()){
                //also pass by bonded interface
                continue;
            }
            $interfaces_enum[$obj->name()] = $obj->interface_name();
        }
        $this->add_enum_field('interface', 'Interface', 'dropdown', '', $interfaces_enum);
        $this->set_field_help('interface','The name of the device to add the IP address to.');

        $proto_enum = array(Safe_ip_address_class::PROTO_DHCP4 =>'IPv4 - Dynamic',
                Safe_ip_address_class::PROTO_STATIC4=>'IPv4 - Static',
                Safe_ip_address_class::PROTO_DHCP6 =>'IPv6 - Dynamic',
                Safe_ip_address_class::PROTO_SLAAC6 =>'IPv6 - Automatic',
                Safe_ip_address_class::PROTO_STATIC6 =>'IPv6 - Static'
        );
        
        $this->add_field('read_only', 'Read Only', 'hidden', 'false');
        $this->add_enum_field('proto', 'Configuration', 'dropdown', Safe_ip_address_class::PROTO_STATIC4, $proto_enum);
        $this->set_field_help('proto','The protocol family type and which boot-time protocol should be used.');

        $this->add_field('hostname', 'Hostname', 'text', '', 40);
        $proto_in_value = Safe_ip_address_class::PROTO_DHCP4 . ',' . Safe_ip_address_class::PROTO_DHCP6 . ',' . Safe_ip_address_class::PROTO_SLAAC6;
        $this->set_field_rules('hostname', 'required[proto,in,'. $proto_in_value .']|valid_domain');
        $this->set_field_help('hostname','Use this option for the DHCP server requires the client to specify a hostname before receiving an IP address.');

        $this->add_enum_field('peerdns', 'Use automatic DNS servers', 'dropdown', 'true', $this->enable_disable_enum());
        $this->set_field_help('peerdns','Use the DNS servers returns from DHCP server.');

        $this->add_enum_field('persistent', 'Persistent DHCP client', 'dropdown', 'true', $this->enable_disable_enum());
        $this->set_field_help('persistent', 'Keep retrying to get a DHCP lease indefinitely');

        $group = array('hostname', 'peerdns', 'persistent', 'address', 'prefix');
        $condition = array();
        foreach($group as $item){
            if(in_array($item, array('hostname', 'peerdns', 'persistent'))){
                $condition[Safe_ip_address_class::PROTO_DHCP4][] = array('field_name' => $item, 'status' => 'enable');
                $condition[Safe_ip_address_class::PROTO_DHCP6][] = array('field_name' => $item, 'status' => 'enable');

                if ($item == 'hostname') {
                    $condition[Safe_ip_address_class::PROTO_SLAAC6][] = array('field_name' => $item, 'status' => 'enable');
                } else {
                    $condition[Safe_ip_address_class::PROTO_SLAAC6][] = array('field_name' => $item, 'status' => 'disable');
                }

                $condition[Safe_ip_address_class::PROTO_STATIC4][] = array('field_name' => $item, 'status' => 'disable');
                $condition[Safe_ip_address_class::PROTO_STATIC6][] = array('field_name' => $item, 'status' => 'disable');
            }else{
                $condition[Safe_ip_address_class::PROTO_DHCP4][] = array('field_name' => $item, 'status' => 'disable');
                $condition[Safe_ip_address_class::PROTO_DHCP6][] = array('field_name' => $item, 'status' => 'disable');
                $condition[Safe_ip_address_class::PROTO_SLAAC6][] = array('field_name' => $item, 'status' => 'disable');
                $condition[Safe_ip_address_class::PROTO_STATIC4][] = array('field_name' => $item, 'status' => 'enable');
                $condition[Safe_ip_address_class::PROTO_STATIC6][] = array('field_name' => $item, 'status' => 'enable');
            }
        }
        $this->create_group('proto', $group,
                array('default' => array( 'value' => Safe_ip_address_class::PROTO_STATIC4, 'status' => 'disable'),
                        'condition' =>$condition
                )
        );
        $this->conditional_control('proto', 'proto');

        $this->add_field('address', 'Address', 'text', '', 40);
        $this->set_field_rules('address', 'required[proto,in,'. Safe_ip_address_class::PROTO_STATIC4 .','. Safe_ip_address_class::PROTO_STATIC6 .']|valid_ip');
        $this->add_field('prefix', ' / ', 'string', '', 10);
        $this->set_field_rules('prefix','required[proto,in,'. Safe_ip_address_class::PROTO_STATIC4 . ','. Safe_ip_address_class::PROTO_STATIC6 .']|integer|greater_than[0]|less_or_equal[128]');
        $this->composite_layout('address', array('prefix'));
        $this->set_field_help('address', ' IP address and network prefix.');
    }
    public function get_interface(){
        //get interface obj name 
        $interface_obj_name  = $this->get_data_value('interface', false);
        return $interface_obj_name;
    }
    public function get_proto(){
        return $this->get_data_value('proto', false);
    }
    public function get_hostname(){
        return $this->get_data_value('hostname');
    }
    public function get_peerdns(){
        return $this->get_data_value('peerdns', false) == 'true' ? 'yes' : 'no';
    }
    public function get_persistent(){
        return $this->get_data_value('persistent', false) == 'true' ? 'yes' : 'no';
    }
    public function get_interface_obj()
    {
        $interface_obj_name = $this->get_interface();
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $interface_obj = $network_module->get_aggregate_object('interface', $interface_obj_name);
        if($interface_obj){
            return $interface_obj;
        }else{
            return false;
        }
    }
    public function get_interface_name()
    {   
        $interface_name = false;
        $interface_obj = $this->get_interface_obj();
        if($interface_obj){
            $interface_name = $interface_obj->interface_name();
        }
        return $interface_name;
    }
    public function get_interface_type()
    {
        $interface_type = false;
        $interface_obj = $this->get_interface_obj();
        if($interface_obj){
            $interface_type = $interface_obj->interface_type();
        }
        return $interface_type;
    }
    public function address(){
        return $this->get_data_value('address');
    }
    public function prefix()
    {
        return $this->get_data_value('prefix');
    }
    public function mask()
    {
        return safe_cidr2netmask($this->get_data_value('prefix'));
    }
    public function ip_type()
    {
        $proto = $this->get_data_value('proto', false);
        $type_data = explode('-', $proto);
        $tmp_type = $type_data[1];
        return $tmp_type;
    }
    public function is_dhcp()
    {
        $proto = $this->get_data_value('proto', false);
        return in_array($proto, $this->_dhcp);
    }
    private $_set_dhcp_ip = false;
    public function validate($data, &$output)
    {
        $new_ip = $this->address();
        //get the old data from db
        $this->unserialize();
        $old_ip = $this->address();
        if($old_ip == $_SERVER['SERVER_ADDR'] && $new_ip != $old_ip){
            $this->set_error('interface', "IP address {$this->address()} is used by current connection. Please use an other IP or add a new IP for connection before modify or delete this one.", $output);
            return false;
        }
        else if($new_ip != $old_ip){
            // Checking if webconfig is listening in the ip address we are
            // trying to modify
            $webconfig = $this->node()->software()->application()->find_module_by_name('webconfig');
            $network = $this->node()->software()->application()->find_module_by_name('network');
            $_data = $webconfig->config()->get_data_values(false);
            $ip_obj = $network->get_ip_by_name($_data['interface']);
            if($ip_obj && $ip_obj->address()==$old_ip){
                $this->set_error('address',"IP address {$old_ip} is been used by web server, if you change the ip address for this interface, you will lost WebUI access", $output);
                return false;
            }
        }
        $parent_result = parent::validate($data, $output);
        
        //check ip
        $ip_type = $this->ip_type();
        $ip = $this->get_data_value('address');
        //save the compress ip into db
        if($ip_type == Safe_ip_address_class::IP_V6){
            $this->set_data_values(array('address' => safe_ipv6_compress($ip)));
        }
        $prefix = $this->get_data_value('prefix');

        $ip_error_msg = 'The Address field must contain a valid IP.';
        $prefix_error_msg = 'The prefix must contain a number less than 32';
        if(!$this->is_dhcp()){
            if($ip_type == Safe_ip_address_class::IP_V4){
                if(!filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4)){
                    $this->set_error('address', $ip_error_msg, $output);
                    $parent_result =  false;
                }
                if($prefix > 32){
                    $this->set_error('prefix', $prefix_error_msg, $output);
                    $parent_result =  false;
                }
            }
            if($ip_type == Safe_ip_address_class::IP_V6){
                if(!filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)){
                    $this->set_error('address', $ip_error_msg, $output);
                    $parent_result =  false;
                }
            }
        }else{
            //clear old ip for dhcp IP obj
            $this->set_data_values(array('address' => ''));
        }
        // prevent from link local address fe80
        if($ip_type == Safe_ip_address_class::IP_V6){
            if(filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)) {
                if(strncasecmp($ip, 'fe80', 4) == 0) {
                    $this->set_error('address',$ip_error_msg, $output);
                    $parent_result =  false;
                }
            }
        }
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        if($parent_result){
            //check if ip conflict
            $ip = $this->address();
            if($ip){
                //check if conflict with network ip
                $ip_conflict_list = $network_module->check_ip_conflict($this);
            
                //check if conflict with cluster floating ip
                $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
                $cluster_ip_conflict_list = $cluster_module->check_ip_conflict($this);
                if($cluster_ip_conflict_list){
                    $ip_conflict_list = array_merge($ip_conflict_list, $cluster_ip_conflict_list);
                }
                if($ip_conflict_list){
                    $error_msg = 'This ip is conflict with : ';
                    $list = array();
                    foreach($ip_conflict_list as $obj){
                        $list[] = $obj->name() . "[{$obj->address()}]";
                    }
                    $error_msg .= join(",", $list);
                    $this->set_error('address', $error_msg, $output);
                    $parent_result = false;
                }
            }    
        }
        
        //check if already have dhcp ip obj
        if($this->is_dhcp()){
            $ip_objs = $network_module->get_aggregate_objects('ip');
            $interface = $this->get_interface();
            $my_name = $this->name();
            $my_ip_type = $this->ip_type();
            foreach($ip_objs as $ip_obj_name => $ip_obj){
                if($ip_obj_name != $my_name && $ip_obj->ip_type() == $my_ip_type && $ip_obj->get_interface() == $interface && $ip_obj->is_dhcp()){
                    $rule_msgs['proto'] = 'Already have a DHCP IP attached to this Interface.';
                    $this->set_error('proto', $rule_msgs['proto'], $output);
                    return false;
                }
            }
        }
        //lo interface
        if($this->get_interface_type() == Sng_network_interface_class::LOOPBACK){
            if($this->is_dhcp()){
                $rule_msgs['proto'] = 'Loopback interface does not support DHCP.';
                $this->set_error('proto', $rule_msgs['proto'], $output);
                $parent_result = false;
            }else{
                //enforce address being on 127 network
                if($this->ip_type() == Safe_ip_address_class::IP_V4){
                    if(!safe_cidr_match_str($this->address(),'127.0.0.0/8') || $this->prefix() < 8){
                        $rule_msgs['address'] = 'Loopback interface supports IPv4 addresses only in the 127.0.0.0/8 range.';
                        $this->set_error('address', $rule_msgs['address'], $output);
                        $parent_result = false;
                    }
                }else{
                    $tmp_ip = safe_ipv6_compress($this->address());
                    if($tmp_ip != '::1'){
                        $rule_msgs['address'] = 'Loopback interface supports only one IPv6 address ::1 .';
                        $this->set_error('address', $rule_msgs['address'], $output);
                        $parent_result = false;
                    }
                }
            }
        }
        // TODO: This should be temporary until a proper solution is in place to account for 'short' (composite) and 'long' (error msg for example) labels
        if ($output['prefix']) {
	    $this->set_error('prefix',str_replace('/', 'prefix', $output['prefix']) , $output);
	}
        return $parent_result;
    }
    public function summary()
    {
        $ip = '';
        $address = $this->get_data_value('address') ;
        $prefix = $this->get_data_value('prefix');
        if($address){
            $ip = $address;
            if($prefix){
                $ip = $ip . '/' . $prefix;
            }
        }else{
            $status_data = Safe_icon('spinner', 'info', array('spin'));
            $ip = array(
                'id' => 'network_ip_' . $this->name(),
                'data' => $status_data,
                'formatter' => 'none',
            );
        }
        
        $hostname = $this->get_data_value('hostname');
        
        $result =  array(
                //'Name' => $this->name(),
                'Interface' => $this->get_interface_name(),
                'Type' => $this->get_data_value('proto'),
                'IP Address' => $ip,
                'Hostname' => $hostname
        );
        return array( 'data' => $result);
    }
    public function can_dispose(&$reason = null)
    {
        $usages = $this->node()->software()->application()->check_object_usage('network','ip', $this->name());
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }elseif($this->is_readonly()){
            $reason['message'] = "Cannot delete system IP address.";
            return false;
        }else{
            $network = $this->node()->software()->application()->find_module_by_name('network');
            $sprs = $network->api_retrieve_source_policy_routing();
            foreach($sprs as $spr){
                if($this->address()==$spr->source_ip()){
                    $usages[] = array('name'=>$this->name(),
                                      'obj_type'=>'source_policy_routing',
                                      'obj_name'=>$spr->name());
                    $reason['message'] = "{$this->address()} is used by a Source Policy Routing";
                    return false;
                }
            }
        }
        return parent::can_dispose($reason);
    }
    public function can_modify(&$output = null)
    {
        if($this->is_readonly()){
            $this->set_error('interface', "Cannot modify system IP address.", $output);
            return false;
        }
        $allow_modify = true;
        $usages = $this->node()->software()->application()->check_object_usage('network','ip', $this->name());
        if($usages){
            $usg_item = array();
            foreach($usages as $obj_item){
                if(isset($obj_item['allow_modify']) &&  !$obj_item['allow_modify']){
                    if($obj_item['name'] == 'rest'){
                        continue;
                    }
                    //if IP obj is in using , can not modify
                    unset($obj_item['allow_modify']);
                    //if the obj have 'description' field, show the obj description
                    if(isset($obj_item['description'])){
                        $usg_item[] = $obj_item['description'];
                    }else{
                        $usg_item[] = join('/', $obj_item);
                    }
                    $allow_modify = false;
                }
            }
            if($usg_item){
                $this->set_error('interface', "IP address {$this->address()} is used by " . join(',', $usg_item), $output);
            }
            if(!$allow_modify){
                return false;
            }
        }
        return parent::can_modify($output);
    }
    public function dispose()
    {
        if(parent::dispose()){
            $receivers = $this->node()->software()->application()->check_object_modified($this, 'network','ip', $this->name());
            return true;
        }
        return false;
    }
    public function is_readonly(&$output = null)
    {
        $interface_obj = $this->get_interface_obj();
        if($interface_obj){
            if($interface_obj && $interface_obj->hardware_type() == Sng_network_interface_class::TRANSCODING){
                //only in development_mode, or rest replace mode allow user to add/remove/change the ip on sngdspX interfaces
                if(!safe_development_mode() && $this->node()->get_config_status('update') != 'rest'){
                    return true;
                }
            }elseif($this->ip_type() == Safe_ip_address_class::IP_V6 &&
                    strncasecmp($this->address(), 'fe80', 4) == 0) {
                return true;
            }elseif($interface_obj->get_data_value('vlan_type', false) == 'macvlan') {
                return true;
            }else{
                return parent::is_readonly($output) ;
            }    
        }
        return false;
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        //do not allow user change the interface, otherwise it will not readonly anymore  
        if($this->is_readonly()){
            $this->set_field_disabled('interface', true);
        }
    }
    public function save(&$output = null)
    {
        $result = parent::save($output);
        if($this->is_readonly() && $this->get_data_value('read_only') == 'false'){
            $this->set_data_values(array('read_only' => 'true'));
            $this->save();
        }
        //not from safepy update ip 
        if($this->is_modified() && $this->is_dhcp()){
            $ip_type = $this->ip_type();
            $interface = $this->get_interface_name();
            $dhcp_file_name = '/tmp/dhcp.' . $interface . '.' .$ip_type;
            $os = $this->node()->os();
            //to do check old obj and remove old IP
            if($this->get_proto() == Safe_ip_address_class::PROTO_SLAAC6){
                //slaac6
                $dhcp_result = false;
                $interface_obj = $this->get_interface_obj();
                $hw_mac = '';
                if($interface_obj){
                    $hw_mac = $interface_obj->mac();
                }
                if($hw_mac){
                    $sub_mac = substr(str_replace(':','',strtolower($hw_mac)), -6);
                    //sent out slaac request
                    $cmd1 = "/sbin/sysctl -w net.ipv6.conf.{$interface}.autoconf=0" ;
                    $cmd2 = "/sbin/sysctl -w net.ipv6.conf.{$interface}.accept_ra=2" ;
                    $cmd3 = "/sbin/sysctl -w net.ipv6.conf.{$interface}.autoconf=1" ;
                    $cmd4 = "/usr/bin/rdisc6 -1 {$interface}";
                    $this->node()->execute('sudo nohup ' . $cmd1);
                    $this->node()->execute('sudo nohup ' . $cmd2);
                    $this->node()->execute('sudo nohup ' . $cmd3);
                    $this->node()->execute('sudo nohup ' . $cmd4);
                    sleep(10);
                    
                    //read slacc ip
                    $cmd_result = array();
                    $cmd1 = "/sbin/ip addr show dev {$interface}" ;
                    $this->node()->execute($cmd1, $cmd_result);
                    if($cmd_result){
                        $cmd_result = preg_grep("/inet6/", $cmd_result);
                        $cmd_result = preg_grep("/inet6 fe80/", $cmd_result, PREG_GREP_INVERT);
                        $cmd_result = preg_grep("/dynamic/", $cmd_result);
                        if($cmd_result){
                            foreach($cmd_result as $tmp_item){
                                $pieces = explode(" ", trim($tmp_item));
                                if(isset($pieces[1])){
                                    $tmp_ip_prefix = explode("/", $pieces[1]);
                                    if(isset($tmp_ip_prefix[0]) && isset($tmp_ip_prefix[1])){
                                        //check if the last 6 str match with the mac address (the ipv6 ip must in expand format)
                                        $sub_slaac_ip = substr(str_replace(':','',strtolower(safe_ipv6_expand($tmp_ip_prefix[0]))), -6);
                                        if($sub_slaac_ip == $sub_mac){
                                            $ip = $this->address();
                                            $prefix = $this->prefix();
                                            if($ip != $tmp_ip_prefix[0] || $prefix != $tmp_ip_prefix[1]){
                                                $new_ip_data['address'] = $tmp_ip_prefix[0];
                                                $new_ip_data['prefix'] = $tmp_ip_prefix[1];
                                                $this->set_data_values($new_ip_data);
                                                $dhcp_result = parent::save();
                                            }else{
                                                $dhcp_result = true;
                                            }
                                        }
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
            }else{
                //dhcp v4 or v6
                //check if the dhclient process is running
                if(!$this->get_dhcp_ip()){
                    //sent out dhcp request, the dhcp ip will be update by the sng-dhcp-handler.sh
                    $script_path = $this->node()->configuration_manager()->directory('sng-base') . '/bin';
                    $tmp_host_name = $this->get_hostname();
                    $cmd1 = "{$script_path}/sng-set-dhcp {$ip_type} {$interface} {$tmp_host_name}" ;
                    $this->node()->execute('sudo nohup ' . $cmd1);
                }
            }
        }
        if($this->is_modified()){
            //send network ip modification event out
            $receivers = $this->node()->software()->application()->check_object_modified($this, 'network','ip', $this->name());
            $network_module = $this->node()->software()->application()->find_module_by_name('network');
            $network_module->check_object_modified($this,'ip',$this->name());
        }
        return $result;
    }
    
    public function get_dhcp_ip(){
        $ip_type = $this->ip_type();
        $interface = $this->get_interface_name();
        $dhcp_file_name = '/tmp/dhcp.' . $interface . '.' .$ip_type;
        //check if the dhclient process is running
        $dh_cmd = "sudo nohup ps -AF|grep -v grep|grep 'dhclient-{$interface}'";
        $cmd_rs = $this->node()->execute($dh_cmd, $cmd_output, false);
        if($cmd_output && file_exists($dhcp_file_name)){
            //dhclient process is running just read the file content
            $tmp_string = file_get_contents($dhcp_file_name);
            $dhcp_data = json_decode($tmp_string, true);
            if(isset($dhcp_data['new_ip']) && $dhcp_data['new_ip'] !='' ){
                $new_ip_data['address'] = $dhcp_data['new_ip'];
                if($dhcp_data['new_mask']){
                    if($ip_type == '4'){
                        $new_ip_data['prefix'] = safe_netmask2cidr($dhcp_data['new_mask']);
                    }else{
                        $new_ip_data['prefix'] = $dhcp_data['new_mask'];
                    }
                }
                $this->set_data_values($new_ip_data);
                $dhcp_result = parent::save();
                return true;
            }
        }
        return false;
    }

    public static function model(&$node){
        $module = $node->software()->application()->find_module_by_name('network');
        $class = get_called_class();
        return new $class($module->node(),$module->object_name().'/ip',$module->build_unique_name('ip'));
    }
}
//
class Sng_loopback_ip_class  extends Sng_ip_address_class
{
    public function configure()
    {
        $interfaces_enum = array('lo' => 'lo');
        $this->add_field('read_only', 'Read Only', 'hidden', 'true');
        $this->add_enum_field('interface', 'Interface', 'dropdown', '', $interfaces_enum);
        $proto_enum = array(
                Safe_ip_address_class::PROTO_STATIC4=>'IPv4 - Static',
                Safe_ip_address_class::PROTO_STATIC6 =>'IPv6 - Static'
        );
        $this->set_field_disabled('interface', true);
        $this->add_enum_field('proto', 'Configuration', 'dropdown', Safe_ip_address_class::PROTO_STATIC4, $proto_enum);
        $this->set_field_disabled('proto', true);
        $this->add_field('hostname', 'Hostname', 'text', '', 40);
        $this->set_field_disabled('hostname', true);
        $this->add_field('address', 'Address', 'text', '', 40);
        $this->set_field_disabled('address', true);
        $this->add_field('prefix', ' / ', 'string', '', 10);
        $this->set_field_disabled('prefix', true);
        $this->composite_layout('address', array('prefix'));
    }
    public function is_readonly()
    {
        if($this->node()->get_config_status('update') == 'rest'){
            return false;
        }else{
            return true;
        }
    }
    public function prepare_edit()
    {
        if($this->node()->get_config_status('update') == 'rest'){
            $this->set_field_disabled('proto', false);
            $this->set_field_disabled('hostname', false);
            $this->set_field_disabled('address', false);
            $this->set_field_disabled('prefix', false);
        }
    }
    public function save(){
        if($this->node()->get_config_status('update') == 'rest'){
            return parent::save();
        }else{
            return true;
        }
    }
    public function can_dispose(&$reason = null)
    {
        if($this->node()->get_config_status('update') == 'rest'){
            return parent::can_dispose();
        }else{
            $reason = " Loopback IP cannot be deleted. ";
            return false;
        }
    }
    public function can_modify(&$output = null)
    {
        if($this->node()->get_config_status('update') == 'rest'){
            return parent::can_modify();
        }else{
            $this->set_error('interface', "Loopback IP cannot be modified.", $output);
            return false;
        }
    }
    public function validate($data, $output)
    {
        if(isset($data['peerdns'])){
            unset($data['peerdns']);
        }
        if(isset($data['persistent'])){
            unset($data['persistent']);
        }
        return parent::validate($data, $output);
    }
}
class Sng_network_ip_add_config extends Safe_configuration_user_class
{
    protected $_operator = 'add';

    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_CREATE)
    {
        parent::__construct($route_name, $action, $data);
    }
    public function run($node)
    {
        $data = $this->data();
        $ip = $data['address'];
        $cidr = $data['prefix'];
        $interface = $data['interface'];
        $proto = $data['proto'];
        $dhcp = array(Safe_ip_address_class::PROTO_DHCP4, Safe_ip_address_class::PROTO_DHCP6, Safe_ip_address_class::PROTO_SLAAC6);
        if(in_array($proto, $dhcp)){
            //pass dhcp
            return true;
        }else{
            if(empty($ip)){
                return false;
            }
        }
        $broadcast = '';
        if(filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)){
            $ip_type = Safe_ip_address_class::IP_V6;
        }else{
            $ip_type = Safe_ip_address_class::IP_V4;
            if($this->_operator == 'add' && !in_array($proto, $dhcp)){
                //ipv4 need add broadcast
                $ip_nmask = safe_cidr2netmask($cidr);
                $ip_range = safe_get_ip_range($ip, $ip_nmask);
                $broadcast = ' broadcast '. long2ip($ip_range['broadcast']) . ' ';
            }
        }
        if ($this->_operator == 'del' && (($ip == '127.0.0.1' && $ip_type == 4) || ($ip == "::1" && $ip_type == 6))) {
            safe_log(SAFE_INFO, "skipping removal of loopback IP address ($ip)..");
            return true;
        }

        // Prepare command:
        $cmd = '/sbin/ip -' . $ip_type . ' addr ' . $this->_operator . ' ' . $ip . '/' . $cidr . $broadcast . ' dev '. $interface;
        $node->execute('sudo nohup ' . $cmd);
        sleep(1);
        return true;
    }
}
class Sng_network_ip_delete_config extends Sng_network_ip_add_config
{
    protected $_operator = 'del';

    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_DELETE)
    {
        parent::__construct($route_name, $data, $action);
    }
}
?>
