<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Network service configuration wrapper
 *
 * @author William Adam
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('service');
safe_require_class('safe_ip_address_class');
safe_module_require_class('sng', 'network_interface');
safe_module_require_class('sng', 'network_ip');
safe_module_require_class('sng', 'network_source_policy_routing');
class Sng_network_config_class extends Safe_configurable_object_class
{
    static function get_validation_rule($field, $mode){
        $rule = '';

        if($field == 'hostname'){
            /* same validation for both modes Sng_cluster_config_class::HA_MODE and Sng_cluster_config_class::STANDALONE_MODE*/
            $rule = 'required|regex_match[' .  Sng_network_service_class::DOMAIN_REGEX . ']';
        }

        return $rule;
    }

    /**
     * @brief Constructor
     *
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return
     */
    public function __construct($node, $parent_name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, "configuration", $node);
    }
    /**
     * @brief Configure the config class object
     *
     * @return
     */
    public function configure()
    {
        // General settings
        $this->add_field('global/networking', 'networking', 'hidden', 'yes', 20);
        $this->add_field('global/nozeroconf', 'nozeroconf', 'hidden', 'yes', 20);
        $this->add_field('global/hostname', 'Host Name', 'text', '', 40);
        $this->set_field_help('global/hostname','Host Name should be the Fully Qualified Domain Name (FQDN), such as hostname.expample.com.');
        $this->set_field_rules('global/hostname', Sng_network_config_class::get_validation_rule('hostname', ''));
        $this->set_ha_field_rules('global/hostname', Sng_network_config_class::get_validation_rule('hostname', Sng_cluster_config_class::HA_MODE));

        // Add General category
        $this->set_field_category('global/hostname','General');

        $this->add_field('global/gatewaydev', 'Default Gateway Interface', 'text', '', 50);
        $this->set_field_help('global/gatewaydev',
                array('Default Gateway Interface is the gateway device, such as eth0. ',
                       'Configure this option if you have multiple interfaces on the same subnet,',
                       'and require one of those interfaces to be the preferred route to the default gateway.'
                      )
               );

        $this->add_field('ipv4/default_gw', 'Default IPV4 Gateway', 'text', '', 40);
        $this->set_field_rules('ipv4/default_gw', 'valid_ip_or_domain[ipv4]');
        $this->set_field_help('ipv4/default_gw','IPV4 address of the network\'s gateway.');

        $this->add_field('ipv6/default_gw', 'Default IPV6 Gateway', 'text', '', 40);
        $this->set_field_rules('ipv6/default_gw', 'valid_ip_or_domain[ipv6]');
        $this->set_field_help('ipv6/default_gw','IPV6 address of the network\'s gateway.');
        $this->set_field_category('global/gatewaydev','Gateway');

        foreach (array(1,2,3,4) as $num) {
            $this->add_field('dns/' . $num, 'Static DNS #' . $num, 'text', '', 40);
            $this->set_field_rules('dns/' . $num, 'valid_ip_or_domain');
        }
        $this->set_field_category('dns/1','DNS');
        $this->set_field_help('dns/1','Static Name server IP address.');

        // where do we find dynamic resolv information
        $resolvconfdir = '/var/run/resolvconf/interfaces/';

        if (is_dir($resolvconfdir)) {
            $autonum = 1;
            foreach (scandir($resolvconfdir) as $num => $filename){
                if (strpos($filename, 'dhcp') === false)
                    continue;

                $tmp_data = explode(".", $filename);
                $ifname = $tmp_data[0];
                $seqnum = 0;
                $lines = file($resolvconfdir . $filename, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
                foreach ($lines as $lnum => $ldata) {
                    if (strpos($ldata, 'nameserver') !== 0)
                        continue;

                    $dnsdata = preg_split('/[ \t;#]+/', $ldata);

                    $fld_name = 'autodns/' . $ifname . '/' . $seqnum;
                    $fld_text = 'Automatic DNS #' . $autonum . ' (' . $ifname . ')';

                    $this->add_field($fld_name, $fld_text, 'hidden', $dnsdata[1], 40);
                    $this->set_field_readonly($fld_name);

                    if ($autonum == 1)
                        $this->set_field_category($fld_name, 'Automatic DNS');

                    $seqnum++;
                    $autonum++;
                };
            };
        };

        return parent::configure();
    }
    public function validate($data="", &$output="")
    {
        if($this->node()->get_config_status('update') == 'rest'){
            //check if it is configuration update from rest pass by check_gateway_subnet
            $check_gateway_subnet = false;
            //by pass hostname validate
        }else{
            $check_gateway_subnet = true;
        }
        foreach($data as $key => $value) {
            if (preg_match("/autodns/i", $key)) {
                /* Autodns setting might be from HA master while configuring slave after reset*/
                safe_log(SAFE_LOG_INFO, "Clearing autodns from network settings: " . $key . " => " . $value);
                unset($data[$key]);
            }
        }

        $parent_result = parent::validate($data, $output);
        if($parent_result){
            $dns_list = array();
            $cfg_data = $this->get_data_values_exploded(false);
            
            // if a gateway is specified, it must be on same subnet as interface
            if($check_gateway_subnet){
                $network_module = $this->node()->software()->application()->find_module_by_name('network');
                $gw_output = array();
                $tmp_rs = $network_module->check_gateway($gw_output);
                if(!$tmp_rs){
                    if($gw_output){
                        foreach($gw_output as $key_name => $out_message){
                            $this->set_error($key_name, $out_message, $output);
                        }
                        $parent_result = false;
                    }
                }
            }
            foreach ($cfg_data['dns'] as $dns_id => $dns){
                $dns_field_name = 'dns/'.$dns_id;
                if($dns!=''){
                    if(!in_array($dns, $dns_list)){
                        $dns_list[] = $dns;
                    }else{
                        $rule_msgs['dns'] = "This DNS is duplicated.";
                        $this->set_error($dns_field_name, $rule_msgs['dns'], $output);
                        $parent_result = false;
                    }
                }
            }    
        }
        return $parent_result;
    }

    public function summary($type = 'horizontal' , $long = false, $min=16)
    {
        $rs = parent::summary($type, $long, $min);
        $cfg_data = $this->get_data_values_exploded(true);
        if($cfg_data['global']['gatewaydev']){
            //support vlan as default gateway
            $network_module = $this->node()->software()->application()->find_module_by_name('network');
            $if_obj = $network_module->api_retrieve_interface($cfg_data['global']['gatewaydev']);
            if($if_obj){
                $if_name = $if_obj->interface_name();
            }else{
                $if_name = '';
            }
            $rs['data']['Default Gateway Interface'] = $if_name;
        }
        foreach ($cfg_data['dns'] as $dns_id => $dns){
            $dns_label_name = $this->get_data_label('dns/'.$dns_id);
            if(empty($rs['data'][$dns_label_name])){
                unset($rs['data'][$dns_label_name]);
            }
        }
        if(isset($cfg_data['autodns'])){
            foreach ($cfg_data['autodns'] as $iface_id => $iface_data){
                foreach ($iface_data as $dns_id => $dns){
                    $dns_label_name = $this->get_data_label('autodns/'.$iface_id.'/'.$dns_id);
                    $rs['data'][$dns_label_name] = $dns;
                }
            }
        }
        return $rs;
    }
    public function set_gateway($type = Safe_ip_address_class::IP_V6, $data = null)
    {
        $new_data = array();
        if($type == Safe_ip_address_class::IP_V6){
            $field_name = 'ipv6/default_gw';
        }else{
            $field_name = 'ipv4/default_gw';
        }
        if(isset($data['gateway']) && $data['gateway'] !=''){
            $new_data[$field_name] = $data['gateway'];
        }
        if(isset($data['interface']) && $data['interface'] !=''){
            $new_data['global/gatewaydev'] = $data['interface'];
        }
        if($new_data){
            $this->set_data_values($new_data);
        }
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $interface_list = $network_module->get_interface_list(Sng_network_interface_class::MANAGEMENT, false);
        $interfaces_enum = array('' => '(Auto)');
        if($interface_list){
            foreach($interface_list as $key => $tmp_value){
                if($key == 'lo'){
                    continue;
                }
                $interfaces_enum[$key] = $tmp_value;
            }
            $this->add_enum_field('global/gatewaydev', 'Default Gateway Interface', 'dropdown', '', $interfaces_enum);
        }

        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        $clustermon_service = $this->node()->software()->application()->find_module_by_name('clustermon');
        if($cluster_module->get_cluster_mode() == 'active-passive'){
            if($clustermon_service->is_local_node_online()){
                $this->set_field_disabled('global/hostname', true);
            }else{
                $hostname_message = "Copy configuration to " . $cluster_module->peer_node_type() . " and then reboot is required after the change.";
                $hostname_change = array(
                    "oninput" => "warnOnInput(event,'" . strval($hostname_message) . "')"
                );
                $this->set_field_attributes('global/hostname', $hostname_change);
            }
        }
    }
}
class Sng_network_route_add_config extends Safe_configuration_user_class
{
    protected $_operator = 'add';

    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_CREATE)
    {
        parent::__construct($route_name, $action, $data);
    }
    public function run($node)
    {
        $data = $this->data();
        if(filter_var($data['gateway'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)){
            $ip_type = Safe_ip_address_class::IP_V6;
        }else{
            $ip_type = Safe_ip_address_class::IP_V4;
        }
        // Prepare command:
        //  ip route add <network-addr>/<network-mask-bits> [via <gateway-addr>] dev <ethernet-device-name>
        $cmd = '/sbin/ip -' . $ip_type . ' route '.$this->_operator.' ';

        if(isset($data['default'])){
            //ip route add default via fe80::207:cbff:aaaa:bbbb dev eth0
            $cmd .= ' default ';
        }else{
            $cidr = $data['prefix'];
            $network = safe_cidr2network($data['address'], $cidr);
            $cmd .= $network;
            // Nework route or host route ?
            if($cidr) {
                $cmd .= '/'.$cidr;
            }
        }

        if (strlen(trim($data['gateway']))){
            $cmd .= ' via '.$data['gateway'];
        }
        if(isset($data['interface']) && !empty($data['interface'])){
            $interface = str_replace('_', '.', $data['interface']);
            $cmd .= ' dev '. $interface;
        }
        $node->execute('sudo nohup ' . $cmd);
        // For now always return true
        // Known valid usecase when ip route command fail:
        //   * add a route on a down interface
        //
        return true;
    }
}

class Sng_network_route_delete_config extends Sng_network_route_add_config
{
    protected $_operator = 'del';
    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_DELETE)
    {
        parent::__construct($route_name, $data, $action);
    }
}
//for change the gateway
class Sng_network_route_replace_config extends Sng_network_route_add_config
{
    protected $_operator = 'replace';
    public function __construct($route_name, $data = array(), $action=Safe_configuration_class::CFG_DELETE)
    {
        parent::__construct($route_name, $data, $action);
    }
}

class Sng_network_route_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
        // Do NOT re-use records on update/delete
        $this->_set_reuse(false);
    }
    public function configure()
    {
        // General settings
        $network_module = $this->node()->software()->application()->find_module_by_name('network');
        $interfaces = $network_module->api_retrieve_interface();
        $interfaces_enum = array();
        foreach($interfaces as $key => $obj){
            if($obj->is_bonded()){
                continue;
            }
            $interfaces_enum[$obj->name()] = $obj->interface_name();
        }

        $this->add_field('address', 'Destination', 'text', '',40);
        $this->set_field_help('address', 'IP address.');
        $this->set_field_rules('address', 'required|valid_ip');

        $this->add_field('prefix', 'Prefix', 'text', '',5);
        $this->set_field_help('prefix', 'The network prefix for the destination IP address.');
        $this->set_field_rules('prefix','required|greater_than[1]|less_or_equal[128]');
        $this->composite_layout('address', array('prefix'));

        $this->add_enum_field('interface', 'Network Interface', 'dropdown', '', $interfaces_enum);
        $this->set_field_help('interface', 'The network interface used to reach destination IP address.' );

        $this->add_field('gateway', 'Gateway', 'text', '',40);
        $this->set_field_help('gateway', 'The default gateway, or an IP address that can be used to reach the destination IP address.');
        $this->set_field_rules('gateway', 'valid_ip');

        return parent::configure();
    }

    public function description()
    {
        $data = $this->get_data_values();
        return $data['interface'] . ' / ' . $data['prefix'] . ' - ' . $data['gateway'];
    }
    public function summary($type , $long, $min = 4)
    {
        return parent::summary($type, $long, $min);
    }
    public function validate($data, &$output)
    {
        // Run regular validation first
        $validate = parent::validate($data, $output);
        if (!$validate) {
            return $validate;
        }
        // Now take care of extra rules:
        // 1/ destination must be unique
        $network_module = $this->node()->find_module_by_name('network');
        if($data['address']){
            $routes = $network_module->api_retrieve_route();
            if($routes){
                foreach($routes as $route){
                    // Skip ourself
                    if($this->name() == $route->name()){
                        continue;
                    }
                    $route_data = $route->get_data_values(false);
                    // 1/ check unique route
                    if($data['address'] == $route_data['address']){
                        $validate = false;
                        $this->set_error('address', 'Destination already used by route ' . $route->name(), $output);
                        break;
                    }
                }
            }
        }
        // 2/ if a gateway is specified, it must be on same subnet as interface
        if($data['gateway']){
            // Get adapter and associated IP address
            // if update from rest api, may not include 'interface'
            $ips = $network_module->get_ips($this->get_data_value('interface', false));
            if(filter_var($data['gateway'], FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)){
                $gateway_ip_type = Safe_ip_address_class::IP_V6;
            }else{
                $gateway_ip_type = Safe_ip_address_class::IP_V4;
            }
            $sub_net = false;
            if($ips){
                foreach($ips as $ip_obj_name => $ip_obj){
                    $ip_address = $ip_obj->address();
                    if($ip_address &&  $gateway_ip_type == $ip_obj->ip_type()){
                        $ip_prefix = $ip_obj->prefix();
                        //ipv4 && ipv6
                        if(safe_cidr_match($data['gateway'], safe_cidr2network($ip_address, $ip_prefix), $ip_prefix)){
                            $sub_net = true;
                            break;
                        }
                    }
                }
            }
            if(!$sub_net){
                $this->set_error('gateway', 'Gateway is not on '.$data['interface'].' subnet.', $output);
                $validate = false;
            }
        }
        return $validate;
    }
}



class Sng_network_service_class extends Safe_service_class
{
    const DOMAIN_REGEX = '/^(?=.{1,254}$)((?=[a-z0-9-]{1,63}\.)(xn--)?[a-z0-9]+(-[a-z0-9]+)*\.)+[a-z]{2,63}$/i';
    const TMP_FILE = '/tmp/restart.network';
    
    public function __construct($software)
    {
        parent::__construct($software->node()->software(), "network", $process_name);
    }

    /**
     * @brief
     *
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("Network");
        // Create the config object
        $cfg = new Sng_network_config_class($this->node(), $this->path() . '/' . $this->name());
        // Synch with DB
        $cfg->configure();
        $cfg->synch();
        // Attach config to module
        $this->set_config($cfg);

        $this->register_aggregate_object('interface',
                array(
                        'name' => 'Interface',
                        'dynamic' => true,
                        'base_path' => parent::object_name().'/interface',
                        'autoname' => true,
                        'controller' => array(
                                'config_manager' => array(
                                        'allow_create_button' => 'add[Add VLAN]',
                                        'summary_buttons' => 'Edit|Delete:confirm|Up:confirm|Down:confirm'
                                ),
                        ),
                        'methods' => array(
                                'create' => array(
                                        'name' => 'Create',
                                        'description' => 'Create a interface',
                                        'request' => 'POST',
                                ),
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a interface',
                                        'request' => 'GET',
                                ),
                                'update' => array(
                                        'name' => 'Update',
                                        'description' => 'Update a interface',
                                        'request' => 'POST',
                                ),
                                'delete' => array(
                                        'name' => 'Delete',
                                        'description' => 'Delete a interface',
                                        'request' => 'POST',
                                ),
                                'status' => array(
                                        'name' => 'Status',
                                        'description' => 'Retrieve interface status',
                                        'request' => 'GET',
                                ),
                                'up' => array(
                                        'name' => 'Up',
                                        'description' => 'Set interface up',
                                        'request' => 'POST',
                                ),
                                'down' => array(
                                        'name' => 'Down',
                                        'description' => 'Set interface down',
                                        'request' => 'POST',
                                ),
                        ),
                )
        );
        $this->register_aggregate_object('ip',
                array(
                        'name' => 'IP',
                        'dynamic' => true,
                        'base_path' => parent::object_name().'/ip',
                        'controller_url' => '/SAFe/sng_network_config',
                        'autoname' => true,
                        'methods' => array(
                                'create' => array(
                                        'name' => 'Create',
                                        'description' => 'Create a IP',
                                        'request' => 'POST',
                                ),
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a IP',
                                        'request' => 'GET',
                                ),
                                'update' => array(
                                        'name' => 'Update',
                                        'description' => 'Update a IP',
                                        'request' => 'POST',
                                ),
                                'delete' => array(
                                        'name' => 'Delete',
                                        'description' => 'Delete a IP',
                                        'request' => 'POST',
                                ),
                        ),
                )
        );

        // Register objects
        $this->register_aggregate_object('route',
            array(
                'name' => 'Static Route',
                'base_path' => $this->object_name() . '/route',
                'dynamic' => true,
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a Static Route',
                        'request' => 'POST'
                        ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a Static Route',
                        'request' => 'GET'
                        ),
                    'update' => array(
                            'name' => 'Update',
                            'description' => 'Update a Static Route',
                            'request' => 'POST'
                    ),
                    'delete' => array(
                            'name' => 'Delete',
                            'description' => 'Delete a Static Route',
                            'request' => 'POST'
                    )
               )
            )
        );
        $this->register_aggregate_object('source_policy_routing',
            array(
                'name' => 'Source Policy Routing',
                'dynamic' => true,
                'base_path' => parent::object_name().'/source_policy_routing',
                'controller_url' => '/SAFe/sng_network_config',
                'autoname' => true,
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Create a Source Policy Routing',
                        'request' => 'POST',
                    ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a Source Policy Routing',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update a Source Policy Routing',
                        'request' => 'POST',
                    ),
                    'delete' => array(
                        'name' => 'Delete',
                        'description' => 'Delete a Source Policy Routing',
                        'request' => 'POST',
                    ),
                ),
            )
        );

        //no need to auto create ip obj for hw interface
        //$this->get_aggregate_objects('interface');
        return parent::configure();
    }
    // for interface check adapters, and match interface
    public function get_aggregate_objects($type, $all = false, $params = array()){
        $objs = parent::get_aggregate_objects($type, $all, $params);
        ksort($objs);
        return $objs;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_interface($name, $data=null, &$output = null) {
        if(isset($data['type'])){
            $class = '';
            //for configuration update, to create physical interface 
            if($data['type'] == Sng_network_interface_class::PHYSICAL || $data['type'] ==Sng_network_interface_class::LOOPBACK){
                $class = "Sng_network_interface_class";
            }elseif($data['type'] == Sng_network_interface_class::VIRTUAL){
                $class = "Sng_network_link_class";
            }elseif($data['type'] == Sng_network_interface_class::BONDING){
                $class = "Sng_network_bonding_class";
            }else{
                $output['type'] = 'Interface type error.';
                return false;
            }
        }else{
            //no data
            $class = "Sng_network_link_class";
        }
        if($class){
            $obj = new $class($this->node(), $this->object_name().'/interface', $name);
            $obj->configure();
            return $obj;
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_interface($name=null, $data=null, &$output = null) {
        //get all adapters
        $objs = $this->get_aggregate_objects('interface');
        if($name) {
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    public function get_interface_list($type = 'all', $return_obj = true) {
        //get all adapters
        $interfaces = $this->api_retrieve_interface();
        $list = array();
        switch ($type) {
            case 'all' :
                $list = $objs;
                break;
            case 'physical_management' :
                //only hardware eth interfaces 
                foreach($interfaces as $key => $obj){
                    if($obj->interface_type() == Sng_network_interface_class::PHYSICAL &&
                            $obj->hardware_type() == Sng_network_interface_class::MANAGEMENT){
                        $list[] = $obj;
                    }
                }
                break;
            case 'management' :
                //only management interface include vlan, bonding interface(no bonded interface )
                foreach($interfaces as $key => $obj){
                    if($obj->hardware_type() == Sng_network_interface_class::MANAGEMENT && !$obj->is_bonded()){
                        //filter out bonded interface
                        $list[] = $obj;
                    }
                }
                break;
            case 'bonding' :
                foreach($interfaces as $key => $obj){
                    if($obj->interface_type() == Sng_network_interface_class::BONDING){
                        $list[] = $obj;
                    }
                }
                break;
        }
        
        if(!$return_obj){
            //return name list
            $interfaces_enum = array();
            if($list){
                foreach($list as $key => $obj){
                    $interfaces_enum[$obj->name()] = $obj->interface_name();
                }
            }
            return $interfaces_enum;
        }
        return $list;
        
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_interface($name, $data=null, &$output = null) {
        $obj = $this->api_retrieve_interface($name);
        if($obj) {
            if ($obj->validate($data,$output)) {
                if (true == $obj->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_delete_interface($name, $data=null, &$output = null) {
        $obj = $this->api_retrieve_interface($name);
        if($obj) {
            if($obj->can_dispose($output)) {
                $macvlan_device = $obj->get_data_value('macvlan_device', false);

                if (strncmp($macvlan_device, 'sngdsp', 6) === 0) {
                    $ip_objs = $this->get_aggregate_objects('ip');
                    foreach($ip_objs as $ip_obj_name => $ip_obj) {
                        if ($ip_obj->get_interface() == $name){
                            $ip_obj->dispose();
                        }
                    }
                }

                return $obj->dispose();
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    private $_sysconfig_data = array();
    private function _get_sysconfig_data(&$output = null) {
        if(empty($this->_sysconfig_data)){
            $cmd_result = array();
            $script_path = $this->node()->configuration_manager()->directory('sng-base') . '/bin';
            $cmd1 = "{$script_path}/sng-system-stats" ;
            $this->node()->execute($cmd1, $cmd_result);
            if($cmd_result){
                $system_config_data = json_decode(join(PHP_EOL,$cmd_result), true);
                $system_config_data = $system_config_data['network'];
                $network_status = array();
                // Let's massage a little this output as not all fields are
                // interesting: see interesting fields below
                $tokens = array(
                    'duplex',
                    'carrier',
                    'link_mode',
                    'mtu',
                    'operstate',
                    'speed',
                );
                foreach($system_config_data as $if_name => $if_stats){

                    foreach($tokens as $k){
                        $network_status[$if_name][$k] = $if_stats[$k];
                    }
                    // Get statistics of course
                    $network_status[$if_name]['stats'] = $if_stats['statistics'];
                }
                $this->_sysconfig_data = $network_status;
            }else{
                $output['message'] = 'Get system status error.';
            }
        }
        return $this->_sysconfig_data;
    }
    public function api_status_interface($name=null, $data=null, &$output = null) {
        //$name : interface obj name
        $output = array();
        $sysconfig_data = $this->_get_sysconfig_data($output);
        //if no data show down
        $interface_list = $this->api_retrieve_interface();
        if($interface_list){
            foreach($interface_list as $key => $interface_obj){
                if(!isset($sysconfig_data[$key])){
                    // Check if its a VLAN interface
                    if ($interface_obj->get_data_value('vlan_type', false) == 'vlan') {
                        $vlan_int = $interface_obj->get_data_value('ifname', false) . '.' . $interface_obj->get_data_value('id', false);

                        // Check for duplicate VLAN interfaces
                        if (!array_key_exists($vlan_int, $sysconfig_data)){
                            $sysconfig_data[$key] = array('operstate' => 'down',
                                'stats' => array(
                                    'tx_bytes' => '',
                                    'replace' => '',
                                    'rx_bytes' => ''
                                )
                            );
                        }
                    } else {
                        $sysconfig_data[$key] = array('operstate' => 'down',
                            'stats' => array(
                                'tx_bytes' => '',
                                'replace' => '',
                                'rx_bytes' => ''
                            )
                        );
                    }
                }
            }
        }
        if($name){
            return $sysconfig_data[$name];
        }else{
            return $sysconfig_data;
        }
    }
    /**
     * @brief
     *
     *
     * @return all interface obj name => system interface name array
     */
    public function get_interface_sys_name_list(){
        $sys_name_list = array();
        $interface_key_list = $this->get_aggregate_objects_keys('interface');
        if($interface_key_list){
            foreach($interface_key_list as $interface_obj_name){
                $obj =  $this->get_aggregate_object('interface', $interface_obj_name);
                if($obj){
                    $sys_name = $obj->interface_name();
                    $sys_name_list[$interface_obj_name] = $sys_name;
                }
            }
        }
        return $sys_name_list;
    }
    public function api_create_ip($name, $data=null, &$output = null) {
        if(isset($data['interface']) && $data['interface'] == 'lo'){
            $class = "Sng_loopback_ip_class";
        }else{
            $class = "Sng_ip_address_class";
        }

        $obj = new $class($this->node(), $this->object_name().'/ip', $name);
        $obj->configure();
        return $obj;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_ip($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('ip');
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_ip($name, $data=null, &$output = null) {
        $ip = $this->api_retrieve_ip($name);
        if($ip) {
            if ($ip->validate($data,$output)) {
                if (true == $ip->save($output)) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_delete_ip($name, $data=null, &$output = null) {
        $ip = $this->api_retrieve_ip($name);
        if($ip) {
            if($ip->can_dispose($output)) {
                return $ip->dispose();
            }
        }
        return false;
    }

    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_route($name, $data=null, &$output = null) {
        $route = new Sng_network_route_class($this->node(), $this->object_name().'/route', $name);
        $route->configure();
        if($data && $route->validate($data, $output)){
            $route->save();
        }
        return $route;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_route($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('route');
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_route($name, $data=null, &$output = null) {
        // We cannot update static route object, must delete old one and create
        // a new one so serialization can occur correctly (ie. delete using ip
        // route del with EXACT same parameters as create.
        // Only exception is if route is not yet serialized, then we can just
        // update as usual (ie. status == NEW)
        $route = $this->api_retrieve_route($name);
        if($route) {
            if ($route->validate($data,$output)) {
                return $route->save($output);
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_delete_route($name, $data=null, &$output = null) {
        $route = $this->api_retrieve_route($name);
        if($route) {
            if($route->can_dispose($output)) {
                return $route->dispose();
            }
        }
        return false;
    }

    public function can_restore($info, &$reason)
    {
        // Prevent calling parent as service status will be checked
        return true;
    }
    /**
     * @brief Create config jobs for dialplan module
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        return $this->generate_config($config_manager);
    }
    /**
     * @brief
     *
     * @param[in out] $obj_type
     *
     * @return
     */
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }

    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    public function generate_config(&$config_manager)
    {
        if(!$this->_generate_config($config_manager)){
            return false;
        }
        return parent::generate_config($config_manager);
    }
    /**
     * @brief Invoked after a successfull write_config
     *
     * @return
     */
    public function post_write_config($obj_type=null)
    {
        // restart service
        //$this->restart();
        return parent::post_write_config($obj_type);
    }
    public function support_reload(&$need_reload = null) {
        parent::support_reload($need_reload);
        return true;
    }
    private function _generate_config_global(&$config_manager)
    {
        //check if default gateway is on the same subnet
        if(!$this->check_gateway($output, $gw_dev)){
            if($output){
                $tmp_error = join(',' , $output);
            }else{
                $tmp_error = 'Default Gateway error.';
            }
            $config_manager->add_error(array(
                    'module' => 'network',
                    'obj_type' => 'configuration',
                    'description' => $tmp_error,
                    'url' => '/SAFe/sng_network_config/modify/network'
            ));
        }
        //Network Global Configuration
        //write the Global Configuration  to /etc/sysconfig/network and /etc/resolv.conf
        //the ifup/ifdown and dhclient will take care the dynamic gateway or dns from dhcp server
        $network_global_config_file = "/etc/sysconfig/network";
        $network_dns_file = "/etc/resolv-static.conf";
        $hostname_file = "/etc/hostname";
        $hostname_value = "";
        $network_global_data = $this->config()->get_data_values_exploded(false);
        $network_global_buffer = array();
        $dns_buffer = array();
        //get the real interface name
        if($network_global_data['global']['gatewaydev']){
            $if_name = '';
            if($network_global_data['global']['gatewaydev']){
                $if_obj = $this->api_retrieve_interface($network_global_data['global']['gatewaydev']);
                if($if_obj){
                    $if_name = $if_obj->interface_name();
                } 
            }
            $network_global_data['global']['gatewaydev'] = $if_name;
        }
        
        
        foreach($network_global_data as $key => $value){
            if($key == 'ipv4'){
                if(!empty($network_global_data[$key]['default_gw'])){
                    //add default ipv4 gw
                    $route_data = array();
                    $route_data['default'] = true;
                    $route_data['gateway'] = $network_global_data[$key]['default_gw'];
                    //to do show detect the first active Ethernet
                    $route_data['interface'] = '';
                    if($network_global_data['global']['gatewaydev']){
                        $route_data['interface'] = $network_global_data['global']['gatewaydev'];
                    }else{
                        if(isset($gw_dev['gw_interface_dev']['ipv4'])){
                            $route_data['interface'] = $gw_dev['gw_interface_dev']['ipv4'];
                        }
                    }
                    $config_manager->add_config(
                            new Sng_network_route_replace_config(
                                    'default', $route_data) );
                    $network_global_buffer[] = sprintf("%s=%s", 'GATEWAY', $network_global_data[$key]['default_gw']);
                }
            }elseif($key == 'ipv6'){
                //if no ipv6 dhcp IP,the default_gw cannot empty
                if(!$have_ipv6_dhcp){
                    if(!empty($network_global_data[$key]['default_gw'])){
                        //replace default ipv6 gw
                        $route_data = array();
                        $route_data['default'] = true;
                        $route_data['gateway'] = $network_global_data[$key]['default_gw'];
                        //to do show detect the first active Ethernet
                        $route_data['interface'] = '';
                        if($network_global_data['global']['gatewaydev']){
                            $route_data['interface'] = $network_global_data['global']['gatewaydev'];
                        }else{
                            if(isset($gw_dev['gw_interface_dev']['ipv6'])){
                                $route_data['interface'] = $gw_dev['gw_interface_dev']['ipv6'];
                            }
                        }
                        $config_manager->add_config(
                                new Sng_network_route_replace_config(
                                        'default', $route_data) );
                        $network_global_buffer[] = sprintf("%s=%s", 'IPV6_AUTOCONF', 'no');
                        $network_global_buffer[] = sprintf("%s=%s", 'IPV6_DEFAULTGW', $network_global_data[$key]['default_gw']);
                    }
                }
                $network_global_buffer[] = sprintf("%s=%s", 'IPV6FORWARDING', 'yes');

            }elseif($key == 'global'){
                //Host Name could be empty after new install or restore from old backup
                if(empty($value['hostname'])){
                    $config_manager->add_error(array(
                            'module' => 'network',
                            'obj_type' => 'configuration',
                            'description' => 'The Host Name field is required.'
                    ));
                }else{
                    $hostname_value = $value['hostname'];
                }

                foreach($value as $sub_key => $sub_value){
                    $network_global_buffer[] = sprintf("%s=%s", strtoupper($sub_key), $sub_value);
                }

            }elseif($key == 'dns'){
                unset($value['mode']);
                foreach($value as $sub_key => $sub_value){
                    if(trim($sub_value)){
                        $dns_buffer[] = sprintf("nameserver %s", trim($sub_value));
                    }
                }
            }
        }
        if($this->node()->os()->check_file_exists($network_global_config_file)){
            $config_manager->add_config(new Safe_configuration_class($network_global_config_file, '', Safe_configuration_class::CFG_DELETE,Safe_configuration_class::CFG_FILE));
        }
        if($this->node()->os()->check_file_exists($hostname_file)){
            $config_manager->add_config(new Safe_configuration_class($hostname_file, '', Safe_configuration_class::CFG_DELETE,Safe_configuration_class::CFG_FILE));        
        }
        $config_manager->add_config(new Safe_configuration_class($hostname_file, $hostname_value, Safe_configuration_class::CFG_UPDATE,safe_configuration_class::CFG_FILE));
        $config_manager->add_config(new Safe_configuration_class($network_global_config_file, $network_global_buffer, Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_FILE));
        $config_manager->add_config(new Safe_configuration_class($network_global_config_file, 'root', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));


        if($this->node()->os()->check_file_exists($network_dns_file)){
            $config_manager->add_config(new Safe_configuration_class($network_dns_file, '', Safe_configuration_class::CFG_DELETE,Safe_configuration_class::CFG_FILE));
        }
        $config_manager->add_config(new Safe_configuration_class($network_dns_file, $dns_buffer, Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_FILE));
        $config_manager->add_config(new Safe_configuration_class($network_dns_file, 'root', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));

        return true;
    }

    private function _generate_config_source_policy_routing($config_manager){
        $source_policy_file = '/etc/sysconfig/network-scripts/source-policy-routing.cfg';
        if($this->node()->os()->check_file_exists($source_policy_file)){
            $config_manager->add_config(new Safe_configuration_class($source_policy_file,'',Safe_configuration_class::CFG_DELETE,Safe_configuration_class::CFG_FILE));            
        }

        $routes = array();
        $policy_routings = $this->api_retrieve_source_policy_routing();
        foreach($policy_routings as $policy_routing){
            $ip_obj = $policy_routing->ip_obj();
            if($ip_obj){
                $routes[] = $policy_routing->iface()."\t".$ip_obj->address()."/".$ip_obj->prefix()."\t".$policy_routing->get_data_value('gateway');
            }
        }
        $config_str = sprintf("GATEWAYS=(\n".implode("\n",$routes)."\n)");
        $config_manager->add_config(new Safe_configuration_class($source_policy_file,$config_str,Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_FILE));
        $config_manager->add_config(new Safe_configuration_class($source_policy_file,'root',Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
        if(count($routes)){
            $cfg = $this->config();
            $data = $cfg->get_data_values();
            $data['global/gatewaydev'] = '';
            $cfg->set_data_values($data);
            if($cfg->validate()){
                $cfg->save();
                $this->set_config($cfg);
            }
        }
        return true;
    }
    
    private function _generate_config_route(&$config_manager)
    {
        // Populate persistent routes
        // 1st remove existing
        $adapters = $this->api_retrieve_interface();
        $ip_type_file_names = array(
                SAFE_IPV4 => '/etc/sysconfig/network-scripts/route-',
                SAFE_IPV6 => '/etc/sysconfig/network-scripts/route6-'
                );
        foreach($adapters as $if){
            foreach($ip_type_file_names as $ip_type => $tmp_file_name){
                $file = $tmp_file_name.$if->name();
                if($this->node()->os()->check_file_exists($file)){
                    $config_manager->add_config(
                            new Safe_configuration_class(
                                    $file, '',
                                    Safe_configuration_class::CFG_DELETE,
                                    Safe_configuration_class::CFG_FILE));
                }
            }
        }
        // Loop around static routes to create interface route def
        $interfaces = array();
        foreach($this->get_aggregate_objects('route') as $route) {
            $data = $route->get_data_values(false);
            $address = $data['address'];
            $if = $data['interface'];
            unset($data['interface']);
            if(safe_get_ip_type($address) == SAFE_IPV4){
                //IPV4 route
                $id = count($interfaces[SAFE_IPV4][$if]);
                // For host route, need to add a /32 netmask
                if($data['prefix']) {
                    $data['netmask'] = safe_cidr2netmask($data['prefix']);
                    $data['address'] = safe_cidr2network($data['address'], $data['prefix']);
                }else{
                    $data['netmask'] = '255.255.255.255';
                }
                unset($data['prefix']);

                $route_def = array();
                foreach($data as $k => $v) {
                    $route_def[strtoupper($k).$id] = $v;
                }
                $interfaces[SAFE_IPV4][$if][] = $route_def;
            }else{
                //IPV6 route
                if(!$data['prefix']) {
                    $data['prefix'] = '128';
                }else{
                    $data['address'] = safe_cidr2network($data['address'], $data['prefix']);
                }
                if($data['gateway']){
                    $tmp_gateway = "via {$data['gateway']}";
                }else{
                    $tmp_gateway = '';
                }
                $interfaces[SAFE_IPV6][$if][] = "{$data['address']}/{$data['prefix']} {$tmp_gateway} dev {$if}";
            }
        }

        // Loop around interface route def to create file
        if($interfaces){
            foreach($interfaces as $ip_type => $tmp_interfaces) {
                foreach($tmp_interfaces as $if => $routes) {
                    $content = array();
                    foreach($routes as $route){
                        if($ip_type == SAFE_IPV4){
                            //IPv4 Write all entries
                            foreach($route as $k => $v){
                                $content[] = $k.'='.$v;
                            }
                        }else{
                            //IPv6
                            $content[] = $route;
                        }
                    }
                    if($adapters[$if]){
                        $file = $ip_type_file_names[$ip_type] . $adapters[$if]->interface_name();
                        $config_manager->add_config(
                                new Safe_configuration_class(
                                        $file, $content) );
                    }
                }
            }
        }

        // we MUST handled deleted routes before new ones
        // Loop around deleted static routes to create ip route del job
        $obj_path = $this->aggregate_object_base_path('route');
        $deleted_routes = Safe_object_serializer_class::get_serializer()->find_deleted_objects($obj_path);
        foreach($deleted_routes as $route => $route_data) {
            $config_manager->add_config(
                    new Sng_network_route_delete_config(
                            $route, $route_data['data']) );
        }

        // Loop around static routes to create ip route add job
        foreach($this->get_aggregate_objects('route') as $route) {
            // Only on new route
            if($route->status() == Safe_object_serializer_class::OBJ_STATUS_NEW) {
                $route_data = $route->get_data_values(false);
                $config_manager->add_config(
                        new Sng_network_route_add_config(
                                $route->name(), $route_data) );
            }
        }

        return true;
    }

    /**
     * @brief
     *
     * @param[in out] $config_manager
     *
     * @return
     */
    private function _generate_config(&$config_manager, $obj_type=null)
    {
        //interface configuration
        //ip configuration
        $ip_objs = $this->get_aggregate_objects('ip');

        //check if have IPV6 IP and DHCP IP
        $have_ipv6 = false;
        $have_ipv4_dhcp = false;
        $have_ipv6_dhcp = false;
        //hold the interfaces with DHCP IP on it (add/update/remove), need to restart them
        $interface_with_dhcp = array();

        foreach($ip_objs as $ip_obj_name => $ip_obj){
            if($ip_obj->ip_type() == Safe_ip_address_class::IP_V6){
                $tmp_interface_obj = $ip_obj->get_interface_obj();
                if($tmp_interface_obj
                        && $tmp_interface_obj->interface_type() != Sng_network_interface_class::LOOPBACK
                        && $tmp_interface_obj->hardware_type() !=Sng_network_interface_class::TRANSCODING){
                    //pass by the IPV6 IP on loopback and transcod
                    $have_ipv6 = true;
                }
                if($ip_obj->is_dhcp()){
                    $have_ipv6_dhcp = true;
                    if($ip_obj->status() != Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE){
                        $interface_with_dhcp[$ip_obj->get_interface_name()] = true;
                    }
                }
            }else{
                //IPV4
                if($ip_obj->is_dhcp()){
                    $have_ipv4_dhcp = true;
                    if($ip_obj->status() != Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE){
                        $interface_with_dhcp[$ip_obj->get_interface_name()] = true;
                    }
                }
            }
        }

        $interface_objs = $this->get_aggregate_objects('interface');
        $if_file_path = "/etc/sysconfig/network-scripts/ifcfg-";

        //delete related ifcfg- files
        $deleted_interface_objs = array();
        $deleted_interface_original_obj_name = array();
        $interface_obj_path = $this->aggregate_object_base_path('interface');
        if($interface_obj_path){
            $deleted_interface_objs = Safe_object_serializer_class::get_serializer()->find_deleted_objects($interface_obj_path);
            foreach($deleted_interface_objs as $interface_obj_name => $interface_data) {
                //remove interface ifcfg- file
                if(isset($interface_data['data']['ifname'])){
                    $interface_name = $interface_data['data']['ifname'];
                    if(isset($interface_data['data']['id'])){
                        $interface_name = $interface_data['data']['ifname'] . '.' . $interface_data['data']['id'];
                    }
                }
                //get the original interface obj name
                $original_obj_name = str_replace('__deleted__','',$interface_obj_name);
                if(strrpos($original_obj_name, '_') !== false){
                    $original_obj_name = substr( $original_obj_name , 0, strrpos($original_obj_name, '_'));
                }
                if(isset($interface_data['data']['type']) && $interface_data['data']['type'] == Sng_network_interface_class::BONDING){
                    $interface_name = $original_obj_name;
                }
                $deleted_interface_obj_name[$original_obj_name] = $interface_name;
                $if_file = $if_file_path . $interface_name;
                if($this->node()->os()->check_file_exists($if_file)){
                    $config_manager->add_config(new Safe_configuration_class($if_file, '', Safe_configuration_class::CFG_DELETE, Safe_configuration_class::CFG_FILE));
                }
            }
        }

        //remove the ip address from the changed interface using ip command
        $ip_obj_path = $this->aggregate_object_base_path('ip');
        $deleted_ip_objs = Safe_object_serializer_class::get_serializer()->find_deleted_objects($ip_obj_path);
        foreach($deleted_ip_objs as $ip_obj_name => $ip_data) {
            $interface_name = '';
            //neet to update  the interface_obj name to real interface name;
            $interface_obj_name = $ip_data['data']['interface'];
            if(isset($interface_objs[$interface_obj_name])){
                $interface_name = $interface_objs[$interface_obj_name]->interface_name();
            }elseif(isset($deleted_interface_original_obj_name[$interface_obj_name])){
                //interface could be deleted
                $interface_name = $deleted_interface_original_obj_name[$interface_obj_name];
            }
            if($interface_name){
                //check if the deleted IP is DHCP
                if($ip_data['data']['proto'] != Safe_ip_address_class::PROTO_STATIC4
                        && $ip_data['data']['proto'] != Safe_ip_address_class::PROTO_STATIC6){
                    $interface_with_dhcp[$interface_name] = true;
                }
                $ip_data['data']['interface'] = $interface_name;
                if($ip_data['data']['address']){
                    $config_manager->add_config(new Sng_network_ip_delete_config($ip_obj_name, $ip_data['data']));
                }
            }
        }

        //remove all 'ifcfg-*:*' for ipv4 virtual ip
        $vip_file_list = $this->node()->os()->search_files($if_file_path.'{*.*,sngdsp*}');
        foreach($vip_file_list as $file){
            $config_manager->add_config(new Safe_configuration_class($file, '', Safe_configuration_class::CFG_DELETE, Safe_configuration_class::CFG_FILE));
        }

        $adapters = $this->hardware_interface();
        $lo_interface = $this->api_retrieve_interface('lo');
        if($lo_interface){
            $adapters['lo'] = $lo_interface;
        }
        //group the ip objs by real interface name and ip type
        $ip_array = array();
        foreach($ip_objs as $ip_obj_name => $ip_obj) {
            $ip_array[$ip_obj->get_interface_name()][$ip_obj->ip_type()][$ip_obj_name] = $ip_obj;
        }
        if($interface_objs){
            foreach($interface_objs as $interface_name => $interface_obj){
                $interface_data = $interface_obj->get_data_values(false);
                if($interface_obj->interface_type() == Sng_network_interface_class::VIRTUAL){
                    // MACVLAN interface
                    if ($interface_obj->get_data_value('vlan_type', false) == 'macvlan') {
                        $interface = $interface_data['ifname'];
                        $if_name = $interface_obj->interface_name();
                    }
                    //VLAN interface
                    else {
                        $interface = $interface_data['ifname'];
                        $if_name = $interface . "." . $interface_data['id'];
                    }
                }else{
                    //hardware interface
                    $interface = $interface_name;
                    $if_name = $interface;
                }
                $if_name = $interface_obj->interface_name();
                $mac = "00:00:00:00:00:00";
                if(isset($adapters[$interface])){
                    $hw_interface_obj = $adapters[$interface];
                    $mac = $hw_interface_obj->mac();
                }
                //interface settings
                $interface_cfg_buffer = array();
                $interface_cfg_buffer[] = sprintf("%s=%s", 'DEVICE', $if_name);
                if($interface_obj->interface_type() == Sng_network_interface_class::VIRTUAL){
                    if ($interface_obj->get_data_value('vlan_type', false) === Sng_network_interface_class::MACVLAN) {
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'DEVICETYPE', 'macvlan');
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'TYPE', 'macvlan');
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'BOOTPROTO', 'none');
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'MACVLAN_PARENT', $interface);
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'MACVLAN_MODE', 'bridge');
                        $mac = $interface_obj->mac();
                    }
                    else {
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'TYPE', 'VLAN');
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'VLAN', 'yes');
                    }
                }elseif($interface_obj->interface_type() != Sng_network_interface_class::BONDING){
                    $interface_cfg_buffer[] = sprintf("%s=%s", 'TYPE', "Ethernet");
                }
                if($interface_obj->interface_type() != Sng_network_interface_class::BONDING){
                    $interface_cfg_buffer[] = sprintf("%s=%s", 'HWADDR', $mac);
                }
                $interface_cfg_buffer[] = sprintf("%s=%s", 'USERCTL', "no");
                if($interface_obj->interface_type() != Sng_network_interface_class::LOOPBACK &&
                        $interface_obj->interface_type() != Sng_network_interface_class::VIRTUAL &&
                        $interface_obj->interface_type() != Sng_network_interface_class::BONDING 
                        ){
                    if($interface_obj->hardware_type() == Sng_network_interface_class::TRANSCODING){
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'ETHTOOL_OPTS', '"-K ' . $if_name . ' gso off tso off highdma off sg off rx off tx off"');
                    }else{
                        if($interface_obj->autoneg() == 'false'){
                            $speed = $interface_obj->speed();
                            $duplex = $interface_obj->duplex();
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'ETHTOOL_OPTS', '"autoneg off speed ' . $speed . ' duplex ' . $duplex . '"');
                        }else{
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'ETHTOOL_OPTS', '"autoneg on"');
                        }    
                    }
                }
                //bonded interface
                $tmp_bonding_interface = $interface_obj->bonding_interface();
                if($tmp_bonding_interface){
                    $interface_cfg_buffer[] = sprintf("%s=%s", 'MASTER', $tmp_bonding_interface);
                    $interface_cfg_buffer[] = sprintf("%s=%s", 'SLAVE', 'yes');
                    $interface_cfg_buffer[] = sprintf("%s=%s", 'BOOTPROTO', 'none');
                }
                if($interface_obj->interface_type() == Sng_network_interface_class::BONDING ){
                    $mode = $interface_obj->get_data_value('bonding_type',false);
                    $interface_cfg_buffer[] = sprintf('%s="mode=%s miimon=100%s"','BONDING_OPTS',$mode,($mode==4? " lacp_rate=1" : ""));
                    $interface_cfg_buffer[] = sprintf("%s=%s", 'TYPE', 'Bond');
                    $interface_cfg_buffer[] = sprintf("%s=%s", 'BONDING_MASTER', 'yes');
                }

                //ip settings
                $ipv4_obj = null; //v4 primary
                $ipv6_obj = null; //v6 primary
                $ipv4_virtual_ip_obj = array();
                $ipv6_virtual_ip_obj = array();

                if(isset($ip_array[$if_name])){
                    //ipv4
                    if(isset($ip_array[$if_name][Safe_ip_address_class::IP_V4])){
                        $ipv4_array = $ip_array[$if_name][Safe_ip_address_class::IP_V4];
                        foreach($ipv4_array as $ip_obj_name => $ip_obj){
                            //first ip as primary
                            //or if is dhcp, set as primary
                            if($ipv4_obj == null || $ip_obj->is_dhcp()){
                                if($ipv4_obj != null){
                                    //if is dhcp, move the first ip to virtual_ip
                                    $ipv4_virtual_ip_obj[$ip_obj->name()] = $ipv4_obj;
                                }
                                $ipv4_obj = $ip_obj;
                            }else{
                                $ipv4_virtual_ip_obj[$ip_obj->name()] = $ip_obj;
                            }
                        }
                    }
                    //ipv6
                    if(isset($ip_array[$if_name][Safe_ip_address_class::IP_V6])){
                        $ipv6_array = $ip_array[$if_name][Safe_ip_address_class::IP_V6];
                        foreach($ipv6_array as $ip_obj_name => $ip_obj){
                            //first ip as primary
                            //or if is dhcp, set as primary
                            if($ipv6_obj == null || $ip_obj->is_dhcp()){
                                if($ipv6_obj != null){
                                    //if is dhcp, move the first ip to virtual_ip
                                    $ipv6_virtual_ip_obj[$ip_obj->name()] = $ip_obj;
                                }
                                $ipv6_obj = $ip_obj;
                            }else{
                                $ipv6_virtual_ip_obj[$ip_obj->name()] = $ip_obj;
                            }
                        }
                    }

                    //primary ipv4
                    if($ipv4_obj){
                        $ipv4_proto = $ipv4_obj->get_proto();
                        if($ipv4_proto == Safe_ip_address_class::PROTO_DHCP4){
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'BOOTPROTO', substr($ipv4_proto,0,-2));
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'DHCP_HOSTNAME', $ipv4_obj->get_hostname());
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'PEERDNS', $ipv4_obj->get_peerdns());
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'PERSISTENT_DHCLIENT', $ipv4_obj->get_persistent());
                            if(!$ipv4_obj->address()){
                                if(!$ipv4_obj->get_dhcp_ip()){
                                    error_log ("WARNING - Cannot get DHCP IPv4 on {$if_name}");
                                }
                            }
                        }elseif($ipv4_proto == Safe_ip_address_class::PROTO_STATIC4){
                            if($interface_obj->interface_type() == Sng_network_interface_class::BONDING ){
                                $interface_cfg_buffer[] = sprintf("%s=%s", 'BOOTPROTO', 'none');
                            }else{
                                $interface_cfg_buffer[] = sprintf("%s=%s", 'BOOTPROTO', substr($ipv4_proto,0,-2));
                            }
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'IPADDR', $ipv4_obj->address());
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'NETMASK', safe_cidr2netmask($ipv4_obj->prefix()));
                        }
                    }
                    //create ipv6 secondary(vip) string
                    $secondary_ip_settings = '';
                    foreach($ipv6_virtual_ip_obj as $secondary_ip_obj_name => $secondary_ip_obj) {
                        if($secondary_ip_settings == ''){
                            $secondary_ip_settings = sprintf("%s=\"%s", 'IPV6ADDR_SECONDARIES', $secondary_ip_obj->address() . '/' . $secondary_ip_obj->prefix());
                        }else{
                            $secondary_ip_settings .= " \\" . PHP_EOL . $secondary_ip_obj->address() . '/' . $secondary_ip_obj->prefix() ;
                        }
                    }
                    //primary ipv6
                    if($ipv6_obj){
                        $ipv6_proto = $ipv6_obj->get_proto();
                        if($ipv6_proto == Safe_ip_address_class::PROTO_DHCP6 ){
                            //need to confirm dhcpv6
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'BOOTPROTO', 'dhcp');
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'DHCPV6C', 'yes');
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'DHCP6_HOSTNAME', $ipv6_obj->get_hostname());
                            if(!$ipv6_obj->address()){
                                if(!$ipv6_obj->get_dhcp_ip()){
                                    error_log ("WARNING - Cannot get DHCP IPv6 on {$if_name}");
                                }
                            }
                        }elseif($ipv6_proto == Safe_ip_address_class::PROTO_SLAAC6){
                            //to be done
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'IPV6_AUTOCONF', 'yes');
                        }elseif($ipv6_proto == Safe_ip_address_class::PROTO_STATIC6){
                            $interface_cfg_buffer[] = sprintf("%s=%s", 'IPV6_AUTOCONF', 'no');
                            $interface_cfg_buffer[] = sprintf("%s=%s/%s", 'IPV6ADDR', $ipv6_obj->address(),$ipv6_obj->prefix());
                        }
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'IPV6INIT', 'yes');
                        if($secondary_ip_settings != ''){
                            $interface_cfg_buffer[] = $secondary_ip_settings .'"';
                        }
                    }

                   //new syntax for ipv4 addresses
                    $vip_id = 0;
                    foreach($ipv4_virtual_ip_obj as $vip_obj_name => $vip_obj){
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'IPADDR' . $vip_id, $vip_obj->address());
                        $interface_cfg_buffer[] = sprintf("%s=%s", 'PREFIX' . $vip_id, $vip_obj->prefix());
                        $vip_id ++;
                    }

                    //bring the interface up on boot if have IP obj on it
                    $onboot = 'yes';
                }else{
                    $network_module = $this->node()->software()->application()->find_module_by_name('network');
                    $if_objs = $network_module->api_retrieve_interface();
                    $onboot = 'no';
                    foreach ($if_objs as $if_key => $if_value) {
                        if ($onboot_user_setting == 'auto' &&
                            $if_value->get_data_value('vlan_type', false) == 'macvlan' &&
                            $if_value->get_data_value('ifname', false) == $interface)
                            {
                                $onboot = 'yes';
                                break;
                        }
                    }
                }
                $onboot_user_setting = $interface_obj->onboot();
                if( $onboot_user_setting != 'auto'){
                    $onboot = $onboot_user_setting;
                }
                if($interface_obj->interface_type()==Sng_network_interface_class::BONDING || $interface_obj->is_bonded()){
                    $onboot = 'yes';
                }
                $interface_cfg_buffer[] = sprintf("%s=%s", 'ONBOOT', $onboot);
                
                //write to interface config files
                $if_file = $if_file_path . $if_name;
                if($this->node()->os()->check_file_exists($if_file)){
                    $config_manager->add_config(
                                new Safe_configuration_class(
                                    $if_file, '',
                                    Safe_configuration_class::CFG_DELETE,
                                    Safe_configuration_class::CFG_FILE));
                }
                $config_manager->add_config(new Safe_configuration_class($if_file, $interface_cfg_buffer, Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_FILE));
                $config_manager->add_config(new Safe_configuration_class($if_file, 'root', Safe_configuration_class::CFG_UPDATE,Safe_configuration_class::CFG_CHOWN));
            }
        }

        //remove old interface
        if($interface_obj_path){
            $deleted_interface_objs = Safe_object_serializer_class::get_serializer()->find_deleted_objects($interface_obj_path);
            foreach($deleted_interface_objs as $interface_obj_name => $interface_data) {
                if( $interface_data['data']['type'] == Sng_network_interface_class::VIRTUAL ||
                        $interface_data['data']['type'] == Sng_network_interface_class::BONDING){
                    //only remove the vlan bonding interface
                    if($interface_data['data']['type'] == Sng_network_interface_class::BONDING){
                        $interface_data['data']['ifname'] = $interface_obj_name;
                    }
                    $config_manager->add_config(new Sng_network_link_down_config($interface_obj_name, $interface_data['data']));
                    $config_manager->add_config(new Sng_network_link_delete_config($interface_obj_name, $interface_data['data']));
                }
            }
        }

        //ips on the updated interface must reapply
        $updated_interface = array();
        //set new interface
        if($interface_objs){
            //check if hw
            foreach($interface_objs as $interface_name => $interface_obj){
                $interface_data = $interface_obj->get_data_values(false);
                if( $interface_data['type']==Sng_network_interface_class::BONDING){
                    $interface_data['ifname'] = $interface_name;
                }
                if($interface_obj->status() == Safe_object_serializer_class::OBJ_STATUS_NEW) {
                    //add virtual interface
                    $config_manager->add_config(new Sng_network_link_add_config($interface_name, $interface_data));
                    $updated_interface[$interface_name] = $interface_obj;
                }
                //if the interface have ip on it bring is up
                if(isset($ip_array[$interface_obj->interface_name()])){
                    $config_manager->add_config(new Sng_network_link_up_config($interface_name, $interface_data));
                }
            }
        }

        //add new ips to interface or re apply ip to new interface(vlan interface updated)
        foreach($ip_objs as $ip_obj_name => $ip_obj){
            if($ip_obj->status() == Safe_object_serializer_class::OBJ_STATUS_NEW || isset($updated_interface[$ip_obj->get_interface()])) {
                //pass by dhcp IP
                if(!$ip_obj->is_dhcp()){
                    $ip_data = $ip_obj->get_data_values(false);
                    $ip_data['interface'] = $ip_obj->get_interface_name();
                    if($ip_data['address']){
                        $config_manager->add_config(new Sng_network_ip_add_config($ip_obj_name, $ip_data));
                    }else{
                        $config_manager->add_error(array(
                                'module' => 'network',
                                'obj_type' => 'ip',
                                'obj_name' => $ip_obj->name(),
                                'description' => 'The IP address is required.'
                        ));
                    }
                }
            }
        }

        //restart the interface with dhcp IP obj(add/update/remove)
        if($interface_with_dhcp){
            foreach($interface_with_dhcp as $tmp_interface_name => $tmp_value){
                $config_manager->add_config(new Sng_network_link_restart_config($interface_name, array('ifname' => $tmp_interface_name)));
            }
        }

        return $this->_generate_config_source_policy_routing($config_manager) && $this->_generate_config_global($config_manager) && $this->_generate_config_route($config_manager);
    }
    public function hardware_interface($if_type = 'all')
    {
        //$if_type management or transcoding
        $allowed_infs = array(
            Sng_network_interface_class::PHYSICAL,
            Sng_network_interface_class::LOOPBACK,
            Sng_network_interface_class::VIRTUAL,
            Sng_network_interface_class::BONDING
        );
        $interfaces = $this->api_retrieve_interface();
        $hardware_interfaces = array();
        foreach($interfaces as $key => $obj){
            $vlan_type = $obj->get_data_value('vlan_type', false);
            $is_macvlan = (isset($vlan_type) && ($vlan_type == Sng_network_interface_class::MACVLAN)) ? true : false;
            if( in_array($obj->interface_type(), $allowed_infs) || $is_macvlan){
                if($if_type == 'all' || $obj->hardware_type() == $if_type || $is_macvlan) {
                    $hardware_interfaces[$key] = $obj;
                }
            }
        }
        return $hardware_interfaces;
    }
    /**
     * check all ip obj if have IPv4 or IPv6
     * @return true/false
     */
    public function have_ip_type($type = Safe_ip_address_class::IP_V6)
    {
        $result = false;
        $ip_obj_keys = $this->get_aggregate_objects_keys('ip');
        foreach($ip_obj_keys as $ip_obj_name){
            $ip_obj = $this->get_aggregate_object('ip', $ip_obj_name);
            if($ip_obj){
                $tmp_ip_type = $ip_obj->ip_type();
                $tmp_ip_address = $ip_obj->address();
                if( $tmp_ip_type == $type && !empty($tmp_ip_address)){
                    $interfaces_obj_name = $ip_obj->get_interface();
                    $interfaces_obj = $this->get_aggregate_object('interface', $interfaces_obj_name);
                    if($interfaces_obj){
                        if($interfaces_obj->hardware_type() != Sng_network_interface_class::TRANSCODING && $interfaces_obj->interface_type() != Sng_network_interface_class::LOOPBACK){
                            $result = true;
                            break;
                        }    
                    }else{
                        safe_log(SAFE_LOG_ERROR, " Unknown network interface  {$interfaces_obj_name}");
                    }
                }
            }
        }
        return $result;
    }

    /**
     * signaling IPs / interfaces are the ones exposed externally
     * - when HA mode is enabled, this means floating IPs exclusively + lo IPs
     * - when HA is disabled, this means management interfaces / IPs
     */

    private function _cluster_mode()
    {
        $cluster_module = $this->node()->software()->application()->find_module_by_name('cluster');
        return ($cluster_module != NULL && $cluster_module->get_cluster_mode() != 'disabled' ? $cluster_module : NULL);
    }

    public function get_signaling_ips($interface_name)
    {
        $cluster_obj = $this->_cluster_mode();

        if ($cluster_obj === NULL) {
            return $this->get_ips($interface_name);
        }

        $floating_ips = $cluster_obj->get_floating_ips($interface_name);
        $network_ips = $interface_name === 'all' ? $this->get_ips('lo') : array();

        $all_ips = array();
        foreach ($floating_ips as $ip_name => $ip_data) {
            $all_ips['floating_ip/' . $ip_name] = $ip_data;
        }
        foreach ($network_ips as $ip_name => $ip_data) {
            $all_ips['network_ip/' . $ip_name] = $ip_data;
        }
        return $all_ips;
    }

    public function signaling_interfaces_ip($with_ip = true)
    {
        $cluster_obj = $this->_cluster_mode();

        if ($cluster_obj === NULL) {
            return $this->interfaces_ip(Sng_network_interface_class::MANAGEMENT, $with_ip);
        }

        $floating_ips = $cluster_obj->get_floating_interfaces_ip($with_ip);
        $network_ips = $this->interfaces_ip(Sng_network_interface_class::MANAGEMENT, $with_ip, 'lo');

        $all_ips = array();
        foreach ($floating_ips as $ip_name => $ip_data) {
            $all_ips['floating/' . $ip_name] = $ip_data;
        }
        foreach ($network_ips as $ip_name => $ip_data) {
            $tmp_obj = $this->get_ip_by_name($ip_name);
            if($tmp_obj){
                $all_ips['network/' . $tmp_obj->address()] = $ip_data;
            }
        }
        return $all_ips;
    }

    /**
     * get ip string list  by hardware interface type
     * @return array()
     */
    public function interfaces_ip($adapter_type = 'all', $with_ip = true, $adapter_name = 'any')
    {
        $interface_objs = $this->get_aggregate_objects('interface');
        $if_enum = array();
        foreach($interface_objs as $if_name => $if) {
            if(($if->hardware_type() == $adapter_type || $adapter_type == 'all') && ($if_name == $adapter_name || $adapter_name == 'any')){
                $ip_objs = $this->get_ips($if_name);
                foreach($ip_objs as $ip_obj){
                    if(strlen($ip_obj->address())){
                        if($with_ip){
                            $if_enum[$ip_obj->name()] = $ip_obj->get_interface_name(true) . ' - ' . $ip_obj->address();
                        }else{
                            $if_enum[$ip_obj->name()] = $ip_obj->get_interface_name(true);
                        }
                    }
                }
            }
        }
        return $if_enum;
    }
    /**
     * get ip obj by interface name
     * @return ip obj
     */
    public function get_ip_by_name($interface_name)
    {
        $obj = null;
        if($interface_name != 'all'){
            $objs = $this->get_aggregate_objects('ip');
            if(isset($objs[$interface_name])){
                $obj = $objs[$interface_name];
            }
        }
        return $obj;
    }
    /**
     * get ip obj list by hardward interface name
     * @return ip obj list
     */
    public function get_ips($interface)
    {
        //get this HW interface related ip objs
        $ips = array();
        $objs = $this->get_aggregate_objects('ip');
        //an obj with a empty name '' will cause error
        if($interface){
            $interface_obj = $this->api_retrieve_interface($interface);
            if($interface_obj){
                $interface_obj_name = $interface_obj->name();
                foreach($objs as $obj_name => $obj){
                    $if_name = $obj->get_interface();//interface obj name
                    if($if_name == $interface_obj_name){
                        $ips[] = $obj;
                    }
                }
            }
        }
        return $ips;
    }
    public function check_ip_conflict($from_obj) {
        $from_obj_name  = $from_obj->name();
        $from_obj_address = $from_obj->address();
        $conflict_list = array();
        $objs = $this->get_aggregate_objects('ip');
        $ip_type = $from_obj->ip_type();
        $ipv6 = false;
        if($ip_type == Safe_ip_address_class::IP_V6){
            $cmp_ip = safe_ipv6_compress($from_obj->address());
            $ipv6 = true;
        }
        if($objs){
            foreach($objs as $name => $obj){
                $to_obj_address = $obj->address();
                //from_obj could be from cluster floadting ip, the name may same, but the path must different
                if( ($name != $from_obj_name || $from_obj->path() != $obj->path()) && $obj->address()){
                    if($obj->address() == $from_obj_address){
                        $conflict_list[] = $obj;
                    }else{
                        //compare ipv6
                        if($ipv6 && $ip_type == $obj->ip_type()){
                            if( $cmp_ip == safe_ipv6_compress($obj->address()) ){
                                $conflict_list[] = $obj;
                            }
                        }
                    }
                }
            }
        }
        return $conflict_list;
    }

    public function check_hostname(&$data){
        $hostname = $this->config()->get_data_value('global/hostname');
        if(!preg_match(Sng_network_service_class::DOMAIN_REGEX,$hostname)){
            $data['message'] = 'Hostname has no valid domain';
            $data['redirect_url'] = '/SAFe/sng_network_config/edit/network/';
            return false;
        }  
        return true;
    }   

    public function checklist(&$list)
    {
        $field_data = array(
            'global/hostname' => $this->config()->get_data_value('global/hostname')
        );
        if($this->ha_mode_active()){
            $validate_result = $this->validate_value($field_data, Sng_network_config_class::get_validation_rule('hostname', Sng_cluster_config_class::HA_MODE));
        }else{
            $validate_result = $this->validate_value($field_data, Sng_network_config_class::get_validation_rule('hostname', ''));
        }

        if(!$validate_result){
            $list[] = array(
                'module' => $this->name(),
                'obj_type' => 'configuration',
                'description' => 'Hostname does not have a valid domain',
                'url' => '/SAFe/sng_network_config/modify/network'
            );
        }
        $sngms_module = $this->node()->software()->application()->find_module_by_name('sngms');
        if ($sngms_module !== NULL) {
            // HW transcoding enabled ?
            $cfg = $sngms_module->config();
            if($cfg->is_enabled()) {
                $transcoding_interfaces = $this->hardware_interface(Sng_network_interface_class::TRANSCODING);
                if (!$transcoding_interfaces) {
                    $list[] = array(
                        'module' => $this->name(),
                        'obj_type' => 'configuration',
                        'description' => 'No Transcoding Interfaces',
                        'url' => '/SAFe/sng_network_config/modify/network'
                    );
                }
            }
        }

        $gw_output = array();
        $tmp_rs = $this->check_gateway($gw_output);
        if(!$tmp_rs){
            if($gw_output){
                $list[] = array(
                'module' => $this->name(),
                'obj_type' => 'configuration',
                'description' => 'Gateway error ' . implode("<br>",$gw_output),
                'url' => '/SAFe/sng_network_config/modify/network'
                );
            }
        }
    }

    public function ha_checklist(&$list)
    {
        $field_data = array(
            'global/hostname' => $this->config()->get_data_value('global/hostname')
        );

        $validate_result = $this->validate_value($field_data, Sng_network_config_class::get_validation_rule('hostname', Sng_cluster_config_class::HA_MODE));
        if(!$validate_result){
            $list[] = array(
                'module' => $this->name(),
                'obj_type' => 'configuration',
                'description' => 'Hostname does not have a valid domain.',
                'url' => '/SAFe/sng_network_config/edit/network/',
                'label' => 'Edit'
            );
        }
    }

    /**
     * @brief Enable troubleshooter support
     *
     * @return
     */
    public function troubleshooter_enabled()
    {
        return true;
    }
    /**
     * @brief Populate troubleshooter configuration parameters (configurable
     * object)
     *
     * @param[in out] $configurable
     *
     * @return
     */
    public function troubleshooter_parameters(&$configurable)
    {
        $_interfaces = $this->hardware_interface();
        $if_list = array();
        foreach($_interfaces as $interface){
            $if_list[$interface->name()] = $interface->interface_name();
        }
        // Enable network troubleshooting ?
        $configurable->add_enum_field($this->name().'/enable', 'Enable', 'dropdown', 'false',$configurable->enable_disable_enum());
        // What interface ?
        $configurable->add_multiple_field($this->name().'/'.'interfaces', 'Interfaces', 'inline_checkbox', '', $if_list);
        $configurable->set_field_rules($this->name().'/'.'interfaces', 'required[,in,__empty__]');
        // Pcap filter
        $configurable->add_field($this->name().'/filter', 'Filter', 'text', '.*', 20);

        return true;
    }
    public function get_action_settings($action=null)
    {
        // Populate action table
        $action_def = array();
        $action_def['restart'] = array(
                'name' => 'Restart Network',
                'icon' => 'repeat',
                'description' => 'Restart Network',
                'request' => 'POST',
                'confirm' => true,
                'reload' => true
        );
        if(file_exists(self::TMP_FILE)){
            $action_def['restart']['execute'] = 'timer_switch_class';
            $action_def['restart']['args'] = array('form a[name="restart"]','safe_button_primary','safe_button_warning');
        }
        $bonding_params = new Sng_network_bonding_class($this->node(), $this->object_name().'/bonding', Safe_object_serializer_class::build_unique_name('bond'));
        $bonding_params->configure();
        $bonding_params->prepare_edit();
        $action_def['bonding'] = array(
                'name' => 'Bonding interface',
                'icon' => 'link',
                'description' => 'Bonding interface',
                'request' => 'POST',
                'class' => $bonding_params,
                'confirm' => true,
                'reload' => true
        );
        $action_def['apply'] = array(
                'name' => 'Apply Network',
                'icon' => 'save',
                'description' => 'Apply Network changes',
                'request' => 'POST',
                'confirm' => true,
                'reload' => true,
                'button_class' => 'danger'
        );
        if(!$this->is_configuration_modified(true)){
            $action_def['apply']['display'] = false;
        }
        if($action) {
            return $action_def[$action];
        }else{
            return $action_def;
        }
    }
    public function api_bonding_action($data=null, &$output = null) {
        $action_cfg = $this->get_action_settings('bonding');
        $param = $action_cfg['class'];
        if ($param->validate($data, $output)) {
            //create a bonding obj
            $data['type'] = Sng_network_interface_class::BONDING;
            $interface_obj =  $this->api_create_interface($data['display_name'], $data, $output);
            //add the bonding interface to all interface list
            //disable the bonded interface
            //attach all ip to bonding obj
            
            if($interface_obj){
                $output['message'] = array(
                        "Create bonding interface succeed."
                );
                $interface_obj->set_data_values($data);
                $interface_obj->save();
                return true;
            }else{
                $output['message'] = array(
                        "Create bonding interface failed."
                );
                $output['reason'] = $output;
                $output['result'] = true;//show error on web
            }
        }
        return false;//return error in api
    }
    public function api_refresh_action($data=null, &$output = null) {
        $action_cfg = $this->get_action_settings('refresh');
        $rc = $this->node()->execute('sudo nohup /usr/local/nsc/hooks/network.restore.post --mode=update');
        if($rc){
            $output['message'] = array(
                    $action_cfg['description'] . " succeed."
            );
        }else{
            $output['message'] = array(
                    $action_cfg['description'] . " failed."
            );
        }
        return $rc;
    }
    public function api_restart_action($data=null, &$output = null) {
        $action_cfg = $this->get_action_settings('restart');
        $the_app = $this->node()->software()->application();

        if (Safe_service_class::STATUS_RUNNING == $the_app->status()){
            $output['message'] = array(
                    $action_cfg['description'] . " failed.",
                    $the_app->name() . ' is running.'
            );
            return false;
        }

        $clustermon_service = $this->node()->software()->application()->find_module_by_name('clustermon');
        if ($clustermon_service && $clustermon_service->status() == Safe_service_class::STATUS_RUNNING) {
            $output['message'] = array(
                    $action_cfg['description'] . " failed.",
                    $clustermon_service->description() . ' is running.'
            );
            return false;
        }

        $rc =  $this->api_restart_service(null, null, $output);
        if($rc){
            if(file_exists(self::TMP_FILE))
                unlink(self::TMP_FILE);
            $output['message'] = array(
                    $action_cfg['description'] . " succeed."
            );
        }else{
            $output['message'] = array(
                    $action_cfg['description'] . " failed."
            );
        }
        return $rc;
    }

    public function api_apply_action($data=null, &$output = null) {
        $action_cfg = $this->get_action_settings('apply');
        $cfg_mgr = $this->node()->configuration_manager();
        // Take care of generate configuration
        if ($this->generate_config($cfg_mgr) && !$cfg_mgr->errors()
        && $cfg_mgr->write_config($this->node())) {
            // clear modified flags
            $this->clear_configuration_modified(true);
            file_put_contents(self::TMP_FILE,'restart');
            $output['message'] = array(
                    $action_cfg['description'] . " succeed. You can restart the network service to active the changes."
                                               );
            $rc = true;
        }else{
            $output['message'] = array(
                    $action_cfg['description'] . " failed."
            );
            $error_msg = $cfg_mgr->errors();
            if($error_msg){
                foreach($error_msg as $tmp_error_item){
                    $output['reason'][] = $tmp_error_item['description'];
                }
            }
            $rc = false;
        }
        return $rc;
    }
    public function api_up_interface($name=null, $data=null, &$output = null) {
        //$name : interface obj name
        $output_msg = '';
        if($name){
            $obj = $this->api_retrieve_interface($name);
            if($obj){
                $if_name = $obj->interface_name();
                $set_set_obj = new Sng_network_link_up_config($if_name, array('ifname' => $if_name));
                $rc = $set_set_obj->run($this->node(), $output_msg);
                if($rc === true) {
                    $output['message'] = array( "Successfully brought 'Up' network interface: ${if_name}.");
                }else{
                    $output['message'] = array( "Set interface ${if_name} up failed:", '');
                    foreach ($output_msg as $line) {
                        $output['message'][] = htmlspecialchars($line);
                    }
                }
                return $rc;
            }
            $output['message'] = array( "Interface not found.");
        }else{
            $output['message'] = array( "Interface name is required.");
        }
        return false;
    }
    public function api_down_interface($name=null, $data=null, &$output = null) {
        //$name : interface obj name
        if($name){
            $obj = $this->api_retrieve_interface($name);
            if($obj){
                //check if have IP using by webui or rest
                $ip_list = $this->get_ips($name);
                if($ip_list){
                    foreach($ip_list as $ip_obj){
                        $usages = $this->node()->software()->application()->check_object_usage('network','ip', $ip_obj->name());
                        if($usages){
                            foreach($usages as $usage_item){
                                if($usage_item['name'] == 'rest'){
                                    $ip = $ip_obj->address();
                                    $output['message'] = array(
                                            "Set interface down failed.",
                                            "The IP:{$ip} on this interface is used by current connection.");
                                    return false;
                                }
                            }
                        }
                    }
                }
                $if_name = $obj->interface_name();
                $set_set_obj = new Sng_network_link_down_config($if_name, array('ifname' => $if_name));
                $rc = $set_set_obj->run($this->node());
                if($rc === true){
                    $output['message'] = array( "Successfully brought 'Down' network interface: ${if_name}.");
                }else{
                    $output['message'] = array( "Set interface down failed.");
                }
                return $rc;
            }
            $output['message'] = array( "Interface not found.");
        }else{
            $output['message'] = array( "Interface name is required.");
        }
        return false;
    }
    public function check_gateway(&$output, &$gw_dev = null){
        $rs = true;
        $cfg_obj = $this->config();
        $cfg_data = $cfg_obj->get_data_values_exploded(false);
        $tmp_default_gw  = array();
        if($cfg_data['ipv4']['default_gw']){
            $tmp_default_gw['ipv4'] = $cfg_data['ipv4']['default_gw'];
        }
        if($cfg_data['ipv6']['default_gw']){
            $tmp_default_gw['ipv6'] = $cfg_data['ipv6']['default_gw'];
        }
        // if a gateway is specified, it must be on same subnet as interface
        if($tmp_default_gw){
            // Get adapter and associated IP address
            $gatewaydev = $cfg_obj->get_data_value('global/gatewaydev', false);
            if($gatewaydev){
                $ips = $this->get_ips($gatewaydev);
            }else{
                $ips = $this->api_retrieve_ip();
            }
            // if update from rest api, may not include 'interface'
            foreach($tmp_default_gw as $gw_type => $gw_ip){
                if($gw_ip){
                    if(filter_var($gw_ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6)){
                        $gateway_ip_type = Safe_ip_address_class::IP_V6;
                    }else{
                        $gateway_ip_type = Safe_ip_address_class::IP_V4;
                    }
                    $sub_net = false;
                    $have_same_type_ip = false;
                    if($ips){
                        foreach($ips as $ip_obj_name => $ip_obj){
                            $ip_address = $ip_obj->address();
                            if($ip_address &&  $gateway_ip_type == $ip_obj->ip_type()){
                                $have_same_type_ip = true;
                                $ip_prefix = $ip_obj->prefix();
                                //ipv4 && ipv6
                                if(safe_cidr_match($gw_ip, safe_cidr2network($ip_address, $ip_prefix), $ip_prefix)){
                                    $sub_net = true;
                                    $gw_dev['gw_interface_dev'][$gw_type] = $ip_obj->get_interface_name();
                                    break;
                                }
                                if($gateway_ip_type == Safe_ip_address_class::IP_V6){
                                    if(strncasecmp($gw_ip, 'fe80', 4) == 0 ) {
                                        $sub_net = true;
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    if(!$sub_net){
                        $tmp_field_name = $gw_type . '/'. 'default_gw';
                        if($have_same_type_ip == false){
                            //if the gateway ip type not match to any of the ip on the interface, show different error message
                            if(!$gatewaydev) {
                                $gatewaydev = 'any';
                            }
                            $output = array($tmp_field_name => "No {$gw_type} IP on {$gatewaydev} interface.");
                        }else{
                            if(!$gatewaydev){
                                $gatewaydev = 'a valid';
                            }
                            $output = array($tmp_field_name => "Default {$gw_type} gateway is not on {$gatewaydev} subnet.");
                        }

                        $rs = false;
                    }
                }
            }
        }
        return $rs;
    }
    function post_reset($info, $reason){
        if($this->node()->get_config_status('update') == 'rest'){
            return true;
        }else{
            return parent::post_reset($info, $reason);
        }
    }

    public function check_object_modified($obj,$obj_type,$obj_name){
        if($obj_type=='ip'){
            $interface = $obj->get_interface_obj();
            $ip_list = $this->get_ips($interface->name());
            $static = true;
            foreach($ip_list as $ip){
                if($ip->is_dhcp()){
                    $static=false;
                    break;
                }
            }
            $this->node()->execute('nohup /etc/ifupdown-local.d/20-peerdns-update '.($static? "down" : "up")." ".$interface->name(),$result, false,true);
        }
    }

    public function api_create_source_policy_routing($name,$data=null,&$output=null){
        $obj = new Sng_network_source_policy_routing_class($this->node(),$this->object_name().'/source_policy_routing',$name);
        $obj->configure();
        return $obj;
    }

    public function api_update_source_policy_routing($name,$data=null,&$output=null){
        $obj = $this->api_retrieve_source_policy_routing($name);
        if($obj){
            if($obj->validate($data,$output)){
                return $obj->save();
            }
        }
        return false;
    }

    public function api_retrieve_source_policy_routing($name=null,$data=null,&$output=null){
        $objs = $this->get_aggregate_objects('source_policy_routing');
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }

    public function api_delete_source_policy_routing($name=null,$data=null,&$output=null){
        $spr = $this->api_retrieve_source_policy_routing($name);
        if($spr){
            if($spr->can_dispose()){
                return $spr->dispose();
            }
        }
        return false;
    }
 }
/* End of file */
