<?php

/*
* Copyright (C) 2018 Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purpose only and is not to be
 * used for any other purpose.
*/
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once('application/helpers/safe_helper.php');
safe_require_class('object');

class Sng_network_source_policy_routing_class extends Safe_configurable_object_class{

    public function __construct($node,$parent_name,$name){
        parent::__construct($parent_name,$name,$node);
    }

    public function configure(){
        $network_module = $this->_node->software()->application()->find_module_by_name('network');
        $if_enum = $network_module->signaling_interfaces_ip();
        $this->add_enum_field('source_ip','Source Ip','dropdown','',$if_enum);
        $this->set_field_rules('source_ip','required');
        $this->add_field('gateway','Default Gateway','text','',40);
        $this->set_field_rules('gateway', 'required[proto,in,'. Safe_ip_address_class::PROTO_STATIC4 .','. Safe_ip_address_class::PROTO_STATIC6 .']|valid_ip');
        return parent::configure();
    }

    private function ip_type($ip){
        if(filter_var($ip,FILTER_VALIDATE_IP,FILTER_FLAG_IPV4))
            return Safe_ip_address_class::IP_V4;
        else if(filter_var($ip,FILTER_VALIDATE_IP,FILTER_FLAG_IPV6))
            return Safe_ip_address_class::IP_V6;
        return null;
    }

    public function validate($data,&$output){
        $gateway = $this->get_data_value('gateway');
        $result = parent::validate($data,$output);
        if($this->ip_type($gateway)!==null){
            $network_module = $this->_node->software()->application()->find_module_by_name('network');
            $ips = $network_module->api_retrieve_ip();
            foreach($ips as $ip){
                if($ip->address()==$gateway){
                    $this->set_error('gateway','The ip is in use by interface '.$ip->get_interface_name());
                    $result=false;
                    break;
                }
            }
        }else{
            $this->set_error('gateway','The Gateway field must contain a valid ip address');
            $result = false;
        }
        return $result;
    }

    public function save(&$output=null){
        return parent::save($output);
    }

    public function can_dispose(&$reason=null){
        $usages = $this->node()->software()->application()->check_object_usage('network','source_policy_routing',$this->name());
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        return parent::can_dispose($reason);
    }

    public function dispose(&$output=null){
        return parent::dispose($output);
    }


    public function source_ip(){
        $source_ip = explode(' - ',$this->get_data_value('source_ip'));
        return (count($source_ip)>0)? $source_ip[1] : null;
    }

    public function iface(){
        $source_ip = explode(' - ',$this->get_data_value('source_ip'));
        return (count($source_ip)>0)? $source_ip[0] : null;
    }

    public function ip_obj(){
        $time = time();
        return Sng_ip_address_class::model($this->_node)->findOne(array('address'=>$this->source_ip()));
    }
    
    public function summary(){
        $ip = $this->source_ip();
        $interface = $this->iface();
        return array('data'=>array(
            'Interface'=> $interface,
            'Source Ip'=> $ip,
            'Gateway'=> $this->get_data_value('gateway')
        ));
    }
}
?>
