<?php

/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: * */
/** coding: utf-8: * */
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Abhishek Arora <aarora@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
 */
/**
 * SNG NTP Linux service wrapper
 *
 * @author Abhishek Arora
 * @version
 */
if (!defined('BASEPATH'))
    exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_auditor_class');
safe_require_class('service');

class Sng_ntpd_config_class extends Safe_configurable_object_class {

    public function __construct($node, $parent_name) {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, "configuration", $node);
    }

    public function configure() {
        $ntp_mode = array(
            'false' => 'Manual',
            'true' => 'Time Server',
        );
        $zone = safe_exec("/usr/local/nsc/bin/pcregrep -o1 '^ZONE=[\"]\K([^\"]+)' /etc/sysconfig/clock", '', '', $returnzone, false);
        $server0 = $server1 = $server2 = $systemtimezone = $date = $time = '';
        $defaultserver = safe_exec("/usr/local/nsc/bin/pcregrep -o1 '^server\s\K([^\s]+)' /etc/ntp.conf", '', '', $return, false);
        // Retrieve live server datetime and set it
        $ntpd_module = $this->node()->software()->application()->find_module_by_name('ntpd');
        $datetime = $ntpd_module->api_retrieve_date('datetime');
        $date = $ntpd_module->api_retrieve_date('date');
        $time = $ntpd_module->api_retrieve_date('time');

        if (!$return) {
            if (isset($defaultserver[0])) {
                $server0 = $defaultserver[0];
            }
            if (isset($defaultserver[1])) {
                $server1 = $defaultserver[1];
            }
            if (isset($defaultserver[2])) {
                $server2 = $defaultserver[2];
            }
        }
        if (count($zone) > 0) {
            $systemtimezone = $zone[0];
            date_default_timezone_set($systemtimezone);
        }

        $this->add_field('config/datetime', 'Server Datetime', 'datetime', $datetime, 49);
        $this->set_field_disabled('config/datetime', true);

        $this->add_enum_field('config/timezone', 'Time Zone', 'dropdown', $systemtimezone, $this->timezone_list());
        $this->set_field_help('config/timezone', array('Select your system timezone.'));
        $this->add_enum_field('config/mode', 'Configuration Mode', 'dropdown', 'true', $ntp_mode);
        $this->set_field_help('config/mode', array('You can select Time Sever or Manual Configuration'));

        $this->add_field('config/timeserver/server/0', 'Server #0', 'text', $server0, 49);
        $this->set_field_rules('config/timeserver/server/0', 'required[config/mode]|valid_ip_or_domain');
        $this->set_field_help('config/timeserver/server/0', 'Enter the server 0 details here');
        $this->add_field('config/timeserver/server/1', 'Server #1', 'text', $server1, 49);
        $this->set_field_rules('config/timeserver/server/1', 'valid_ip_or_domain');
        $this->set_field_help('config/timeserver/server/1', 'Enter the server1 details here');
        $this->add_field('config/timeserver/server/2', 'Server #2', 'text', $server2, 49);
        $this->set_field_rules('config/timeserver/server/2', 'valid_ip_or_domain');
        $this->set_field_help('config/timeserver/server/2', 'Enter the server 2 details here');

        $this->add_field('config/date', 'Date', 'date', $date, 20);
        $this->set_field_rules('config/date', 'valid_date');
        $this->set_field_help('config/date', 'Enter the system date.');
        $this->add_field('config/time', 'Time', 'time', $time, 20);
        $this->set_field_rules('config/time', 'valid_time');
        $this->set_field_help('config/time', 'Enter the system time.');
        $control_group_fields = array('config/timeserver/server/0', 'config/timeserver/server/1', 'config/timeserver/server/2', 'config/date', 'config/time');

        foreach ($control_group_fields as $group_key) {
            if ($group_key == 'config/date' || $group_key == 'config/time') {
                $condition_data['true'][] = array('field_name' => $group_key, 'status' => 'disable');
                $condition_data['false'][] = array('field_name' => $group_key, 'status' => 'enable');
            } else {
                $condition_data['true'][] = array('field_name' => $group_key, 'status' => 'enable');
                $condition_data['false'][] = array('field_name' => $group_key, 'status' => 'disable');
            }
        }

        $this->create_group('configuration-mode', $control_group_fields, array('default' => array('value' => 'true', 'status' => 'enable'), 'condition' => $condition_data));
        $this->conditional_control('config/mode', 'configuration-mode');

        $this->set_field_category('config/enable', 'Edit Configuration');

        return parent::configure();
    }

    public function save(){
        return parent::save();
    }

    public function timezone_list() {
        $zones_array = array();
        $timestamp = time();
        $_zone_list = array();
        $_zone_list = safe_exec("find /usr/share/zoneinfo/posix -type f  | grep -v .tab", '', '', $rc, false);

        foreach ($_zone_list as $key => $zone) {
            $zone = str_replace("/usr/share/zoneinfo/posix/", "", $zone);
            $zones_array[$zone] = $zone;
        }
        return $zones_array;
    }

    public function summary($type = 'horizontal', $long = false, $min = 8){

        $data = parent::summary($type,$long,8);

        // Retrieve live server datetime and set it
        $ntpd_module = $this->node()->software()->application()->find_module_by_name('ntpd');
        $datetime = $ntpd_module->api_retrieve_date('datetime');
        $date = $ntpd_module->api_retrieve_date('date');
        $time = $ntpd_module->api_retrieve_date('time');

        // Formatting datetime, date, and time
        $datetime = date('D, M, d, Y h:ia', strtotime($datetime));
        $date = date('Y-m-d', strtotime($date));
        $time = date('H:i', strtotime($time));

        // Set live server datetime
        $data['data']['Server Datetime'] = $datetime;

        // Config mode is set to manual
        if($this->get_data_value('config/mode') == 'Manual'){
            foreach($data['data'] as $key => $value){
                // Unset time server field
                if($key == 'Server #0' || $key == 'Server #1' || $key == 'Server #2'){
                    unset($data['data'][$key]);
                }
                // Set date and time field to the live server datetime
                if($key == 'Date'){
                    $data['data'][$key] = $date;
                }
                if($key == 'Time'){
                    $data['data'][$key] = $time;
                }
            }
        }

        // Config mode is set to timer server
        if($this->get_data_value('config/mode') == 'Time Server'){
            foreach($data['data'] as $key => $value){
                // Unset date and time field
                if($key == 'Date' || $key == 'Time'){
                    unset($data['data'][$key]);
                }
            }
        }

        return $data;
    }
}
