<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Abhishek Arora <aarora@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG SSH Linux service wrapper
 *
 * @author Abhishek Arora
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');
safe_require_class('safe_auditor_class');
require_once('sng_ntpd_config_class.php');

class Sng_ntpd_set_datetime_config extends Safe_configuration_user_class {

    public function __construct($date_time) {
        parent::__construct('', Safe_configuration_class::CFG_NONE, $date_time);
    }

    public function run($node) {
        $date_time = $this->data();
        // build cmd line
        $datecommand = "date " . $date_time['date'];
        $hwclockcommand = "hwclock --set --date " . $date_time['hwclock'];
        // Execute
        $node->execute('sudo nohup timedatectl set-ntp no');
        $node->execute('sudo nohup ' . $datecommand);
        $node->execute('sudo nohup hwclock --systohc');
        $node->execute('sudo nohup ' . $hwclockcommand);
        return true;
    }

}


class Sng_ntpd_service_starttimeserver extends Safe_configuration_user_class {

    public function run($node) {
        //$ntpd_service = $node->find_module_by_name('ntpd');
        //$ntpd_service->start();
        ////auto start on startup
        //$ntpd_service->set_startup(Safe_service_class::STARTUP_AUTOMATIC);

        $node->execute('sudo nohup timedatectl set-ntp yes');
        $node->execute('systemctl start chronyd');
        return true;
    }

}

class Sng_ntpd_service_stoptimeserver extends Safe_configuration_user_class {

    public function run($node) {
        $ntpd_service = $node->find_module_by_name('ntpd');
        $ntpd_service->stop();
        //do not auto start on startup
        $ntpd_service->set_startup(Safe_service_class::STARTUP_MANUAL);

        $node->execute('sudo nohup timedatectl set-ntp no');
        $node->execute('systemctl stop chronyd');
        return true;
    }

}

class Sng_ntpd_set_datetime_class extends Safe_configurable_object_class {
    public function __construct($node) {
        // Parent constructor to invoke unserialize if needed
        parent::__construct("auto_fill", "Sng_ntpd_set_datetime_class", $node);
    }

    public function configure() {
        $this->add_field('config/date', 'Date', 'date', '', 20);
        $this->set_field_rules('config/date', 'valid_date');
        $this->set_field_help('config/date', 'Enter the system date.');
        $this->add_field('config/time', 'Time', 'time', '', 20);
        $this->set_field_help('config/time', 'Enter the system time.');
        return parent::configure();
    }
}

/******************************************************************************
 *   NTP Module
 ******************************************************************************/
class Sng_ntpd_service_class extends Safe_service_class
{
    public function __construct($app)
    {
        parent::__construct($app->node()->software(), "ntpd");
    }
    /**
     * @brief
     *         
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("NTP");
        // Create the config object
        $cfg = new Sng_ntpd_config_class($this->node(), $this->path() . '/' .$this->name());
        $cfg->configure();
        $cfg->synch();
        // Attach config to module
        $this->set_config($cfg);

        // Register aggregate objects
        $this->register_aggregate_object('datetime',
            array(
                'name' => 'Datetime',
                'dynamic' => true,
                'base_path' => parent::object_name().'/datetime',
                'configurable' => false,
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve server datetime',
                        'request' => 'GET',
                    ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update server datetime',
                        'request' => 'POST',
                    ),
                )
            )
        );

        return parent::configure();
    }
    /**
     * @brief 
     *
     * @param[in out] $config_manager
     *
     * @return 
     */
    public function generate_config(&$config_manager)
    {
        if(!$this->_generate_config($config_manager)){
            return false;
        }
        return parent::generate_config($config_manager);
    }
    public function can_restore($info, &$reason)
    {
        // Prevent calling parent as service status will be checked
        return true;
    }
    /**
     * @brief Create config jobs for dialplan module
     *           
     * @param[in out] $config_manager
     *           
     * @return
     */
    public function reload_generate_config(&$config_manager, $obj_type=null)
    {
        return $this->generate_config($config_manager);
    }
    /**
     * @brief 
     *
     * @param[in out] $obj_type
     *
     * @return 
     */
    public function reload_clear_modified($obj_type=null)
    {
        return $this->clear_configuration_modified();
    }
    /**
     * @brief 
     *
     * @param[in out] $need_reload
     *
     * @return 
     */
    public function support_reload(&$need_reload = null) 
    {
        parent::support_reload($need_reload);
        return true;
    }
    /**
     * @brief 
     *
     * @param[in out] $config_manager
     *
     * @return 
     */
    private function _generate_config(&$config_manager, $obj_type=null)
    {
        // Don't generate if configuration not modified
        if(!$this->is_configuration_modified()){
            return true;
        }
        $data = $this->config()->get_data_values_exploded(false);
        // 1st create all common jobs
        // - Set timezone
        $config_manager->add_config(
                new Safe_configuration_class("/etc/localtime", '', Safe_configuration_class::CFG_DELETE, Safe_configuration_class::CFG_FILE));
        $file_link = array(
            'original' => "/usr/share/zoneinfo/" . $data['config']['timezone'],
            'target' => "/etc/localtime",
        );
        $config_manager->add_config(
                new Safe_configuration_class(
                $file_link, '', Safe_configuration_class::CFG_LINK, Safe_configuration_class::CFG_FILE));
        $config_manager->add_config(
                new Safe_configuration_class("/etc/localtime", 'root', Safe_configuration_class::CFG_UPDATE, Safe_configuration_class::CFG_CHOWN));
        ini_set('date.timezone', $data['config']['timezone']);
        date_default_timezone_set($data['config']['timezone']);
        // Serialize the clock file as per the selected timezone
        $timezone_info[] = 'ZONE="' . $data['config']['timezone'] . '"';
        if ($data['config']['timezone'] == 'UTC') {
            $timezone_info[] = 'UTC=true';
        } else {
            $timezone_info[] = 'UTC=false';
        }
        $timezone_info[] = 'ARC=false';
        $config_manager->add_config(
                new Safe_configuration_class("/etc/sysconfig/clock", $timezone_info, Safe_configuration_class::CFG_UPDATE, Safe_configuration_class::CFG_FILE));
        // 3- Change back ownership to root so service won't complain
        $config_manager->add_config(
                new Safe_configuration_class("/etc/sysconfig/clock", 'root', Safe_configuration_class::CFG_UPDATE, Safe_configuration_class::CFG_CHOWN));
        if ('true' == $data['config']['mode']) {
            $ntpd_conf_file = "/etc/chrony.conf";
            // First, cleanup conf by removing all server lines
            $sed_data[] = "/^server\s/d";
            //  create config tasks
            // 1- Ensure we can edit this file (change ownership to webconfig)
            $config_manager->add_config(
                    new Safe_configuration_class($ntpd_conf_file, 'webconfig', Safe_configuration_class::CFG_UPDATE, Safe_configuration_class::CFG_CHOWN));
            // 2- Apply sed to file
            $config_manager->add_config(
                    new Safe_configuration_class($ntpd_conf_file, $sed_data, Safe_configuration_class::CFG_UPDATE, Safe_configuration_class::CFG_SED));
            // add those new servers
            foreach ($data['config']['timeserver']['server'] as $key => $server) {
                // Add new sed data to add this server to file
                if ($server != '') {
                    $sed_append_server[] = '$aserver ' . $server . ' iburst';
                }
            }
            $config_manager->add_config(
                    new Safe_configuration_class($ntpd_conf_file, $sed_append_server, Safe_configuration_class::CFG_UPDATE, Safe_configuration_class::CFG_SED));

            // 3- Change back ownership to root so service won't complain
            $config_manager->add_config(
                    new Safe_configuration_class($ntpd_conf_file, 'root', Safe_configuration_class::CFG_UPDATE, Safe_configuration_class::CFG_CHOWN));
            // start (or restart) ntp service (in a config job)
            $config_manager->add_config(new Sng_ntpd_service_stoptimeserver());
            $config_manager->add_config(new Sng_ntpd_service_starttimeserver());
        } else {
            // Set date/time manually
            // Stop ntpd daemon
            $config_manager->add_config(new Sng_ntpd_service_stoptimeserver());

            // Update date and hwclock
            if ($data['config']['date'] != '' && $data['config']['time'] != '') {
                $changed_date['date'] = date("mdHiY", strtotime($data['config']['date'] . $data['config']['time']));
                $hwclocktime['hwclock'] = date("d M Y H:i", strtotime($data['config']['date'] . $data['config']['time']));
                $config_manager->add_config(new Sng_ntpd_set_datetime_config($changed_date));
            } else if ($data['config']['date'] != '' && $data['config']['time'] == '') {
                $changed_date['date'] = date("mdHiY", strtotime($data['config']['date'] . date('h:ia')));
                $hwclocktime['hwclock'] = date("d M Y H:i", strtotime($data['config']['date'] . date('h:ia')));
                $config_manager->add_config(new Sng_ntpd_set_datetime_config($changed_date));
            } else if ($data['config']['date'] == '' && $data['config']['time'] != '') {
                $changed_date['date'] = date("mdHiY", strtotime(date('Y-m-d') . $data['config']['time']));
                $hwclocktime['hwclock'] = date("d M Y H:i", strtotime(date('Y-m-d') . $data['config']['time']));
                $config_manager->add_config(new Sng_ntpd_set_datetime_config($changed_date));
            }

            // Start ntpd daemon
            //$config_manager->add_config(new Sng_ntpd_service_starttimeserver());
        }

        return true;
    }

    public function api_retrieve_date($name=null, $data=null, &$output=null) {
        if ($name == 'date') {
            $cmd_output = implode(safe_exec("date +%F", '', '', $returndate, false));
        } else if ($name == 'time') {
            $cmd_output = implode(safe_exec("date +%T", '', '', $returndate, false));
        } else if ($name == 'datetime'){
            $cmd_output = implode(safe_exec("date", '', '', $returndate, false));
        }
        return $cmd_output;
    }

    public function api_update_datetime($name=null, $data=null, &$output=null) {

        // Change System Date And Time
        if ($data['config']['date'] != '' && $data['config']['time'] != '') {
            $changed_date['date'] = date("mdHiY", strtotime($data['config']['date'] . $data['config']['time']));
            $hwclocktime['hwclock'] = date("d M Y H:i", strtotime($data['config']['date'] . $data['config']['time']));
            $config_manager->add_config(new Sng_ntpd_set_datetime_config($changed_date));
        } else if ($data['config']['date'] != '' && $data['config']['time'] == '') {
            $changed_date['date'] = date("mdHiY", strtotime($data['config']['date'] . date('h:ia')));
            $hwclocktime['hwclock'] = date("d M Y H:i", strtotime($data['config']['date'] . date('h:ia')));
            $config_manager->add_config(new Sng_ntpd_set_datetime_config($changed_date));
        } else if ($data['config']['date'] == '' && $data['config']['time'] != '') {
            $changed_date['date'] = date("mdHiY", strtotime(date('Y-m-d') . $data['config']['time']));
            $hwclocktime['hwclock'] = date("d M Y H:i", strtotime(date('Y-m-d') . $data['config']['time']));
            $config_manager->add_config(new Sng_ntpd_set_datetime_config($changed_date));
        }
        unset($data['config']['time']);
        unset($data['config']['date']);
    }
   
}
