<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2018  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Provisioning Module
 *
 * @author Santiago Tramoyeres Cuesta
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');

class Sng_provisioning_module_class  extends Safe_module_class{

    public function __construct($app){
        parent::__construct($app->node()->software(),"provisioning");
    }

    public function configure(){
        $this->set_description("Provisioning");

        return parent::configure();
    }

    public function get_data_settings(){
        return array(
            'template'=>array(
                'description'=>'Provisioning',
                'file_name'=>'config.toml',
                'singleton'=>true,
                'list'=>false,
                'filesystem'=>true,
                'pattern'=>'/provisioning/*.toml',
                'log_file_pattern'=>'/tmp/provisioning_TYPE_.log',
                'script_command'=>'/provisioning/common/run.sh',
                'methods'=>array(
                    'update'=>array(
                        'name'=>'Update',
                        'icon'=>'save',
                        'description'=>'Update template from base 64 encoded string',
                        'request'=>'POST'
                    ),
                    'upload'=>array(
                        'name'=>'Upload',
                        'icon'=>'save',
                        'description'=>'Update template from file',
                        'request'=>'POST'
                    ),
                    'register'=>array(
                        'name'=>'Register to EMS',
                        'icon'=>'',
                        'description'=>'Register the SBC to EMS',
                        'request'=>'GET'
                    ),
                    'retrieve'=>array(
                        'name'=>'Retrieve',
                        'description'=>'Retrieve template content',
                        'request'=>'GET'
                    ),
                    'apply'=>array(
                        'name'=>'Apply',
                        'icon'=>'',
                        'description'=>'Apply template',
                        'request'=>'GET'
                    )
                )
            )
        );
    }
    
    public function api_update_template($name=null,$data=null,&$output=null){
        if(!isset($data['data'])){
            $output['message']= 'Data does not exist as a request field';
            $ret = false;
        }else{
            $settings = $this->get_data_settings();
            $template_file = dirname($settings['template']['pattern']).'/'.$settings['template']['file_name'];
            $file_data = base64_decode($data['data']);
            if(file_exists($template_file))
            $this->node()->chmod_file($template_file,'660');
            $succeed = $this->node()->write_file($template_file,$file_data);
            if($succeed){
                $this->node()->chown_file($template_file,'root');
                $this->node()->chmod_file($template_file,'600');
                $output['message'] = 'File saved successfully';
                $ret = true;
            }else{
                $output['message'] = 'Fail to save Provisioning file';
                $ret = false;
            }            
        }
        return $ret;
    }
   
    public function api_upload_template($name=null,$data=null,&$output=null){
        $settings = $this->get_data_settings();
        $up_method = $settings['template']['methods']['upload']['request'];
        if($up_method){
            $template_file = dirname($settings['template']['pattern']).'/'.$settings['template']['file_name'];
            if(file_exists($template_file))
            $this->node()->chmod_file($template_file,'660');
            $ret = $this->upload_data('template',$template_file,$output,$up_method);
            $this->node()->chmod_file($template_file,'600');
        }else{
            $output['message'] = 'Upload request method error.';
            $ret = false;
        }
        return $ret;
    }

    private function apply($action = 'config'){
        $ret = false;
        $action = strtolower($action);
        $allowed_actions = array('config'=>'config','register'=>'ems');
        if(!isset($allowed_actions[$action])){
            $output['message'] = 'Action not allowed';
        }else{
            $settings = $this->get_data_settings();
            $rs1 = 255;
            $log_file = preg_replace('#TYPE#',$action,$settings['template']['log_file_pattern']);
            if(file_exists($log_file)){
                $this->node()->os()->delete_file($log_file);
            }
            $result1 = Safe_sudo_exec($settings['template']['script_command']." ".$log_file." ".$allowed_actions[$action]." ",'','',$rs1);
            $ret = $rs1 == 0;
        }
        return $ret;
    }

    private function get_log($action){
        $settings = $this->get_data_settings();
        $log_file = preg_replace('#TYPE#',$action,$settings['template']['log_file_pattern']);
        $out = array();
        if(file_exists($log_file)){
            $content = file_get_contents($log_file);
            $content = str_replace("\r",PHP_EOL,$content);
            $out = explode(PHP_EOL,$content);
        }
        return $out;
    }
    
    public function api_apply_template($name=null,$data=null,&$output=null){
        $ret = $this->apply('config');
        $title = 'Provisioning: Apply Template '.($ret? 'Success' : 'Fail');
        $out = array_merge(array($title,str_repeat('-',strlen($title))),$this->get_log('config'));
        $output['message'] = implode(PHP_EOL,$out);
        return $ret;
    }

    public function api_register_template($name=null,$data=null,&$output=null){
        $ret = $this->apply('register');
        $title = 'Provisioning: Register SBC '.($ret? 'Success' : 'Fail');
        $out = array_merge(array($title,str_repeat('-',strlen($title))),$this->get_log('config'));
        $output['message'] = implode(PHP_EOL,$out);
        return $ret;
    }

    public function api_retrieve_template($name=null,$data=null,&$output=null){
        $settings = $this->get_data_settings();
        $template_file = dirname($settings['template']['pattern']).'/'.$settings['template']['file_name'];
        if(file_exists($template_file)){
            $this->node()->chmod_file($template_file,'650');
            $content = file_get_contents($template_file);
            $this->node()->chmod_file($template_file,'600');
            $content = str_replace("\r",PHP_EOL,$content);
            $output['data'] = array('content'=>$content);
            return array('content'=>$content);
        }else{
            $output['error'] = 'File not found';
        }
        return $ret;
    }
}

?>
