<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Kent <khong@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * SNG storage module class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');
safe_require_class('safe_auditor_class');

class Sng_storage_config_class extends Safe_configurable_object_class
{
    /**
     * @brief Constructor
     *
     * @param[in out] $parent_name
     * @param[in out] $name
     *
     * @return
     */
    public function __construct($node, $parent_name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, "configuration", $node);
    }
    /**
     * @brief Configure the config class object
     *
     * @return
     */
    public function configure()
    {
        // General settings
        $this->add_field('messages', 'Storage Messages', 'text', '', 40);
        $this->set_field_readonly('messages');
        return parent::configure();
    }
    
    public function summary($type = 'horizontal' , $long = false, $min=9)
    {
        $message = array();
        $module = $this->node()->software()->application()->find_module_by_name('storage');
        $raid = $module->api_retrieve_raid();
        if($raid){
            //have raid 
            $changes = $module->api_retrieve_change();
            if($changes){
                foreach($changes as $key => $change_obj){
                    $message['change'][] = $change_obj->get_change_message();
                }
                $message['title'] = 'RAID messages:';
            }else{
                $message['title'] = 'RAID in Active, Clean and Sync State.';
            }
        }
        return array(
                    'data' => array('raid' => $message),
                    );
    }
    public function save(){
        return true;
    }
    //storage module readonly
    public function is_modified(){
        return false;
    }
}
/**
 *
 * save the raid change event to Sng_storage_raid_status_config_class
 *
 */
class Sng_storage_raid_status_config_class extends Safe_configurable_object_class
{
    const DEVICE_REMOVE = "device-remove";
    const DEVICE_RECOVERY = "device-recovery";
    const DEVICE_FAIL = "device-fail";
    const DEVICE_SPARE = "device-spare";
    private $change_type = array(
                Sng_storage_raid_status_config_class::DEVICE_REMOVE => array(
                        'type' => 'warning',
                        'title' => 'HDD Removed.',
                        'description' => 'Please shutdown appliance and insert a new HDD.',
                        ),
                Sng_storage_raid_status_config_class::DEVICE_RECOVERY => array(
                        'type' => 'warning',
                        'title' => 'HDD Recovery in progress.',
                        ),
                Sng_storage_raid_status_config_class::DEVICE_FAIL => array(
                        'type' => 'warning',
                        'title' => 'HDD Failure',
                        'description' => 'Please shutdown appliance and replace the failure disk.',
                        ),
                Sng_storage_raid_status_config_class::DEVICE_SPARE => array(
                        'type' => 'info',
                        'title' => 'New HDD detected.',
                        ),
            );
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        // Contact fields
        $this->add_field('type', 'Type', 'text', '', 40);
        //json encode data
        $this->add_field('data', 'Data', 'text', '', 40);
        return parent::configure();
    }
    public function get_change_type(){
        return $this->get_data_value('type', false);
    }
    public function get_change_data(){
        return $this->get_data_value('data', false);
    }
    public function can_recover(){
        if($this->get_change_type() == Sng_storage_raid_status_config_class::DEVICE_SPARE){
            return true;
        }
        return false;
    }
    public function get_change_device(){
        $data = $this->get_change_data();
        if($data){
            if(isset($data['device']) && isset($data['device']['@attributes'])){
                unset($data['device']['@attributes']);
                $tmp_device= '';
                foreach($data['device'] as $key => $value){
                    if($value){
                        $tmp_device .= "{$key}:{$value} ";
                    }
                }
                return $tmp_device;
            }
        }
        return false;
    }
    public function get_change_replace(){
        $data = $this->get_change_data();
        if($data){
            if(isset($data['replaces']) && isset($data['replaces']['device'])){
                unset($data['replaces']['device']['@attributes']);
                $tmp_device= '';
                foreach($data['replaces']['device'] as $key => $value){
                    if($value){
                        $tmp_device .= "{$key}:{$value} ";
                    }
                }
                return $tmp_device;;
            }
        }
        return false;
    }
    public function get_change_message(){
        $output = array();
        $type = $this->get_change_type();
        $message = $this->change_type[$type];
        switch ($type) {
            case Sng_storage_raid_status_config_class::DEVICE_REMOVE:
                $message['device'] = $this->get_change_device();
                break;
            case Sng_storage_raid_status_config_class::DEVICE_RECOVERY:
                break;
            case Sng_storage_raid_status_config_class::DEVICE_FAIL:
                $message['device'] = $this->get_change_device();
                break;
            case Sng_storage_raid_status_config_class::DEVICE_SPARE:
                $message['device'] = $this->get_change_replace() . ' replaced by :' .
                                     $this->get_change_device();
                break;
        }
        return  $message;
    }
    public function save(){
        return true;
    }
}
class Sng_storage_raid_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->add_field('state', 'State', 'string', '', 50);
        $this->add_field('progress', 'Progress', 'string', '', 50);
        // Contact fields
        return parent::configure();
    }
    public function get_progress()
    {
        return $this->get_data_value('progress', false);
    }
    public function get_state()
    {
        return $this->get_data_value('state', false);
    }
    public function summary($type = 'horizontal' , $long = false, $min=3)
    {
        //circle spinner
        if('vertical' == $type){
            return parent::summary($type, $long, 20);
        }else{
            $class = 'info';
            $raid_state = $this->get_data_value('state', false);
            if($raid_state != 'normal'){
                $class = 'error';
            }
            $result =  array(
                        'Name' => $this->name(),
                        'State'=>  array(
                            'id' => 'raid_'.$this->name().'_status',
                            'data' => Safe_icon('circle', $class) . strtoupper($raid_state),
                            'formatter' => 'none',
                            ),
                        ''=>  array(
                                'id' => 'raid_'.$this->name().'_percent',
                                'data' => '',
                        )
                    );
            return array( 'data' => $result);
        }
    }
    public function is_readonly(){
        true;
    }
    public function save(){
        return true;
    }
}
class Sng_storage_fs_config_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $this->add_field('mount_point', 'Mount Point', 'string', '', 50);
        $this->add_field('size', 'Size', 'string', '', 50);
        $this->add_field('used', 'Used', 'string', '', 50);
        $this->add_field('avail', 'Avail', 'string', '', 50);
        $this->add_field('used_percent', 'Use%', 'string', '', 50);
        
        // Contact fields
        return parent::configure();
    }
    public function get_mount_point()
    {
        return $this->get_data_value('mount_point', false);
    }
    public function get_used($label = true)
    {
        $percent = $this->get_data_value('used_percent', false);
        if(!$percent){
            $percent = '0';
        }
        if($lable){
            $percent = $percent . '%';
        }
        return $percent;
    }
    public function summary($type = 'horizontal' , $long = false, $min=3)
    {
        //circle spinner
        if('vertical' == $type){
            return parent::summary($type, $long, 20);
        }else{
            $result =  array(
                    
                    'Mounted on' => $this->get_data_value('mount_point', false),
                    'Size'=>  $this->get_data_value('size', false),
                    'Used'=>  $this->get_data_value('used', false),
                    'Avail'=>  $this->get_data_value('avail', false),
                    'Usage (%)'=> array(
                            'id' => 'fs_' . str_replace('/', '_', $this->get_data_value('mount_point', false)) ,
                            'data' => $this->get_used() 
                            ) 
                    );
            return array( 'data' => $result);
        }
    }
    public function is_readonly(){
        true;
    }
    public function save(){
        return true;
    }
}
/******************************************************************************
 *   storage Module
 ******************************************************************************/
class Sng_storage_module_class extends Safe_module_class
{
    private $_fs = array();
    private $_raids = array();
    private $_changes = array();
    private $_script_path = '';
    public function __construct($app)
    {
        parent::__construct($app->node(), "storage");
        $this->_script_path = $this->node()->configuration_manager()->directory('sng-base') . '/bin';
    }
    /**
     * @brief
     *         
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("Storage");
        $cfg = new Sng_storage_config_class($this->node(), $this->path() . '/' . $this->name());
        // Synch with DB
        $cfg->configure();
        // Attach config to module
        $this->set_config($cfg);
        $this->register_aggregate_object('configuration', 
                    array(
                        'name' => 'Configuration',
                        'base_path' => false,
                        'readonly' => true,
                        'singleton' => true,
                        'configurable' => false,
                        'display' => true,//display the not configurable obj
                        'methods' => array(
                            'retrieve' => array(
                                'name' => 'Retrieve',
                                'request' => 'GET',
                            )
                        ),
                    )
                );
        $this->register_aggregate_object('fs',
                array(
                        'name' => 'File System',
                        // No base path as not stored in DB + avoid Safe_object doing
                        // magic
                        'base_path' => false, // $this->object_name().'/array',
                        'readonly' => true,  // do not allow delete, add...
                        'configurable' => false,
                        'display' => true,
                        'controller' => array(
                                'config_manager' => array(
                                        'summary_buttons' => ''
                                ),
                        ),
                        'methods' => array(
                                'retrieve' => array(
                                        'name' => 'Retrieve',
                                        'description' => 'Retrieve a file system',
                                        'request' => 'GET'
                                ),
                        )
                )
        );
        $this->register_aggregate_object('raid', 
            array(
                'name' => 'RAID',
                // No base path as not stored in DB + avoid Safe_object doing 
                // magic
                'base_path' => false, // $this->object_name().'/raid',
                'readonly' => true,  // do not allow delete, add...
                'configurable' => false,
                'display' => true,
                'controller' => array(
                        'config_manager' => array(
                                'summary_buttons' => 'show[Detail]'
                        ),
                ),
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve an raid',
                        'request' => 'GET'
                        ),
                    ),
                    'status' => array(
                            'name' => 'Status',
                            'description' => 'Retrieve raid status',
                            'request' => 'GET',
                    ),
               )
            );
        return parent::configure();
    }
    public function get_action_settings($action=null)
    {
        $action_def = array();
        //check if raid changes need to recover 
        if($this->need_recover()){
            $action_def['recover'] = array(
                    'name' => 'Recover Raid',
                    'icon' => 'repeat',
                    'description' => 'Recover Raid',
                    'request' => 'POST',
                    'confirm' => true,
                    'reload' => true
            );
        }
        if($action) {
            return $action_def[$action];
        }else{
            return $action_def;
        }
    }
    public function need_recover()
    {
        //go over change objs check can revoer
        $change_objs = $this->api_retrieve_change();
        if($change_objs){
            foreach($change_objs as $obj){
                if($obj->can_recover()){
                    return true;
                }
            }
        }
        return false;
    }
    private function _build_fs_objects()
    {
        if(!$this->_fs){
            $fs_stats = $this->node()->os()->fs_info(true);
            if($fs_stats){
                $_fs = array();
                $id = 0;
                foreach($fs_stats as $mount_point => $fs_info){
                    $obj_name = 'fs'.$id;
                    $id++;
                    $fs_info['mount_point'] = $mount_point;
                    $obj = new Sng_storage_fs_config_class($this->node(), $this->object_name().'/fs', $obj_name);
                    $obj->configure();
                    $obj->set_data_values($fs_info);
                    $_fs[$obj_name] = $obj;
                }  
            }
            $this->_fs = $_fs;
            ksort($this->_fs);
        }
        return $this->_fs;
    }
    
    private function _build_raid_objects()
    {
        if(!$this->_raids){
            $rc = $this->node()->os()->execute("nohup {$this->_script_path}/sng-raid-tool --status", $cmd_output, true, true);
            $cmd_output = implode('', $cmd_output);
            
            // Flush _raids
            $_raids = array();
            // 1/ Issue sng-raid-tool --status
            // 2/ Process output
            $xml_status = simplexml_load_string(trim($cmd_output));
            if($xml_status){
                // Search for array
                $array_query = $xml_status->xpath("//arrays/array");
                foreach($array_query as $a){
                    $name = trim((string)$a[name]);
                    if(!$name){
                        continue;
                    }
                    $obj = new Sng_storage_raid_config_class($this->node(), $this->object_name().'/raid', $name);
                    $obj->configure();
                    // Dynamically creates parameters
                    $tmp_obj_data = $obj->get_data_values(false);
                    foreach($a->children() as $child_name => $child_data){
                        // Add all text attributes
                        if(isset($tmp_obj_data[$child_name]) && !$child_data->children()){
                            //$obj->add_field($child_name, ucfirst($child_name), 'string', trim($child_data), '50');
                            $tmp_obj_data[$child_name] = trim($child_data);
                        }
                    }
                    
                    // Check if a <progress> element exists
                    $progress = 'N/A';
                    $state_query = $a->xpath("./*/progress");
                    if($state_query){
                        $progress = trim(trim($state_query[0]));
                    }
                    $tmp_obj_data['progress'] = $progress;
                    $obj->set_data_values($tmp_obj_data);
                    // store raid
                    $_raids[$name] = $obj;
                }
                ksort($_raids);
            }else if(false === $xml_status){
                safe_log(SAFE_LOG_ERROR, 'WARNING - Fail to parse RAID status');
            }
            
            $this->_raids = $_raids;
        }
        return $this->_raids;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    private function _build_change_objects()
    {
        if(!$this->_changes){
            $rc = $this->node()->os()->execute("nohup {$this->_script_path}/sng-raid-tool --changes --commit", $cmd_output, true, true);
            $cmd_output = implode('', $cmd_output);
            // Flush _changes
            $_changes = array();
            // 1/ Issue sng-raid-tool --changes
            // 2/ Process output
            $change_list = Safe_xml_to_array($cmd_output);
            if($change_list){
                if(isset($change_list['change'])){
                    if(!isset($change_list['change'][0])){
                        $tmp_data = $change_list['change'];
                        $change_list['change'] = array();
                        $change_list['change'][] = $tmp_data;
                    }
                    foreach($change_list['change'] as $key => $change_item){
                        $tmp_data = array();
                        if(isset($change_item['@attributes']) && isset($change_item['@attributes']['type'])){
                            $tmp_data['type'] = $change_item['@attributes']['type'];
                            if($tmp_data['type']){
                                unset($change_item['@attributes']);
                                $tmp_data['data'] = $change_item;
                                $obj_name = 'change'.$key;
                                $obj = new Sng_storage_raid_status_config_class($this->node(), $this->object_name().'/change', $obj_name);
                                $obj->configure();
                                $obj->set_data_values($tmp_data);
                                $_changes[$obj_name] = $obj;
                            }
                        }
                    }
                }
                $this->_changes = $_changes;
            }
        }
        
        return $this->_changes;
    }
    public function api_retrieve_status($name=null, $data=null, &$output = null) {
        $rc = $this->node()->os()->execute("nohup {$this->_script_path}/sng-raid-details", $cmd_output, true, true);
        $cmd_output = implode(PHP_EOL, $cmd_output);
        return $cmd_output;
    }
    public function api_status_raid($name=null, $data=null, &$output = null) {
        if($name){
            $rc = $this->node()->os()->execute('nohup /sbin/mdadm --detail /dev/'.$name, $cmd_output, true, true);
        }else{
            $rc = $this->node()->os()->execute("nohup {$this->_script_path}/sng-raid-details", $cmd_output, true, true);
        }
        $cmd_output = implode(PHP_EOL, $cmd_output);
        $key_word = array('degraded' =>'error',
                          'removed' => 'error',
                          'recovering' => 'error',
                          'spare rebuilding' => 'error',
                          'clean' => 'info',
                          'active sync' => 'info'
                        );
        
        foreach($key_word as $key => $color){
            $cmd_output = str_replace($key, "<b><span class='safe_{$color}'>{$key}</span></b>", $cmd_output);
        }
        return $cmd_output;
    }

    public function api_retrieve_raid($name=null, $data=null, &$output = null) {
        $_objs = $this->_build_raid_objects();
        if($name){
            return $_objs[$name];
        }else{
            return $_objs;
        }
    }
    public function api_retrieve_fs($name=null, $data=null, &$output = null) {
        $_objs = $this->_build_fs_objects();
        if($name){
            return $_objs[$name];
        }else{
            return $_objs;
        }
    }
    public function api_retrieve_change($name=null, $data=null, &$output = null) {
        $_objs = $this->_build_change_objects();
        if($name){
            return $_objs[$name];
        }else{
            return $_objs;
        }
    }
    public function api_recover_change($name=null, $data=null, &$output = null)
    {
        $rc = $this->node()->os()->execute("nohup {$this->_script_path}/sng-raid-tool --recover auto --commit", $output, true, true);
        sleep(2);
        //the first change request will return "device-fail"
        $this->node()->os()->execute("nohup {$this->_script_path}/sng-raid-tool --changes --commit", $cmd_output, true, true);
        return $rc;
    }
    public function api_recover_action($data=null, &$output = null) {
        $action_cfg = $this->get_action_settings('recover');
        $rc =  $this->api_recover_change(null, null, $output);
        if($rc){
            $output['message'] = array(
                    $action_cfg['description'] . " started."
            );
        }else{
            $output['message'] = array(
                    $action_cfg['description'] . " failed  to start."
            );
        }
        return $rc;
    }
    public function api_retrieve_configuration($obj_name=null, $params=null, &$output=null)
    {
        $summary = $this->config()->summary();
        $this->config()->set_data_values(array('messages' => $summary['data']));
        return $this->config();
    }
}
/* End of file sng_update_module_class.php */
