<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * William Adam <wadam@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * SNG System module class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');
safe_require_class('safe_auditor_class');
ini_set('memory_limit','1024M');
set_time_limit(3000);

class Sng_system_user_class extends Safe_configurable_object_class
{
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        // Contact fields
        $this->add_field('name', 'Name', 'text', '','50');
        $this->add_field('email', 'Email Address', 'text', '','50');
        $this->set_field_category('name', 'Contact');
        $this->add_enum_field('access', 'Allow Access', 'dropdown', 'true', $this->enable_disable_enum());
        $this->set_field_help('access','If disabled the system will block ssh/web access for the user.');

        $this->add_enum_field('sudoer', 'Sudoer', 'dropdown', 'true', $this->enable_disable_enum());
        $this->set_field_help('sudoer','If enabled it will add the user to sudoers list.');
        // Credentials fields
        $this->add_field('secure/password', 'Password', 'password', '','30');
        $this->set_field_rules('secure/password', 'required');
        $this->add_field('secure/verify', 'Verify', 'password', '','30');
        $this->set_field_rules('secure/verify', 'matches[secure/password]');
        $this->set_field_category('secure/password', 'Credentials');
        // SSH fields
        $this->add_enum_field('ssh/enable', 'SSH Login Without Password', 'dropdown', 'false',$this->enable_disable_enum());
        $this->set_field_help('ssh/enable','Public key authentication allows user to login to this host via the SSH protocol without a password.');
        $this->add_field('ssh/publickey', 'SSH Public Key', 'textarea', '',array(100, 7));
        $this->set_field_help('ssh/publickey',
                array('Paste your public key here.',
                      'The publickey only support "ssh-rsa" or "ssh-dsa".')
                );
        $this->set_field_rules('ssh/publickey', 'required[ssh/enable]');
        
        $this->create_group('sshkey',
                array(
                        'ssh/publickey'
                )
        );
        
        $this->conditional_control('ssh/enable', 'sshkey');
        $this->set_field_category('ssh/enable', 'SSH');
        
        return parent::configure();
    }
    public function prepare_edit()
    {
        parent::prepare_edit();
        if($this->status() == Safe_object_serializer_class::OBJ_STATUS_NEW){
            if($this->get_data_value('email', false) == ''){
                $host = $this->node()->hostname();
                $email_def = $this->name().'@'.$host;
                $this->add_field('email', 'Email Address', 'text', $email_def,'50');
            }
        }
    }
    public function summary($type = 'horizontal' , $long = false, $min = 4){
        $desc_custom = array('name', 'email', 'access', 'sudoer');
        $table_line = array("ID" => $this->name());

        foreach ($desc_custom as $desc_key){
            $table_line[$this->get_data_label($desc_key)] = $this->get_data_value($desc_key);
        }
        return array('data' => $table_line);
    }
    public function dispose()  
    {
        $prog = $this->node()->configuration_manager()->directory('sng-base') . '/bin/sng-user-mgmt';
        // Complete cmd
        $cmd .= ' --action=remove --force=true --user='.$this->name();
        // Ready to executeo
        $output = array();
        $rc = $this->node()->os()->execute($prog.' '.$cmd, $output, true, true);

        if($rc){
            $rc = parent::dispose();
            if($rc){
                $this->clear_deleted();
            }
            return $rc;
        }else{
            return false;
        }
    }
    public function can_modify()
    {
        $user = $_SESSION['user_login'];
        if ($user == $this->name() && $this->get_data_value('access') == 'Disable') {
            $this->set_error('access', 'You cannot disable the account you are logged in with. Please login with another user.');
            return false;
        }
        else if ($this->name() == 'root' && $this->get_data_value('sudoer') == 'Disable') {
            $this->set_error('sudoer', 'root cannot be removed from sudoers !');
            return false;
        }
        else return true;
    }
    public function save()
    {
        $update_psw = true;
        // Get the object status
        // For new user, then backend must be invoked using add
        // For modified user, then backend must be invoked using password
        $status = $this->status();
        /* There are 3 situations for which we need to run post save command in order to lock/unlock an account (--action=lock)
        // 1) if it is a new  user (--action=add) and the account access is set to Enabled (--action=lock --access=enabled)
        // 2) UNLOCK ACCOUNT: if --action=update and the account access is set to Enabled (--action=lock --access=enabled)
        // 3) LOCK ACCOUNT  : if --action=update and the account access is set to Disabled (--action=lock --access=disabled)
        */
        $stat = $this->get_data_value('access');
        $sudoer = $this->get_data_value('sudoer') == 'Enable' ? ' --sudoer ' : '';
        $do_lock = false;
        $user_info = $this->get_os_account_info();
        switch($status){
        case Safe_object_serializer_class::OBJ_STATUS_NEW:
            if ($stat == 'Disable') $do_lock = true;
            if($this->name() != 'root'){
                $cmd = '--action=add' . $sudoer;
                if($user_info){
                    $cmd = '--action=update';
                }else{
                    $cmd = '--action=add';
                }
                $cmd = $cmd . $sudoer;
                break;
            }else{
                // Have to skip password for root as it already has one
                $update_psw = false;
            }
        case Safe_object_serializer_class::OBJ_STATUS_UP_TO_DATE:
        case Safe_object_serializer_class::OBJ_STATUS_MODIFIED:
            $do_lock = true;
            $cmd = '--action=update' . $sudoer;
            break;
        default:
            return false;
        }

        $prog = $this->node()->configuration_manager()->directory('sng-base') . '/bin/sng-user-mgmt';
        // Complete cmd
        $cmd .= ' --user='.$this->name();
        // Update psw ?
        if($update_psw){
            $cmd .= ' --hashed-password="' . $this->get_password() . '" ';
        }
        $cmd .= ' --name="'.$this->description().'"';
        // Ready to executeo
        $output = array();
        $rc = $this->node()->os()->execute($prog.' '.$cmd, $output, true, true);

        // Lock user account
        if ($do_lock) {
            $post_cmd = '--action=lock --access=' . $stat . ' --user=' . $this->name();
            $rc = $this->node()->os()->execute($prog.' '.$post_cmd, $output, true, true);
        }

        if($rc){
            //get the user's home diretory again
            $user_home_dir = '';
            if(!$user_info){
                //if new user need to get the user info again
                $user_info = $this->get_os_account_info();
            }
            if(isset($user_info['home'])){
                $user_home_dir = $user_info['home'];
            }
            //check if enable ssh login without password
            if($user_home_dir != '' && file_exists($user_home_dir)){
                $os = $this->node()->os();
                $os->chown_file($user_home_dir, 'webconfig');
                $tmp_dir = $user_home_dir . '/.ssh';
                $tmp_file_name = $tmp_dir . '/authorized_keys';
                if($this->get_data_value('ssh/enable', false) == 'true'){
                    //create authorized_keys file
                    if(!file_exists($tmp_dir)){
                        $os->mkdir($tmp_dir);
                    }
                    $pub_key = $this->get_data_value('ssh/publickey', false);
                    if($pub_key){
                        $os->chown_file($tmp_dir, 'webconfig');
                        if(file_exists($tmp_file_name)){
                            $os->delete_file($tmp_file_name);
                        }
                        file_put_contents($tmp_file_name, $pub_key);
                        $os->chown_file($tmp_file_name, $this->name() . ':' . $this->name());
                    }
                    $os->chown_file($tmp_dir, $this->name() . ':' . $this->name());
                }else{
                    //delete authorized_keys file
                    $os->delete_file($tmp_file_name);
                }
                $os->chown_file($user_home_dir, $this->name() . ':' . $this->name());
            }
            parent::save();
            $this->clear_modified();
            return true;
        }else{
            return false;
        }
    }
    public function get_password()
    {
        return $this->get_data_value('secure/password', false);
    }
    public function validate($data, &$out_message)
    {
        $rs = parent::validate($data, $out_message);
        if($rs && isset($data['secure/password'])){
            if(!safe_is_hash_password($data['secure/password'])){
                // if password not hashed the hash it
                $new_data['secure/password'] = safe_hash_password($data['secure/password']);
                $new_data['secure/verify'] = $new_data['secure/password'];
            }
            $this->set_data_values($new_data);
        }
        if($this->get_data_value('ssh/enable', false) == 'true'){
            $pub_key = $this->get_data_value('ssh/publickey', false);
            //only support "ssh-rsa" or "ssh-dsa".
            if (preg_match('/^ssh-[r|d]sa/i', $pub_key) !== 1) {
                $this->set_error('ssh/publickey', 'SSH publickey only support "ssh-rsa" or "ssh-dsa".');
                $rs = false;
            }
        }
        return $rs;
    }

    public function can_dispose(&$reason=null)
    {
        // Check root user
        if('root' == $this->name()){
            $reason['message'] = 'User root cannot be removed !';
            return false;
        }
        if ($_SESSION['user_login'] == $this->name()) {
            $reason['msg'] = 'You cannot delete the user you are logged in with!';
            return false;
        }
        // Check if a module uses this one
        $usages = $this->node()->check_object_usage('system', 'user', $this->name());
        if($usages){
            $reason['message'] = "{$this->name()} is used by ";
            $reason['obj'] = $usages;
            return false;
        }
        // Check user not currently logged in
        $prog = $this->node()->configuration_manager()->directory('sng-base') . '/bin/sng-user-mgmt';
        // Complete cmd
        $cmd .= ' --action=logged --force=true --user='.$this->name();
        // Ready to executeo
        $output = array();
        $rc = $this->node()->os()->execute($prog.' '.$cmd, $output, true, true);
        if(!$rc){
            $reason['message'] = $output[0];
            return false;
        }
        return true;
    }
    public function email()
    {
        return $this->get_data_value('email', false);
    }
    public function description()
    {
        $name = $this->get_data_value('name');
        if($name){
            return $name;
        }else{
            return $this->name();
        }
    }
    public function get_os_account_info()
    {
        //ask back end script to return linux account info
        $prog = $this->node()->configuration_manager()->directory('sng-base') . '/bin/sng-user-mgmt';
        $cmd = ' --action=info --user='.$this->name();
        $output = array();
        $rc = $this->node()->os()->execute($prog.' '.$cmd, $output, true, true);
        if(!$rc){
            return false;
        }
        foreach ($output as $key => $tmp_line){
            if($tmp_line == '{'){
                break;
            }else{
                unset($output[$key]);
            }
        }
        $tmp_string = join(PHP_EOL,$output);
        $account_info =  json_decode($tmp_string, true);
        return $account_info;
    }
}
class Sng_system_update_config_class extends Safe_configurable_object_class
{
    const UPDATE = "update";
    const REPLACE = "replace";
    public function __construct($node, $parent_name, $name)
    {
        // Parent constructor to invoke unserialize if needed
        parent::__construct($parent_name, $name, $node);
    }
    public function configure()
    {
        $mode = array(
            self::UPDATE => 'Update existing configuration',
            self::REPLACE => 'Replace existing configuration',
            );
        $this->add_enum_field('mode', 'Mode', 'dropdown', 'update', $mode);
        $this->add_field('data', 'Configuration data', 'text', '', 50);
        $this->set_field_rules('data', 'required');

        return parent::configure();
    }
}

/******************************************************************************
 *   System Module
 ******************************************************************************/
class Sng_system_module_class extends Safe_module_class
{
    private $_users = array();
    const STORAGE_WARNING_LIMIT = 95;
    const STORAGE_WARNING_FILE  = '/tmp/storage_limit.warning';
    public function __construct($app)
    {
        parent::__construct($app->node(), "system");
    }
    /**
     * @brief
     *         
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("System");
        // Register objects
        $this->register_aggregate_object('user', 
            array(
                'name' => 'User',
                'dynamic' => true,
                'controller_url' => '/SAFe/sng_user_config',
                'methods' => array(
                    'create' => array(
                        'name' => 'Create',
                        'description' => 'Add a User',
                        'request' => 'POST'
                        ),
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve a User',
                        'request' => 'GET'
                        ),
                    'update' => array(
                            'name' => 'Update',
                            'description' => 'Update a User',
                            'request' => 'POST'
                    ),
                    'delete' => array(
                            'name' => 'Delete',
                            'description' => 'Delete a User',
                            'request' => 'POST'
                    )
               )
            )
        );
        // Register objects
        $t = new Sng_system_update_config_class($this->node(), 'temp', 'system_update_config');
        $t->configure();
        $p = $t->get_data();
        $this->register_aggregate_object('configuration', 
            array(
                'name' => 'System Configuration',
                'dynamic' => true,
                'base_path' => false,
                'singleton' => true,
                'readonly' => false,
                'configurable' => false,
                'methods' => array(
                    'retrieve' => array(
                        'name' => 'Retrieve',
                        'description' => 'Retrieve System Configuration',
                        'request' => 'GET'
                        ),
                    'update' => array(
                        'name' => 'Update',
                        'description' => 'Update System Configuration',
                        'request' => 'POST',
                        'class' => $p
                        )
                    )
                )
            );
        // Register objects
        $this->register_aggregate_object('os', 
            array(
                'name' => 'OS',
                'dynamic' => true,
                'base_path' => false,
                'singleton' => true,
                'readonly' => true,
                'configurable' => false,
                'methods' => array(
                    'status' => array(
                        'name' => 'Status',
                        'description' => 'Status',
                        'request' => 'GET'
                        ),
                    )
                )
            );
        // Register objects
        $this->register_aggregate_object('storage',
                array(
                        'name' => 'Storage',
                        'dynamic' => true,
                        'base_path' => false,
                        'singleton' => true,
                        'readonly' => true,
                        'configurable' => false,
                        'methods' => array(
                                'status' => array(
                                        'name' => 'Status',
                                        'description' => 'Status',
                                        'request' => 'GET'
                                ),
                        )
                )
        );
        // Register objects
        $this->register_aggregate_object('service', 
            array(
                'name' => 'Service',
                'dynamic' => true,
                'base_path' => false,
                'singleton' => true,
                'readonly' => true,
                'methods' => array(
                    'status' => array(
                        'name' => 'Status',
                        'description' => 'Status',
                        'request' => 'GET'
                        ),
                    )
                )
            );
        // Check if 'root' user defined
        // if not, create it
        $root = $this->api_retrieve_user('root');
        if(!$root){
            $host = $this->node()->hostname();
            $data = array(
                'name' => 'System Administrator',
                'secure/password' => 'sangoma',
                'secure/verify' => 'sangoma',
                'email' => 'root@' . $host
            );
            
            $root = $this->api_create_user('root', $data);
            //get the hash password from linux account
            if($account_info = $root->get_os_account_info()){
                $root->set_data_values(array(
                                        'secure/password' => $account_info['password'],
                                        'secure/verify' => $account_info['password'])
                                    );
            }
            $root->save();
        }
        return parent::configure();
    }
    public function post_restore($info, &$reason)
    {
        $rs = parent::post_restore($info, $reason);
        //trigger to restore user to linux system 
        $users = $this->api_retrieve_user();
        if($users){
            foreach($users as $key => $obj){
                if($key != 'root'){
                    if(!$obj->get_os_account_info()){
                        $obj->save();
                    }
                }
            }
        }
        return $rs;
    }
    public function new_fields(&$config_manager, $type)
    {
        $label = 'User ID';
        $config_manager->add_field('profile-name', $label, 'string', '', 50);
        $config_manager->set_field_rules('profile-name', 'required|alpha_dash');
    }
    const RESET_CONFIG = "config";
    const RESET_DATA = "data";
    const RESET_FACTORY = "factory";
    public function get_action_settings($action=null)
    {
        // Populate action table
        $action_def = array();

        // Shutdown action
        $shutdown_params =  new Safe_configurable_object_class('autofill', 'system_shutdown_params');
        $shutdown_mode = array(
            'reboot' => 'Restart',
            'halt' => 'Shutdown'
            );

        $shutdown_params->add_enum_field('mode', 'Mode', 'dropdown', 'reboot', $shutdown_mode);
        $shutdown_params->add_field('delay', 'Delay in second', 'hidden', '1','5');
        //$shutdown_params->set_field_rules('delay','required|greater_or_equal[1]');
        $action_def['shutdown'] = array(
                        'name' => 'Shutdown',
                        'button' => 'Apply',
                        'icon' => 'power-off',
                        'description' => 'Shutdown System',
                        'request' => 'POST',
                        'class' => $shutdown_params,
                        'redirect_url' => '?reserved_logout=y',
                        'controller_url' => array('ui' => '/SAFe/sng_system_shutdown')
                    );
        // Reset action
        $reset_params =  new Safe_configurable_object_class('autofill', 'system_reset_params');
        $reset_params->add_enum_field('type', 'Type', 'dropdown', '', 
            array(
                //self::RESET_CONFIG => 'Configuration', 
                //self::RESET_DATA => 'Data',
                self::RESET_FACTORY => 'Factory Settings and Reboot'
            )
        );
        $methods = array(
                'network' => 'Keep Network Configuration',
                'license' => 'Keep License Configuration',
        );
        $reset_params->add_multiple_field('reset_options', 'Options', 'inline_checkbox', array(), $methods);
        $action_def['reset'] = array(
                        'name' => 'Reset',
                        'icon' => 'eraser',
                        'pre' => 'You can only access the system using the current root credentials after factory '.
                        'reset. If you do not have the current root credentials, please cancel this '.
                        'dialog and change the root password before proceeding.  ',
                        'description' => 'Reset System',
                        'request' => 'POST',
                        'class' => $reset_params,
                        'redirect_url' => ''
                    );

        // Want all or just a specific one ?
        if($action) {
            return $action_def[$action];
        }else{
            return $action_def;
        }
    }
    public function get_action_url($action, $type='ui')
    {
        $action_setting = $this->get_action_settings($action);
        if($action_setting && isset($action_setting['controller_url'])){
            if(isset($action_setting['controller_url'][$type])){
                return $action_setting['controller_url'][$type];
            }
        }
        return false;
    }    
    public function api_shutdown_action($data=null, &$output = null) {
        $action_cfg = $this->get_action_settings('shutdown');
        $param = $action_cfg['class'];
        if ($param->validate($data, $output)) {
            $data = $param->get_data_values(false);
            $rc =  $this->node()->os()->shutdown($data, $output);
            return $rc;
        }
        return false;
    }
    public function backup_network_setting(){
        $backup_file_name = '/tmp/network_setting_backup.tgz';
        $network_backup_file_list =  array(
                '/etc/hosts',
                '/etc/sysconfig/network',
                '/etc/sysconfig/network-scripts/route-*',
                '/etc/sysconfig/network-scripts/ifcfg-eth*',
                '/etc/resolv.conf',
                '/etc/resolv-static.conf'
        );
        $tmp_network_backup_string = '';
        foreach ($network_backup_file_list as $file_item){
            $tmp_network_backup_string = $tmp_network_backup_string . $file_item . ' ';
        }
        $this->node()->os()->delete_file($backup_file_name);
        if($tmp_network_backup_string){
            $cmd = "/bin/tar --ignore-failed-read -cpszf  {$backup_file_name} {$tmp_network_backup_string} > /dev/null 2>&1";
            $output = array();
            $rc = $this->node()->os()->execute($cmd, $output, false, true);
            if($rc) {
                return true;
            }
        }
        return false;
    }
    public function restore_network_setting($keep_network=false){
        $backup_file_name = '/tmp/network_setting_backup.tgz';
        if(file_exists($backup_file_name)){
            $cmd = "/bin/tar -xpsvzf {$backup_file_name} -C / > /dev/null 2>&1";
            $output = array();
            $rs_tar = $this->node()->os()->execute($cmd, $output, false, true);
            if (!$rs_tar) {
                error_log('ERROR - Restore network setting backup failed.');
            }else{
                error_log('Restore network setting backup files OK.');
                //restart network service 
                $network = $this->node()->find_module_by_name('network');
                $rc_restart_network =  $network->api_restart_service(null, null, $output);
                if($rc_restart_network){
                    error_log('Restart network service OK.');
                    $rc_load_network = $this->node()->execute('sudo nohup /usr/local/nsc/hooks/network.restore.post --mode=update');
                    if(!$rc_load_network){
                        error_log('ERROR - Reload network setting failed.');
                    }else{
                        error_log('Reload network setting OK.');
                        //apply network changes
                        $network->clear_cache();
                        $output=array();
                        $success = $network->api_apply_action(null,$output);
                        // if keep_network we avoid error messages before reset ends.
                        if($success){
                            error_log('Apply network setting OK.');
                        }else if($keep_network){
                            error_log('WARNING - Keep network is on (so error ignored), but apply network settings fail due to: '.json_encode($output));
                        }
                        else{
                            error_log('ERROR - Apply network settings fail due to: '. json_encode($output));
                        }
                        return $success || $keep_network;
                    }    
                }else{
                    error_log('ERROR - Restart network service failed.');
                }
            }
        }else{
            error_log('ERROR - Network setting file not found.');
        }
        return false;
    }
    public function api_reset_action($data=null, &$output = null) {
        $action_cfg = $this->get_action_settings('reset');
        $param = $action_cfg['class'];
        if(isset($data['type'])){
            $reset_type = $data['type'];
            if($reset_type == self::RESET_CONFIG){// call from internal replace configuration, not delete license file
                $data['type'] = self::RESET_FACTORY;//pass by validation
                $reset_type = self::RESET_CONFIG;
            }
        }else{
            $output['result'] = false;
            $output['reason'] = 'Reset type error';
            $output['message'] = $action_cfg['description'].' failed.';
            $output['redirect_url'] = '/SAFe/sng_system_reset';
            return true;
        }
        
        $_output_validate = array();
        if ($param->validate($data, $_output_validate)) {
            $output['page_title'] = $action_cfg['description'];
            $_output_reset = array();
            $restor_data = array();
            if(isset($data['reset_options']) && $data['reset_options']){
                $restore_data = array('backup_exclude_opts' => $data['reset_options']);
                foreach($data['reset_options'] as $tmp_item){
                    if(strtolower($tmp_item) == 'network'){
                        $keep_network = true;
                        continue;
                    }
                    if(strtolower($tmp_item) == 'license'){
                        $keep_license = true;
                        continue;
                    }
                }
            }
            if('config' == $reset_type){
                $restore_data = array('backup_exclude_opts' => array('network', 'license'));
            }
            if($reset_type == self::RESET_FACTORY){
                //check if the '/var/sng/factory-reset/sng-factory-reset-image.tgz' exist
                $factory_reset_image = '/var/sng/factory-reset/sng-factory-reset-image.tgz';
                if (!$this->node()->os()->check_file_exists($factory_reset_image)) {
                    $output['result'] = false;
                    $output['reason'] = 'Cannot read factory reset image file';
                    $output['message'] = $action_cfg['description'].' '. $reset_type .' failed.';
                    $output['redirect_url'] = '/SAFe/sng_system_reset';
                    return true;
                }else{
                    //copy the  sng-factory-reset-image.tgz to arche
                    if($this->node()->os()->copy_file($factory_reset_image, '/usr/local/nsc/archive/sng-factory-reset-image.tgz')){
                        $data_key = 'sng-factory-reset-image.tgz';
                    }else{
                        //copy error
                        $output['result'] = false;
                        $output['reason'] = 'Cannot copy factory reset image file';
                        $output['message'] = $action_cfg['description'].' '. $reset_type .' failed.';
                        $output['redirect_url'] = '/SAFe/sng_system_reset';
                        return true;
                    }
                }
                
                $user_list = array();
                $user_obj_list = $this->api_retrieve_user();
                if($user_obj_list){
                    foreach($user_obj_list as $username => $user_obj){
                        $user_list[] = $username;
                    }
                }
                //backup network files
                if(isset($keep_network) && $keep_network){
                    if(!$this->backup_network_setting()){
                        $output['result'] = false;
                        $output['reason'] = 'Can not create network setting backup file.';
                        $output['message'] = 'Backup network setting failed.';
                        return true;
                    }
                }
                //call api_restore_archive, nsc is running return false
                if(!$this->node()->software()->application()->api_restore_archive($data_key, $restore_data, $_output_reset)){
                    $rc = false;
                }else{
                    $rc = true;
                    if('factory' == $reset_type){
                        //remove the user
                        if($user_list){
                            foreach($user_list as $username){
                                if($username != 'root'){
                                    $prog = $this->node()->configuration_manager()->directory('sng-base') . '/bin/sng-user-mgmt';
                                    $cmd  = ' --action=remove --force=true --user='.$username;
                                    $tmp_rc = $this->node()->os()->execute($prog.' '.$cmd, $tmp_msg, true, true);
                                }else{
                                    $prog = $this->node()->configuration_manager()->directory('sng-base') . '/bin/sng-user-mgmt';
                                    $cmd  = ' --action=lock --access=enable --user='.$username;
                                    $tmp_rc = $this->node()->os()->execute($prog.' '.$cmd, $tmp_msg, true, true);
                                }
                            }
                        }
                        if(!isset($keep_license) || !$keep_license){
                            //remove the license file too, when factory reset
                            $this->node()->os()->delete_file('/usr/local/nsc/conf/license/License.txt');
                            $this->node()->os()->delete_file('/usr/local/nsc/conf/license/License.txt.sig');
                        }
                        if(isset($keep_network) && $keep_network){
                            //restore network
                            if(!$this->restore_network_setting($keep_network)){
                                $output['result'] = false;
                                $output['reason'] = 'Can not create network setting backup file.';
                                $output['message'] = 'Backup network setting failed.';
                                return true;
                            }
                        }
                        $this->node()->os()->shutdown(array("mode" => 'reboot'), $output);
                    }
                }
            }else{
                $this->node()->set_config_status('update', 'rest');
                //internal reset clear all configurations
                $rc =  Safe_get_product()->do_reset(array("type" => 'config'), $_output_reset);
            }
           
            if($rc){
                $output['message'] = array(
                    $action_cfg['description'].' '.$reset_type.' succeed.',
                );
                if('factory' == $reset_type){
                    $output['message'][] = 'Restarting system ...';
                }
                $output['result'] = true;
                $output['redirect_url'] = '/SAFe/sng_system_shutdown';
            }else{
                $output['result'] = false;
                $output['reason'] = $_output_reset;
                $output['message'] = $action_cfg['description'].' '.$param->get_data_value('type').' failed.';
            }
        }else{
            $output['result'] = false;
            $output['reason'] = $_output_validate;
            $output['message'] = $action_cfg['description'].' '.$param->get_data_value('type').' failed.';
        }
        return true;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_create_user($name, $data=null, &$output = null) {
        /* foreach(debug_backtrace() as $trace){ */
        /*     if(isset($trace['file'])) */
        /*         print "[".date('Y-m-d H:i:s')."] ".__METHOD__."::".$trace['file'].":".$trace['line']."\n"; */
        /* } */

        $view = new sng_system_user_class($this->node(), $this->aggregate_object_base_path('user'), $name);
        $view->configure();
        if($data) {
            $view->set_data_values($data);
        }
        return $view;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_retrieve_user($name=null, $data=null, &$output = null) {
        $objs = $this->get_aggregate_objects('user');
        if($name){
            return $objs[$name];
        }else{
            return $objs;
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_update_user($name, $data=null, &$output = null) {
        $user = $this->api_retrieve_user($name);
        if($user) {
            if ($user->validate($data,$output)) {
                if (true == $user->save()) {
                    return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_delete_user($name, $data=null, &$output = null) {
        $user = $this->api_retrieve_user($name);
        if($user) {
            if($user->can_dispose($output)) {
                return $user->dispose();
            }
        }
        return false;
    }
    /**
     * @brief Check if we can create agg obj
     *
     * @param[in out] $type
     * @param[in out] $name
     * @param[in out] $reason
     *
     * @return 
     */
    public function can_create_aggregate_object($type, $name, &$reason="")
    {
        // base class check
        if(!parent::can_create_aggregate_object($type, $name, $reason)){
            return false;
        }
        // User object ?
        if('user' == $type){
            // Ask backend if user exists ?
            $prog = $this->node()->configuration_manager()->directory('sng-base') . '/bin/sng-user-mgmt';
            // Complete cmd
            $cmd .= ' --action=exist --user='.$name;
            // Ready to execute
            $output = array();
            $rc = $this->node()->os()->execute($prog.' '.$cmd, $output, true, true, $return_var);

            if(!$rc){
                //return_var  254'User exists', 255'User is system user'
                //event if user exists in the passwd, allow to overwrite it
                if($return_var != '254'){
                    $reason = $output[0];
                    return false;
                }
            }
        }
        return true;
    }
    public function statistics()
    {
        return $this->node()->os()->statistics();
    }
    public function api_status_os($name=null, $data=null, &$output = null)
    {
        $system_stats = array();
        // Get OS statistics
        $system = $this->node()->os()->statistics();
        $system_stats['stats'] = $system;

        // Get Network stats (in the end all should come from os()->statistics() )
        $network = $this->node()->find_module_by_name('network');
        $network_stats = $network->api_status_interface();
        foreach($network_stats as $if => $if_stats){
            $system_stats['stats']['network']['interface'][$if] = $if_stats;
        }
        $network_ip = $network->api_retrieve_ip();
        if($network_ip){
            foreach($network_ip as $ip_obj_name => $ip_obj){
                $address = $ip_obj->address();
                $prefix = $ip_obj->prefix();
                if($address){
                    if($prefix){
                        $address = $address . '/' . $prefix;
                    }
                }
                $system_stats['stats']['network']['ip'][$ip_obj_name] = $address ;
            }
        }
        

        // Compute series
        $memory_usage = $system['memory']['percent']['used'];
        $memory_free = $system['memory']['percent']['free'];

        $mem_data = array(
            array('label' => 'Free', 'data' => $memory_free, 'color' => '#6699FF'),
            array('label' => 'Used', 'data' => $memory_usage, 'color' => '#FFC266'),
        );
        $system_stats['series']['memory'] = $mem_data;

        // Get Cpu
        $cpu_user = $system['cpu']['percent']['user'];
        $cpu_system = $system['cpu']['percent']['system'];
        $cpu_wait = $system['cpu']['percent']['wait'];
        $cpu_idle = $system['cpu']['percent']['idle'];
        $cpu_idle = 100.0 - $cpu_user - $cpu_system - $cpu_wait;

        $cpu_data = array(
            array('label' => 'User', 'data' => $cpu_user, 'color' => '#FFC266'),
            array('label' => 'System', 'data' => $cpu_system, 'color' => '#DBB84D'),
            array('label' => 'Wait', 'data' => $cpu_wait, 'color' => '#D65C33'),
            array('label' => 'Idle', 'data' => $cpu_idle, 'color' => '#6699FF'),
        );
        $system_stats['series']['cpu'] = $cpu_data;
        return $system_stats;
    }
    public function api_status_storage($name=null, $data=null, &$output = null)
    {
        $system_stats = array();
        // FS stats series
        $storage = $this->node()->find_module_by_name('storage');
        $array_stats = $storage->api_retrieve_fs();
        $fs_id = 0;
        $flg_set_show_storage_warning=false;
        foreach($array_stats as $k => $v){
            $used = $v->get_used(false) ;
            if($used > self::STORAGE_WARNING_LIMIT){
                $flg_set_show_storage_warning=true;
            }
            $free = (string)(100 - $used);
            $fs_data = array(
                    array('label' => 'Free', 'data' => $free, 'color' => '#6699FF'),
                    array('label' => 'Used', 'data' => $used, 'color' => '#FFC266'),
            );
            $tmp_fs_data = $v->get_data_values(false);
            $mount_point = $tmp_fs_data['mount_point'];
            unset($tmp_fs_data['mount_point']);
            $system_stats['series']['fs'][$mount_point] = $fs_data;
            $system_stats['stats']['fs'][$mount_point] = $tmp_fs_data;
        }
        if($flg_set_show_storage_warning){
            if(!file_exists(self::STORAGE_WARNING_FILE)){
                file_put_contents(self::STORAGE_WARNING_FILE,1);
            }
        }else{
             if(file_exists(self::STORAGE_WARNING_FILE)){
               unlink(self::STORAGE_WARNING_FILE);
             }
        }
        $array_stats = $storage->api_retrieve_raid();
        foreach($array_stats as $array_name => $array_obj){
            $system_stats['stats']['raid'][$array_name] = array(
                    'state' => $array_obj->get_state(),
                    'progress' => $array_obj->get_progress(),
            );
        }
        return $system_stats;
    }
    public function handle_global_messages(&$msg_list){            
         if(file_exists(self::STORAGE_WARNING_FILE)){
              $msg_list[] = array(
                        'module' => 'system',
                        'cache_tag' => 'storage is full',
                        'global_errors' => array(
                                "message" => "Storage Almost Full. Some partition usage exceeds 95%",
                                "buttons" => array(array( 'label' => 'Check','url' => '/SAFe/sng_dashboard'))
                         ));
         }
    }
    public function api_status_service($name=null, $data=null, &$output = null)
    {
        $service_stats = array();
        $services = $this->node()->software()->services();

        foreach($services as $service){
            if($service->allow_dashboard()){
                $service_stats[$service->name()] = $service->api_status_service();
                $service_stats[$service->name()]['description'] = $service->description();
                $service_stats[$service->name()]['category'] = $service->category();
            }
        }
        return $service_stats;
    }
    public function api_retrieve_configuration($name=null, $data=null, &$output=null)
    {
        $configuration_data = array();

        $software = $this->node()->software();
        $modules = array();
        // Check if we want all
        $modules = $software->application()->modules();
        $modules = array_merge($modules, $software->services());
        foreach($modules as $mod_name => $module) {
            $agg_objs = $module->has_aggregate_objects(true);
            foreach($agg_objs as $obj_type => $obj_def) {
                if($obj_def['configurable'] === false) continue;
                if($obj_def['pseudo']){
                    continue;
                }
                $objs = $module->get_aggregate_objects($obj_type);
                if(!is_array($objs)){
                    if(is_subclass_of($objs, 'Safe_object_class')){
                        $configuration_data[$mod_name][$obj_type] = $objs->get_data_values(false);
                    }
                }else{
                    foreach($objs as $obj_name => $obj){
                        if(is_subclass_of($obj, 'Safe_object_class')){
                            $configuration_data[$mod_name][$obj_type][$obj_name] = $obj->get_data_values(false);
                            // Check this one has sub object
                            $agg_child_objs = $obj->has_aggregate_objects(true);
                            foreach($agg_child_objs as $sub_type => $sub_def){
                                if($sub_def['configurable'] === false) continue;
                                if($sub_def['pseudo']){
                                    continue;
                                }
                                $sub_objs = $obj->get_aggregate_objects($sub_type);
                                foreach($sub_objs as $sub_name => $sub_obj){
                                    $configuration_data[$mod_name][$obj_type][$obj_name][$sub_type][$sub_name] = $sub_obj->get_data_values(false);
                                }
                            }
                        }else if(is_array($obj)){
                            $configuration_data[$mod_name][$obj_type][$obj_name] = $obj;
                        }
                    }
                }
            }
            //pass by if configuration definition ['configurable'] === false
            $configuration_def = $module->get_aggregate_object_definition('configuration');
            if(!isset($configuration_def['configurable']) || $configuration_def['configurable'] !== false){
                if($cfg_obj = $module->config()){
                    $configuration_data[$mod_name]['configuration'] = $cfg_obj->get_data_values(false);
                }
            }
        }
        return $configuration_data;
    }

    private function _update_configuration($obj, $data, &$output, $validation_check = true)
    {
        $rc = true;
        // Next level is either object itself or children
        // Do we have children ?
        $child_obj_def = $obj->get_aggregate_object_definition();
        
        $children = array();
        if($child_obj_def){
            // Save for later
            foreach($child_obj_def as $child_name => $child_def){
                $obj->clear_cache($child_name);
                if(isset($data[$child_name])){
                    $children[$child_name] = $data[$child_name];
                    unset($data[$child_name]); 
                }
            }
        }
        // Something remain in data ?
        if(!empty($data)){
            //if the data is for module self update the module config obj
            $_output = array();
            if(is_subclass_of($obj, "Safe_module_class")){
                if(isset($data['configuration'])){
                    if($validation_check){
                        //load the module config data from db for validation
                        $module_config_obj = $obj->config();
                        if($module_config_obj){
                            $module_config_obj->configure();
                            $module_config_obj->synch();
                        }
                    }
                    if(!$obj->api_update_configuration($obj->name(), $data['configuration'], $_output, $validation_check)){
                        if(!empty($_output)){
                            $output['configuration'] = $_output;
                            error_log ( "WARNING - validate obj "  . get_class($obj) . ' [' . $obj->name() .']' . var_export($_output,true) );
                        }
                        //even the configuration have error, sub child can still add to db like network
                        //return false;
                    }
                }
            }else{
                //validation
                if($validation_check){
                    //filter out disabled data field first
                    $obj->prepare_edit();
                    $data = $obj->data_filter($data);
                    if(!$obj->is_readonly()){
                        if($obj->validate($data, $_output)){
                            if(substr($obj->name(), 0, 18) == 'auto_firewall_rule'){
                                $obj->set_data_values(array('read_only' => 'true'));
                            }
                            $obj->save();
                            error_log ( "INFO - save obj " . get_class($obj) . ' [' . $obj->name() .']' );
                        }else{
                            if(!empty($_output)){
                                $output = $_output;
                                error_log ( "WARNING - validate obj "  . get_class($obj) . ' [' . $obj->name() .']' . var_export($_output,true) );
                            }
                            return false;
                        }
                    }
                }else{
                    //just save data to db without trigger any event 
                    $obj->set_data_values($data);
                    $obj->serialize();
                    error_log ( "INFO - add db record " . get_class($obj) . ' [' . $obj->name() .']' );
                }
            }
        }else{
            // To create object with default values, user may not submit any
            // $data. But here $obj may also be a module, so need to ensure we can
            // invoke save by checking it's a Safe_configurable_object_class
            // object
            if(is_a($obj, "Safe_configurable_object_class")){
                $obj->save();
                error_log ( "INFO - add obj with default value " . get_class($obj) . ' [' . $obj->name() .']' );
            }
        }
        // Proceed with childs
        foreach($child_obj_def as $child_name => $child_def){
            if(isset($children[$child_name])){
                $obj->clear_cache($child_name);
                $child_keys = $obj->get_aggregate_objects_keys($child_name);
                // Proceed all child
                foreach($children[$child_name] as $obj_name => $obj_data){
                    $_output = array();
                    $child = null;
                    // Update or create ?
                    if($child_keys && in_array($obj_name, $child_keys)){
                        //update
                        $child = $obj->get_aggregate_object($child_name, $obj_name);
                    }else{
                        //create, add $obj_data parameter (for network interface, different type using different clss)
                        if(substr($obj_name, 0, 18) == 'auto_firewall_rule'){
                            continue;
                        }
                        $child = $obj->create_aggregate_object($child_name, $obj_name, $_output, $obj_data);
                    }
                    // Update the child
                    if(null != $child && false !== $child){
                        //if IP obj check conflic and delete the old one
                        if($child_name == 'ip' && $obj->name() == 'network'){
                            $child->set_data_values($obj_data);
                            $ip_conflict_list = $obj->check_ip_conflict($child);
                            if($ip_conflict_list){
                                foreach($ip_conflict_list as $tmp_ip_obj){
                                    $list[] = $obj->name();
                                    $tmp_ip_obj->dispose();
                                    safe_log(SAFE_LOG_INFO, "Updating IP obj {$obj->name()} => {$obj_name} with IP address:{$child->address()}");
                                }
                                $obj->clear_cache($child_name);
                            }
                            //check if already have dhcp ip obj, delete the old one, add new one
                            if($child->is_dhcp()){
                                $ip_objs = $obj->get_aggregate_objects('ip');
                                $interface = $child->get_interface();
                                $my_name = $child->name();
                                $my_ip_type = $child->ip_type();
                                foreach($ip_objs as $ip_obj_name => $ip_obj){
                                    if($ip_obj_name != $my_name && $ip_obj->ip_type() == $my_ip_type && $ip_obj->get_interface() == $interface && $ip_obj->is_dhcp()){
                                        $ip_obj->dispose();
                                        safe_log(SAFE_LOG_INFO, "Updating DHCP IP obj {$obj->name()} => {$obj_name} on interface:{$interface}");
                                    }
                                }
                                $obj->clear_cache($child_name);
                            }
                        }
                        if(!$this->_update_configuration($child, $obj_data, $_output, $validation_check)){
                            $rc = false;
                        }
                    }else{
                        $rc = false;
                        error_log ( "WARNING - create obj fail "  . $child_name . ' [' . $obj_name .']' . var_export($_output,true) );
                    }
                    if(!empty($_output)){
                        $output[$child_name][$obj_name] = $_output;
                    }
                }
                $obj->clear_cache($child_name);
            }
        }
        // clear agg_objects_cache list and force reload objs from db next time
        $obj->clear_cache();
        return $rc;
    }
    public function api_update_configuration($name=null, $data=null, &$output=null)
    {
        $rc = true;
        $t = new Sng_system_update_config_class($this->node(), 'temp', 'system_update_config');
        $t->configure();
        if(!$t->validate($data, $output)){
            return false;
        }
        $mode = $t->get_data_value('mode', false);
        $configuration_data = $t->get_data_value('data', false);
        // Check if data is supplied as a string or an array
        if(!is_array($configuration_data)){
            // this should be a json string then ;)
            $configuration_data = json_decode($configuration_data, true);
            if(JSON_ERROR_NONE != json_last_error()){
                $output = 'Fail to decode json data: ' . json_last_error_msg();
                return false;
            }
        }
        safe_log(SAFE_LOG_INFO, 'Updating system configuration mode=' . $mode);
        // Do we have to wipe configuration ?
        if('replace' == $mode){
            // wipe config, fingers crossed ;)
            $action_cfg = $this->get_action_settings('reset');
            $param = $action_cfg['class'];
            $param_data = $param->get_data_values(false);
            $param_data['type'] = self::RESET_CONFIG;
            $_output = array();
            $rc_reset = $this->api_reset_action($param_data, $_output);
            if(!$rc_reset){
                $output = $_output;
                return false;
            }
        }
        $software = $this->node()->software();
        $modules = array();
        // Check if we want all
        $modules = $software->application()->modules();
        $modules = array_merge($modules, $software->services());
        //re oreder the data squence
        $new_configuration_data = array();
        foreach($modules as $mod_name => $tmp_module) {
            if(isset($configuration_data[$mod_name])){
                if($mod_name == 'network'){
                    //order the squence of network obj, interface first
                    $sub_data_sq = array("configuration", "interface", 'ip', 'route');
                    $new_configuration_data[$mod_name]  = array();
                    foreach($sub_data_sq as $tmp_obj_type){
                        if(isset($configuration_data[$mod_name][$tmp_obj_type])){
                            $new_configuration_data[$mod_name][$tmp_obj_type] = $configuration_data[$mod_name][$tmp_obj_type];
                        }
                    }                    
                }else{
                    $new_configuration_data[$mod_name] = $configuration_data[$mod_name];
                }
            }
        }
        $this->node()->set_config_status('update', 'rest');
        // let's go over all module entries and recurse
        if('replace' == $mode){
            //only do this in replace mode
            $update_data_output = array();
            foreach($new_configuration_data as $module_name => $module_data){
                if($module_name == 'nsc'){
                    //ignor nsc settings
                    continue;
                }
                // 1st level is module
                $module = $this->node()->find_module_by_name($module_name);
                if(!$module){
                    $update_data_output[$module_name] = 'Unknown module.';
                    continue; 
                }
                $_output = array();
                //pass by validation check first
                $rc_set_data =$this->_update_configuration($module, $module_data, $_output, false);
                
                if(!$rc_set_data || !empty($_output)){
                    if(!empty($_output)){
                        $update_data_output[$module_name] = $_output;
                    }
                    $rc = false;
                }
            }
            if(!empty($update_data_output)){
                $output['data_update'] = $update_data_output;
                $rc = false;
            }
        }
        $validation_output = array();
        foreach($new_configuration_data as $module_name => $module_data){
            if($module_name == 'nsc'){
                //ignor nsc settings
                continue;
            }
            $module = $this->node()->find_module_by_name($module_name);
            if(!$module){
                $output[$module_name] = 'Unknown module.';
                continue;
            }
            $_output2 = array();
            //validate data and save again
            $rc_validate = $this->_update_configuration($module, $module_data, $_output2, true);
            if(!$rc_validate || !empty($_output2)){
                if(!empty($_output2)){
                    $validation_output[$module_name] = $_output2;
                }
                $rc = false;
            }
        }
        if(!empty($validation_output)){
            $output['data_validation'] = $validation_output;
            $rc = false;
        }

        if($rc == false && 'replace' == $mode){
            /* api_reset_action called above removes network config files so we need to restore network config or
               network access will be lost on system reboot */
            $network = $this->node()->find_module_by_name('network');
            $network->clear_cache();
            $success = $network->api_apply_action(null,$output_3);
            if($success){
                error_log('Restoring network setting OK.');
            }else{
                error_log('ERROR - Restoring network settings fail due to: '. json_encode($output_3));
            }
        }

        $this->node()->clear_config_status('update');
        return $rc;
    }
    public function users()
    {
        $this->_users = $this->get_aggregate_objects('user');
        return $this->_users;
    }
    public function root_enabled()
    {
        $user = $this->get_aggregate_object('user', 'root');
        if (isset($user)) {
            return $user->get_data_value('access');
        } else return 'Disable';
    }
    public function checklist(&$list)
    {
        $users = $this->users();
        if($this->node()->get_config_status('update') == 'rest'){
            return parent::checklist($list);
        }else{
            if (isset($users['root']) && count($users) == 1){
                $list[] = array(
                        'module' => 'system',
                        'obj_type' => 'user',
                        'description' => $this->root_enabled() !== 'Enable' ? 'Non-root user must be defined' : 'Non-root user must be defined and the login access of root user must be disabled',
                );
            } else {
                $root_enabled = $this->root_enabled();
                if ($root_enabled == 'Enable') {
                    $list[] = array(
                            'module' => 'system',
                            'obj_type' => 'user',
                            'description' => 'Root user must be disabled',
                    );
                }
            }    
        }
    }
}
/* End of file sng_update_module_class.php */
