<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Transcoding adapter configuration class
 *
 * @author William Adam
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('product');
class Sng_transcoding_config_class extends Safe_configurable_object_class
{
    const HIDDEN_MODE = 'hardware-hidden';
    const SOFTWARE_MODE = 'software';
    public function __construct($node, $parent_name)
    {
        parent::__construct($parent_name, 'configuration', $node);
    }
    public function detect_interfaces()
    {
        $mgmt_if = $this->_node->hardware()->adapters();
        $if_enum = array();
        foreach ($mgmt_if as $if_name => $if) {
            if (strlen($if->ip_address())) {
                if (!strlen($def_if)) $def_if = $if->name();
                $if_enum[$if->name() ]['name'] = $if->name();
                $if_enum[$if->name() ]['type'] = $if->type();
            }
        }
        // We must have at least one entry or we're in trouble
        if (!strlen($if_enum)) {
            // TODO(wadam) - Do something
            
        }
        return $if_enum;
    }
    /**
     * Determines which mode the object is in, and returns which adapters to use for signalling
     * @return string interfaces
     */
    public function adapter_type()
    {
        $mode = $this->require_transcoding_interface();
        if ($mode == true) $interfaces = null;
        else $interfaces = Safe_hardware_adapter_class::ETHERNET_ADAPTER;
        return $interfaces;
    }
    /**
     * Checks if media interfaces are enabled
     * @return boolean
     */
    public function media_check()
    {
        $config = $this->configure();
        return $this->is_enabled(); 
    }
    /**
     * Returns the IP Mode of the media interfaces (Hidden/Exposed)
     * @return string
     */
    public function ip_mode()
    {
        $transcoding_mode_data = $this->get_data_value('mode', false);
        return $transcoding_mode_data;
    }
    public function vlan_id()
    {
        return $this->get_data_value('vlan_id', false);
    }
    /**
     * Determines whether a transcoding interface is required for signalling
     * @return boolean
     */
    public function require_transcoding_interface()
    {
        $config = $this->configure();
        $transcoding_mode_data = $this->get_data_values(false);
        $mode = false;
        //Put hidden in if statement just in case
        if ($transcoding_mode_data['mode'] == Sng_transcoding_config_class::HIDDEN_MODE){
            $mode = true;
        }
        return $mode;
    }
    public function configure()
    {
        $network= $this->node()->software()->application()->find_module_by_name('network');
        $transcoding_interfaces = $network->hardware_interface(Sng_network_interface_class::TRANSCODING);
        
        $internal_adapter = false;
        if($transcoding_interfaces){
            $internal_adapter = true;
        }
        $mode = array(
            Sng_transcoding_config_class::SOFTWARE_MODE => 'Software',
            Sng_transcoding_config_class::HIDDEN_MODE  => 'Hardware Hidden',
        );
        
        $media_int_group_field = array();
        $global_group_field = array();
        if ($internal_adapter) {
            $default_udp_begin = '14000';
            $default_udp_end = '17999';
        }else{
            $default_udp_begin = '10000';
            $default_udp_end = '20000';
        }
        
        $this->add_enum_field('mode', 'Transcoding Mode', 'dropdown', Sng_transcoding_config_class::HIDDEN_MODE, $mode);
        $this->set_field_help('mode', 
                array(
                        'Sets the Transcoding Mode',
                        array(
                                'Software' => array(
                                        'Disable media interfaces.',
                                        ),
                              ),
                        array(
                                'Hidden' => array(
                                        'All media interfaces will be hidden and protected from the external network. Signaling and media will flow together thru the primary network interfaces (ie, eth0 and eth1).',
                                        'Do NOT plug an ethernet cable to the media ethernet port (D500/D100).'
                                        )
                             ),
                        array(
                                'Exposed' => array(
                                        'All media interfaces will be exposed in the network. Media will flow thru the media ethernet port only.',
                                        'You must connect an ethernet cable to the media ethernet port (D500/D100)'
                                        )
                            )
                    )
            );
        
        $this->add_field('udp_begin', 'First UDP port', 'text', $default_udp_begin);
        $this->set_field_rules('udp_begin', 'required|valid_port_number|greater_or_equal[5004]');
        $this->set_field_help('udp_begin', 'Starting UDP port to be assigned to the media modules.');
        $this->add_field('udp_end', 'Last UDP port', 'text', $default_udp_end);
        $this->set_field_rules('udp_end', 'valid_port_number');
        $this->set_field_help('udp_end', 'Ending UDP port to be assigned to the media modules.');
        $this->add_field('vlan_id', 'VLAN Identifier', 'string', '', 10);
        $this->set_field_rules('vlan_id', 'greater_than[0]|less_than[4097]');
        $this->set_field_help('vlan_id', 'Specify the VLAN identifier that will be used on all media modules discovered.');
        $this->add_enum_field('g729_transcoding', 'Enable G729 Transcoding', 'dropdown', 'false', $this->enable_disable_enum());
        $this->set_field_help('g729_transcoding', 
            array(
                'Enable/Disable G729 transcoding',
                'When working in software mode, this toggle allows enabling disabling transcoding for G729 codec.',
                'By default in software mode G729 codec is used in passthrough mode where the SBC just forwards the reveived',
                'G729 packets without encoding/decoding them.',
            )
        );
        $this->set_field_category('mode', 'General');
        
        $this->add_field('ip_address', 'Base IPv4 address', 'text', '10.10.0.1');
        
        $this->set_field_rules('ip_address', 'required|valid_ip[ipv4]');
        
        $this->set_field_help('ip_address', array(
            'Starting IP Address to be assigned to the media modules.',
            'For every module discovered, the IP address will be incremented by 1.'
        ));
        $this->add_field('ext_ip_address', 'Base External IPv4 address', 'text', '');
        $this->set_field_rules('ext_ip_address', 'valid_ip[ipv4]');
        $this->set_field_help('ext_ip_address', array(
                'This is the starting IP address that system will use in the SDP offer or answer. For every module discovered, the IP address will be incremented by 1.',
                'If not specified, system will use the assigned IP address to the media module(default behavior).',
                '',
                'When system is behind a firewall (ie: DMZ or port-forwarding) with RTP ports being forwarded to the media module,
    		it may be desirable to specify this parameter to an IP address that will be mapped by the firewall port-forwarding capabilities to the IP address of the media module.
    		If not set when working under a DMZ or port-forwarding scenario, calls may end up not having audio.
    		'
        ));
        $this->add_field('mask', 'IPv4 Network Mask', 'text', '255.255.255.0');
        $this->set_field_rules('mask', 'required|valid_ip[ipv4]');
        $this->set_field_help('mask', 'Subnet mask to be assigned to the media modules.');
        $this->add_field('gateway', 'Default IPv4 Gateway', 'text', '10.10.0.100');
        $this->set_field_rules('gateway', 'valid_ip[ipv4]');
        $this->set_field_help('gateway', 'Default Gateway to be assigned to the media modules.');
        
        //set a default ip for Base IPv6 address
        $this->add_field('ipv6_address', 'Base IPv6 address', 'text', '2001:db8::99:2', 40);
        $this->set_field_rules('ipv6_address', 'valid_ip[ipv6]');
        
        $this->add_field('ext_ipv6_address', 'Base External IPv6 address', 'text', '', 40);
        $this->set_field_rules('ext_ipv6_address', 'valid_ip[ipv6]');
        
        $this->add_field('ipv6_prefix', 'IPv6 prefix', 'text', '120');
        $this->set_field_rules('ipv6_prefix', 'integer|greater_than[0]|less_or_equal[128]');
        
        $this->add_field('ipv6_gateway', 'Default IPv6 Gateway', 'text', '2001:db8::99:1', 40);
        $this->set_field_rules('ipv6_gateway', 'valid_ip[ipv6]');
        
        
        // Create group for mode control
        $media_int_group_field = array_merge($media_int_group_field, array(
                    'ip_address',
                    'ipv6_address',
                    'vlan_id',
                    'ext_ip_address',
                    'ext_ipv6_address',
                    'mask',
                    'ipv6_prefix',
                    'gateway',
                    'ipv6_gateway',
                    'udp_end',
                    'g729_transcoding',
                    'vqe/acoustic_echo',
                    'vqe/rx_adaptive_noise_reduction',
                    'vqe/rx_automatic_level_control',
                    'vqe/rx_automatic_level_control_target_dbm0',
                    'vqe/tx_automatic_level_control',
                    'vqe/tx_automatic_level_control_target_dbm0',
                    'jitter/mode',
                    'jitter/max_pdv_ms',
                    'jitter/min_delay_ms',
                    'jitter/initial_delay_ms'
                ));
        $hardware_enable_file = array(
                                            'ip_address',
                                            'ipv6_address',
                                            'vlan_id',
                                            'ext_ip_address',
                                            'ext_ipv6_address',
                                            'mask',
                                            'ipv6_prefix',
                                            'gateway',
                                            'ipv6_gateway',
                                            'udp_end',
                                            'g729_transcoding'
                                            );
        foreach($media_int_group_field as $group_key){
            if($group_key == 'udp_end' || $group_key == 'g729_transcoding'){
                $condition_data[Sng_transcoding_config_class::HIDDEN_MODE][] = array('field_name' =>$group_key, 'status' => 'disable');
                $condition_data[Sng_transcoding_config_class::SOFTWARE_MODE][] = array('field_name' =>$group_key, 'status' => 'enable');
            }elseif(in_array($group_key, $hardware_enable_file)){
                $condition_data[Sng_transcoding_config_class::HIDDEN_MODE][] = array('field_name' =>$group_key, 'status' => 'disable');
                $condition_data[Sng_transcoding_config_class::SOFTWARE_MODE][] = array('field_name' =>$group_key, 'status' => 'disable');
            }else{
                $condition_data[Sng_transcoding_config_class::HIDDEN_MODE][] = array('field_name' =>$group_key, 'status' => 'enable');
                $condition_data[Sng_transcoding_config_class::SOFTWARE_MODE][] = array('field_name' =>$group_key, 'status' => 'disable');
            }
        }
        $this->create_group('media_int',
                $media_int_group_field,
                array('default' => array( 'value' => 'true', 'status' => 'enable'),
                        'condition' => $condition_data
                )
        );
        $this->conditional_control('mode', 'media_int');

        // VQE
        $this->add_enum_field('vqe/acoustic_echo', 'Acoustic Echo Cancellation', 'dropdown', '0', $this->enable_disable_enum(false));
        $this->set_field_help('vqe/acoustic_echo',
            array(
                'Enable acoustic echo cancellation for all media sessions.',
                'This reduces echo on calls but increases the possibility of double-talk clipping.',
                'Use this value for situations where the echo path is highly non-linear.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 35%.</font>')
            );

        $this->add_enum_field('vqe/rx_adaptive_noise_reduction', 'Adaptive Noise Reduction', 'dropdown', '0', $this->enable_disable_enum(false));
        $this->set_field_help('vqe/rx_adaptive_noise_reduction',
            array(
                'Enable Adaptive Noise Reduction for all media sessions.',
                'This removes the noise present while leaving voice intact.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 35%.</font>')
            );

        $this->add_enum_field('vqe/rx_automatic_level_control', 'Automatic Level Control (RX)', 'dropdown', '0', $this->enable_disable_enum(false));
        $this->set_field_help('vqe/rx_automatic_level_control',
            array(
                'Enable RX Automatic Level Control for all media sessions.',
                'Using the level threshold limit specified in Automatic Level Control Target, dBm0 level is being filtered.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 15%.</font>')
            );
        $this->add_field('vqe/rx_automatic_level_control_target_dbm0', 'Target', 'integer', -21, 5);
        $this->set_field_rules('vqe/rx_automatic_level_control_target_dbm0', 'required[vqe/rx_automatic_level_control]|integer|greater_or_equal[-42]|less_or_equal[-6]');
        $this->composite_layout('vqe/rx_automatic_level_control', array('vqe/rx_automatic_level_control_target_dbm0'));

        $this->add_enum_field('vqe/tx_automatic_level_control', 'Automatic Level Control (TX)', 'dropdown', '0', $this->enable_disable_enum(false));
        $this->set_field_help('vqe/tx_automatic_level_control',
            array(
                'Enable TX Automatic Level Control for all media sessions.',
                'Using the level threshold limit specified in Automatic Level Control Target, dBm0 level is being filtered.',
                '<font color="red">Note that enabling this feature will reduce your call capacity in about 15%.</font>')
            );
        $this->add_field('vqe/tx_automatic_level_control_target_dbm0', 'Target', 'integer', -21, 5);
        $this->set_field_rules('vqe/tx_automatic_level_control_target_dbm0', 'required[vqe/tx_automatic_level_control]|integer|greater_or_equal[-42]|less_or_equal[-6]');
        $this->composite_layout('vqe/tx_automatic_level_control', array('vqe/tx_automatic_level_control_target_dbm0'));

        // Create conditional groups
        $this->create_group('rx_alc', array('vqe/rx_automatic_level_control_target_dbm0'),
                array(  'default' => array( 'value' => '0', 'status' => 'disable'),
                        'condition' => array('1' => array('field_name' =>'vqe/rx_automatic_level_control_target_dbm0','status' => 'disable'))
                        )
                );
        $this->conditional_control('vqe/rx_automatic_level_control', 'rx_alc');

        $this->create_group('tx_alc', array('vqe/tx_automatic_level_control_target_dbm0'),
                array(  'default' => array( 'value' => '0', 'status' => 'disable'),
                        'condition' => array('1' => array('field_name' =>'vqe/tx_automatic_level_control_target_dbm0','status' => 'disable'))
                )
                );
        $this->conditional_control('vqe/tx_automatic_level_control', 'tx_alc');

        // jitter buffer configuration
        $this->add_enum_field('jitter/mode', 'Jitter Mode', 'dropdown',
                'adaptive', array ( 
                    'static' => 'Static',
                    'adaptive' => 'Adaptive' 
               )
            );
        $this->set_field_help('jitter/mode', array ( 
                    'Jitter buffer mode to operate on.',
                    'Static allocates a fixed buffer.',
                    'Adaptive changes the buffer on runtime depending on the network'
                    )
                ); 

        $this->add_field('jitter/initial_delay_ms', 'Jitter Initial
                Delay', 'integer', 40, 5);
        $this->set_field_rules('jitter/initial_delay_ms',
                'required[jitter/mode]|integer|greater_or_equal[20]|less_or_equal[jitter/max_pdv_ms]'); 
        $this->set_field_help('jitter/initial_delay_ms','Initial delay (buffer size) in milliseconds.'); 
 
        $this->add_field('jitter/max_pdv_ms', 'Jitter Max PDV',
                'integer', 100, 5);
        $this->set_field_rules('jitter/max_pdv_ms',
                'required[jitter/mode]|integer|greater_or_equal[20]|less_or_equal[200]');
        $this->set_field_help('jitter/max_pdv_ms','Maximum tolerated
                packet delay variation in milliseconds.'); 

        $this->add_field('jitter/min_delay_ms', 'Jitter Minimal Delay',
                'integer', 20, 5);
        $this->set_field_rules('jitter/min_delay_ms',
                'required[jitter/mode]|integer|greater_or_equal[20]|less_or_equal[jitter/initial_delay_ms]');
        $this->set_field_help('jitter/min_delay_ms','Minimal delay tolerated in milliseconds.');

        $conditions['adaptive'][] = array ( 'field_name' =>
                'jitter/min_delay_ms', 'status' => 'enable');

        $conditions['adaptive'][] = array ( 'field_name' =>
                'jitter/max_pdv_ms', 'status' => 'enable');
        
        $conditions['static'][] = array ( 'field_name' =>
                'jitter/min_delay_ms', 'status' => 'disable');

        $conditions['static'][] = array ( 'field_name' =>
                'jitter/max_pdv_ms', 'status' => 'disable');
        
        $this->create_group('jitter_group', array('jitter/max_pdv_ms',
                    'jitter/min_delay_ms'),
                array(
                    'default' => array( 'value' => 'adaptive', 'status' => 'enable'),
                     'condition' => $conditions
                     )
                );
        $this->conditional_control('jitter/mode', 'jitter_group');

        // Set categories
        
        $this->set_field_category('ip_address', ' Network IPv4');
        $this->set_field_category('ipv6_address', 'Network IPv6');
        $this->set_field_category('vqe/acoustic_echo', 'VQE');

        $this->set_field_category('jitter/mode', 'Jitter'); 

        return parent::configure();
    }
    public function is_enabled()
    {
        $mode = $this->get_data_value('mode', false);
        return (Sng_transcoding_config_class::HIDDEN_MODE == $mode);
    }
    /**
     * Checks if the transcoder is enabled, and if it isn't, it will check to see the codec is a PCM Codec
     * @param string  $codec
     * @return boolean
     */
    public function transcoder_media_check($codec, $media_profile )
    {
        $config = $this->configure();
        if ($this->is_enabled()) {
            $codecs = array_keys($media_profile->hw_codec_enum());
        } else {
            $codecs = array_keys($media_profile->sw_codec_enum());
        }
        return in_array($codec, $codecs);
    }

    public function udp_ports(&$start, &$stop)
    {
        $data = $this->get_data_values(false);
        $start = $data['udp_begin'];
        $stop = $data['udp_end'];
    }
    public function validate($data, &$output){
        $validate_result = parent::validate($data, $output);
        $new_data = $this->get_data_values(false);
        $port_begin = intval($new_data['udp_begin']);
        $port_end = intval($new_data['udp_end']);
        $rule_msgs = array('udp_end' => 'Last UDP port must greater than first UDP port');
        
        if ($new_data['mode'] == Sng_transcoding_config_class::SOFTWARE_MODE) {
            //SW or HW exposed mode
            if($port_begin >= $port_end ){
                $validate_result = false;
                $this->set_error('udp_end', $rule_msgs['udp_end'], $output);
            }
        }

        return $validate_result;
    }
    /**
     * @brief Save method - Override to send obj modified event
     *
     * @return 
     */
    public function save(&$output = null)
    {
        $result = parent::save($output);
        if($result){
            //send modification event out
            //to trigger firewall add/remove rules
            $path_info = $this->path_info();
            $receivers = $this->node()->software()->application()->check_object_modified($this, $path_info['module'], $path_info['type'], $this->name());
        }
        if($this->is_modified()){
            //send transcoding configuration modification event out
            $receivers = $this->node()->software()->application()->check_object_modified($this, 'transcoding','configuration', $this->name());
            //module detection for HW mode
            if($result && $this->is_enabled()){
                //check if it is configuration update from rest pass by auto detect
                if($this->node()->get_config_status('update') != 'rest'){
                    $network= $this->node()->software()->application()->find_module_by_name('network');
                    $transcoding_interfaces = $network->hardware_interface(Sng_network_interface_class::TRANSCODING);
                    //internal adapters
                    if($transcoding_interfaces){
                        $sngms_module = $this->node()->software()->application()->find_module_by_name('sngms');
                        $tmp_output = array();
                        $result = $sngms_module->api_detect_action(null, $tmp_output);
                        if(!$result){
                            $output['mode'] = implode(' ',$tmp_output['message']);
                            return !$result;
                        }
                    }    
                }
            }
        }
        return $result;
    }
    public function summary($type = 'horizontal' , $long = false){
        $mode = $this->ip_mode();
        if($mode == Sng_transcoding_config_class::SOFTWARE_MODE){
            $summary = parent::summary($type, $long, 3);
        }elseif($mode == Sng_transcoding_config_class::HIDDEN_MODE){
            $summary = parent::summary($type, $long, 2);
        }else{
            $summary = parent::summary($type, $long, 5);
            $network_module = $this->node()->software()->application()->find_module_by_name('network');
            $have_ipv6 = $network_module->have_ip_type(Safe_ip_address_class::IP_V6);
            if($have_ipv6){
                $summary['data'][$this->get_data_label('ipv6_address')] = $this->get_data_value('ipv6_address', false);
            }
        }
        return $summary;
    }
    /**
     * @brief get_port method - return the port used in this obj
     *
     * @return
     */
    public function get_port()
    {
        $port = array();
        $mode =  $this->get_data_value('mode', false);
        //only return port data in SOFTWARE_MODE
        if($mode == Sng_transcoding_config_class::SOFTWARE_MODE){
            $path_info = $this->path_info();
            $mod = $this->node()->software()->application()->find_module_by_name($path_info['module']);
            $desc = $mod->description() . ' ' . $this->name();
            $begin = $this->get_data_value('udp_begin', false);
            $end = $this->get_data_value('udp_end', false);
            $port[] = array(
                        'protocol' => 'udp', 
                        'field'=> 'udp_range', 
                        'port_begin' => $begin, 
                        'port_end' => $end, 'desc' => $desc . ' port range.');
        }
        return $port;
    }
}
/* End of file sng_transcoding_config_class.php */
