<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * Node class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_os_class');
safe_require_class('safe_software_class');
safe_require_class('safe_object_class');

abstract class Sng_transcoding_helper_class extends Safe_object_class
{
    private $_script_path = '';
    abstract protected function create_module($mac);
    /*{

        $module = new Sng_transcoding_module_class($this, $this->object_name()."/module" ,$mac[1]);
        return $module;
    }*/
    public function check_ip_valid($ip_address, $ip_list)
    {
        $found_ip = false;
        foreach ($ip_list as $ip) {
            if ($ip == $ip_address) $found_ip = true;
        }
        if ($found_ip) return false;
        else return true;
    }
    private function get_script_path()
    {
        if($this->_script_path == ''){
            $this->_script_path = Safe_get_product()->local_node()->configuration_manager()->directory('sng-base') . '/bin';
        }
        return $this->_script_path;
    }
    public function upgrade_modules($node, $if_name, &$output=array())
    {
        $log_file = '/tmp/module_update.log';
        $rc = $node->os()->execute("nohup {$this->get_script_path()}/sng-update-transcoding-modules update " . $if_name . ' ' . $log_file, $output, true, true);
        return $rc;
    }
    public function check_modules_version($node, &$output=array(), &$modules_status=array())
    {
        $rc = $node->os()->execute("nohup {$this->get_script_path()}/sng-update-transcoding-modules verify", $output, true, true, $ret_val, false);
        // Prepare array output
        // Format:
        // MAC=00-0c-90-1b-4b-b0 Status=outdated
        // MAC=00-0c-90-1b-67-06 Status=ok
        foreach($output as $line){
            if(preg_match('/MAC=(.*) Status=(.*)/', $line, $match)){
                $modules_status[$match[1]] = (strtolower($match[2]) === 'ok')?true:false;
            }
        }
        return $rc;
    }
    public function version($node)
    {
        $output = array();
        $rc = $node->os()->execute("nohup {$this->get_script_path()}/sng-update-transcoding-modules version", $output, true, true, $ret_val, false);
        if($rc){
            return $output[0];
        }
        return $rc;
    }
    public function detect_modules($node, $if_name, &$arguments, &$ip_list = array())
    {
        $modules_count = 0;
        $output = array();
        $rs = $node->execute("sudo nohup {$this->get_script_path()}/sng-detect-transcoding-modules " . $if_name, $output);
        $arguments['ifname'] = $if_name;
        if($rs && $output){
            foreach ($output as $mod_def) {
                if(empty(trim($mod_def))){
                    continue;
                }
                // Process line, format =
                // 1. MAC=00-0C-90-1B-4F-97 Ver=01.04.08-B7-PR Capacity=480 EthPort=1
                $raw_data = explode(' ', $mod_def);
                $mac = explode('=', $raw_data[1]);
                $ver = explode('=', $raw_data[2]);
                $cap = explode('=', $raw_data[3]);
                $port = explode('=', $raw_data[4]);
                //Check IP Address if in use or not.
                $valid_ip = false;
                if (isset($arguments['ip_address']) && $arguments['ip_address']!='') {
                    $ip_addr = $arguments['ip_address'];
                    while (!($valid_ip = $this->check_ip_valid($ip_addr, $ip_list))) {
                        // Increment base IP
                        increment_ip($ip_addr);
                    };
                    $arguments['ip_address'] = $ip_addr;
                    // Add this ip address to used ones
                    $ip_list[] = $ip_addr;
                }
                $valid_ip = false;
                if (isset($arguments['ipv6_address']) && $arguments['ipv6_address']!='') {
                    $ipv6_addr = $arguments['ipv6_address'];
                    while (!($valid_ip = $this->check_ip_valid($ipv6_addr, $ip_list))) {
                        // Increment base IP
                        increment_ip($ipv6_addr);
                    };
                    $arguments['ipv6_address'] = $ipv6_addr;
                    // Add this ip address to used ones
                    $ip_list[] = $ipv6_addr;
                }
                //Check if base IP address in use or not
                $valid_ip = false;
                if (isset($arguments['ext_ip_address']) && $arguments['ext_ip_address'] != "") {
                    $ip_addr = $arguments['ext_ip_address'];
                    while (!($valid_ip = $this->check_ip_valid($ip_addr, $ip_list))) {
                        // Increment base IP
                        increment_ip($ip_addr);
                    };
                    $arguments['ext_ip_address'] = $ip_addr;
                    // Add this ip address to used ones
                    $ip_list[] = $ip_addr;
                }
                $valid_ip = false;
                if (isset($arguments['ext_ipv6_address']) && $arguments['ext_ipv6_address'] != "") {
                    $ipv6_addr = $arguments['ext_ipv6_address'];
                    while (!($valid_ip = $this->check_ip_valid($ipv6_addr, $ip_list))) {
                        // Increment base IP
                        increment_ip($ipv6_addr);
                    };
                    $arguments['ext_ipv6_address'] = $ip_addr;
                    // Add this ip address to used ones
                    $ip_list[] = $ipv6_addr;
                }
                if (isset($arguments['mode']) && Sng_transcoding_config_class::HIDDEN_MODE == $arguments['mode']) {
                    // Aggregate mode, update UDP range
                    $diff = $arguments['udp_end'] - $arguments['udp_begin'];
                    $arguments['udp_end'] = $arguments['udp_begin'] + $diff;
                }
                //Save Module Information
                $module = $this->create_module($mac[1]);
                $module->configure($ver[1], $cap[1], $port[1], $arguments);
                $module->save();
                $modules_count++;
                // Prepare UDP range for next module if in aggreagate mode
                if (isset($arguments['mode']) && Sng_transcoding_config_class::HIDDEN_MODE == $arguments['mode']) {
                    // Aggregate mode, update UDP range
                    $diff = $arguments['udp_end'] - $arguments['udp_begin'];
                    $arguments['udp_begin'] = $arguments['udp_end'] + 1;
                    $arguments['udp_end'] = $arguments['udp_begin'] + $diff;
                }
            }
        }
        return $modules_count;
    }
}

