<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Network Config wrapper page
 * @author Santiago Tramoyeres Cuesta <scuesta@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_update_checker_class');
safe_module_require_class('sng','vinfo');

class Sng_update_checker extends Safe_update_checker
{
    public function __construct(&$configuration=array()){
        $this->config = Safe_update_checker::extend(array('download_id_file'=>'/tmp/SNGUpdateChecker.did',
                                                          'cached_file'=>'/tmp/SNGUpdateCheckerCachedFile.json',
                                                          'threshold_time'=>1200,
                                                          'ftp_server'=>'ftp.sangoma.com',
                                                          'ftp_username'=>'anonymous',
                                                          'ftp_password'=>'',
                                                          'ftp_base_dir'=>'nsc',
                                                          'ftp_port'=>21,
                                                          'connection_timeout'=>10,
                                                          'match_file_pattern'=>'#nsc-(.+)\.update\.tgz#i',
                                                          ),$configuration);
    }
    
    /**
     * @brief Returns link from version
     * @param Safe_vinfo
     * return string
     */
    public function get_link(&$vinfo){
        if($vinfo){
            $info = $vinfo->info();
            return 'ftp://'.$this->ftp_server.'/'.$this->ftp_base_dir.'/'.$info[Sng_vinfo::MAJOR_VERSION].'.'.$info[Sng_vinfo::MINOR_VERSION].'/updates/nsc-'.$vinfo.'.update.tgz';
        }
        return null;
    }
    
    /**
     * @brief Returns Changelog link from version
     * @param Safe_vinfo
     * return string
     */
    public function get_changelog_link(&$vinfo){
        if($vinfo){
            $info = $vinfo->info();
            return 'ftp://'.$this->ftp_server.':'.$this->ftp_port.'/'.$this->ftp_base_dir.'/'.$info[Sng_vinfo::MAJOR_VERSION].'.'.$info[Sng_vinfo::MINOR_VERSION].'/Changelog';
        }
        return null;
    }
    
    /**
     * @brief Checks if string match file pattern
     * @param string
     * return boolean
     */
    public function match_update_file($string){
        return preg_match($this->match_file_pattern,$string,$match)? $match[1] : null;
    }
    

    public function exists(&$version){
        list($server,$user,$password) = $this->get_connection_credentials();
        $info = $version->info();
        $ftp = & parent::get_ftp_connection($this->ftp_server,$this->ftp_username,$this->ftp_password,$this->ftp_port,$this->connection_timeout);
        if(($files=$ftp->list_files('/'.$this->ftp_base_dir.'/'.$info[Sng_vinfo::MAJOR_VERSION].'.'.$info[Sng_vinfo::MINOR_VERSION].'/updates'))){
            foreach($files as $file){
                $file = basename($file);
                if(($match=$this->match_update_file($file))){
                    $sng_vinfo = new Sng_vinfo($match);
                    if($sng_vinfo->eq($version))
                        return true;
                }
            }
        }
        return false;
    }
    /**
     * @brief Get next main branch for a given one from ftp server
     * @param ftp Resource
     * @param string
     * return string
     */
    public function get_next_branch(&$ftp,$branch){
        $next = $branch;
        list($major,$minor) = explode('.',$branch);    
        if(($files = $ftp->list_files('/'.$this->ftp_base_dir.'/'.$major.'.'.($minor+1))) && is_array($files) && $files!=array())
            $next = $major.'.'.($minor+1);
        else if(($files = $ftp->list_files('/'.$this->ftp_base_dir.'/'.($major+1).'.0')) && is_array($files) && $files!=array())
            $next = ($major+1).'.0';
        return $next;
    }
    
    /**
     * @brief Get latest branch published on ftp server
     * @param ftp Resource
     * return string || null
     */
    public function get_latest_branch(&$ftp){
        $dirs = $ftp->list_files('/'.$this->ftp_base_dir);
        if($dirs && is_array($dirs) && $dirs!=array()){
            foreach($dirs as $i => $dir)
                if(!is_numeric($dir)){
                    unset($dirs[$i]);
                }
            rsort($dirs);
            return $dirs[0];
        }
        return null;
    }
    
    /**
     * @brief Get array connection credentials
     * return array(hostname,username,password)
     */  
    public function get_connection_credentials(){
        return array($this->ftp_server,$this->ftp_port,$this->ftp_username,$this->ftp_password,$this->connection_timeout);
    }
    
    /**
     * @brief Get latest version from ftp server
     * @param ftp Resource
     * return Sng_vinfo || null
     */
    public function get_latest_version(&$ftp){
        $dir = $this->get_latest_branch($ftp);
        return $this->get_latest_version_on_branch($ftp,$dir);
    }
    
    
    /**
     * @brief Gets latest version on a given branch
     * @param ftp Resource
     * @param string
     * return Safe_vinfo || null
     */
    public function get_latest_version_on_branch(&$ftp,$branch=null){
        if($branch){
            $files = $ftp->list_files('/'.$this->ftp_base_dir.'/'.$branch.'/updates');
            $sarr = array();
            if($files){
                foreach($files as $file)
                    if(($match = $this->match_update_file($file)))
                        $sarr[] = $match;	 
                $sarr = Safe_vinfo::sort($sarr,'Sng_vinfo');
                return $sarr[count($sarr)-1];	  
            }
        }
        return null;
    }

    /**
     * @brief Gets version file size
     * @param ftp Resource
     * @param Safe_vinfo
     * return integer
     */
    public function get_version_size(&$ftp,&$version){
        if($ftp && $version){
            $info = $version->info();
            $path = '/'.$this->ftp_base_dir.'/'.$info[Sng_vinfo::MAJOR_VERSION].'.'.$info[Sng_vinfo::MINOR_VERSION].'/updates/nsc-'.$version.'.update.tgz';
            return $ftp->size($path);
        }
        return 0;
    }
    
    /**
     * @brief Gets next version from ftp for a given Safe_vinfo
     * @param ftp Resource
     * @param Sng_vinfo
     * return Sng_vinfo
     */
    public function get_next_update_version(&$ftp,&$vInfo){
        $info = $vInfo->info();
        $curr_branch = $info[Sng_vinfo::MAJOR_VERSION].'.'.$info[Sng_vinfo::MINOR_VERSION];
        $next = $this->get_latest_version_on_branch($ftp,$curr_branch);
        if($next && $next->gth($vInfo))
            return $next;
        else if($next && ($next_branch = $this->get_next_branch($ftp,$curr_branch)) && $next_branch && $next_branch!=$curr_branch){
            return $this->get_latest_version_on_branch($ftp,$next_branch);
        }    
        return $vInfo; 
    }

    public function save_on_cache(&$info){
    }

    /**
     * @brief Overrides get_next_version from Safe_update_checker_class to use
     * cached file instead accessing ftp server everytime. If file does not
     * exists it will be created here.
     * @param safe_vinfo_class 
     * @return array
     */
    public function get_next_version(&$vInfo){        
        safe_require_class('safe_semaphore_class');
        $arEq=true;
        $isGreat = false;
        $link = null;
        $next = "";
        $size = 0;
        $status = null;
        $rate = 0;
        $sem = new Safe_semaphore_class($this->cached_file);
        if(file_exists($this->cached_file) && $sem->lock()){            
            $output = (array) json_decode(file_get_contents($this->cached_file));
            $sem->release();
            if($output){
                $output['status'] =  $this->is_downloading()? "downloading" : "";
                $output['rate'] = $output['status'] == 'downloading'? min(100,round(($this->progress()/$output['size'])*100,2)) : 0;
            }
        }
        else{
            $output = (array) parent::get_next_version($vInfo);
            file_put_contents($this->cached_file,json_encode($output));
        }
        return $output!=null? $output : array('current'=>''.$vInfo,'new_version'=>($arEq? null : "".$next),'is_greater'=>$isGreat,'link'=>$link,'size'=>$size,'status'=>$status,'rate'=>$rate);; 
    }

    /**
     * @brief Removes the cached file if exists. First ensures file has the
     * default webconfig user
     */
    public function remove_cached_file(){
        safe_require_class('safe_semaphore_class');
        $sem = new Safe_semaphore_class($this->cached_file);
        if(file_exists($this->cached_file) && $sem->lock()){
            safe_sudo_exec("chown webconfig:webconfig ".$this->cached_file);
            unlink($this->cached_file);
            $sem->release();
        }
    }
}

?>
