<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/*
 * SNG REST module class
*/
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_module_class');
safe_module_require_class('fs','module');
safe_module_require_class('fs','sngms_module');

class Sng_update_module_class extends Safe_module_class
{
    protected $_obj_base_path = '';
    private $_update_checker;
    /**
     * @brief
     *           
     * @param[in out] $fs_app
     *           
     * @return
     */
    public function __construct($app)
    {
        parent::__construct($app->node()->software(), "update");
        $this->_update_checker = null;
    }
    public function get_data_settings($check_status = true)
    {
        $cfg_manager = $this->node()->configuration_manager();

        // Let us to know if file exist on default nsc download directory
        $exist_file_func = function(&$module,$data_name,$data_key){
            $data_cfg = $module->get_data_settings();
            $directory = isset($data_cfg['package']) && isset($data_cfg['package']['pattern'])? dirname($data_cfg['package']['pattern']) : '.';
            return file_exists($directory.'/'.$data_key);
        };

        $data = array(
                'package' => array(
                        'description' => function($module,$data_name,$data_key) use (&$exist_file_func){
                            return $exist_file_func($module,$data_name,$data_key)? 'Software Package' : 'Available Downloads';
                        },
                        'filesystem' => true,
                        'singleton' => true,
                        'pattern' => $cfg_manager->directory('base') . '/rpm-update/*.tgz',
                        'global_capabilities_no_data' => array(
                                'upload'
                        ) ,
                        'max_files'=>function($module,$data_name){
                            return count($module->get_data_files($data_name));
                        },
                        'capabilities' => array(
                                array('install',function(&$module,$data_name,$data_key) use (&$exist_file_func,$check_status){
                                        $output = array();
                                        return $exist_file_func($module,$data_name,$data_key) 
                                        && !$module->is_downloading($data_key)
                                        && $module->can_install_data($info=null,$output)
                                            && !$module->is_initializing();
                                    }),
                                array('delete',function(&$module,$data_name,$data_key) use (&$exist_file_func,$check_status){
                                        $output_data = array();
                                        $is_downloading = $module->is_downloading($data_key);
                                        if($module->is_initializing())
                                            return false;
                                        if(!$is_downloading && $check_status && !$module->can_install_data($info=null,$output_data))
                                            return $exist_file_func($module,$data_name,$data_key) && $output_data['status']!=1;
                                        return !$is_downloading && $exist_file_func($module,$data_name,$data_key);
                                    }),
                                array('view',function(&$module,$data_name,$data_key){
                                    return !$module->is_initializing();
                                }),
                                array('cancel',function(&$module,$data_name,$data_key){
                                        return $module->is_downloading($data_key);
                                    }),
                                array('download',function(&$module,$data_name,$data_key) use (&$exist_file_func){
                                        return !$exist_file_func($module,$data_name,$data_key);
                                    })
                              ),
                        'methods' => array(
                                'upload' => array(
                                        'name' => 'Upload',
                                        'icon' => 'upload',
                                        'description' => array( 'Upload a package file.' ),
                                        'request' => 'POST',
                                ),
                                'description' => array(
                                        'name' => 'Get Package desription',
                                        'description' => 'Get Package desription',
                                        'request' => 'GET',
                                ),
                                'install' => array(
                                        'name' => 'Install',
                                        'description' => 'Install the update',
                                        'request' => 'POST',
                                ),
                                'status' => array(
                                        'name' => 'Get update status',
                                        'description' => 'Get update status',
                                        'request' => 'GET',
                                ),
                                'delete' => array(
                                        'name' => 'Delete',
                                        'description' => array( 'Delete a package file.' ),
                                        'request' => 'POST',
                                ),
                                'log' => array(
                                        'name' => 'Log',
                                        'description' => array( 'Get Package install log.' ),
                                        'request' => 'GET',
                                ),
                                'check'=>array(
                                               'name'=>'Chek',
                                               'description'=>array('Check for a new available version'),
                                               'request'=>'GET',
                                               ),
                                'download'=>array(
                                                  'name'=>'Download',
                                                  'description'=>array('Download a package from ftp'),
                                                  'request'=>'GET'
                                                  ),
                                'cancel'=>array(
                                                'name'=>'Cancel',
                                                'description'=>array('Cancel downloading package'),
                                                'request'=>'GET'
                                                )                                
                                           ),
                                   ),
                      );

        // To use  anonymous function let us  to decide to  show capabilities in
        // execution time, so we can dispense with code below
        
        /* $output_data = array(); */
        /* if($check_status && !$this->can_install_data($info = null, $output_data)){ */
        /*     unset($data['package']['capabilities']); */
        /*     $status = -1000; */
        /*     if(isset($output_data['status'])){ */
        /*         $status = $output_data['status']; */
        /*     } */
        /*     if($status == 1){ */
        /*         $data['package']['capabilities'] = array('view'); */
        /*     }else{ */
        /*         $data['package']['capabilities'] = array('delete','view'); */
        /*     } */
        /* } */
        return $data;
    }
    /**
     * @brief
     *         
     * @return
     */
    public function configure()
    {
        // Set the module description
        $this->set_description("Update");
        // Register objects
        /*
        $this->register_aggregate_object('package',
                array(
                        'name' => 'Package',
                        'base_path' => $this->object_name() . '/package',
                        'methods' => array(
                                'description' => array(
                                        'name' => 'Get Package desription',
                                        'description' => 'Get Package desription',
                                        'request' => 'GET',
                                ),
                                'start' => array(
                                        'name' => 'Install',
                                        'description' => 'Install the update',
                                        'request' => 'POST',
                                ),
                                'status' => array(
                                        'name' => 'Get update status',
                                        'description' => 'Get update status',
                                        'request' => 'GET',
                                ),
                                'upload' => array(
                                        'name' => 'Upload a Package',
                                        'description' => 'Upload an update',
                                        'request' => 'POST',
                                ),
                        ),
                )
        );
        */
        return parent::configure();
    }

    /**
     * @brief Gets an Sng_update_checker instance
     * @param Array || null
     * @return Sng_update_checker
     */
    protected function get_update_checker($configuration=null){
        if(!$this->_update_checker || $configuration == null){
            safe_module_require_class('sng','update_checker');
            
            if(!$configuratior){
                $the_product = Safe_get_product();
                $configuration = $this->get_ftp_config($the_product);
            }
            
            $this->_update_check = new Sng_update_checker($configuration);
        }
        return $this->_update_check;
    }


    /**
     * @brief Check if a given file is downloading
     * @param string
     * @return boolean
     */
    public function is_downloading($file){
        $cleaned_filename = preg_replace(array('#_\.#','#nsc-#','#\.update.+$#'),array('.','',''),$file);        
        $data_cfg = $this->get_data_settings(false);
        $dir = dirname($data_cfg['package']['pattern']);
        
        $sng_up_check = $this->get_update_checker();
        $version = new Sng_vinfo($cleaned_filename);

        $file_exists = file_exists($dir.'/'.$file);
        $is_downloading = $sng_up_check->is_downloading() && 
            ($dversion = new Sng_vinfo($sng_up_check->downloading_version())) && 
            $version->eq($dversion);
        return $file_exists && $is_downloading;
    }

    /**
     * @brief Overrides Safe_module_class::get_data_files to attach new version file if exists
     *
     */
    public function get_data_files($data_name){
        $data_files = parent::get_data_files($data_name);
        $output = array();

        $this->api_check_package(null,null,$output);
        $config = $this->get_data_settings();
        if(!empty($output['local_filename']))
            $data_files[$output['local_filename']] = dirname($config['package']['pattern']).'/'.$output['local_filename'];
        return $data_files;
    }

    /**
     * @brief Downloads update package file calling api method
     * @param string
     * @param string
     * @return boolean
     */
    public function download_data($data_name,$data_key=null){
        $sng_up_checker = $this->get_update_checker();
        $output = array();
        $data_key=preg_replace('#_\.#','.',$data_key);
        if(($match = $sng_up_checker->match_update_file($data_key))){
            $data_files = parent::get_data_files($data_name);
            if(is_array($data_files) && $data_files!=array())
                $this->delete_data($data_name,$data_files[0]['Name']);
            return $this->api_download_package($data_name,$match,$output);
        }
        return true;
    }


    public function cancel_download($data_name,$data_key=null){
        $sng_up_checker = $this->get_update_checker();
        $output = array();
        $data_key = preg_replace('#_\.#','.',$data_key);
        if(($match=$sng_up_checker->match_update_file($data_key)))
            return $this->api_cancel_package($data_name,$match,$output);
        return true;
    }
    /**
     * @brief
     * clear the dir
     *
     * @return
     */
    public function delete_data($data_name, $data_key = null){
        parent::delete_data($data_name, $data_key);
        $data_cfg = $this->get_data_settings();
        $data_cfg = $data_cfg[$data_name];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $output = array();
        $rc = $os->rmdir($ext['dirname'].'/tmp');
        $rc = $os->mkdir($ext['dirname']);
        return true;
    }
    public function check($data_name = null, $param = null, &$output = null){
        $data_cfg = $this->get_data_settings();
        $data_cfg = $data_cfg[$data_name];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $file_list = $os->search_files($file);
        if($file_list)
        {
            $os = $this->node()->os();
            $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh check ', $output, true, true);
            if($rc){
                return true;
            }
        }
        return false;
    }
    public function info($data_name = null, $param = null, &$output){
        // Check if specified update package can be install
        $package_data = array();
        $rs = $this->api_description_package($data_name, $param, $package_data);
        if($rs){
            //make sure all keys are lowercase and no space (replace by '_')
            $tmp = array();
            foreach($package_data as $key => $value){
                $tmp[strtolower(str_replace(' ','_',$key))] = $value;
            }
            $tmp['type'] = 'update';
            $tmp['data_name'] = $data_name;
            $tmp['data'] = $param;
            return $tmp;
        }
        $output[] = 'Fail to get package description !';
        return false;
    }
    public function log(&$output = null){
        $data_cfg = $this->get_data_settings();
        $data_cfg = $data_cfg['package'];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $file_list = $os->search_files($file);
        if($file_list)
        {
            $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh logs ', $output, true, true);
            if($rc){
                return true;
            }else{
                $output = null;
            }
        }
        return false;
    }
    public function describe_data($data_name = null, $data_key = null, &$output = null)
    {
        // if data_key  is null launch  api_description_package, if not  check if
        // file  exists,  and  if   so,  launch  api_description_package  anyway,  else  get
        // information from ftp server
        if($data_key!=null){
            safe_module_require_class('sng','update_checker');

            $data_cfg = $this->get_data_settings(false);
            $dir = dirname($data_cfg['package']['pattern']);

            $data_key_clean = preg_replace(array('#_\.#','#nsc-#','#\.update.+$#'),array('.','',''),$data_key);
            $sng_up_check = $this->get_update_checker();
            $version = new Sng_vinfo($data_key_clean);

            $file_exists = file_exists($dir.'/'.$data_key);
            $is_downloading = $this->is_downloading($data_key);
            
            $output['Version'] = sprintf('%s',$version);
            $output['Product'] = 'nsc';
               
            // If file does not exists is because data_key comes from previous api_check_package call,
            // so doble check with Sng_update_checker::exists method to ensure version really exists 
            // on server.
            if(!$file_exists && $sng_up_check->exists($version)){
                    $output['Status'] = 'Not downloaded';
                    return true;
            }else if($is_downloading){
                $output['Status'] = 'Downloading';
                return true;
            }
                
        }
        
        return $this->api_description_package($data_name, null, $output);
    }
    public function install_data($data_name = null, $data_key = null, &$output = null)
    {
        return $this->api_install_package($data_name, $data_key, $output);
    }
    public function can_install_data($info = null, &$output = null)
    {       
        $output_data = array();
        $rs = $this->api_status_package($info['data_name'], $info['data'], $output_data);
        $status = -1000;
        if(isset($output_data['status'])){
            $status = $output_data['status'];
        }
        $output = $output_data;
        # 0 - Completed success
        # 1 - Running
        # -1 - Not Initialized/Dead
        # -2 - Not started //254
        if($status == 254){
            return true;
        }
        return false;
    }
    public function is_initializing(){
        $data_cfg = $this->get_data_settings();
        $data_cfg = $data_cfg['package'];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $file_list = $os->search_files($file);
        $output=array();
        $res = false;
        if($file_list){
            $os->execute('ps aux | grep "'.preg_replace('#\.#','\.',basename($file_list[0])).'"',$output,false);
            if(count($output)>2){
                $res =true;
            }else{
                $output = array();
                $os->execute('ps aux | grep "update\.sh initialize"',$output,false);
                if(count($output)>2)
                    $res = true;
            }
        }
        return $res;
    }
 
    public function initialize_data($data_name = null, $data_key = null, &$output = null)
    {
        $data_cfg = $this->get_data_settings();
        $data_cfg = $data_cfg['package'];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $file_list = $os->search_files($file);
        if($file_list)
        {
            $os->rmdir($ext['dirname'].'/tmp');
            if ($os->mkdir($ext['dirname'].'/tmp')){
                $rc = $os->execute('tar -xzvf ' . $file_list[0].' -C '. $ext['dirname'].'/tmp/', $output, true, true);
                if($rc){
                    // run mkdir again so ownership is changed
                    $os->mkdir($ext['dirname'].'/tmp');
                    //init
                    $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh initialize ', $output, true, true);
                    if($rc){
                        return true;
                    }
                }
            }
        }
        $this->delete_data($data_name);
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_description_package($name, $data = null, &$output = null){
        $data_cfg = $this->get_data_settings(true);
        $data_cfg = $data_cfg['package'];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $file_list = $os->search_files($file);
        if($file_list)
        {
            $iter=10;
            $init = false;
            $sleep=2;
            $message = "The update file still initializing after ".($iter*$sleep)." seconds";
            while(($init = $this->is_initializing()) && --$iter>0){
                sleep($sleep);
            }
            if($iter==0){
                safe_log(SAFE_LOG_ERROR, 'ERROR - '.$message);
                if(!isset($output['Status']))
                    $output['Status']= array();
                $output['Status']['data'] = '<font color="red">'.$message.'</font>';
                $output['Status']['formatter'] = "none";
                
                return true;
            }
            $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh version ', $version, true, true);
            if($rc){
                foreach($version as $line){
                    $items = explode('=',$line);
                    $output[$items[0]] = $items[1];
                }
                $output_data = array();
                $tmp_rs = $this->api_status_package($name, $data, $output_data);
                $description = '';
                if(isset($output_data['description'])){
                    if(is_array($output_data['description'])){
                        $description = join('', $output_data['description']);
                    }else{
                        $description = $output_data['description'];
                    }
                }
                if($description){
                    if(isset($output_data['status']) && $output_data['status'] == 254){
                        $output['Status'] = $description;
                    }else{
                        $output['Status']['data'] = "<font color=red>{$description}</font>";
                        $output['Status']['formatter'] = "none";
                    }
                }
                return true;
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_install_package($name, $data = null, &$output = null){
        $info = $this->info($name, $data, $output);
        $the_product = Safe_get_product();
        $is_api_request = isset($_SERVER['HTTP_X_API_KEY']);       
        if($info && $the_product->can_install_data($info, $output)){
            // Remove update cached file
            $sng_up_check = $this->get_update_checker();

            $data_cfg = $this->get_data_settings();
            $data_cfg = $data_cfg['package'];
            $file = $data_cfg['pattern'];
            $ext  = pathinfo($file);
            $os = $this->node()->os();
            $file_list = $os->search_files($file);
            if($file_list)
            {
                $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh start ', $output, true, true);
                if($rc){
                    sleep(1);
                    do {
                        $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh status ', $output, true, true, $status, false);
                        sleep(0.5);
                    } while(1 == $status);
                    if($status==0 && $is_api_request){
                        $application = $this->node()->software()->application();
                        $fs_module = $application->find_module_by_name('sngms');
                       if($this->fw_needs($fs_module,Fs_sngms_module_class::UPDATE)){
                            if(!$this->check_transcoding_interface_up($application,$output) ||
                                 // update media firmware
                               !$fs_module->api_fwupdate_action($data=array(),$output) ||
                               // detect modules
                               !$fs_module->api_detect_action($data=array(),$output) ||
                               // if everything went fine, apply configuration
                               !$application->api_apply_configuration(null,$params=array('call_from'=>'view_controller'),$output)){
                                if(isset($output['message'])){
                                    safe_log(SAFE_LOG_ERROR,'ERROR - '.$output['message']);
                                }
                                else{
                                    $error = "Not able to update media firmware";
                                    safe_log(SAFE_LOG_ERROR,'ERROR - '.$error);
                                    $output['message'] = $error;
                                }
                                $status=1;
                            }
                       }else if(!$application->api_apply_configuration(null,$params=array('call_from'=>'view_controller'),$output)){
                           if(isset($output['message'])){
                               safe_log(SAFE_LOG_ERROR,'ERROR - '.$output['message']);
                           }
                           else{
                               $error = "Not able to apply configuration";
                               safe_log(SAFE_LOG_ERROR,'ERROR - '.$error);
                               $output['message'] = $error;
                           }
                           $status=1;                           
                       }
                    }
                    if(0 != $status){
                        $log_output = $output;
                        $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh logs ', $log_output, true, true, $status, false);
                        safe_log(SAFE_LOG_ERROR, 'ERROR - Install Package returns error <'.$status.'>');
                    }
                    //clear modified obj and check error cache
                    $this->clear_list_cache();
                    if($status==0){
                        $sng_up_check->remove_cached_file();
                    }
                    return ($status==0);
                }
            }
        }
        return false;
    }

    private function check_transcoding_interface_up(&$application,$output){
        $network_module = $application->find_module_by_name('network');
        $interfaces_transcoding = $network_module->hardware_interface(Sng_network_interface_class::TRANSCODING);
        $rc=true;
        foreach($interfaces_transcoding as $interface_name => $interface_obj){
            if(!$interface_obj->link_status() && $rc){
                $rc = $network_module->api_up_interface($interface_obj->name(),null,$output);
            }
        }
        return $rc;
    }
        
    private function fw_needs(&$module,$what){
        $list = array();
        $module->checklist($list);
        foreach($list as $item)
            if($item['claim_to']==$what)
                return true;

        return false;
    }
    
    public function api_start_package($name, $data = null, &$output = null){
        $data_cfg = $this->get_data_settings();
        $data_cfg = $data_cfg['package'];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $file_list = $os->search_files($file);
        if($file_list)
        {
            $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh start ', $output, true, true);
            if($rc){
                return true;
            }
        }
        return false;
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_status_package($name = null, $data = null, &$output = null){
        $data_cfg = $this->get_data_settings(false);
        $data_cfg = $data_cfg['package'];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $file_list = $os->search_files($file);
        if($file_list)
        {
            $status = null;
            $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh status ', $status_text, true, true, $status, false);
            $output['description'] = $status_text;
            $output['status'] = $status;
            $logs = array();
            $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh logs ', $logs, true, true, $_status, false);
            $output['log'] = $logs;
            return true;
        }else{
            return false;
        }
    }
    /**
     * @brief
     *
     * @param[in out] $name
     * @param[in out] $data
     * @param[in out] $output
     *
     * @return
     */
    public function api_upload_package($name, $data = null, &$output = null){
        $data_def = $this->get_data_settings();
        $upload_api_method  = $data_def['package']['methods']['upload']['request'];
        if($upload_api_method){
            //initialize_data called in  upload_data() 
            return $this->upload_data('package', $name , $output , $upload_api_method);
        }else{
            $output['message'] = 'Upload request method error.';
            return false;
        }
    }
    public function api_delete_package($name=null, $data = null, &$output = null){
        return $this->delete_data('package', $name);
    }
    private function log_format($str){
        return '<pre>' .$str . '</pre>';
    }
    public function api_log_package($name = null, $data = null, &$output = null){
        $rs = $this->log($output);
        if(!$output){
            $output[] = '';
        }
        $output = array_map(array($this, 'log_format'), $output);
        return $rs;
    }

    private function get_ftp_config(&$the_product){
        // Preparing configuration fields
        $the_app = $the_product->local_node()->software()->application();
        $safe_app_settings = $the_app->find_module_by_name('application');
        $preferences = $safe_app_settings->api_retrieve_preferences();
        $configuration=array();
        foreach(array('ftp_server','ftp_username','ftp_password','ftp_base_dir','ftp_port','connection_timeout') as $field_name){
            $configuration[$field_name] = $preferences->get_data_value($field_name);
            if(!$configuration[$field_name])
                unset($configuration[$field_name]);
        }
        // If empty, get default from configuration manager
        if($configuration==array()){
            $conf_manager = $the_product->local_node()->configuration_manager();
            $up_config_file = $conf_manager->get_update_config('package');
            $configuration = parse_ini_file($up_config_file);
            // If error configuration equal to empty array
            $configuration = $configuration? $configuration : array();
        }
        return $configuration;
    }

    public function api_check_package($name=null, $data = null, &$output = null){        
        $sng_up_check = $this->get_update_checker();
        $the_product = Safe_get_product();
        $version = is_string($data)? new Sng_vinfo($data) : new Sng_vinfo($the_product->local_node()->software()->version());
        $output = $sng_up_check->get_next_version($version);
        if($output)
            $output['local_filename'] = self::ci_filename_format(basename($output['link'])); 	
        return true;
    }
    
    
    
    /**
     * @brief Returns formatted CodeIgniter filename
     * @param string || null
     * @return string || null
     */
    public static function ci_filename_format($filename=null){
        return $filename? preg_replace('#_\.#','.',preg_replace('#\.#','_.',strtolower($filename)),1) : null;
    }

    public function api_download_package($name=null,$data=null,&$output){
        $this->api_check_package($name,$data,$output);
        if(isset($output['link']) && $output['link']){
            $sng_up_check = $this->get_update_checker();
            // Checking if new version update file is already downloaded
            $basename = self::ci_filename_format(basename($output['link']));
            $data_cfg = $this->get_data_settings();
            $data_cfg = $data_cfg['package'];
            $pathinfo = pathinfo($data_cfg['pattern']);
            $download_path = $pathinfo['dirname'].'/'.$basename;	
            $link = $output['link'];
            unset($output['link']);
            $file_exists = file_exists($download_path);
            $vInfo = !empty($output['new_version'])? new Sng_vinfo($output['new_version']) : new Sng_vinfo($output['current']);
            // If file does not exists so we can start to download package file
            if(!$file_exists)
                $this->delete_data($name);
            if(!$file_exists && $sng_up_check->download($this->node()->os(),$vInfo,$download_path)){
                if($this->initialize_data('package',$basename,$output_msg,$name))
                    return true;
                // If exists and it is not downloading (thinking in different users or javascript request) 
            }else if($file_exists && !$sng_up_check->is_downloading())
                return true;
            else{
                unset($output['new_version']);
                unset($output['is_greater']);
                unset($output['current']);
                $output['message'] = 'Fail to initialize '.$data_cfg['methods']['upload']['description'][0];
            }
            if($output_msg)
                $output['reason'] = $output_msg;
        }
        return true;
    }

    public function api_cancel_package($name=null, $data=null, &$output){
        $sng_up_check = $this->get_update_checker();
        if($sng_up_check->is_downloading())
            $sng_up_check->cancel($this->node()->os());
        return true;
    }
    
    public function get_data_file_content_by_key($data_name = null, $key= null, $line_count = null, $max_size = null)
    {
        $output = array();
        $data_cfg = $this->get_data_settings();
        $data_cfg = $data_cfg['package'];
        $file = $data_cfg['pattern'];
        $ext  = pathinfo($file);
        $os = $this->node()->os();
        $file_list = $os->search_files($file);
        $basenames = array_map(function($file){
                return basename($file);
            },$file_list);        
        if($file_list && in_array($key,$basenames) && !$this->is_downloading($key))
        {
            $os = $this->node()->os();
            if($this->can_install_data()){
                $rc = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh check ', $check, true, true);
                $rc2 = $os->execute($ext['dirname'].'/tmp/update/scripts/update.sh changelog ', $changelog, true, true);
                if($rc && $rc2){
                    if($check) {
                        $output[] = 'Packages to be updated:';
                        $output[] = '----------------------';
                        $output = array_merge($output, $check);
                        $output[] = '';
                        $output[] = '';
                    }
                    if($changelog) {
                        $output[] = 'Changelog:';
                        $output[] = '---------';
                        $output = array_merge($output, $changelog);
                    }
                }
            }
        }else{
            $sng_up_check = $this->get_update_checker();
            $the_product = Safe_get_product();
            $version = new Sng_vinfo($the_product->local_node()->software()->version());

            $output[] = file_get_contents($sng_up_check->get_changelog_link($version));
        }
        $install_log = array();
        $rs3 = $this->log($install_log);
        if($rs3){
            if($install_log) {
                $output[] = 'Install log:';
                $output[] = '---------';
                $output = array_merge($output, $install_log);
            }
        }
        return implode(PHP_EOL,$output);
    }
}
/* End of file sng_update_module_class.php */
