<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Network Config wrapper page
 * @author Santiago Tramoyeres Cuesta <scuesta@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_class('safe_vinfo_class');

class Sng_vinfo extends Safe_vinfo{
  
    const MAJOR_VERSION = 0;
    const MINOR_VERSION = 1;
    const PATCH_VERSION = 2;
    const BUILD_VERSION = 3;
    const RELEASE_VERSION = 4;
    
    public function __construct($info=null){
        if($info)
            $this->info = $this->parse($info);
    }
    
    /**
     * @brief Casting function from version to int. 
     * return int
     */
    public function to_int(){
        $int = '';
        for($i=0;$i<self::RELEASE_VERSION;$i++)
            $int.=str_pad($this->info[$i],$i+1,'0',STR_PAD_LEFT);
        return intval($int);
    }
    
    /**
     * @brief Check if current release is beta version
     * return boolean
     */ 
    public function is_beta(){
        return $this->info[self::RELEASE_VERSION]=='beta';
    }
    
    /**
     * @brief Check if current release is stable
     * return boolean
     */ 
    public function is_stable(){
        return $this->info[self::RELEASE_VERSION]=='ga';
    }
    
    /**
     * @brief Check if current release is development
     * return boolean
     */
    public function is_dev(){
        return $this->info[self::RELEASE_VERSION]=='dev';
    }
    
    /**
     * @brief Parses a version information
     * @param stdClass || Safe_version || array || Safe_vInfo
     * return array
     */
    public function parse($info){
        if(is_string($info))
            $this->info = $this->_parse_string($info);
        else if(is_array($info))
            $this->info = $this->_parse_array($info);
        else if(is_object($info) && get_class($info)=='stdClass')
            $this->info = $this->_parse_array((array) $info);
        else if(is_object($info) && get_class($info)=='Safe_version')
            $this->info = $this->_parse_array($info->info());
        else if(is_object($info) && get_class($info)=='Safe_vInfo')
            $this->info = $info->info();
        else
            $this->info = null;
        return $this->info;
    }
    
    /**
     * @brief Parses a string version definition
     * @param string
     * return array() || null
     */
    protected function _parse_string($info){
        if(preg_match('#([0-9]+)\.([0-9]+)\.([0-9]+)-([0-9]+)-*([A-Za-z]+)*$#',$info,$match)){
            $arr = array();
            $arr[self::MAJOR_VERSION] = intval($match[1]);
            $arr[self::MINOR_VERSION] = intval($match[2]);
            $arr[self::PATCH_VERSION] = intval($match[3]);
            $arr[self::BUILD_VERSION] = intval($match[4]);
            $arr[self::RELEASE_VERSION] = isset($match[5])? strtolower($match[5]) : null;
            return $arr;
        }
        return null;
    }
    
    /**
     * @brief Parses an array
     * @param array
     * return array() || null
     */
    protected function _parse_array($info){
        $keys = array_keys($info);
        $length = count($keys);
        $arr = array();
        if($length>self::RELEASE_VERSION){
            if($length && is_numeric($keys[0])){
                $arr[self::MAJOR_VERSION] = is_numeric($info[self::MAJOR_VERSION])? intval($info[self::MAJOR_VERSION]) : null;
                $arr[self::MINOR_VERSION] = is_numeric($info[self::MINOR_VERSION])? intval($info[self::MINOR_VERSION]) : null;
                $arr[self::PATCH_VERSION] = is_numeric($info[self::PATCH_VERSION])? intval($info[self::PATCH_VERSION]) : null;
                $arr[self::BUILD_VERSION] = is_numeric($info[self::BUILD_VERSION])? intval($info[self::BUILD_VERSION]) : null;
                $arr[self::RELEASE_VERSION] = is_string($info[self::RELEASE_VERSION])? strtolower($info[self::RELEASE_VERSION]) : null;
            }else if($length && is_string($keys[0])){
                $arr[self::MAJOR_VERSION] = isset($info['major_version']) && is_numeric($info['major_version'])? intval($info['major_version']) : null;
                $arr[self::MINOR_VERSION] = isset($info['minor_version']) && is_numeric($info['minor_version'])? intval($info['minor_version']) : null;
                $arr[self::PATCH_VERSION] = isset($info['patch_version']) && is_numeric($info['patch_version'])? intval($info['patch_version']) : null;
                $arr[self::BUILD_VERSION] = isset($info['build_version']) && is_numeric($info['build_version'])? intval($info['build_version']) : null;
                $arr[self::RELEASE_VERSION] = isset($info['release_version']) && is_string($info['release_version'])? strtolower($info['release_version']) : null;
            }
            foreach($arr as $p)
                if($p===null)
                    return null;
            return $arr;
        }
        return null;
    }
    
    public function __toString(){
        $info = $this->info();
        $release = $info[self::RELEASE_VERSION]!=null? strtoupper($info[self::RELEASE_VERSION]) : 'Unknow';
        $release = $this->is_beta()? ucwords($info[self::RELEASE_VERSION]) : ($this->is_dev()? ucwords($info[self::RELEASE_VERSION]) : $release);
        return  !$release? $info[self::MAJOR_VERSION].'.'.$info[self::MINOR_VERSION].'.'.$info[self::PATCH_VERSION] : $info[self::MAJOR_VERSION].'.'.$info[self::MINOR_VERSION].'.'.$info[self::PATCH_VERSION].'-'.$info[self::BUILD_VERSION].'-'.$release;
    }
    
}


?>