<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Leonardo Lang <lang@sangoma.com>
 */
/*
 * Dashboard
*/

if(!isset($cwmp_container_width)){
    $cwmp_container_width = 'half';
}

echo safe_webtabletitleopen("CWMP / TR-069 status", '100%', 'sng_cwmp_stats');
?>
  <tbody id="sng_cwmp_stats_table">

<?php
    function printcol($data, $row_class, $initial=false, $bold=true, $extra='') {
        echo ($initial ? "<tr>" : "") . "<td ".$extra." style=\"text-wrap:none; white-space:nowrap;".
            ($bold ? " font-weight: bold; color: black;" : "")."\" class=\"".$row_class."\">".$data."</td>";
    }

    function centermsg($data, $size) {
        return '<div style="line-height: '.$size.'em; vertical-align: middle;">'.$data.'</div>';
    }

    function trimall($data) {
        return trim($data, "\t\n\r\0 \"");
    }

    $nextrow = 'odd';

    $none_status = array('IDLE', 'NOT CONNECTED');
    $fail_status = array('FAILURE', 'UNKNOWN');

    $txtdata = NULL;
    $retcode = 0;
    $script_path = Safe_get_product()->local_node()->configuration_manager()->directory('sng-base') . '/bin';
    exec("{$script_path}/sng-cwmp-client --tr069 --shell --nofail GetSessionInfo",
         $txtdata, $retcode);

    if ($retcode === 0) {
        foreach ($txtdata as $line) {
            $got = explode("=", $line);
            $name = trimall(str_replace('Session_', '', $got[0]));
            $info = trimall($got[1]);
            $vcls = 'safe_status_table_value';

            if ($name == 'Status') {
                $name = 'Current connection status';

                if (in_array($info, $fail_status)) {
                    $vcls = 'safe_status_table_value safe_status_error';
                } else if (in_array($info, $none_status)) {
                    $vcls = 'safe_status_table_value safe_status_stopped';
                } else {
                    $vcls = 'safe_status_table_value safe_status_running';
                };
            } else if ($name == 'LastSuccess') {
                $name = 'Last successful connection';
                if ($info == '0') {
                    $info = 'No successful past connection';
                    $vcls = 'safe_status_table_value safe_status_stopped';
                } else {
                    $info = strftime('%F %X', intval($info));
                };
            };

            printcol($name, $nextrow, true, false);
            printcol($info, $nextrow.' '.$vcls);
            $nextrow = $nextrow == 'odd' ? 'even' : 'odd';
        };

        $request_parameters = array("URL","PeriodicInformEnable","PeriodicInformInterval","ConnectionRequestURL");

        $txtdata1 = array();
        $retcode1 = 0;
        exec("{$script_path}/sng-cwmp-client --nofail --shell Get " .
             "'[(\"Device.ManagementServer\",[\"".join("\",\"",$request_parameters)."\"])]'",
             $txtdata1, $retcode1);

        $txtdata2 = array();
        $retcode2 = 0;
        exec("{$script_path}/sng-cwmp-client --nofail --shell GetConfig '[\"acs_origin\"]'",
             $txtdata2, $retcode2);

        sort($txtdata1);

        $txtdata = array_merge($txtdata1, $txtdata2);

        if ($retcode1 === 0 && $retcode2 === 0) {
            foreach ($txtdata as $line) {
                $got = explode("=", $line);
                $name = trimall(str_replace(array('Device_ManagementServer_', 'CFG_'), '', $got[0]));
                $info = trimall($got[1]);
                $bold = true;

                if ($name == 'URL') {
                    $name = 'ACS URL';
                } else if ($name == 'PeriodicInformEnable') {
                    $name = 'Periodic INFORM';
                    $info = $info == '1' ? 'Enabled' : 'Disabled';
                } else if ($name == 'PeriodicInformInterval') {
                    $name = 'INFORM interval';
                    $info = $info . ' seconds';
                } else if ($name == 'ConnectionRequestURL') {
                    $name = 'Local Connection Request URL';
                } else if ($name == 'acs_origin') {
                    $name = 'ACS parameters origin';
                }

                if ($info == '') {
                    $info = '(empty)';
                    $bold = false;
                }

                printcol($name, $nextrow, true, false);
                printcol($info, $nextrow.' safe_status_table_value', false, $bold);
                $nextrow = $nextrow == 'odd' ? 'even' : 'odd';
            }
        } else {
            printcol(centermsg('Unable to query ACS parameters (code='.$retcode1.'/'.$retcode2.')', '8'), $nextrow, true, false, 'colspan="2"');
        };
    } else {
        printcol(centermsg('(service not running)', '10'), $nextrow, true, false, 'colspan="2"');
    }
?>
</tbody>
<?php echo safe_webtableclose() ?>
</div>
