<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/* 
 * Dashboard
 *
 * Parameters:
 *     $system     = System data series 
 *                    ['cpu'] => CPU data series (pie)
 *                    ['memory'] => Memory data series (pie)
 *                    ['events'] => Events/hours data series (bar with 
 *                    info/warning/critical)
 *     $services   = Array of services data series
 *                   For each services
 *                     ['cpu'] => CPU data series (vertical bar)
 *                     ['memory'] => Memory data series (vertical bar)
 *                     ['events'] => Count of events for this service (just 
 *                     number)
 *     $curren_date = Current datetime in string format
 *     $hour_range = Hour range
*/
?>
<script type="text/javascript">

// A custom label formatter used by several of the plots
function plotPieLabelFormatter(label, series) {
    return "<div style='font-size:8pt; text-align:center; padding:2px; color:white;'>" + label + "<br></br>" + Math.round(series.percent) + "%</div>";
}

function renderSystem(data_system){
    try{
    var placeholder = $("#plot_pie_system_cpu");
    $.plot(placeholder, data_system.series.cpu, {
      series: {
        pie: { 
          show: true,
            radius: 1,
            label: {
              show: true,
              radius: 2/3,
              formatter: plotPieLabelFormatter,
              threshold: 0.1
            }
        }
      },
      legend: {
        show: true,
        container: $("#system_cpu_legend"),
        noColumns:1
      }
    });
    var placeholder = $("#plot_pie_system_memory");
    $.plot(placeholder, data_system.series.memory, {
      series: {
        pie: { 
          show: true,
            radius: 1,
            label: {
              show: true,
              radius: 2/3,
              formatter: plotPieLabelFormatter,
              threshold: 0.1
            }
        }
      },
      legend: {
        show: true,
        container: $("#system_memory_legend"),
        noColumns:1
      }
    });
    }catch(exp){
        console.log(exp);
    }
}
function fetchDataSystem(){
    function onDataReceived(data){
        renderSystem(data);
    }
    $.ajax({
        url:"/SAFe/rest/api/status/system/os",
            type: "GET",
            dataType:"json",
            success:onDataReceived
    });
    setTimeout(fetchDataSystem, 10 * 1000);
}
$(function() {
    //var data_system = <?php echo $system;?>;
    //renderSystem(data_system);
    //setTimeout(fetchDataSystem, 5 * 1000);

});

</script>
<div class='safe_dashboard_container quarter'>
<?php echo safe_webtabletitleopen("CPU", '100%', '') ?>
  <tr>
    <td>
      <div id="plot_pie_system_cpu" class="plot pie">
        <i class="fa fa-spinner fa-spin fa-4x" style="margin-top: 30px"></i>
      </div>
    </td>
    <td>
      <div id="system_cpu_legend" class="plot_legend"></div>
    </td>
  </tr>
<?php echo safe_webtableclose() ?>
</div>
<div class='safe_dashboard_container quarter'>
<?php echo safe_webtabletitleopen("Memory", '100%', '') ?>
  <tr>
    <td>
      <div id="plot_pie_system_memory" class="plot pie">
        <i class="fa fa-spinner fa-spin fa-4x" style="margin-top: 30px"></i>
      </div>
    </td>
    <td>
      <div id="system_memory_legend" class="plot_legend"></div>
    </td>
  </tr>
<?php echo safe_webtableclose() ?>
</div>
<div class='safe_dashboard_container quarter'>
<?php echo safe_webtabletitleopen("Storage", '100%', '') ?>
  <tr>
    <td>
      <div id="storage_fs_details">
        <table class='safe_stats_table'>
            <tr><td align='center'>
            <i class="fa fa-spinner fa-spin fa-4x" style="margin-top: 30px"></i>
            </td></tr>
        </table>
      </div>
  </tr>
<?php echo safe_webtableclose() ?>
</div>
<div class='safe_dashboard_container quarter'>
<?php echo safe_webtabletitleopen("Network", '100%', '') ?>
  <tr>
    <td>
      <div id="network_status_details">
        <table class='safe_stats_table'>
            <tr><td align='center'>
            <i class="fa fa-spinner fa-spin fa-4x" style="margin-top: 30px"></i>
            </td></tr>
        </table>
      </div>
    </td>
  </tr>
<?php echo safe_webtableclose() ?>
</div>
