<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Auditor Status  page
 * @author Kent <khong@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
require_once ('application/controllers/Safe_CI_Controller.php');


class Sng_auditor_status  extends Safe_CI_Controller
{
    private $_monitor = null;
    public function __construct()
    {
        parent::__construct();
        $this->_monitor = $this->_the_app->find_module_by_name('monitor');
    }
    public function index()
    {
        $this->events();
    }
    public function level_events($level=null)
    {
        $this->events(null, $level);
    }
    public function module_events($module=null)
    {
        $this->events($module);
    }
    public function events($module=null, $level=null)
    {
        // Set the basic data
        $data['page_summary'] = 'This page displays Audit Events Report.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-logs.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-logs.png';

        // global messages are handled here
        $this->_the_product->local_node()->handle_global_messages($data);

        //$the_product = Safe_get_product();
        $data['page_title'] = $this->_the_product->name() . ' - Audit Events';
        $this->load->view("safe_header", $data);
        $module_obj = $this->_the_app->find_module_by_name($module);
        if($module_obj){
            $event_data['table_title'] = $module_obj->description();
        }else{
            $module = null;
            $event_data['table_title'] = 'All Services';
        }
        if(!in_array($level, $this->_monitor->severity_level_enum())){
            $level = null;
        } 
        $event_data['table_title'] .= ' - Events';
        $event_data['module'] = $module;
        $event_data['level'] = $level;
        // Build up module list
        $event_data['filter']['module']['label'] = 'Module';
        foreach($this->_the_app->node()->modules() as $mod){
            $event_data['filter']['module']['enum'][$mod->description()] = $mod->name();
        }
        $event_data['filter']['level']['label'] = 'Severity';
        $event_data['filter']['level']['enum'] = $this->_monitor->severity_level_enum();
        
        // Get events
        $event_data['events'] = $this->_monitor->events_list($module, $level);
        //safe_var_dump($events);
        // Display eventlist
        $module_actions = array (  
                'action_settings' => $this->_monitor->get_action_settings(),
                'base_url' => '/SAFe/' . $this->router->fetch_class(),
                'module_name' =>   $this->_monitor->name()
        );
        $this->load->view("safe_module_actions", $module_actions);
        $this->load->view('sng_event_list', $event_data);
        $this->load->view("safe_footer", $data);
    }
    public function flush($module=null)
    {
        //flush the events
        $this->_monitor->api_flush_action();
        $data['redirect_url'] = '/SAFe/sng_auditor_status/';
        $data['result'] = true;
        echo json_encode($data);
    }
    
    public function action_confirm($module_name, $action_name)
    {
        // Check post request
        $post = $this->input->post();
        // Retrieve module to work on
        $description = 'Events';
        $module = $this->_the_app->find_module_by_name($module_name);
        $action_cfg = $module->get_action_settings();
        if(isset($action_cfg[$action_name])){
            $action_cfg = $action_cfg[$action_name];
            $data['page_title'] = $description . ' : '  . $action_cfg['name'];
            $data['message'] = 'Are you sure you want to ' . $action_cfg['name'].'?';
            $data['message_type'] = 'notice';
            $data['result'] = true;
            $data['action_url'] = base_url($this->router->fetch_class()). '/'. $action_name;
            $data['setting']['title'] = 'Processing ' . $action_cfg['name'];
            if(isset($action_cfg['timeout'])){
                $timeout = $action_cfg['timeout'];
            }else{
                $timeout = '60';
            }
            $data['setting']['sec'] = $timeout;
            $data['setting']['msg'] = 'Ooops ! '. $action_cfg['name']. ' time out...';
            $data['setting']['label'] = 'Close';
            $data['setting']['url'] = '';
            echo json_encode($data);
        }
    }
}
