<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2015  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Cluster Status page
 * @author Reza Samimi
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
require_once ('application/controllers/Safe_CI_Controller.php');
safe_require_class('product');
safe_require_class('service');
safe_require_class('table');
safe_require_class('crud_table');

class Sng_cluster_status extends Safe_CI_Controller
{
    private $_script_path = "";
    private $_checklist = array();

    public function __construct()
    {
        parent::__construct();
        // Validate Checklist
        // $this->_the_product->local_node()->checklist($this->_checklist);
        $this->_script_path = $this->_the_product->local_node()->configuration_manager()->directory('sng-base');
    }

    public function index()
    {
        $this->_panel();
    }

    private function _form_button($id, $label, $action, $extra_style = '')
    {
        return '<span id="' . $id . '" style="display: inline-block; ' . $extra_style . '">
                    <form action="' . $action . '" method="post" enctype="multipart/form-data">
                        <input value="' . $label . '" type="button" onclick="safe_handle_confirm_button(this);" class="ui-state-default ui-corner-all">
                    </form>
                </span>';
    }

    private function _panel()
    {
        // Set the basic data
        $data['page_title'] = 'Cluster Status';
        $data['page_summary'] = 'This page allows controlling application hardware and software.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';
        $data['errors'] = array();

        // global messages are handled here
        $this->_the_product->local_node()->handle_global_messages($data);

        $warnings = array();
        $errors = array();
        $unrecognized = array();

        //Load the library for the control panel
        $this->load->library('sng_cluster_status_lib');
        $sng_control_lib = new Sng_cluster_status_lib();

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM!?
        }

        $cluster_status = $cluster_monitor->api_status_monitor();

        safe_log(SAFE_LOG_ERROR, "STATUS: " . print_r($cluster_status, true));

        $is_running = ($cluster_status['status'] == 'running');

        // scan for errors on node operations
        foreach ($cluster_monitor->check_errors() as $node_name => $node_data) {
            foreach ($node_data as $res_name => $res_data) {
                $pretty_name = Sng_clustermon_service_class::resource_pretty_name($res_name);
                array_push($operations_errors, '"' . $pretty_name . '" failed to start on "' . $node_name . '": ' . $operdata['rc'][1] . '.');
            }

            if (count($operations_errors) != 0) {
                $id_cleanup = 'cleanup-' . $res_name;
                array_push($errors, array(implode('<br>', $operations_errors) . (count($operations_errors) > 1 ? '<br><br>' : ''),
                    '<div id="'.$id_cleanup.'" style="float:right;">' .
                    $this->_form_button($id_cleanup, 'Acknowledge', 'sng_control_panel/resource_confirm/' . $res_name . '/cleanup')));
            }
        }

        $rings_rows = '';
        $rings_class = 'odd';

        $rings_status = $cluster_monitor->rings();
        ksort($rings_status);

        $ring_id_map = array('0' => 'Dedicated', '1' => 'Backup');

        foreach ($rings_status as $ring_name => $ring_obj) {
            $ring_data = $cluster_monitor->api_retrieve_rings($ring_name);

            $bound_address = safe_array_get($ring_data['id'], 'none');
            $status_text = safe_array_get($ring_data['status'], 'unknown');

            list($status_class, $status_name) = strpos($status_text, 'active with no faults') !== false ?
                array('started', 'active') : array('stopped', 'faulty');

            $ring_description = safe_array_get($ring_id_map[$ring_name], 'RING ID ' . $ring_name);

            $rings_rows .= '<tr class="' . $rings_class . '">' .
                            '<td style="font-weight:bold; width:12%;height: 25px; text-align:left;">' . $ring_description . '</td>' .
                            '<td style="width:13%; text-align:center;">' . $bound_address . '</td>' .
                            '<td class="' . $status_class . '" style="width:12%;">' . strtoupper($status_name) . '</td>' .
                            '<td style="width:1%;"></td>' .
                            '<td style="text-align:left;">' . $status_text . '</td></tr>';

            $rings_class = ($rings_class == 'odd') ? 'even' : 'odd';
        }

        if (strlen($rings_rows) == 0) {
            $rings_rows =
                '<tr class="odd">
                    <td colspan=5 style="text-align: center; height: 25px;"> (no rings information available) </td>
                </tr>';
        }

        $db_class = 'stopped';
        $db_local_state = NULL;
        $db_cluster_state = NULL;
        $db_cluster_class = NULL;

        $db_status = $cluster_monitor->api_status_database();

        safe_log(SAFE_LOG_ERROR, "DB: " . print_r($db_status, true));

        if (count($db_status) != 0) {
            $db_class = 'started';
            $db_local_state = $db_status['wsrep_local_state_comment'];
            if (strpos($db_local_state, ' ') === FALSE) {
                $db_local_state = strtoupper($db_local_state);
            }
            $db_cluster_status = strtoupper($db_status['wsrep_cluster_status']);
            $tmpsize = $db_status['wsrep_cluster_size'];
            $db_cluster_size = $tmpsize == 0 ? '(empty)' : ($tmpsize == 1 ? $tmpsize . ' node' : $tmpsize . ' nodes');
        };

        $options_class = 'odd';
        $options_rows = '';

        foreach ($cluster_monitor->options() as $otype_name => $otype_obj) {
            $otype_data = $cluster_monitor->api_retrieve_options($otype_name);

            foreach ($otype_data as $opt_name => $opt_data) {
                $name = 'Option "' . $otype_name . '.' . $opt_name . '"';
                $value = $opt_data;

                if ($otype_name == 'resources') {
                    if ($opt_name == 'resource-stickiness') {
                        $name = 'Cost of migration for resources (resource stickiness)';
                        $value = $value == 'INFINITY' ? 'Always' : $value;
                    } else if ($opt_name == 'migration-threshold') {
                        $name = 'Maximum number of failures for triggering migration (migration threshold)';
                    } else if ($opt_name == 'failure-timeout') {
                        $name = 'Timeout for clearing past failures from migrated resource (failure timeout)';
                    }
                } else if ($otype_name == 'operations' && $opt_name == 'timeout') {
                    $name = 'Timeout for declaring operations as failed (operation timeout)';
                }

                $options_rows .=
                    '<tr class="' . $options_class . '">
                        <td style="font-weight:bold; width:500px;height: 25px; text-align: left;">' . $name . '</td>
                        <td style="text-align: left;">' . $value . '</td>
                    </tr>';

                $options_class = $options_class == 'odd' ? 'even' : 'odd';
            }
        };

        if (strlen($options_rows) == 0) {
            $options_rows =
                '<tr class="odd">
                    <td colspan=4 style="text-align: center; height: 25px;"> (no option information) </td>
                </tr>';
        }

        $data_tables = array();

        if ($is_running) {
            $data_tables[] = array(
                'table_title' => 'Cluster Network Rings',
                'table' => '
                    <thead>
                    <tr>
                        <th class="mytableheader" style="white-space:nowrap; text-align:left;">Link</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;">Local address</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;">Status</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;"></th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:left;">Description</th>
                    </tr>
                    </thead>
                    <tbody>' . $rings_rows . '</tbody>
                </table>'
            );

            $data_tables[] = array(
                'table_title' => 'Database Cluster Status',
                'table' => '
                    <thead>
                    <tr>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;">Cluster State</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;">Cluster Size</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;">Service</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;"></th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:left;">Replication State</th>
                    </tr>
                    </thead>
                    <tbody>
                    <tr class="odd">
                        <td style="text-align:center; font-weight:bold; width:13%">' . $db_cluster_status . '</td>
                        <td style="text-align:center; font-weight:bold; width:12%">' . $db_cluster_size . '</td>
                        <td class="' . $db_class . '" style="width:12%;">' . strtoupper($db_class) . '</td>
                        <td style="width:1%;"></td>
                        <td style="text-align:left; font-weight:bold; width:63%">' . $db_local_state . '</td>
                    </tr>
                    </tr>
                    </tbody>
                </table>'
            );

            $data_tables[] = array(
                'table_title' => 'Global Cluster Options',
                'table' => '
                    <tbody>' . $options_rows .
                    '</tbody>
                </table>'
            );

        } else {
            if ($cluster_status['status'] == 'error') {
                $reason = $cluster_status['reason'];
                switch ($reason) {
                    case 'timeout-send':
                        array_push($errors, 'Cluster Monitor running but not accepting requests.');
                        break;
                    case 'timeout-recv':
                        array_push($errors, 'Cluster Monitor running but not responding.');
                        break;
                    case 'decode-failure':
                        array_push($errors, 'Invalid status data received from Cluster Monitor.');
                        break;
                    default:
                        array_push($errors, 'Error processing Cluster Monitor data (' . $reason . ')');
                        break;
                }
            } else {
                if ($is_running) {
                    array_push($warnings, 'Cluster Monitor service is starting...');
                } else {
                    array_push($warnings, 'Cluster Monitor service is stopped.');
                }
            }
        }

        $sng_control_lib->check_errors($unrecognized, $errors, $data['notif_error'], $table, $data);
        $sng_control_lib->check_warnings($warnings, $data['notif_warn']);

        $this->load->view("safe_header", $data);
        $this->load->view("safe_error");
        $this->load->view("safe_notifications");

        foreach ($data_tables as $data_table) $this->load->view("safe_table", $data_table);

        $this->load->view("safe_footer", $data);
    }
}
