<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * SNG Control Panel
 *
 * @author Shaunt Libarian
 * @version
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
require_once ('application/controllers/Safe_CI_Controller.php');
safe_require_class('product');
safe_require_class('service');
safe_require_class('table');
safe_require_class('crud_table');
class Sng_control_panel extends Safe_CI_Controller
{
    const ALLOW_RESOURCES_MAINTENANCE = false;
    const ALLOW_NODES_RESOURCES_BAN = false;
    const ALLOW_NODES_MAINTENANCE = false;
    const ALLOW_NODES_RESOURCES_PREFER = false;

    private $_base_path = '';
    private $_checklist = array();

    public function __construct()
    {
        parent::__construct();
        // Validate Checklist
        $this->_the_product->local_node()->checklist($this->_checklist);
        $this->_base_path = $this->_the_product->local_node()->configuration_manager()->directory('sng-base');
    }
    public function index()
    {
        // Prevent using panel if checklist failed
        // Allow panel access if system is running
        if(!$this->_checklist || Safe_service_class::STATUS_RUNNING == $this->_the_app->status()){
            $this->_panel();
        }else{
            $this->_wizard();
        }
    }

    private function _wizard($full=true)
    {
        // Set the basic data
        $data['page_title'] = 'Control Panel';
        $data['page_summary'] = 'This page allows controlling application hardware and software.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';
        $data['errors'] = array();

        // Display reason why we're not displaying control panel
        $data['notif_error'][] = $this->_the_app->description() . ' configuration is not completed.';
        // Checklist table
        $data['table_title'] = 'Configuration Checklist';
        $table = new Safe_table_class();
        $table->apply_data_formatter = false;
        foreach($this->_checklist as $item){
            $module = $this->_the_app->find_module_by_name($item['module']);
            if($module){
                $mod_name = $module->description();
                $obj_def = $module->get_aggregate_object_definition($item['obj_type']);
                if($obj_def){
                    $obj_type = $obj_def['name'];
                    $fix_url = isset($item['url'])? $item['url'] : $obj_def['controller_url'];
                }else{
                    $obj_type = $item['obj_type'];
                }
                $obj_name = $item['obj_name'];
            }else{
                $mod_name = $item['module'];
                $obj_type = $item['obj_type'];
                $obj_name = $item['obj_name'];
            }
            $table->data[] = array(
                array('data'=>safe_icon('exclamation-triangle', 'error'), 'formatter' => 'none'),
                $mod_name,
                $obj_type,
                $obj_name,
                $item['description'],
            );
            $table->line_button[] = array(
                array(
                    'url' => $fix_url,
                    'name' => 'Configure',
                    'label' => ''
                )
            );

        }
        $data['table'] = $table->generate();

        if($full)
            $this->load->view("safe_header", $data);
        $this->load->view("safe_error");
        $this->load->view("safe_notifications");
        $this->load->view("safe_table", $data);
        if($full)
            $this->load->view("safe_footer", $data);
    }
    private function _config_status($full=true)
    {
        $data['page_title'] = 'Configuration Manager';
        $data['page_summary'] = 'This page allows managing System configuration.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-network.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-network.png';

        $configuration_module = $this->_the_app;
        // Check if configuration need to be regenerated
        $modified_list = array();
        $output = array();
        $param['call_from'] = 'view_controller';
        $result = $configuration_module->api_status_configuration(null, $param, $output);
        if($result['modified'] == true){
            if (Safe_service_class::STATUS_RUNNING == $configuration_module->status()){
                //system running
                $data['notif_error'][] = safe_form_with_submit_button('<b>Configuration is out of date and System is Running.</b></br></br>' . 'You can apply new configuration and restart System using ', '/SAFe/sng_config_manager/generate/true', // TODO(wadam) - Need to get this url from application
                        'Apply & Restart');
            }
            $modified_list = $result['reloadable'];
            if(is_array($modified_list)){
                $data['table_size'] = '100%';
                $table = new Safe_crud_table_class();
                $data['table_title'] = $table->set_title('Reloadable objects');
                // Prepare heading
                $table->set_headings(array(
                        'Module',
                        'Type',
                        'Object(s)',
                ));
                $table->set_controller_url('/SAFe/sng_config_manager');
                foreach($modified_list as $mod_name => $value){
                    foreach($value as $type_name => $obj_list){
                        $objs = '';
                        foreach($obj_list as $obj_name => $obj_status){
                            if($objs != ''){
                                $objs.='<br/>';
                            }
                            $objs .= '(' . $obj_status . ') ';
                            $objs .= $obj_name;
                        }
                        $table->add_row(array(
                                $mod_name,
                                $type_name,
                                $objs,
                        ));
                    }
                }
                $table->allow_create(false);
                $table->footer_button = array(
                        array(
                                'url' => '/SAFe/sng_config_manager/dynamic_reload',
                                'name' => 'Apply & Reload'
                        )
                );
                // Generate table HTML code
                $data['table'] = $table->generate_table();
            }

        }else{
            // Configuration is up to date
            $data['notif_warn'][] = 'Configuration is up to date.';
        }
        if($full)
            $this->load->view("safe_header", $data);
        $this->load->view("safe_notifications", $data);
        if($data['table']) {
            $this->load->view("safe_table", $data);
        }
        if($full)
            $this->load->view("safe_footer", $data);
    }

    private function _generate_actions($actions)
    {
        $id = microtime();
        $retdata = array();

        foreach ($actions as $action) {
            $selected = '';
            if (count($retdata) == 0) {
                $retdata[] = '<form id="form-' . $id . '" action="sng_control_panel/' . $action['action'] . '" method="post" enctype="multipart/form-data"
                                    style="padding-left: 0.5em; white-space: nowrap;">
                              <select onchange="javascript:document.getElementById(\'form-' . $id . '\').action = this.value;" style="width: 15em;">';
                $selected = ' selected';
            }

            $retdata[] = '<option value="sng_control_panel/' . $action['action'] . '"' . $selected . '>' . $action['label'] . '</option>';
        }

        $retdata[] = '</select>
                  <input value="Execute" type="button" onclick="safe_handle_confirm_button(this);" class="ui-state-default ui-corner-all">
                  </form>';

        return implode('', $retdata);
    }

    private function _generate_services_actions($action_map, $service, $enable_global = true)
    {
        $title_map = array('start' => 'Start', 'stop' => 'Stop', 'reset' => 'Reset');

        $id = microtime();
        $retdata = array();

        if ($enable_global) {
            $global_actions = array_diff(array_keys($action_map), array('reset', 'none'));
            sort($global_actions);

            foreach ($global_actions as $action) {
                $selected = '';
                if (count($retdata) == 0) {
                    $retdata[] = '<form id="form-' . $id . '" action="sng_control_panel/' . $action . '_confirm/' . $service . '" method="post" enctype="multipart/form-data">
                                  <select onchange="javascript:document.getElementById(\'form-' . $id . '\').action = this.value;" style="min-width: 13em; max-width: 13em;" >';
                    $selected = ' selected';
                }

                $retdata[] = '<option value="sng_control_panel/' . $action . '_confirm/' . $service . '"' . $selected . '>' . $title_map[$action] . ' on all nodes</option>';
            }
        }

        foreach ($action_map as $action => $action_data)
        {
            if ($action == 'none') continue;

            if (count($retdata) == 0) {
                $retdata[] = '<form id="form-' . $id . '" action="sng_control_panel" method="post" enctype="multipart/form-data">
                              <select onchange="javascript:document.getElementById(\'form-' . $id . '\').action = this.value;" style="min-width: 13em; max-width: 13em; text-align: center;">
                              <option value="sng_control_panel" selected> &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;(select action)</option>';
            }

            foreach ($action_data as $node_name => $node_data)
            {
                $retdata[] = '<option value="sng_control_panel/' . $action . '_confirm/' . $service . '/' . $node_name . '">' . $title_map[$action] . ' on ' . $node_name . '</option>';
            }
        }

        $disabled = '';

        if (count($retdata) == 0) {
            $retdata[] = '<form id="form-' . $id . '" action="sng_control_panel" method="post" enctype="multipart/form-data" disabled>
                          <select onchange="javascript:document.getElementById(\'form-' . $id . '\').action = this.value;" style="min-width: 13em; max-width: 13em; text-align: center;" disabled>
                              <option value="sng_control_panel" selected disabled> &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;(no actions)</option>
                          </select>';
            $disabled = ' disabled';
        } else {
            $retdata[] = '</select>';
        }

        $retdata[] =
                '<input value="Execute" type="button" onclick="safe_handle_confirm_button(this);" class="ui-state-default ui-corner-all"'.$disabled.'>
                 </form>';

        return implode('', $retdata);
    }

    private function _is_cluster_enabled()
    {
        $cluster_module = $this->_the_app->find_module_by_name('cluster');
        return $cluster_module != NULL && $cluster_module->get_cluster_mode() != 'disabled';
    }

    private function _panel()
    {
        if ($this->_is_cluster_enabled()) {
            return $this->_cluster_panel();
        } else {
            return $this->_standard_panel();
        }
    }

    private function _standard_panel()
    {
        // Set the basic data
        $data['page_title'] = 'Control Panel';
        $data['page_summary'] = 'This page allows controlling application hardware and software.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';

        $license = $this->_the_app->node()->software()->license();
        // global messages are handled here
        $this->_the_product->local_node()->handle_global_messages($data);

        $data['errors'] = array();

        //Load the library for the control panel
        $this->load->library('sng_control_panel_lib');
        $sng_control_lib = new Sng_control_panel_lib();
        //Creates the data to put inside the table
        //Can place HTML directly into the array elements
        //$this->_the_product = Safe_get_product();
        $sng_control_lib->_the_product = $this->_the_product;
        //Pulls all services required for the gateway and checks their status
        $services = $this->_the_product->local_node()->software()->services();
        $warnings = array();
        $error = array();
        $unrecognized = array();
        // Categorize services
        $service_cat = array();
        foreach ($services as $service) {
            // Check user can control this service
            //if ($service->allow_user_ctl())
            if($license->allow_service($service))
                $service_cat[$service->category() ][] = $service;
        }
        // re-organize to have following category order
        // 'Application'
        // ...
        // 'Other'
        $app_cat = $service_cat['Application'];
        unset($service_cat['Application']);
        $other_cat = $service_cat['Other'];
        unset($service_cat['Other']);
        $new_service['Application'] = $app_cat;
        // Add all other categories
        foreach ($service_cat as $k => $v) $new_service[$k] = $v;
        if (!empty($other_cat)) $new_service['Other'] = $other_cat;
        unset($service_cat);
        $service_cat = $new_service;
        // Loop around all categories to create separate tables
        foreach ($service_cat as $cat_name => $services) {
            //Declare Table classes to be used
            $table = new Safe_table_class();
            $tmp_data = array();
            //Prepare the Control Panel Table Title
            $tmp_data['table_title'] = $cat_name . ' Services';
            //Creates the headers of the table
            $table->heading = array(
                "Service",
                "Status",
                "Uptime",
                'CPU(%)',
                'Memory(%)'
            );
            $table->heading_button = '';
            foreach ($services as $service) {
                if ($service->allow_user_ctl() === 'hide') continue;
                // Retrieve status
                $stats = array();
                $mode = $service->startup();
                switch ($mode) {
                case Safe_service_class::STARTUP_AUTOMATIC:
                    $to_mode_name = "To Manual";
                    $url = 'sng_control_panel/set_mode/' . Safe_service_class::STARTUP_MANUAL;
                    break;

                case Safe_service_class::STARTUP_MANUAL:
                default:
                    $to_mode_name = "  To Auto  ";
                    $url = 'sng_control_panel/set_mode/' . Safe_service_class::STARTUP_AUTOMATIC;
                    break;
                }
                $line_button = "";
                switch ($rc = $service->status($stats)) {
                case Safe_service_class::STATUS_RUNNING:
                    $status = 'STARTED';
                    $status_class = 'started';
                    if ($service->allow_user_ctl()) {
                        $line_button = array(
                            array(
                                'url' => 'sng_control_panel/stop_confirm',
                                'name' => 'Stop',
                                'type' => 'confirm'
                            ) ,
                        );
                    }
                    break;

                case Safe_service_class::STATUS_STOPPED:
                    $status = "STOPPED";
                    $status_class = 'stopped';
                    if ($service->allow_user_ctl()) {
                        $line_button = array(
                            array(
                                'url' => 'sng_control_panel/start_confirm',
                                'name' => 'Start',
                                'label' => '',
                                'type' => 'confirm'
                            ) ,
                        );
                    }
                    break;

                case Safe_service_class::STATUS_UNRECOGNIZED:
                    //display as not installed
                    $status = $service->status_text($rc);
                    $status_class = 'error';
                    break;

                case Safe_service_class::STATUS_ERROR:
                    //some error happened on start
                    $status = 'ERROR';
                    $status_class = 'error';
                    $line_button = array(
                        array(
                            'url' => 'sng_control_panel/reset_confirm',
                            'name' => 'Reset',
                            'label' => '',
                            'type' => 'confirm'
                        )
                    );
                    $error[] = $service->description();
                    break;

                /* In any other cases, just display Reset */
                case Safe_service_class::STATUS_DEAD:
                default:
                    $status = 'UNKNOWN';
                    $status_class = 'error';
                    $line_button = array(
                        array(
                            'url' => 'sng_control_panel/reset_confirm',
                            'name' => 'Reset',
                            'label' => '',
                            'type' => 'confirm'
                        )
                    );
                    $error[] = $service->description();
                    break;
                }
                $table->line_button[] = $line_button;

                //Appends to the $table->data array. This adds the service and its status as an element
                array_push($table->data, array(
                    array(
                        'data' => $service->description() ,
                        'style' => 'font-weight:bold; width:30%;height: 25px;'
                    ) ,
                    array(
                        'data' => $status,
                        'style' => 'width:12%;',
                        'class' => $status_class
                    ) ,
                    $stats['cpu']['time'],
                    $stats['cpu']['percent'],
                    $stats['mem']['percent']
                ));
                //Creates the primary key of each line to pass when the form is parsed
                //$table->primary_key = array(1,2);
                array_push($table->primary_key, $service->name());
            }
            $table->footer_button = array(
                array(
                    'url' => base_url('sng_control_panel') ,
                    'name' => 'Refresh'
                )
            );
            if($table->data){
                $tmp_data['table'] = $table->generate();
                $data_tables[] = $tmp_data;
            }
        }
        $data['errors'] = array();

        //Calls function that checks for notification messages
        $sng_control_lib->check_errors($unrecognized, $error, $data['notif_error'], $table, $data);
        $sng_control_lib->check_warnings($warnings, $data['notif_warn']);
        $this->load->view("safe_header", $data);
        $this->load->view("safe_error");
        $this->load->view("safe_notifications");

        foreach ($data_tables as $data_table) $this->load->view("safe_table", $data_table);

        $this->load->view("safe_footer", $data);
    }

    private function _cluster_panel()
    {
        $auto_refresh = is_numeric($_POST['refresh']) ? $_POST['refresh'] : $_GET['refresh'];
        if (is_numeric($auto_refresh) && $auto_refresh > 0) {
            header('refresh: ' . $auto_refresh . '; url=sng_control_panel?refresh=' . $auto_refresh);
        };

        //Load the library for the control panel
        $this->load->library('sng_control_panel_lib');
        $sng_control_lib = new Sng_control_panel_lib();
        //Creates the data to put inside the table
        //Can place HTML directly into the array elements
        //$this->_the_product = Safe_get_product();
        $sng_control_lib->_the_product = $this->_the_product;

        // Set the basic data
        $data['page_title'] = 'Control Panel';
        $data['page_summary'] = 'This page allows controlling application hardware and software.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';

        // global messages are handled here
        $this->_the_product->local_node()->handle_global_messages($data);

        $data['errors'] = array();

        $warnings = array();
        $errors = array();
        $unrecognized = array();

        $monitor_tables = $this->_cluster_clustermon_tables($warnings, $unrecognized, $errors);
        $services_tables = $this->_cluster_services_tables($warnings, $unrecognized, $errors);

        // lets not change the api and avoid unexpected headaches
        $unused_table = null;


        //Calls function that checks for notification messages
        $sng_control_lib->check_errors($unrecognized, $errors, $data['notif_error'], $unused_table, $data);
        $sng_control_lib->check_warnings($warnings, $data['notif_warn']);

        $this->load->view("safe_header", $data);
        $this->load->view("safe_error");
        $this->load->view("safe_notifications");

        foreach ($monitor_tables as $data_table)
            $this->load->view("safe_table", $data_table);
        foreach ($services_tables as $data_table)
            $this->load->view("safe_table", $data_table);

        $this->load->view("safe_footer", $data);
    }

    private function _form_button($id, $label, $action, $extra_style = '', $has_confirm = true, $with_span = true)
    {
        return ($with_span ? '<span id="' . $id . '" style="display: inline-block; ' . $extra_style . '">' : '') .
                    '<form action="' . $action . '" method="post" enctype="multipart/form-data">
                        <input value="' . $label . '" ' . ($has_confirm ? 'type="button" onclick="safe_handle_confirm_button(this);"' : 'type="submit"') .
                            ' class="ui-state-default ui-corner-all">
                    </form>' .
                ($with_span ? '</span>' : '');
    }

    private function _relative_time($ts)
    {
        $rel_secs = time() - intval($ts);
        $rel_secs = $rel_secs >= 0 ? $rel_secs : null;

        if ($rel_secs !== null) {
            $rel_text = ($rel_secs % 60) . ' ' . ($rel_secs > 1 ? 'secs' : 'sec');
            $rel_mins = intval($rel_secs / 60);
            if ($rel_mins !== 0) {
                $rel_text = ($rel_mins % 60) . ' ' . ($rel_mins > 1 ? 'mins' : 'min') . ' ' . $rel_text;
                $rel_hour = intval($rel_mins / 60);
                if ($rel_hour !== 0) {
                    $rel_text = ($rel_hour % 24) . ' ' . ($rel_hour > 1 ? 'hours' : 'hour') . ' ' . $rel_text;
                    $rel_days = intval($rel_hour / 24);
                    if ($rel_days !== 0) {
                        $rel_text = $rel_days . ' ' . ($rel_days > 1 ? 'day' : 'days') . ' ' . $rel_text;
                    }
                }
            }
            return $rel_text . ' ago';
        }

        return NULL;
    }

    private function _cluster_clustermon_tables(&$warnings, &$unrecognized, &$errors)
    {
        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');

        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
        }

        $cluster_status = safe_array_get($cluster_monitor->api_status_monitor()['status'], 'unknown');

        $is_initialized = false;
        $has_runnable = false;

        // check if stickiness is infinite
        $is_stickiness_infinite = false;
        foreach (safe_array_get($cluster_monitor->options()['resources'], array()) as $key => $val) {
            if ($key == 'resource-stickiness' && $val == 'INFINITY') {
                $is_stickiness_infinite = true;
            }
        }

        $operations_errors = array();

        // scan for errors on node operations
        foreach ($cluster_monitor->check_errors() as $node_name => $node_data) {
            foreach ($node_data as $res_name => $res_data) {
                $pretty_name = Sng_clustermon_service_class::resource_pretty_name($res_name);
                $operations_errors[$res_name][] = '"' . $pretty_name . '" failed on "' . $node_name . '": ' . $res_data . '.';
            };
        };

        foreach ($operations_errors as $res_name => $res_errors) {
            $id_cleanup = 'cleanup-' . $res_name;
            array_push($errors, array(implode('<br>', $res_errors) . '<br>' . (count($res_errors) > 1 ? '<br>' : ''),
                $this->_form_button($id_cleanup, 'Acknowledge', 'sng_control_panel/resource_confirm/' . $res_name . '/cleanup', '', true, false)));
        };

        $master_nodes = array();

        $resources_rows = '';
        $resources_footer = '';
        $resources_class = 'odd';

        $nsc_group_attr = $cluster_monitor->api_retrieve_group(Sng_clustermon_service_class::CLUSTER_GROUP_NAME);
        $nsc_group_maint_mode = safe_array_get($nsc_group_attr['maintenance'], 'off');
        $nsc_group_target_role = safe_array_get($nsc_group_attr['target-role'], 'unknown');

        $nodes_has_started = false;
        $nodes_has_stopped = false;

        $monitor_nodes = array_keys($cluster_monitor->nodes());
        sort($monitor_nodes);

        $monitor_resources = array_keys($cluster_monitor->resources());
        sort($monitor_resources);

        $self_node_name = gethostname();
        $self_node_uuid = Sng_clustermon_service_class::machine_uuid();

        $self_clustermon_stats = array();
        $self_clustermon_status = $cluster_monitor->status($self_clustermon_stats);

        $self_node_status = NULL;
        $self_node_status_extra = '';

        if ($self_clustermon_status == Safe_service_class::STATUS_STOPPED) {
            $cman_status = $this->_the_product->local_node()->software()->service('cman')->status();
            $pcmk_status = $this->_the_product->local_node()->software()->service('pacemaker')->status();

            if ($cman_status === 0 && $pcmk_status === 0) {
                $self_node_status = 'unmonitored';
            } else {
                $self_node_status = 'stopped';
            }
        } else {
            foreach ($monitor_nodes as $node_name) {
                $node_attr = $cluster_monitor->api_retrieve_node($node_name);
                $node_data = $cluster_monitor->api_status_node($node_name);

                if ($node_attr['uuid'] !== $self_node_uuid) {
                    continue;
                }

                $self_node_status = safe_array_get($node_data['status'], null);
            }

            if ($self_node_status === null) {
                if ($cluster_status === 'unknown') {
                    $self_node_status = strtolower($cluster_monitor->status_text($self_clustermon_status));
                } else if ($cluster_status === 'not-running' && $self_clustermon_status == Safe_service_class::STATUS_RUNNING) {
                    $self_node_status = 'starting';
                } else {
                    $self_node_status = ($cluster_status == 'running') ? 'joining' : str_replace('-', ' ', $cluster_status);
                }
            } else {
                if ($cluster_status !== 'running') {
                    $self_node_status_extra = ' (' . substr($cluster_status, 0, 1) . ')';
                }
            }
        }

        $resources_errors = array();
        $nodes_status_map = array();

        foreach ($monitor_nodes as $node_name) {
            $node_attr = $cluster_monitor->api_retrieve_node($node_name);
            $node_data = $cluster_monitor->api_status_node($node_name);

            $node_status = NULL;

            if ($node_attr['uuid'] !== $self_node_uuid) {
                $node_status = safe_array_get($node_data['status'], null);

                list($rc, $stats) = $cluster_monitor->services_status($node_name, 'clustermon');
                if ($node_status === null) {
                    $node_status = ($rc !== Safe_service_class::STATUS_ERROR) ?
                        strtolower(Safe_service_class::get_status_text($rc)) : 'unreachable';
                } else {
                    if ($node_status !== 'offline') {
                        if ($rc == Safe_service_class::STATUS_ERROR) {
                            // unable to connect, but heartbeat didn't timeout yet
                            $node_status = 'unavailable';
                        } else if ($rc != Safe_service_class::STATUS_RUNNING) {
                            // clustermon stopped but cluster is still running
                            $node_status = 'unmonitored';
                        }
                    }
                }

                if ($node_status == 'running' && $self_clustermon_status === Safe_service_class::STATUS_RUNNING) {
                    if ($self_node_status === 'online') {
                        // we are online but the other node is still not
                        $node_status = 'not joined';
                    } else if (in_array($self_node_status, array('initializing', 'joining'))) {
                        // if we are initializing/joining, show other node also on same state
                        $node_status = $self_node_status;
                    }
                }
            } else {
                $node_status = $self_node_status;
                $node_status_extra = $self_node_status_extra;
            }

            foreach ($node_attr as $attr_name => $attr_data) {
                if ($attr_name == 'standby' and $attr_data == 'on') {
                    if ($node_status == 'online') {
                        $node_status = 'disabled';
                    }
                }
            }

            if (in_array($node_status, array('online', 'initializing', 'joining'))) {
                if (!$is_stickiness_infinite) {
                    $nsc_score = safe_array_list_get($node_attr, array('location', 'NSC'));
                    if ($nsc_score !== '-INFINITY') {
                        $has_runnable = true;
                    }
                } else {
                    $has_runnable = true;
                }
            }

            if (in_array($node_status, array('running', 'joining', 'online', 'disabled'))) {
                $nodes_has_started = true;
            } else {
                $nodes_has_stopped = true;
            }

            $node_status_map[$node_name] = array($node_status, $node_status_extra);

            foreach ($monitor_resources as $res_name) {
                $failed_number = safe_array_list_get($node_data, array('state-attributes', 'fail-count-' . $res_name), NULL);
                $failed_last = safe_array_list_get($node_data, array('state-attributes', 'last-failure-' . $res_name), NULL);

                if ($failed_number !== NULL && $failed_last !== NULL) {
                    $resources_errors[$res_name][$node_name] = array('number' => strval($failed_number), 'last' => $failed_last);
                }
            }
        }

        $res_has_started = false;
        $res_has_stopped = false;

        foreach ($monitor_resources as $res_name) {
            $res_attr = $cluster_monitor->api_retrieve_resource($res_name);
            $res_data = $cluster_monitor->api_status_resource($res_name);

            $target_role = safe_array_get($res_attr['target-role'], 'Started');
            $maint_mode = safe_array_get($res_attr['maintenance'], 'off');

            $res_nodes_id = safe_array_list_get($res_data, array('status', 'nodes'), array());
            $res_status = safe_array_list_get($res_data, array('status', 'name'), 'unknown');

            $res_nodes = array();
            $res_notes = array();
            $res_actions = array();


            $res_node_list = $cluster_monitor->get_cluster_nodes_id();
            foreach ($res_nodes_id as $id) {
                    array_push($res_nodes, $res_node_list[$id]['name']);
            }

            list($running_state, $running_class, $running_nodes) = array('unknown', 'error', '');

            if ($res_status == 'start') {
                if ($target_role == 'Stopped') {
                    list($running_state, $running_class, $running_nodes) = array('stopping', 'started', implode(', ', $res_nodes));
                } else {
                    list($running_state, $running_class, $running_nodes) = array('started', 'started', implode(', ', $res_nodes));
                    if ($res_name == Sng_clustermon_service_class::CLUSTER_SWITCH_NAME) {
                        $master_nodes = array_unique(array_merge($master_nodes, $res_nodes));
                    }
                }
                $res_has_started = true;
            } else {
                if ($target_role == 'Started') {
                    if ($has_runnable) {
                        if (isset($operations_errors[$res_name]) || isset($resources_errors[$res_name])) {
                            list($running_state, $running_class) = array('not running', 'error');
                        } else {
                            list($running_state, $running_class) = array('starting', 'started');
                        }
                    } else {
                        list($running_state, $running_class) = array('not runnable', 'error');
                    }
                } else {
                    list($running_state, $running_class, $running_nodes) = array('stopped', 'stopped', '');
                }
                $res_has_stopped = true;
            }

            list($maint_action, $maint_label) = ($maint_mode == 'on') ? array('/clear', 'Clear maintenance mode') : array('/set', 'Set maintenance mode');

            if ($target_role == 'Started' || $res_status == 'start') {
                $res_actions[] = array('label' => ($target_role == 'Started' && $res_status == 'start' ? 'Stop' : 'Force stop') . ' resource',
                    'action' => 'resource_confirm/' . $res_name . '/stop');
            }

            if ($target_role == 'Stopped' || $res_status == 'stop') {
                $res_actions[] = array('label' => ($target_role == 'Stopped' && $res_status == 'stop' ? 'Start' : 'Force start') . ' resource',
                    'action' => 'resource_confirm/' . $res_name . '/start');
            }

            if ($maint_mode != 'off' || Sng_control_panel::ALLOW_RESOURCES_MAINTENANCE) {
                $res_actions[] = array('label' => $maint_label, 'action' => 'maintenance_confirm/resource/' . $res_name . $maint_action);
            }

            if (isset($operations_errors[$res_name])) {
                $res_actions[] = array('label' => 'Cleanup failed operations', 'action' => 'resource_confirm/' . $res_name . '/cleanup');
            }

            $resources_rows .=
                    '<tr class="' . $resources_class . '">
                        <td style="font-weight:bold; width:25%;height: 25px;">' . Sng_clustermon_service_class::resource_pretty_name($res_name) . '</td>
                        <td class="' . $running_class . '" style="width:12%;">' . strtoupper($running_state) .
                            (($nsc_group_maint_mode === 'on' || $maint_mode === 'on') ? ' (M)' : '') . '</td>
                        <td style="width: 1%;"></td>
                        <td style="font-weight:bold;">' . $running_nodes . '</td>
                        <td><div id="buttons3" style="float:right;">' .
                            $this->_generate_actions($res_actions) .
                       '</div></td></tr>';

            $resources_class = ($resources_class == 'odd' ? 'even' : 'odd');
        }

        if ($nsc_group_attr !== NULL && count($nsc_group_attr) != 0) {
            list($maint_action, $maint_label) = $nsc_group_maint_mode === 'on' ? array('clear', 'Clear global maintenance mode') : array('set', 'Set global maintenance mode');

            if ($res_has_stopped || $nsc_group_target_role == 'Stopped') {
                $resources_footer .=
                    $this->_form_button('action-start-' . Sng_clustermon_service_class::CLUSTER_GROUP_NAME, 'Start all resources',
                        'sng_control_panel/group_confirm/' . Sng_clustermon_service_class::CLUSTER_GROUP_NAME . '/start',
                            'padding: 3px; padding-bottom: 5px;');
            }

            if ($res_has_started || $nsc_group_target_role == 'Started') {
                $resources_footer .=
                    $this->_form_button('action-stop-' . Sng_clustermon_service_class::CLUSTER_GROUP_NAME, 'Stop all resources',
                        'sng_control_panel/group_confirm/' . Sng_clustermon_service_class::CLUSTER_GROUP_NAME . '/stop',
                            'padding: 3px; padding-bottom: 5px;');
            }

            $resources_footer .=
                $this->_form_button('maintenance-' . Sng_clustermon_service_class::CLUSTER_GROUP_NAME, $maint_label,
                    'sng_control_panel/maintenance_confirm/group/' . Sng_clustermon_service_class::CLUSTER_GROUP_NAME . '/'. $maint_action,
                        'padding: 3px; padding-bottom: 5px;');
        };

        $nodes_rows = '';
        $nodes_class = 'odd';
        $nodes_footer = '';

        foreach ($monitor_nodes as $node_name) {
            $node_attr = $cluster_monitor->api_retrieve_node($node_name);
            $node_data = $cluster_monitor->api_status_node($node_name);

            list($node_status, $node_status_extra) = safe_array_get($node_status_map[$node_name], array(NULL, NULL));

            if ($node_status === NULL) {
                safe_log(SAFE_LOG_ERROR, 'unable to find status for ' . $node_name . ', skipping..');
                continue;
            }

            $node_notes = array();
            $node_actions = array();
            $status_class = safe_array_matcher($node_status,
                array('started' => array('online', 'starting', 'initializing', 'joining', 'running', 'configuring', 'terminating'),
                      'stopped' => array('offline', 'stopped')), 'error');

            foreach ($monitor_resources as $res_name) {
                $failed_data = safe_array_list_get($resources_errors, array($res_name, $node_name), array('number' => NULL, 'last' => NULL));

                if ($failed_data['number'] !== NULL && $failed_data['last'] !== NULL) {
                    $failed_rel_text = $this->_relative_time($failed_data['last']);
                    $pretty_name = Sng_clustermon_service_class::resource_pretty_name($res_name);
                    $node_notes[] = '<strong>' . $pretty_name . '</strong> failed <strong>'.$failed_data['number'].
                                    '</strong> times, last: <strong>'. ($failed_rel_text ? $failed_rel_text : '(unknown)').
                                    '</strong> ('. strftime('%F %T', $failed_data['last']) .')';
                }
            }

            $node_active = in_array($node_status, array('joining', 'online', 'disabled'));
            $node_running = $node_active || in_array($node_status, array('running'));

            if ($node_running) {
                $node_actions[] = array('label' => 'Stop cluster service', 'action' => 'stop_confirm/clustermon/' . $node_name);
            } else {
                $node_actions[] = array('label' => 'Start cluster service', 'action' => 'start_confirm/clustermon/' . $node_name);
            }

            $nsc_score = null;

            if (!$is_stickiness_infinite) {
                $nsc_score = safe_array_list_get($node_attr, array('location', 'NSC'));
                if ($nsc_score === 'INFINITY') {
                    $node_notes[] = 'Persistent migration set or migration in progress - services will attempt to run on this node.';
                } else if ($nsc_score === '-INFINITY') {
                    $node_notes[] = 'Resources banned from this node - <strong>NOT elegible for failover</strong>';
                }
            }

            $maint_mode = 'off';

            if ($node_active) {
                $maint_mode = safe_array_get($node_attr['maintenance'], 'off');

                list($maint_action, $maint_label) = ($maint_mode == 'on') ? array('/clear', 'Clear maintenance mode') : array('/set', 'Set maintenance mode');
                list($standby_action, $standby_label) = ($node_status == 'disabled') ? array('/clear', 'Enable this node') : array('/set', 'Disable this node');

                $node_actions[] = array('label' => $standby_label, 'action' => 'standby_confirm/' . $node_name . $standby_action);

                if ($maint_mode != 'off' || Sng_control_panel::ALLOW_NODES_MAINTENANCE) {
                    $node_actions[] = array('label' => $maint_label, 'action' => 'maintenance_confirm/node/' . $node_name . $maint_action);
                }
            }

            if ($nodes_has_started) {
                if (!$is_stickiness_infinite) {
                    $node_master = in_array($node_name, $master_nodes);

                    $banning_action = null;
                    $migrate_action = null;

                    if (!in_array($nsc_score, array('INFINITY', '-INFINITY', null))) {
                        safe_log(SAFE_LOG_ERROR, 'node "' . $node_name . '" has unsupported location constraint "'.$nsc_score.'" - ignoring');
                    } else {
                        if ($nsc_score == 'INFINITY') {
                            $migrate_action = false;
                            $banning_action = ($node_master && Sng_control_panel::ALLOW_NODES_RESOURCES_BAN) ? true : null;
                        } else {
                            if ($node_master) {
                                if ($nsc_score == null) {
                                    $banning_action = (Sng_control_panel::ALLOW_NODES_RESOURCES_BAN ? true : null);
                                } else {
                                    // NOTE: -INFINITY + node_master = impossible situation, nothing sane to do
                                    safe_log(SAFE_LOG_ERROR, 'inconsistent state: node "' . $node_name . '" is a master node but location constraint is -INFINITY');
                                }
                            } else {
                                if ($nsc_score == null) {
                                    $migrate_action = ($node_status != 'disabled') ? true : null;
                                    $banning_action = (Sng_control_panel::ALLOW_NODES_RESOURCES_BAN ? true : null);
                                } else {
                                    $banning_action = false;
                                }
                            }
                        }
                    }

                    if ($migrate_action !== null) {
                        if ($migrate_action && $node_status != 'offline') {
                            $node_actions[] = array('label' => 'Migrate to this node', 'action' => 'migrate_confirm/' . $node_name);
                        }

                        if (!$migrate_action || Sng_control_panel::ALLOW_NODES_RESOURCES_PREFER) {
                            $node_actions[] = array('label' => ($migrate_action ? 'Always prefer this node' : 'Clear migration for this node'),
                                'action' => 'migrate_to_confirm/' . $node_name . ($migrate_action ? '/set' : '/clear'));
                        }
                    }

                    if ($banning_action !== null) {
                        $node_actions[] = array('label' => ($banning_action ? 'Ban resources on this node' : 'Clear resources ban for this node'),
                            'action' => 'migrate_away_confirm/' . $node_name . ($banning_action ? '/set' : '/clear'));
                    }
                }
            }

            $node_status_cluster = array();

            if ($maint_mode == 'on') {
                $node_status_cluster[] = 'M';
            }

            if ($nsc_score == 'INFINITY') {
                $node_status_cluster[] = 'P';
            } else if ($nsc_score == '-INFINITY') {
                $node_status_cluster[] = 'B';
            }

            $node_name_extra = '';

            if ($self_node_uuid == $node_attr['uuid']) {
                $node_name_extra .= ' <small>(localhost)</small>';
            }

            if (count($master_nodes) !== 0 && in_array($node_name, $master_nodes)) {
                $node_name_extra .= ' <strong><small>[MASTER]</small></strong>';
            }

            $nodes_rows .= '<tr class="' . $nodes_class . '">
                            <td style="font-weight:bold; width:25%;height: 25px;">' . $node_name . $node_name_extra . '</td>
                            <td class="' . $status_class . '" style="width:12%;">' . strtoupper($node_status . $node_status_extra) .
                                (count($node_status_cluster) !== 0 ? ' [' . implode(',', $node_status_cluster) . ']' : '') .
                            '</td>
                            <td style="width:1%;"></td>
                            <td><div>' . implode('<br>', $node_notes) . '</div></td>
                            <td><div id="'.$node_name.'-actions" style="float:right;">' .
                            $this->_generate_actions($node_actions) .
                           '</div></td></tr>';

            $nodes_class = ($nodes_class == 'odd' ? 'even' : 'odd');
            $is_initialized = true;
        };

        if ($nodes_has_stopped) {
            $nodes_footer .= $this->_form_button('node-start-all', 'Start all nodes',
                'sng_control_panel/start_confirm/clustermon',
                'padding: 3px; padding-bottom: 5px;');
        }
        if ($nodes_has_started) {
            $nodes_footer .= $this->_form_button('node-stop-all', 'Stop all nodes',
                'sng_control_panel/stop_confirm/clustermon',
                'padding: 3px; padding-bottom: 5px;');
        }

        /* now we start constructing the tables... */

        $data_tables = array();

        $data_tables[] = array(
                'table_title' => 'Cluster Nodes',
                'table' =>
                    ($nodes_rows === '') ?
                   '<tr><td style="text-align: center;" class="odd">(no node information available)</td></tr></table>' :
                   '<thead>
                    <tr>
                        <th class="mytableheader" style="white-space:nowrap; text-align:left;">Nodes</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;">Status</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;"></th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:left;">Notes</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;"></th>
                    </tr>
                    </thead>
                    <tbody>' . $nodes_rows . '</tbody>
                    </table>
                    <table cellspacing="0" cellpadding="0" width="100%" border="0" class="mytablelegend">
                    <tbody>
                    <tr>
                        <td>
                            <div>' . $nodes_footer . '</div>
                        </td>
                    </tr>
                    </tbody>
                    </table>'
        );

        $data_tables[] = array(
            'table_title' => 'Cluster Resources',
            'table' =>
                ($resources_rows === '') ?
                   '<tr><td style="text-align: center;" class="odd">(no resource information available)</td></tr></table>' :
                   '<thead>
                    <tr>
                        <th class="mytableheader" style="white-space:nowrap; text-align:left;">Resource</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;">Status</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:left;"></th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:left;">Location</th>
                        <th class="mytableheader" style="white-space:nowrap; text-align:center;"></th>
                    </tr>
                    </thead>
                    <tbody>' . $resources_rows . '</tbody>
                    </table>
                    <table cellspacing="0" cellpadding="0" width="100%" border="0" class="mytablelegend">
                    <tbody>
                    <tr>
                        <td>
                            <div>' . $resources_footer . '</div>
                        </td>
                    </tr>
                    </tbody>
                    </table>'
        );

        return $data_tables;
    }

    private function append_if_not_empty(&$string, $extra)
    {
        if (strlen($string) !== 0) {
            $string .= $extra;
        }
    }

    private function _cluster_services_tables(&$warnings, &$unrecognized, &$errors)
    {
        $license = $this->_the_app->node()->software()->license();
        //Pulls all services required for the gateway and checks their status
        $services = $this->_the_product->local_node()->software()->services();
        // Categorize services
        $service_cat = array();
        foreach ($services as $service) {
            // Check user can control this service
            //if ($service->allow_user_ctl())
            if($license->allow_service($service))
                $service_cat[$service->category() ][] = $service;
            
        }
        // re-organize to have following category order
        // 'Application'
        // ...
        // 'Other'
        $app_cat = $service_cat['Application'];
        unset($service_cat['Application']);
        $other_cat = $service_cat['Other'];
        unset($service_cat['Other']);
        $new_service['Application'] = $app_cat;
        // Add all other categories
        foreach ($service_cat as $k => $v) $new_service[$k] = $v;
        if (!empty($other_cat)) $new_service['Other'] = $other_cat;
        unset($service_cat);
        $service_cat = $new_service;

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
        }

        $node_keys = array_keys($cluster_monitor->nodes());
        $node_list = array();

        $self_node_name = gethostname();
        $self_node_uuid = Sng_clustermon_service_class::machine_uuid();

        $found_self = false;
        foreach ($node_keys as $node_name) {
            $node_attr = $cluster_monitor->api_retrieve_node($node_name);
            if ($node_attr['uuid'] == $self_node_uuid) {
                $node_list[$node_name] = true;
                $self_node_name = $node_name;
                $found_self = true;
            } else {
                $node_list[$node_name] = false;
            }
        }

        if (!$found_self) {
            safe_log(SAFE_LOG_INFO, 'current node "' . $self_node_name . '" not found in node list, adding manually');
            $node_list[$self_node_name] = true;
        }

        ksort($node_list);

        $self_clustermon_stats = array();
        $self_clustermon_status = $cluster_monitor->status($self_clustermon_stats);

        $nsc_group_attr = $cluster_monitor->api_retrieve_group(Sng_clustermon_service_class::CLUSTER_GROUP_NAME);
        $nsc_group_maint_mode = safe_array_get($nsc_group_attr['maintenance'], 'off');

        $res_cluster_attr = $cluster_monitor->api_retrieve_resource(Sng_clustermon_service_class::CLUSTER_SWITCH_NAME);
        $res_cluster_maint_mode = safe_array_get($res_cluster_attr['maintenance'], 'off');

        // Loop around all categories to create separate tables
        foreach ($service_cat as $cat_name => $services) {
            //Declare Table classes to be used
            $table = new Safe_table_class();
            $tmp_data = array();
            //Prepare the Control Panel Table Title
            $tmp_data['table_title'] = $cat_name . ' Services';
            //Creates the headers of the table
            $table_headers = array();
            $nodes_headers = array();
            foreach ($node_list as $node_name => $node_self) {
                array_push($table_headers,
                      '<th class="mytableheader" colspan="4" style="text-align: center;">'.$node_name.'</th>');
                array_push($nodes_headers,
                      '<th class="mytableheader" style="text-align: center; border-left: 1px solid #c9c9c9;">Status</th>
                       <th class="mytableheader" style="text-align: center; width: 75px;">Uptime</th>
                       <th class="mytableheader" style="text-align: center; width: 75px;">CPU</th>
                       <th class="mytableheader" style="text-align: center; width: 75px; border-right: 1px solid #c9c9c9;">Mem</th>');
            };
            $table->heading = array();
            $table->heading_button = '';

            foreach ($services as $service) {

                if ($service->allow_user_ctl() === 'hide') continue;

                // just NSC for now
                $managed_service = ($service->name() === 'nsc');

                $data_status = array(
                    array('data' => $service->description() ,
                          'style' => 'font-weight:bold; width:25%;height: 25px;')
                );

                $action_map = array();
                $has_action = false;

                foreach ($node_list as $node_name => $node_self) {
                    $node_attr = NULL;
                    $node_data = NULL;

                    if ($node_self) {
                        $node_attr = array('uuid' => $self_node_uuid);
                        $node_data = array();
                    } else {
                        $node_attr = $cluster_monitor->api_retrieve_node($node_name);
                        $node_data = $cluster_monitor->api_status_node($node_name);
                    }

                    $node_status = safe_array_get($node_data['status'], null);
                    $maint_mode = safe_array_get($node_attr['maintenance'], 'off');

                    list($rc, $stats) = array(Safe_service_class::STATUS_ERROR, array());

                    if ($node_attr['uuid'] == $self_node_uuid) {
                        // Retrieve status with stats
                        $rc = $service->status($stats);
                    } else {
                        list($rc, $stats) = $cluster_monitor->services_status($node_name, $service->process_name());
                    }

                    $status_data = array();

                    switch ($rc)
                    {
                        case Safe_service_class::STATUS_RUNNING:
                            $self_running_status = array('initializing', 'joining', 'online');
                            $peer_running_status = array_merge($self_running_status, array(null));

                            $started = true;

                            if ($managed_service) {
                                $started = in_array($node_status, ($node_attr['uuid'] === $self_node_uuid ? $self_running_status : $peer_running_status));
                            }

                            $status_data = array('status' => ($started ? 'STARTED' : 'UNMONITORED'), 'class' => ($started ? 'started' : 'error'), 'action' => 'stop');
                            break;

                        case Safe_service_class::STATUS_STOPPED:
                            $status_data = array('status' => 'STOPPED', 'class' => 'stopped', 'action' => 'start');
                            break;

                        case Safe_service_class::STATUS_UNRECOGNIZED:
                            //display as not installed
                            if ($node_attr['uuid'] == $self_node_uuid) {
                                $status_data = array('status' => $service->status_text($rc), 'class' => 'error', 'action' => 'none');
                            } else {
                                $status_data = array('status' => 'UNKNOWN', 'class' => 'error', 'action' => 'none');
                            }
                            break;

                        case Safe_service_class::STATUS_ERROR:
                            if ($node_attr['uuid'] == $self_node_uuid) {
                                $status_data = array('status' => 'ERROR', 'class' => 'error', 'action' => 'reset');
                                $errors[] = $service->description();
                            } else {
                                $status_data = array('status' => 'UNREACHABLE', 'class' => 'error', 'action' => 'none');
                            }
                            break;

                        /* in any other cases, just display Reset */
                        case Safe_service_class::STATUS_DEAD:
                        default:
                            $status_data = array('status' => 'UNKNOWN', 'class' => 'error', 'action' => 'reset');
                            $errors[] = $service->description();
                            break;
                    }

                    $this->append_if_not_empty($stats['cpu']['percent'], '%');
                    $this->append_if_not_empty($stats['mem']['percent'], '%');

                    array_push($data_status,
                        array( 'data' => $status_data['status'],
                               'style' => 'width:12%;',
                               'class' => $status_data['class']),
                        array( 'data' => $stats['cpu']['time'],
                               'style' => 'text-align: center;' ),
                        array( 'data' => $stats['cpu']['percent'],
                               'style' => 'text-align: center;' ),
                        array( 'data' => $stats['mem']['percent'],
                               'style' => 'text-align: center;' )
                    );

                    // NOTE: special handling for managed services, no action allowed in managed mode
                    if ($managed_service && in_array($status_data['action'], array('start', 'stop'))) {
                        if ($maint_mode === 'off' && $nsc_group_maint_mode === 'off' && $res_cluster_maint_mode === 'off') {
                            $status_data['action'] = 'none';
                        }
                    }

                    if ($status_data['action'] !== 'none') {
                        $has_action = true;
                    }

                    $action_map[ $status_data['action'] ][$node_name] = $status_data;
                }

                // NOTE: skip managed service on services table if nothing to show
                if ($managed_service && $self_clustermon_status === Safe_service_class::STATUS_RUNNING && !$has_action) {
                    continue;
                }

                $line_button = array(array('type' => 'as-is', 'data' => $this->_generate_services_actions($action_map, $service->name(), !$managed_service)));
                $table->line_button[] = $line_button;

                //Appends to the $table->data array. This adds the service and its status as an element
                array_push($table->data, $data_status);
                //Creates the primary key of each line to pass when the form is parsed
                //$table->primary_key = array(1,2);
                array_push($table->primary_key, $service->name());
            }

            if($table->data){
                $tmp_data['table'] = '<thead>
                    <tr>
                         <th class="mytableheader" rowspan="2">Service</th>' . join('', $table_headers) .
                        '<th class="mytableheader" rowspan="2" style="text-align: center;"></th>
                    </tr>
                    <tr>' . join('', $nodes_headers) . '</tr>
                </thead>' . $table->generate();

                $data_tables[] = $tmp_data;
            }
        }

        $auto_refresh = is_numeric($_POST['refresh']) ? $_POST['refresh'] : $_GET['refresh'];
        $valid_refresh = is_numeric($auto_refresh) && $auto_refresh > 0;

        $data_tables[] = array(
            'table_title' => NULL,
            'table_id' => 'refresh',
            'table' => '
                <tr style="text-align: center;">
                    <td><div style="display: inline-block; white-space: nowrap;">
                    Auto-refresh (sec):
                    <form style="display: inline-block;" action="sng_control_panel" method="post" enctype="multipart/form-data">
                        <input name="refresh" type="number" min="0" max="999" value="' . ($valid_refresh ? $auto_refresh : '0') . '" style="padding-left: 3px;">
                        <input value="Update" type="submit" class="ui-state-default ui-corner-all">
                    </form><span style="display: inline-block; margin-left: 10px; padding-left: 5px; border-left: 1px solid #AAAAAA;">' .
                    $this->_form_button('refresh', 'Refresh now', 'sng_control_panel', '', false) .
                '</span></div></td></tr></table>'
        );

        return $data_tables;
    }

    public function _cluster_verify_action($service_name, $node_name, $action, &$data, &$rc)
    {
        if (!$this->_is_cluster_enabled())
            return false;

        if ($node_name === gethostname())
            return false;

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
            return false;
        }

        $result = false;
        $output = null;

        $process_name = $service_name;

        $rest_data = $node_name == NULL ? array() : array('nodes' => array($node_name));

        if ($action == 'start' && $service_name == 'clustermon' && $node_name === NULL) {
            $rest_data['wait-master'] = 'true';
        }

        $service = $this->_the_product->local_node()->software()->service($service_name);
        if ($service === null) {
            safe_log(SAFE_LOG_ERROR, 'could not retrieve process name for service "' . $service_name . '": service not found');
        } else {
            $process_name = $service->process_name();
        }

        if ($action == 'start') {
            $result = $cluster_monitor->api_start_services($process_name, $rest_data, $output);
        } else if ($action == 'stop') {
            $result = $cluster_monitor->api_stop_services($process_name, $rest_data, $output);
        } else if ($action == 'restart') {
            $result = $cluster_monitor->api_restart_services($process_name, $rest_data, $output);
        } else {
            $output = array('Unsupported action "' . $action . '" for service "' . $service_name . '" (' . $process_name . ')');
        }

        if ($result === true) {
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        }
        $data['notif_warn'] = $output;
        return true;
    }

    public function start_confirm($service_name, $node_name = NULL)
    {
        $service = $this->_the_product->local_node()->software()->service($service_name);
        $data['page_title'] = 'Service ' . $service->description() ;
        if ($service->can_start($service_result)) {
            $data['message'] = 'Confirm start service <b>' . $service->description() . '</b>';
            if ($this->_is_cluster_enabled()) {
                if ($node_name != NULL) {
                    $data['message'] .= ' on node <b>' . $node_name . '</b>?';
                } else {
                    $data['message'] .= ' on <b>all nodes</b>?';
                }
            } else {
                $data['message'] .= '?';
            }
            $data['message_type'] = 'notice';
            $data['result'] = true;
            $data['action_url'] = base_url($this->router->fetch_class()). '/start/'. $service_name;
            if ($node_name != NULL) {
                $data['action_url'] .= '/' . $node_name;
            }
            $data['setting']['title'] = 'Starting ' .$service->description();
            $data['setting']['sec'] = '60';
            if (in_array($service_name, array('tdmgw', 'clustermon'))) {
                $data['setting']['sec'] = '90';
                $data['setting']['title'] =  $data['setting']['title'] .
                "<br/><br/><small>(this operation can take up to {$data['setting']['sec']} seconds)</small><br/>";
            }
            $data['setting']['msg'] = 'Ooops ! Server time out...';
            $data['setting']['label'] = 'Back';
            $data['setting']['url'] = base_url('sng_control_panel');
            echo json_encode($data);
        }else{
            $data['message'] = 'Start ' .$service_name .' error.';
            $reason = array();
            foreach ($service_result as $failure_reason) {
                $reason[] = $failure_reason['msg'];
            }
            if($reason != null){
                $data['message'] =  $data['message'] .'<br/><br/>' .implode('<br/>',$reason);
            }
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }
    //Function that runs when starting a process
    public function start($service_name, $node_name = NULL)
    {
        // verify if logged
        //safe_authenticate ();
        list($data, $rc) = array(array(), 0);
        if (!$this->_cluster_verify_action($service_name, $node_name, 'start', $data, $rc)) {
            //Start Service
            $result = null;
            $output = null;
            $mode = null;
            $service_result = null;
            //$the_product = Safe_get_product();
            $service = $this->_the_product->local_node()->software()->service($service_name);
            // Set the basic data
            if ($service->can_start($service_result)) {
                $result = $service->start($output);
                $mode = $service->set_startup(Safe_service_class::STARTUP_AUTOMATIC);
                // Redirect to main page on success
                if (!$result && $mode){
                    $data['result'] = true;
                    $data['redirect_url'] =  base_url('sng_control_panel');
                    echo json_encode($data);
                    exit();
                }
                $data['notif_warn'] = $output;
            } else {
                foreach ($service_result as $failure_reason) {
                    if (isset($failure_reason['url'])) {
                        $data['notif_error'][] = safe_form_with_submit_button($failure_reason['msg'], $failure_reason['url'], $failure_reason['label']);
                    } else {
                        $data['notif_warn'][] = $failure_reason['msg'];
                    }
                }
            }
        }
        //return error message
        $data['message'] = 'Start ' .$service_name .' error.';
        if($data['notif_warn']){
            $data['message'] =  $data['message'] .'<br/>' .implode('<br/>',$data['notif_warn']);
            unset($data['notif_warn']);
        }
        $data['message_type'] = 'alert';
        $data['result'] = false;
        echo json_encode($data);
    }
    public function reset_confirm($service_name, $node_name = NULL)
    {
        $service = $this->_the_product->local_node()->software()->service($service_name);
        $data['page_title'] = 'Reset Service ' . $service->description() ;
        $data['message'] = 'Confirm to reset  service : <b>' . $service->description() . '</b>?';
        $data['message_type'] = 'notice';
        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()). '/reset/'. $service_name;
        if ($node_name != NULL) {
            $data['action_url'] .= '/' . $node_name;
        }
        $data['setting']['title'] = 'Resetting '.$service->description();
        $data['setting']['sec'] = '60';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_control_panel');
        echo json_encode($data);
    }
    public function reset($service_name, $node_name = NULL)
    {
        if ($node_name != NULL && $node_name != gethostname()) {
            // TODO: IMPLEMENT RESET REMOTELY
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        }
        // verify if logged
        //safe_authenticate ();
        //Restart Service
        $retCode = null;
        //$the_product = Safe_get_product();
        $service = $this->_the_product->local_node()->software()->service($service_name);
        $service->reset($retCode);
        // Redirect to main page on success
        if ($retCode == 0) {
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        }else {
            $data['message'] = 'Reset ' .$service_name .' error.';
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }
    //Function that runs when restarting a process
    public function restart($service_name, $node_name = NULL)
    {
        // verify if logged
        //safe_authenticate ();
        list($data, $rc) = array(array(), 0);
        if (!$this->_cluster_verify_action($service_name, $node_name, 'restart', $data, $rc)) {
            //Restart Service
            $result = null;
            $output = null;
            //$the_product = Safe_get_product();
            $service = $this->_the_product->local_node()->software()->service($service_name);
            $result = $service->restart(false, $output);
            // Redirect to main page on success
            if (!$result) safe_redirect(base_url('sng_control_panel'));
        } else {
            if ($rc !== 0) safe_redirect(base_url('sng_control_panel'));
        }
        //Declare Table classes to be used
        $table = new Safe_table_class();
        //Prepare the Control Panel Table Title
        $data['table_title'] = "Starting " . $service_name;
        $table->line_button = null;
        $table->data = $output;
        $table->footer_button = array(
            array(
                'url' => base_url('sng_control_panel') ,
                'name' => 'Back'
            )
        );
        $data['table'] = $table->generate();
        // Set the basic data
        $data['page_title'] = 'Control Panel';
        $data['page_summary'] = 'This page allows controlling application hardware and software.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';
        $data['errors'] = array();
        $this->load->view("safe_header", $data);
        $this->load->view("safe_error");
        //$this->output->append_output( $service_name );
        $this->load->view("sng_control_panel_start", $data);
        $this->load->view("safe_footer", $data);
    }
    public function stop_confirm($service_name, $node_name = NULL)
    {
        $service = $this->_the_product->local_node()->software()->service($service_name);

        $data['page_title'] = 'Service ' . $service->description() ;
        $data['message_type'] = 'notice';

        $data['message'] .= 'Stop service <b>' . $service->description() . '</b>';
        if ($this->_is_cluster_enabled()) {
            if ($node_name != NULL) {
                $data['message'] .= ' on node <b>' . $node_name . '</b>?';
            } else {
                $data['message'] .= ' on <b>all nodes</b>?';
            }
        } else {
            $data['message'] .= '?';
        }

        if ($service_name === 'clustermon') {
            $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
            if ($cluster_monitor !== NULL) {
                $master_status = $cluster_monitor->check_master(NULL, $node_name);

                if (($master_status === true) || ($master_status === false && $node_name === NULL)) {
                    $data['message_type'] = 'alert';
                    $data['message'] .= '<br><p>Stopping the <b>'.$service->description().'</b> service will also terminate the <b>NetBorder Session Controller</b> services';
                    if ($node_name === NULL) {
                        $data['message'] .= ' completely on the cluster';
                    } else {
                        $data['message'] .= ' on this node.<br><p>The slave node will attempt to host the services if available and enabled';
                    }
                    $data['message'] .= '.';
                }
            }
        }

        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()). '/stop/'. $service_name;
        if ($node_name != NULL) {
            $data['action_url'] .= '/' . $node_name;
        }
        $data['setting']['title'] = 'Stopping '.$service->description() ;
        $data['setting']['sec'] = '60';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_control_panel');
        echo json_encode($data);
    }
    //Function that runs when stopping a process
    public function stop($service_name, $node_name = NULL)
    {
        if($service_name=='nsc'){
            safe_set_session('stopping_nsc', 'true');
        }

        // verify if logged
        //safe_authenticate ();
        list($data, $rc) = array(array(), 0);
        if (!$this->_cluster_verify_action($service_name, $node_name, 'stop', $data, $rc)) {
            //Stop Service
            $result = null;
            $output = null;
            $mode = null;
            //$the_product = Safe_get_product();
            $service = $this->_the_product->local_node()->software()->service($service_name);
            $result = $service->stop($output);
            $mode = $service->set_startup(Safe_service_class::STARTUP_MANUAL);

            // Redirect to main page on success
            if (!$result && $mode) {
                $data['result'] = true;
                $data['redirect_url'] = base_url('sng_control_panel');
                echo json_encode($data);
                exit();
            } else {
                $data['notif_warn'] = $output;
            }
        }
        $data['message'] = 'Stop ' .$service_name .' error.';
        if($data['notif_warn']) {
            $data['message'] =  $data['message'] .'<br/><br/>' .implode('<br/>',$data['notif_warn']);
        }
        $data['message_type'] = 'alert';
        $data['result'] = false;
        echo json_encode($data);
    }
    //Changes the service mode
    public function set_mode($mode, $service_name)
    {
        // verify if logged
        //safe_authenticate ();
        $result = false;
        //$the_product = Safe_get_product();
        $service = $this->_the_product->local_node()->software()->service($service_name);
        // check mode to apply
        switch ($mode) {
        case Safe_service_class::STARTUP_MANUAL:
        case Safe_service_class::STARTUP_AUTOMATIC:
            $result = $service->set_startup($mode);
            break;

        default:
            echo "Invalid mode " . $mode;
            break;
        }
        // Redirect to main page on success
        if ($result) safe_redirect(base_url('sng_control_panel'));
        //Declare Table classes to be used
        $table = new Safe_table_class();
        $table->data = array(
            '<b>' . $service_name . '</b> was not able to be set as ' . $mode . ' Startup'
        );
        //Prepare the Control Panel Table Title
        $data['table_title'] = "Changing " . $service_name . ' startup mode';
        $table->line_button = null;
        $table->footer_button = array(
            array(
                'url' => base_url('sng_control_panel') ,
                'name' => 'Back'
            )
        );
        $data['table'] = $table->generate();
        // Set the basic data
        $data['page_title'] = 'Control Panel';
        $data['page_summary'] = 'This page allows controlling application hardware and software.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-register.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-register.png';

        $this->load->view("safe_header", $data);
        $this->load->view("safe_error");
        $this->load->view("safe_table", $data);
        $this->load->view("safe_footer", $data);
    }

    /**** RESOURCE START/STOP/CLEANUP ****/

    public function resource_confirm($resource_name,$action)
    {
        $caption = NULL;

        switch ($action) {
            case 'start':
                $caption = 'Starting';
                break;
            case 'stop':
                $caption = 'Stopping';
                break;
            case 'cleanup':
                $caption = 'Cleaning up';
                break;
            default:
                $data['message'] = 'Unknown action "' . $action . '" on resource "' . $resource_name . '".';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $pretty_name = Sng_clustermon_service_class::resource_pretty_name($resource_name);
        $data['page_title'] = 'Resource ' . $pretty_name;
        $data['message'] = 'Confirm '.$action.' of resource <b>' . $pretty_name . '</b>?';
        $data['setting']['title'] = $caption.' '.$pretty_name. '...';

        $data['message_type'] = 'notice';
        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()). '/resource/'. $resource_name.'/'.$action;
        $data['setting']['sec'] = '90';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_cluster_status');
        echo json_encode($data);
    }

    public function resource($resource_name, $action)
    {
        $output = '';
        $target = NULL;
        $result = NULL;

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
        }

        switch ($action) {
            case 'start':
            case 'stop':
                $target = ($action == 'start' ? 'Started' :  'Stopped');
                $result = $cluster_monitor->api_update_resource($resource_name, array('target-role' => $target), $output);
                break;

            case 'cleanup':
                $result = $cluster_monitor->api_cleanup_resource($resource_name, array(), $output);
                break;

            default:
                $data['message'] = 'Unknown action: ' . $action . '.';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        if ($result) {
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        }else{
            $data['message'] = $output['message'];
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }

    /**** GROUP START/STOP ****/

    public function group_confirm($group_name,$action)
    {
        $caption = NULL;

        switch ($action) {
            case 'start':
                $caption = 'Starting';
                break;
            case 'stop':
                $caption = 'Stopping';
                break;
            default:
                $data['message'] = 'Unknown action "' . $action . '" on group "' . $group_name . '".';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
            return;
        }

        $data['page_title'] = $group_name . ' resources';
        $data['message'] = 'Confirm '.$action.' of group <b>' . $group_name . ' ' . $cluster_monitor->resource_pretty_name($group_name) . '</b>?';
        $data['setting']['title'] = $caption.' resources...';

        $data['message_type'] = 'notice';
        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()). '/group/'. $group_name.'/'.$action;
        $data['setting']['sec'] = '90';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_control_panel');
        echo json_encode($data);
    }

    public function group($group_name, $action)
    {
        $output = array();
        $target = '';

        switch ($action) {
            case 'start':
                $target = 'Started';
                break;
            case 'stop':
                $target = 'Stopped';
                break;

            default:
                $data['message'] = 'Unknown action "' . $action . '" on group "' . $group_name . '".';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
            return;
        }

        if ($cluster_monitor->api_update_group($group_name, array('target-role' => $target), $output))
        {
            $data['result'] = true;
            $data['redirect_url'] = base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        }else{
            $data['message'] = $output['message'];
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }

    /**** MAINTENANCE START/STOP ****/

    public function maintenance_confirm($object_type, $object_name, $action)
    {
        list($caption, $caption2) = array(NULL, NULL);

        switch ($action) {
            case 'set':
                $caption = 'Set';
                $caption2 = 'Setting';
                break;
            case 'clear':
                $caption = 'Clear';
                $caption2 = 'Clearing';
                break;
            default:
                $data['message'] = 'Unknown action "' . $action . '" maintenance mode on ' . $object_type . ' "' . $object_name . '".';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        switch ($object_type) {
            case 'resource':
                $pretty_name = Sng_clustermon_service_class::resource_pretty_name($object_name);
                $data['page_title'] = $pretty_name;
                $data['message'] = $caption . ' maintenance mode for resource <b>' . $pretty_name . '</b>?';
                $data['setting']['title'] = $caption2 . ' maintenance mode for resource <b>'. $pretty_name. '</b>...';
                break;
            case 'group':
                $pretty_name = Sng_clustermon_service_class::resource_pretty_name($object_name);
                $data['page_title'] = $pretty_name;
                $data['message'] = $caption . ' maintenance mode for group <b>' . $object_name . '&nbsp;' . $pretty_name . '</b>?';
                $data['setting']['title'] = $caption2 . ' maintenance mode for group <b>'. $object_name . '&nbsp;' . $pretty_name. '</b>...';
                break;
            case 'node':
                $data['page_title'] = 'Node ' . $object_name;
                $data['message'] = $caption . ' maintenance mode for node <b>' . $object_name . '</b>?';
                $data['setting']['title'] = $caption2 . ' maintenance mode for node <b>'. $object_name. '</b>...';
                break;
            default:
                $data['message'] = 'Unknown object type "' . $object_type . '".';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $data['message_type'] = 'notice';
        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()). '/maintenance/'.$object_type.'/'.$object_name.'/'.$action;
        $data['setting']['sec'] = '90';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_control_panel');
        echo json_encode($data);
    }

    public function maintenance($object_type, $object_name, $action)
    {
        $output = array();
        $maint_status = '';

        switch ($action) {
            case 'set':
                $maint_status = 'on';
                break;
            case 'clear':
                $maint_status = 'off';
                break;
            default:
                $data['message'] = 'Unknown action: ' . $action . '.';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $action_data = array('maintenance' => $maint_status);

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
            return;
        }

        $result = null;

        switch ($object_type) {
            case 'resource':
                $result = $cluster_monitor->api_update_resource($object_name, $action_data, $output);
                break;
            case 'group':
                $result = $cluster_monitor->api_update_group($object_name, $action_data, $output);
                break;
            case 'node':
                $result = $cluster_monitor->api_update_node($object_name, $action_data, $output);
                break;
            default:
                $data['message'] = 'Unknown object type "' . $object_type . '".';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        // Redirect to main page on success
        if ($result) {
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        } else {
            //$pretty_name = in_array($object_type, array('resource', 'group')) ? $cluster_monitor->resource_pretty_name($object_name) : $object_name);
            $data['message'] = $output['message']; // 'Unable to '. $action .' maintenance on ' .$pretty_name .' (code='.$rc.').'.$output;
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }

    /**** STANDBY START/STOP ****/

    public function standby_confirm($node_name, $action)
    {
        $caption = NULL;
        $name_text = '';
        $extra_text = '';
        $msg_type = 'notice';

        switch ($action) {
            case 'set':
                $caption = 'Disable';
                $name_text = 'Disabling';
                $msg_type = 'alert';
                $extra_text = '<br><p>Disabling a node will prevent it from hosting any resources and migrate any resources currently being hosted by it.';
                break;
            case 'clear':
                $caption = 'Enable';
                $name_text = 'Enabling';
                break;
            default:
                $data['message'] = 'Unknown action "' . $action . '" for standby.';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $data['page_title'] = 'Node ' . $node_name;
        $data['message'] = $caption . ' node <b>' . $node_name . '</b>?' . $extra_text;
        $data['message_type'] = $msg_type;
        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()). '/standby/'.$node_name.'/'.$action;
        $data['setting']['title'] = $name_text . ' node '. $node_name. '...';
        $data['setting']['sec'] = '90';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_control_panel');
        echo json_encode($data);
    }
    public function standby($node_name, $action)
    {
        $output = array();
        $action_data = NULL;

        switch ($action) {
            case 'set':
                $action_data = 'on';
                break;
            case 'clear':
                $action_data = 'off';
                break;
            default:
                $data['message'] = 'Unknown action "' . $action . '" for standby.';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
            return;
        }

        if ($cluster_monitor->api_update_node($node_name, array('standby' => $action_data), $output))
        {
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        }else{
            $data['message'] = $output['message']; // 'Unable to '. $action .' standby on ' .$node_name .' (code='.$rc.').'.$output;
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }

    /**** MIGRATE ****/

    public function migrate_confirm($node_name)
    {
        $data['page_title'] = 'Migration for ' . $node_name;
        $data['message'] = 'Migrate resources to node <b>' . $node_name . '</b>?';
        $data['message_type'] = 'notice';
        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()). '/migrate/'.$node_name;
        $data['setting']['title'] = 'Migrating to node <b>'. $node_name . '</b>...<br><br><small>(this operation may take up to a minute)</small>';
        $data['setting']['sec'] = '240';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_control_panel');
        echo json_encode($data);
    }

    public function migrate($node_name)
    {
        $output = '';
        $value = NULL;

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
            return;
        }

        if ($cluster_monitor->api_migrate_node($node_name, array(), $output)) {
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        } else{
            $data['message'] = $output['message'];
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }

    /**** MIGRATE_TO / UNMIGRATE_TO ****/

    public function migrate_to_confirm($node_name, $action)
    {
        switch ($action) {
            case 'set':
                $data['page_title'] = 'Migration for ' . $node_name;
                $data['message'] = 'Migrate resources to node <b>' . $node_name . '</b>?';
                $data['setting']['title'] = 'Migrating to node <b>'. $node_name . '</b>...<br><br><small>(this operation may take a couple of minutes)</small>';
                break;
            case 'clear':
                $data['page_title'] = 'Migration for ' . $node_name;
                $data['message'] = 'Clear migration to node <b>' . $node_name . '</b>?';
                $data['setting']['title'] = 'Clearing migration to node <b>'. $node_name . '</b>...';
                break;
            default:
                $data['message'] = 'Unknown action: ' . $action . '.';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $data['message_type'] = 'notice';
        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()). '/migrate_to/'.$node_name.'/'.$action;
        $data['setting']['sec'] = '240';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_control_panel');
        echo json_encode($data);
    }

    public function migrate_to($node_name, $action)
    {
        $output = '';
        $value = NULL;

        switch ($action) {
            case 'set':
                $value = 'INFINITY';
                break;
            case 'clear':
                $value = 'null';
                break;
            default:
                $data['message'] = 'Unknown action: ' . $action . '.';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
            return;
        }

        if ($cluster_monitor->api_update_node($node_name, array('location' => array('NSC' => $value)), $output)) {
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        } else{
            $data['message'] = $output['message'];
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }

    /**** MIGRATE_AWAY / UNMIGRATE_AWAY ****/

    public function migrate_away_confirm($node_name, $action)
    {
        switch ($action) {
            case 'set':
                $data['page_title'] = 'Ban resources on ' . $node_name;
                $data['message'] = 'Ban resources on node <b>' . $node_name . '</b>?';
                $data['setting']['title'] = 'Banning resources on node <b>' . $node_name . '</b>...';
                break;
            case 'clear':
                $data['page_title'] = 'Clear resources ban on ' . $node_name;
                $data['message'] = 'Clear resources ban on node <b>' . $node_name . '</b>?';
                $data['setting']['title'] = 'Clearing resources ban on node <b>' . $node_name . '</b>...';
                break;
            default:
                $data['message'] = 'Unknown action: ' . $action . '.';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $data['message_type'] = 'notice';
        $data['result'] = true;
        $data['action_url'] = base_url($this->router->fetch_class()).'/migrate_away/'.$node_name.'/'.$action;
        $data['setting']['sec'] = '90';
        $data['setting']['msg'] = 'Ooops ! Server time out...';
        $data['setting']['label'] = 'Back';
        $data['setting']['url'] = base_url('sng_control_panel');
        echo json_encode($data);
    }
    public function migrate_away($node_name, $action)
    {
        $output = '';
        $value = NULL;

        switch ($action) {
            case 'set':
                $value = '-INFINITY';
                break;
            case 'clear':
                $value = 'null';
                break;
            default:
                $data['message'] = 'Unknown action: ' . $action . '.';
                $data['message_type'] = 'alert';
                $data['result'] = false;
                echo json_encode($data);
                return;
        }

        $cluster_monitor = $this->_the_product->local_node()->software()->application()->find_module_by_name('clustermon');
        if ($cluster_monitor == NULL) {
            // KABOOM! TODO: handle
            return;
        }

        if ($cluster_monitor->api_update_node($node_name, array('location' => array('NSC' => $value)), $output)) {
            $data['result'] = true;
            $data['redirect_url'] =  base_url('sng_control_panel');
            echo json_encode($data);
            exit();
        } else{
            $data['message'] = $output['message'];
            $data['message_type'] = 'alert';
            $data['result'] = false;
            echo json_encode($data);
        }
    }

    /*
     * get system running status
    */
    public function get_system_status($format = 'json'){
        $data = array();
        // Default is system ok
        $global_status = 'ok';
        $application_service = null;
        foreach ($this->_services as $service) {
            if('Safe_application_class' == get_class($service)){
                $application_service = $service;
            }
            $data['services'][$service->name()]['label'] = $service->description();
            $data['services'][$service->name()]['status'] = $service->status_text();
            switch($service->status()){
                case Safe_service_class::STATUS_RUNNING:
                case Safe_service_class::STATUS_STOPPED:
                    break;
                case Safe_service_class::STATUS_UNRECOGNIZED:
                case Safe_service_class::STATUS_ERROR:
                case Safe_service_class::STATUS_DEAD:
                default:
                    $global_status = 'warning';
                    break;
            }
        }
        if ($application_service != null){
            $system_status = $application_service->status();
        }else{
            $system_status = 'notfound';
        }
        switch ($system_status) {
            case Safe_service_class::STATUS_RUNNING:
                $data['system']['status']       = $global_status;//STARTED
                $data['system']['description']  = 'System started.';
                $data['system']['url']          = '/SAFe/sng_control_panel';
                break;
            case Safe_service_class::STATUS_STOPPED:
                $data['system']['status']        = "warning";//STOPPED
                $data['system']['description']   = 'System stopped.';
                $data['system']['url']           = '/SAFe/sng_control_panel';
                break;
            case Safe_service_class::STATUS_UNRECOGNIZED:
            case Safe_service_class::STATUS_ERROR:
            case Safe_service_class::STATUS_DEAD:
            default:
                $data['system']['status']        = 'error';//UNKNOWN
                $data['system']['description']   = 'System error!';
                $data['system']['url']           = '/SAFe/sng_control_panel';
                break;
        }
        if($format == 'array'){
            return $data;
        }else{
            echo json_encode($data);
            return true;
        }
    }
    /*
     * get configuration status
    */
    private function _checklist_data()
    {
        $data = array();
        foreach($this->_checklist as $item){
            $module = $this->_the_app->find_module_by_name($item['module']);
            if($module){
                $mod_name = $module->description();
                $obj_def = $module->get_aggregate_object_definition($item['obj_type']);
                if($obj_def){
                    $obj_type = $obj_def['name'];
                    $fix_url = $obj_def['controller_url'];
                }else{
                    $obj_type = $item['obj_type'];
                }
                $obj_name = $item['obj_name'];
            }else{
                $mod_name = $item['module'];
                $obj_type = $item['obj_type'];
                $obj_name = $item['obj_name'];
            }
            $data[] = array(
                $mod_name,
                $obj_type,
                $obj_name,
                $item['description'],
                $fix_url,
            );
        }
        return $data;
    }
    public function get_configuration_status($format = 'json'){
        if ($this->_checklist) {
            $data['status']       = 'error';
            $data['description']  = 'Configuration not completed.';
            $data['url']          = '/SAFe/sng_config_manager';
            $data['checklist']    = $this->_checklist_data();
            ob_start();
            $this->_wizard(false);
            $data['html'] = ob_get_contents();
            ob_clean();
        }else if (!$this->_the_product->local_node()->is_configuration_generated()) {
            $data['status']       = 'error';
            $data['description']  = 'Configuration not generated.';
            $data['url']          = '/SAFe/sng_config_manager';
        } else if ($this->_the_product->local_node()->is_configuration_modified()) {
            $data['status']       = 'warning';
            $data['description']  = 'Configuration modified.';
            $data['url']          = '/SAFe/sng_config_manager';
            ob_start();
            $this->_config_status(false);
            $data['html'] = ob_get_contents();
            ob_clean();
        }else{
            $data['status']       = 'ok';
            $data['description']  = 'Configuration is up to date.';
            $data['url']          = '/SAFe/sng_config_manager';
            ob_start();
            $this->_config_status(false);
            $data['html'] = ob_get_contents();
            ob_clean();
        }
        if($format == 'array'){
            return $data;
        }else{
            echo json_encode($data);
            return true;
        }
    }
    /*
     * get configuration status
    */
    public function get_status_html(){
        $data = $this->get_system_status('array');
        $data['configuration'] = $this->get_configuration_status('array');
        $this->load->view("safe_iframe_notice", $data);
    }
    public function system_status_json()
    {
        $data = $this->get_system_status('array');
        echo json_encode($data);
    }
    public function configuration_status_json()
    {
        $data['configuration'] = $this->get_configuration_status('array');
        echo json_encode($data);
    }
    /*
     * get Notification  status
    */
    public function get_notifications(){
        /*
        $db = $this->_the_app->db();
        $db->connect();
        $select = '*';
        $conditions = "status ='0'";
        $order_by = 'id desc';
        $result = $db->select('notification', $conditions, 10, 0, $order_by, '', $select, '');
        $db->disconnect();
        */
        $test_data ='[{
                    	"id": "4",
                    	"add_time": "2013-04-04 15:03:06",
                    	"notification_type": "warning",
                    	"status": "0",
                    	"description": "system warning 223!",
                    	"url": ""
                    },
                    {
                    	"id": "3",
                    	"add_time": "2013-04-04 15:03:05",
                    	"notification_type": "warning",
                    	"status": "0",
                    	"description": "system warning 222!",
                    	"url": ""
                    },
                    {
                    	"id": "2",
                    	"add_time": "2013-04-04 15:03:05",
                    	"notification_type": "error",
                    	"status": "0",
                    	"description": "system error 112!",
                    	"url": ""
                    },
                    {
                    	"id": "1",
                    	"add_time": "2013-04-04 15:03:05",
                    	"notification_type": "error",
                    	"status": "0",
                    	"description": "system error 111!",
                    	"url": "/SAFe/sng_config_manager/edit/rtcpmon"
                    }]
                ';
        $result = json_decode($test_data);

        $notification['number'] = 0;//count($result);
        if($notification['number'] > 0){
            $notification['data'] = $result;
        }
        echo json_encode($notification);
    }
}
