<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Network Config wrapper page
 * @author William Adam <william.adam@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_controller('sng','data_manager');
class Sng_license extends Sng_data_manager
{

    const NO_DEPLOYMENT_INFO = 'No deployment information';
    const HW_NOT_MATCH = 'Hardware information not match';
    const NO_HW_INFO = 'There is no information about your hardware in your current License';

    public function __construct()
    {
        parent::__construct();
    }
    public function index()
    {
        return $this->manage('application','license');
    }

    /**
     * @brief Prepare modal form to ask customer about deployment_name if needed
     * @param array $data to be filled by required json data
     * @param object module object
     * @param object license object
     * @param string $action to perform by submit button default refresh
     */
    private function prepare_form(&$data,&$module,&$license,$action='refresh'){
        // Prepare modal form
        $max_sessions = $license->get_max_session();
        $max_sessions = $max_sessions? $max_sessions : 0;
        $param = new Safe_configurable_object_class('autofill','refresh-param');
        $param->add_field('deployment-name','Deployment Name','test','','50%');
        $param->set_field_rules('deployment-name','required');
        $param->configure();
        
        $uc_action = ucwords($action);
        $config = $param->get_data();
        
        $this->load->library("safe_form_class",$config);                
        
        $data['page_title'] = 'Downloading license from Portal server ...';
        $data['form_title'] = ucwords($module->description())." : ".$uc_action . $module->aggregate_object_name($data_name);
        $data['form'] = $this->safe_form_class->form_input_array();
        $data['form_cfg'] = $this->safe_form_class->config;
        $data['label'] = $this->safe_form_class->form_label_array();
        $data['form_open'] = $this->safe_form_class->form_open();
        $data['message'] = "Deploymnet name not found for your SBC on Sangoma Portal! <br/> Please Register/Login in to 
             Sangoma Portal and create a deployment name for your SBC.<br/> 
             <br/><a href=\"https://wiki.freepbx.org/display/SBC/SBC+Portal+Deployment+Name\" style=\"float:right;border: 0px\" target=\"_black\">Click here for wiki instructions.</a><br/>";
        ob_start();
        ob_clean();
        $this->load->view('safe_form_json',$data);
        $data['content'] = ob_get_contents();
        ob_clean();
        
        $data['result'] = true;
        $data['reload'] = false;
        $data['action_method']='POST';
        $data['buttons'] = array(
            array('value'=>$uc_action,'name'=>$action,'type'=>'submit'),
            array('value'=>'Cancel','name'=>'cancel','type'=>'submit'),
        );
    }
    
    /**
     * @brief Return hdd serial number or mac number depending on which kind of
     * box software is running. If software is running on VM it returns mac number, if
     * it is on real hardware, method returns hdd serial numbers.
     * @param object module
     * @return array hdd serial number or mac number.
     */
    private function get_hardware_info(&$module){
        $hardware = $module->node()->hardware();
        $hw_info = array();
        if($hardware->is_virtual()){
            $network = $this->_the_app->find_module_by_name('network');
            $interfaces = $network->api_retrieve_interface();
            $hw_info['mac'] = array();
            foreach($interfaces as $interface)
                $hw_info['mac'][] = $interface->mac();
        }else{
            $hdds = $hardware->hdd_info();
            $hw_info['hdd'] = array();
            foreach($hdds as $dev => $info)
                $hw_info['hdd'][] = $info['serialno'];
        }
        return $hw_info;
    }

    

    /**
     * @brief prepares the params for the sng_license command. It returns true
     * or false whether it fails or not. It gets required information from
     * License file and checks if hd or mac number matches what it is in the
     * hardware where sbc is running.
     * @param object module object
     * @param object license object
     * @param array params for sng_license execution
     * @param array post information coming from ui
     * @param array empty array to place error messages
     * @return boolean true/false
     */
    private function prepare_params(&$module,&$license,&$params,&$post,&$output){
        $params = array_merge(array('--action update',
                                    '--out_format json'),$params);      

        $hardware = $module->node()->hardware();
        $hw_match = false;
        $hw_info = $this->get_hardware_info($module);

        $status = true;
        if(isset($post['deployment-name'])){
            $params[] = '--deployment_name '.$post['deployment-name'];
        }
        else if(($serial=$license->info('SBC-Serial')) && !empty($serial)){
            $params[] = '--serial '.$serial;            
        }
        else if(($deployment_name = $license->info('Deployment-Name')) && !empty($deployment_name)){
            $params[] = "--deployment_name ".$deployment_name;
        }
        else{
            $output[] = self::NO_DEPLOYMENT_INFO;
            $status = false;
        }
        if(($hd_serial = $license->info('HD-Serial')) && !empty($hd_serial)){
            if(isset($hw_info['hdd'])){
                $hw_info['hdd'][] = implode(',',$hw_info['hdd']);
                $hd_serial_lower = strtolower(trim($hd_serial));
                foreach($hw_info['hdd'] as $serial_no)
                    if(strtolower(trim($serial_no))==$hd_serial_lower){
                        $hw_match = true;
                        break;
                    }
            }
            if($hardware->is_virtual() || !$hw_match){
                $output[] = self::HW_NOT_MATCH;
                $status = false;
            }
            else
                $params[] = '--hdd_serial '.$hd_serial;
                
        }
        else if(($mac = $license->info('MAC')) && !empty($mac)){
            if(isset($hw_info['mac'])){
                $hw_info['mac'][] = implode(',',$hw_info['mac']);
                $mac_lower = strtolower(trim($mac));
                foreach($hw_info['mac'] as $_mac)
                    if(strtolower(trim($_mac))==$mac_lower){
                        $hw_match = true;
                        break;
                    }
                if(!$hw_match){
                    $output[] = self::HW_NOT_MATCH;
                    $status = false;
                }else
                    $params[] = '--mac '.$mac;
            }
        }
        
        else{
            $output[] = self::NO_HW_INFO;
            $status=false;
        }

        return $status;
    }

    /**
     * @brief This method is called previous to the refresh one it is
     * used to send some process feed back to the user.
     * @param string module name
     * @param string data name
     * @param string data key     
     */
    public function refresh_form($module_name,$data_name,$data_key=NULL){
        $data= array();
        $data['page_title'] = 'License';
        $data['page_summary'] = 'This page allows managing License.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-network.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-network.png';

        $data['setting'] = array('title'=>'Downloading license from Portal server ...',
                                 'sec'=>'60',
                                 'label'=>'Close',
                                 'waitbox'=>array('ui_type'=>'spinning_wheel'),
                                 'msg'=>'Ooops ! Downloading time out...');
        $data['action_url'] = base_url($this->router->fetch_class()).'/refresh/'.$module_name.'/'.$data_name;
        echo json_encode($data);
    }
    
    /**
     * @brief method handler of the refresh button on license ui
     * @param string module name
     * @param string data name
     * @param string data key     
     */
    public function refresh($module_name,$data_name,$data_key=NULL){
        $data = array();
        $post = $this->input->post();
        $module = $this->_the_app->find_module_by_name($module_name);
        $license = $module->node()->software()->license();
        $cmd = "/usr/local/sng/bin/sng-license";
        $os = $module->node()->os();
        $params = array();
        $output = array();
        
        $data['type'] = 'error';
        $data['result'] = false;
        $data['page_title'] = 'License refresh error.';
        if($this->prepare_params($module,$license,$params,$post,$output)){            
            $output = array();
            $os->execute($cmd.' '.implode(' ',$params),$output);
            if(!empty($output)){
                $out = @json_decode($output[0]);
                $ok = json_last_error()==0 && isset($out->status) && $out->status;
                if($ok && isset($out->license)){
                    $data_cfg = $this->_the_app->get_data_settings();
                    $data_cfg = $data_cfg['license'];
                    $dest = dirname($data_cfg['pattern']);
                    $os->execute('wget '.$out->license.' -O '.$dest.'/'.basename($out->license));
                    $output = array();
                    if($this->_the_app->initialize_data('license',basename($out->license),$output)){
                        $data['page_title'] = 'License refresh ok.';
                        $data['message'] = 'License has been successfully updated';
                        $data['type'] = 'notice';
                        $data['result'] = true;
                        $data['redirect_url'] = '/SAFe/sng_license';
                    }else{
                        $data['message'] = implode(" ",$output);
                    }
                }else{
                    $data['message'] = json_last_error()!=0? json_last_error_msg() : "[License Server error] ".$out->message;
                }
            }
        }else if(in_array(self::NO_DEPLOYMENT_INFO,$output)){
            $this->prepare_form($data,$module,$license,'refresh');
            $data['action_url'] = base_url($this->router->fetch_class()).'/refresh/'.$module_name.'/'.$data_name;
            if($data_key!=NULL)
                $data['action_url'] = $data['action_url'].'/'.$data_key;
        }else
            $data['message'] = implode(" ",$output);
        
        echo json_encode($data);
    }
}
