<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * Kent <khong@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Module action page
 * @author Kent <khong@sangoma.com>
 *
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
require_once ('application/controllers/Safe_CI_Controller.php');


class Sng_module_action extends Safe_CI_Controller
{
    protected function update_settings(&$action_setting, $data){
        //overwrite this funtion to adjust confirm dialog by post data
        return true;
    }
    public function action($module_name, $action_name)
    {
        if (!isset($module_name)) $this->controller_exit('Service/Module name required.');
        if (!isset($action_name)) $this->controller_exit('Action name required.');
        // Retrieve module to work on
        $module = $this->_the_app->find_module_by_name($module_name);
        if(!$module){
            $this->controller_exit('Service/Module not found.');
        }
        $base_url = $module->name();
        // Prepare Description
        $description = $module->description();
        //handle module actions
        $action_cfg = $module->get_action_settings();
        if(isset($action_cfg[$action_name])){
            $action_cfg = $action_cfg[$action_name];
            if (!isset($action_cfg['class']) || $action_cfg['class'] == null) {
                //no confirm dialog
                $this->controller_exit('Action parameter class required.');
            }else{
                $post = $this->input->post();
                $title = ucwords($module->description() . ' : ' . $action_cfg['name']);
                $param = $action_cfg['class'];
                if (isset($post['action'])) {
                    unset($post['action']);
                    $validation_errors = array();
                    // update the configurable object with posted values
                    $param->set_data_values($post);
                    // Prepare the form for validation with posted data
                    $config = $param->get_data();
                    $this->load->library('safe_form_class', $config);
                    // Check new data are valid
                    $result = false;
                    if ($param->validate($post, $validation_errors)) {
                        // do action
                        //run real action in module
                        $method_name = 'api_'.$action_name.'_action';
                        $output = array();
                        if(method_exists($module,$method_name)){
                            $this->update_settings($action_cfg, $post);
                            $result = true;
                            //set action data
                            $action_data = array();
                            $action_data['module'] = $module_name;
                            $action_data['action'] = $action_name;
                            $confirm_obj['page_title'] = $action_cfg['name'];
                            $confirm_obj['message'] = "{$action_cfg['pre']} Are you sure you want to {$action_cfg['description']}?";
                            $confirm_obj['message_type'] = 'alert';
                            $confirm_obj['action_url'] = "/SAFe/rest/api/{$action_name}/{$module_name}";
                            $confirm_obj['ok_data'] = $post;//form data
                            if(isset($action_cfg['button_name'])){
                                $confirm_obj['ok_name'] = $action_cfg['button_name'];
                            }else{
                                $confirm_obj['ok_name'] = $action_cfg['name'];
                            }
                            $confirm_obj['ok_method'] = 'POST';

                            $waiting_obj['label'] = 'Close';
                            $waiting_obj['msg'] = "Ooops ! {$action_cfg['description']} time out...";
                            $waiting_obj['sec'] = 360;
                            $waiting_obj['title'] = "Processing {$action_cfg['description']}...";
                            if(isset($action_cfg['redirect_url'])){
                                $waiting_obj['url'] = $action_cfg['redirect_url'];
                            }else{
                                $waiting_obj['url'] = base_url($this->router->fetch_class().'/action/'. $module_name. '/' . $action_name);
                            }
                            $action_data['confirm_obj'] = $confirm_obj;
                            $action_data['waiting_obj'] = $waiting_obj;
                            $action_confirm_view = 'safe_action_confirm';
                            if(isset($action_cfg['confirm_view'])){
                                $action_confirm_view = $action_cfg['confirm_view'];
                            }
                        }else{
                            $data['notif_warn'][] = $action_cfg['name'] . ' not found.';
                        }
                    }
                }else{
                    $config = $param->get_data();
                    $this->load->library("safe_form_class", $config);
                }

                // global messages are handled here
                $this->_the_product->local_node()->handle_global_messages($data);

                // Set the basic data
                $data['page_title'] = $title;
                $data['form_title'] = ucwords($module->description()) . ' : '  . $action_cfg['name'];
                $data['form'] = $this->safe_form_class->form_input_array();
                $data['form_cfg'] = $this->safe_form_class->config;
                $data['label'] = $this->safe_form_class->form_label_array();
                $data['form_open'] = $this->safe_form_class->form_open();
                if(!isset($action_cfg['button'])){
                    $action_cfg['button'] = ucfirst($action_cfg['name']);
                }
                $data['buttons'] = array(
                        array(
                                'name' =>  'action',
                                'value' => $action_cfg['button'],
                                'type' => 'submit'
                        )
                );

                // Get the fields group
                // send the group array in our form view
                $data['group'] = $param->get_group();
                $data['group_condition'] = $param->get_group_condition();

                // Display the views
                $this->load->view("safe_header", $data);
                $this->load->view("safe_notifications", $data);
                // set the form attributes
                $frm_attributes = array(
                        'id' => 'module_action'
                );
                $this->safe_form_class->set_form_attributes($frm_attributes);
                $data["crud_allowed"] = true;
                $this->load->view("safe_form", $data);
                if($result){
                    $this->load->view($action_confirm_view, $action_data);
                }
                $this->load->view("safe_footer", $data);
            }
        }else{
            $this->controller_exit('Action not found.');
        }
    }

}
