<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * FS Routing Config wrapper page
 * @author William Adam <william.adam@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_controller('sng','config_manager');
class Sng_provisioning_config extends Sng_config_manager
{
    private $_conf_file = '/provisioning/config.toml';
    public function __construct($module_name = 'routing')
    {
        parent::__construct();
    }
    public function index()
    {
        return $this->edit();
    }

    /**
     * Edit a file 
     * 
     */
    public function edit()
    {
        // Handle redirect
        if (isset($post['cancel']))
            redirect (base_url($this->router->fetch_class()) ,'refresh' );

        $post = $this->input->post();

        if($this->_the_product->local_node()->os()->read_file($this->_conf_file, $content)) {
            if(is_array($content)){
                $config['data'] = implode("\r\n",$content);
            }else{
                $config['data'] = $content;
            }
        }else{
            $config['data'] = '';
        }

        // Set the basic data
        $data ['page_title'] = 'Provisioning';
        $data ['page_summary'] = 'This page allows managing the Provisioning.';
        $data ['timeout_sec'] = 600;

        //Prepare table structure
        $data['table_size'] = '100%';
        $data['table_title'] = $data ['page_title'] . ' : '.$conf_file;
        $data['text_mode'] = "php";
        $data['keyboard_mode'] = "standard";

        if(isset($config['data']) && !empty($config['data'])) {
            $data['editor_data'] = htmlspecialchars($config['data']);
        } else {
            $data['editor_data'] = "";
        }
        
        $data['url_save'] = base_url($this->router->fetch_class()).'/save/';
        $data['url_apply'] = base_url($this->router->fetch_class()).'/apply_confirm/';
        $data['url_register'] = base_url($this->router->fetch_class()).'/register_confirm/';
        
        $this->load->view("safe_header", $data );
        $this->load->view("safe_confirm_dialog", $data);//load confirm dialog js and css

        $this->load->view('safe_notifications',$data);
        $this->load->view("safe_provisioning_editor",$data);
        $this->load->view("safe_footer", $data );
    }

    public function save(){
        $post = $this->input->post();
        // Default is error
        $response = array('result' => 'ERROR' );
        if (isset($post['data'])) {
            $file_data =  base64_decode($post['data']);
            $this->_the_product->local_node()->chmod_file($this->_conf_file,'777');
            $success = $this->_the_product->local_node()->write_file($this->_conf_file,$file_data );
            if ($success) {
                $this->_the_product->local_node()->chown_file($this->_conf_file,'root');
                $this->_the_product->local_node()->chmod_file($this->_conf_file,'600');
                $response['result'] = "OK";
                $response['message'] = "File saved";
            }else{
                $response['message'] = "Fail to save Provisioning file";
            }
        }
        echo json_encode($response);
    }
    public function apply_confirm($type = 'apply') {
        if($type == 'apply'){
            $title = 'Apply';
            $disply_msg = 'Apply Provisioning';
        }elseif($type == 'register'){
            $title = 'Register to EMS';
            $disply_msg = 'Register to EMS';
        }else{
            $response = array(
                    "page_title" => "Provisioning : Error",
                    "message" => "Type error! ",
                    "message_type" => "alert",
                    "result" => false,
                    "close_dialog" =>  false
            );
            echo json_encode($response);
            exit();
        }
        $response['page_title'] = "Provisioning : Apply";
        $response['message'] = "Are you sure you want to {$disply_msg}?";
        $response['message_type'] = "notice";
        $response['result'] = true;
        $response['action_url'] = base_url($this->router->fetch_class()).'/' .$type;
        $response['setting']['title'] = $disply_msg;
        $response['setting']['sec'] = '3000';
        $response['setting']['msg'] = "Ooops ! {$disply_msg} time out...";
        $response['setting']['label'] = "Close";
        $response['setting']['url'] = "";
        $response['setting']['waitbox'] = array(
                    "ui_type" => "processing_log",
			        "log_url" => base_url($this->router->fetch_class())."/get_{$type}_log",
			        "reloadtime"=> "5",
			        "log_timeout"=> "600",
			        "action_url"=> ""
                );
        echo json_encode($response);
    }
    public function apply($type = 'apply') {
        if($type == 'apply'){
            $title = 'Apply';
            $disply_msg = 'Apply Provisioning';
            $action = 'config';
        }elseif($type == 'register'){
            $title = 'Register to EMS';
            $disply_msg = 'Register to EMS';
            $action = 'ems';
        }else{
            $response = array(
                    "page_title" => "Provisioning : Error",
                    "message" => "Type error! ",
                    "message_type" => "alert",
                    "result" => false,
                    "close_dialog" =>  false
            );
            echo json_encode($response);
            exit();
        }
        $rs1 = 255;
        $log_file = "/tmp/provisioning_{$type}.log";
        if(file_exists($log_file)){
            $this->_the_product->local_node()->os()->delete_file($log_file);//clear the old logs
        }
        $result1 = Safe_sudo_exec("/provisioning/common/run.sh {$log_file} {$action} ",'','',$rs1);
        $response_result = false;
        $message_type = 'alert';
        if($rs1 == 0){
            $msg = 'SUCCESS';
            $response_result = true;
            $message_type = 'info';
        }elseif($rs1 == 255){
            $msg = 'SKIPPED';
        }else{
            $msg = 'FAILURE';
        }
        $log_data = $this->get_log($type);
        if(is_array($log_data)){
            $log_msg = implode('',$log_data);
        }else{
            $log_msg = $log_data;
        }
        $response = array(
                "page_title" => "Provisioning : {$title}",
                "message" => "{$disply_msg} " . $msg .'<br>' . $log_msg,
                "message_type" => $message_type,
                "result" => $response_result,
                "reload" => true,
                "redirect_url" => '',
                "close_dialog" =>  false
                );
        echo json_encode($response);
    }
    public function register_confirm() {
        $this->apply_confirm('register');
    }
    public function register() {
        $this->apply('register');
    }
    private function log_format($str){
        return '<pre>' .$str . '</pre>';
    }
    public function get_log($type = 'apply')
    {
        $log_file = "/tmp/provisioning_{$type}.log";
        if(file_exists($log_file)){
            $tmp_content = file_get_contents($log_file);
            //replace ^M
            $tmp_content = str_replace("\r", PHP_EOL, $tmp_content);
            $tmp_content = htmlspecialchars($tmp_content);
            $output = explode(PHP_EOL, $tmp_content);
        }else{
            $tmp_content = $log_file . ' not found!';
            $output[] = $tmp_content;
        }
        if(is_array($output)){
            $output = array_map(array($this, 'log_format'), $output);
        }else{
            $output[] = '';
        }
        return $output;
    }
    public function get_apply_log($type = 'apply')
    {
        $rs_data['status'] = true;
        $rs_data['data'] = $this->get_log($type);
        echo json_encode($rs_data);
    }
    public function get_register_log()
    {
        $this->get_apply_log('register');
    }
}
