<?php
/** vim: set tabstop=4 softtabstop=4 shiftwidth=4 textwidth=80 smarttab expandtab: **/
/** coding: utf-8: **/
/*
 * Copyright (C) 2012  Sangoma Technologies Corp.
 * All Rights Reserved.
 *
 * Author(s)
 * your name <your_name@sangoma.com>
 *
 * This code is Sangoma Technologies Confidential Property.
 * Use of and access to this code is covered by a previously executed
 * non-disclosure agreement between Sangoma Technologies and the Recipient.
 * This code is being supplied for evaluation purposes only and is not to be
 * used for any other purpose.
*/
/**
 * Network Config wrapper page
 * @author William Adam <william.adam@sangoma.com>
 *         
 */
if (!defined('BASEPATH')) exit('No direct script access allowed');
require_once ('application/helpers/safe_helper.php');
safe_require_controller('sng','data_manager');
class Sng_update_manager extends Sng_data_manager
{
    public function __construct()
    {
        parent::__construct();
    }
    public function index()
    {
        return $this->manage('update','package');
    }
    /*
     * confirm to install update package
    *
    * @param string $module_name being install
    * @param string $data_name being install
    * @param string $data_key being install
    */
    public function install_confirm($module_name, $data_name, $data_key)
    {
        $module = NULL;
        // Get back the module
        $module = $this->_the_app->find_module_by_name($module_name);
        $data_cfg = $module->get_data_settings();
        $data_cfg = $data_cfg[$data_name];
        //check if can install
        $output = array();
        $info = $module->info($data_name, $data_key, $output);
        $the_product = Safe_get_product();
        // Clear session cache before check install data
        safe_session_clear_cache();
        if($info && $the_product->can_install_data($info, $output)){
            $data['message'] = 'Are you sure you would like to install <b>' . $data_cfg['description'] .' '.$data_key . '</b>?';
            $data['message_type'] = 'notice';
            $data['result'] = true;
        }else{
            $data['message'] = "Failed to install ". $data_cfg['description'] .' '.$data_key;
            if($output){
                $data['reason'] = $output;
            }
            $data['message_type'] = 'alert';
            $data['result'] = false;
        }
        $data['page_title'] = 'Install ' .$data_cfg['description'];
        $data['action_url'] = base_url($this->router->fetch_class().'/install_action/'.$module_name.'/'.$data_name.'/'.$data_key);
        $data['action_method'] = 'POST';
        echo json_encode($data);
    }
    /*
     * redirect to install update package
    *
    * @param string $module_name being install
    * @param string $data_name being install
    * @param string $data_key being install
    */
    public function install_action($module_name, $data_name, $data_key)
    {
        // Set the basic data
        $module = NULL;
        // Get back the module
        $module = $this->_the_app->find_module_by_name($module_name);
        $data_cfg = $module->get_data_settings();
        $data_cfg = $data_cfg[$data_name];
        
        $data['page_title'] = 'Install ' .$data_cfg['description'].' '.$data_key;
        $data['result'] = true;
        $data['redirect_url'] = base_url($this->router->fetch_class()). '/install/'.$module_name. '/' . $data_name. '/' . $data_key;
        echo json_encode($data);
    }
    public function install($module_name, $data_name, $data_key = NULL)
    {
        // Clear session cache before output browser (clear menu data)
        safe_session_clear_cache();
        // Set the basic data
        $data['page_title'] = 'Configuration Manager';
        $data['page_summary'] = 'This page allows managing the system configuration.';
        $data['page_large_icon'] = '/images/icons/32x32/icon-network.png';
        $data['page_small_icon'] = '/images/icons/16x16/icon-network.png';
        // Get back the module
        $module = $this->_the_app->find_module_by_name($module_name);
        $data_cfg = $module->get_data_settings();
        // Grab only the data we're interested in
        if (isset($data_cfg[$data_name])) $data_cfg = $data_cfg[$data_name];
        // From this point we will display something for sure, so display header
        $this->load->view("safe_header", $data);
        $data['wait_message'] = '<b>Installing ' .$data_cfg['description'].' '.$data_key.' ...</b>';
        $this->load->view("safe_confirm_dialog", $data);
        $data['timeout_sec'] = 600;
        $data['timeout_msg'] = 'Ooops ! Install timeout...';
        $data['ui_type'] = 'processing_log';
        $data['log_url'] = '/SAFe/sng_update_manager_alias/get_install_log';
        $data['action_url'] = '/SAFe/sng_rest/api/shutdown/system';//install successful action url
        $this->load->view("safe_wait", $data);
        safe_flush_ouput();
        $param = null;
        $output = array();
        $actions = array();
        if($module->api_install_package($data_name, $param, $output)){
            $system_module = $this->_the_app->find_module_by_name('system');
            $restart_url = $system_module->get_action_url('shutdown');
            if($restart_url){
                $data['notif_warn'][] = $data_cfg['description'].' - '.$data_key.' installed successfully.';
                $data['notif_warn'][] = ' ';
                $data['notif_warn'][] = 'Please restart the system.';
                $actions['action_settings']['shutdown'] = array (
                                        'icon' => 'power-off',
                                        'scope' => 'global',
                                        'name' => 'Restart System',
                                        'description' => array ('Restart the system.'),
                                        'request' => 'GET',
                                        'button_class' => 'danger',
                                        'action_url' =>  $restart_url,
                                        'button' => true
                                );
            }
            $this->_the_app->add_reboot_request($this->_the_app->name(), $data_cfg['description'].' - '.$data_key.' installed successfully.');
            $data['install_upgrade'] = true;
            $this->load->view("safe_wait", $data);
        }else{
            $output_message = isset($output['message']) && trim($output['message']) ? trim($output['message']) : null;
            $output_reason = isset($output['reason']) && trim($output['reason']) ? trim($output['reason']) : null;
            $message = $output_message . '<br/>' . $output_reason;

            if($message && !preg_match('/interface|running|firmware/i', $message)){
                $data['notif_error'][] = $data_cfg['description'].' - '.$data_key.' install failed.';
                $cluster_module = $this->_the_app->find_module_by_name('cluster');
                if($cluster_module !== NULL && $cluster_module->get_cluster_mode() !== 'disabled'){
                    $data['notif_error'][] .= "<br/>HA upgrade failure: <a href='https://wiki.freepbx.org/display/SBC/HA+Upgrade+Failure' target='_blank'>Click here for wiki instructions.</a>";
                }else{
                    $data['notif_error'][] .= $message;
                }
            }else if($message){
                $data['notif_error'][] = $data_cfg['description'].' - '.$data_key.' install failed.';
                $data['notif_error'][] .= $message;
            }else{
                $data['notif_error'][] = $data_cfg['description'].' - '.$data_key.' install failed.';
            }
        }
        $actions['action_settings']['back'] = array (
                                'icon' => 'hand-o-left',
                                'scope' => 'global',
                                'name' => 'Back',
                                'description' => array ('Back'),
                                'request' => 'GET',
                                'action_url' =>  base_url($this->router->fetch_class()).'/manage/'.$module_name.'/'.$data_name,
                                'button' => true
                        );
        $actions['action_settings']['detail'] = array (
                'icon' => 'file-text-o',
                'scope' => 'global',
                'name' => 'Detail',
                'description' => array ('Detail'),
                'request' => 'GET',
                'action_url' =>  base_url($this->router->fetch_class()).'/view/'.$module_name.'/'.$data_name.'/'.$data_key,
                'button' => true
        );
        
        $this->load->view("safe_module_actions", $actions);
        $this->load->view("safe_notifications", $data);
        $this->load->view("safe_footer", $data);
    }
    public function get_install_log()
    {
        $module = $this->_the_app->find_module_by_name('update');
        $rs = $module->api_log_package($data_name, $param, $output);
        $rs_data['status'] = true;
        $rs_data['data'] = $output;
        echo json_encode($rs_data);
    }

    public function view($module_name,$data_name,$data_key=NULL){
        $ajax = $this->router->uri->segment(6);
        if($ajax){
            $module = $this->_get_module($module_name);
            $data=array('content'=>json_encode(array('title'=>'Changelog: '.$this->router->uri->segment(5),'content'=>$module->get_data_file_content_by_key($data_name,$data_key))));
            $this->load->view('safe_ajax',$data);
        }
        else
            parent::view($module_name,$data_name,$data_key);
    }
}
